/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   alertd.h
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

#ifndef ALERTD_H_
#define ALERTD_H_

#define ALERTD_PIDFILE "etc/run/alertd.pid"

/* how often does alertd wake up and try to flush queues */
#define ALERTD_FLUSHGRAIN 60000 

/* sometimes alertd sends an alert from within the alert processing
   function, but it should never recurse deeper than this */
#define MAX_ALERT_RECURSIONS 2

typedef struct TLog
{
  char *basePath;
  FILE *idxFile;
  FILE *datFile;
  FILE *txtFile;
  time_t nextRotation;
} TLog, *PLog;

typedef struct TAlertd
{
  PDaemon Daemon;
  char *Hostname;
  ULONG Serial;
  time_t NextFlushTime;
  ULONG FlushGrain;
  char *InFifo;
  int FacCount;
  PFacHandle FacHandles;
  int AltFacCount;
  PFacHandle AltFacHandles;
  BOOL TraceOn;
  BOOL Forward;
  PLog DefLog;
} TAlertd, *PAlertd;

extern PAlertd gpAlertd;

int Alertd_Config(void);
void Alertd_Unconfig(void);
int Alertd_AlertdErr(char *, int, ULONG, ...);
void Alertd_Bail(ULONG);
int Alertd_ProcAlert(PAlert);
int Alert_OnData(PDGram, void *);
int Alertd_Main(PDaemon);
int Alertd_OnStart(PDaemon);
int Alertd_OnStop(PDaemon);
int Alertd_LoadFac(PNVPair, void *);
int Alertd_OnBadFifo(ULONG);

int Log_Open(PLog, int);
void Log_Close(PLog);
int Log_Rotate(PLog);

int LogEntry_Text(ULONG, time_t, 
		  PAlertRes, 
		  PAlertSrc,
		  FILE *);
int LogEntry_Binary(ULONG, time_t, 
		    PAlertRes, 
		    PAlertSrc,
		    FILE *, FILE *);

#endif
