/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   alertqueue.c
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

#include "sys-include.h"
#include "libnfr.h"
#include "nfr_res.h"
#include "fac.h"
#include "alertd.h"
#include "alertqueue.h"

/* bug: this file needs serious work */

#ifdef DEBUG_FDS

#define WRITE_CURFD() pfd(__FILE__,__LINE__)

void pfd(char *file, int line)
{
  FILE *fdfile = NULL;
  fdfile = fopen("FDS", "a");
  fprintf(fdfile, "%s:%d - fds: %d\n", file, line, fdfile->_fileno);
  fclose(fdfile);
}
#else /* DEBUG_FDS */

#define WRITE_CURFD()

#endif /* DEBUG_FDS */

TAlertQueue *AlertQueue_New(void)
{
  return (PAlertQueue) calloc(1, sizeof(TAlertQueue));
}

void AlertQueue_Delete(PAlertQueue pAlertQueue)
{
  if (pAlertQueue != NULL)
    {
      AlertQueue_Clear(pAlertQueue);
      free(pAlertQueue);
    }
}

void AlertQueue_Clear(PAlertQueue pAlertQueue)
{
  if (pAlertQueue->ExecPath != NULL)
    free(pAlertQueue->ExecPath);
  if (pAlertQueue->ExecArgs != NULL)
    free(pAlertQueue->ExecArgs);
  if (pAlertQueue->QueuePath != NULL)
    free(pAlertQueue->QueuePath);
  if (pAlertQueue->QueueDatPath != NULL)
    free(pAlertQueue->QueueDatPath);
  if (pAlertQueue->TermSeq != NULL)
    free(pAlertQueue->TermSeq);
  memset(pAlertQueue, 0, sizeof(TAlertQueue));
}

int AlertQueue_Init(char *Name, PAlertQueue pAlertQueue)
{
  FILE *QueueDatFile = NULL;
  PCf pCf = NULL;
  char *CfVal = NULL;
  char *CfPath = NULL;
  ULONG Result = 0;

  if ((Name == NULL) || (pAlertQueue == NULL))
    return NFR_ERR_UNDEFINED;

  pAlertQueue->QueuePath = nfrPath(ALERT_QUEUE_BASE_PATH, Name, 
				   "/queue", NULL);

  pAlertQueue->QueueDatPath = nfrPath(ALERT_QUEUE_BASE_PATH, Name, 
				      "/queue.dat", NULL);
  
  CfPath = nfrPath(ALERT_QUEUE_BASE_PATH, Name, "/queue.cf", NULL);
  
  pCf = Cf_New(0);
  Result = Cf_Read(CfPath, pCf);
  if(Result != NFR_ERR_SUCCESS)
    {
      fprintf(stderr, "\nalertd: Warning: unable to read config file "
	      "\"%s\"\n", 
	      CfPath);
    }
  free(CfPath);

  CfVal = Cf_ExpandVal("Flush_Time", pCf);
  if (CfVal == NULL)
    pAlertQueue->FlushTimeInc = 0;
  else
    pAlertQueue->FlushTimeInc = StrToSecs(CfVal);

  pAlertQueue->NextFlushTime = pAlertQueue->FlushTimeInc + time(NULL);
  
  CfVal = Cf_ExpandVal("Flush_Count", pCf);
  if (CfVal == NULL)
    pAlertQueue->FlushCount = 0;
  else
    pAlertQueue->FlushCount = strtoul(CfVal, NULL, 0);
  
  /* bug: need to support more types of flushing */
  pAlertQueue->FlushMethod = AlertQueue_FlushPopen;

  CfVal = Cf_ExpandVal("Exec_Path", pCf);
  if (CfVal == NULL)
    pAlertQueue->ExecPath = NULL;
  else 
    {
      pAlertQueue->ExecPath = calloc(1, strlen(CfVal) + 1);
      strcpy(pAlertQueue->ExecPath, CfVal);
    }

  CfVal = Cf_ExpandVal("Exec_Args", pCf);
  if (CfVal == NULL)
    pAlertQueue->ExecArgs = NULL;
  else 
    {
      pAlertQueue->ExecArgs = calloc(1, strlen(CfVal) + 1);
      strcpy(pAlertQueue->ExecArgs, CfVal);
    }
  
  CfVal = Cf_ExpandVal("Exec_Terminator", pCf);
  if (CfVal == NULL)
    {
      pAlertQueue->TermSeq = calloc(1, 3);
      pAlertQueue->TermSeq[0] = '\n';
      pAlertQueue->TermSeq[1] = EOF;
    }
  else 
    {
      pAlertQueue->TermSeq = calloc(1, strlen(CfVal) + 1);
      strcpy(pAlertQueue->TermSeq, CfVal);
    }

  Cf_Delete(pCf);

  pAlertQueue->Count = 0;
  QueueDatFile = fopen(pAlertQueue->QueueDatPath, "r");
  WRITE_CURFD();
  if (!QueueDatFile)
    pAlertQueue->Count = 0;
  else
    {
      fread(&(pAlertQueue->Count), sizeof(pAlertQueue->Count), 1, 
	    QueueDatFile);
      fclose(QueueDatFile);
    }
  WRITE_CURFD();
  return NFR_ERR_SUCCESS;
}

int AlertQueue_Add( PAlertRes pAlertRes, 
		    PAlertSrc pAlertSrc,
		    time_t Timestamp, 
		    PAlertQueue pAlertQueue)
{
  /* non-reentrant */
  static FILE *f = NULL;
  char TimeStr[32];
  char QueueFile[MAX_PATHLEN];
  FILE *Out = NULL;

  strcpy(TimeStr, ctime(&Timestamp));
  TimeStr[19] = 0;

  WRITE_CURFD();
  if(pAlertRes && pAlertSrc)
    {
      pAlertQueue->Count++;
      Out = fopen(pAlertQueue->QueuePath, "a");
      if(!Out)
	{
	  Alertd_AlertdErr(__FILE__, __LINE__,
			   NFR_ERR_CANTOPENFILE, pAlertQueue->QueuePath, NULL);
	  return NFR_ERR_CANTOPENFILE;
	}

      fprintf(Out, "%s %s %s[%ld]: %s\n", 
	      TimeStr + 4, 
	      gpAlertd->Hostname,
	      pAlertSrc->Name,
	      pAlertSrc->Pid,
	      pAlertRes->Text);

      fclose(Out);
      Out = fopen(pAlertQueue->QueueDatPath, "w");
      if(!Out)
	{
	  Alertd_AlertdErr(__FILE__, __LINE__,
			   NFR_ERR_CANTWRITE, pAlertQueue->QueueDatPath, NULL);
	  return NFR_ERR_CANTWRITE;
	}
      fwrite(&(pAlertQueue->Count), sizeof(pAlertQueue->Count), 1, Out);
      fclose(Out);
    }

  if((pAlertQueue->Count >= pAlertQueue->FlushCount) || 
     (Timestamp >= pAlertQueue->NextFlushTime))
    {
	pAlertQueue->NextFlushTime = Timestamp + pAlertQueue->FlushTimeInc;
	if(pAlertQueue->Count > 0)
	  pAlertQueue->FlushMethod(pAlertQueue);
    }
  WRITE_CURFD();
  return NFR_ERR_SUCCESS;
}

int AlertQueue_FlushPopen(PAlertQueue pAlertQueue)
{
  char TmpFile[MAX_PATHLEN];
  char CmdLine[MAX_CMDLINE];
  char Buf[256];
  FILE *Out = NULL;
  FILE *In = NULL;
  int ByteCount = 0;
  int TimeNow = 0;
  pid_t pid = 0;

  /** Debugging Code **/
  /* FILE *eric = NULL; */
  /** Debugging Code **/

  WRITE_CURFD();
  TimeNow = time(NULL);

  if (pAlertQueue->Count == 0)
    return NFR_ERR_UNDEFINED;

  sprintf(TmpFile, "%s-%d.tmp", pAlertQueue->QueuePath, TimeNow);
  sprintf(CmdLine, "%s %s", pAlertQueue->ExecPath, pAlertQueue->ExecArgs);

  rename(pAlertQueue->QueuePath, TmpFile);

  pAlertQueue->Count = 0;
  Out = fopen(pAlertQueue->QueueDatPath, "w");
  if(!Out)
    {
      Alertd_AlertdErr(__FILE__, __LINE__,
		       NFR_ERR_CANTWRITE, pAlertQueue->QueueDatPath, NULL);
      return NFR_ERR_CANTWRITE;
    }

  fwrite(&(pAlertQueue->Count), sizeof(pAlertQueue->Count), 1, Out);
  fclose(Out);
 
  In = fopen(TmpFile, "r");
  if(!In)
    {
      Alertd_AlertdErr(__FILE__, __LINE__,
		       NFR_ERR_CANTOPENFILE, TmpFile, NULL);
      return NFR_ERR_CANTOPENFILE;
    }

  /** Debugging Code **/
  /** This code spits the command line alert out **/
  /** so you can see what is getting executed. **/
  /*
  if((eric = fopen("/tmp/erictest","a")) == NULL) {
    fprintf(stderr,"Could not open: /tmp/erictest");
  } 
  fprintf(eric,CmdLine);
  fprintf(eric,"\n");
  fclose(eric);
  */
  /** Debugging Code **/

  /* fork() here so lengthy flushes are ok */
  pid = fork();
  if(pid == 0)
    {
      /* first child */
      /* fork twice to avoid night of the living dead */
      pid = fork();
      if(pid < 0)
	{
	  /* log some error */
	}
      else if (pid > 0)
	exit(0); /* first child dies, leaving child 2 to init */

      /* now we're child 2 with init as our parent so we don't zombify */
      Out = popen(CmdLine, "w");
      if(!Out)
	{
	  Alertd_AlertdErr(__FILE__, __LINE__,
			   NFR_ERR_EXEC, CmdLine, NULL);
	  exit(NFR_ERR_EXEC); 
	}
      ByteCount = fread(Buf, 1, sizeof(Buf), In);
      while (ByteCount > 0)
	{
	  fwrite(Buf, 1, ByteCount, Out);
	  ByteCount = fread(Buf, 1, sizeof(Buf), In);      
	}
      fputs(pAlertQueue->TermSeq, Out);
      pclose(Out);
      fclose(In);
      remove(TmpFile);
      WRITE_CURFD();
      exit(0); 
    }
  else if(pid < 0)
    {
      fclose(In);
      return NFR_ERR_FORK;
      WRITE_CURFD();
    }
  else
    {
      /* original parent */
      fclose(In);
      waitpid(pid, NULL, 0); /* wait for child 1 to fork child 2 and die */
      WRITE_CURFD();
      return NFR_ERR_SUCCESS;
    }
}
