/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   fac.c
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

#include "sys-include.h"
#include <syslog.h>
#include "libnfr.h"
#include "nfr_res.h"
#include "fac.h"
#include "alertd.h"


/* these facility functions should only return an error if they
   encounter an unrecoverable error.  any return value other than
   NFR_ERR_SUCCESS will cause any alert directed at that facility
   to be directed elsewhere until alertd is restarted */
int Fac_NfrLog(ULONG Serial, time_t Timestamp, PAlertRes pAlertRes, 
	       PAlertSrc pAlertSrc, PFacHandle pFac)
{
  ULONG Result = NFR_ERR_SUCCESS;
  ULONG tmpResult = 0;
  

  if(!(gpAlertd->DefLog->txtFile &&
       gpAlertd->DefLog->datFile &&
       gpAlertd->DefLog->idxFile))
    {
      Log_Close(gpAlertd->DefLog);
      Log_Open(gpAlertd->DefLog, 0);
    }

  Result = LogEntry_Text(Serial, Timestamp, pAlertRes, pAlertSrc, 
			 gpAlertd->DefLog->txtFile);
  
  Result = LogEntry_Binary(Serial, Timestamp, pAlertRes, pAlertSrc, 
			   gpAlertd->DefLog->datFile, 
			   gpAlertd->DefLog->idxFile);
      
  return NFR_ERR_SUCCESS;
}

int Fac_Ack(ULONG Serial, time_t Timestamp, PAlertRes pAlertRes, 
	    PAlertSrc pAlertSrc, PFacHandle pFac)
{
  char *filename;
  char TimeStr[32];
  char SerialStr[10];
  FILE *f = NULL;

  strcpy(TimeStr, ctime(&Timestamp));
  TimeStr[19] = 0;

  sprintf(SerialStr,"%ld", Serial);

  /* bug: there's an awful lot of malloc'n' here */
  filename = nfrPath("", gpAlertd->DefLog->basePath, "unacked/", 
		     SerialStr, NULL);
  f = fopen(filename, "w");
  free(filename);

  if(!f)
    {
      Alertd_AlertdErr(__FILE__,__LINE__,
		       NFR_ERR_CANTWRITE, filename, NULL);
      return NFR_ERR_CANTWRITE;
    }
  fprintf(f, 
	  "******************************************************************"
	  "\n%s - Alert recieved from %s:\n%s\n" 
	  "******************************************************************"
	  "\n\n",
	  TimeStr + 4, 
	  pAlertSrc->Name,
	  pAlertRes->Text);
	  
  fclose(f);
  return NFR_ERR_SUCCESS;
}

int Fac_Console(ULONG Serial, time_t Timestamp, PAlertRes pAlertRes, 
		PAlertSrc pAlertSrc, PFacHandle pFac)
{
  char TimeStr[32];
  FILE *Out = NULL;

  strcpy(TimeStr, ctime(&Timestamp));
  TimeStr[19] = 0;
  ConsoleMsg("alertd: %s %s[%ld]: (%ld) %s", 
	     TimeStr + 4, 
	     pAlertSrc->Name,
	     pAlertSrc->Pid,
	     Serial, 
	     pAlertRes->Text);
}

int Fac_SysLog(ULONG Serial, time_t Timestamp, PAlertRes pAlertRes, 
	       PAlertSrc pAlertSrc, PFacHandle pFac)
{
  static BOOL isOpen = FALSE;
  int priority = 0;
  /* bug: map the severity to some syslog level */
  
   if(!isOpen) 
    {
      openlog("alertd", LOG_PID, LOG_USER);
      isOpen = TRUE;
    }
    

  switch(ResCode_GetAlertSev(pAlertRes->ResCode))
    {
    case SEV_INFO:
      priority = LOG_INFO;
      break;

    case SEV_WARNING:
      priority = LOG_WARNING;
      break;
      
    case SEV_ERROR:
      priority = LOG_ERR;
      break;

    default:
      priority = LOG_INFO;
    }
      
  syslog(priority,
	 "%s[%ld]: (%ld) %s", 
	 pAlertSrc->Name,
	 pAlertSrc->Pid,
	 Serial, 
	 pAlertRes->Text);


  return NFR_ERR_SUCCESS;
}

int Fac_Generic(ULONG Serial, time_t Timestamp, PAlertRes pAlertRes, 
		PAlertSrc pAlertSrc, PFacHandle pFac)
{
  if(pFac == NULL)
    return NFR_ERR_UNDEFINED;
  return AlertQueue_Add(pAlertRes, pAlertSrc, Timestamp, 
			pFac->AlertQueue);
}


