/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   log.c
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

#include "sys-include.h"
#include "libnfr.h"
#include "nfr_res.h"
#include "fac.h"
#include "alertd.h"

char *Log_MakeName(struct tm *pTime)
{
  static char name[32];
  char months[][12] = {
    "Jan", "Feb", "Mar", "Apr", "May", "Jun", 
    "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
  };


  sprintf(name, "%s_%d_%d", 
	  months[pTime->tm_mon], 
	  pTime->tm_mday,
	  pTime->tm_year + 1900);
  return name;
}

int Log_Open(PLog pLog, int tOffset)
{
  int minutesWastedToday = 0;
  time_t tNow;
  time_t tUse;
  char *idxPath = NULL;
  char *datPath = NULL;
  char *txtPath = NULL;
  char logName[16];
  char *logPath = NULL;

  struct tm *pTime = NULL;
  tNow = time(0);
  tUse = tNow + tOffset;
  pTime = localtime(&tUse);

  if(gpAlertd->Forward)
    strncpy(logName, "forward", sizeof(logName));
  else
    strncpy(logName, Log_MakeName(pTime), sizeof(logName));

  if(pLog->basePath)
    free(pLog->basePath);

  logPath = nfrConcat("", "data/alerts/", gpAlertd->Hostname, "/",  
		      logName, "/",  NULL);

  pLog->basePath = nfrPath("", logPath, NULL);

  if(!pLog->basePath)
    {
      if(!logPath)
	logPath = "<null>";

      fprintf(stderr, "alertd: Error: can't access or create directory: "
	      "%s: ", logPath, strerror(errno));
      ConsoleMsg("alertd: Error: can't access or create directory: "
		 "%s: ", logPath, strerror(errno));
      return NFR_ERR_CANTOPENFILE;
    }
  free(logPath);

  idxPath = nfrConcat(pLog->basePath, "/log.idx", NULL);
  datPath = nfrConcat(pLog->basePath, "/log.dat", NULL);
  txtPath = nfrConcat(pLog->basePath, "/log.txt", NULL);

  pLog->idxFile = fopen(idxPath, "a");
  if(!pLog->idxFile)
    {
      fprintf(stderr, "alertd: `Error: can't write file: %s error: %s\n", 
	      idxPath, strerror(errno));      
      ConsoleMsg("alertd: Error: can't write file: %s error: %s", 
		 idxPath, strerror(errno));      
      return NFR_ERR_CANTWRITE;
    }

  pLog->datFile = fopen(datPath, "a");
  if(!pLog->datFile)
    {
      fprintf(stderr, "alertd: `Error: can't write file: %s error: %s\n", 
	      datPath, strerror(errno));      
      ConsoleMsg("alertd: Error: can't write file: %s error: %s", 
		 datPath, strerror(errno));      
      return NFR_ERR_CANTWRITE;
    }

  pLog->txtFile = fopen(txtPath, "a");
  if(!pLog->txtFile)
    {
      fprintf(stderr, "alertd: `Error: can't write file: %s error: %s\n", 
	      txtPath, strerror(errno));      
      ConsoleMsg("alertd: Error: can't write file: %s error: %s", 
		 txtPath, strerror(errno));      
      return NFR_ERR_CANTWRITE;
    }

  free(idxPath);
  free(datPath);
  free(txtPath);

  pLog->nextRotation = 
    tNow + 86400 - ((pTime->tm_hour * 60 + pTime->tm_min) * 60);

  return NFR_ERR_SUCCESS;
}


void Log_Close(PLog pLog)
{
  if(!pLog)
    return;

  if(pLog->idxFile)
    fclose(pLog->idxFile);
  if(pLog->datFile)
    fclose(pLog->datFile);
  if(pLog->txtFile)
    fclose(pLog->txtFile);
  if(pLog->basePath)
    {
      free(pLog->basePath);
      pLog->basePath = NULL;
    }
}

int Log_Rotate(PLog pLog)
{
  Log_Close(pLog);
  return Log_Open(pLog, 0);
}

int LogEntry_Text(ULONG Serial, time_t Timestamp, 
		  PAlertRes pAlertRes, 
		  PAlertSrc pAlertSrc,
		  FILE *f)
{
  char TimeStr[32];
  int msglen = 0;
  ULONG Result = 0;
 
  strcpy(TimeStr, ctime(&Timestamp));
  TimeStr[19] = 0;

  fprintf(f, "%s %s %s[%ld]: (%ld) %s\n", 
	  TimeStr + 4, 
	  gpAlertd->Hostname,
	  pAlertSrc->Name,
	  pAlertSrc->Pid,
	  Serial, 
	  pAlertRes->Text);
  fflush(f);

  return NFR_ERR_SUCCESS;
}


int LogEntry_Binary(ULONG Serial, time_t Timestamp, 
		    PAlertRes pAlertRes, 
		    PAlertSrc pAlertSrc,
		    FILE *fLog, FILE *fIdx)
{
  ULONG Idx = 0;
  ULONG NBOIdx = 0;
  ULONG NBOSerial = 0;
  ULONG NBOTimestamp = 0;
  ULONG Result = 0;

  NBOSerial = htonl(Serial);
  NBOTimestamp = htonl(Timestamp);
  Idx = ftell(fLog);
  fwrite(&NBOSerial, 1, sizeof(NBOSerial), fLog);
  fwrite(&NBOTimestamp, 1, sizeof(NBOTimestamp), fLog);
  ResId_Write((PResId)pAlertSrc, NULL, fLog);
  AlertSrc_WriteBody(pAlertSrc, NULL, fLog);
  ResId_Write((PResId)pAlertRes, NULL, fLog);
  AlertRes_WriteBody(pAlertRes, NULL, fLog);

  NBOIdx = htonl(Idx);
  fwrite(&NBOIdx, 1, sizeof(NBOIdx), fIdx);
  fwrite(&NBOSerial, 1, sizeof(NBOSerial), fIdx);

  fflush(fLog);
  fflush(fIdx);

  return NFR_ERR_SUCCESS;
}

