/*
        Copyright(C) 1997 Network Flight Recorder, Inc.
        All rights reserved.

        Use and distribution of this software and its source code
        are governed by the terms and conditions of the
        Network Flight Recorder Software License ("LICENSE.TXT")

By Mark Sienkiewicz / NFR

*/


#include "sys-include.h"

#include "counter.h"
#include "vsch.h"
#include "cf.h"
#include "list.h"
#include "archive.h"

int one=1;

struct vsch_record vr;
void **data;
int cur_time = 0;

sendit(char *s)
{
int x;
while (isspace(*s))
	s++;
x=atoi(s);
if (x > 0)
	cur_time = x;
fwrite(&one,sizeof(int),1,stdout);
fwrite(&cur_time,sizeof(int),1,stdout);
for (x=0; x<vr.num_fields; x++)
	{
	switch (vr.f[x].type)
		{
	case VSCH_TIME_T:
	case VSCH_INT:
	case VSCH_COUNTER:	/* nfrd doesn't have counters; use int */
		fwrite(data[x],sizeof(int),1,stdout);
		break;
	case VSCH_STR:
		{
		int n = strlen(data[x]);
		fwrite(&n,sizeof(int),1,stdout);
		fwrite(data[x],n,1,stdout);
		}
		break;
	case VSCH_IP:
		fwrite(data[x],1,4,stdout);
		break;
	case VSCH_ETHERMAC:
		fwrite(data[x],1,6,stdout);
		break;
	default:
		abort();
		}
	}
}

main(int argc,char **argv)
{
PCf conf;
char b[1000];
int x;

if ( (! (conf=Cf_New(0)) )
        || Cf_Read(argv[1],conf) )
        {
        perror(argv[1]);
        exit(1);
        }

load_vsch_record(&vr, conf);

data = calloc(vr.num_fields, sizeof(void *));

if (!data)
	abort();

for (x=0; x<vr.num_fields; x++)
	data[x]=malloc(sizeof(int)+6);	/* wasteful; so what ? */

while (fprintf(stderr,":"), fflush(stderr), fgets(b,sizeof(b),stdin))
	{
	switch (b[0])
		{
	case 's':
		sendit(b+1);
		break;
	case '0': case '1': case '2': case '3': case '4': 
	case '5': case '6': case '7': case '8': case '9':
		read_data(b);
		break;
	case 't':
		timestamp(b+1);
		break;
	case 'p':
		printschema();
		break;
	default:
		fprintf(stderr,"?\n");
		}
	fflush(stdout);
	}

}


timestamp(char *s)
{
int x=atoi(s);
if (x > 0)
	cur_time =x;
x=57007;
fwrite(&x,sizeof(int),1,stdout);
fwrite(&cur_time,sizeof(int),1,stdout);
}


printschema()
{
int x;
for (x=0; x<vr.num_fields; x++)
	{
	fprintf(stderr,
		"%d %-20s %-20s\n",x, vr.f[x].primary_type, vr.f[x].label);
	}
fprintf(stderr,"\n\nschema = [ 1,\t1");
for (x=0; x<vr.num_fields; x++)
	{
	/*
	* pay attention here:  we translate the VSCH_ values into the
	* values that N code uses and that nfrd uses internally.
	* Someday, nfrd will accept symbolic names.
	*/
	switch (vr.f[x].type)
		{
	case VSCH_TIME_T:
	case VSCH_INT:
	case VSCH_COUNTER:
		fprintf(stderr,", 1");
		break;
	case VSCH_IP:
		fprintf(stderr,", 6");
		break;
	case VSCH_STR:
		fprintf(stderr,", 2");
		break;
	case VSCH_BLOB:
		fprintf(stderr,", 8");
		break;
	case VSCH_ETHERMAC:
		fprintf(stderr,", 9");
		break;
	default:
		fprintf(stderr,", ????");
		}
	}
fprintf(stderr," ]\n");
fflush(stderr);
}

read_data(char *s)
{
int x;
x=atoi(s);
s=strchr(s,' ');
if (!s) s="";
if (x >= vr.num_fields)
	{
	fprintf(stderr,"only %d fields\n",vr.num_fields);
	return;
	}
switch (vr.f[x].type)
	{
case VSCH_TIME_T:
case VSCH_INT:
case VSCH_COUNTER:	/* nfrd doesn't have counters; use int */
	sscanf(s,"%d",data[x]);
	break;
case VSCH_STR:
	{
	int n = strlen(s);
	free(data[x]);
	data[x]=malloc(n+1);
	strcpy(data[x],s);
	}
	break;
case VSCH_IP:
	{
	int n[4];
	if (sscanf(s,"%d.%d.%d.%d",&n[0],&n[1],&n[2],&n[3]) == 4)
		{
		((char *)data[x])[0] = n[0];
		((char *)data[x])[1] = n[1];
		((char *)data[x])[2] = n[2];
		((char *)data[x])[3] = n[3];
		}
	else
		fprintf(stderr,"bad scan\n");
	}
	break;
case VSCH_ETHERMAC:
	{
	int n[6];
	if (sscanf(s,"%x:%x:%x:%x:%x:%x",&n[0],&n[1],
		&n[2],&n[3],&n[4],&n[5]) == 6)
		{
		((char *)data[x])[0] = n[0];
		((char *)data[x])[1] = n[1];
		((char *)data[x])[2] = n[2];
		((char *)data[x])[3] = n[3];
		((char *)data[x])[4] = n[4];
		((char *)data[x])[5] = n[5];
		}
	else
		fprintf(stderr,"bad scan\n");
	}
	break;
default:
	abort();
	}
}
