/*
        Copyright(C) 1997 Network Flight Recorder, Inc.  
        All rights reserved.

        Use and distribution of this software and its source code
        are governed by the terms and conditions of the
        Network Flight Recorder Software License ("LICENSE.TXT")

By Mark Sienkiewicz / NFR

*/

#include "vsch.h"

#define MAX_AXES 20

/*
* the axis_map tells you how to pluck axis values out of the
* schema.  It also has the offset of the data value.  For each
* histogram type, you will keep one axis_map for each schema type.
*/
/*
* this should have been an array of structures instead of a structure
* of arrays.  Probably next version.
*/
struct axis_map
	{
	char	use_data;
	char * 	count_label;
	char * 	data_label;
	query_match_t	primary_type_query[MAX_AXES];
	query_match_t	axis_query[MAX_AXES];
	array_subscript_compare	array_compare_fn[MAX_AXES];

	struct vsch_record r;		/* mostly for ../libback/nfrd_int.c */
	};



struct hist
	{
	time_t	min_time;	/* time range this histogram covers */
	time_t	max_time;	/* time range this histogram covers */
	array_t	data;
	int	interval;	/* what duration this histogram covers */
	array_tag_t tag;
	char *	prefix;
	};


/*
* struct everything is everything you need to know about a particular
* histogram.  Currently, this program can only do one histogram
* type at a time.  struct everything is an artifact from a time
* when we thought we might handle more than one histogram in a
* single process.
*/
struct everything
	{
	struct hist all_time;		/* histogram since dawn of time */
	struct hist current;		/* short-duration histogram */
	struct hist previous;	/* previous one */

	/*
	* axis maps for all known schema types
	*/
	struct axis_map axis_map;

	time_t	time_marching;
	int	last_update;	/* last time written to file */
	int	sync_time;	/* how often to update files */
	};

int
enter_histogram(struct hist * hist, struct axis_map *axis_map,
        void *p, counter *dataparam);

struct axis_map *
pick_some_axis_map(struct everything *h);

int histogram_init( char * config_file_name, struct everything *the_histogram,
	PCf *p);

void free_histogram(struct hist *h);

int write_histogram(struct hist *h, char *my_recorder_name, char *subset );

/*
* this is just here as type checking during the transition
*/
struct io_buffer
	{
	char data[1000];
	};

extern struct everything the_histogram ;

int write_histogram_archive_file(struct hist *h, char *filename);

int
read_histogram_file(struct hist *h, char *filename, int limit_to_time_range,
	int (*query_fn)());

extern char *my_recorder_name;
/*
* names of temp files
*/
char *all_time_name, *current_name, *previous_name;

int write_array_sub(array_t a, void **b, int *sizes, int bindex,
        struct axis_map * axis_map, int axis_map_index, 
        struct vsch_buf *vb);

extern int (*bufferize_func)(void *b, int size, int *size_array);

/*
* flags about which histograms we are keeping
*/
extern int have_all_time;
extern int have_current;
