/*
        Copyright(C) 1997 Network Flight Recorder, Inc.  
        All rights reserved.

        Use and distribution of this software and its source code
        are governed by the terms and conditions of the
        Network Flight Recorder Software License ("LICENSE.TXT")

By Mark Sienkiewicz / NFR

*/

/*
*******************************************************************************
* 
* boring include files
*
*/

#include "sys-include.h"

#include "allocaters.h"
#include "array.h"
#include "cf.h"
#include "counter.h"
#include "vsch.h"
#include "../../lib/back/query.h"
#include "everything.h"
#include "home.h"
#include "archive.h"

/*
* debug stuff - should go to debug.h
*/
#define DEBUG(x,y) { if (debug >= x) { y; fflush(debugfp); } }
extern int debug;
FILE *debugfp;

/*
* acK!
*/

int use_time_as_axis = 0;

/*
* which histograms are we keeping?  set from the config file, but assume
* we have both unless specified otherwise.
*/
int have_current = 1;
int have_all_time = 1;


/*
*******************************************************************************
*
* config file reading stuff
*
* the config file is opened by the main program.  the PCf is passed
* to these various functions.
*
*/

static load_axis_info(PCf conf, struct everything * histogram)
{
int x,y,found;
char buf[100],*s;
struct axis_map *am = & histogram->axis_map;

load_vsch_record(&am->r, conf);

s=Cf_GetVal("count_label",conf);
if (!s) s="";
am->count_label=s;

s=Cf_GetVal("data_label",conf);
if (!s) s="";
am->data_label=s;

s=Cf_GetVal("data", conf);
if ( ( !s ) ||  ( !*s ) )
	am->use_data=0;
else
	am->use_data=1;
}



counter counter_0, counter_1;


/* bug: need better file names */

init_histograms(PCf conf, struct everything * h)
{
char *s,*t;
time_t tyme;

	/*
	* min_time and max_time for all_time is forever.  Beware the
	* January 2038 problem. :)
	*/
h->all_time.min_time=0;
h->all_time.max_time= ((unsigned int)~0) >> 1;

h->all_time.tag = array_tag_create();
h->all_time.data = array_create(h->all_time.tag);

	/*
	* rollover is the interval we want to collect histograms over.
	* This will be the smallest resolution you will be able to look
	* at later on.
	*/
h->current.interval=0;
s=Cf_GetVal("rollover",conf);
if (s)
	h->current.interval=atoi(s);

	/*
	* If there was no rollover interval specified, there is no
	* histogram to be done.  In this case, we will only keep the
	* all-time histogram.
	*
	* if there is, compute the time range of the current time period
	* rounded of according to the interval.  e.g. if the period is
	* 5 minutes, a current time of 16:47 gets us 16:45-16:50
	*/
if (h->current.interval)
	{
	h->current.min_time = 0;
	h->current.max_time = 0;
	h->current.tag = array_tag_create();
	h->current.data = array_create(h->current.tag);

	h->previous.min_time = 0;
	h->previous.max_time = 0;
	h->previous.tag = array_tag_create();
	h->previous.data = array_create(h->previous.tag);

	}

/*
* sync time is how often to write the files so they will be available
* for queries to look at.
*/
s=Cf_GetVal("sync_time",conf);
if (s)
	h->sync_time= atoi(s);
else
	h->sync_time= h->current.interval / 20;
if (h->sync_time == 0)
	h->sync_time = 60;
else if (h->sync_time < 10)
	h->sync_time = 10;

h->last_update = 0;

/*
* the initial state of the histogram is loaded by the recorder, but
* not by other programs.
*/
}

int new_cell_alerts = 0;
char *new_alert_text="";

char *all_time_name, *current_name, *previous_name;

char *my_recorder_name;

histogram_init( char * config_file_name, struct everything *the_histogram,
	PCf *config)
{
PCf conf;
char *s;

	counter_0 = counter_atoi("0");
	counter_1 = counter_atoi("1");

	s=strchr(config_file_name,'/');
	if (s)
		my_recorder_name=strdup(s+1);
	else
		my_recorder_name=strdup(config_file_name);
	s=strrchr(my_recorder_name,'.');
	if (s)
		*s=0;

	if ( (! (conf=Cf_New(0)) )
		|| Cf_Read(config_file_name,conf) )
		{
		perror(config_file_name);
		exit(1);
		}
	if (config)
		*config = conf;

	{
	char *s;
	s=Cf_GetVal("debug", conf);
	if (s)
		{
		debug=atoi(s);
		if (debug == 0) debug=5;
		s=strchr(s,',');
		if (s)
			{
			s++;
			debugfp=fopen(s,"w");
			if (!debugfp)
				{
				perror(s);
				exit(1);
				}
			}
		else
			debugfp=stdout;
		fprintf(debugfp,"debug level is %d\n",debug);
		fflush(debugfp);
		}
	}

	{
	char *s;
	s=Cf_GetVal("suppress_all_time", conf);
	if (s)
		have_all_time = 0;
	s=Cf_GetVal("suppress_current", conf);
	if (s)
		have_current = 0;
	}

	{
	char *s;
	s=Cf_GetVal("new_cell_alerts", conf);
	if (s)
		{
		while (isspace(*s)) s++;
		if (*s == 't')
			{
			new_cell_alerts=1;
			s=Cf_GetVal("new_alert_text",conf);
			if (s)
				new_alert_text = s;
			else
				new_alert_text = "";
			if (!have_all_time)
				{
				fprintf(stderr,
"new_cell_alerts + suppress_all_time are incompatible.  all_time *is* kept.\n");
				have_all_time=1;
				/* bug: should use alert */
				}
			}
		}
	}

	the_histogram->time_marching = 0;


	load_axis_info(conf, the_histogram);
        init_histograms(conf, the_histogram);
	all_time_name=nfrPath("data","/",my_recorder_name,"/all-time",NULL);
	current_name=nfrPath("data","/",my_recorder_name,"/current",NULL);
	previous_name=nfrPath("data","/",my_recorder_name,"/previous",NULL);
}


/*
*******************************************************************************
*
*/

int my_array_int_cmp( void *first, int first_size, void *second, 
	int second_size)
{
int x;
x= * (int *) first - * (int *) second;
return x;
}

int my_array_counter_cmp( void *first, int first_size, void *second, 
	int second_size)
{
return counter_cmp( *(counter *)first, *(counter *)second );
}

set_axis_map_compare_function(struct axis_map *am)
{
int x;
for (x=0; x<am->r.num_fields; x++)
	{
	switch (am->r.f[x].type)
		{
	case VSCH_TIME_T:
	case VSCH_INT:
		am->array_compare_fn[x] = my_array_int_cmp;
		break;
	case VSCH_BLOB:
	case VSCH_IP:
	case VSCH_STR:
	case VSCH_ETHERMAC:
		am->array_compare_fn[x] = array_binary_cmp;
		break;
	case VSCH_COUNTER:
		am->array_compare_fn[x] = my_array_counter_cmp;
		break;
	default:
		abort();	/* never happens */
		}
	}
}


/*
*******************************************************************************
*
*/

int
enter_histogram(struct hist * hist, struct axis_map *axis_map, 
	void *pp, counter *dataparam)
{
int x,datasize;
array_t array_entry;
array_node curr;
counter data[2];
array_tag_t tag;
int new_flag=0;
int tmp;
unsigned char *p = pp;

tag=hist->tag;


if (! axis_map->array_compare_fn[0] )
	set_axis_map_compare_function(axis_map);

for ( x = 0 ; x < axis_map->r.num_fields; x++)
	{
	if (! axis_map->r.f[x].diminished)
		break;
	}

if (x >= axis_map->r.num_fields)
	{
	fprintf(stderr,"all axes collapsed!\n");
	printf("all axes collapsed!\n");
	abort();	/* bug: return normal error */
	}

tmp = array_find(hist->data, &curr, 
	p + axis_map->r.f[x].offset,
	axis_map->r.f[x].size, 1, tag,
	axis_map->array_compare_fn[x]);

switch (tmp)
	{
case array_error:
	abort(); /* bug: report error */
	break;
case array_new:
	new_flag=1;
case array_ok: ;
	}

x++;	/* on to next axis */

	/*
	* use subsequent axis values to walk through a tree of arrays
	* until you have gone to the depth of all the axes.
	*/
for ( ; x<axis_map->r.num_fields; x++)
	{
        if (axis_map->r.f[x].diminished)
		continue;

	if (array_get(curr, (void *)0, (void *)0,
		&array_entry, sizeof(array_entry), &datasize) != array_ok)
				{ abort(); } /* never happens */
	if (datasize != sizeof(array_t))
		{
		array_entry=array_create(tag);
		if (array_set(curr, &array_entry, sizeof (array_t), tag)
			!= array_ok) 
				{ abort(); } /* bug: handle it */
		}
	tmp = array_find(array_entry, &curr, 
		p+axis_map->r.f[x].offset, axis_map->r.f[x].size, 
			1, tag, axis_map->array_compare_fn[x]);
	switch (tmp)
		{
	case array_error:
		abort(); /* bug: report error */
		break;
	case array_new:
		new_flag=1;
	case array_ok: ;
		}
	}

	/*
	* having walked through all the axes, we are still looking
	* at an array cell.  This one holds the data.  We fetch 
	* the data out.
	*/
if (array_get(curr, (void *)0, (void *)0,
		&data[0], sizeof(data), &datasize) != array_ok)
	{ abort(); } /* never happens */

	/*
	* if the data size is 0, it has never been initialized.
	* this might be a better place to recognize a new cell being added.
	*/
if (datasize == 0)
	{
	data[0]=counter_0;
	data[1]=counter_0;
	}

	/*
	* if we were given dataparam, we are busy loading values into
	* the array - add up the values.  If we were not, we are entering
	* a record, so add 1 to the count of records seen.
	* (e.g. this might be number of packets.)
	*/
if (dataparam)
	data[0] = counter_add ( data[0], dataparam[0] );
else
	data[0] = counter_add ( data[0], counter_1 );

	/*
	* if we are using data, we also have a value to add up (e.g.
	* this might be packet size.)  The same thing about loading
	* the array applies.  Set the value in the array using the
	* proper data size (2 counter_t if use_data, 1 counter_t
	* otherwise).
	*/
if (axis_map->use_data)
	{
	if (datasize != sizeof(data))
		data[1]=0;
	if (dataparam)
		data[1] = counter_add(data[1],dataparam[1]);
	else
		{
		int n;
		memcpy(&n,(p+axis_map->r.data_offset), sizeof(int));
		data[1] = counter_add(data[1], counter_from_int(n));
		}
	if (array_set(curr, &data[0], sizeof(data), tag) != array_ok)
		{ abort(); } /* bug: handle error */
	}
else
	{
	if (array_set(curr, &data[0], sizeof(data[0]), tag) != array_ok)
		{ abort(); } /* bug: handle error */
	}

return new_flag;
}


/*
*******************************************************************************
*
*/

void free_histogram(struct hist *h)
{
array_tag_purge(h->tag);
h->tag = 0;
h->data = (void *)0;
}

/*
*******************************************************************************
*
*/

int (*bufferize_func)(void *b, int size, int *size_array);

bufferize_array(struct vsch_buf *v, void **b, int *size_array, counter *data)
{
int x, bindex; 
int err=0;
v->buffer_offset=0;
{ int q=1; vsch_putbuf(v,VSCH_INT, &q, 0); }
for (x=0, bindex=0; x < the_histogram.axis_map.r.num_fields; x++)
	{
	if (the_histogram.axis_map.r.f[x].diminished)
		continue;
	err |= vsch_putbuf(v, the_histogram.axis_map.r.f[x].type, b[bindex],
			size_array[bindex]);
	bindex++;
	}
if (the_histogram.axis_map.use_data)
	err |= vsch_putbuf(v,VSCH_COUNTER, &data[1],0);
err |= vsch_putbuf(v,VSCH_COUNTER, &data[0],0);

if (!err)
	return (*bufferize_func)(v->buffer, v->buffer_offset, size_array);
else
	fprintf(stderr,"Yow! error in bufferize\n");
return err;
}


write_array_sub(array_t a, 
	void **b, int *sizes, int bindex,
	struct axis_map * axis_map, int axis_map_index, 
	struct vsch_buf *vb)
{
array_scan_t sc;
array_node n;
void *sub;
static unsigned char data[100];
int actual_sub, actual_data;
int this_axis_map_index, next_axis_map_index;

/*
* a is the nth level array
*/

while ((axis_map->r.f[axis_map_index].diminished) 
	&& (axis_map_index < axis_map->r.num_fields))
	axis_map_index++;

this_axis_map_index = axis_map_index;

next_axis_map_index = axis_map_index + 1;

while ((axis_map->r.f[next_axis_map_index].diminished) 
	&& (next_axis_map_index < axis_map->r.num_fields))
	next_axis_map_index++;

sc=array_start_scan(a);
if (next_axis_map_index == axis_map->r.num_fields)
	{
	/*
	* this is the last non-diminished axis
	*/
	while (n=array_scan(sc))
		{
		if (array_get(n,&sub,&actual_sub,
			data,sizeof(data),&actual_data) != array_ok)
				{abort(); } /* bug */
		sizes[bindex] = actual_sub;
		b[bindex]=sub;
		bufferize_array(vb,b,sizes,(counter *)&data[0]);
		}
	}
else
	{
	
	while (n=array_scan(sc))
		{
		if (array_get(n,&sub,&actual_sub,
			data,sizeof(data),&actual_data) != array_ok)
				{abort(); } /* bug */
		sizes[bindex] = actual_sub;
		b[bindex]=sub;
		if ( * (array_t *)data )
			write_array_sub( * (array_t *)data, b, sizes, bindex+1, 
				axis_map, next_axis_map_index, vb);
		}
	}
array_end_scan(sc);
}



/*
*******************************************************************************
*
*/
/*
* bug: be nice to have some error reporting
*/
extern struct everything the_histogram;

static Archive *data_file = NULL;
static time_t write_histogram_timestamp;

static int write_histogram_bufferize_func(void *b, int size, int *sizes)
{
if (archive_write(data_file, size, 
	b, write_histogram_timestamp, "") < 0)
		{
		perror("archive_write");	
		return 1;
		}
return 0;
}


write_histogram_archive(struct hist *h, char *my_recorder_name, char *subset )
{
void *b[MAX_AXES];
int sizes[MAX_AXES];
int x;
struct vsch_buf record_buffer;

/*
* don't bother recording empty histograms.  it's easier to test here
* than anywhere else.
*/
if (h->max_time == 0)
	return;

if (! data_file )
	data_file = archive_openwrite(my_recorder_name,subset,h->min_time,"");

if (!data_file)
	{
	extern int archive_errno;
	fprintf(stderr,"archive open error: %s\n",astrerror(archive_errno));
	return;
	}

vsch_initbuf(&record_buffer,12);

bufferize_func = write_histogram_bufferize_func;

/*
* each histogram starts with a 0 type record that contains the two times
*/
x=0;
vsch_putbuf(&record_buffer,VSCH_INT, &x,0);
vsch_putbuf(&record_buffer,VSCH_INT,&h->min_time,0);
vsch_putbuf(&record_buffer,VSCH_INT,&h->max_time,0);

if (archive_write(data_file, record_buffer.buffer_offset, 
	(void *)record_buffer.buffer, h->min_time, "") < 0)
	perror("archive_write");

/*
* recursively write everything else.
*/
write_histogram_timestamp = h->min_time;

write_array_sub(h->data, b, sizes, 0, 
	&the_histogram.axis_map, 0, &record_buffer);

/*
* each histogram ends with a type 2 record
*/
record_buffer.buffer_offset=0;
x=2;
vsch_putbuf(&record_buffer, VSCH_INT, &x,0);

if (archive_write(data_file, record_buffer.buffer_offset, 
	(void *)record_buffer.buffer, h->min_time, "") < 0)
	perror("archive_write");

/*
* done.
*/
free(record_buffer.buffer);
archive_flush(data_file);
}

close_histogram_archive()
{
if (data_file)
	archive_close(data_file);
}


static FILE *temp_file;

static int write_temp_file_bufferize_func(void *b, int size, int *sizes)
{
if ((fwrite(&size,1,sizeof(int),temp_file) != sizeof(int))
	|| (fwrite(b,1,size,temp_file) != size))
	{
	perror("write_temp_file_bufferize_func");	
	return 1;
	}
return 0;
}


write_histogram_file(struct hist *h, char *filename)
{
void *b[MAX_AXES];
int sizes[MAX_AXES];
int x;
struct vsch_buf record_buffer;

DEBUG(4,fprintf(debugfp,"writing histogram file -%s-\n",filename))

temp_file = fopen(filename,"w");

if (!temp_file)
	{
	int t=errno;
	fprintf(stderr,"can't open histogram output file %s: %s\n",filename,
		sys_errlist[errno]);
DEBUG(4,fprintf(debugfp,"error - %s\n",sys_errlist[t]));
	return;
	}

vsch_initbuf(&record_buffer,12);

bufferize_func = write_temp_file_bufferize_func;

/*
* each histogram starts with a 0 type record that contains the two times
*/
x=0;
vsch_putbuf(&record_buffer,VSCH_INT, &x,0);
vsch_putbuf(&record_buffer,VSCH_INT,&h->min_time,0);
vsch_putbuf(&record_buffer,VSCH_INT,&h->max_time,0);

write_temp_file_bufferize_func(record_buffer.buffer,
	record_buffer.buffer_offset,0);

/*
* recursively write everything else.
*/
write_histogram_timestamp = h->min_time;

/*
* don't try recording the data in an empty histogram.  DO write
* over the file, though.
*/
if (h->max_time != 0)
	write_array_sub(h->data, b, sizes, 0, &the_histogram.axis_map, 0, 
		&record_buffer);

/*
* each histogram ends with a type 2 record
*/
record_buffer.buffer_offset=0;
x=2;
vsch_putbuf(&record_buffer, VSCH_INT, &x,0);

write_temp_file_bufferize_func(record_buffer.buffer,
	record_buffer.buffer_offset,0);
/*
* done.
*/
free(record_buffer.buffer);
fclose(temp_file);
}

read_histogram_file(struct hist *h,  char *filename, int limit_to_time_range,
	int (*query_fn)())
{
FILE *f= NULL;
struct vsch_buf buf;
int x, e;
int min,max;

vsch_initbuf(&buf,12);

f=fopen(filename,"r");
if (!f)
	{
	if (errno != ENOENT)	/* ok to have missing file sometimes */
		perror(filename);
	goto done;
	}

if (read_temp_file_record(f, &buf))
	{e = 0; goto error;}

if (vsch_getbuf(&buf, VSCH_INT, &x, (void *)0))
	{e = 1; goto error;}
if (limit_to_time_range)
	{
	if (vsch_getbuf(&buf, VSCH_INT, &min, (void *)0))
		{e = 2; goto error;}
	if (vsch_getbuf(&buf, VSCH_INT, &max, (void *)0))
		{e = 3; goto error;}

DEBUG(3,fprintf(stderr,"limiting to time range %8d - %8d\n",h->min_time,
		h->max_time))
DEBUG(3,fprintf(stderr,"file        time range %8d - %8d\n",min, max))
	/*
	* skip this file if no part of it overlaps the desired area
	* h->min_time is the earliest we want
	* h->max_time is the latest we want
	*/
	if (h->max_time <= min)
		{
		DEBUG(3,fprintf(stderr,"desire max <= min\n"))
		goto done;
		}
	if (h->min_time >= max)
		{
		DEBUG(3,fprintf(stderr,"desire min >= max\n"))
		goto done;
		}
	}
else
	{
	if (vsch_getbuf(&buf, VSCH_INT, &h->min_time, (void *)0))
		{e = 4; goto error;}
	min = h->min_time;
	if (vsch_getbuf(&buf, VSCH_INT, &h->max_time, (void *)0))
		{e = 5; goto error;}
	}

for (;;)
	{
	if (read_temp_file_record(f, &buf))
		{e = 6; goto error;}
	if (vsch_getbuf(&buf, VSCH_INT, &x, (void *)0))
		{e = 7; goto error;}
	if (x == 2)
		goto done;
	if (x == 1)
		{
		counter data[2];
		if (use_time_as_axis)
			{
			memcpy(buf.buffer,&min,sizeof(int));
			buf.read_point=0;
			}
		for (x=0; x< the_histogram.axis_map.r.num_fields; x++)
			{
			the_histogram.axis_map.r.f[x].offset=vsch_ptrbuf(&buf,
				the_histogram.axis_map.r.f[x].type, 
				&the_histogram.axis_map.r.f[x].size);
			}
		/* pluck out data elements */
		if (the_histogram.axis_map.use_data)
			vsch_getbuf(&buf,VSCH_COUNTER, &data[1], (void *)0);
		/* only have to check error on last one to see error */
		if (vsch_getbuf(&buf,VSCH_COUNTER, &data[0], (void *)0 ))
			{ e = 8; goto error;}
		if ((! query_fn) || (*query_fn)(&buf))
			{
			enter_histogram(h, 
				&the_histogram.axis_map, buf.buffer, data);
			}
		continue;
		}
	else
		{e = 9; goto error;}
	}

/* come here at normal end */

done:
	if (f) fclose(f);
	free(buf.buffer);
	return 0;

error:
	if (f) fclose(f);
	free(buf.buffer);
	return -1;
}


int read_temp_file_record(FILE *f, struct vsch_buf *b)
{
int size;
if (fread(&size,1,sizeof(int),f) != sizeof(int))
	return -1;
if (size > b->buffer_size)
	{
	b->buffer_offset=0;
	if (vsch_buffer_grow(b, size))
		return -1;
	}
if (fread(b->buffer,1,size,f) != size)
	return -1;
b->buffer_offset=size;
b->read_point=0;
return 0;
}

/*
* BUGBUG:  This looks so much like read_histogram_file() it isn't
* even funny.
*/
read_histogram_archive(struct hist *h, char *my_recorder_name,
	char *subset, int min_time, int max_time, int (*query_fn)())
{
Archive *f = (void *)0;
struct vsch_buf buf;
int x, t, record_min_time, record_max_time;
int dont_load;

DEBUG(3,fprintf(stderr,"begin read_histogram_archive\n"))
vsch_initbuf(&buf,12);

/*
* round min_time down and max_time up to intervals, so we
* get some data for partial-period queries.  This breaks near limits
* of the range of time_t.
*/
x=the_histogram.current.interval;
min_time = (min_time / x ) * x;
t = ((max_time+x-1) / x) * x;
if (t > max_time)
	max_time = t;
DEBUG(3,fprintf(stderr,"archive_openread: my name %s time %8d - %8d \n",
	my_recorder_name, min_time, max_time))
f=archive_openread(my_recorder_name, "data", min_time, max_time);
if (!f)
	{
	fprintf(stderr,"archive open error: %s\n",astrerror(archive_errno));
	goto done;
	}


/*
* run up to the first type 0 record, skipping any possible fragment
* of a histogram that may be left over after spaceman starts eating
* files.
*/
more:

DEBUG(3,fprintf(stderr,"more:\n"))
for (;;)
	{
	switch (get_archive(f, &buf))
		{
	case 0:  goto done;
	case -1: goto error;
		}

	if (vsch_getbuf(&buf, VSCH_INT, &x, (void *)0))
		goto error;
	if (x == 0)
		break;
	}
DEBUG(3,fprintf(stderr,"found 0 record\n"))

if (vsch_getbuf(&buf, VSCH_INT, &record_min_time, (void *)0))
	goto error;

DEBUG(3,fprintf(stderr,"record_min_time %d\n",record_min_time))

if (vsch_getbuf(&buf, VSCH_INT, &record_max_time, (void *)0))
	goto error;

DEBUG(3,fprintf(stderr,"type 0 record, time %8d - %8d\n",record_min_time,
	record_max_time))

dont_load=0;
if (record_max_time <= min_time)
	dont_load=1; 
if (record_min_time >= max_time)
	dont_load=1;

for (;;)
	{
	switch (get_archive(f, &buf))
		{
	case 0:  goto error;
	case -1: goto error;
		}
	if (vsch_getbuf(&buf, VSCH_INT, &x, (void *)0))
		goto error;
	DEBUG(4,fprintf(stderr,"record type %d\n",x))
	if (x == 2)
		goto more;
	if (x == 1)
		{
		counter data[2];
		if (dont_load)
			continue;
		if (use_time_as_axis)
			{
			memcpy(buf.buffer,&record_min_time,sizeof(int));
			buf.read_point=0;
			}
		for (x=0; x< the_histogram.axis_map.r.num_fields; x++)
			{
			the_histogram.axis_map.r.f[x].offset=vsch_ptrbuf(&buf,
				the_histogram.axis_map.r.f[x].type, 
				&the_histogram.axis_map.r.f[x].size);
			}
		/* pluck out data elements */
		if (the_histogram.axis_map.use_data)
			vsch_getbuf(&buf,VSCH_COUNTER, &data[1], (void *)0);
		/* only have to check error on last one to see error */
		if (vsch_getbuf(&buf,VSCH_COUNTER, &data[0], (void *)0 ))
			goto error;
		if ((! query_fn) || (*query_fn)(&buf))
			{
			DEBUG(4,fprintf(stderr,"entered\n"))
			enter_histogram(h, 
				&the_histogram.axis_map, buf.buffer, data);
			}
		continue;
		}
	else
		goto error;
	}

/* come here at normal end */

done:
DEBUG(3,fprintf(stderr,"read_histogram_archive done\n"))
	if (f) { archive_close(f); }
	free(buf.buffer);
	return 0;

error:
DEBUG(3,fprintf(stderr,"read_histogram_archive done with ERROR\n"))
	if (f) archive_close(f);
	if(buf.buffer)
	  free(buf.buffer);
	return -1;
}

