#include "sys-include.h"
#include "allocaters.h"
#include "array.h"
#include "cf.h"
#include "counter.h"
#include "vsch.h"
#include "../../lib/back/query.h"
#include "everything.h"
#include "home.h"
#include "archive.h"
#include "mapping.h"

extern struct everything the_histogram ;

/*
Use like this:


modes are
1	text
2	excel spreadsheet
3	html
4	bar
5	pie
6	scatter

*/


#define LINE_BEGIN -1
#define LINE_END -2
#define LINE_COLSEP -3

/**********************************************************************
* functions to do output conversion
*/


/*
* quote things so they get imported properly.  You could look for the
* list of characters that require to be quoted, but I believe this
* will be correct in all case.  And I won't have to handle a bug report
* about missing some oddball special character.
*/
static excel_expand(int width, char *s, char *color)
{
if (width == LINE_END)
	printf("\n");
if (width == LINE_COLSEP)
	printf(",");
if (width < 0)
	return;
putchar('"');
while (*s)
	{
	if (*s == '"')
		putchar('"');
	putchar(*s++);
	}
putchar('"');
}

static print_plain(int width, char *s, char *color)
{
if (width == LINE_END) 
	printf("\n");
if (width < 0)
	return;
printf("%-*s ",width,s);
}

static html_expand(int width, char *s, char *color)
{
if (width == LINE_END)
	printf("</tr>\n");
if (width == LINE_BEGIN)
	printf("<tr>\n");
if (width < 0)
	return;
printf("<td>");
if (color)
	printf("<font color=\"%s\">",color);
while (*s)
	{
	if (*s == '&')
		printf("&amp;");
	else if (*s == '<')
		printf("&lt;");
	else if (*s == '>')
		printf("&gt;");
	else if (*s == '"')
		printf("&quot;");
	else
		putchar(*s);
	s++;
	}
printf("</font>");
printf("</td>\n");
}



/**********************************************************************
* compute real value of the data point for nuke_percent
*/

int do_nuke;
	/*
	* 0 - no nukeage
	* 1 - nuke on count
	* 2 - nuke on sum
	*/

static counter max_value;
static counter nuke_percent_counter;
static counter counter_100;

static compute_range( void *b, int size, int *size_array)
{
counter data[2]; struct vsch_buf buf; counter data_value;
int x;

vsch_initbuf(&buf,0);
buf.buffer = b;
buf.buffer_size=buf.buffer_offset=size;
buf.read_point=0;

vsch_ptrbuf(&buf,VSCH_INT, (void *)0);
for (x=0; x < the_histogram.axis_map.r.num_fields; x++)
	{
	if (the_histogram.axis_map.r.f[x].diminished)
		continue;
	vsch_ptrbuf(&buf,the_histogram.axis_map.r.f[x].type
		, &the_histogram.axis_map.r.f[x].size);
	}
/* pluck out data elements */
if (the_histogram.axis_map.use_data)
	vsch_getbuf(&buf,VSCH_COUNTER, &data[1], (void *)0);
if (vsch_getbuf(&buf,VSCH_COUNTER, &data[0], (void *)0 ))
	goto error;

data_value = data[0];	/* or sometimes data[1] */

if (counter_cmp(data_value,max_value) > 0)
	max_value = data_value;

return 0;

error:
printf("\nERROR!\n");
return 1;
}



/**********************************************************************
* plain text
*/

int (*print_mode)(int mode, char *text, char *color);

static print_header()
{
int x;
struct axis_map *am = &the_histogram.axis_map;
print_mode(LINE_BEGIN,"",NULL);
for (x=0; x < am->r.num_fields; x++)
        {
        if (am->r.f[x].diminished)
                continue;
	print_mode(am->r.f[x].printwidth, am->r.f[x].label,NULL);
	print_mode(LINE_COLSEP,"",NULL);
        }
if (am->use_data)
	{
	print_mode(10,am->data_label,NULL);
	print_mode(LINE_COLSEP,"",NULL);
	}
print_mode(10,am->count_label,NULL);
print_mode(LINE_END,"",NULL);
}

static print_line(void *pb, int size, int *size_array)
{
char buffer[1000];
int x,y;
counter data[2];
struct vsch_buf buf;
unsigned char *b = pb;

vsch_initbuf(&buf,0);

if (do_nuke)
	{
	counter x;
	/*
	* lame - have to walk down to the end to find the data
	* value.  This doesn't hurt too bad because we are just
	* displaying query results, but we could skip it completely
	* if the data value was at the beginning instead of at
	* the end.
	*/
	buf.buffer = b;
	buf.buffer_size=buf.buffer_offset=size;
	buf.read_point=0;
	vsch_ptrbuf(&buf,VSCH_INT, (void *)0);
	for (x=0; x < the_histogram.axis_map.r.num_fields; x++)
		{
		if (the_histogram.axis_map.r.f[x].diminished)
			continue;
		vsch_ptrbuf(&buf,the_histogram.axis_map.r.f[x].type
			, &the_histogram.axis_map.r.f[x].size);
		}
	/* pluck out data elements */
	if (the_histogram.axis_map.use_data)
		vsch_getbuf(&buf,VSCH_COUNTER, &data[1], (void *)0);
	/* only have to check error on last one to see error */
	if (vsch_getbuf(&buf,VSCH_COUNTER, &data[0], (void *)0 ))
		goto error;
	if (do_nuke == 1)
		x = counter_mul(data[0], counter_100);
	else
		x = counter_mul(data[1], counter_100);
	x = counter_div(x, max_value);
	if (counter_cmp(x,nuke_percent_counter) < 0)
		return;
	}

print_mode(LINE_BEGIN,"",NULL);


buf.buffer = b;
buf.buffer_size=buf.buffer_offset=size;
buf.read_point=0;

vsch_getbuf(&buf,VSCH_INT, &y, (void *)0);
if (y != 1)
	printf("\nYOW! type != 1!\n");
for (x=0; x < the_histogram.axis_map.r.num_fields; x++)
	{
	int tmpsize;
	unsigned char *s;
	struct display_mode *dm;
	if (the_histogram.axis_map.r.f[x].diminished)
		continue;
	s= b + vsch_ptrbuf(&buf,the_histogram.axis_map.r.f[x].type
		, &tmpsize );
	dm=mapping_lookup(the_histogram.axis_map.r.f[x].mapping,
		s, the_histogram.axis_map.r.f[x].size);
	if (dm && dm->string[0])
		print_mode(the_histogram.axis_map.r.f[x].printwidth,
			dm->string, dm->color);
	else
		{
		if ((the_histogram.axis_map.r.f[x].type == VSCH_TIME_T)
			&& (print_mode == excel_expand))
			{
			struct tm * t;
			int i;
			time_t clock;
			memcpy(&i,s,sizeof(int));
			clock = i;
			t= localtime(&clock);
			strftime(buffer,sizeof(buffer),"%e-%b-%Y %H:%M:%S", t);
			print_mode(the_histogram.axis_map.r.f[x].printwidth,
				buffer,NULL);
			}
		else
			{
			vsch_print_type(buffer, s,
				the_histogram.axis_map.r.f[x].type ,
				tmpsize
				);
			print_mode(the_histogram.axis_map.r.f[x].printwidth,
				buffer,dm?dm->color:NULL);
			}
		}
	print_mode(LINE_COLSEP,"",NULL);
	}
/* pluck out data elements */
if (the_histogram.axis_map.use_data)
	{
	vsch_getbuf(&buf,VSCH_COUNTER, &data[1], (void *)0);
	vsch_print_type(buffer, &data[1], VSCH_COUNTER, 0);
	print_mode(10,buffer,NULL);
	print_mode(LINE_COLSEP,"",NULL);
	}
/* only have to check error on last one to see error */
if (vsch_getbuf(&buf,VSCH_COUNTER, &data[0], (void *)0 ))
	goto error;
vsch_print_type(buffer, &data[0], VSCH_COUNTER, 0);
print_mode(10,buffer,NULL);
print_mode(LINE_END,"",NULL);
return 1;

error:
printf("\nERROR!\n");
return 1;
}


/*
*******************************************************************************
* begin graph related stuff
*/


static int graph_mode;	/* controls how we do some of it */
static bufferize_graph(void *pb, int size, int *size_array)
{
char buffer[1000];
int x,y;
counter data[2];
struct vsch_buf buf;
unsigned char *b =pb;
struct display_mode *dm;


vsch_initbuf(&buf,0);

if (do_nuke)
	{
	counter x;
	/*
	* lame - have to walk down to the end to find the data
	* value.  This doesn't hurt too bad because we are just
	* displaying query results, but we could skip it completely
	* if the data value was at the beginning instead of at
	* the end.
	*/
	buf.buffer = b;
	buf.buffer_size=buf.buffer_offset=size;
	buf.read_point=0;
	vsch_ptrbuf(&buf,VSCH_INT, (void *)0);
	for (x=0; x < the_histogram.axis_map.r.num_fields; x++)
		{
		if (the_histogram.axis_map.r.f[x].diminished)
			continue;
		vsch_ptrbuf(&buf,the_histogram.axis_map.r.f[x].type
			, &the_histogram.axis_map.r.f[x].size);
		}
	/* pluck out data elements */
	if (the_histogram.axis_map.use_data)
		vsch_getbuf(&buf,VSCH_COUNTER, &data[1], (void *)0);
	/* only have to check error on last one to see error */
	if (vsch_getbuf(&buf,VSCH_COUNTER, &data[0], (void *)0 ))
		goto error;
	if (do_nuke == 1)
		x = counter_mul(data[0], counter_100);
	else
		x = counter_mul(data[1], counter_100);
	x = counter_div(x, max_value);
	if (counter_cmp(x,nuke_percent_counter) < 0)
		return;
	}

buf.buffer = b;
buf.buffer_size=buf.buffer_offset=size;
buf.read_point=0;

dm = (void *)0;

vsch_getbuf(&buf,VSCH_INT, &y, (void *)0);
if (y != 1)
	printf("\nYOW! type != 1!\n");
for (x=0; x < the_histogram.axis_map.r.num_fields; x++)
	{
	void *tmpb;
	int tmpsize;
	if (the_histogram.axis_map.r.f[x].diminished)
		continue;
	tmpb = b + vsch_ptrbuf(&buf,the_histogram.axis_map.r.f[x].type
		, &tmpsize),
        dm=mapping_lookup(the_histogram.axis_map.r.f[x].mapping,
                tmpb, the_histogram.axis_map.r.f[x].size);
        if (dm && dm->string[0])
		printf("%s\n",dm->string);	/* sends label */
        else 
		{
		vsch_print_type(buffer, 
			tmpb,
			the_histogram.axis_map.r.f[x].type ,
			tmpsize
			);
		printf("%s\n",buffer);	/* sends label */
		}
	}
/* pluck out data elements */
if (the_histogram.axis_map.use_data)
	vsch_getbuf(&buf,VSCH_COUNTER, &data[1], (void *)0);
if (vsch_getbuf(&buf,VSCH_COUNTER, &data[0], (void *)0 ))
	goto error;

if (graph_mode == 6)
	{
	vsch_print_type(buffer, &data[0], VSCH_COUNTER, 0);	/* or data[1] */
	printf("%s\n",buffer);	/* sends raw data point */
	data[0] = counter_div( counter_mul(data[0],counter_100),
		max_value );
	vsch_print_type(buffer, &data[0], VSCH_COUNTER, 0);	/* or data[1] */
	printf("%s\n",buffer);	/* sends proportional data point */
	}
else
	{
	vsch_print_type(buffer, &data[0], VSCH_COUNTER, 0);	/* or data[1] */
	printf("%s\n",buffer);	/* sends data point */
	}

if (dm)
	printf("%s\n",dm->color); /* sends color */
else
	printf("%s\n","blue");	  /* sends color */

return 0;

error:
printf("\nERROR!\n");
return 1;
}

/*
*******************************************************************************
*/


print_histogram(struct hist *h, int mode, int nuke_percent, int nuke_which)
{
void *b[MAX_AXES];
int sizes[MAX_AXES];
struct vsch_buf record_buffer;
int x;

vsch_initbuf(&record_buffer,0);

max_value = counter_from_int(1);
counter_100 = counter_from_int(100);
if (nuke_percent > 0)
	{
	nuke_percent_counter = counter_from_int(nuke_percent);
	bufferize_func = compute_range;
	write_array_sub(h->data, b, sizes, 0, 
			&the_histogram.axis_map, 0, &record_buffer);
	if (nuke_which)
		do_nuke = 2;
	else
		do_nuke = 1;
	}
else
	{
	do_nuke = 0;
	}

switch (mode)
	{
case 1:
	print_mode = print_plain;
	print_header();

	bufferize_func = print_line;
	print_mode = print_plain;
	break;
case 2:
	print_mode = excel_expand;
	print_header();

	bufferize_func = print_line;
	print_mode = excel_expand;
	break;
case 3:
	print_mode = html_expand;
	printf("<table border>\n");
	print_header();

	bufferize_func = print_line;
	print_mode = html_expand;
	break;
case 4:
	printf("bar-graph\n1\n");
	printf("1\n%s\n",my_recorder_name);	/* graph title */
	printf("%s\n",the_histogram.axis_map.count_label); /* or data label */
	/*
	* should always find one
	*/
	for (x=0; x < the_histogram.axis_map.r.num_fields; x++)
		{
		if (the_histogram.axis_map.r.f[x].diminished)
			continue;
		printf("%s\n",the_histogram.axis_map.r.f[x].label);
		}

	graph_mode = mode;
	bufferize_func = bufferize_graph;
	print_mode = (void *)0;
	break;
case 5:
	printf("pie-chart\n1\n");
	printf("1\n%s\n",my_recorder_name);
	graph_mode = mode;
	bufferize_func = bufferize_graph;
	print_mode = (void *)0;
	break;
case 6:
	printf("scatter-plot\n1\n");
	printf("1\n%s\n",my_recorder_name);
	if (! do_nuke)
		{
		bufferize_func = compute_range;
		write_array_sub(h->data, b, sizes, 0, 
				&the_histogram.axis_map, 0, &record_buffer);
		}
	graph_mode = mode;
	bufferize_func = bufferize_graph;
	print_mode = (void *)0;
	break;
default:
	printf("Bogus print_histogram mode: %d\n",mode);
	return;
	}

write_array_sub(h->data, b, sizes, 0, &the_histogram.axis_map, 0, &record_buffer);


switch (mode)
	{
case 3:
	printf("</table>\n");
	break;
	}
}
