/*
        Copyright(C) 1997 Network Flight Recorder, Inc.  
        All rights reserved.

        Use and distribution of this software and its source code
        are governed by the terms and conditions of the
        Network Flight Recorder Software License ("LICENSE.TXT")

By Mark Sienkiewicz / NFR

*/

/*
*
* Histogram recorder.
*	
*/
#include "sys-include.h"

#define DEBUG(x,y) { if (debug >= x) { y; fflush(debugfp); } }
FILE *debugfp;
int debug=0;

extern int new_cell_alerts;

/*
*******************************************************************************
* 
* boring include files
*
*/

#include "allocaters.h"
#include "array.h"
#include "cf.h"
#include "counter.h"
#include "vsch.h"
#include "../../lib/back/query.h"
#include "everything.h"


struct everything the_histogram = { 0 } ;

counter counter_0, counter_1;

main( int argc, char **argv )
{

	/*
	* read config files, create initial data structures, etc.
	*/
	histogram_init(argv[1], &the_histogram, (PCf *)0);

	/*
	* load the current histograms from their data files.  This
	* lets us pick up where we left off (losing only a little data)
	* if we are restarted.  It also lets us keep the all-time
	* histogram accurate.
	*/
	if (have_all_time)
		read_histogram_file(&the_histogram.all_time,all_time_name,
			0,(void *)0);
	if (have_current)
		read_histogram_file(&the_histogram.current,current_name,
			0,(void *)0);

	calc_alert_names(argv[1]);


	/*
	* process input forever
	*/
	for (;;)
		{
		int type;
DEBUG(12,fprintf(debugfp,"attempt to read stdin\n"))
		if (! intDecode(stdin, &type ))
			break;
DEBUG(11,fprintf(debugfp,"read input type %d\n",type))
		switch (type)
			{
		case 1:
			{
			extern void *read_record_buffer;
			int n,x;
			n = read_record(stdin, &the_histogram.axis_map.r,					the_histogram.axis_map.use_data);
			if (n < 0)
				break;
#if 0
			for (x=0; x<n; x++)
				printf(" %02x ",*(unsigned char *)
					(read_record_buffer+x));
			printf("\n\n\n\n");
#endif
			common_decoder( read_record_buffer, &the_histogram );
			}
			break;
		case 57007: /* timestamp from nfrd */
			{
			int tyme;
			if (!intDecode(stdin, &tyme))
				break;
			time_progresses(&the_histogram, tyme);
			}
			break;
		default:
			fprintf(stderr,"%s: recieved bogus schema type: %x\n",
				my_recorder_name,
				type);
			abort();	/* shouldn't happen - unrecoverable */
					/* even if it does. */
			}
		}

	DEBUG(5,
		fprintf(debugfp,"writing histograms at end of main\n"))

	if (have_all_time)
		{
		write_histogram_file(&the_histogram.all_time, all_time_name );
		free_histogram(&the_histogram.all_time);
		}
	if (have_current)
		{
		write_histogram_file(&the_histogram.current,  current_name );
		free_histogram(&the_histogram.current);
		write_histogram_file(&the_histogram.previous,  previous_name );
		free_histogram(&the_histogram.previous);
		}
	close_histogram_archive();
}



/*
* common_decoder decodes the individual schema and enters the data into
* the histogram
*/
common_decoder(void *p, struct everything *h)
{
int e;
struct axis_map *a = &h->axis_map;
int time = a->r.time;
	/*
	* do timekeeping if necessary.
	*/
if (time > h->time_marching)
	{
DEBUG(6,fprintf(debugfp,"time %d > time_marching %d\n",time,h->time_marching))
	time_progresses(h, time);
	the_histogram.time_marching= time;
	}

	/*
	* add the data into the all-time histogram.  Note that the
	* timekeeping may have rotated the histograms.
	*/
DEBUG(6,fprintf(debugfp,"entering histogram with time %d\n",time))
if (have_all_time)
	if (enter_histogram(&h->all_time, a, (void *) p, (void *)0))
		{
		/*
		* Generate an alert on a new cell.
		*/
		DEBUG(1,fprintf(debugfp,"new cell\n"))
		if (new_cell_alerts)
			toss_new_cell_alert(p,a);
		}

	/*
	* add the data into the proper interval, if we are in fact keeping
	* short term histograms
	*/
if (have_current)
	{
	e=0;
	if (time >= h->current.min_time &&
		time <= h->current.max_time )
		{
		enter_histogram(&h->current,a,(void *)p,(void *)0); 
		e=1;
		}
	else if (time >= h->previous.min_time &&
		time <= h->previous.max_time )
		{
		enter_histogram(&h->previous,a,(void *)p,(void *)0); 
		e=1;
		}

	if (!e)
		{
		DEBUG(1, fprintf(debugfp,
		"record not in time range. t=%d cur=[%d,%d] prev=[%d,%d]\n",
			time,
			h->current.min_time, h->current.max_time,
			h->previous.min_time, h->previous.max_time 
			))
		}
	}
return 1;
}

/*
* rotate_histogram
*
* It's time to start a new histogram.
* The current becomes the previous, the old previous gets archived,
* we create a new current.
*
* bug: don't crash while this process is going on.
*/

rotate_histogram(time_t tyme)
{
char b[20];
struct hist *new;
if (!have_current)
	{
	DEBUG(5,fprintf(debugfp,"rotating histogram suppressed\n"))
	return;
	}
DEBUG(5,fprintf(debugfp,"rotating histogram - write previous\n"))


/*
* archive the old previous and discard it
*/
write_histogram_archive(&the_histogram.previous, my_recorder_name, "data");
free_histogram(&the_histogram.previous);

/*
* current becomes previous
*/
the_histogram.previous=the_histogram.current;

/*
* write the new previous into the previous file
*/
write_histogram_file(&the_histogram.previous,  previous_name );

/*
* create a new current
*/
new= &the_histogram.current;

new->min_time = ( tyme / new->interval ) * new->interval;
new->max_time = new->min_time + new->interval;

new->tag=array_tag_create();
new->data = array_create(new->tag);

if (debug >= 2)
	{ 
	array_dump_performance(debugfp);
	fflush(debugfp);
	}
DEBUG(5,fprintf(debugfp,"rotating histogram - write new\n"))
/*
* write the new current file.
*/
write_histogram_file(new, current_name);
}


time_progresses(struct everything *h, time_t tyme)
{
	/*
	* We know time is later than the next "interesting" time.
	* this function should do this:
	* - write old histograms to files
	* - shift down the short term histogram
	* - free any obsolete histograms
	* - zero out any new ones
	*/

DEBUG(7,fprintf(debugfp,"time passing %d\n",tyme))

if (have_current && (tyme >= h->current.max_time))
	{
	DEBUG(5,fprintf(debugfp,"time to rotate short term histogram\n"))
	rotate_histogram(tyme);
	}
else if (tyme > h->last_update + h->sync_time)
	{
	h->last_update=tyme;
	DEBUG(5,fprintf(debugfp,"writing all histograms because time passed\n"))
	if (have_all_time)
		write_histogram_file(&h->all_time, all_time_name);
	if (have_current)
		{
		write_histogram_file(&h->current, current_name);
		write_histogram_file(&h->previous, previous_name);
		}
	}
}

