/*
        Copyright(C) 1997 Network Flight Recorder, Inc.
        All rights reserved.

        Use and distribution of this software and its source code
        are governed by the terms and conditions of the
        Network Flight Recorder Software License ("LICENSE.TXT")

By Mark Sienkiewicz / NFR

This architecture is kind of tedious.  There is a print_header,
print_record, and print_trailer for each output type.  It would be
nice if a lot more of the code was shared by the bunch of them.

*/


#include "sys-include.h"

#include "counter.h"
#include "vsch.h"
#include "cf.h"
#include "list.h"
#include "mapping.h"

extern struct columns *thislist;

/*
* this is a buffer that accumulates the record as it arrives.
*/
extern int record_size;
extern int this_record_size;
extern void *buffer;

int previous_time = -1;

print_record_text()
{
int x;
int i;
counter c;
unsigned char *p = buffer;
int remaining=this_record_size;
char buf[1000];

/*
walk through the buffer printing each field
*/

	{
/*
* bug: fix this - localize at least, if this function is going to stay
*/
	int n; char *s; time_t t;
	memcpy(&n,&thislist->time,sizeof(int));
	t=n;
	s = ctime(&t);
	s[24]=0;
	printf("%s ",s);
	previous_time = thislist->time;
	}

for (x=0; x<thislist->r.num_fields; x++)
	{
	struct display_mode *dm;
	if (thislist->c[x].diminished)
		continue;
	dm = mapping_lookup(thislist->r.f[x].mapping,
		thislist->c[x].offset, thislist->r.f[x].size);
	if (dm && dm->string[0] )
		{
		printf("%-*s ",thislist->r.f[x].printwidth, dm->string);
		}
	else
	 switch (thislist->r.f[x].type)
		{
	case VSCH_COUNTER:
		vsch_print_type(buf,thislist->c[x].offset,VSCH_COUNTER,0);
		printf("%*s ",thislist->r.f[x].printwidth,buf);
		break;
	case VSCH_INT:
	case VSCH_TIME_T:
		vsch_print_type(buf,thislist->c[x].offset,VSCH_INT,0);
		printf("%*s ",thislist->r.f[x].printwidth,buf);
		break;
	case VSCH_IP:
		vsch_print_type(buf,thislist->c[x].offset,VSCH_IP,0);
		printf("%-*s ",thislist->r.f[x].printwidth,buf);
		break;
	case VSCH_STR:
		{
		int w = 0;
		i=thislist->r.f[x].size;
		p = thislist->c[x].offset;
		while (i > 0)
			{
			putchar(*(char *)p);
			p++; i--; w++;
			}
		while (w < thislist->r.f[x].printwidth)
			{ putchar(' ');  w++; }
		putchar(' ');
		}
		break;
	case VSCH_ETHERMAC:
		{
		vsch_print_type(buf,thislist->c[x].offset,VSCH_ETHERMAC,0);
		printf("%-*s ",thislist->r.f[x].printwidth,buf);
		break;
		}
	case VSCH_BLOB:
		{
		int w = 0;
		i=thislist->r.f[x].size;
		p = thislist->c[x].offset;
		while (i > 0)
			{
			printf("%02x ",*(unsigned char *)p);
			p++; i--; w += 3;
			}
		while (w < thislist->r.f[x].printwidth)
			{ putchar(' ');  w++; }
		putchar(' ');
		}
		break;
	default:
		abort();
		}
	}
printf("\n");
}

print_header_text()
{
int x;
printf("%-24s ","Time");
for (x=0; x<thislist->r.num_fields; x++)
	{
	if (thislist->c[x].diminished)
		continue;
	switch (thislist->r.f[x].type)
		{
	case VSCH_COUNTER:
	case VSCH_INT:
	case VSCH_TIME_T:
	case VSCH_ETHERMAC:
	case VSCH_IP:
	case VSCH_STR:
	case VSCH_BLOB:
		printf("%-*s ",thislist->r.f[x].printwidth
				,thislist->r.f[x].label);
		break;
	default:
		abort();
		}
	}
printf("\n");
}

print_trailer_text()
{
}

/*****************************************************************************
******************************************************************************
******************************************************************************
******************************************************************************
******************************************************************************/

print_record_html()
{
int x;
int i;
counter c;
unsigned char *p = buffer;
int remaining=this_record_size;
char buf[1000];

/*
walk through the buffer printing each field
*/
printf("<tr>");

	{
/*
* bug: fix this - localize at least, if this function is going to stay
*/
	int n; char *s; time_t t;
	memcpy(&n,&thislist->time,sizeof(int));
	t=n;
	s = ctime(&t);
	s[24]=0;
	printf("<td>%s</td>",s);
	previous_time = thislist->time;
	}

for (x=0; x<thislist->r.num_fields; x++)
	{
	struct display_mode *dm;
	if (thislist->c[x].diminished)
		continue;
	printf("<td>");
	dm = mapping_lookup(thislist->r.f[x].mapping,
		thislist->c[x].offset, thislist->r.f[x].size);
	if (dm)
		printf("<font color=\"%s\">",dm->color);
	if (dm && dm->string[0] )
		{
		printf("%-*s ",thislist->r.f[x].printwidth, dm->string);
		}
	else
	 switch (thislist->r.f[x].type)
		{
	case VSCH_COUNTER:
	case VSCH_INT:
	case VSCH_TIME_T:
	case VSCH_IP:
	case VSCH_ETHERMAC:
		vsch_print_type(buf,thislist->c[x].offset,
			thislist->r.f[x].type,0);
		printf("%s",buf);
		break;
	case VSCH_STR:
	case VSCH_BLOB:
		i=thislist->r.f[x].size;
		p = thislist->c[x].offset;
		while (i > 0)
			{
			char c = * (char *)p;
			if (c == '&')
				printf("&amp;");
			else if (c == '<')
				printf("&lt;");
			else if (c == '>')
				printf("&gt;");
			else if (c == '"')
				printf("&quot;");
			else
				putchar(c);
			p++; i--;
			}
		break;
	default:
		abort();
		}
	if (dm)
		printf("</font>");
	printf("</td>");
	}
printf("</tr>\n");
}

print_header_html()
{
int x;
printf("<table border width=\"100%%\">\n");
printf("<tr>");
printf("<td><center>Time</center></td>");
for (x=0; x<thislist->r.num_fields; x++)
	{
	if (thislist->c[x].diminished)
		continue;
	printf("<td><center>");
	quote_html(thislist->r.f[x].label);
	printf("</center></td>\n");
	}
printf("</tr>\n");
}

print_trailer_html()
{
printf("</table>\n");
}

quote_html(char *s)
{
while (*s)
	{
	char c = * s++;
	if (c == '&')
		printf("&amp;");
	else if (c == '<')
		printf("&lt;");
	else if (c == '>')
		printf("&gt;");
	else if (c == '"')
		printf("&quot;");
	else
		putchar(c);
	}
}

/*****************************************************************************
******************************************************************************
******************************************************************************
******************************************************************************
******************************************************************************/

print_record_excel()
{
int x;
int i;
counter c;
unsigned char *p = buffer;
int remaining=this_record_size;
char buf[1000];

/*
* output time
*/
	{
	struct tm * t;
	int i;
	time_t clock;
	memcpy(&i,&thislist->time,sizeof(int));
	clock = i;
	t= localtime(&clock);
	strftime(buf,sizeof(buf),"%e-%b-%Y %H:%M:%S", t);
	printf("\"%s\"",buf);
	}

for (x=0; x<thislist->r.num_fields; x++)
	{
	if (thislist->c[x].diminished)
		continue;
	printf(",");
	switch (thislist->r.f[x].type)
		{
	case VSCH_COUNTER:
	case VSCH_INT:
	case VSCH_TIME_T:
	case VSCH_IP:
	case VSCH_ETHERMAC:
		vsch_print_type(buf,thislist->c[x].offset,
			thislist->r.f[x].type,0);
		printf("%s",buf);
		break;
	case VSCH_STR:
	case VSCH_BLOB:
		i=thislist->r.f[x].size;
		p = thislist->c[x].offset;
		printf("\"");
		while (i > 0)
			{
			char c = * (char *)p;
			if (c == '"')
				printf("\"\"");
			else
				putchar(c);
			p++; i--;
			}
		printf("\"");
		break;
	default:
		abort();
		}
	}
printf("\n");
}

print_header_excel()
{
int x;
char *s;
printf("\"Time\"");
for (x=0; x<thislist->r.num_fields; x++)
	{
	if (thislist->c[x].diminished)
		continue;
	printf(",\"");
	for (s= thislist->r.f[x].label; *s; s++)
		if (*s == '"')
			printf("\"\"");
		else
			putchar(*s);
	printf("\"");
	}
printf("\n");
}

print_trailer_excel()
{
}

