/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   ackdump.c
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

/* define DEBUG_LEV 1 to get dumps of the queries, and enable cmdline
   debugging */
#define DEBUG_LEV 1

#include "sys-include.h"

#include "cgiutils.h"
#include "libnfr.h"
#include "nfr_res.h"

int main(int argc, char **argv)
{
  ULONG Result = 0;
  PCf QueryFields = NULL;
  DIR *dir = NULL;
  struct dirent *entry = NULL;
  ULONG Offset = 0;
  ULONG Serial = 0;
  ULONG IdxSerial = 0;
  char *TargetDir = NULL;
  char *LogName = NULL;
  char *IdxName = NULL;
  FILE *IdxFile = NULL;
  FILE *LogFile = NULL;
  char Buf[256];
  int Size = 0;
  int i = 0;
  int r = 0;
  int Start = 0;
  int Stop = 0;
  char *errStr = NULL;

  CgiGoHome();

  CGI_DEBUGGER();

  QueryFields = Cf_New(0);
  Cgi_QueryToCf(QueryFields);

  TargetDir = Cf_GetVal("target_dir", QueryFields);
  fprintf(stderr, "TargetDir=%s\n", TargetDir);
  if(!TargetDir)
    {
      Cgi_Response("ackdump: no target_dir", 0, 1);
      exit(1);
    }

  LogName = Cf_GetVal("log_name", QueryFields);
  fprintf(stderr, "LogName=%s\n", LogName);
  if(!LogName)
    {
      Cgi_Response("ackdump: no log_name", 0, 1);
      exit(1);
    }

  IdxName = Cf_GetVal("idx_name", QueryFields);
  fprintf(stderr, "IdxName=%s\n", IdxName);
  if(!IdxName)
    {
      Cgi_Response("ackdump: no idx_name", 0, 1);
      exit(1);
    }

  dir = opendir(TargetDir);

  if (!dir) 
    {
      errStr = nfrConcat("ackdump: ", strerror(errno), 
			 ": ", TargetDir, NULL);
      if(errno == ENOENT)
	{
	  Cgi_Response(errStr, 0, 7);
	  exit(0);
	}
      Cgi_Response(errStr, 0, 1);
      exit(1);
    }  

  IdxFile = fopen(IdxName, "r");
  if(!IdxFile)
    {
      errStr = nfrConcat("ackdump: ", strerror(errno), 
			 ": ", IdxName, NULL);
      if(errno == ENOENT)
	{
	  Cgi_Response(errStr, 0, 7);
	  exit(0);
	}
      Cgi_Response(errStr, 0, 1);
      exit(1);
    }  
  
  LogFile = fopen(LogName, "r");
  if(!LogFile)
    {
      errStr = nfrConcat("ackdump: ", strerror(errno), 
			 ": ", LogName, NULL);
      if(errno == ENOENT)
	{
	  Cgi_Response(errStr, 0, 7);
	  exit(0);
	}
      Cgi_Response(errStr, 0, 1);
      exit(1);
    }  
  
  Cgi_Response("dumping selected alerts", 0, 0);
  
  Result = fread(&Offset, sizeof(Offset), 1, IdxFile);
  while (entry = readdir(dir))
    {
      Serial = strtoul(entry->d_name,NULL,10);
      if(Serial > 0)
	{
	  /* fprintf(stderr, "looking for %ld\n", Serial); */
	  IdxSerial = 0;
	  Start = ntohl(Offset);
	  while(TRUE)
	    {
	      fread(&IdxSerial, sizeof(IdxSerial), 1, IdxFile);
	      IdxSerial = ntohl(IdxSerial);
	      /* fprintf(stderr, "trying one, #%ld\n", IdxSerial);  */
	      Result = fread(&Offset, sizeof(Offset), 1, IdxFile);
	      if (Result != 1) 
		{
		  fseek(LogFile, 0, SEEK_END);
		  Stop = ftell(LogFile);
		}
	      else
		Stop = ntohl(Offset);
	      
	      if(IdxSerial == Serial)
		{
		  fprintf(stderr, "found one, #%ld\n", IdxSerial);
		  fprintf(stderr, "log offset = %ld\n", Start);
		  Size = Stop - Start;
		  fseek(LogFile, Start, SEEK_SET);
		  i = Size / sizeof(Buf);
		  r = Size % sizeof(Buf);
		  while(i--)
		    {
		      fprintf(stderr, "writing buffered\n");
		      fread(Buf,1,sizeof(Buf),LogFile);
		      fwrite(Buf,1,sizeof(Buf),stdout);
		    }
		  fprintf(stderr, "writing remainder\n");
		  fread(Buf,1,r,LogFile);
		  fwrite(Buf,1,r,stdout);
		  rewind(IdxFile);
		  Result = fread(&Offset, sizeof(Offset), 1, IdxFile);
		  break;
		}
	      if (Result != 1)
		break;
	      Start = Stop;
	    }
	  
	}
    }

  Cf_Delete(QueryFields);
  exit(0);
}
