/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   acknowledge.c
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

/* define DEBUG_LEV 1 to get dumps of the queries, and enable cmdline
   debugging */
#define DEBUG_LEV 1

#include "sys-include.h"

#include "cgiutils.h"
#include "libnfr.h"
#include "nfr_res.h"

int main(int argc, char **argv)
{
  ULONG Result = 0;
  PCf QueryFields = NULL;
  char *SerialStr = NULL;
  char *SrcPath = NULL;
  char *DestPath = NULL;
  char TimeStr[32];
  time_t Time = 0;
  FILE *Out = NULL;
  char *UserName = NULL;
  char *UserAddr = NULL;
  char *errStr = NULL;
  char *hostname = NULL;
  char *date = NULL;

  CgiGoHome();

  CGI_DEBUGGER();

  QueryFields = Cf_New(0);
  Cgi_QueryToCf(QueryFields);

  SerialStr = Cf_GetVal("serial", QueryFields);
  fprintf(stderr, "SerialStr=%s\n", SerialStr);
  if(!SerialStr)
    {
      Cgi_Response("acknowledge: no serial number", 0, 1);
      exit(1);
    }

  hostname = Cf_GetVal("hostname", QueryFields);
  fprintf(stderr, "hostname=%s\n", SerialStr);
  if(!SerialStr)
    {
      Cgi_Response("acknowledge: no hostname", 0, 1);
      exit(1);
    }

  date = Cf_GetVal("date", QueryFields);
  fprintf(stderr, "date=%s\n", SerialStr);
  if(!SerialStr)
    {
      Cgi_Response("acknowledge: no date", 0, 1);
      exit(1);
    }

  SrcPath = nfrPath("data/alerts/", hostname, "/", date, "/unacked/", SerialStr, NULL);
  DestPath = nfrPath("data/alerts/", hostname, "/", date, "/acked/", SerialStr, NULL);

  Result = rename(SrcPath, DestPath);

  if(Result != 0)
    {
      errStr = nfrConcat("acknowledge: cannot rename ", SrcPath,
			 " to ", DestPath, "failed: ", 
			 strerror(errno), NULL);
      Cgi_Response(errStr, 0, 1);
      exit(1);
    }

  Out = fopen(DestPath,"a");
  if(!Out)
    {
      errStr = nfrConcat("acknowledge: ", DestPath, 
			 ": ", strerror(errno), NULL);
      Cgi_Response(errStr, 0, 1);
      exit(1);
    }

  Time = time(0);
  strcpy(TimeStr, ctime(&Time));
  TimeStr[19] = 0;
  UserName = getenv("REMOTE_USER");
  if(!UserName)
    UserName = "<unknown user>";
  UserAddr = getenv("REMOTE_ADDR");
  if(!UserAddr)
    UserAddr = "<unknown address>";
  fprintf(Out, 
	  "\n*\n"
	  "%s - Alert acknowledged by %s from %s" 
	  "\n*\n",
	  TimeStr, 
	  UserName,
	  UserAddr);
  fclose(Out);

  errStr = nfrConcat("alert ", SerialStr, " acknowledged", NULL);
  Cgi_Response(errStr,0,0);

  Cf_Delete(QueryFields);
  exit(0);
}
