/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   alertdump.c
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

/* define DEBUG_LEV 1 to get dumps of the queries, and enable cmdline
   debugging */
#define DEBUG_LEV 1

#include "sys-include.h"

#include "cgiutils.h"
#include "libnfr.h"
#include "nfr_res.h"

int main(int argc, char **argv)
{
  FILE *In = NULL;
  FILE *IdxFile = NULL;
  ULONG Offset = 0;
  PCf QueryFields = NULL;
  BYTE Buf[256];
  int Bytes = 0;
  int Result = 0;
  char *LogName = NULL;
  char *IdxName = NULL;
  char *errStr = NULL;

  CgiGoHome();

  CGI_DEBUGGER();

  QueryFields = Cf_New(0);
  Cgi_QueryToCf(QueryFields);

  LogName = Cf_GetVal("log_name", QueryFields);
  if(!LogName)
    {
      Cgi_Response("alertdump: no log_name", 0, 1);
      exit(1);
    }

  IdxName = Cf_GetVal("idx_name", QueryFields);
  if(!IdxName)
    {
      Cgi_Response("alertdump: no idx_name", 0, 1);
      exit(1);
    }

  IdxFile = fopen(IdxName, "r");
  if(!IdxFile)
    {
      errStr = nfrConcat("alertdump: ", strerror(errno), 
			 ": ", IdxName, NULL);
      if(errno == ENOENT)
	{
	  Cgi_Response(errStr, 0, 7);
	  exit(0);
	}
      Cgi_Response(errStr, 0, 1);
      exit(1);      
    }

  In = fopen(LogName, "r");
  if(!In)
    {
      errStr = nfrConcat("alertdump: ", strerror(errno), 
			 ": ", LogName, NULL);
      if(errno == ENOENT)
	{
	  Cgi_Response(errStr, 0, 7);
	  exit(0);
	}
      Cgi_Response(errStr, 0, 1);
      exit(1);
    }
  

  Cgi_Response("dumping alert log", 0, 0);

  while ((Bytes=fread(Buf, 1, sizeof(Buf),In)) > 0)
    fwrite(Buf,1,Bytes,stdout);
  
  fclose(In);

  Cf_Delete(QueryFields);

  SExit(0);
}
