/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   fifordr.c
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

#include "sys-include.h"

/* define DEBUG_LEV 1 to get dumps of the queries, and enable cmdline
   debugging */
#define DEBUG_LEV 1

#include "cgiutils.h"
#include "libnfr.h"
#include "nfr_res.h"

char *gFifoName = NULL;

int OnIdle(PDGServer pDGSrv)
{
  static int TickCount = 0;

  /* if Tick is ever changed to non-zero you betta call htonl()!  */
  ULONG Tick = 0;	

  fprintf(stderr,"tick %d\n", TickCount++);
  fwrite(&Tick, sizeof(Tick), 1, stdout);
  fflush(stdout);
  return NFR_ERR_SUCCESS;
}


int OnData(PDGram pDGram, PDGServer pDGSrv)
{
  ULONG Tick = 0L;
  ULONG DataLen = 0L;

  Tick = htonl(1L);
  DataLen = (ULONG) pDGram->Len;
  DataLen = htonl(DataLen);
  fwrite(&Tick, sizeof(Tick), 1, stdout);
  fwrite(&DataLen, sizeof(DataLen), 1, stdout );
  fwrite( (char *)DGram_GetData(pDGram), pDGram->Len, 1, stdout );
  fflush(stdout);
  return NFR_ERR_SUCCESS;
}

int OnErr(ULONG Error, PDGServer pDGSrv)
{
  /* if we just got an EOF in the pipe and nothing else, that's ok.
     it happens whenever a client closes its end of the pipe,
     also under bsdi we get EAGAIN sometimes */
  /****
  if((Error == LIBNFR_ERR_EOF) || (Error == LIBNFR_ERR_AGAIN))
    return NFR_ERR_SUCCESS;
  ***/

  if(Error == LIBNFR_ERR_EOF)
    return NFR_ERR_SUCCESS;
  return Error;
}

void OnSignal(int SigNo)
{
  fprintf(stderr,"fifordr: exiting on signal %d\n", SigNo);
  Fifo_Destroy(gFifoName);
  SExit(0);
}

int main(int argc, char **argv)
{
  /* non-reentrant */
  int Result = 0;
  PCf QueryFields;
  TDGServer *pDGSrv;
  
  MimeHeader(stdout,"200 OK", NULL, "text/plain", NULL);

  CGI_DEBUGGER();

  fflush(stdout);

  signal(SIGPIPE, OnSignal);

  QueryFields = Cf_New(0);
  Cgi_QueryToCf(QueryFields);	/* handle GETs or POSTs */


  gFifoName = Cf_ExpandVal("fifo_name", QueryFields);
  Result = Fifo_Create(gFifoName);
  if (Result != LIBNFR_ERR_SUCCESS)
    return NFR_ERR_UNDEFINED;
  else
    {
      pDGSrv = DGServer_New();
      pDGSrv->fdRead = Fifo_OpenRead(gFifoName);
      pDGSrv->DataProc = OnData;
      pDGSrv->IdleProc = OnIdle;
      pDGSrv->ErrProc = OnErr;
      pDGSrv->MaxWait = 1000;

      Result = DGServer_Run(pDGSrv);

      close(pDGSrv->fdRead);
      Fifo_Destroy(gFifoName);
      
      DGServer_Delete(pDGSrv);
    }
  SExit(Result);
}
