/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   mkfac.c
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

#include "sys-include.h"

#define DEBUG_LEV 1

#include "cgiutils.h"
#include "libnfr.h"

int main(int argc, char **argv)
{
  PCf pCf;
  PCf QueryFields;
  char *name = NULL;
  char *type = NULL;
  ULONG Result = 0;
  ULONG NBOResult = 0;
  char basePath[] = "etc/alertd/facilities/";
  char *tpltPath = NULL;
  char *destPath = NULL;
  FILE *inFile = NULL;
  FILE *outFile = NULL;
  char buf[256];
  int bytes = 0;
  char *errStr = NULL;

  CgiGoHome();

  CGI_DEBUGGER();

  QueryFields = Cf_New(0);
  Cgi_QueryToCf(QueryFields);
  pCf = Cf_New(0);
  name = Cf_ExpandVal("name", QueryFields);
  if(!name)
    {
      Cgi_Response("CGI Error: no \"name\" field", 0, 1);
      exit(1);
    }

  type = Cf_ExpandVal("type", QueryFields);
  if(!type)
    {
      Cgi_Response("CGI Error: no \"type\" field", 0, 1);
      exit(1);
    }

  destPath = nfrConcat(basePath, name, NULL);
  fprintf(stderr,"making dir: %s\n", destPath);
  Result = mkdir(destPath, 0700);
  if(Result)
    {
      errStr = nfrConcat(destPath, ": ", strerror(errno), NULL);
      Cgi_Response(errStr, 0, 1);
      exit(1);
    }

  /* first copy the gui.cf file */
  tpltPath = nfrConcat(basePath, "templates/", type, "/gui.cf", NULL);
  inFile = fopen(tpltPath, "r");
  if(!inFile)
    {
      errStr = nfrConcat(tpltPath, ": ", strerror(errno), NULL);
      Cgi_Response(errStr, 0, 1);
      exit(1);
    }
  destPath = nfrConcat(basePath, name, "/gui.cf", NULL);
  outFile = fopen(destPath, "w");
  if(!outFile)
    {
      errStr = nfrConcat(destPath, ": ", strerror(errno), NULL);
      Cgi_Response(errStr, 0, 1);
      exit(1);
    }
  while ((bytes=fread(buf, 1, sizeof(buf), inFile)) > 0)
    fwrite(buf, 1, bytes, outFile);
  fclose(inFile);
  fclose(outFile);

  /* now copy the queue.cf file */
  tpltPath = nfrConcat(basePath, "templates/", type, "/queue.cf", NULL);
  inFile = fopen(tpltPath, "r");
  if(!inFile)
    {
      errStr = nfrConcat(tpltPath, ": ", strerror(errno), NULL);
      Cgi_Response(errStr, 0, 1);
      exit(1);
    }
  destPath = nfrConcat(basePath, name, "/queue.cf", NULL);
  outFile = fopen(destPath, "w");
  if(!outFile)
    {
      errStr = nfrConcat(destPath, ": ", strerror(errno), NULL);
      Cgi_Response(errStr, 0, 1);
      exit(1);
    }
  while ((bytes=fread(buf, 1, sizeof(buf), inFile)) > 0)
    fwrite(buf, 1, bytes, outFile);
  fclose(inFile);
  fclose(outFile);

  /* send OK */
  Cgi_Response(NULL, 0, 0);
  exit(0);
}
