/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   resdump.c
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

/* define DEBUG_LEV 1 to get dumps of the queries, and enable cmdline
   debugging */
#define DEBUG_LEV 1

#include "sys-include.h"

#include "cgiutils.h"
#include "libnfr.h"
#include "nfr_res.h"
#include "librc.h"

int DumpFac(PSymbol pSymbol, int i, void *p)
{
  ResId_Write((PResId)pSymbol->Data, NULL, stdout);
  Res_WriteBody((PResId)pSymbol->Data, NULL, stdout);
  return NFR_ERR_SUCCESS;
}

int main(int argc, char **argv)
{
  BOOL DumpAll = TRUE;
  char *SrcFile = NULL;
  char *SrcFiles = NULL;
  FILE *In = NULL;
  PCf QueryFields = NULL;
  char *FilterArg = NULL;
  char *FilterArgs = NULL;
  PResId pResId = NULL;
  int type = 0;
  PSymbol pSymbol = NULL;

  printf("Content-type: application/octet-stream\r\n\r\n");
  CgiGoHome();

  CGI_DEBUGGER();

  QueryFields = Cf_New(0);
  Cgi_QueryToCf(QueryFields);

  Rc_Init();
  gVerbosity = 1;

  if((SrcFiles = Cf_GetVal("src_files", QueryFields)) != NULL)
    {
      fprintf(stderr, "dumping from files: %s\n", SrcFiles);
      SrcFile = strtok(SrcFiles, ",");
      while(SrcFile != NULL)
	{
	  while(isspace(*SrcFile))
	    SrcFile++;
	  In = fopen (SrcFile, "r");
	  if(!In)
	    {
	      perror(SrcFile);
	      exit(1);
	    }
	  Rc_Parse(RC_MODE_SCAN | RC_MODE_COMPILE, In, NULL, NULL);
	  fclose(In);
	  SrcFile = strtok(NULL, ",");
	}
      pResId = NULL;
      SrcFile = NULL;
    }
  else
    {
      fprintf(stderr, "no src files specified\n");
      Rc_Cleanup();
      SExit(1);
    }

  if((FilterArgs = Cf_GetVal("types", QueryFields)) != NULL)
    {
      fprintf(stderr, "dumping all of type: %s\n", FilterArgs);
      FilterArg = strtok(FilterArgs, ",");
      while(FilterArg != NULL)
	{
	  while (isspace(*FilterArg))
	    FilterArg++;
	  type = atoi(FilterArg);
	  SymTab_ForEach(type, DumpFac, 0, NULL, gpResTab->SymTab);
	  FilterArg = strtok(NULL, ",");
	}
      DumpAll = FALSE;
      pResId = NULL;
      FilterArg = NULL;
    }

  if((FilterArg = Cf_GetVal("ranges", QueryFields)) != NULL)
    {
      /* bug: put something here */
      fprintf(stderr,"ranges not implemented!\n");
      DumpAll = FALSE;
      pResId = NULL;
      FilterArg = NULL;
    }

  if((FilterArgs = Cf_GetVal("items", QueryFields)) != NULL)
    {
      fprintf(stderr, "dumping these: %s\n", FilterArgs);
      FilterArg = strtok(FilterArgs, ",");
      while(FilterArg != NULL)
	{
	  while (isspace(*FilterArg))
	    FilterArg++;
	  pResId = ResTab_Get(FilterArg, gpResTab);
	  if (pResId != NULL)
	    {
	      ResId_Write(pResId, NULL, stdout);
	      Res_WriteBody(pResId, NULL, stdout);
	    }
	  FilterArg = strtok(NULL, ",");
	}
      DumpAll = FALSE;
      pResId = NULL;
      FilterArg = NULL;
    }

  if(DumpAll)
    ResTab_Dump(gpResTab, stdout);

  Rc_Cleanup();

  Cf_Delete(QueryFields);

  SExit(0);
}
