/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   resmerge.c
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

/* define DEBUG_LEV 1 to get dumps of the queries, and enable cmdline
   debugging */
#define DEBUG_LEV 1

#include "sys-include.h"

#include "cgiutils.h"
#include "libnfr.h"
#include "nfr_res.h"
#include "librc.h"

int main(int argc, char **argv)
{
  FILE *In = NULL;
  FILE *Out = NULL;
  char *TargetName = NULL;
  PResTab pDiffTab = NULL;
  ULONG Result = 0;
  char TempName[] = "tmp/rcmrginXXXXXX";
  char FacTempName[] = "tmp/rcfacmrginXXXXXX";
  char BakName[256];
  char *errStr = NULL;

  CgiGoHome();

  CGI_DEBUGGER();

  TargetName = getenv("PATH_INFO");
  if(!TargetName)
    {
      Cgi_Response("resmerge: no target file", 0, 1);
      exit(1);
    }
  /* make sure that the path is relative */
  while(*TargetName == '/')
    TargetName++;
  fprintf(stderr, "target: %s\n", TargetName);
  
  Rc_Init();
  gVerbosity = 1;

  /* the facs have to be there already so that we can resolve them as the
     alert structs are recieved.  */
  In = fopen("etc/resources/fac_res.src", "r");
  if(!In)
    {
      errStr = nfrConcat("resmerge: ", strerror(errno), 
			 ": etc/resources/fac_res.src", NULL);
      Cgi_Response(errStr, 0, 1);
      exit(1);
    }

  Result = Rc_Parse(RC_MODE_SCAN | RC_MODE_COMPILE, In, NULL, NULL);
  if(Result != NFR_ERR_SUCCESS)
    {
      Cgi_Response("resmerge: parse failed", 0, 1);
      unlink(TempName);      
      exit(1);
    }

  pDiffTab = gpResTab;
  gpResTab = NULL;
  fclose(In);
  Rc_Cleanup();

  Result = ResTab_Load(pDiffTab, stdin);
  if(Result != NFR_ERR_SUCCESS)
    {
      Cgi_Response("resmerge: resource table load failed", 0, 1);
      unlink(TempName);      
      exit(1);
    }

  /**** now do pass 2 ****/

  Rc_Init();

  In = fopen("etc/resources/fac_res.src", "r");
  if(In == NULL)
  if(!In)
    {
      errStr = nfrConcat("resmerge: ", strerror(errno), 
			 ": etc/resources/fac_res.src", NULL);
      Cgi_Response(errStr, 0, 1);
      exit(1);
    }

  mktemp(FacTempName);
  Out = fopen(FacTempName, "w");
  if(!Out)
    {
      errStr = nfrConcat("resmerge: ", strerror(errno), 
			 ": ", FacTempName, NULL);
      Cgi_Response(errStr, 0, 1);
      exit(1);
    }

  Result = Rc_Parse(RC_MODE_SCAN | RC_MODE_COMPILE | RC_MODE_MERGE_DIFFS, 
		    In, Out, pDiffTab); 
  fclose (In);

  In = fopen(TargetName, "r");
  if(!In)
    {
      errStr = nfrConcat("resmerge: ", strerror(errno), 
			 ": ", TargetName, NULL);
      Cgi_Response(errStr, 0, 1);
      exit(1);
    }

  mktemp(TempName);
  Out = fopen(TempName, "w");
  if(!Out)
    {
      errStr = nfrConcat("resmerge: ", strerror(errno), 
			 ": ", TempName, NULL);
      Cgi_Response(errStr, 0, 1);
      exit(1);
    }

  Result = Rc_Parse(RC_MODE_SCAN | RC_MODE_COMPILE | RC_MODE_MERGE_DIFFS, 
		    In, Out, pDiffTab);


  if(Result == NFR_ERR_SUCCESS)
    {
      fprintf(stderr,"Rc_Parse() successful, renaming tmp files\n");
      sprintf(BakName, "%s.bak", TargetName);
      unlink(BakName);
      rename(TargetName, BakName);
      rename(TempName, TargetName);
      unlink("etc/resources/fac_res.src.bak");
      rename("etc/resources/fac_res.src", "etc/resources/fac_res.src.bak");
      rename(FacTempName, "etc/resources/fac_res.src");
    }
  else
    {
      Cgi_Response("resmerge: parse failed", 0, 1);
      unlink(TempName);      
      exit(1);
    }

  Cgi_Response("resmerge: parse succeeded", 0, 0);

  ResTab_Delete(pDiffTab);

 
  Rc_Cleanup();
  fclose(In);
  fclose(Out);
  exit(0);
}
