/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   alert.h
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

/*****************************************************************************
 * alert.h
 * functions and structures used to process alerts
 *****************************************************************************/
#ifndef ALERT_H_
#define ALERT_H_

#include "xtypes.h"
#include "resource.h"
#include "alertres.h"
#include "alertsrc.h"

#define ALERT_SEND_TIMEOUT 2000
#define MAXSIZE_ALERT 1000
#define ALERTD_FIFO "etc/run/alertd/alertsin"
#define ALERTD_MON_FIFO "etc/httpd/cgi-bin/tmp/alertdmon"
#define DEF_ALERT_ID_STRING "DEF_ALERT_ID"
#define DEF_ALERT_TEXT "resource not found for alert 0x%.8lX. "\
                       "first 3 insertions: $(1), $(2), $(3)"

#define ALERT_PRINT_SOURCE_INFO 1

extern BOOL gKeepOpenFlag;

/*****************************************************************************
 * The TAlert type is a variable length structure consisting of a
 * message code and insertion strings intended to be used with the
 * message's format string from the resource file. The Inserts member
 * points to a series of InsCount char strings delimited by \0's.
 * InsLen is the total length of all of the char strings, including the
 * \0 terminators
 */
typedef struct TAlert
{
  time_t Timestamp;
  ULONG ResCode;
  ULONG SrcCode;
  pid_t SrcPid;
  int   InsCount;
  int   InsLen;
  BYTE *Inserts;
} TAlert, *PAlert;

/*****************************************************************************
 * Alert_Generate() - takes an alert code and an alert source code. A
 * TAlert struct containing the alert code and the alert source code
 * is sent to the alert daemon for processing. UsrAlert_Generate() is
 * the same except that it takes the id strings of the resources
 * rather than their numeric identifiers.  Alert_GenerateEx() is the
 * same except source file and linenumber info is included as the
 * first two arguments.  UsrAlert_Generate() and UsrAlert_GenerateEx()
 * work similarly except that they take the string identifiers of the
 * alert and alert source instead of the numeric ids.  */

int Alert_Generate(ULONG, ULONG);
int Alert_GenerateEx(char *, int, ULONG, ULONG);

int UsrAlert_Generate(char *, char *);
int UsrAlert_GenerateEx(char *, int, char *, char *);


/*****************************************************************************
 * Alert_Args_Generate() - takes an alert source res code and an alert
 * resource code and any number of char* arguments.  The last argument
 * must be NULL. A TAlert struct containing the alert resource code
 * and the insertion strings is sent to the alert daemon for
 * processing. Alert_Args_GenerateEx() is the same except source file
 * and linenumber info is included as the first two arguments.
 * UsrAlert_ArgsGenerate() and UsrAlert_Args_GenerateEx() work
 * similarly except that they take the string identifiers of the alert
 * and alert source instead of the numeric ids 
 * VAlert_Args_Generate() and VAlert_Args_GenerateEx() are the same
 * as Alert_Args_Generate() and Alert_Args_GenerateEx() respectively,
 * except that they take a NULL terminated array of char * instead of
 * a variable number of arguments. */

int Alert_Args_Generate(ULONG, ULONG, ...);
int Alert_Args_GenerateEx(char *, int, ULONG, ULONG, ...);

int UsrAlert_Args_Generate(char *, char *, ...);
int UsrAlert_Args_GenerateEx(char *, int, char *, char *, ...);

int VAlert_Args_Generate(ULONG, ULONG, char **);
int VAlert_Args_GenerateEx(char *, int, ULONG, ULONG, char **);



/*****************************************************************************
 * Alert_Send() - just stuffs an asembled alert down the pipe to the daemon
 */
int Alert_Send(PAlert);


/*****************************************************************************
 * Alert_ToAlertRes() - This function merges the insertion strings
 * from pAlert into the Text member of the returned PAlertRes.
 * Occurrences of "$(n)" will be replaced with the nth insertion
 * string.  Any "$(n)" that occurs after the insertion strings are all
 * used is replaced with "?" Alert_ToAlertResEx() takes a set of flags
 * defined above */
PAlertRes Alert_ToAlertRes(PAlert);
PAlertRes Alert_ToAlertResEx(ULONG, PAlert);

PAlertSrc Alert_ToAlertSrc(PAlert);

#endif
