/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   alertres.h
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

/******************************************************************************
 * AlertRes.h - functions and structures used in handling AlertRes
 * Resource code diagram:
 *
 *  3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
 *  1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
 * + - - - - - - - - - +-----------+-------------------------------+
 * | res-specific use  | Res Type  |         Res ID                |
 * + - - - - - - - - - +-----------+-------------------------------+
 *
 * AlertRes:          
 * +---------------+---+                                            
 * |    unused     |Sev|                                            
 * +---------------+---+
 *                      
 *****************************************************************************/
#ifndef ALERTRES_H_
#define ALERTRES_H_

#include "xtypes.h"
#include "rescodes.h"
#include "resid.h"
#include "alertsrc.h"

#define ALERTRES_MAXSIZE 4096
#define FAC_CODES_MAXSIZE 255

typedef BYTE FacCode;

typedef struct TAlertRes
{
  RESID_REQ_FIELDS;		/* see resid.h */
  /* struct TAlertRes *next; */
  BYTE Severity;
  USHORT TextLen;
  char *Text;
  USHORT FacCodeCount;
  FacCode *FacCodes;
} TAlertRes, *PAlertRes;

ULONG AlertRes_WriteBody(PAlertRes , ULONG *,  FILE *);

ULONG AlertRes_ReadBody(PAlertRes , ULONG *, FILE *);

/******************************************************************************
 * AlertRes_OpenEx(ULONG AlertResCode, BOOL KeepOpen) 
 * - creates a TAlertRes struct and
 * initializes it with the resource information corresponding to
 * AlertResCode. On success the a pointer to the new TAlertRes struct is
 * returned. If KeepOpen is TRUE the resource file is kept open */
PAlertRes AlertRes_OpenEx(ULONG, BOOL);
#define AlertRes_Open(U) AlertRes_OpenEx(U, FALSE)

/******************************************************************************
 * AlertRes_Close(PAlertRes pAlertRes) - performs cleanup and destroys the
 * TAlertRes struct pointed to by pAlertRes.  */
void AlertRes_Close(PAlertRes);

/******************************************************************************
 * AlertRes_SetText(char *Text, USHORT Size, PAlertRes) - First frees
 * the storage pointed to by pAlertRes->Text (if any) and then copies
 * Text to a newly allocated buffer and sets the pAlertRes->Text to
 * point to it.  AlertRes_SetFacCodes() and AlertRes_SetId() behave
 * similarly */
int AlertRes_SetText(char *, USHORT, PAlertRes);
int AlertRes_SetFacCodes(BYTE *, USHORT, PAlertRes);
int AlertRes_SetId(char *, USHORT, PAlertRes);

/******************************************************************************
 * AlertRes_GetText(char *Buf, USHORT Size, PAlertRes pAlertRes) - puts
 * the contents (up to Size bytes) into Buf.  Procedures should use
 * this rather than accessing pAlertRes->Text directly.  The pointer to
 * Buf is returned on success.  AlertRes_GetFacCodes() and
 * AlertRes_GetId() behave similarly */
char *AlertRes_GetText(char *, USHORT, PAlertRes);
BYTE *AlertRes_GetFacCodes(BYTE *, USHORT, PAlertRes);
char *AlertRes_GetId(char *, USHORT, PAlertRes);

/******************************************************************************
 * AlertRes_New() - allocates memory for a TAlertRes and returns a
 * pointer to the new structure */
PAlertRes AlertRes_New(void);

/******************************************************************************
 * AlertRes_Delete(PAlertRes pAlertRes) - frees the space pointed to by
 * pAlertRes. */
void AlertRes_Delete(PAlertRes);

#define AlertRes_Close(P) AlertRes_Delete(P)

#endif
