/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   cf.h
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

/******************************************************************************
 * cf.h - use these routines and structures to access config files or
 * registry keys
 *****************************************************************************/
#ifndef CF_H_
#define CF_H_

#include "xtypes.h"
#include "symtab.h"

#define MAXSIZE_CF_VAL 1024
#define MAXSIZE_CF_NAME 64

#define  CF_FLAG_PUT_LF  1
#define  CF_FLAG_MODE_MERGE  (1 << 1)
#define  CF_FLAG_MODE_STRIP   (1 << 2)

#define CF_NVFLAG_MERGED 1

enum
{
  CF_WANT_NAME,
  CF_GET_NAME,
  CF_WANT_ASS,
  CF_WANT_VAL,
  CF_GET_VAL,
  CF_GET_QVAL,
  CF_GET_QVAL_ESC,
  CF_HAVE_NO_NVPAIR,
  CF_HAVE_GOOD_NVPAIR,
  CF_HAVE_BROKEN_NVPAIR,
  CF_HAVE_NO_NVPAIR_EOF,
  CF_HAVE_GOOD_NVPAIR_EOF,
  CF_HAVE_BROKEN_NVPAIR_EOF
};

enum
{
  NV_EXP_INIT,
  NV_EXP_ESC,
  NV_EXP_VAR,
  NV_EXPANDED
};

typedef struct
{
  ULONG Mode;
  ULONG State;
  PSymTab SymTab;
} TCf, *PCf;

typedef struct
{
  ULONG Flags;
  USHORT State;
  char *Name;
  USHORT NameLen;
  char *Val;
  USHORT ValLen;
  char *ExpVal;
  USHORT ExpValLen;
  char QChar;
} TNVPair, *PNVPair;

PCf Cf_New(int Size);

void Cf_Delete(PCf);

void Cf_NVInit(PNVPair);

int Cf_NVOnComplete(ULONG, PNVPair, FILE *, FILE *, PCf);

int CfNVParse(ULONG, PNVPair, FILE *, FILE *, PCf);

int Cf_WriteUnmergedNV(PNVPair, void *);

#define Cf_NVRead(PNV, FP) Cf_NVParse(0L, PNV, FP, NULL, NULL)

#define Cf_Read(IN, PCF) Cf_Parse(0L, IN, NULL, PCF)

int Cf_Parse(ULONG, char *, char *, PCf);

int Cf_SaveAs(char *, PCf);

int Cf_Update(char *, PCf);

int Cf_AppendTo(char *, PCf);

char *Cf_GetVal(char *, PCf);

char *Cf_ExpandVal(char *, PCf);


/* WARNING: the "Take" in these two functions  stands for take
   *responsibility* - the caller must free() the returned
   pointer when it is no longer needed, It will *not* be freed
   by the Cf_Delete() function */
char *Cf_TakeVal(char *, PCf);
TNVPair *Cf_TakeNVPair(char *, PCf);


#define Cf_SetVal(n, v, pcf) Cf_NVCreate(n, v, strlen(v), pcf)

PNVPair Cf_GetNVPair(char *, PCf);

typedef int (*PfnCfIterate)(PNVPair, void *);
ULONG Cf_ForEach(PfnCfIterate, void *, PCf);

int Cf_Iterate(PSymbol, int, void *);

int Cf_NVCreate(char *, char *, int, PCf);

int Cf_NVSymPickle(PSymbol, int, void *);

void Cf_NVOnDelete(PSymbol);

#endif
