/*
        Copyright(C) 1997 Network Flight Recorder, Inc.
        All rights reserved.

        Use and distribution of this software and its source code
        are governed by the terms and conditions of the
        Network Flight Recorder Software License ("LICENSE.TXT")

By Mark Sienkiewicz / NFR

*/

#ifndef NFR_COUNTER_H
#define NFR_COUNTER_H

#include "opts.h"

/*
* counter is a type that is bigger than 32 bits.  You might use
* int on a machine with 64 bit integers.  Here, we use long long
* to get the same thing.  
*
* If you get stuck, you can always make it type int, but watch
* out for overflows - you can cram about 1.5 * 2^36 bytes through 
* a 10 megabit ethernet in a single day.
*/

/*
* the application is forbidden to know that counter is a long long.
* it may not be.  COUNTER_TYPE is defined in the opts.h for each
* system
*/

typedef COUNTER_TYPE counter;

 
/*
* convert signed text number into a counter
*/
counter counter_atoi( char * s);		/* converts string to 

/*
* convert a counter into printable text. s must point at a buffer of
* size COUNTER_TEXT_SIZE
*/
#define COUNTER_TEXT_SIZE 25
void counter_itoa(char *s, counter c);


/*
* promote an integer to a counter
*/
counter counter_from_int(int x); 

/*
* demote a counter to an integer.  no detection of lost precision.
*/
int counter_to_int(counter x); 


/*
* basic math with counters.  no detection of divide by zero errors.
* no detection of overflow.
*/
counter counter_add( counter a, counter b );	/* a + b */
counter counter_sub( counter a, counter b);	/* a - b */
counter counter_mul( counter a, counter b);	/* a * b */
counter counter_div( counter a, counter b);	/* a / b */
counter counter_mod( counter a, counter b);	/* a % b */

/*
* compare two counters; returns 1 if a > b, -1 if a < b, 0 if a == b
*/
int counter_cmp(counter a, counter b);	

#endif
