/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   daemon.h
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

/*****************************************************************************
 * daemon.h - structures and functions to isolate daemon code from OS
 * specifics
 *****************************************************************************/

#ifndef _DAEMON_H_
#define _DAEMON_H_

#include "xtypes.h"

#define NFR_STAT_RUNNING 1
#define NFR_STAT_STOPPED 2

typedef pid_t TDaemonId;

typedef struct TDaemon
{
  ULONG Status;
  TDaemonId DaemonId;
  int (*Main)(struct TDaemon *);
  int (*OnNew)(struct TDaemon *);
  int (*OnDelete)(struct TDaemon *);
  int (*OnStart)(struct TDaemon *);
  int (*OnStop)(struct TDaemon *);
  int (*OnPause)(struct TDaemon *);
  int (*OnContinue)(struct TDaemon *);
  void *Data;			/* data of 'ur choice */
} TDaemon, *PDaemon;

typedef int (*PfnDaemonCallback)(PDaemon);

PDaemon Daemon_New(void);

void Daemon_Delete(PDaemon);

int Daemon_Start(PDaemon, int);

int Daemon_Stop(PDaemon, int);

int Daemon_Kill(PDaemon, int);

void OnTermSigs(int);

#endif
