/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   dgram.h
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

#ifndef DGRAM_H_
#define DGRAM_H_

#include "xtypes.h"

typedef struct
{
  ULONG Type;
  USHORT Len;
} TDGram, *PDGram;

#define DGRAM_MAXSIZE 512 
/* bug: should be PIPE_BUF to take advantage of systems with big buffers */

#define DGRAM_HDR_SIZE sizeof(TDGram)
#define DGRAM_DATA_MAXSIZE (DGRAM_MAXSIZE - DGRAM_HDR_SIZE)

#define TIMEOUT_NEVER -1

#define DGram_GetData(pdg) ((unsigned char *)pdg + sizeof(TDGram))

int DGram_SetData(void *,	/* addr of data */
		  int,		/* len of data */
		  PDGram);	/* dest datagram */

int DGram_Recv_fd(int,		/* fd to read from */
		  PDGram,	/* pointer to TDGram to write to */
		  int,		/* size of the TDGram */
		  int);		/* timeout in microseconds */

int DGram_Send_fd(int,		/* fd to write to */
		  PDGram,	/* pointer to TDGram to send */
		  int);		/* timeout in microseconds */

typedef struct TDGServer
{
  int fdRead;
  ULONG MaxWait;
  int (*DataProc)(PDGram, struct TDGServer *);
  int (*IdleProc)(struct TDGServer *);
  int (*ErrProc)(ULONG, struct TDGServer *);
  void *Data;
}TDGServer, *PDGServer;

typedef int (*PfnDGDataProc)(PDGram, PDGServer);
typedef int (*PfnDGIdleProc)(PDGServer);
typedef int (*PfnDGErrProc)(ULONG, PDGServer);


enum
{
  DGSRV_BUSY_NOIDLE,
  DGSRV_BUSY,
  DGSRV_IDLE,
  DGSRV_ERROR
};

TDGServer *DGServer_New(void);
void DGServer_Delete(TDGServer *);
int DGServer_Run(TDGServer *);

#endif
