/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   list.h
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

/******************************************************************************
 * TList - a simple sorted linked list.  These are intended to be used
 * as a base for specialized non-intrusive lists and other dynamically
 * sized structures like stacks and queues.
 *****************************************************************************/
#ifndef _LIST_H_
#define _LIST_H_

#include "xtypes.h"

typedef struct TListNode
{
  struct TListNode *Next;
} TListNode, *PListNode;

typedef struct TNIListNode
{
  struct TNIListNode *Next;
  void *Data;
} TNIListNode, *PNIListNode;

/* If you want your list sorted, supply a comparison function which
 * returns -1, 0, or 1 in the same manner as strcmp() */
typedef int (*PfnCompare)(PListNode, PListNode);

typedef struct TList
{
  ULONG Count;
  PfnCompare Compare;
  PListNode Head;
  PListNode Tail;
} TList, *PList;

typedef struct TNIList
{
  ULONG Count;
  PfnCompare Compare;
  PListNode Head;
  PListNode Tail;
  PNIListNode CurNode;
} TNIList, *PNIList;

PList List_New(PfnCompare);
void List_Delete(PList);
PListNode List_Insert(PListNode, PList);
PListNode List_AddToHead(PListNode, PList);
PListNode List_AddToTail(PListNode, PList);
PListNode List_Remove(PListNode, PList);
PListNode List_RemoveHead(PList);
PListNode List_RemoveTail(PList);
void *NIList_Insert(void *, PNIList);
void *NIList_AddToHead(void *, PNIList);
void *NIList_AddToTail(void *, PNIList);
void *NIList_Remove(void *, PNIList);
void *NIList_RemoveHead(PNIList);
void *NIList_RemoveTail(PNIList);
void *NIList_GetFirst(PNIList);
void *NIList_GetNext(PNIList);

#endif
