/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   rangeres.h
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

/******************************************************************************
 * rangeres.h - functions and structures used in handling resource ranges
 *
 * Resource code diagram:
 *
 *  3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
 *  1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
 * + - - - - - - - - - +-----------+-------------------------------+
 * | res-specific use  | Res Type  |         Res ID                |
 * + - - - - - - - - - +-----------+-------------------------------+
 *
 * RangeRes:          
 * +-------------------+                                            
 * |      unused       |                                            
 * +-------------------+
 *                      
 *****************************************************************************/
#ifndef RANGERES_H_
#define RANGERES_H_

#include "xtypes.h"
#include "rescodes.h"
#include "resid.h"
#include "list.h"

typedef struct TRangeRes
{
  RESID_REQ_FIELDS;
  USHORT Min;
  USHORT Max;
  char MemberType; 
  USHORT HighNo;		/* temporary */
  /* TNIList *NumsUsed; */
} TRangeRes, *PRangeRes;

TRangeRes *RangeRes_New(void);
void RangeRes_Delete(TRangeRes *);
ULONG RangeRes_WriteBody(TRangeRes *, ULONG *,  FILE *);
ULONG RangeRes_ReadBody(TRangeRes *, ULONG *, FILE *);


#endif /* sentry */
