/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   rescodes.h
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

/******************************************************************************
 * Resource code diagram:
 *
 *  3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
 *  1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
 * + - - - - - - - - - +-+---------+-------------------------------+
 * | res-specific use  |u| Res Type|         Res Number            |
 * + - - - - - - - - - +-+---------+-------------------------------+
 *
 * the bit marked "u" (bit 21) is set if this is a resource which has 
 * been created by or can be deleted by users.  Most NFR generated resources 
 * do not have this bit set.
 * The NFR resources reside in ~nfr/etc/resources/nfr
 * and user resources are in ~nfr/etc/resources/usr
 *
 * AlertRes:          
 * +---------------+---+                                            
 * |    unused     |Sev|                                            
 * +---------------+---+
 *                      
 * AlertSrc:          
 * +-------------------+                                            
 * |      unused       |                                            
 * +-------------------+
 *                      
 * Facility:          
 * +-------------------+                                            
 * |      unused       |                                            
 * +-------------------+
 *                      
 * StringRes:          
 * +-------------------+                                            
 * |      unused       |                                            
 * +-------------------+
 *                      
 *****************************************************************************/
#ifndef _RESCODES_H_
#define _RESCODES_H_

#include "xtypes.h"

typedef ULONG TResNo;

#define DEF_RESCODE 0

/* bitmasks and shifts */
#define RESNO_SHIFT 0
#define RESNO_MASK (0x0000FFFF << RESNO_SHIFT)
#define RESTYPE_SHIFT 16
#define RESTYPE_MASK (0x0000001F << RESTYPE_SHIFT)
#define USR_SHIFT 21
#define USR_MASK (0x00000001 << USR_SHIFT)
#define SEV_SHIFT 22
#define SEV_MASK (0x00000003 << SEV_SHIFT)

/* accessor macros */
#define ResCode_GetResNo(rc) ((rc & RESNO_MASK) >> RESNO_SHIFT)
#define ResCode_ClearResNo(rc) (rc &= ~RESNO_MASK)
#define ResCode_SetResNo(rc, i) ResCode_ClearResNo(rc); \
                                 rc |= ((i << RESNO_SHIFT) & RESNO_MASK)
#define ResCode_GetResType(rc) ((rc & RESTYPE_MASK) >> RESTYPE_SHIFT)
#define ResCode_ClearResType(rc) (rc &= ~RESTYPE_MASK)
#define ResCode_SetResType(rc, i) ResCode_ClearResType(rc); \
                                 rc |= ((i << RESTYPE_SHIFT) & RESTYPE_MASK)
#define ResCode_GetAlertSev(rc) ((rc & SEV_MASK) >> SEV_SHIFT)
#define ResCode_ClearAlertSev(rc) (rc &= ~SEV_MASK)
#define ResCode_SetAlertSev(rc, i) ResCode_ClearAlertSev(rc); \
                                   rc |= ((i << SEV_SHIFT) & SEV_MASK)
#define ResCode_IsUser(rc) (rc & USR_MASK)
#define ResCode_ClearUser(rc) (rc &= ~USR_MASK)
#define ResCode_SetUser(rc) (rc |= (USR_MASK))

#endif
