/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   resource.h
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

/******************************************************************************
 * resource.h - functions, macros, and structures used in processing
 * nfr resources
 *
 *****************************************************************************/
#ifndef RESOURCE_H_
#define RESOURCE_H_

#include "xtypes.h"
#include "symtab.h"
#include "rescodes.h"
#include "resid.h"
#include "rangeres.h"
#include "alertres.h"
#include "alertsrc.h"
#include "stringres.h"

/* Res types */
enum
{
  ID_FAC = 1,
  ID_SEV,
  RES_RANGE,
  RES_ALERT,
  RES_ALERT_SRC,
  RES_STRING
};


/* res file names */
#define RESFILE_PATH "etc/resources/"
#define RESFILE_ALERTS "alert.res"
#define RESFILE_ALERT_SRCS "alertsrc.res"
#define RESFILE_STRINGS "strings.res"
#define RES_MAPFILE "res.map"

#define RES_MAX_STRLEN 1024
#define RES_MAX_INSERTS 64

#define RES_HDR_SIZE 256
#define RES_INDEX_OFFSET(Idx) ( (Idx) * (2 * sizeof(ULONG))  + \
				(RES_HDR_SIZE) )
#define RES_DATA_START(Size) (RES_INDEX_OFFSET(Size+1))
typedef struct TResFile
{
  FILE *File;
  BOOL Dirty;
  ULONG Size;
  ULONG DataPtr;
  ULONG IndexPtr;
} TResFile, *PResFile;


enum
{
  RESEXP_INIT,
  RESEXP_ESC,
  RESEXP_WANTNUM,
  RESEXP_GETNUM
};

int Res_Dump(TSymbol *, int, void *);

ULONG Res_ReadBody(TResId *, ULONG *, FILE *);
ULONG Res_WriteBody(TResId *, ULONG *, FILE *);

int Res_ASNPrintf(char *, char *, char **, int, int);

ULONG ResFile_WriteRes(TResId *, PResFile);
ULONG ResFile_ReadRes(TResId *, PResFile);

/******************************************************************************
 * int ResFile_IdxAddEntry(ULONG ResCode, PResFile pResFile); */
int ResFile_IdxAddEntry(ULONG, PResFile);

/******************************************************************************
 * int ResFile_Seek(ULONG ResCode, PResFile pResFile); */
int ResFile_Seek(ULONG, PResFile);

/******************************************************************************
 * PResFile ResFile_Create(char *ResName, ULONG Size) */
PResFile ResFile_Create(char *, ULONG);


/******************************************************************************
 * PResFile ResFile_Open(char *ResName) */
PResFile ResFile_Open(char *);

/******************************************************************************
 * void ResFiles_Refresh() */
void ResFiles_Refresh();

/******************************************************************************
 * void ResFile_Close(PResFile pResFile) */
void ResFile_Close(PResFile);

/******************************************************************************
 * int ResFile_WriteHdr(PResFile pResFile) */
int ResFile_WriteHdr(PResFile);


#endif
