/***************************************************************************** 
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT") 

  File:   spaceman.h
  Author: Kent Landfield <kent@nfr.net>
 
 *****************************************************************************/ 
#ifndef _SPACEMAN_H_
#define _SPACEMAN_H_

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "daemon.h"

#define SPACEMAN_CONFIG  "etc/spaceman.cf"
#define SPACEMAN_FIFO    "etc/run/spacemand/infifo"
#define SPACEMAN_PIDFILE "etc/run/spacemand/spidfile"

#define EXPIRE_BAD_LOC_FORMAT 100

#define SECSPERMIN      60
#define SECSPERHOUR   3600
#define SECSPERDAY   86400
#define SECSPERWEEK 604800

/*
** Action defines
*/

#define NOACTION        0
#define PURGE           1
#define SAVE            2
#define PROCESS         3
#define PROCESSNSAVE    4
#define PROCESSNPURGE   5

/*
** node_type defines
*/
#define RKIVEINDEX 0
#define DIRONLY    1

struct indexinfo {
	char *indexpath;
	long dbsize;          /* size in bytes */
	time_t starttime;     /* start date/time of data */
	time_t endtime;       /* end date/time of data */
};

struct node {
	int node_type;        /* Recorder vs  Directory */
	char *backend;
	char *recorder;
	long quota;
	time_t age;
	int priority; 
	int action; 
	char *command; 
	char *location; 
	struct indexinfo dbinfo;
	struct node *next;
};

#define NFR_SPACE_PIDREAD      -1
#define NFR_SPACE_RUNNING      -2
#define NFR_SPACE_FIFOCREAT    -3
#define NFR_SPACE_FIFOOPEN     -4
#define NFR_SPACE_CONFIGBAD    -5
#define NFR_SPACE_CANTOPENFILE -6
#define NFR_SPACE_CANTCREATPID -7

#define SIGNAL_TYPE void

int spaceman_main(PDaemon pDaemon);
int spaceman_stop(PDaemon pDaemon);
void report(char *, ... );
char *itoa(int i);
int expire_file(struct node *t, char *filename, struct tm *now);

#endif /* _SPACEMAN_H_ */
