/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   stringres.h
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

/******************************************************************************
 * stringres.h - functions and structures used in handling alert source 
 * resources.
 * Resource code diagram:
 *
 *  3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
 *  1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
 * + - - - - - - - - - +-----------+-------------------------------+
 * | res-specific use  | Res Type  |         Res ID                |
 * + - - - - - - - - - +-----------+-------------------------------+
 *
 * StringRes:          
 * +-------------------+                                            
 * |      unused       |                                            
 * +-------------------+
 *                      
 *****************************************************************************/
#ifndef STRINGRES_H_
#define STRINGRES_H_

#include "xtypes.h"
#include "rescodes.h"
#include "resid.h"

typedef struct TStringRes
{
  RESID_REQ_FIELDS;		/* common res fields, see resid.h */
  USHORT StringLen;
  char *String;
} TStringRes, *PStringRes;


ULONG StringRes_WriteBody(PStringRes , ULONG *,  FILE *);

ULONG StringRes_ReadBody(PStringRes , ULONG *, FILE *);

/******************************************************************************
 * StringRes_Open(ULONG StringResCode) - creates a TStringRes struct
 * and initializes it with the resource information corresponding to
 * StringResCode. On success the a pointer to the new TStringRes
 * struct is returned. */
PStringRes StringRes_Open(ULONG);

/******************************************************************************
 * StringRes_Close(PStringRes pStringRes) - performs cleanup and
 * destroys the TStringRes struct pointed to by pStringRes.  */
void StringRes_Close(PStringRes);

/******************************************************************************
 * StringRes_SetString(char *Text, USHORT Size, PStringRes) - First
 * frees the storage pointed to by pStringRes->String (if any) and
 * then copies String to a newly allocated buffer and sets the
 * pStringRes->String to point to it.  StringRes_SetId() behaves
 * similarly */
int StringRes_SetString(char *, USHORT, PStringRes);
int StringRes_SetId(char *, USHORT, PStringRes);

/******************************************************************************
 * StringRes_GetString(char *Buf, USHORT Size, PStringRes pStringRes)
 * - puts the contents (up to Size bytes) into Buf.  Procedures should
 * use this rather than accessing pStringRes->String directly.  The
 * pointer to Buf is returned on success.  StringRes_GetId() behaves
 * similarly */
char *StringRes_GetString(char *, USHORT, PStringRes);
char *StringRes_GetId(char *, USHORT, PStringRes);

char *StringRes_Load(ULONG);
char *StringRes_Build(ULONG, ...);

/******************************************************************************
 * StringRes_New() - allocates memory for a TStringRes and returns a
 * pointer to the new structure */
PStringRes StringRes_New(void);

/******************************************************************************
 * StringRes_Delete(PStringRes pStringRes) - frees the space pointed
 * to by pStringRes. */
void StringRes_Delete(PStringRes);

#define StringRes_Close(P) StringRes_Delete(P)

#endif
