/*
        Copyright(C) 1997 Network Flight Recorder, Inc.  
        All rights reserved.

        Use and distribution of this software and its source code
        are governed by the terms and conditions of the
        Network Flight Recorder Software License ("LICENSE.TXT")

By Mark Sienkiewicz / NFR

*/

#ifndef NFR_VSCH_H
#define NFR_VSCH_H

/*
* here are the types we know about
*
* bug: add documentation here
*
* bug: some of these functions don't exist yet.
*/

enum vsch_type { 
	VSCH_ERROR = 0,	/* not a type - indicates a problem */
	VSCH_TIME_T = 1, 
	VSCH_INT = 2, 
	VSCH_COUNTER = 3, 
	VSCH_IP = 4, 
	VSCH_STR = 5,		/* has variable size */
	VSCH_BLOB = 6,		/* has variable size */
	VSCH_ETHERMAC =7,	/* size = 6 */
	};

/*
* a general structure for representing records
*/
struct vsch_field
	{
	enum vsch_type	type;	/* what type is it */
	int		offset;	/* offset into some buffer */
	int		size;	/* if the type has a variable size */
	char *		primary_type;
	char *		secondary_type;
	char *		label;
	int		diminished;
	int		printwidth;	/* hack - remove later */
	struct display_mapping *mapping;
	};

struct vsch_record
	{
	int	num_fields;
	int	time;	     /* many records are time stamped; use if needed*/
	int	data_offset;
	struct vsch_field *f;	/* array of fields */
	};


#if 0
/*
* various functions for putting things to files
*/

int vsch_put_time_t(FILE *f, time_t x);

int vsch_put_int(FILE *f, int x);

int vsch_put_counter(FILE *f, counter x);

int vsch_put_ip(FILE *f, void *i);

int vsch_put_str(FILE *f, char *i);

/*
* various functions for getting things back from files
*/

int vsch_get_time_t(FILE *f, int endian, time_t *x);

int vsch_get_int(FILE *f, int endian, int *x);

int vsch_get_counter(FILE *f, int endian, counter *x);

int vsch_get_ip(FILE *f, int endian, void *x);

int vsch_get_str(FILE *f, int endian, char **x);

/*
* various functions for human entered strings into things
*/

int vsch_input_time_t(char *s, time_t *t);

int vsch_input_int(char *s, int *i);

int vsch_input_counter(char *s, time_t *t);

int vsch_input_ip(char *s, time_t *t);

int vsch_input_str(char *s, time_t *t);

#endif /* 0 */

/*
* turn object into human readable string
*/

/* bug: this interface sucks.  first arg should be a vsch_buf */
/*
* for now, b must point at 1000 bytes of space
*/
int vsch_print_type(char *b, void *data, int type, int size);


enum vsch_type vsch_string_to_type(char *s);

/*
* bug: describe interface here
*/
struct vsch_buf
        {
        unsigned char *buffer;
        int     buffer_size;
        int     buffer_offset;
	int	read_point;
        };

int vsch_putbuf(struct vsch_buf *v, enum vsch_type type, void *data, int size);
int vsch_buffer_grow( struct vsch_buf *v, int min_growth );
void vsch_initbuf(struct vsch_buf *v, int size);
int vsch_getbuf(struct vsch_buf *v, enum vsch_type type, void *data,
        int *size);

/* returns offset (& size if variable) of next element */
int vsch_ptrbuf(struct vsch_buf *v, enum vsch_type type, int *size);


#endif
