/*

Copyright(C) 1997 Network Flight Recorder, Inc.
All rights reserved.

Use and distribution of this software and its source code
are governed by the terms and conditions of the
Network Flight Recorder Software License ("LICENSE.TXT")

By Mark Sienkiewicz / NFR

*/

#include "sys-include.h"
#include "cf.h"
#include "counter.h"
#include "vsch.h"
#include "list.h"

load_vsch_record(struct vsch_record *p, PCf conf)
{
int x;
char *s;
int num_columns;

s = Cf_GetVal("num_columns",conf);
if (!s)
	{
	fprintf(stderr,"need num_columns\n");
	exit(1);
	}
num_columns = atoi(s);
if (num_columns < 1)
	{
	fprintf(stderr,"need >= 1 num_columns\n");
	exit(1);
	}

p->num_fields = num_columns;

/*
* allocate 1 extra in case we need to use time as axis in histogram
*/
p->f = calloc(num_columns + 1, sizeof(struct vsch_field));
if (!p->f)
	{
	fprintf(stderr,"nomem\n");
	exit(1);
	}

for (x=0; x< num_columns; x++)
	{
	char b[100];
	int t;
	sprintf(b,"column_%d_type",x+1);
	s=Cf_GetVal(b,conf);
	if (!s)
		{
		fprintf(stderr,"need column %d type\n",x+1);
		exit(1);
		}
	p->f[x].primary_type = s;
	t = query_expand_type(s, &p->f[x].secondary_type);
	if (t == VSCH_ERROR)
		{
		fprintf(stderr,"unknown type in column %d\n",x+1);
		exit(1);
		}
	p->f[x].type=t;
	p->f[x].size=vsch_size(t);
	sprintf(b,"column_%d_label",x+1);
	s=Cf_GetVal(b,conf);
	if (s)
		p->f[x].label=s;
	else
		p->f[x].label="";
	sprintf(b,"column_%d_printwidth", x+1);

	p->f[x].diminished = 0;
	s=Cf_GetVal(b,conf);
	if (s)
		p->f[x].printwidth = atoi(s);
	else
		{
		switch (t)
			{
		case VSCH_INT: case VSCH_COUNTER: case VSCH_TIME_T:
			p->f[x].printwidth=10;
			break;
		case VSCH_IP:
			p->f[x].printwidth=16;
			break;
		case VSCH_STR: case VSCH_BLOB:
			p->f[x].printwidth=30;
			break;
		default:
			p->f[x].printwidth=15;
			}
		}

	if (p->f[x].printwidth < strlen(p->f[x].label) )
			p->f[x].printwidth = strlen(p->f[x].label);
	}
}

