/*
*        Copyright(C) 1997 Network Flight Recorder, Inc. 
*        All rights reserved. 
* 
*        Use and distribution of this software and its source code 
*        are governed by the terms and conditions of the 
*        Network Flight Recorder Software License ("LICENSE.TXT") 
* 
*By Mark Sienkiewicz / NFR 
* 
*/ 


#include "sys-include.h"
#include "mapping.h"
#include "home.h"
#include "query.h"

struct display_node
	{
	struct display_node *next;
	struct query_match *what;
	struct display_mode mode;
	};

struct display_mapping
	{
	struct display_node *first;
	int type;
	struct display_mode other;
	};

struct display_mapping *
mapping_open(int vsch_type)
{
struct display_mapping *dm;
dm=calloc(1,sizeof(struct display_mapping));
if (!dm)
	abort(); /* bug */
dm->type=vsch_type;
dm->other.string="";
dm->other.color="";
return dm;
}

mapping_load(struct display_mapping *dm, char *name1, char *name2, char *name3 )
{
FILE *f;
char b[1000];
int line;
struct display_node *x, *prev = NULL;
char *s = nfrConcat(name1, "/", name2, "/", name3, ".map", NULL);
f=fopen(s,"r");
if (!f)
	{
	perror(s);
	return 0;
	}
/*
* line 1 - vsch type of mapping in this file
*/
if (!fgets(b,sizeof(b),f))
	{
	fprintf(stderr,"%s: no mapping type in file\n",s);
	return 0;
	}
if (atoi(b) != dm->type)
	{
	fprintf(stderr,"%s: bad mapping type in file - wanted %d\n",
		s,dm->type);
	return 0;
	}
/*
* line 2 - mapping of "other", if it is not a comment
*/
if (fgets(b,sizeof(b),f))
	{
	if ((b[0] != '#') && (b[0] != '\n'))
		display_mode_break_line(&dm->other,b);
	}
line=3;
if (dm->first)
	{
	for (prev = dm->first ; prev->next ; prev=prev->next)
		;
	}
while (fgets(b,sizeof(b),f))
	{
	char *s;
	if (b[0] == '#')
		continue;
	if (b[0] == '\n')
		continue;
	if (! (s=strchr(b,'\n')))
		{
		int c;
		fprintf(stderr,"%s: query too long at line %d\n",s,line);
		do
			c=getc(f);
		while ((c != EOF) && (c != '\n'));
		}
	*s='\0';
	line++;
	x=calloc(1,sizeof(struct display_node));
	if (!x)
		abort(); /* bug */
	x->what = query_compile(dm->type,b);
	if (!fgets(b,sizeof(b),f))
		fprintf(stderr,"%s: expected display mode at line %d\n",line);
	else
		display_mode_break_line(&x->mode,b);
	if (prev)
		prev->next = x;
	else
		dm->first = x;
	prev = x;
	}
fclose(f);
return 1;
}

display_mode_break_line(struct display_mode *mode,char *b)
{
char *s;
s=strchr(b,'\n');
if (s) *s='\0';
else { } /* bug: warn of bad read */
s=strchr(b,' ');
if (s)
	{
	*s++ = 0;
	mode->string=strdup(s);
	}
else
	mode->string=strdup("");
mode->color=strdup(b);
}

struct display_mode *
mapping_lookup(struct display_mapping *dm, void *value, int size)
{
struct display_node *n;
if (!value)
	return &dm->other;
for (n=dm->first; n; n=n->next)
	{
	if (query_compare(n->what, value, dm->type, size))
		return &n->mode;
	}
return (void *)0;
}

