/*
*        Copyright(C) 1997 Network Flight Recorder, Inc. 
*        All rights reserved. 
* 
*        Use and distribution of this software and its source code 
*        are governed by the terms and conditions of the 
*        Network Flight Recorder Software License ("LICENSE.TXT") 
* 
*By Mark Sienkiewicz / NFR 
* 
*/ 

 
/*
* WARNING:  Due to bugs in various versions of yacc, the C code that
* this file produces is edited by sed afterwards.  It will say
*       s/yy/yy_str_/g
*
* I suggest you not use variables with yy in their name.
*/



%token tok_str 260
%{
#define yacc_stupid_tok_str 260

/*
* Some versions of yacc do not ouput the defines for the tokens until
* _after_ all of the code you have here.  This hack lets me know the
* token number, so I can include my lexical analyzer here. If you
* start the have more than a few token types, you'll want to pull
* the lexical analyzer out into a seperate file and use the y.tab.h
* method to get it included. Also fix the makefile to rename the .h
* file.
* 
* I'm not going to do all that for one token.
*/

#define YYDEBUG 1

#include "sys-include.h"

#include "counter.h"
#include "vsch.h"
#include "query.h"

%}


%union  {
#include "query_union.h"
}

%{

/*
* input string
*/
extern char *yacc_input_string;

/*
* last token string, if token is a type that makes a string
*/
extern char *query_lex_string;


/*
* lexical analyzer:
* white space is ignored
* "asd adf adf " is a string, quotes delimit it
* adsfa is an unquoted string, whitespace, - or , delimits it
* - is a token
* , is a token
* EOF is a token
*/

/*
* changed to yy_str_getchar by sed
*/
#define yygetchar() (*yacc_input_string ? *yacc_input_string++ : -1 )

/*
* changed to yy_str_lex by sed
*/
yylex()
{
int c;
eatwsp:
	c=yygetchar();
	if (isspace(c)) goto eatwsp;
	switch (c)
		{
	case '-':
	case ',':
	case -1:
		return c;
	case '"':
		query_lex_init();
		for (;;)
			{
			c = yygetchar();
			if ((c == -1) || (c == '"'))
				break;
			query_lex_add(c);
			}
fprintf(stderr,"token:%s:\n",query_lex_string);
		return yacc_stupid_tok_str;
	default:
		query_lex_init();
		query_lex_add(c);
		for (;;)
			{
			c=yygetchar();
			if ((c == ',') || (c == '-'))
				{
				yacc_input_string --;
				break;
				}
			if (c == -1)
				break;
			if (isspace(c))
				break;
			query_lex_add(c);
			}
fprintf(stderr,"token:%s:\n",query_lex_string);
		return yacc_stupid_tok_str;
		}
}

/*
* changed to yy_str_error by sed
*/
static yyerror(char *s)
{
extern had_yacc_error;
had_yacc_error=1;
}


%}

%%

list:   
		{
		}
	| listitem
		{
		query_add_list($1);
fprintf(stderr,"listitem :%s:%s:\n",$1.string_range.first,$1.string_range.second);
		$$=$1;
		}
        | list ',' listitem
		{
		query_add_list($3);
		$$ = $1;
fprintf(stderr,"listitem :%s:%s:\n",$3.string_range.first,$3.string_range.second);
		}
	;

listitem: range
		{
		$$= $1;
		}
        | string
		{
		$$ = $1;
		}
	;

string:	tok_str
		{
		$$.string_range.count=1;
		$$.string_range.first = query_lex_string;
		$$.string_range.second = query_lex_string;
fprintf(stderr,"string :%s:\n",$$.string_range.first);
		}
	;

range:	string '-' string 
		{
		$$.string_range.count=2;
		$$.string_range.first = $1.string_range.first;
		$$.string_range.second = $3.string_range.first;
fprintf(stderr,"range :%s:%s:\n",$1.string_range.first,$3.string_range.first);
		}
	;

