/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   cgiutils.c
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

#include "sys-include.h"

#include "nfr_res.h"
#include "libnfr.h"
#include "cgiutils.h"

void Cgi_Debug(int Argc, char **Argv)
{
  char DumpName[256];
  char *Name = NULL;
  char *p = NULL;


  fprintf(stderr, "Cgi_Debug called from %s\n", Argv[0]);

  p = Argv[0];
  Name = Argv[0];
  while (*p != 0)
    {
      if(*p++ == '/')
        Name = p;
    }
  if (Name == NULL)
    Name = "cgi";
  sprintf(DumpName, "tmp/%s.out", Name);
  if(getenv("REQUEST_METHOD") == NULL) /* we're being run sans httpd */
    Cgi_InitLocalTest(DumpName); 
  else
    {
      Cgi_DumpQuery(DumpName); 
      Cgi_InitLocalTest(DumpName);
    }
  fprintf(stderr, "HTTP_USER_AGENT: %s\n", getenv("HTTP_USER_AGENT"));
}

void Cgi_InitLocalTest(char *Infile)
{
  static char LenStr[64];
  static char QueryStr[4096];
  char c;
  ULONG Len;
  FILE *f = NULL;

  if (getenv("HTTP_USER_AGENT") == NULL)
    putenv("HTTP_USER_AGENT=andrew's cgi debugger/0.6.6.6");
  f = freopen(Infile, "r", stdin);
  c = fgetc(stdin);
  if(c == 'P')
    {
      fread(&Len, sizeof(Len), 1, stdin);
      sprintf(LenStr, "CONTENT_LENGTH=%lu", Len);
      putenv("REQUEST_METHOD=POST");
      putenv("CONTENT_TYPE=application/x-www-form-urlencoded");
      putenv(LenStr);
    }
  else if(c == 'G')
    {
      fread(&Len, sizeof(Len), 1, stdin);
      strcpy(QueryStr, "QUERY_STRING=");
      fgets(QueryStr + strlen(QueryStr), 
	    sizeof(QueryStr) - strlen(QueryStr) - 1, 
	    stdin);		/* blech, but it's only debugging code */
      putenv("REQUEST_METHOD=GET");
      putenv(QueryStr);
    }
  
  else
    {
      fputs("hosed dump file\n", stderr);
      SExit(1);
    }
}

int Cgi_DumpQuery(char *Outfile) 
{
  int DataLen;
  char *Data;
  FILE *Out;
  char *Str;

  Str = getenv("REQUEST_METHOD");
  if (!Str)
    return NFR_ERR_UNDEFINED;

  fprintf(stderr, "Cgi_DumpQuery() debug output to %s\n", Outfile);
  Out = fopen(Outfile, "w");
  if(!Out)
    {
      perror(Outfile);
      return NFR_ERR_UNDEFINED;
    }

  if(strcmp(Str, "POST") == 0)
    {
      fprintf(stderr, "Cgi_DumpQuery() doing the POST thang\n");
      fputc('P', Out);
      DataLen = atoi(getenv("CONTENT_LENGTH"));
      Data = (char *) SCalloc(1, DataLen + 1);
      fread(Data, 1, DataLen, stdin);
      fwrite(&DataLen, 1, sizeof(DataLen), Out);
      fwrite(Data, 1, DataLen, Out);
      SFree(Data);
    }
  else if(strcmp(Str,"GET") == 0)
    {
      fprintf(stderr, "Cgi_DumpQuery() doing the GET thang\n");
      fputc('G', Out);
      Data = getenv("QUERY_STRING");
      DataLen=strlen(Data);
      fwrite(&DataLen, 1, sizeof(DataLen), Out);
      fwrite(Data, 1, DataLen, Out);
    }

  else
    {
      fprintf(stderr, "Cgi_DumpQuery() doing the exit thang\n");
      SExit(1);
    }
  
  fclose(Out);

  return NFR_ERR_SUCCESS;
}

int MimeHeaderEx(FILE *Out, char *ResultCode, char *Date, char *ContentType, 
		 char *ContentEncoding, int ContentLength,
		 char *WindowTarget)
{
  time_t Time;
  char DateBuf[25];
  char DefResultCode[] = "200 OK";
  char DefContentType[] = "text/html";

  if(Date == NULL)
    {
      Time = time(0);
      memcpy(DateBuf, ctime(&Time), 24); /* copy all but the LF */
      DateBuf[25] = 0;
      Date = DateBuf;
    }

  if (Out == NULL)
    Out = stdout;

  if (ResultCode == NULL)
    ResultCode = DefResultCode;

  if (ContentType == NULL)
    ContentType = DefContentType;

  fprintf (Out,
	   "HTTP/1.0 %s\r\n"
	   "Date: %s\r\n"
	   "MIME-Version: 1.0\r\n"
	   "Content-type: %s\r\n",
	   ResultCode, Date, ContentType);
  
  if (ContentEncoding != NULL)
    fprintf(Out, "Content-encoding: %s\r\n", ContentEncoding);
    
  if (ContentLength > -1)
    fprintf(Out, "Content-length: %d\r\n", ContentLength);
    
  if (WindowTarget != NULL)
    fprintf(Out, "Window-target: %s\r\n", WindowTarget);
    
  fputc('\r', Out);
  fputc('\n', Out);
  return NFR_ERR_SUCCESS;
}

int Cgi_QueryToCfEx(ULONG Flags, TCf *pCf)
{
  ULONG DataLen = 0;
  char *Str;
  char *NextPair;
  char *Data = NULL;
  BOOL DataMalloced = FALSE;
  char *Name = NULL;
  char *Val = NULL;

  Str = getenv("REQUEST_METHOD");
  if (!Str)
    return NFR_ERR_UNDEFINED;

  if(strcmp(Str,"POST") == 0)
    {
      if(!(Flags & ACCEPT_POST))
	{
	  fprintf(stderr, "I want a \"POST\" not a \"%s\"\n", Str);
	  return NFR_ERR_UNDEFINED;
	}

      Str = getenv("CONTENT_TYPE");
      if (Str == NULL)
	{
	  fprintf(stderr, "unrecognized content type: (NULL)\n");
	  return NFR_ERR_UNDEFINED;
	}
      
      if(strcmp(Str,"application/x-www-form-urlencoded") == 0)
	;			/* it's ok */

      else if(strcmp(Str,"multipart/form-data") == 0)
	;			/* it's ok */

      else			/* it's not ok */
	{
	  fprintf(stderr, "unrecognized content type: \"%s\"\n", Str);
	  return NFR_ERR_UNDEFINED;
	}
      
      Str = getenv("CONTENT_LENGTH");
      if (!Str)
	return NFR_ERR_UNDEFINED;
      DataLen = strtoul(Str, NULL, 0);
      Data = (char *) SCalloc(1, DataLen + 1);
      DataMalloced = TRUE;
      if (fread(Data, 1, DataLen, stdin) != DataLen)
	return NFR_ERR_UNDEFINED;
    }
  else if (strcmp(Str,"GET") == 0)
    {
      if(!(Flags & ACCEPT_GET))
	return NFR_ERR_UNDEFINED;
      Data = getenv("QUERY_STRING");
      DataLen=strlen(Data);
    }
  
  
  NextPair = Data;
  while(NextPair < (Data + DataLen))
    {
      if( (*NextPair == 0) || (isspace(*NextPair)) )
	break;

      Name = NextPair;
      while( (*NextPair != '&') && (*NextPair != 0) && (!isspace(*NextPair)) )
	NextPair++;
      *NextPair++ = 0;

      plustospace(Name);	/* it's really still the whole pair here */
      unescape_url(Name);

      Val = Name;
      /* bug: the following is bad bad bad fix to handle blabla=\0 */
      while((*Val != '=') && (*Val != 0))
	Val++;
      *Val++ = 0;

      /* have to call strlen cause the URL decoding may shorten the length */
      Cf_NVCreate(Name, Val, strlen(Val), pCf);
    }
  if(DataMalloced)
    SFree(Data);
  return NFR_ERR_SUCCESS;
}

int Cgi_CfToForm(TCf *pCf, FILE *Input, FILE *Output)
{
  char c;
  char VarName[MAXSIZE_CF_NAME];
  ULONG VarNameLen = 0;
  char *ExpVar;
  ULONG State = FORM_INIT;

  while((c = fgetc(Input)) != EOF)
    {
      switch (State)
	{
	case FORM_INIT:
	  if (c == '$')
	    State = FORM_GET_VARNAME;
	  else
	    fputc(c, Output);
	  break;

	case FORM_GET_VARNAME:
	  if ( (isalnum(c)) || (c == '_') )
	    VarName[VarNameLen++] = c;
	  else
	    {
	      VarName[VarNameLen] = 0;
	      ExpVar = Cf_GetVal(VarName, pCf);
	      if (ExpVar != NULL)
		fwrite(ExpVar, strlen(ExpVar), 1, Output);
	      fputc(c, Output);
	      VarNameLen = 0;
	      State = FORM_INIT;
	    }
	  break;
	}
    }
  return NFR_ERR_SUCCESS;
}

int Cgi_Response(char *msg, int len, int Result)
{
  ULONG nboResult = 0;
  ULONG nboLen = 0;
  int bytes = 0;
  static BOOL sentHeader = FALSE;

  if(msg)
    fputs(msg, stderr);

  if(!sentHeader)
    {
      printf("Content-type: application/octet-stream\r\n\r\n");
      sentHeader = TRUE;
    }

  if((len == 0) && (msg))
    len = strlen(msg);

  nboLen = htonl(len);
  nboResult = htonl(Result);

  fprintf(stderr,"\nlen:\t\t%ld\nnboLen:\t\t%ld\n"
	  "Result:\t\t%ld\nnboResult:\t%ld\n",
	  len, nboLen, Result, nboResult);

  bytes = fwrite(&nboResult, 1, sizeof(nboResult), stdout);
  if(bytes < sizeof(nboResult))
    {
      perror("stdout");
      return NFR_ERR_UNDEFINED;
    }

  bytes = fwrite(&nboLen, 1, sizeof(nboLen), stdout);
  if(bytes < sizeof(nboLen))
    {
      perror("stdout");
      return NFR_ERR_UNDEFINED;
    }

  if(len > 0)
    {
      bytes = fwrite(msg, 1, len, stdout);
      if(bytes < len)
	{
	  perror("stdout");
	  return NFR_ERR_UNDEFINED;
	}
    }

  fputs("done with Cgi_Response()\n", stderr);
  return NFR_ERR_SUCCESS;
}

/*****************************************************************************
 * the rest of this file is taken from utils.c in the cgi-src dir in the
 * ncsa httpd distribution
 ****************************************************************************/

char x2c(char *what) {
    register char digit;

    digit = (what[0] >= 'A' ? ((what[0] & 0xdf) - 'A')+10 : (what[0] - '0'));
    digit *= 16;
    digit += (what[1] >= 'A' ? ((what[1] & 0xdf) - 'A')+10 : (what[1] - '0'));
    return(digit);
}

void unescape_url(char *url) {
    register int x,y;

    for(x=0,y=0;url[y];++x,++y) {
        if((url[x] = url[y]) == '%') {
            url[x] = x2c(&url[y+1]);
            y+=2;
        }
    }
    url[x] = '\0';
}

void plustospace(char *str) {
    register int x;

    for(x=0;str[x];x++) if(str[x] == '+') str[x] = ' ';
}

void escape_shell_cmd(char *cmd) {
    register int x,y,l;

    l=strlen(cmd);
    for(x=0;cmd[x];x++) {
        if(ind("&;`'\"|*?~<>^()[]{}$\\",cmd[x]) != -1){
            for(y=l+1;y>x;y--)
                cmd[y] = cmd[y-1];
            l++; /* length has been increased */
            cmd[x] = '\\';
            x++; /* skip the character */
        }
    }
}

int rind(char *s, char c) {
    register int x;
    for(x=strlen(s) - 1;x != -1; x--)
        if(s[x] == c) return x;
    return -1;
}

int ind(char *s, char c) {
    register int x;

    for(x=0;s[x];x++)
        if(s[x] == c) return x;

    return -1;
}

