/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   cf.c
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

/******************************************************************************
 * cf.c - use these routines and structures to access config files or
 * registry keys
 *****************************************************************************/
#include "sys-include.h"
#include "cf.h"

#include "symtab.h"
#include "memsafe.h"
#include "xtypes.h"

#include "libnfr_err.h"


PCf Cf_New(int Size)
{
  PCf pNewCf = NULL;
  
  pNewCf = SCalloc(1, sizeof(TCf));

 /* we can use the Size arg to optimize this if we ever begin using really
  * huge ones of these, for now we'll ignore it */
  pNewCf->SymTab = SymTab_New(512); 
  return(pNewCf);
}

void Cf_Delete(PCf pCf)
{
  SymTab_DeleteAll(Cf_NVOnDelete, pCf->SymTab);
  SFree(pCf);
}

void Cf_NVOnDelete(PSymbol pSym)
{
  if(((PNVPair)(pSym->Data))->Val != NULL)
    SFree(((PNVPair)(pSym->Data))->Val);
  if(((PNVPair)(pSym->Data))->ExpVal != NULL)
    SFree(((PNVPair)(pSym->Data))->ExpVal);
  SFree((void *)(pSym->Data));
}

int Cf_SaveAs(char *CfName, PCf pCf)
{
  FILE *CfFile;

  CfFile = fopen (CfName, "w");
  if(!CfFile)
    return LIBNFR_ERR_UNDEFINED;
  SymTab_ForEach(ALL_TYPES, Cf_NVSymPickle, CF_FLAG_PUT_LF, CfFile, pCf->SymTab);
  return LIBNFR_ERR_SUCCESS;
}

int Cf_AppendTo(char *CfName, PCf pCf)
{
  FILE *CfFile;

  CfFile = fopen (CfName, "a");
  if(!CfFile)
    return LIBNFR_ERR_UNDEFINED;
  SymTab_ForEach(ALL_TYPES, Cf_NVSymPickle, CF_FLAG_PUT_LF, CfFile, pCf->SymTab);
  return LIBNFR_ERR_SUCCESS;
}

/* issue: should Cf_ be case sensitive? */

char *Cf_GetVal(char *Name, PCf pCf)
{
  PSymbol pSym = NULL;
  int Result = 0;

  Result = SymTab_Peek(Name, &pSym, pCf->SymTab);
  if (Result == SYM_NOT_FOUND)
    return NULL;		/* it wasn't there :( */
  return ((PNVPair)pSym->Data)->Val;
}

char *Cf_TakeVal(char *Name, PCf pCf)
{
  PNVPair pNV = NULL;
  char *Val = NULL;

  pNV = Cf_TakeNVPair(Name, pCf);
  if (pNV == NULL)
    return NULL;
  if(pNV->ExpVal != NULL)
    SFree(pNV->ExpVal);
  Val = pNV->Val;
  SFree(pNV);
  return Val;
}

PNVPair Cf_TakeNVPair(char *Name, PCf pCf)
{
  PSymbol pSym = NULL;
  int Result = 0;
  PNVPair pNV = NULL;

  return (PNVPair) SymTab_Remove(Name, pCf->SymTab);
}

int Cf_Merge(char *CfName, PCf pCf)
{
  char Tplt[] = "tmp/cfudXXXXXX";
  char *TempName = NULL;

  TempName = mktemp(Tplt);
  rename(CfName, TempName);
  Cf_Parse(CF_FLAG_MODE_MERGE, TempName, CfName, pCf);
  unlink(TempName);
  return LIBNFR_ERR_SUCCESS;
}

ULONG Cf_ForEach(PfnCfIterate pIterate, void *PArg, PCf pCf)
{
  return SymTab_ForEach(ALL_TYPES, Cf_Iterate, (int) pIterate, 
			PArg, pCf->SymTab);
}


PNVPair Cf_GetNVPair(char *Name, PCf pCf)
{
  PSymbol pSym = NULL;
  int Result = 0;

  Result = SymTab_Peek(Name, &pSym, pCf->SymTab);
  if (Result == SYM_NOT_FOUND)
    return NULL;		/* it wasn't there :( */
  return (PNVPair)pSym->Data;
}

char *Cf_ExpandVal(char *Name, PCf pCf)
{
  PSymbol pSym = NULL;
  int Result = 0;
  TNIList ValList;
  char *p;
  char Buf[MAXSIZE_CF_VAL];
  char VarName[MAXSIZE_CF_VAL];
  USHORT VarNameLen;
  PNVPair pNV;
  

  memset(&ValList, 0, sizeof(TNIList));

  Result = SymTab_Peek(Name, &pSym, pCf->SymTab);
  if (Result == SYM_NOT_FOUND)
    return NULL;		/* it wasn't there :( */

  pNV = (PNVPair) pSym->Data;
  if (pNV->State == NV_EXPANDED)
    return pNV->ExpVal;


  pNV->State = NV_EXP_INIT;
  pNV->ExpValLen = 0;
  VarNameLen = 0;

  for(p = pNV->Val; pNV->State != NV_EXPANDED; p++)
    {
      switch (pNV->State)
	{
	case NV_EXP_INIT:
	  if (*p == '$')
	    pNV->State = NV_EXP_VAR;
	  else if (*p == '\\') 
	        pNV->State = NV_EXP_ESC;
 	  else if (*p == 0)
            {
              Buf[pNV->ExpValLen] = 0;
              pNV->State = NV_EXPANDED;
            }
	  else
	    Buf[pNV->ExpValLen++] = *p;
	  break;

	case NV_EXP_VAR:
	  if ((isalnum(*p)) || (*p == '_'))
	    VarName[VarNameLen++] = *p;
	  else if (*p == 0)
	    {
	      pNV->State = NV_EXPANDED;
	      VarName[VarNameLen] = 0;
	      if ( (strcmp(VarName, Name)) &&
                   (SymTab_Peek(VarName, NULL, pCf->SymTab)) == SYM_FOUND)
		{
		  strncpy(Buf + pNV->ExpValLen++, 
			  Cf_ExpandVal(VarName, pCf),
			  sizeof(Buf) - pNV->ExpValLen - 1);
		  pNV->ExpValLen += strlen(Buf + pNV->ExpValLen);
		}
	      VarNameLen = 0;
              Buf[pNV->ExpValLen] = 0;
	    }
	  else if (*p == '$')
	    {
	      pNV->State = NV_EXP_VAR;
	      VarName[VarNameLen] = 0;
	      if ( (strcmp(VarName, Name)) &&
                   (SymTab_Peek(VarName, NULL, pCf->SymTab)) == SYM_FOUND)
		{
		  strncpy(Buf + pNV->ExpValLen++, 
			  Cf_ExpandVal(VarName, pCf),
			  sizeof(Buf) - pNV->ExpValLen - 1);
		  pNV->ExpValLen += strlen(Buf + pNV->ExpValLen);
		}
	      VarNameLen = 0;
	    }
	  else if (*p == '\\')
	    {
	      VarName[VarNameLen] = 0;
	      if ( (strcmp(VarName, Name)) &&
                   (SymTab_Peek(VarName, NULL, pCf->SymTab)) == SYM_FOUND)
		{
		  strncpy(Buf + pNV->ExpValLen++, 
			  Cf_ExpandVal(VarName, pCf),
			  sizeof(Buf) - pNV->ExpValLen - 1);
		  pNV->ExpValLen += strlen(Buf + pNV->ExpValLen);
		}
	      VarNameLen = 0;
	        pNV->State = NV_EXP_ESC;
	    }
	  else
	    {
	      pNV->State = NV_EXP_INIT;
	      VarName[VarNameLen] = 0;
	      if ( (strcmp(VarName, Name)) &&
                   (SymTab_Peek(VarName, NULL, pCf->SymTab)) == SYM_FOUND)
		{
		  strncpy(Buf + pNV->ExpValLen++, 
			  Cf_ExpandVal(VarName, pCf),
			  sizeof(Buf) - pNV->ExpValLen - 1);
		  pNV->ExpValLen += strlen(Buf + pNV->ExpValLen);
		}
	      Buf[pNV->ExpValLen++] = *p;
	      VarNameLen = 0;
	    }
	  break;

	case NV_EXP_ESC:
	  pNV->State = NV_EXP_INIT;
	  if(*p == 'n')
	    Buf[pNV->ExpValLen++] = '\n';
	  else if(*p == 't')
	    Buf[pNV->ExpValLen++] = '\t';
	  else if (*p == ' ')
	    pNV->State = NV_EXP_INIT;
	  else if (*p == 0)
	    pNV->State = NV_EXPANDED;
	  else
	    Buf[pNV->ExpValLen++] = *p;
	  break;
	}

      if (pNV->ExpValLen >= sizeof(Buf))
	{
	  pNV->State = NV_EXPANDED;
	  break;
	}
    }

  p = Buf;
  if(pNV->QChar != 0)
    {
      pNV->ExpValLen -= 2;
      p++;
      *(p + pNV->ExpValLen) = 0;
    }
  pNV->ExpVal = (char *) SCalloc(1, pNV->ExpValLen + 1);
  memcpy(pNV->ExpVal, p, pNV->ExpValLen);
  pNV->State = NV_EXPANDED;
  return pNV->ExpVal;
}



int Cf_Iterate(PSymbol pSym, int pIterate, void *PArg)
{
  return ((PfnCfIterate) pIterate) ((PNVPair)pSym->Data, PArg);
}

int Cf_NVOnComplete(ULONG Flags, PNVPair pNV, FILE *InFile, FILE *OutFile, 
		    PCf pCf)
{
  ULONG xxWSLen = 0;
  char *xxWS = NULL;
  PNVPair NewPair = NULL;
  int Result = 0;
  
  if (pCf->State >= CF_HAVE_NO_NVPAIR)
    return LIBNFR_ERR_SUCCESS;	/* we must've already been here */
  else if (pCf->State == CF_GET_QVAL)
    pCf->State = CF_HAVE_BROKEN_NVPAIR;	/* never saw end quote :( */
  else if (pNV->NameLen == 0)
    pCf->State = CF_HAVE_NO_NVPAIR; /* didn't even get a name */
  else
    {
      pCf->State = CF_HAVE_GOOD_NVPAIR;

      /* get rid of any trailing whitespace on the value and dump it
         to the output, if necessary */
      xxWS = (pNV->Val + ( pNV->ValLen - 1) );
      while (isspace(*xxWS))
	{
	  xxWS--;
	  xxWSLen++;
	}
      xxWS++;
      pNV->ValLen -= xxWSLen;

      /* have to do this now so that the call to Cf_GetNVPair() will work */
      *(pNV->Name + pNV->NameLen) = 0;

      if(Flags & CF_FLAG_MODE_MERGE)
	{
	  NewPair = Cf_GetNVPair(pNV->Name, pCf);
	  
	  /* don't overwrite unless explicitly requested */

	  if(NewPair == NULL)	/* use the val from the .cf file */
	    {
	      fwrite(pNV->Val, pNV->ValLen, 1, OutFile);
	    }

	  else			/* use the val from the Cf obj */
	    {
	      NewPair->Flags |= CF_NVFLAG_MERGED;
	      fwrite(NewPair->Val, NewPair->ValLen, 1, OutFile);
	    }
	  fwrite(xxWS, xxWSLen, 1, OutFile);
	}

      /* we do this here 'cause if we do it earlier we'll hose xxWS */
      *(pNV->Val + pNV->ValLen) = 0; 
    }       
  return LIBNFR_ERR_SUCCESS;
}

int Cf_NVParse(ULONG Flags, PNVPair pNV, FILE *InFile, FILE *OutFile, PCf pCf)
{
  int Result = 0;
  char c = 0;

  while (TRUE)
    {
      c = fgetc(InFile);

      /* first handle any of the really special chars */
      if(c == '\n')
	{
	  Cf_NVOnComplete(Flags, pNV, InFile, OutFile, pCf);
	  Result = pCf->State;
	  if(Flags & CF_FLAG_MODE_MERGE)
	    fputc(c, OutFile);
	  if (pCf->State > CF_HAVE_NO_NVPAIR)
	    return LIBNFR_ERR_SUCCESS; /* only return when we've found one */
	  pCf->State = CF_WANT_NAME; /* reset state */
	}

      else if(c == EOF)
	{
	  Cf_NVOnComplete(Flags, pNV, InFile, OutFile, pCf);
	  if(pCf->State >= CF_HAVE_NO_NVPAIR)
	    pCf->State += 3;	/* bump up to corresponding eof state */
	  else
	    pCf->State = CF_HAVE_NO_NVPAIR_EOF;
	  return LIBNFR_ERR_SUCCESS;
	}

      else if(c == '#')
	{
	  Cf_NVOnComplete(Flags, pNV, InFile, OutFile, pCf);
	  if(Flags & CF_FLAG_MODE_MERGE)
	    fputc(c, OutFile);
	}

      else			/* handle all the other chars */
	{
	  switch (pCf->State)
	    {
	    case CF_WANT_NAME:
	      if(Flags & CF_FLAG_MODE_MERGE)
		fputc(c, OutFile);
	      if(isalnum(c) || (c == '_'))
		{
		  *(pNV->Name) = c;
		  pNV->NameLen = 1;
		  pCf->State = CF_GET_NAME;
		}
	      else if(c == '=')
		pCf->State = CF_HAVE_NO_NVPAIR; /* ignore rest of line */
	      else
		pNV->NameLen = 0;
	      break;
	  
	    case CF_GET_NAME:
	      pNV->ValLen = 0;	/* need to find a better place to do this?  */
	      if(isalnum(c) || (c == '_'))
		{
		  if(Flags & CF_FLAG_MODE_MERGE)
		    fputc(c, OutFile);
		  if(pNV->NameLen == (MAXSIZE_CF_NAME - 1))
		    break;
		  *(pNV->Name + pNV->NameLen++) = c;
		}
	      else if(c == '=')
		{
		  if(Flags & CF_FLAG_MODE_MERGE)
		    fputc(c, OutFile);
		  pCf->State = CF_WANT_VAL;
		}
	      else
		{
		  if(Flags & CF_FLAG_MODE_MERGE)
		    fputc(c, OutFile);
		  pCf->State = CF_WANT_ASS;
		}
	      break;

	    case CF_WANT_ASS:		/* oh, come on! it stands for assIGNment. */
	      if(c == '=')
		{
		  if(Flags & CF_FLAG_MODE_MERGE)
		    fputc(c, OutFile);
		  pNV->ValLen = 0;
		  pCf->State = CF_WANT_VAL;
		}
	      else if(Flags & CF_FLAG_MODE_MERGE)
		fputc(c, OutFile);
	      break;
    
	    case CF_WANT_VAL:
	      if((c == '\'') || (c == '\"'))
		{
		  pNV->QChar = c;
		  pNV->ValLen = 1;
		  *(pNV->Val) = c;
		  pCf->State = CF_GET_QVAL;
		}
	      else if(isspace(c))
		{
		  if(Flags & CF_FLAG_MODE_MERGE)
		    fputc(c, OutFile);
		  pNV->ValLen = 0;
		}
	      else
		{
		  pNV->QChar = 0;
		  *(pNV->Val) = c;
		  pNV->ValLen = 1;
		  pCf->State = CF_GET_VAL;
		}
	      break;
	
	    case CF_GET_VAL:
	      if(pNV->ValLen == (MAXSIZE_CF_VAL - 1))
		break;
	      else
		*(pNV->Val + pNV->ValLen++) = c;
	      break;

	    case CF_GET_QVAL:
	      if(pNV->ValLen == (MAXSIZE_CF_VAL - 1))
		break;
	      else if(c == pNV->QChar)
		{
		  *(pNV->Val + pNV->ValLen++) = c;
		  pCf->State = CF_GET_VAL; /* indicate we saw the end quote */
		  Cf_NVOnComplete(Flags, pNV, InFile, OutFile, pCf) ;
		}
	      else if(c == '\\')
		{
		  *(pNV->Val + pNV->ValLen++) = c;
		  pCf->State = CF_GET_QVAL_ESC;
		}
	      else
		*(pNV->Val + pNV->ValLen++) = c;
	      break;

	    case CF_GET_QVAL_ESC:
	      pCf->State = CF_GET_QVAL;
	      *(pNV->Val + pNV->ValLen++) = c;
	      break;

	    case CF_HAVE_NO_NVPAIR:
	    case CF_HAVE_GOOD_NVPAIR:
	    case CF_HAVE_BROKEN_NVPAIR:
	    default:
	      if(Flags & CF_FLAG_MODE_MERGE)
		fputc(c, OutFile);
	      break;
	    }
	}
    }
}

void Cf_NVInit(PNVPair pNV)
{
  pNV->NameLen = 0;
  if(pNV->Name != NULL)
    *(pNV->Name) = 0;
  pNV->ValLen = 0;
  if(pNV->Val != NULL)
    *(pNV->Val) = 0;
  pNV->ExpValLen = 0;
  if(pNV->ExpVal != NULL)
    *(pNV->ExpVal) = 0;
  pNV->QChar = 0;
}

int Cf_Parse(ULONG Flags, char *InName, char *OutName, PCf pCf)
{
  char NameBuf[MAXSIZE_CF_NAME];
  int Result = 0;
  char ValBuf[MAXSIZE_CF_VAL];
  TNVPair NVBuf;
  FILE *InFile = stdin;
  FILE *OutFile = stdout;

  memset(&NVBuf, 0, sizeof(TNVPair));
  if(InName != NULL)
    {
      InFile = fopen(InName, "r");
      if (InFile == NULL)
	return LIBNFR_ERR_UNDEFINED;	
      /* bug: should check errno or GetLastError() and return something
       * appropriate */
    }

  if(OutName != NULL)
    {
      OutFile = fopen(OutName, "w");
      if (OutFile == NULL)
	return LIBNFR_ERR_UNDEFINED;	
      /* bug: should check errno or GetLastError() and return something
       * appropriate */
    }

  NVBuf.Name = NameBuf;
  NVBuf.Val = ValBuf;

  pCf->State = CF_WANT_NAME;
  while (TRUE)
    {
      Cf_NVInit(&NVBuf);
      Cf_NVParse(Flags, &NVBuf, InFile, OutFile, pCf);
      
      /* get out if there are any problems */
      if ( (pCf->State == CF_HAVE_NO_NVPAIR_EOF) || 
	   (pCf->State == CF_HAVE_BROKEN_NVPAIR_EOF) )
	break;


      /* add this NVPair if we're not in merge mode, and then get out
         if the file is spent */
      else if (pCf->State == CF_HAVE_GOOD_NVPAIR_EOF &&
	       ( !(Flags & CF_FLAG_MODE_MERGE) ) )
	{
	  Cf_NVCreate(NVBuf.Name, NVBuf.Val, NVBuf.ValLen, pCf);
	  break;
	}

      /* add this NVPair if we're not in merge mode, then keep going */
      else if (pCf->State == CF_HAVE_GOOD_NVPAIR &&
	       ( !(Flags & CF_FLAG_MODE_MERGE) ) )
	{
	  Cf_NVCreate(NVBuf.Name, NVBuf.Val, NVBuf.ValLen, pCf);
	}
      pCf->State = CF_WANT_NAME;
    }

  /* if we're in merge mode, all the NVPairs appearing in the source
     file have been updated, so now we need to add any NVPairs that
     appear only in the source Cf object */
  if (Flags & CF_FLAG_MODE_MERGE)
    {
      fputc('\n', OutFile);
      Cf_ForEach(Cf_WriteUnmergedNV, OutFile, pCf);
    }


  if(InFile != stdin)
    fclose (InFile);
  if(OutFile != stdout)
    fclose (OutFile);
  return LIBNFR_ERR_SUCCESS;
}

int Cf_WriteUnmergedNV(PNVPair pNV, void *f)
{
  /* this function exists only to enable merging a Cf obj int a file
     which doesn't already contain at least names for all NVPairs
     appearing in the Cf obj */
  if(!(pNV->Flags & CF_NVFLAG_MERGED))
    {
      fwrite(pNV->Name, pNV->NameLen, 1, (FILE *)f);
      fputc('=', (FILE *)f);
      fwrite(pNV->Val, pNV->ValLen, 1, (FILE *)f);
      fputc('\n', (FILE *)f);
    }	
}

int Cf_NVCreate(char *Name, char *Val, int ValLen, PCf pCf)
{
  PSymbol pSym = NULL;
  PNVPair pNV = NULL;
  int Result = 0;

  /* don't bother with it if it doesn't hav a name */
  if ((Name == NULL) || (strlen(Name) == 0))
    return LIBNFR_ERR_SUCCESS;

  /* bug: should check this return value and decide whether to
   * overwrite an existing entry based on the Mode member of pCf */
  Result = SymTab_Look(Name, &pSym, pCf->SymTab);

  if((void *)pSym->Data != NULL)
    {
      if (((PNVPair)pSym->Data)->Val != NULL)
        SFree(((PNVPair)pSym->Data)->Val);
      SFree((void *)pSym->Data);
    }
  pSym->Data = (ULONG) SCalloc(1, sizeof(TNVPair));
  pNV = (PNVPair) pSym->Data;
  pNV->Name = pSym->String;
  pNV->NameLen = pSym->StrLen;
  pNV->ValLen = ValLen;
  pNV->Val = (char *) SCalloc(1, ValLen + 1);
  memcpy(pNV->Val, Val, ValLen);
  if( (*(pNV->Val) == '\'') || (*(pNV->Val) == '\"') )
    pNV->QChar = *(pNV->Val);
  return LIBNFR_ERR_SUCCESS;
}  

int Cf_NVSymPickle(PSymbol pNVSym, int Mode, void *f)
{
  PNVPair pNV = (PNVPair) pNVSym->Data;
  FILE *CfFile = (FILE *) f;
  int i = 0;

  fwrite(pNV->Name, pNV->NameLen, 1, CfFile);
  if(pNV->Val != NULL)
    {
      fputc('=', CfFile);
      fwrite(pNV->Val, pNV->ValLen, 1, CfFile);    
      fputc('\n', CfFile);
      return LIBNFR_ERR_SUCCESS;
    }

  /* we don't have the prepickled value so we'll have to pickle one */
  fwrite("=\"", 2, 1, CfFile);
  while(i < pNV->ExpValLen)
    {
      switch(*(pNV->ExpVal + i))
	{
	case '\n':
	  fwrite("\\n", 2, 1, CfFile);
	  break;
	case '\t':
	  fwrite("\\t", 2, 1, CfFile);
	  break;
	default:
	  fwrite((pNV->ExpVal + i), 1, 1, CfFile);
	  break;
	}
      i++;
    }
  fwrite("\"", 1, 1, CfFile);
  if (Mode & CF_FLAG_PUT_LF)
    fwrite("\n", 1, 1, CfFile);
  
  return LIBNFR_ERR_SUCCESS;
} 


