/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   fifo.c
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

#include "sys-include.h"
#include "opts.h"

#include "fifo.h"

#include "memsafe.h"

#include "libnfr_err.h"


int Fifo_Create(char *Name)
{
  assert(Name != NULL);

  if (mkfifo(Name, 0600) == 0)
    return LIBNFR_ERR_SUCCESS;

  /* bug: fix the error handling below */
  else
    {
      switch (errno)
	{
	case EEXIST:
	  return LIBNFR_ERR_UNDEFINED;
	default:
	  return LIBNFR_ERR_UNDEFINED;
	}
    }
}


int Fifo_OpenRead(char *Name)
{
  /* FIFO_OPEN_READMODE is defined in opts.h */
  return open(Name, FIFO_OPEN_READMODE | O_NDELAY);
}

int Fifo_OpenWrite(char *Name)
{
  /* FIFO_OPEN_WRITEMODE is defined in opts.h */
  return open(Name, FIFO_OPEN_WRITEMODE | O_NDELAY);
}

void Fifo_Destroy(char *Name)
{
  assert(Name != NULL);

  unlink(Name);
}

int Fifo_SendDGram(char *Fifo, int Type, void *Data, int DataLen, int Timeout)
{  
  BYTE Buf[DGRAM_MAXSIZE];
  int fdWrite;
  int Result;

  assert(Fifo != NULL);

  /* bug: prolly only really need to initialize the header portion of
     Buf[], since the data portion gets memcpy()'d to */
  memset(Buf, 0, sizeof(Buf));

  ((PDGram) Buf)->Type = Type;
  DGram_SetData(Data, DataLen, ((PDGram) Buf));

  fdWrite = open(Fifo, O_WRONLY | O_NDELAY);
  if(fdWrite < 0)
    {
    switch(errno)
      {
      case ENOENT:
	return LIBNFR_ERR_FNOEXIST;
      default:
	return LIBNFR_ERR_UNDEFINED;
      }
    }
  Result = DGram_Send_fd(fdWrite, ((PDGram) Buf), Timeout);
  close (fdWrite);
  return Result;
}




  
