/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   memsafe.c
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

/* testing cvs - hi */
/* testing fixlf again - hi */
/* testing fixlf yet again - hi */

#include "sys-include.h"

#include "memsafe.h"
#include "xtypes.h"

#ifdef MEM_DEBUG

#define MAX_PTRS 4096

int Memsafe_AllocCount = 0;
int Memsafe_AllocSerial = 0;
int Memsafe_AllocByteCount = 0;
ULONG Memsafe_AllocPtrs[MAX_PTRS];

void *SCalloc(size_t q, size_t s)
{
  static BOOL Initialized = FALSE;
  void *p = NULL;

  if(!Initialized)
    {
      memset(Memsafe_AllocPtrs, 0, sizeof(Memsafe_AllocPtrs));
      Initialized = TRUE;
    }

  if((q < 1) || (s < 1))
    fprintf(stderr,"woahhh, bad allocation size (q=%d, s=%d)!!\n", q, s);

  p = (void *) calloc(q,s);
  if (p == NULL)
  {
    fprintf(stderr,"woahhh, memory trouble!!\n");
    exit (0);
  }
  if(Memsafe_AllocSerial < MAX_PTRS)
    Memsafe_AllocPtrs[Memsafe_AllocSerial++] = (ULONG) p;
  Memsafe_AllocCount++;
  Memsafe_AllocByteCount += (q*s);
  return p;
}

void SFree(void *p)
{
  int i = 0;

  while(i < MAX_PTRS)
    {
      if (Memsafe_AllocPtrs[i] == (ULONG) p)
	{
	  Memsafe_AllocPtrs[i] = 0;
	  break;
	}
      i++;
    }
  free(p);
  Memsafe_AllocCount--;
}

void SExit(int ExitCode)
{
  int i = 0;


#ifdef MEM_REPORT
  fprintf(stderr, "%d bytes alloced in %d calloc()s\n", Memsafe_AllocByteCount,
          Memsafe_AllocSerial);
#endif

  if (Memsafe_AllocCount == 0)
    exit(ExitCode);
  if (Memsafe_AllocCount < 0)
  fprintf(stderr, "\nwhoahh there, slick, %d too many items freed!\n\n",
	  -Memsafe_AllocCount);
  if (Memsafe_AllocCount > 0)
    {
      fprintf(stderr, "\nwhoahh there, slick, %d items not freed!\n\n",
	      Memsafe_AllocCount);
      while(i < MAX_PTRS)
	{
	  if (Memsafe_AllocPtrs[i] != 0)
	    fprintf(stderr, "\nSCalloc() index %d may be a problem\n", i);
          i++;
	}
    }
  if (ExitCode == 0)
    exit(-1);
  exit(ExitCode);
}

#endif /* MEM_DEBUG */
