/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   rangeres.c
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

#include "sys-include.h"

#include "memsafe.h"
#include "rangeres.h"

#include "libnfr_err.h"


TRangeRes *RangeRes_New(void)
{
  return (TRangeRes *) SCalloc(1, sizeof(TRangeRes));
}

void RangeRes_Delete(TRangeRes *pRangeRes)
{
  ResId_Delete((PResId)pRangeRes);
}

ULONG RangeRes_WriteBody(PRangeRes pRangeRes, ULONG *pByteCounter, 
			 FILE *Out)
{
  ULONG Result = 0;
  ULONG NBOMin = 0;
  ULONG NBOMax = 0;
  ULONG NBOMemberType = 0;
  ULONG NBOHighNo = 0;
  /* always store numeric things as network byte order ulongs */

  NBOMin = htonl(pRangeRes->Min);
  Result = fwrite(&NBOMin, 1, sizeof(NBOMin), Out);
  if (Result != sizeof(NBOMin))
    return LIBNFR_ERR_UNDEFINED;
  if(pByteCounter != NULL)
    *pByteCounter += Result;

  NBOMax = htonl(pRangeRes->Max);
  Result = fwrite(&NBOMax, 1, sizeof(NBOMax), Out);
  if (Result != sizeof(NBOMax))
    return LIBNFR_ERR_UNDEFINED;
  if(pByteCounter != NULL)
    *pByteCounter += Result;

  NBOMemberType = htonl(pRangeRes->MemberType);
  Result = fwrite(&NBOMemberType, 1, sizeof(NBOMemberType), Out);
  if (Result != sizeof(NBOMemberType))
    return LIBNFR_ERR_UNDEFINED;
  if(pByteCounter != NULL)
    *pByteCounter += Result;

  NBOHighNo = htonl(pRangeRes->HighNo);
  Result = fwrite(&NBOHighNo, 1, sizeof(NBOHighNo), Out);
  if (Result != sizeof(NBOHighNo))
    return LIBNFR_ERR_UNDEFINED;
  if(pByteCounter != NULL)
    *pByteCounter += Result;

  return LIBNFR_ERR_SUCCESS;
}


ULONG RangeRes_ReadBody(PRangeRes pRangeRes, ULONG *pByteCounter, FILE *In)
{
  ULONG Result = 0;
  ULONG NBOMin = 0;
  ULONG NBOMax = 0;
  ULONG NBOMemberType = 0;
  ULONG NBOHighNo = 0;

  Result = fread(&NBOMin, 1, sizeof(NBOMin), In);
  if (Result != sizeof(NBOMin))
    return LIBNFR_ERR_UNDEFINED;
  if(pByteCounter != NULL)
    *pByteCounter += Result;
  pRangeRes->Min = (USHORT) ntohl(NBOMin);

  Result = fread(&NBOMax, 1, sizeof(NBOMax), In);
  if (Result != sizeof(NBOMax))
    return LIBNFR_ERR_UNDEFINED;
  if(pByteCounter != NULL)
    *pByteCounter += Result;
  pRangeRes->Max = (USHORT) ntohl(NBOMax);

  Result = fread(&NBOMemberType, 1, sizeof(NBOMemberType), In);
  if (Result != sizeof(NBOMemberType))
    return LIBNFR_ERR_UNDEFINED;
  if(pByteCounter != NULL)
    *pByteCounter += Result;
  pRangeRes->MemberType = (USHORT) ntohl(NBOMemberType);

  Result = fread(&NBOHighNo, 1, sizeof(NBOHighNo), In);
  if (Result != sizeof(NBOHighNo))
    return LIBNFR_ERR_UNDEFINED;
  if(pByteCounter != NULL)
    *pByteCounter += Result;
  pRangeRes->HighNo = (USHORT) ntohl(NBOHighNo);

  return LIBNFR_ERR_SUCCESS;
}
