/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   resid.c
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

#include "sys-include.h"

#include "memsafe.h"
#include "rescodes.h"
#include "resource.h"
#include "rangeres.h"
#include "alertres.h"

#include "libnfr_err.h"

TResId *ResId_New(void)
{
  return (TResId *) SCalloc(1, sizeof(TResId));
}

void ResId_Delete(TResId *pResId)
{
  if(pResId->Id != NULL)
    SFree(pResId->Id);
  if(pResId->DispName != NULL)
    SFree(pResId->DispName);
  SFree(pResId);
}

int ResId_SetId(char *Id, USHORT Size, PResId pResId)
{
  if (pResId->Id != NULL)
    SFree(pResId->Id);
  pResId->Id = SCalloc(1, Size + 1);
  memcpy(pResId->Id, Id, Size + 1);
  pResId->IdLen = Size;
  return LIBNFR_ERR_SUCCESS;
}

int ResId_SetDispName(char *Fac, USHORT Size, PResId pResId)
{
  if (pResId->DispName != NULL)
    SFree(pResId->DispName);
  pResId->DispName = SCalloc(1, Size + 1);
  memcpy(pResId->DispName, Fac, Size + 1);
  pResId->DispNameLen = Size;
  return LIBNFR_ERR_SUCCESS;
}

ULONG ResId_Read(TResId *pResId, ULONG *pByteCounter, FILE *In)
{
  ULONG Result = 0;
  ULONG NBOResCode = 0;
  ULONG NBOIdLen = 0;
  ULONG NBODispNameLen = 0;
  ULONG NBOState = 0;
  /* always store numeric things as network byte order ulongs */

  Result = fread(&NBOResCode, 1, sizeof(NBOResCode), In);
  if (Result != sizeof(NBOResCode))
    return LIBNFR_ERR_UNDEFINED;
  if(pByteCounter != NULL)
    *pByteCounter += Result;
  pResId->ResCode = ntohl(NBOResCode);

  Result = fread(&NBOIdLen, 1, sizeof(NBOIdLen), In);
  if (Result != sizeof(NBOIdLen))
    return LIBNFR_ERR_UNDEFINED;
  if(pByteCounter != NULL)
    *pByteCounter += Result;
  pResId->IdLen = ntohl(NBOIdLen);

  pResId->Id = (char *) SCalloc(1, pResId->IdLen + 1);
  Result = fread(pResId->Id, 1, pResId->IdLen, In);
  if (Result != pResId->IdLen)
    return LIBNFR_ERR_UNDEFINED;
  if(pByteCounter != NULL)
    *pByteCounter += Result;

  Result = fread(&NBODispNameLen, 1, sizeof(NBODispNameLen), In);
  if (Result != sizeof(NBODispNameLen))
    return LIBNFR_ERR_UNDEFINED;
  if(pByteCounter != NULL)
    *pByteCounter += Result;
  pResId->DispNameLen = (USHORT) ntohl(NBODispNameLen);

  pResId->DispName = (char *) SCalloc(1, pResId->DispNameLen + 1);
  Result = fread(pResId->DispName, 1, pResId->DispNameLen, In);
  if (Result != pResId->DispNameLen)
    return LIBNFR_ERR_UNDEFINED;
  if(pByteCounter != NULL)
    *pByteCounter += Result;

  Result = fread(&NBOState, 1, sizeof(NBOState), In);
  if (Result != sizeof(NBOState))
    return LIBNFR_ERR_UNDEFINED;
  if(pByteCounter != NULL)
    *pByteCounter += Result;
  pResId->State = (char) ntohl(NBOState);

  return LIBNFR_ERR_SUCCESS;
}

ULONG ResId_Write(PResId pResId, ULONG *pByteCounter, FILE *Out)
{
  ULONG Result = 0;
  ULONG NBOIdLen = 0;
  ULONG NBOResCode = 0;
  ULONG NBODispNameLen = 0;
  ULONG NBOState = 0;
  /* always store numeric things as network byte order ulongs */

  NBOResCode = htonl(pResId->ResCode);
  Result = fwrite(&NBOResCode, 1, sizeof(NBOResCode), Out);
  if (Result != sizeof(NBOResCode))
    return LIBNFR_ERR_UNDEFINED;
  if(pByteCounter != NULL)
    *pByteCounter += Result;
  
  NBOIdLen = htonl(pResId->IdLen);
  Result = fwrite(&NBOIdLen, 1, sizeof(NBOIdLen), Out);
  if (Result != sizeof(NBOIdLen))
    return LIBNFR_ERR_UNDEFINED;
  if(pByteCounter != NULL)
    *pByteCounter += Result;
  
  Result = fwrite(pResId->Id, 1, pResId->IdLen, Out);
  if (Result != pResId->IdLen)
    return LIBNFR_ERR_UNDEFINED;
  if(pByteCounter != NULL)
    *pByteCounter += Result;
  
  NBODispNameLen = htonl(pResId->DispNameLen);
  Result = fwrite(&NBODispNameLen, 1, sizeof(NBODispNameLen), Out);
  if (Result != sizeof(NBODispNameLen))
    return LIBNFR_ERR_UNDEFINED;
  if(pByteCounter != NULL)
    *pByteCounter += Result;

  Result = fwrite(pResId->DispName, 1, pResId->DispNameLen, Out);
  if (Result != pResId->DispNameLen)
    return LIBNFR_ERR_UNDEFINED;
  if(pByteCounter != NULL)
    *pByteCounter += Result;

  NBOState = htonl(pResId->State);
  Result = fwrite(&NBOState, 1, sizeof(NBOState), Out);
  if (Result != sizeof(NBOState))
    return LIBNFR_ERR_UNDEFINED;
  if(pByteCounter != NULL)
    *pByteCounter += Result;
  
  return LIBNFR_ERR_SUCCESS;
}
