/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   utils.c
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

#include "sys-include.h"

#include "libnfr_err.h"

#include "utils.h"
#include "memsafe.h"

ULONG ConsoleMsg(char *FmtString, ...)
{
  FILE *Out = NULL;
  va_list ap = NULL;
  ULONG Result = LIBNFR_ERR_SUCCESS;

  Out = fopen("/dev/console","w");
  if(Out)
    {
      va_start(ap, FmtString);
      fputs("\a\r\n", Out);
      vfprintf(Out, FmtString, ap);
      fputs("\r\n", Out);
      va_end(ap);
      fclose(Out);
    }
  else
    {
      Result = LIBNFR_ERR_UNDEFINED;
      
      Out = stderr;
      if(Out)
	{
	  va_start(ap, FmtString);
	  vfprintf(Out, FmtString, ap);
	  va_end(ap);
	}
      else
	Result = LIBNFR_ERR_UNDEFINED;
    }

  return Result;
}

ULONG StrToSecs(char *String)
{
  char *End;
  ULONG Seconds;

  Seconds = strtoul(String, &End, 0);
  switch(*End)
    {
    case 'm':
    case 'M':
      Seconds *= 60;
      break;
    case 'h':
    case 'H':
      Seconds *= 3600;
      break;
    case 'd':
    case 'D':
      Seconds *= 86400;
      break;
    }
  return Seconds;
}

long StrToSSecs(char *String)
{
  char *End;
  long Seconds;

  Seconds = strtol(String, &End, 0);
  switch(*End)
    {
    case 'm':
    case 'M':
      Seconds *= 60;
      break;
    case 'h':
    case 'H':
      Seconds *= 3600;
      break;
    case 'd':
    case 'D':
      Seconds *= 86400;
      break;
    }
  return Seconds;
}
