#define PORT_TEST

/*
        Copyright(C) 1997 Network Flight Recorder, Inc.  
        All rights reserved.

        Use and distribution of this software and its source code
        are governed by the terms and conditions of the
        Network Flight Recorder Software License ("LICENSE.TXT")

By Mark Sienkiewicz / NFR

*/

#include "sys-include.h"

#include "xtypes.h"
#include "counter.h"
#include "vsch.h"

/*
* bug: this didn't really turn out the way I hoped.  It is kind of
* a half step between a historical implementation that needs to be
* phased out and a better solution for the future.
*/

/*
* bug: you will find that various places in this code assume all sorts
* of non-portable things about the architecture we are running on.
* much of that should be fixed in due course.
*/


int vsch_print_type(char *b, void *data, int type, int size)
{
switch (type)
        {
case VSCH_TIME_T:
	{
	char *s = ctime(data);	/* bug: locale! */
	strcpy(b,s);
	s=strchr(b,'\n');
	if (s) *s=0;
	}
	return 0;
case VSCH_INT:
	{
	int i;
	memcpy(&i,data,sizeof(int));
        sprintf(b,"%d",i);
	}
        return 0;
case VSCH_IP:
        {
        unsigned char *s;
        s=(unsigned char *)data;
        sprintf(b,"%d.%d.%d.%d",s[0],s[1],s[2],s[3]);
        }
        return 0;
case VSCH_COUNTER:
	{
	counter c;
	memcpy(&c,data,sizeof(counter));
	counter_itoa(b,c);
	}
	return 0;
case VSCH_STR:
	if (size > 999)
		size = 999;
	memcpy(b,data,size);
	b[size]=0;
	return 0;
case VSCH_ETHERMAC:
	{
        unsigned char *s;
        s=(unsigned char *)data;
        sprintf(b,"%02x:%02x:%02x:%02x:%02x:%02x",
		s[0],s[1],s[2],s[3],s[4],s[5]);
        }
	return 0;
        }
abort();/* bug*/
return 0;
}

vsch_size(int type)
{
switch (type)
	{
case VSCH_TIME_T:
case VSCH_INT:
	return sizeof(int);
case VSCH_COUNTER:
	return sizeof(counter);
case VSCH_IP:
	return 4;
case VSCH_STR:
	return sizeof(char *);
case VSCH_BLOB:
	return sizeof(char *);
case VSCH_ETHERMAC:
	return 6;
	}
abort(); /* bug */
}


#if 0
vsch_put_time_t(FILE *f, time_t x)
{
fwrite(&x,sizeof(x),1,f);
}

vsch_put_int(FILE *f, int x)
{
fwrite(&x,sizeof(x),1,f);
}

vsch_put_counter(FILE *f, counter x)
{
fwrite(&x,sizeof(x),1,f);
}

vsch_put_ip(FILE *f, void *x)
{
fwrite(x,4,1,f);
}

vsch_put_str(FILE *f, char *s)
{
int a=strlen(s);
fwrite(&a,sizeof(int),1,f);
fwrite(s,a,1,f);
}


int vsch_get_time_t(FILE *f, int endian, time_t *x)
{
if (fread((void *)x,4,1,f) != 1)
	return 1;
return 0;
}

int vsch_get_int(FILE *f, int endian, int *x)
{
if (fread((void *)x,4,1,f) != 1)
	return 1;
return 0;
}

int vsch_get_counter(FILE *f, int endian, counter *x)
{
if (fread((void *)x,8,1,f) != 1)
	return 1;
return 0;
}

int vsch_get_str(FILE *f, int endian, char **x)
{
int a;
char *s;
if (fread(&a,sizeof(int),1,f) != 1)
	return 1;
*x = malloc(a+1);
(*x)[a]='\0';
if (fread(*x,1,a,f) != a)
	{
	free(*x);
	*x = (char *)0;
	return 1;
	}
return 0;
}


vsch_get_ip(FILE *f, int endian, void *x)
{
if (fread(x,4,1,f) != 1)
	return 1;
return 0;
}

#endif /* 0 */

enum vsch_type
vsch_string_to_type(char *s)
{
static struct
	{
	char *name;
	enum vsch_type type;
	} xl[] = 
	{
	{ "time",	VSCH_TIME_T },
	{ "int",	VSCH_INT },
	{ "counter",	VSCH_COUNTER },
	{ "ip",		VSCH_IP },
	{ "str",	VSCH_STR },	/* deprecated */
	{ "string",	VSCH_STR },
	{ "blob",	VSCH_BLOB },
	{ "ethermac",	VSCH_ETHERMAC },
	{ 0,0 }
	};
int x;
for (x=0; xl[x].name; x++)
	if (strcmp(xl[x].name,s) == 0)
		return xl[x].type;
return VSCH_ERROR;
}

int vsch_putbuf(struct vsch_buf *v, enum vsch_type type, void *data, int size)
{
switch (type)
	{
case VSCH_TIME_T:
case VSCH_INT:
#ifdef PORT_TEST
	if (sizeof(int) != 4)
		abort();
#endif
	if ( (int) ((BYTE *)v->buffer_size - v->buffer_offset) < 4)
		if (vsch_buffer_grow(v,4)) return 1;
	memcpy((BYTE *)v->buffer+v->buffer_offset,data,4);
	v->buffer_offset += 4;
	return 0;

case VSCH_COUNTER:
#ifdef PORT_TEST
	if (sizeof(counter) != 8)
		abort();
#endif
	if (v->buffer_size - v->buffer_offset < 8)
		if (vsch_buffer_grow(v,8)) return 1;
	memcpy((BYTE *)v->buffer+v->buffer_offset,data,8);
	v->buffer_offset += 8;
	return 0;

case VSCH_IP:
	if (v->buffer_size - v->buffer_offset < 4)
		if (vsch_buffer_grow(v,4)) return 1;
	memcpy((BYTE *)v->buffer+v->buffer_offset,data,4);
	v->buffer_offset += 4;
	return 0;

case VSCH_STR:
case VSCH_BLOB:
	if (v->buffer_size - v->buffer_offset < 4+size)
		if (vsch_buffer_grow(v,4+size)) return 1;
	memcpy((BYTE *)v->buffer+v->buffer_offset,&size,4);
	memcpy((BYTE *)v->buffer+v->buffer_offset+4,data,size);
	v->buffer_offset += 4+size;
	return 0;

case VSCH_ETHERMAC:
	if (v->buffer_size - v->buffer_offset < 6)
		if (vsch_buffer_grow(v,6)) return 1;
	memcpy((BYTE *)v->buffer+v->buffer_offset,data,6);
	v->buffer_offset += 6;
	return 0;
	}
abort();
}

vsch_buffer_grow( struct vsch_buf *v, int min_growth )
{
int add;
void *tmp;
add = v->buffer_size;
if (add > 1000)
        add = 1000;
if (min_growth > add)
        add = min_growth;


tmp=realloc(v->buffer,v->buffer_size + add);

if (! tmp)
        return 1;
v->buffer=tmp;
v->buffer_size += add;
return 0;
}

void vsch_initbuf(struct vsch_buf *v, int size)
{
v->buffer = NULL;
if (size > 0)
	v->buffer=malloc(size);
else
	v->buffer=(void *)0;
if (v->buffer)
	v->buffer_size=size;
else
	v->buffer_size=0;
v->buffer_offset=0;
}

/*
* the intent is that vsch_getbuf will only be used for 
* types that have a fixed size.  In practice, it only really gets
* used for VSCH_INT and VSCH_COUNTER
*/
int vsch_getbuf(struct vsch_buf *v, enum vsch_type type, void *data,
	int *size)
{
switch (type)
	{
case VSCH_TIME_T:
case VSCH_INT:
#ifdef PORT_TEST
	if (sizeof(int) != 4)
		abort();
#endif
	if (v->buffer_offset - v->read_point < 4)
		return -1;
	memcpy(data,(BYTE *)v->buffer+v->read_point,4);
	v->read_point += 4;
	return 0;

case VSCH_COUNTER:
#ifdef PORT_TEST
	if (sizeof(counter) != 8)
		abort();
#endif
	if (v->buffer_offset - v->read_point < 8)
		return -1;
	memcpy(data,(BYTE *)v->buffer+v->read_point,8);
	v->read_point += 8;
	return 0;

case VSCH_IP:
	if (v->buffer_offset - v->read_point < 4)
		return -1;
	memcpy(data,(BYTE *)v->buffer+v->read_point,4);
	v->read_point += 4;
	return 0;

case VSCH_ETHERMAC:
	if (v->buffer_offset - v->read_point < 6)
		return -1;
	memcpy(data,(BYTE *)v->buffer+v->read_point,6);
	v->read_point += 6;
	return 0;

default:
	abort();
	return -1;
	}
}

int vsch_ptrbuf(struct vsch_buf *v, enum vsch_type type, int *size)
{
int rval;

#define NEED(XX) if (v->buffer_offset - v->read_point < XX) \
	{ v->read_point = v->buffer_offset; return -1; }

#define RET(XX) rval = v->read_point; v->read_point += XX ; \
	return rval;

switch (type)
	{
case VSCH_TIME_T:
case VSCH_INT:
#ifdef PORT_TEST
	if (sizeof(int) != 4)
		abort();
#endif
	NEED(4)
	RET(4)

case VSCH_COUNTER:
#ifdef PORT_TEST
	if (sizeof(counter) != 8)
		abort();
#endif
	NEED(8)
	RET(8)

case VSCH_IP:
	NEED(4)
	RET(4)

case VSCH_STR:
case VSCH_BLOB:
	{
	int size_tmp;
	NEED(4)
	memcpy(&size_tmp,(BYTE *)v->buffer+v->read_point,4);
	if (size)
		*size = size_tmp;
	v->read_point += 4;

	NEED(size_tmp)
	RET(size_tmp)
	}

case VSCH_ETHERMAC:
	NEED(6)
	RET(6)

	}
abort();
}
