/* parse.c */
/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   create.c
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

#include "sys-include.h"

/* #define DEBUG_LEV 1 */
#include "debug.h"

#include "libnfr.h"
#include "rc.h"

#include "ytab.h"


int Rc_CreateAlertRes(PSymbol pSym, PNIList pFacList, 
		      char *SevId, char *Text)
{
  PAlertRes pNewAlertRes = NULL;
  FacCode FacCodes[FAC_CODES_MAXSIZE];
  PSymbol pSymbol = NULL;
  char *FacString;
  USHORT NumFacs;
  int Result = 0;
  int facCode = 0;
  PResId rid = NULL;
  
  if(pSym->Type != RES_ALERT)
    {
      Rc_Error("symbol \"%s\" redefined as type RES_ALERT", 
	       pSymbol->String);
      return NFR_ERR_SUCCESS;
    }
    
  pNewAlertRes = AlertRes_New();

  pNewAlertRes->ResCode = pSym->Data;

  pSym->Data = (ULONG) pNewAlertRes;

  AlertRes_SetId(pSym->String, pSym->StrLen, pNewAlertRes);
  
  /* bug: this is the second malloc done for this string */
  AlertRes_SetText(Text, strlen(Text), pNewAlertRes);
  
  Result = SymTab_Look(SevId, &pSymbol, gpResTab->SymTab);

  if ( (Result == SYM_FOUND) && (pSymbol->Type == ID_SEV) )
    {
      pNewAlertRes->Severity = (BYTE) pSymbol->Data;
    }
  else if (Result == SYM_FOUND)
    {
      Rc_Error("symbol \"%s\" redefined as type ID_SEV", pSymbol->String);
      return RC_ERR_UNDEFINED;
    }
  else 
    {
      Rc_Error("undefined ID_SEV symbol \"%s\"", pSymbol->String);
      return RC_ERR_UNDEFINED;
    }


  NumFacs = pFacList->Count;
  while (pFacList->Count)
    {
      /* NextString decrements the Count member of the list */
      FacString = NextString(pFacList);

      /* we always want to use the difftab if it exists, since it's
	 existance indicates that the facility mappings may be off
	 in the table we're currently buiding */
      if(gpDiffTab)
	Result = SymTab_Peek(FacString, &pSymbol, gpDiffTab->SymTab);
      else
	Result = SymTab_Peek(FacString, &pSymbol, gpResTab->SymTab);

      if (Result != SYM_FOUND)
	{
	  /* check for the special case that it's one that's been deleted 
	     from the gui, and if it has than just let it go and 
	     it'll get sorted out in the merge */
	  if(gpDiffTab)
	    {
	      Result = SymTab_Peek(FacString, &pSymbol, gpDiffTab->SymTab);
	      if(Result == SYM_FOUND)
		{
		  if(((PResId)pSymbol->Data)->State == RESID_STATE_DELETED)
		    NumFacs--;
		}
	      else
		{
		  Rc_Error("undefined ID_FAC symbol \"%s\"", FacString);
		  return RC_ERR_UNDEFINED;
		}
	    }
	  else
	    {
	      Rc_Error("undefined ID_FAC symbol \"%s\"", FacString);
	      return RC_ERR_UNDEFINED;
	    }
	}
      if (pSymbol->Type != ID_FAC)
	{
	  Rc_Error("symbol \"%s\" redefined as type ID_FAC", FacString);
	  return RC_ERR_UNDEFINED;
	}
      facCode = ResCode_GetResNo(((PResId)pSymbol->Data)->ResCode);
      FacCodes[pFacList->Count] = (BYTE) facCode;
    }

  AlertRes_SetFacCodes(FacCodes, NumFacs, pNewAlertRes);
  return NFR_ERR_SUCCESS;
}

int Rc_CreateAlertSrc(PSymbol pSym, char *Name, char *Desc)
{
  PAlertSrc pNewAlertSrc = NULL;
  int Result = 0;
  
  if(pSym->Type != RES_ALERT_SRC)
    {
      Rc_Error("symbol \"%s\" redefined as type RES_ALERT", 
	       pSym->String);
      return NFR_ERR_SUCCESS;
    }
    
  pNewAlertSrc = AlertSrc_New();
  pNewAlertSrc->ResCode = pSym->Data;

  /* bug: this is the second malloc done for these strings */
  AlertSrc_SetId(pSym->String, pSym->StrLen, pNewAlertSrc);
  AlertSrc_SetName(Name, strlen(Name), pNewAlertSrc);
  AlertSrc_SetDesc(Desc, strlen(Desc), pNewAlertSrc);

  pSym->Data = (ULONG) pNewAlertSrc;
  return NFR_ERR_SUCCESS;
}

int Rc_CreateStringRes(PSymbol pSym, char *String)
{
  PStringRes pNewStringRes = NULL;
  int Result = 0;
  
  if(pSym->Type != RES_STRING)
    {
      Rc_Error("symbol \"%s\" redefined as type RES_ALERT", 
	       pSym->String);
      return NFR_ERR_SUCCESS;
    }

  pNewStringRes = StringRes_New();
  pNewStringRes->ResCode = pSym->Data;

  /* bug: this is the second malloc done for these strings */
  StringRes_SetId(pSym->String, pSym->StrLen, pNewStringRes);
  StringRes_SetString(String, strlen(String), pNewStringRes);

  pSym->Data = (ULONG) pNewStringRes;
  return NFR_ERR_SUCCESS;
}

int Rc_CreateFacRes(PSymbol pSym, char *DispName)
{
  PResId pNewFacRes = NULL;
  int Result = 0;
  
  if(pSym->Type != ID_FAC)
    {
      Rc_Error("symbol \"%s\" redefined as type ID_FAC", 
	       pSym->String);
      return NFR_ERR_SUCCESS;
    }

  pNewFacRes = ResId_New();
  pNewFacRes->ResCode = pSym->Data;

  /* bug: this is the second malloc done for these strings */
  ResId_SetId(pSym->String, pSym->StrLen, pNewFacRes);
  ResId_SetDispName(DispName, strlen(DispName), pNewFacRes);

  pSym->Data = (ULONG) pNewFacRes;
  return NFR_ERR_SUCCESS;
}

int Rc_SetId(char *IdString, ULONG ResCode)
{ 
  int Result;
  PSymbol pSymbol;

  Result = SymTab_Look(IdString, &pSymbol, gpResTab->SymTab);
  if (Result != SYM_FOUND)
    {
      pSymbol->Type = ResCode_GetResType(ResCode);
      pSymbol->Data = ResCode;
    }
  else
    Rc_Error("symbol \"%s\" redefined", pSymbol->String);
  return NFR_ERR_SUCCESS;
}

int Rc_AssignIdInRange(char *IdString, ULONG Type)
{ 
  int Result;
  PSymbol pSymbol = NULL;
  PSymbol pPresetSym = NULL;
  TResId ResId;

  memset (&ResId, 0, sizeof(TResId));
  ResCode_SetResType(ResId.ResCode, Type);
  Result = SymTab_Look(IdString, &pSymbol, gpResTab->SymTab);
  if (Result == SYM_FOUND)
    {
      Rc_Error("\"%s\" redefined", IdString);
    }
  else
    {
      pSymbol->Type = Type;

      Result = ResTab_AssignResNo(0, gCurRangeId, &ResId, gpResTab);

      if (Result != NFR_ERR_SUCCESS)
	return Result;

      pSymbol->Data = ResId.ResCode;
    }
  return NFR_ERR_SUCCESS;
}

