/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   iterators.c
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

#include "sys-include.h"

/* #define DEBUG_LEV 1 */
#include "debug.h"

#include "libnfr.h"
#include "rc.h"

#include "ytab.h"

/******************************************************************************
 * Iterative functions for use with ForEachSymbol()
 *****************************************************************************/

int PrintAlertResFields(PSymbol pSymbol, int i, void *p)
{
  DBG_OUT("\nAlertId=%s\nResNo=%ld\nAlertText=%s\nAlertTextLen=%d\n",
          pSymbol->String,
          ResCode_GetResNo( ((PAlertRes)pSymbol->Data)->ResCode),
          ((PAlertRes)pSymbol->Data)->Text,
          ((PAlertRes)pSymbol->Data)->TextLen);
  return NFR_ERR_SUCCESS;
}

int PrintAlertSrcFields(PSymbol pSymbol, int i, void *p)
{
  DBG_OUT("\nAlertSrcId=%s\nResNo=%ld\nName=%s\nDesc=%s\n",
          pSymbol->String,
          ResCode_GetResNo( ((PAlertSrc)pSymbol->Data)->ResCode),
          ((PAlertSrc)pSymbol->Data)->Name,
          ((PAlertSrc)pSymbol->Data)->Desc);
  return NFR_ERR_SUCCESS;
}

int PrintStringResFields(PSymbol pSymbol, int i, void *p)
{
  DBG_OUT("\nStringResId=%s\nResNo=%ld\nString=%s\n",
          pSymbol->String,
          ResCode_GetResNo( ((PStringRes)pSymbol->Data)->ResCode),
          ((PStringRes)pSymbol->Data)->String);
  return NFR_ERR_SUCCESS;
}

int PrintFacFields(PSymbol pSymbol, int i, void *p)
{
  DBG_OUT("\nFacId=%s\nFacNo=0x%.8lX\n",
          pSymbol->String, pSymbol->Data);
  return NFR_ERR_SUCCESS;
}

int PrintFacNVPair(PSymbol pSymbol, int i, void *File)
{
  fprintf((FILE *) File, "%s = %ld\n",
          pSymbol->String, 
	  ResCode_GetResNo( ( (PResId)pSymbol->Data )->ResCode) );
  return NFR_ERR_SUCCESS;
}

int PrintSevFields(PSymbol pSymbol, int i, void *p)
{
  DBG_OUT("\nSevId=%s\nSevNo=0x%.8lX\n",
          pSymbol->String, pSymbol->Data);
  return NFR_ERR_SUCCESS;
}

int PrintGenDefine(PSymbol pSymbol, int i, void *File)
{
  fprintf( (FILE *) File, "#define %-30s0x%.8lX\n",
           pSymbol->String, pSymbol->Data);
  return NFR_ERR_SUCCESS;
}

int PrintResDefine(PSymbol pSymbol, int i, void *File)
{
  fprintf( (FILE *) File, "#define %-30s0x%.8lX\n",
           pSymbol->String, ((PResId)pSymbol->Data)->ResCode);
  return NFR_ERR_SUCCESS;
}

int PrintMapEntry(PSymbol pSymbol, int i, void *File)
{
  fprintf( (FILE *) File, "%s=%ld\n",
           pSymbol->String, ((PResId)pSymbol->Data)->ResCode);
  return NFR_ERR_SUCCESS;
}

int AlertRes_DeleteSym(PSymbol pSymbol, int iArg, void *pArg)
{
  AlertRes_Delete((PAlertRes)pSymbol->Data);
  return NFR_ERR_SUCCESS;
}

int AlertSrc_DeleteSym(PSymbol pSymbol, int iArg, void *pArg)
{
  AlertSrc_Delete((PAlertSrc)pSymbol->Data);
  return NFR_ERR_SUCCESS;
}

int StringRes_DeleteSym(PSymbol pSymbol, int iArg, void *pArg)
{
  StringRes_Delete((PStringRes)pSymbol->Data);
  return NFR_ERR_SUCCESS;
}

int RangeRes_DeleteSym(PSymbol pSymbol, int iArg, void *pArg)
{
  RangeRes_Delete((PRangeRes)pSymbol->Data);
  return NFR_ERR_SUCCESS;
}

int ResId_DeleteSym(PSymbol pSymbol, int iArg, void *pArg)
{
  if((void *)pSymbol->Data != NULL)
    ResId_Delete((PResId)pSymbol->Data);
  pSymbol->Data = 0;
  return NFR_ERR_SUCCESS;
}

int ResSym_Write(PSymbol pSymbol, int iArg, void *pArg)
{
  ResFile_WriteRes( (PResId)pSymbol->Data, (PResFile)pArg);
  return NFR_ERR_SUCCESS;
}

int AddResToArray(PSymbol pSym, int ArraySize, void *pArray)
{
  if( ResCode_GetResNo(((PResId)pSym->Data)->ResCode) >= ArraySize)
    return RC_ERR_UNDEFINED;
  ( (PSymbol *) pArray)[ResCode_GetResNo(((PResId)pSym->Data)->ResCode)] = 
    pSym;
}

int AddSymToArray(PSymbol pSym, int ArraySize, void *pArray)
{
  if(pSym->Data >= ArraySize)
    return RC_ERR_UNDEFINED;
  ( (PSymbol *) pArray)[pSym->Data] = pSym;
}

int TryAddIdToRange(PSymbol pSym, int i, void *pArg)
{
  PRangeRes pRange = (PRangeRes)pArg;
  PResId pResId = (PResId) pSym->Data;

  if((pRange->MemberType == ID_FAC) &&
     (pResId->State == RESID_STATE_ADDED))
    {
      /* there's only one fac range, so if this is a new fac, that's where
	 it goes */

      Rc_MergeIdFac(pResId->Id);
    }
  else if((ResCode_GetResNo(pResId->ResCode) == pRange->Min) &&
     (pResId->State == RESID_STATE_ADDED))
    {
      /* there are arbitrarily many of the other types of ranges, so we
	 have to check the ResNO.  The GUI sets the ResNo to the min for
	 the desired range for these resources */

      switch(pRange->MemberType)
	{
	case RES_ALERT:
	  Rc_MergeAlertRes(pResId->Id);
	  break;

	case RES_ALERT_SRC:
	  Rc_MergeAlertSrc(pResId->Id);
	  break;

	case RES_STRING:
	  Rc_MergeStringRes(pResId->Id);
	  break;

	default:
	  break;
	}
    }

  return NFR_ERR_SUCCESS;
}

