/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   merge.c
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

#include "sys-include.h"

/* #define DEBUG_LEV 1 */
#include "debug.h"

#include "libnfr.h"
#include "rc.h"

#include "ytab.h"


ULONG Rc_MergeFacIdRangeOpen(char *RangeType)
{
  ULONG Result = SYM_NOT_FOUND;
  PSymbol pSym = NULL;
  PRangeRes pRangeRes = NULL;
  
  Result = SymTab_Peek(gCurRangeId, &pSym, gpDiffTab->SymTab);

  if(Result == SYM_NOT_FOUND)
    Result = SymTab_Peek(gCurRangeId, &pSym, gpResTab->SymTab);
  
  pRangeRes = (PRangeRes) pSym->Data;

  if(pRangeRes->State == RESID_STATE_DELETED)
    return NFR_ERR_SUCCESS;

  fprintf(gMergeTarget,"\n%s\n{\n", 
	  RangeType);
  
}

ULONG Rc_MergeIdRangeClose()
{
  ULONG Result = SYM_NOT_FOUND;
  PSymbol pSym = NULL;
  PRangeRes pRangeRes = NULL;
  
  Result = SymTab_Peek(gCurRangeId, &pSym, gpDiffTab->SymTab); 

  if(Result == SYM_NOT_FOUND)
    Result = SymTab_Peek(gCurRangeId, &pSym, gpResTab->SymTab);
  
  pRangeRes = (PRangeRes) pSym->Data;

  if(pRangeRes->State == RESID_STATE_DELETED)
    return NFR_ERR_SUCCESS;

  /* check for any new members which have been added to this range */
  SymTab_ForEach(pRangeRes->MemberType, TryAddIdToRange, 0, pRangeRes, 
		 gpDiffTab->SymTab);


  fprintf(gMergeTarget, "\n}\n");
  return NFR_ERR_SUCCESS;
}

ULONG Rc_MergeIdRangeOpen(char *RangeType)
{
  ULONG Result = SYM_NOT_FOUND;
  PSymbol pSym = NULL;
  PRangeRes pRangeRes = NULL;
  
  Result = SymTab_Peek(gCurRangeId, &pSym, gpDiffTab->SymTab);

  if(Result == SYM_NOT_FOUND)
    Result = SymTab_Peek(gCurRangeId, &pSym, gpResTab->SymTab);
  
  pRangeRes = (PRangeRes) pSym->Data;

  if(pRangeRes->State == RESID_STATE_DELETED)
    return NFR_ERR_SUCCESS;

  fprintf(gMergeTarget,"\n%s %s %d - %d \"%s\"\n{\n", 
	  RangeType,
	  pRangeRes->Id,
	  pRangeRes->Min,
	  pRangeRes->Max,
	  pRangeRes->DispName);
  
}

ULONG Rc_MergeIdFac(char *FacId)
{
  ULONG Result = SYM_NOT_FOUND;
  PSymbol pSym = NULL;
  PResId pResId = NULL;
  
  Result = SymTab_Peek(FacId, &pSym, gpDiffTab->SymTab);

  if(Result == SYM_NOT_FOUND)
    Result = SymTab_Peek(FacId, &pSym, gpResTab->SymTab);
  
  pResId = (PResId) pSym->Data;

  if(pResId->State == RESID_STATE_DELETED)
    return NFR_ERR_SUCCESS;

  fprintf(gMergeTarget,"\n\t%s \"%s\";\n", 
	  pResId->Id, pResId->DispName);

  pResId->State = RESID_STATE_STABLE;

  return NFR_ERR_SUCCESS;
}

ULONG Rc_MergeStringRes(char *StringResId)
{
  ULONG Result = SYM_NOT_FOUND;
  PSymbol pSym = NULL;
  PStringRes pStringRes = NULL;
  
  Result = SymTab_Peek(StringResId, &pSym, gpDiffTab->SymTab);

  if(Result == SYM_NOT_FOUND)
    Result = SymTab_Peek(StringResId, &pSym, gpResTab->SymTab);
  
  pStringRes = (PStringRes) pSym->Data;

  if(pStringRes->State == RESID_STATE_DELETED)
    return NFR_ERR_SUCCESS;

  fprintf(gMergeTarget,"\n\t%s \"%s\";\n", 
	  pStringRes->Id, pStringRes->String);

  pStringRes->State = RESID_STATE_STABLE;

  return NFR_ERR_SUCCESS;
}

ULONG Rc_MergeAlertSrc(char *AlertSrcId)
{
  ULONG Result = SYM_NOT_FOUND;
  PSymbol pSym = NULL;
  PAlertSrc pAlertSrc = NULL;
  
  Result = SymTab_Peek(AlertSrcId, &pSym, gpDiffTab->SymTab);

  if(Result == SYM_NOT_FOUND)
    Result = SymTab_Peek(AlertSrcId, &pSym, gpResTab->SymTab);
  
  pAlertSrc = (PAlertSrc) pSym->Data;

  if(pAlertSrc->State == RESID_STATE_DELETED)
    return NFR_ERR_SUCCESS;

  fprintf(gMergeTarget,"\n\t%s\n\t{\n\t\t\"%s\";\n\t\t\"%s\";\n\t}\n", 
	  pAlertSrc->Id, pAlertSrc->Name, pAlertSrc->Desc);
  
  pAlertSrc->State = RESID_STATE_STABLE;

  return NFR_ERR_SUCCESS;
}

ULONG Rc_MergeAlertRes(char *AlertId)
{
  PSymbol pSym = NULL;
  PSymbol pFacSym = NULL;
  PAlertRes pAlertRes = NULL;
  ULONG Result = SYM_NOT_FOUND;
  int i = 0, j = 0;

  Result = SymTab_Peek(AlertId, &pSym, gpDiffTab->SymTab);

  if(Result == SYM_NOT_FOUND)
    Result = SymTab_Peek(AlertId, &pSym, gpResTab->SymTab);

  pAlertRes = (PAlertRes) pSym->Data;

  if(pAlertRes->State == RESID_STATE_DELETED)
    return NFR_ERR_SUCCESS;

  fprintf(gMergeTarget,"\n\t%s\n\t{\n\t\t", pAlertRes->Id);

  for(i=0; i < pAlertRes->FacCodeCount; i++)
    {
      if (i > 0)
	fputc(' ', gMergeTarget);
      j = pAlertRes->FacCodes[i];

      /* we need to see if the facility is still valid  - if it's
	 not we don't write it to the new .src file */
      if(((PResId)(gFacCodeSyms[j])->Data)->State != RESID_STATE_DELETED)
	{
	  fwrite( (gFacCodeSyms[j])->String, (gFacCodeSyms[j])->StrLen,
		  1, gMergeTarget );
	}
      fputc(' ', gMergeTarget);
    }
  fprintf(gMergeTarget, ";\n\t\t");

  j = pAlertRes->Severity;  
  fwrite( (gSevCodeSyms[j])->String, (gSevCodeSyms[j])->StrLen,
	  1, gMergeTarget );
  fprintf(gMergeTarget, ";\n\t\t\"%s\";\n\t}\n", pAlertRes->Text);

  pAlertRes->State = RESID_STATE_STABLE;

  return NFR_ERR_SUCCESS;
}

