/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   parse.c
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

#include "sys-include.h"

/* #define DEBUG_LEV 1 */
#include "debug.h"

#include "libnfr.h"
#include "rc.h"

#include "ytab.h"

PSymbol gFacCodeSyms[FAC_CODES_MAXSIZE];
PSymbol gSevCodeSyms[4];

void Rc_Init()
{
  gpMemList = SCalloc(1, sizeof(TList));
  gpResTab = ResTab_New();
}

void Rc_Cleanup()
{
  ResTab_Delete(gpResTab);
  MemList_FreeAll();
  SFree(gpMemList);
}

ULONG Rc_Parse(ULONG ModeFlags, FILE *In, FILE *Out, TResTab *pDiffTab)
{
  int i = 0;
  int FacCount = 0;
  ULONG Result = 0;
  PSymbol pSymbol = NULL;
  PResId pResId = NULL;
  TIdentifier FixedIds[] = {
    "SEV_NONE",         ID_SEV,  SEV_NONE, 
    "SEV_INFO",         ID_SEV,  SEV_INFO, 
    "SEV_WARNING",      ID_SEV,  SEV_WARNING, 
    "SEV_ERROR",        ID_SEV,  SEV_ERROR 
  };


  yyin = In;

  if(ModeFlags & RC_MODE_MERGE_DIFFS) 
    {
      gpDiffTab = pDiffTab;

      /* bug: fix this fac stuff for when one of the facs has been
         deleted by another process */

      fprintf(stderr, "in RC_MODE_MERGE_DIFFS (gFacCodeSyms init)\n");
      memset(gFacCodeSyms, 0, sizeof(gFacCodeSyms));
      
      /* first stick in the fac mappings from the diff tab */
      SymTab_ForEach(ID_FAC, AddResToArray, FAC_CODES_MAXSIZE, 
		     gFacCodeSyms, pDiffTab->SymTab);
    }


  if(ModeFlags & RC_MODE_SCAN)
    {
      DBG_OUT("\n------- beginning pass in RC_MODE_SCAN -----------\n");
      gMode = RC_MODE_SCAN;		/* scan srcs, collect identifiers */

      if(!gpCurFilename)
	gpCurFilename = "<ERROR_NO_FILE>";
      gCurLineNo = 0;

      gFatLadySing = FALSE;
      while (!gFatLadySing)
	yyparse();
      
      /* set the values for the hard-coded Ids, we do it here so that any
       * attempts to set them in the .src file will simply be overwritten */
      for(i=0; i < (sizeof(FixedIds) / sizeof(TIdentifier)); i++)
	{
	  Result = SymTab_Look(FixedIds[i].Id, &pSymbol, gpResTab->SymTab);
	  pSymbol->Type = FixedIds[i].Type;
	  pSymbol->Data = FixedIds[i].Val;
	}
    }

  if(ModeFlags & RC_MODE_COMPILE)
    {
      DBG_OUT("\n------- beginning pass in RC_MODE_COMPILE -----------\n");
      gMode = RC_MODE_COMPILE;	/* compile, assign resource values */
      rewind (yyin);

      if(!gpCurFilename)
	gpCurFilename = "<ERROR_NO_FILE>";
      gCurLineNo = 0;

      gFatLadySing = FALSE;
      while (!gFatLadySing)
	yyparse();
    }

  if(ModeFlags & RC_MODE_MERGE_DIFFS)
    {
      DBG_OUT("\n------- beginning pass in RC_MODE_MERGE_DIFFS -----------\n");

      /* these never change, so they will never exist in the difftab */
      memset(gSevCodeSyms, 0, sizeof(gSevCodeSyms));
      SymTab_ForEach(ID_SEV, AddSymToArray, 4, gSevCodeSyms, gpResTab->SymTab);


      /* bug: should use a macro or sizeof() in the above calls to
         SymTabForEach() */

      gMode = RC_MODE_MERGE_DIFFS;
      gMergeTarget = Out;
      rewind (yyin);

      if(!gpCurFilename)
	gpCurFilename = "<ERROR_NO_FILE>";
      gCurLineNo = 0;

      gFatLadySing = FALSE;
      while (!gFatLadySing)
	yyparse();
      fclose (gMergeTarget);
    }

  gFixedResFlag = TRUE;

  return NFR_ERR_SUCCESS;
}


