/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   rc.h
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

#ifndef _RC_H_
#define _RC_H_

#include "libnfr.h"
#include "librc.h"
#include "restab.h"

#define RC_ERR_RANGE_EXCEEDED 1

#ifndef NFR_RES_H_  
/* there's a few res codes that are fixed, so we define them here */

/* Alert severities */
#define SEV_NONE 0
#define SEV_INFO 1
#define SEV_WARNING 2
#define SEV_ERROR 3

#define NFR_ERR_SUCCESS               0x00000000

#endif /* NFR_RES_H_ */

#define RC_ERR_UNDEFINED 1

/* global vars */
extern PResTab gpDiffTab;
extern PSymTab gpPresetsTab;
extern ULONG gMode;
extern ULONG gUnragedIdNo;
extern char *gCurRangeId;
extern PNIList gpMemList;
extern char *gpCurFilename;
extern ULONG gCurLineNo;
extern BOOL gFatLadySing;
extern PSymbol gFacCodeSyms[];
extern PSymbol gSevCodeSyms[];
extern FILE *yyin;
extern FILE *gMergeTarget;

typedef struct Identifier
{
  char *Id;
  int Type;
  ULONG Val;
} TIdentifier, *PIdentifier;

/* create.c */
int Rc_CreateAlertRes(PSymbol, PNIList, char *, char *);
int Rc_CreateAlertSrc(PSymbol, char *, char *);
int Rc_CreateStringRes(PSymbol, char *);
int Rc_CreateFacRes(PSymbol, char *);
int Rc_SetId(char *, ULONG);
int Rc_AssignIdInRange(char *, ULONG);

/* iterators.c */
int PrintAlertResFields(PSymbol, int, void *);
int PrintAlertSrcFields(PSymbol, int, void *);
int PrintStringResFields(PSymbol, int, void *);
int PrintFacNVPair(PSymbol, int, void *);
int PrintFacFields(PSymbol, int, void *);
int PrintSevFields(PSymbol, int, void *);
int PrintUntypedId(PSymbol, int, void *);
int PrintMapEntry(PSymbol, int, void *);
int PrintResDefine(PSymbol, int, void *);
int PrintGenDefine(PSymbol, int, void *);
int AddResToArray(PSymbol, int, void *);
int AddSymToArray(PSymbol, int, void *);
int AlertRes_DeleteSym(PSymbol, int, void *);
int AlertSrc_DeleteSym(PSymbol, int, void *);
int StringRes_DeleteSym(PSymbol, int, void *);
int RangeRes_DeleteSym(PSymbol, int, void *);
int ResId_DeleteSym(PSymbol, int, void *);
int TryAddIdToRange(PSymbol, int, void *);
int ResSym_Write(PSymbol, int, void *);

/* merge.c */
ULONG Rc_MergeIdRangeClose();
ULONG Rc_MergeFacIdRangeOpen(char *);
ULONG Rc_MergeFacIdRangeClose();
ULONG Rc_MergeIdRangeOpen(char *);
ULONG Rc_MergeAlertRes(char *);
ULONG Rc_MergeAlertSrc(char *);
ULONG Rc_MergeStringRes(char *);

/* support.c */
void Rc_Warning(char *, ...);
void Rc_Error(char *, ...);
void *MemList_Add(void *, int);
void MemList_Free(void *);
void MemList_FreeAll(void);
int yyerror(char *);

char *QStrAddChar(char);

int yylex(void);
int yyparse(void);


/* macros */
#define AddString(s,l) (char *) NIList_Insert((void *)s, l);
#define RemoveString(s,l) (char *) NIList_Remove((void *)s, l)
#define NextString(l) (char *) NIList_RemoveTail(l)
#define FreeStrings(l) while(NIList_RemoveTail(l))

#define MemList_strdup(s) (char *)MemList_Add((void *)s, strlen(s)+1)

#endif
