%{
/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   rc.lex
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

/*
 *  Lexer for nfr resource compiler
 */

#include "sys-include.h"

/* uncomment this for TONS of lexer debug ralphage */
/* #define DEBUG_LEV 1   */
#include "debug.h"

#include "libnfr.h"

#include "xtypes.h"
#include "rc.h"
#include "ytab.h"

#define STATE_STACK_SIZE 32

int gStateStack[STATE_STACK_SIZE];
int gStatePtr = 0;
int gRightBraces = 0;

typedef struct _TSrcFile
{
  char *Name;
  ULONG Line;
} TSrcFile, *PSrcFile;


%}

%x S_SLCOMMENT S_QSTRING

AttributesKey     ("_attributes")
FixedResKey       ("_fixed")
FacRangeKey       ("_facilities")
AlertRangeKey     ("_alerts")
AlertSrcRangeKey  ("_alert_srcs")
StringRangeKey    ("_strings")
AlphaString       ([A-Za-z_][A-Za-z0-9_]+)
NumString         ([0-9]+)
HexNumString      ("0x"[0-9A-Fa-f]+)
WSChar            ([ \t])
SLComment         ("#")
LF                ("\n")
Quote             ("\"")
EscBackslash      ("\\\\")
EscLF             ("\\n")
EscCR             ("\\r")
EscTab            ("\\t")
EscQuote          ("\\\"")
EscAny            ("\\".)

%%


<S_QSTRING>{Quote} {
  DBG_OUT("</S_QSTRING>");
  BEGIN INITIAL;
  yylval.sval = QStrAddChar(0);
  return OnToken(QSTRING);
}

<S_QSTRING>{EscQuote} {  
  QStrAddChar('\\');
  QStrAddChar('\"');
}

<S_QSTRING>. {
  QStrAddChar(*yytext);
}

<S_SLCOMMENT>. {
}

<S_SLCOMMENT>{LF} {
  gCurLineNo++;
  DBG_OUT("</S_COMMENT><LF>\n%ld\t: ",  gCurLineNo);
  BEGIN INITIAL;
}

\{ {
  gRightBraces++;
  DBG_OUT("\"{\"(%d)", gRightBraces);
  return OnToken('{');
}

\} {
  if (gRightBraces == 0)
    return 0;
  DBG_OUT("\"}\"(%d)", gRightBraces);
  if (gRightBraces-- == 1)
    unput('}');
  return OnToken('}');
}

{SLComment} {
  DBG_OUT("<S_SLCOMMENT>");
  BEGIN S_SLCOMMENT;
}

{Quote} {
  DBG_OUT("<S_QSTRING>");
  BEGIN S_QSTRING;
}

{AttributesKey} {
  DBG_OUT("<ATTRIB_TAG>");
  return OnToken(ATTRIB_TAG);
}

{FixedResKey} {
  DBG_OUT("<FIXED_RESFILE>");
  return OnToken(FIXED_RESFILE);
}

{FacRangeKey} {
  DBG_OUT("<FAC_RANGE>");
  yylval.sval = MemList_strdup(yytext);
  return OnToken(FAC_RANGE);
}

{AlertRangeKey} {
  DBG_OUT("<ALERT_RANGE>");
  yylval.sval = MemList_strdup(yytext);
  return OnToken(ALERT_RANGE);
}

{AlertSrcRangeKey} {
  DBG_OUT("<ALERT_SRC_RANGE>");
  yylval.sval = MemList_strdup(yytext);
  return OnToken(ALERT_SRC_RANGE);
}

{StringRangeKey} {
  DBG_OUT("<STRING_RANGE>");
  yylval.sval = MemList_strdup(yytext);
  return OnToken(STRING_RANGE);
}

{AlphaString} {
  DBG_OUT("<ALPHASTRING>");
  yylval.sval = yytext;
  return OnToken(ALPHASTRING);
}

{NumString} {
  DBG_OUT("<NUMBER>(d)");
  yylval.ldval = strtol(yytext, NULL, 10);
  return OnToken(NUMBER);
}

{HexNumString} {
  DBG_OUT("<NUMBER>(X)");
  yylval.ldval = strtol(yytext, NULL, 16);
  return OnToken(NUMBER);
}

{WSChar} {
}

<INITIAL,S_SLCOMMENT,S_QSTRING>{LF} {
  gCurLineNo++;
  DBG_OUT("<LF>\n%ld\t: ",  gCurLineNo);
}

. {
  DBG_OUT("\"%c\"", *yytext);
  return OnToken(*yytext);/* return any char that falls through to here */
}

%%

int OnToken(int Tok)
{
  return Tok;			
  /* this will eventually contain some helpers for the comment and
     whitespace preservation code */
}

char *QStrAddChar(char c)
{
  static BOOL StrBufFull;
  static char QStrBuf[4096];
  static ULONG QStrBufIdx = 0;
  
  if(QStrBufIdx < sizeof(QStrBuf))
    {
      QStrBuf[QStrBufIdx++] = c;
      if (c == 0)
	{
	  QStrBufIdx = 0;
	  return QStrBuf;
	}
      else
	return NULL;
    }
  else if (!StrBufFull)
    {
      StrBufFull = TRUE;
      Rc_Warning("String exceeds max length,"
		 "truncated to %d bytes\n",
		 sizeof(QStrBuf));
    }
  return NULL;
}

int yywrap(void)
{
  DBG_OUT("eof in %s at %ld\n", gpCurFilename, gCurLineNo);
  gFatLadySing = TRUE;
  return 1;
}
 
