/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   rc.y
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

%{
/*
 * yacc grammer for rc, the nfr resource compiler
 */
#include "sys-include.h"

#include "rc.h"

/* uncomment this for TONS of lexer debug ralphage */
/* #define DEBUG_LEV 1 */
#include "debug.h"

%}

%union {
  long int ldval;
  char *sval;
  TNIList Listval;
}

%token <sval> QSTRING ALPHASTRING
%token <ldval> NUMBER
%token <sval> FAC_RANGE ALERT_RANGE ALERT_SRC_RANGE STRING_RANGE
%token ERROR FIXED_RESFILE ATTRIB_TAG
%type <sval> FacIdRangeDef StringIdRangeDef AlertIdRangeDef AlertSrcIdRangeDef
%type <sval> AlphaString QString
%type <Listval> AlphaStringList

%%

ResDefs: 
|        AttributeSec | FacRanges | StringRanges | AlertRanges | AlertSrcRanges
{
}
;

AttributeSec: Attrib_Tag '{' '}'
|             Attrib_Tag '{' Attributes '}'
{
  DBG_OUT("end attributes section\n"); 
  switch (gMode)
    {
    case RC_MODE_SCAN:
      break;

    case RC_MODE_COMPILE:
      break;

    case RC_MODE_MERGE_DIFFS:
      fprintf(gMergeTarget, "}\n");
      break;
    }
}
;

Attrib_Tag: ATTRIB_TAG
{ 
  DBG_OUT("begin attributes section\n"); 
  switch (gMode)
    {
    case RC_MODE_SCAN:
      break;

    case RC_MODE_COMPILE:
      break;

    case RC_MODE_MERGE_DIFFS:
      fprintf(gMergeTarget, "_attributes\n{\n");
      break;
    }
}


Attributes: Attribute ';'
|           Attributes Attribute ';'
{
}
;

Attribute: FixedResfile
{
}
;


FixedResfile: FIXED_RESFILE
{ 
  DBG_OUT("directive: FixedResfile\n"); 
  switch (gMode)
    {
    case RC_MODE_SCAN:
      DBG_OUT("setting gFixedResFlag to FALSE\n"); 
      gFixedResFlag = FALSE;
      break;

    case RC_MODE_COMPILE:
      break;

    case RC_MODE_MERGE_DIFFS:
      fprintf(gMergeTarget, "\t_fixed;\n");
      break;
    }
}



FacRanges: FacRange 
;

FacRange: FacIdRangeDef '{' '}'
{ 
  DBG_OUT("end of FacIdRange\n"); 
  switch (gMode)
    {
    case RC_MODE_SCAN:
      gCurRangeId = NULL;
      break;

    case RC_MODE_COMPILE:
      break;

    case RC_MODE_MERGE_DIFFS:
      Rc_MergeIdRangeClose();
      break;
    }
}

|           FacIdRangeDef '{' FacIds '}'
{ 
  DBG_OUT("end of FacIdRange\n"); 
  switch (gMode)
    {
    case RC_MODE_SCAN:
      gCurRangeId = NULL;
      break;

    case RC_MODE_COMPILE:
      break;

    case RC_MODE_MERGE_DIFFS:
      Rc_MergeIdRangeClose();
      break;
    }
}
;

FacIds: FacId
|       FacIds FacId
;


FacId: AlphaString QString ';'
{ 
  ULONG Result = 0;
  PSymbol pSymbol = NULL;

  switch (gMode)
    {
    case RC_MODE_SCAN:
      DBG_OUT("%s declared as a facility\n", $1); 
      Rc_AssignIdInRange($1, ID_FAC);
      break;
      
    case RC_MODE_COMPILE:
      Result = SymTab_Look($1, &pSymbol, gpResTab->SymTab); 
      if (Result == SYM_FOUND)
	Rc_CreateFacRes(pSymbol, $2);
      else
	Rc_Error("undefined symbol \"%s\"", $1);
      break;
      break;
      
    case RC_MODE_MERGE_DIFFS:
      Rc_MergeIdFac($1);
      break;
    }
}
;

FacIdRangeDef: FAC_RANGE
{ 
  ULONG Result = 0;
  static char FacRangeId[] = "_FAC_ID";
  static char FacRangeDispName[] = "Alert facilities";

  DBG_OUT("_FAC_ID range spans 0 to %ld\n", FAC_CODES_MAXSIZE);
  $$ = FacRangeId;
  gCurRangeId = FacRangeId;
  switch (gMode)
    {
    case RC_MODE_SCAN:
      Result = ResTab_DefineRange(FacRangeId, 1, FAC_CODES_MAXSIZE, 
				  FacRangeDispName, ID_FAC, NULL, gpResTab);
      if (Result != NFR_ERR_SUCCESS)
	Rc_Error("Range conflict involving %s", FacRangeId);
      break;

    case RC_MODE_COMPILE:
      break;

    case RC_MODE_MERGE_DIFFS:
      Rc_MergeFacIdRangeOpen($1);
      break;
    }
}
;











StringRanges: StringRange
|             StringRanges StringRange
;


StringRange: StringIdRangeDef '{' '}'
{ 
  DBG_OUT("end of StringRange: \"%s\"\n", $1);
  Rc_Warning("empty string range %s", $1); 
  switch (gMode)
    {
    case RC_MODE_SCAN:
      gCurRangeId = NULL;
      break;

    case RC_MODE_COMPILE:
      break;

    case RC_MODE_MERGE_DIFFS:
      Rc_MergeIdRangeClose(gCurRangeId);
      break;
    }
}

|            StringIdRangeDef '{' StringDefs '}'
{ 
  DBG_OUT("end of StringRange: \"%s\"\n", $1); 
  switch (gMode)
    {
    case RC_MODE_SCAN:
      gCurRangeId = NULL;
      break;

    case RC_MODE_COMPILE:
      break;

    case RC_MODE_MERGE_DIFFS:
      Rc_MergeIdRangeClose(gCurRangeId);
      break;
    }
}
;


StringDefs: StringDef
|           StringDefs StringDef
;


StringDef: AlphaString QString ';'
{ 
  PSymbol pSymbol = NULL;
  int Result = 0;

  DBG_OUT("%s defined as \"%s\"\n", $1, $2); 

  switch (gMode)
    {
    case RC_MODE_SCAN:
      Rc_AssignIdInRange($1, RES_STRING);
      break;

    case RC_MODE_COMPILE:
      Result = SymTab_Look($1, &pSymbol, gpResTab->SymTab); 
      if (Result == SYM_FOUND)
	Rc_CreateStringRes(pSymbol, $2);
      else
	Rc_Error("undefined symbol \"%s\"", $1);
      break;

    case RC_MODE_MERGE_DIFFS:
      Rc_MergeStringRes($1);
      break;
    }
  
}
;


StringIdRangeDef: STRING_RANGE AlphaString NUMBER '-' NUMBER QString
{ 
  ULONG Result = 0;

  DBG_OUT("Range \"%s\" spans %ld to %ld\n", $2, $3, $5);
  $$ = $2;
  gCurRangeId = $2;
  switch (gMode)
    {
    case RC_MODE_SCAN:
      Result = ResTab_DefineRange($2, $3, $5, $6, RES_STRING, NULL, gpResTab);
      if (Result != NFR_ERR_SUCCESS)
	Rc_Error("Range conflict involving %s", $2);
      break;

    case RC_MODE_COMPILE:
      break;

    case RC_MODE_MERGE_DIFFS:
      Rc_MergeIdRangeOpen($1);
      break;
    }
}
;




AlertRanges: AlertRange
|            AlertRanges AlertRange
;


AlertRange: AlertIdRangeDef '{' '}'
{ 
  DBG_OUT("end of AlertRange: \"%s\"\n", $1); 
  Rc_Warning("empty alert range %s", $1); 
  switch (gMode)
    {
    case RC_MODE_SCAN:
      gCurRangeId = NULL;
      break;

    case RC_MODE_COMPILE:
      break;

    case RC_MODE_MERGE_DIFFS:
      Rc_MergeIdRangeClose();
      break;
    }
}

|           AlertIdRangeDef '{' AlertDefs '}'
{ 
  DBG_OUT("end of AlertRange: \"%s\"\n", $1); 
  switch (gMode)
    {
    case RC_MODE_SCAN:
      gCurRangeId = NULL;
      break;

    case RC_MODE_COMPILE:
      break;

    case RC_MODE_MERGE_DIFFS:
      Rc_MergeIdRangeClose(gCurRangeId);
      break;
    }
}
;


AlertDefs: AlertDef
|          AlertDefs AlertDef
;


AlertDef: AlphaString '{' AlphaStringList ';' AlphaString ';' QString ';' '}'
{ 
  PSymbol pSymbol = NULL;
  PSymbol pDiffSymbol = NULL;
  int Result;

  DBG_OUT("AlertDef: \"%s\"\n", $1); 

  switch (gMode)
    {
    case RC_MODE_SCAN:
      Rc_AssignIdInRange($1, RES_ALERT);
      FreeStrings(&$3);
      /* Rc_AssignIdList(&$3, ID_UNTYPED); */
      /* don't need to assign ids to severities, they're automatic */
      break;

    case RC_MODE_COMPILE:
      Result = SymTab_Peek($1, &pSymbol, gpResTab->SymTab); 
      if (Result != SYM_FOUND)
	{
	  Rc_Error("undefined symbol \"%s\"", $1);
	  break;
	}
      else
	Result = Rc_CreateAlertRes(pSymbol, &$3, $5, $7);

      if(Result != NFR_ERR_SUCCESS)
	AlertRes_Delete((PAlertRes)ResTab_Remove($1, gpResTab));
      break;

    case RC_MODE_MERGE_DIFFS:
      Rc_MergeAlertRes($1);
      FreeStrings(&$3);
      break;
    }

}
;


AlertIdRangeDef: ALERT_RANGE AlphaString NUMBER '-' NUMBER QString
{ 
  ULONG Result = 0;

  DBG_OUT("Range \"%s\" spans %ld to %ld\n", $2, $3, $5);
  $$ = $2;
  gCurRangeId = $2;
  switch (gMode)
    {
    case RC_MODE_SCAN:
      Result = ResTab_DefineRange($2, $3, $5, $6, RES_ALERT, NULL, gpResTab);
      if (Result != NFR_ERR_SUCCESS)
	Rc_Error("Range conflict involving %s", $2);
      break;

    case RC_MODE_COMPILE:
      break;

    case RC_MODE_MERGE_DIFFS:
      Rc_MergeIdRangeOpen($1);
      break;
    }
}
;




AlertSrcRanges: AlertSrcRange
|               AlertSrcRanges AlertSrcRange
;


AlertSrcRange: AlertSrcIdRangeDef '{' '}'
{ 
  DBG_OUT("end of AlertSrcRange: \"%s\"\n", $1); 
  Rc_Warning("empty alert source range %s", $1); 
  switch (gMode)
    {
    case RC_MODE_SCAN:
      gCurRangeId = NULL;
      break;

    case RC_MODE_COMPILE:
      break;

    case RC_MODE_MERGE_DIFFS:
      Rc_MergeIdRangeClose(gCurRangeId);
      break;
    }
}

|           AlertSrcIdRangeDef '{' AlertSrcDefs '}'
{ 
  DBG_OUT("end of AlertSrcRange: \"%s\"\n", $1); 
  switch (gMode)
    {
    case RC_MODE_SCAN:
      gCurRangeId = NULL;
      break;

    case RC_MODE_COMPILE:
      break;

    case RC_MODE_MERGE_DIFFS:
      Rc_MergeIdRangeClose(gCurRangeId);
      break;
    }
}
;


AlertSrcDefs: AlertSrcDef
|             AlertSrcDefs AlertSrcDef
;


AlertSrcDef: AlphaString '{' QString ';' QString ';' '}'
{ 
  PSymbol pSymbol;
  int Result;

  DBG_OUT("AlertSrcDef: \"%s\"\n", $1); 

  switch (gMode)
    {
    case RC_MODE_SCAN:
      Rc_AssignIdInRange($1, RES_ALERT_SRC);
      break;

    case RC_MODE_COMPILE:
      Result = SymTab_Look($1, &pSymbol, gpResTab->SymTab); 
      if (Result == SYM_FOUND)
	Rc_CreateAlertSrc(pSymbol, $3, $5);
      else
	Rc_Error("undefined symbol \"%s\"", $1);
      break;

    case RC_MODE_MERGE_DIFFS:
      Rc_MergeAlertSrc($1);
      break;
    }
}
;


AlertSrcIdRangeDef: ALERT_SRC_RANGE AlphaString NUMBER '-' NUMBER QString
{ 
  ULONG Result = 0;

  DBG_OUT("Range \"%s\" spans %ld to %ld\n", $2, $3, $5);
  $$ = $2;
  gCurRangeId = $2;
  switch (gMode)
    {
    case RC_MODE_SCAN:
      Result = ResTab_DefineRange($2, $3, $5, $6, RES_ALERT_SRC, NULL, 
				  gpResTab);
      if (Result != NFR_ERR_SUCCESS)
	Rc_Error("Range conflict involving %s", $2);
      break;

    case RC_MODE_COMPILE:
      break;

    case RC_MODE_MERGE_DIFFS:
      Rc_MergeIdRangeOpen($1);
      break;
    }
}
;


AlphaStringList: 
{ 
  memset(&$$, 0, sizeof(TNIList));
}

|                AlphaString
{ 
  memset(&$$, 0, sizeof(TNIList));
  AddString($1, &$$); 
} 

|                AlphaStringList AlphaString
{ AddString($2, &$$); }
;


AlphaString: ALPHASTRING
  { $$ = MemList_strdup($1); }
;


QString: QSTRING
  { $$ = MemList_strdup($1); }
;

%%

