/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   support.c
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

/******************************************************************************
 * support and main routines used by the parser in rc.y
 *****************************************************************************/
#include "sys-include.h"

/* #define DEBUG_LEV 1 */
#include "debug.h"

#include "libnfr.h"
#include "rc.h"

#include "ytab.h"



void *MemList_Add(void *pSrc, int Size)
{
  void *pDest;

  pDest = SCalloc(1, Size);
  memcpy(pDest, pSrc, Size);
  NIList_Insert(pDest, gpMemList);
  return (pDest);
}

void MemList_Free(void *pTarget)
{
  NIList_Remove(pTarget, gpMemList);
  SFree(pTarget);
}

void MemList_FreeAll()
{
  void *p;

  while ((p=NIList_RemoveHead(gpMemList)))
    SFree(p);
}

void Rc_Warning(char *FmtStr, ...)
{
  va_list pArgs;

  gWarnings++;
  if (gVerbosity < 1)
    return;
  fprintf(stderr, "%s:%ld (approx) WARNING: ", gpCurFilename, gCurLineNo);
  va_start(pArgs, FmtStr);
  vfprintf(stderr, FmtStr, pArgs);
  va_end(pArgs);
  fputc('\n', stderr);
}

void Rc_Error(char *FmtStr, ...)
{
  va_list pArgs;

  gErrors++;
  fprintf(stderr, "%s:%ld (approx) ERROR: ", gpCurFilename, gCurLineNo);
  va_start(pArgs, FmtStr);
  vfprintf(stderr, FmtStr, pArgs);
  va_end(pArgs);
  fputc('\n',  stderr);
}

int yyerror(char *ErrStr)
{
  Rc_Error(ErrStr);
  return -1;
}

 

