/***************************************************************************** 
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT") 

  File:   aheader.c 
  Author: Kent Landfield <kent@nfr.net>
 
 *****************************************************************************/  
#include "sys-include.h"
#include "archive.h"

void free_aow_mem(Archive *rkive);

/*
**************************************************************
** archive_header - Return the archive header of the file.
**                  rkive->fp file pointer is positioned on
**                  the first record ready for reading.
**
** Parameters:
**     path     - The path to the file 
**
** Returns:       Pointer to Archive structure filled with the 
**                files meta header information
**************************************************************
*/

Archive *archive_header(char *path) 
{
    Archive *rkive;
    struct stat st;
    long reclen;

    if ((stat(path, &st) == 0) && ((st.st_mode & S_IFMT) != S_IFREG)) {
        archive_errno = ARCHIVE_CANT_STAT;
        return(NULL);
    }

    /* Need to allocate memory for the Archive structure */

    if ((rkive = (Archive *)calloc(1, sizeof(Archive))) == NULL) {
         archive_errno = ARCHIVE_NO_MEMORY;
         return(NULL);
    }

    /* Open the archive specified at path */

    if ((rkive->fp = fopen(path,"rb")) == NULL) {
         archive_errno = ARCHIVE_CANT_OPEN_ARCHIVE;
         free_aow_mem(rkive);
         return(NULL);
    }

    /* Read the file header structure */

    if (fread(&rkive->head, sizeof(struct header), 1, rkive->fp) == 0) {
         archive_errno = ARCHIVE_HEADER_READFAILED;
         free_aow_mem(rkive);
         return(NULL);
    }

    /* Read the recorder supplied blob */

    reclen = rkive->head.datastart - sizeof(struct header);
    
    if (reclen > 0) {
        if (fseek(rkive->fp, sizeof(struct header), SEEK_SET) != 0) {
             archive_errno = ARCHIVE_CANT_SEEK;
             free_aow_mem(rkive);
             return(NULL);
        }

        rkive->info = (char *)calloc(1, reclen+1);

        if (fread(rkive->info, reclen, 1, rkive->fp) == 0) {
             archive_errno = ARCHIVE_BLOB_READFAILED;
             free_aow_mem(rkive);
             return(NULL);
        }
    }
    return(rkive);
}
