/***************************************************************************** 
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT") 

  File:   anotify.c 
  Author: Kent Landfield <kent@nfr.net>
 
 *****************************************************************************/  
#include "sys-include.h"
#include "spaceman.h"
#include "archive.h"

/*
***************************************************************************
** _archive_notify_spaceman - Notify the space management system of new file
**
** Abstract:   Notify space management so that it can be alerted to the
**             existance of a new file.  The new file is then checked
**             to assure it has not pushed an expire threshold.
**
** Parameters: pointer to Archive struct returned from archive_openwrite()
**
** Returns:    1 - success, 0 failure and  archive_errno set
** NOTE:       This really isn't an error if it fails.  This is an
**             advisory to the space management subsystem to very that
**             thresholds are not being exceeded.  If spaceman is not
**             running then things will be cleaned up when it is started
**             next time. 
***************************************************************************
*/

static char *Fifopath = SPACEMAN_FIFO;

int archive_notify_spaceman(Archive *rkive)
{
    char *strcpy();
    char pbuf[BUFSIZ];        /* Full messae to be sent */
    int fd;                      /* File descriptor        */
    int sendlen;                 /* length of message to send */
    struct indx_rec Irec;
    int rc = 1;

    strcpy(Irec.backend, rkive->head.backend);
    strcpy(Irec.ctype, rkive->head.ctype);
    strcpy(Irec.path, rkive->path);
    Irec.starttime = rkive->head.starttime;

    if ((fd = open(SPACEMAN_FIFO,O_WRONLY|O_NDELAY,0666)) == -1) {
       return(-1);
    }

    sendlen = sizeof(struct indx_rec);

    if (write(fd, &Irec, (unsigned)sendlen) != sendlen) {
        (void) close(fd);
 
        return(0);
    }

    (void) close(fd);
    return(1);
}
