/***************************************************************************** 
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT") 

  File:   cdate.c 
  Author: Kent Landfield <kent@nfr.net>
 
 *****************************************************************************/

#include "sys-include.h"

#define USAGE  "usage: %s [ -v ] [ time-string-val-in-secs ... ]\n"

int verbose = 0;

extern int opterr;

void cdate(char *dtstr)
{
    long tstamp;
    struct tm *now;

    tstamp = atol(dtstr);
    now = localtime(&tstamp);

    printf("Time %s == %.2d/%.2d/%.2d %.2d:%.2d:%.2d\n", dtstr,
        now->tm_mon,now->tm_mday,now->tm_year,
        now->tm_hour,now->tm_min,now->tm_sec);
}

int main(int argc, char **argv)
{
   int c;

   opterr = 0;
   verbose = 0;

   if (argc > 1) {
      while ((c = getopt(argc, argv, "v")) != EOF) {
         switch (c) {
             case 'v':
                 verbose++;
                 break;
             default:
                 (void) fprintf(stderr,USAGE, argv[0]);
                 return(1);
         }
      }
   }

  /* Accept times on stdin or on the command line */

   if ((optind >= argc) || (argc == 1)) {  /* file from stdin */
        char buf[BUFSIZ];
        while (fgets(buf,sizeof buf,stdin) != NULL) {
               buf[strlen(buf) -1] = '\0';
               cdate(buf);
        }
   }
   else { 
        for (; optind < argc; optind++)    /* process files to print */
             cdate(argv[optind]);
   }
   return(0);                              /* terminate this process */
}
