/***************************************************************************** 
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT") 

  File:   rdindex.c 
  Author: Kent Landfield <kent@nfr.net>
 
 *****************************************************************************/ 

#include "sys-include.h"
#include "archive.h"

#define USAGE  "usage: %s [ -v ] [ -p backend || path to index file ]\ n"

int verbose = 0;

char *tmstring(time_t tstamp)
{
    struct tm *now;
    static char dtstr[80];

    now = localtime(&tstamp); 
    sprintf(dtstr,"%.2d/%.2d/%.2d %.2d:%.2d:%.2d",now->tm_mon,now->tm_mday,
                        now->tm_year,now->tm_hour,now->tm_min,now->tm_sec);
    return(dtstr);
}

int dump_index(char *indexfile)
{
    int rec;
    FILE *ifp;

    struct indx_rec Irec;

    if ((ifp = fopen(indexfile,"rb")) == NULL) {
         (void) fprintf(stderr,"Cant open %s\n", indexfile);
         return(1);
    }

    fprintf(stderr,"%s: Opened\n", indexfile);

    rec = 0;

    /* read the index records */

    while (fscanf(ifp,"%s %s %s %ld", Irec.backend, Irec.ctype, Irec.path, &Irec.starttime) != EOF) {

        rec++;
        fprintf(stderr,"Record %d:\n", rec);

        fprintf(stderr,"\tBackend: %s\n", Irec.backend);
        fprintf(stderr,"\tContent type: %s\n", Irec.ctype);
        fprintf(stderr,"\tPath to file: %s\n", Irec.path);

        fprintf(stderr,"\tStarttime: %ld - %s\n", 
                    Irec.starttime, tmstring(Irec.starttime));
    }
    fclose(ifp);
    fprintf(stderr,"%s: Closed\n", indexfile);

    return(0);
}

int main(int argc, char **argv)
{
   char path[BUFSIZ];
   int c;
   extern char *optarg;
   extern int optind;
   extern int opterr;

   opterr = 0;
   verbose = 0;

   if (argc > 1) {
      while ((c = getopt(argc, argv, "p:v")) != EOF) {
         switch (c) {
             case 'p': /* Build path to index file from package name */
                   sprintf(path,"%s/%s/index",INDEXDIR,optarg);
                   return(dump_index(path));
             case 'v':
                 verbose++;
                 break;
             default:
                 return(1);
         }
      }
   }

   /* Accept pathnames to file to dump on stdin or on the command line */

   c = 0;

   if ((optind >= argc) || (argc == 1)) {  /* file from stdin */
        char buf[BUFSIZ];
        while (fgets(buf,sizeof buf,stdin) != NULL) {
               buf[strlen(buf) -1] = '\0';
               c += dump_index(buf);
        }
   }
   else { 
        for (; optind < argc; optind++)    /* process files to print */
             c += dump_index(argv[optind]);
   }
   return(c);                              /* terminate this process */
}

