/***************************************************************************** 
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT") 

  File:   rtest.c 
  Author: Kent Landfield <kent@nfr.net>
 
 *****************************************************************************/ 

#include "sys-include.h"
#include "archive.h"

#define DEFAULT_BACKEND "webwatcher/webwatcher"

extern int archive_debug;
extern int opterr;

void dump_rkive(Archive *rkive);

void print_record(struct rec *rc, char *buf, char *id)
{
    fprintf(stderr,"\nTime Stamp    = [%ld]\n", rc->tstamp);
    fprintf(stderr,"ID Length     = [%d]\n", rc->id_len);
    fprintf(stderr,"Data Length   = [%ld]\n", rc->data_len);
    fprintf(stderr,"Next Record @ = [%ld]\n", rc->next_rec);
    fprintf(stderr,"Last Record @ = [%ld]\n", rc->last_rec);
    fprintf(stderr,"Record ID     = [%s]\n", id);
    fprintf(stderr,"Record        = [%s]\n", buf);
}

int usage(char *progname)
{
   fprintf(stderr,"%s [-dh] [-c content_type] [-e endtime] [-p package/backend] [-s starttime]\n", progname);
   fprintf(stderr,"\t-d              : Turn on verbose debugging\n");
   fprintf(stderr,"\t-h              : Show header contents as well\n");
   fprintf(stderr,"\t-c content_type : content type indicator\n");
   fprintf(stderr,"\t-e endtime      : time in seconds to stop reading\n");
   fprintf(stderr,"\t-p backend      : backend recording from\n");
   fprintf(stderr,"\t-s starttime    : time in seconds to start recording from\n");
   return(1);
}


int main(int argc, char **argv)
{
   char *buf;
   char *ctype;
   char *backend;
   char unique_id[BUFSIZ];

   int c;
   long nbytes;
   time_t endtime;
   time_t starttime;
   Archive *rkive;
   struct rec *rp;

   int show_header = 0;

   opterr = 0;

   endtime = 0;
   starttime = 0;
   ctype = NULL;
   backend = DEFAULT_BACKEND;

   if (argc > 1) {
      while ((c = getopt(argc, argv, "c:de:hs:p:")) != EOF) {
         switch (c) {
             case 'c': /* Content_type to use */
                 ctype = optarg;
                 break;
             case 'd': /* verbose debugging stuff */
                 archive_debug = 1;
                 break;
             case 'e': /* endtime */
                 endtime = atol(optarg);
                 break;
             case 'h': /* show header info as well */
                 show_header++;
                 break;
             case 's': /* starttime */
                 starttime = atol(optarg);
                 break;
             case 'p': /* package/backend */
                 backend = optarg;
                 break;
             default:  /* get outta here */
                 return(usage(argv[0]));
         }
      }
   }

   if (starttime != 0)
       fprintf(stderr,"Starttime == %ld\n", starttime);
   if (endtime != 0)
       fprintf(stderr,"Endtime == %ld\n", endtime);

   if ((rkive = archive_openread(backend,ctype,starttime,endtime)) == NULL) {
       fprintf(stderr,"openread failed - error == %d: %s\n", 
                      archive_errno, astrerror(archive_errno));
       return(1);
   }

   if (show_header)
       dump_rkive(rkive);

   nbytes = 2048;

   if ((buf = (char *)calloc(1, nbytes)) == NULL) {
       fprintf(stderr,"Cant allocate buffer\n");
       return(1);
   }

   adebug("Record Pos == %d\n", ftell(rkive->fp));

   /* Need to make unique_id a definable width up to a max of BUFSIZ */

   while((rp = archive_read(rkive,nbytes,buf,endtime,unique_id)) != NULL) {
          print_record(rp, buf, unique_id);
          adebug("\nPos == %d\n", ftell(rkive->fp));
   }

   free(buf);

   if (archive_errno != 0)
       fprintf(stderr,"read failed - error == %d: %s\n", 
                      archive_errno, astrerror(archive_errno));

   if (!archive_close(rkive))
       fprintf(stderr,"close failed - error == %d: %s\n", 
                      archive_errno, astrerror(archive_errno));

   return(0);                              /* terminate this process */
}
