/***************************************************************************** 
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT") 

  File:   wtest.c 
  Author: Kent Landfield <kent@nfr.net>
 
 *****************************************************************************/ 
#include "sys-include.h"
#include "archive.h"

unsigned sleep();

#define DEFAULT_BACKEND "webwatcher/webwatcher"

char recorder_blob[BUFSIZ];

extern int archive_debug;
extern int opterr;

char *astrerror();  

char *records[] = {
"Yo mama dont dance and",
"Yo daddy dont rock and roll.",
"That aint no fun!",
"And that aint no life!",
"They live it just the same.",
"They really have no choice.",
"They rather have a Rolls Royce,",
"But with beautiful kids their moneys not here",
"Its in the kids cars and college years.",
"              guess who ?",
};

#define TESTBLOB "This is a test blob and I want to see it intact so I know it works as it should"

int usage(char *progname)
{
   fprintf(stderr,"%s [-d] [ -b blob ] [-c content_type] [-p package/backend] [-s starttime] [-t timer]\n", progname);
   fprintf(stderr,"\t-d              : Turn on verbose debugging\n");
   fprintf(stderr,"\t-b blob         : supply recorder blob string\n");
   fprintf(stderr,"\t-c content_type : content type indicator\n");
   fprintf(stderr,"\t-p backend      : backend recording from\n");
   fprintf(stderr,"\t-s starttime    : time in seconds to start recording from\n");
   fprintf(stderr,"\t-t timer        : time interval between writing records\n");
   return(1);
}

int main(int argc, char **argv)
{
   Archive *rc;
   time_t starttime;

   int c;
   int timer;
   int empty;
   char *ctype;
   char *backend;
   char buf[2048];

   opterr = 0;
   empty = 0;
   timer = 1;

   ctype = NULL;
   backend = DEFAULT_BACKEND;
   starttime = time((long *)0);
   strcpy(recorder_blob, TESTBLOB);

   if (argc > 1) {
      while ((c = getopt(argc, argv, "b:c:dep:s:t:")) != EOF) {
         switch (c) {
             case 'b':   /* Substitute Recorder Blob */
                 strcpy(recorder_blob, optarg);
                 break;
             case 'c':   /* Content_type to use */
                 ctype = optarg;
                 break;
             case 'd':   /* verbose debugging stuff */
                 archive_debug++;
                 break;
             case 'e':   /* create empty data files */
                 empty++;
                 break;
             case 'p':   /* package/backend */
                 backend = optarg;
                 break;
             case 's':   /* starttime */
                 starttime = atol(optarg);
                 break;
             case 't':   /* interval between records */
                 timer = atoi(optarg);
                 fprintf(stderr,"New records on %d second intervals\n",timer);
                 break;
             default:   
                 return(usage(argv[0]));
         }
      }
   }
   
   rc = archive_openwrite(backend,ctype,starttime,recorder_blob);
   if (rc == NULL) {
       fprintf(stderr,"openwrite failed - %d:%s\n",
                       archive_errno, astrerror(archive_errno));
       return(1);
   }

   if (!empty) {
       for (c = 0; c < 10; c++) {
          fprintf(stderr,"Record Time == %ld\n",starttime);

          sprintf(buf,"%s - %s", rc->path, records[c]);

          archive_write(rc,strlen(buf),buf,starttime,"BOOGIEwoogie");

          fprintf(stderr,"%s written\n", buf);

          starttime += (long)timer;
   
          archive_flush(rc);
      
          if (timer > 0)
              sleep((unsigned)timer);
       }
   }

   if (archive_close(rc))
       fprintf(stderr,"archive_openwrite SUCCEEDED\n");
   else
       fprintf(stderr,"archive_close troubles: %d:%s\n", 
                       archive_errno, astrerror(archive_errno));

   return(0);                              /* terminate this process */
}
