/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/


#include "nfrd.h"
/*
** basicPickle.
**
** support for lowest level of types which we can pickle.
*/

/*
** ------------------------------
*/

typedef struct basicIntSchema {
	int             basicInt;
} basicIntSchema_t ;

intPickle( f, rs )
FILE * f;
basicIntSchema_t * rs;
{
    /*
    ** Do what we need to do to pickle the type contents...
    */
    if (f) {
	fprintf( f, "%d\n", rs->basicInt );
	return(1);
    } else {
	return(0);
    }

}


/*
** ------------------------------
*/

typedef struct basicStrSchema {
	char *		basicStr;
} basicStrSchema_t;

stringPickle( f, rs )
FILE * f;
basicStrSchema_t * rs;
{
    if (f) {
	char * c = rs->basicStr;
	/*
	** print out the string... watching out for quoting...
	*/
	fputc( '"', f );
	while( *c ) {
	    switch(*c) {
		case '\n':
		    fputc( '\\', f );
		    fputc( 'n', f );
		    break;
		case '\t':
		    fputc( '\\', f );
		    fputc( 't', f );
		    break;
		case '\\':
		    fputc( '\\', f );
		    fputc( '\\', f );
		    break;
		case '"':
		    fputc( '\\', f );
		    fputc( '"', f );
		    break;
		default:
		    fputc( *c, f );
	    }
	    c++;
	}
	fputc( '"', f );
	fputc( '\n', f );
	return(1);
    } else {
	return(0);
    }
}

/*
** ------------------------------
*/

typedef struct basicListSchema {
	ListMem *	basicList;
} basicListSchema_t ;

/*
** we pickle the list by marking the type of the list,
** then just pickling each of the members.
** When we're done 
*/
listPickle( f, rs )
FILE * f;
basicListSchema_t * rs;
{
    if (f) {
	ListMem * l = rs->basicList;
	while(l) {
	    pickle( f, &l->data );
	    l = l->next;
	}
	fprintf( f, "<end-of-basicList>\n" );
	return(1);
    } else {
	return(0);
    }
}

/*
** ------------------------------
*/

typedef struct basicRecorderSchema {
    recorder_t *        basicRecorder;
} basicRecorderSchema_t;

recorderPickle( f, rs )
FILE * f;
basicRecorderSchema_t * rs;
{
    if (f) {
	if (rs->basicRecorder->r_argv) {
	    fprintf( f, "%s\n", rs->basicRecorder->r_argv[0] );
	} else {
	    fprintf( f, "<no name>\n" );
	}
	return(1);
    } else {
	return(0);
    }
}


/*
** ------------------------------
*/

typedef struct basicSchemaData {
    schemaData_t *	basicSchemaData;
} basicSchemaData_t;

schemaDataPickle( f, rs )
FILE * f;
basicSchemaData_t * rs;
{
    if (f) {
	/* XXX not yet -- maybe sometime later. */
	fprintf( f, "<schemaData>\n" );
	return(1);
    } else {
	return(0);
    }
}

/*
** ------------------------------
*/

typedef struct basicIpv4HostSchema {
	struct in_addr        basicIpv4Host;
} basicIpv4HostSchema_t ;

ipaddrPickle( f, rs )
FILE * f;
basicIpv4HostSchema_t * rs;
{
    if (f) {
	fprintf( f, "%s\n", inet_ntoa( rs->basicIpv4Host ) );
	return(1);
    } else {
	return(0);
    }
}

/*
** ------------------------------
*/

typedef struct basicIpv4NetSchema {
	ipv4net_t *        basicIpv4Net;
} basicIpv4NetSchema_t ;

ipnetPickle( f, rs )
FILE * f;
basicIpv4NetSchema_t * rs;
{
    if (f) {
	struct in_addr m;
	fprintf( f, "%s\n", inet_ntoa( rs->basicIpv4Net->ip_addr ) );
	m.s_addr = rs->basicIpv4Net->ip_mask;
	fprintf( f, "%s\n", inet_ntoa( m ) );
	return(1);
    } else {
	return(0);
    }
}

/*
** ------------------------------
*/

typedef struct basicBlobSchema {
	blob_t *        basicBlob;
} basicBlobSchema_t ;

blobPickle( f, rs )
FILE * f;
basicBlobSchema_t * rs;
{
    /*
    ** do what we need to do to pickle the type contents...
    */
    if (f) {
	fprintf( f, "%d\n", rs->basicBlob->b_len );
	base64Encode( f, rs->basicBlob->b_data, rs->basicBlob->b_len );
	return(1);
    } else {
	return(0);
    }
}

/*
** base64 encoder.
**
*/

static char basis_64[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
	"abcdefghijklmnopqrstuvwxyz0123456789+/";

base64Encode(f, d, len)
FILE *f;
char * d;
int len;
{
    int ct=0;

    while(len >= 3) {
	pickle64chunk( f, d[0], d[1], d[2], 0 );
	len -= 3;
	d += 3;
	ct += 4;
	if (ct > 71) {
	    putc('\n', f);
	    ct = 0;
	}
    }
    switch(len) {
	case 2:
	    pickle64chunk( f, d[0], d[1], 0, 1 );
	    putc('\n', f);
	    break;

	case 1:
	    pickle64chunk( f, d[0], 0, 0, 2 );
	    putc('\n', f);
	    break;

    }

    if (ct) putc('\n', f );
}
 
pickle64chunk(f, a, b, c, pads)
FILE *f;
int a, b, c, pads;
{
    putc(basis_64[a>>2], f);
    putc(basis_64[((a & 0x3)<< 4) | ((b & 0xf0) >> 4)], f);
    if (pads == 2) {
        putc('=', f);
        putc('=', f);
    } else if (pads) {
        putc(basis_64[((b & 0xf) << 2) | ((c & 0xf0) >>6)], f);
        putc('=', f);
    } else {
        putc(basis_64[((b & 0xf) << 2) | ((c & 0xc0) >>6)], f);
        putc(basis_64[c & 0x3f], f);
    }
}

/*
** ethmac encoder.
** Just split out the value, pretty printed :^)
*/
struct basicEthMacSchema {
	ethMac_t *      basicEthMac;
};

typedef struct basicEthMacSchema basicEthMacSchema_t;

ethmacPickle( f, rs )
FILE * f;
basicEthMacSchema_t * rs;
{
    ethMac_t * e = rs->basicEthMac;
    fprintf( f, "%2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x",
	    e->e_addr[0], e->e_addr[1], e->e_addr[2],
	    e->e_addr[3], e->e_addr[4], e->e_addr[5] );
}
