/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/

#include "nfrd.h"

/*
** basicUnpickle.
**
** support for lowest level of types which we can un-pickle.
*/

/*
** ------------------------------
*/

typedef struct basicIntSchema {
	int             basicInt;
} basicIntSchema_t ;

intUnpickle( f, rs )
FILE * f;
basicIntSchema_t * rs;
{
    /*
    ** Do what we need to do to unpickle the type contents...
    */
    if (f) {
	if (fscanf( f, "%d", &rs->basicInt ) == 1) {
	    return(1);
	} else {
	    return(0);
	}
    } else {
	return(0);
    }

}

/*
** ------------------------------
*/

typedef struct basicStrSchema {
	char *          basicStr;
} basicStrSchema_t;

stringUnpickle( f, rs )
FILE * f;
basicStrSchema_t * rs;
{
    /*
    ** Do what we need to do to unpickle the type contents...
    */
    if (f) {
	int len = 0; /* the end-of-string */
	int totLen = 64;
	char * data = (char *)malloc( totLen );
	int q;
	int c;

	/* We're looking for the lead in quote. */
	while( (c = fgetc( f )) != EOF ) {
	    if ((c == '\'') || (c == '"')) {
		q = c; /* this is the quote we're looking for... */

		/* 
		** add characters till we reach the other quote.
		*/
		while( (c = fgetc( f )) != EOF ) {
		    if (c == q) {
			we_are_done:

			if (len+1 != totLen)
			    data = (char *)realloc( data, len+1 );
			data[len] = 0;
			rs->basicStr = data;
			return(1);

		    } else switch (c) {
			case '\\':		/* literal next */
			    c = fgetc( f );
			    if (c == EOF) {
				goto we_are_done;  /* not sure what else to do */
			    } else if (c == 't') { /* replace with tab. */
				c = '\t';
			    } else if (c == 'n') { /* replace with newline. */
				c = '\n';
			    }

			case '\n':		/* should we issue a warning? */
			default:
			    data[len++] = c;
			    if (len >= totLen) {
				totLen += 32;
				data = (char *)realloc( data, totLen );
			    }

		    }
		}
	    }
	}
	goto we_are_done;
    } else {
	return(0);
    }

}


/*
** ------------------------------
*/

typedef struct basicListSchema {
	ListMem *        basicList;
} basicListSchema_t ;

listUnpickle( f, rs )
FILE * f;
basicListSchema_t * rs;
{
    /*
    ** do what we need to do to unpickle the type contents...
    */
    if (f) {
	/*
	** For lists, we unpickle the next thing... the input
	** stream is annotated enough to allow us to figure
	** out what comes next.  When the return'ed thing
	** doesn't have a type associated, we know we've got
	** to the end of the list...
	**
	** Each call to unpickle either returns one new MemVal,
	** which we wrap in a ListVal, and add to the list we're
	** building, or it returns zero... which we'll use to
	** indicate the list is done.
	**
	*/
	extern MemVal * unpickle( FILE * );

	ListMem * l = 0; 	/* last element we've made */
	MemVal * m;

	rs->basicList = 0;

	while ( m = unpickle( f ) ) {
	    /* wrap `m' into a listval. */
	    ListMem * v = (ListMem *)calloc( 1, sizeof(*v) );
	    v->data = *m;
	    free(m);

	    /*
	    ** keep the list fifo.
	    */
	    if (rs->basicList == 0) {
		rs->basicList = v;
	    }

	    if (l) {
		l->next = v;
	    } else {
		l = v;
	    }
	}

	return(1);
    } else {
	return(0);
    }
}


/*
** ------------------------------
*/

typedef struct basicRecorderSchema {
	recorder_t *        basicRecorder;
} basicRecorderSchema_t ;

recorderUnpickle( f, rs )
FILE * f;
basicRecorderSchema_t * rs;
{
    /*
    ** The recorder type has several strings associated with it...
    ** representing what it takes to re-incarnate it.
    */
    if (f) {
	rs->basicRecorder = (recorder_t *)malloc( sizeof(*rs->basicRecorder) );
	memset( rs->basicRecorder, 0, sizeof(*rs->basicRecorder) ); /* LATER! */
	rs->basicRecorder->r_argv = crack(strdup("Do Something here"));
	return(1);
    } else {
	return(0);
    }
}

/*
** ------------------------------
*/

typedef struct basicSchemaData {
    schemaData_t *	basicSchemaData;
} basicSchemaData_t;

schemaDataUnpickle( f, rs )
FILE * f;
basicSchemaData_t * rs;
{
    /*
    ** We don't yet unpickle schemaData contents.
    ** <tho we'll make sure we get back what we wrote.
    */
    if (f) {
	char word[128];

	if (fscanf( f, "%s", word ) != 1) {
	    return(0);
	}
	if (strcmp( word, "<schemaData>" ) == 0) {
	    return(1);
	} else {
	    return(0);
	}
    } else {
	return(0);
    }
}

/*
** ------------------------------
*/

typedef struct basicIpv4HostSchema {
	struct in_addr        basicIpv4Host;
} basicIpv4HostSchema_t ;

ipaddrUnpickle( f, rs )
FILE * f;
basicIpv4HostSchema_t * rs;
{
    /*
    ** Read a word from the inbound stream. 
    ** Then convert it into an in_addr
    ** 
    */
    if (f) {
	char word[128];

	if (fscanf( f, "%s", word ) != 1) {
	    return(0);
	}
	/*
	** i'd prefer to use inet_aton, but some machines don't have it.
	*/
	{
	    int a,b,c,d;
	    if (sscanf( word, "%d.%d.%d.%d", &a, &b, &c, &d ) != 4) {
		return(1);
	    }
	}
	rs->basicIpv4Host.s_addr = inet_addr( word );
	return(1);
    } else {
	return(0);
    }
}

/*
** ------------------------------
*/

typedef struct basicIpv4NetSchema {
	ipv4net_t *        basicIpv4Net;
} basicIpv4NetSchema_t ;

ipnetUnpickle( f, rs )
FILE * f;
basicIpv4NetSchema_t * rs;
{
    /*
    ** Unpickle two host addresses... then combine them into one net address.
    */
    if (f) {
	basicIpv4HostSchema_t net;
	struct in_addr mask;

	if (ipaddrUnpickle(f, &net)) {
		return(0);
	}
	if (ipaddrUnpickle(f, &mask)) {
		return(0);
	}
	/*
	** combine the two to make an address.
	*/
	rs->basicIpv4Net = (ipv4net_t *)malloc( sizeof(*rs->basicIpv4Net) );
	rs->basicIpv4Net->ip_addr = net.basicIpv4Host;
	rs->basicIpv4Net->ip_mask = mask.s_addr;
	return(1);
    } else {
	return(0);
    }
}

/*
** ------------------------------
*/

typedef struct basicBlobSchema {
	blob_t *        basicBlob;
} basicBlobSchema_t;

blobUnpickle( f, rs )
FILE * f;
basicBlobSchema_t * rs;
{
    /*
    ** do what we need to do to unpickle the type contents...
    */
    if (f) {
	if (fscanf( f, "%d", &rs->basicBlob->b_len ) != 1) {
	    return(0);
	}
	base64Decode( f, rs->basicBlob->b_data, rs->basicBlob->b_len );
	return(1);
    } else {
	return(0);
    }
}

/*
** base64decoder.
** Read from 'f', we already know what length we expect...
** the storage for the data is already allocated...
*/

static char index_64[128] = {
    -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1,
    -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1,
    -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,62, -1,-1,-1,63,
    52,53,54,55, 56,57,58,59, 60,61,-1,-1, -1,-1,-1,-1,
    -1, 0, 1, 2,  3, 4, 5, 6,  7, 8, 9,10, 11,12,13,14,
    15,16,17,18, 19,20,21,22, 23,24,25,-1, -1,-1,-1,-1,
    -1,26,27,28, 29,30,31,32, 33,34,35,36, 37,38,39,40,
    41,42,43,44, 45,46,47,48, 49,50,51,-1, -1,-1,-1,-1
    };

base64Decode( f, data, len )
FILE * f;
char * data;
int len;
{
    /*
    ** actual characters to read are ((len+2)/3*4)
    */
    int tlen = ((len+2)/3)*4;
    int rlen = 0;

    while (tlen) {
	int a, b, c, d;
	int ai, bi, ci, di;
	int x, y, z;

	while( (a = getc( f )) != '\n' );
	while( (b = getc( f )) != '\n' );
	while( (c = getc( f )) != '\n' );
	while( (d = getc( f )) != '\n' );

	ai = (ai > 127)? -1 : index_64[a];
	bi = (bi > 127)? -1 : index_64[b];
	ci = (ci > 127)? -1 : index_64[c];
	di = (di > 127)? -1 : index_64[d];

	tlen -= 4;
	len -= 3;

	if ((ai == EOF) || (bi == EOF) || (ci == EOF) || (di == EOF)) {
	    /* unpickle failed. */
	    return(0);
	} else {
	    x = (ai << 2) | ((bi & 0x30) >> 4);
	    y = ((bi & 0xf) << 4) | ((ci & 0x3c) >> 2);
	    z = ((ci & 2) << 6) | (di);

	    switch(len) {
		case 0:
		    break;
		default:
		    data[2] = z;
		case 2:
		    data[1] = y;
		case 1:
		    data[0] = x;
		    break;
	    }
	}
    }
    return(0);

}

/*
** ------------------------------
** ethmacUnpickle.
**
** we'll use fscanf...
** like xx:xx:xx:xx:xx:xx.
*/

struct basicEthMacSchema {
	ethMac_t *      basicEthMac;
};

typedef struct basicEthMacSchema basicEthMacSchema_t;

ethmacUnpickle( f, rs )
FILE * f;
basicEthMacSchema_t * rs;
{
    /*
    ** we do the simple thing here.  We try to yank out
    ** the word, then try to convert it... we're looking
    ** for something like xx:xx:xx:xx:xx:xx
    **			  01234567890123456
    **			            1111111
    */
    if (f) {
	char word[128];

	if (fscanf( f, "%s", word ) != 1) {
	    return(0);
	}
	if ((ishex(word[0])) && (ishex(word[1])) && (word[2] == ':')
	&&  (ishex(word[3])) && (ishex(word[4])) && (word[5] == ':')
	&&  (ishex(word[6])) && (ishex(word[7])) && (word[8] == ':')
	&&  (ishex(word[9])) && (ishex(word[10])) && (word[11] == ':')
	&&  (ishex(word[12])) && (ishex(word[13])) && (word[14] == ':')
	&&  (ishex(word[15])) && (ishex(word[16])) ) {
		
	    rs->basicEthMac->e_addr[0] = 
		((makehex(word[0])) << 4) | (makehex(word[1]));
	    rs->basicEthMac->e_addr[1] = 
		((makehex(word[3])) << 4) | (makehex(word[4]));
	    rs->basicEthMac->e_addr[2] = 
		((makehex(word[6])) << 4) | (makehex(word[7]));
	    rs->basicEthMac->e_addr[3] = 
		((makehex(word[9])) << 4) | (makehex(word[10]));
	    rs->basicEthMac->e_addr[4] = 
		((makehex(word[12])) << 4) | (makehex(word[13]));
	    rs->basicEthMac->e_addr[5] = 
		((makehex(word[15])) << 4) | (makehex(word[16]));

	    return(1);
	} else {
	    return(0);
	}
    } else {
	return(0);
    }
    return(0); /* fauilure */
}

