/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/


#include <stdio.h>

#include "nfrd.h"

/*
** clone the second item, then add it to the existing list.
**
*/

int
blt_add( lm, mm )
MemVal * lm;		/* pointer to the list */
MemVal * mm;
{
    ListMem * lp = (ListMem *)lm->val.v;
    ListMem * np = (ListMem *)tmpcalloc(sizeof(ListMem)); /* new one */
    if (np == 0) {
	return(-1);
    }
    np->next = 0;

    memtmpvalclonedata( &(np->data), mm );

    /*
    ** link it in (at the end?)
    */
    if (lp == 0) {
	lm->val.v = np;
    } else {
	while (lp->next != 0) {
	    lp = lp->next;
	}
	lp->next = np;
    }
    return(0);

}


/*
** listadd.  First argument is a list, the rest of the arguments are the
** things to add to the list.  For sanity reasons, we  currently don't
** allow addition of list to lists.
**
** We build the new list with temporary mallocs... any assign will
** place them into appropriate memory...
*/
void
nfr_blt_listadd(Machine *m,int argc,int firstarg,int result)
{
	MemVal * lp = &m->m_mem[firstarg], *vp;

	if (lp->typ != NFR_TYPE_LIST) {
	    /*
	    ** allow the situation where the list isn't created yet.
	    */
	    if ((lp->typ != NFR_TYPE_ERR) && (lp->val.i != NFR_ERR_NOVAR)) {
		m->m_mem[result].typ = NFR_TYPE_ERR;
		m->m_mem[result].val.i = NFR_ERR_USER;
		return;
	    } else {
		/*
		** Create a new null list head...
		*/
		if ((vp = (MemVal *)tmpcalloc( sizeof(*vp) )) == 0) {
		    m->m_mem[result].typ = NFR_TYPE_ERR;
		    m->m_mem[result].val.i = NFR_ERR_OOM;
		    return;
		}
		vp->typ = NFR_TYPE_LIST;
		vp->val.i = 0;
	    }
	} else {
	    /*
	    ** Clone the existing list, in preparation for addition of new entries.
	    */
	    if ((vp = (MemVal *)tmpcalloc( sizeof(*vp) )) == 0) {
		m->m_mem[result].typ = NFR_TYPE_ERR;
		m->m_mem[result].val.i = NFR_ERR_OOM;
		return;
	    }
	    if (memtmpvalclonedata( vp, lp ) != 0) {
		m->m_mem[result].typ = NFR_TYPE_ERR;
		m->m_mem[result].val.i = NFR_ERR_OOM;
		return;
	    }
	}


	firstarg++;
	argc--;

	while(argc--) {
		if (blt_add( vp, &m->m_mem[firstarg] ) != 0) {
		    m->m_mem[result].typ = NFR_TYPE_ERR;
		    m->m_mem[result].val.i = NFR_ERR_OOM;
		    return;
		}
		firstarg++;
	}

	/* echo does not return a value. put a NULL on the stack */
	m->m_mem[result].typ = NFR_TYPE_LIST;
	m->m_mem[result].val.v = vp->val.v;
}
