/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/

#include <stdio.h>

#include "nfrd.h"

/*
** alert.
**
*/

Filter alertFilter;	/* just to hang the sym off of */

#ifndef DEFAULT_ALERTS_NFR_MAP
#define	DEFAULT_ALERTS_NFR_MAP	"~/etc/resources/nfr/res.map"
#endif

#ifndef DEFAULT_ALERTS_USR_MAP
#define DEFAULT_ALERTS_USR_MAP	"~/etc/resources/usr/res.map"
#endif


/*
** ------------------------------
** alertAddSym.
** used by the callout to iterate across configuration file.
**
** Find the resource files in ~nfr/etc/resources/nfr/res.map
** and  ~nfr/etc/resources/usr/res.map.  Use Cf_read to populate
** structures which we will then walk across all the values
** and create a Sym * which contains them.  The user can then
** reference those map values for the alerts ...
*/
int alertAddSym( TNVPair * pair , void * rock )
{
    Filter * af = (Filter *)rock;
    MemVal mem;

    mem.typ = NFR_TYPE_INT;
    mem.val.i = atoi(pair->Val);

    run_assign( (Filter *)rock, pair->Name, NFR_OP_GASGN, &mem );
}


/*
** ------------------------------
** nfr_blt_alertInit.
**
** initialize alerts.
**
**
*/

nfr_blt_alertInit()
{
    TCf * cp;
    char * p;

    cp = Cf_New(0);
    if (Cf_Read( p = nfrPath( DEFAULT_ALERTS_NFR_MAP, 0 ), cp ) != NFR_ERR_SUCCESS) {
	Alert_Args_Generate( NFR_SRC_NFRD, NFR_ERR_CANTOPENFILE,
		DEFAULT_ALERTS_NFR_MAP, 0 );
    }
    free(p);

    if (Cf_Read( p = nfrPath( DEFAULT_ALERTS_USR_MAP, 0 ), cp ) != NFR_ERR_SUCCESS ) {
	Alert_Args_Generate( NFR_SRC_NFRD, NFR_ERR_CANTOPENFILE,
		DEFAULT_ALERTS_USR_MAP, 0 );
    }
    free(p);

    alertFilter.f_scope = "alert";
    alertFilter.f_scopeNdx = strHash(alertFilter.f_scope);

    /*
    ** iterate through all the names..
    */
    Cf_ForEach( alertAddSym, &alertFilter, cp );

    Cf_Delete(cp);
}


/*
** As many parameters as the user wants.  The first
** two values are 
**
** alert( int, int, <as many values as the user wants> );
**
** We'll convert some of the values (as many as we can)
** into strings before we pass them onto the alert daemon.
**
*/

void
nfr_blt_alert(Machine *m,int argc,int firstarg,int result)
{
    if (argc < 2) {
	userError:  
	m->m_mem[result].typ = NFR_TYPE_ERR;
	m->m_mem[result].val.i = NFR_ERR_USER;
	return;
    } else {
	/*
	** The first two arguments are integers. 
	** The rest ought to be strings.
	** Someday we can convert all the types into strings.
	*/
	MemVal * first = &m->m_mem[firstarg];
	MemVal * second = &m->m_mem[firstarg+1];
	int many = argc - 2 /* requred args */ + 1 /* zero terminator */;  
	char ** alertArgs;

	if (first->typ != NFR_TYPE_INT || second->typ != NFR_TYPE_INT) {
	    goto userError;
	}

	alertArgs = (char **)tmpcalloc( sizeof(*alertArgs)*many );
	alertArgs[many-1] = 0;
	{
	    int i;
	    for ( i = 2; i<argc; ++i) {
		MemVal * arg = &m->m_mem[firstarg+i];
		switch( arg->typ ) {
		    case NFR_TYPE_STR:
			alertArgs[i-2] = arg->val.c;
			break;

		    case NFR_TYPE_INT:
			{
			    char buff[64];
			    char * bp;

			    sprintf( buff, "%d", arg->val.i );
			    bp = (char *)tmpalloc( strlen(bp) + 1 );
			    strcpy( bp, buff );
			    alertArgs[i-2] = bp;
			}
			break;

		    default:
			{
			}
		}
	    }
	}
	if (VAlert_Args_Generate( first->val.i, second->val.i, alertArgs ) != NFR_ERR_SUCCESS) {
	    log( FL, "alert: can't send alert.\n" );
	}

	return;
    }
}
