/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/

#include <stdio.h>

#include "nfrd.h"


void
blt_echo( mp )
MemVal * mp;
{
    switch(mp->typ) {
	    case	NFR_TYPE_STR:

		{
		char * s = mp->val.c;
		if (s) {
		    /* if (strchr( mp->val.c, '"')) XXX maybe XXX */
		    if (0) {
			/*
			** if we have an imbedded quote, show the string.
			*/
			uiCurrPrintf("\"");
			while (s[0]) {
			    if (s[0] == '"') {
				uiCurrPrintf( "\\\"" );
			    } else {
				uiCurrPrintf( "%c", s[0] );
			    }
			    s++;
			}
			uiCurrPrintf("\"");
		    } else {
			uiCurrPrintf( "%s", mp->val.c );
		    }
		} else {
		    uiCurrPrintf("<null-string>" );
		}
		}
	    break;

	    case	NFR_TYPE_INT:
	    uiCurrPrintf("%d",mp->val.i);
	    break;

	    case	NFR_TYPE_LIST:
	    {
		ListMem * lp = mp->val.v;
		int any = 0;

		uiCurrPrintf( "[" );
		while (lp) {
		    if (any++)
			uiCurrPrintf( "," );
		    blt_echo( &lp->data );
		    lp = lp->next;
		}
		uiCurrPrintf( "]" );
	    }
	    break;

	    case	NFR_TYPE_RECORDER:
	    {
	    extern char * recorderName();
	    uiCurrPrintf("recorder %x=<%s>",
		(unsigned int)mp->val.v, recorderName (mp->val.v) );
	    }
	    break;

	    case	NFR_TYPE_SCHEMADATA:
	    {
	    uiCurrPrintf("schema<%x>", (unsigned int)mp->val.v );
	    }
	    break;

	    case	NFR_TYPE_IPV4HOST:
	    {
	    uiCurrPrintf("host(%s)", inet_ntoa( mp->val.h ) );
	    }
	    break;

	    case	NFR_TYPE_IPV4NET:
	    {
	    struct in_addr mask;
	    mask.s_addr = mp->val.n->ip_mask;

		/* see if we can display this in cidr style notation. */
		{
		    int i = 32;
		    while (i--) {
			if (mp->val.n->ip_mask == htonl(0xffffffff << i)) {
			    uiCurrPrintf( "net(%s/%d)", 
				    inet_ntoa( mp->val.n->ip_addr ), 32-i );
			    break;
			}
		    }
		    if (i == -1) {
			uiCurrPrintf("net(%s", inet_ntoa( mp->val.n->ip_addr ) );
			uiCurrPrintf(":%s)", inet_ntoa( mask ) );
		    }
		}
	    }
	    break;

	    case	NFR_TYPE_BLOB:
	    dump( 0, mp->val.b->b_data, mp->val.b->b_len );
	    break;

	    case	NFR_TYPE_ETHMAC:
	    displayEnetMac( &mp->val.e );
	    break;

	    case	NFR_TYPE_ERR:
	    uiCurrPrintf("error(%s)", util_err2str(mp->val.i) );
	    break;

	    case	NFR_TYPE_PATTERN:
	    uiCurrPrintf("pattern(%s)", mp->val.p->p_str );
	    break;

	    case	NFR_TYPE_TRIGGER:
	    {
		int i;
		trigger_t * t = mp->val.t;

		uiCurrPrintf( "trigger: <" );
		for (i = 0; i<t->t_ndx; ++i) {
		    int many = 0;
		    int sep = 0;
		    Sym * l;

		    if (i) {
			uiCurrPrintf( ", " );
		    }
		    uiCurrPrintf( "%s: ", t->t_whom[i] );
		    for( l = t->t_nv[i]; l ; l = l->s_next ) {
			MemVal * m = &l->s_data;
			if (many == 0) {
			    uiCurrPrintf( "(" );
			}
			if (sep) {
			    uiCurrPrintf( ", " );
			}
			uiCurrPrintf( "%s: ", symName(l) );
			blt_echo( m );
			many++;
			sep++;
		    }
		    if (many) {
			uiCurrPrintf( ") " );
		    }
		}
		uiCurrPrintf( ">" );
	    }
	    break;

	    case	NFR_TYPE_MAP:
		uiCurrPrintf( "map. %x\n", mp->val.v );
		break;

	    case	NFR_TYPE_MAPBAG:
		{
		    MapBag * mb = (MapBag *)mp->val.v;
		    if (mb) {
			int mbi;
			for (mbi=0; mbi<mb->mb_width; ++mbi) {
			    Map * map = mb->mb_maps[mbi];
			    while (map) {
				Sym * sym = map->m_sym;
				while (sym) {
				    /*
				    ** first print the index.
				    */
				    uiCurrPrintf( "\n" );
				    switch (SYM_FLAG_TYPE(sym)) {
					case SYM_FLAG_TYPE_INT:
					    uiCurrPrintf( "[%d] ", sym->s_int ); 
					    break;
					case SYM_FLAG_TYPE_NAME:
					    uiCurrPrintf( "[%s] ", symName(sym) ); 
					    break;
					case SYM_FLAG_TYPE_HOST:
					    uiCurrPrintf( "[%s] ",
						util_ntoa( sym->s_host.s_addr ) );
					    break;
					default:
					    uiCurrPrintf( "[" );
					    blt_echo( sym->s_memVal );
					    uiCurrPrintf( "] " );
						
				    }
				    /*
				    ** now print the value 
				    ** note: we don't recurse for
				    ** array's within arrays, since 
				    ** we can quickly cause infinite
				    ** recursion.
				    */
				    if (sym->s_data.typ == NFR_TYPE_MAPBAG) {
					uiCurrPrintf( "[array]" );
				    } else {
					blt_echo( &sym->s_data );
				    }
				    sym = sym->s_next;
				}
				map = map->m_next;
			    }
			}
		    }
		}
		break;

	    default:
	    uiCurrPrintf("error(%d,%d)", mp->val.i, mp->typ );
	    break;
    }
}


/*
** echo arguments. some operator overloading is done to handle
** ints/strings intelligently.
*/
void
nfr_blt_echo(Machine *m,int argc,int firstarg,int result)
{
	while(argc--) {
		blt_echo( &m->m_mem[firstarg] );
		firstarg++;
	}

	/* echo does not return a value. put a NULL on the stack */
	m->m_mem[result].typ = NFR_TYPE_ERR;
	m->m_mem[result].val.i = NFR_ERR_USER;
}
