/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/


#include <stdio.h>

#include "nfrd.h"

/*
** maptosym.
**
** Takes two parameters, the first must be a MAPBAG, the second an index.
** We look up the MAP for that index inside the MAPBAG, and return the
** Map we find.
**
*/
void
nfr_blt_maptosym(Machine *m,int argc,int firstarg,int result)
{
	MemVal * mp = &m->m_mem[firstarg]; /* mapbag */
	MemVal * ip = &m->m_mem[firstarg+1]; /* index */
	char * name;
	int index;

	if ((mp->typ != NFR_TYPE_MAPBAG)
	|| ((ip->typ != NFR_TYPE_INT) && (ip->typ != NFR_TYPE_STR)) ) {
		Alert_Args_Generate( NFR_SRC_NFRD, NFR_NFRD_MAPTOSYMERR,
			util_type2str(mp->typ), util_type2str(mp->typ), 0 );
		m->m_mem[result].typ = NFR_TYPE_ERR;
		m->m_mem[result].val.i = NFR_ERR_USER;
		return;
	}
	switch(ip->typ) {
	    case NFR_TYPE_INT:
		index = ip->val.i;
		break;
	    case NFR_TYPE_STR:
		name = ip->val.c;
		index = strHash(name);
		break;
	}

	{
	    MapBag * mb = mp->val.v;
	    Map * map = mapBagFind( mb, index, name );
	    if (map == 0)  {
		/* create one */
		map = mapNew( 0, index, name, 0 );
		mapBagAdd( mb, map );
	    }
	    m->m_mem[result].typ = NFR_TYPE_MAP;
	    m->m_mem[result].val.v = map;

	}

	return;
}
