/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/

#include "nfrd.h"

/*
** regex...
**
** home of regcomp, and rexexec.
**
**
** there's other stuff here to help integrate NFR_TYPE_PATTERN
** into the run environment.
**
*/

pattern_t *
patternNew( str )
char * str;
{
    pattern_t * new = (pattern_t *)malloc( sizeof(*new ) );
    /* Assert new XXX */
    if (new == 0) {
	return(0);
    }
    new->p_str = strdup(str);
    if (regcomp( &new->p_regex, new->p_str, REG_EXTENDED|REG_NOSUB ) ) {
	if (new->p_str) {
	    free(new->p_str);
	    new->p_str = 0;
	}
	free(new);
	return(0);
    }
    return(new);
}

pattern_t *
patternTmpNew( str )
char * str;
{
    pattern_t * new = (pattern_t *)tmpcalloc( sizeof(*new) );
    /* Assert new XXX */
    if (new == 0) {
	return(0);
    }
    new->p_str = tmpstrdup(str);
    if (regcomp( &new->p_regex, new->p_str, REG_EXTENDED|REG_NOSUB ) ) {
	return(0);
    }
    return(new);
}


void
patternFree( p )
pattern_t * p;
{
    if(p->p_str) {
	p->p_str = 0;
	free(p->p_str);
    }
    free(p);
}

/*
** Pass in two memVals, one must be a pattern, one must be a string.
** return 1 for match, 0 for not...
*/
patternMatch( MemVal * a, MemVal * b )
{
    pattern_t * p = 0;
    char * s = 0;
    if (a->typ == NFR_TYPE_STR) {
	s = a->val.c;
    }
    if (b->typ == NFR_TYPE_STR) {
	s = b->val.c;
    }
    if (a->typ == NFR_TYPE_PATTERN) {
	p = a->val.p;
    }
    if (b->typ == NFR_TYPE_PATTERN) {
	p = b->val.p;
    }
    if ((s == 0) || (p == 0)) {
	return(0);
    }
    if (regexec( &p->p_regex, s, 0, 0, 0 ) == 0) {
	/* matched */
	return(1);
    } else {
	/* didn't match */
	return(0);
    }
}

/*
** bltin interface to regcomp.
** builds a NFR_TYPE_PATTERN.
*/

void
nfr_blt_regcomp(Machine *m,int argc,int firstarg,int result)
{

    if ((argc != 1) && ( m->m_mem[firstarg].typ != NFR_TYPE_PATTERN)) {
	userError: 
	m->m_mem[result].typ = NFR_TYPE_ERR;
	m->m_mem[result].val.i = NFR_ERR_USER;
	return;
    }

    m->m_mem[result].typ = NFR_TYPE_PATTERN;
    m->m_mem[result].val.p = patternTmpNew( m->m_mem[firstarg].val.c );
    if (m->m_mem[result].val.p == 0) {
	    goto userError;
    }
    return;
}

/*
** bltin interface to regexec.
** 
*/

void
nfr_blt_regexec(Machine *m,int argc,int firstarg,int result)
{

    if ((argc < 2)
    && (m->m_mem[firstarg].typ != NFR_TYPE_PATTERN)
    && (m->m_mem[firstarg+1].typ != NFR_TYPE_STR)) {
	userError: 
	m->m_mem[result].typ = NFR_TYPE_ERR;
	m->m_mem[result].val.i = NFR_ERR_USER;
	return;
    }

    if (patternMatch( &m->m_mem[firstarg], &m->m_mem[firstarg+1] )) {
	/* matched */
	m->m_mem[result].typ = NFR_TYPE_INT;
	m->m_mem[result].val.i = 1;
    } else {
	/* failed */
	m->m_mem[result].typ = NFR_TYPE_INT;
	m->m_mem[result].val.i = 0;
    }
    return;
}
