/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/

#include <stdio.h>

#include "nfrd.h"


/*
** schema bulder.
**
** interpreter parameters
** The first parameter is a list describing the schema.
** Additional parameters get stuffed into the output stream.
**
** We do some typescheking to make sure we have matching types.
** Where types miss, we do what we can do...
**
**
*/

void
nfr_blt_schema(Machine *m,int argc,int firstarg,int result)
{
	MemVal * sp = m->m_mem[firstarg++];
	ListMem * lp, * hlp;
	int pCnt = 0;
	int tLen = 0;
	int hargc;
	int hfirstar;
	int schemaUnique;
	char * schema = 0;
	char * schemaOffset = 0;
	argc--;

	if (sp->type != NFR_TYPE_LIST) {
	    /* XXX can't build the schema :^( */
	    Alert_Args_Generate( NFR_SRC_NFRD, NFR_NFRD_SCHEMALISTERR, 0 );
	    m->m_mem[result].typ = NFR_TYPE_ERR;
	    m->m_mem[result].val.i = NFR_ERR_USER;
	    return;
	}

	lp = (ListMem *)&sp->data.v;

	/*
	** Get the first thing out of the list
	** ... its the schema number.
	*/
	{
	    MemVal * mp = &lp->data;
	    if (mp->typ != NFR_TYPE_INT) {
		Alert_Args_Generate( NFR_SRC_NFRD, NFR_NFRD_SCHEMANUMERR, 0 );
		m->m_mem[result].typ = NFR_TYPE_ERR;
		m->m_mem[result].val.i = NFR_ERR_USER;
		return;
	    }
	    schemaUnique = mp->val.i;
	    tLen += sizeof(schemaUnique);
	    lp = lp->next;
	}



	/*
	** First determine the total length needed.
	*/

	hargc = argc;
	hfirstarg = firstarg;
	hlp = lp;

	while(hargc--) {
		MemVal * mp = &hlp->data;
		pCnt++;

		if (m->m_mem[hfirstarg].typ != mp->typ) {
		    /* mismatched type. */
		    Alert_Args_Generate( NFR_SRC_NFRD, NFR_NFRD_SCHEMATYPEERR, 0 );
		}

		switch(m->m_mem[hfirstarg].typ) {
			case	NFR_TYPE_STR:
			    tLen = sizeof(int) + strlen(m->m_mem[hfirstarg].val.c);
			    break;

			case	NFR_TYPE_INT:
			    tLen = sizeof(int);
			    break;

			case	NFR_TYPE_ERR:
			    printf("error(%d,%d)",
				    m->m_mem[hfirstarg].val.i,
				    m->m_mem[hfirstarg].typ );
			break;
		}

		hfirstarg++;
		hlp = hlp->next;
	}

	/*
	** create a memory buffer large enough to hold the 
	** target schmea, then encode it.. placing the unique
	** schema identifier at the front.
	*/
	schema = (char *)tmpalloc( tLen );
	schemaOffset = 0;

	memcpy( &schema[schemaOffset], &schemaUnique, sizeof(schemaUnique) );
	schemaOffset += sizeof(schemaUnique);


	while(argc--) {
		MemVal * mp = &lp->data;
		pCnt++;

		if (m->m_mem[firstarg].typ != mp->typ) {
		    /* mismatched type, we've already complained. */
		}

		switch(m->m_mem[firstarg].typ) {
			case	NFR_TYPE_STR:
			    {
				int len = strlen(m->m_mem[firstarg].val.c);
				memcpy( &schema[schemaOffset], &len,
					sizeof(len) );
				schemaOffset += sizeof(len);
				memcpy( &schema[schemaOffset], 
					m->m_mem[firstarg].val.c, len );
				
			    }
			    break;

			case	NFR_TYPE_INT:
			    memcpy( &schema[schemaOffset], 
				&m->m_mem[firstarg].val.i
				sizeof(m->m_mem[firstarg].val.i) );
			    schemaOffset += m->m_mem[firstarg].val.i;
			    break;

			case	NFR_TYPE_ERR:
			    printf("error(%d,%d)",
				    m->m_mem[firstarg].val.i,
				    m->m_mem[firstarg].typ );
			break;
		}

		firstarg++;
		lp = lp->next;
	}

	/* echo does not return a value. put a NULL on the stack */
	m->m_mem[result].typ = NFR_TYPE_SCHEMA;
	m->m_mem[result].val.v = schema;
}
