/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/

#include <stdio.h>

#include "nfrd.h"

/*
** nfr_blt_strlen
**
** If the parameter is a string, then return its length.
** 0 means zero length string.
**
** strlen( parm )
**
** Additional parameters are silently ignored... XXX
**
*/
void
nfr_blt_strlen(Machine *m,int argc,int firstarg,int result)
{
	MemVal * sp = &m->m_mem[firstarg]; /* string */

	if ((sp->typ != NFR_TYPE_STR) && (sp->typ != NFR_TYPE_BLOB)) {
		m->m_mem[result].typ = NFR_TYPE_ERR;
		m->m_mem[result].val.i = NFR_ERR_USER;
		return;
	}


	if (sp->typ == NFR_TYPE_STR) {
	    if (sp->val.c) {
		m->m_mem[result].val.i = strlen(sp->val.c);
	    } else {
		m->m_mem[result].val.i = 0;
	    }
	} else
	if (sp->typ == NFR_TYPE_BLOB) {
	    if (sp->val.b) {
		m->m_mem[result].val.i = sp->val.b->b_len;
	    } else {
		m->m_mem[result].val.i = 0;
	    }
	}
	m->m_mem[result].typ = NFR_TYPE_INT;
}


/*
** nfr_blt_strcat
**
** Cat's together the parameters.   Returns the concatenated string.
** allows integers as well as strings.  Allows lists to be strcat'd
** together as well.
**
** strcat( parm, ... )
**
**
*/

void
nfr_blt_strcat(Machine *m,int argc,int firstarg,int result)
{
    int argi;
    int len = 0;

    for ( argi = 0; argi < argc; ++argi ) {
	MemVal * mem = &m->m_mem[firstarg+argi];
	switch ( mem->typ ) {
	    case NFR_TYPE_STR:
		/* only add a length if there's a string really there */
		if (mem->val.c) {
		    len += strlen(mem->val.c);
		}
		break;

	    case NFR_TYPE_BLOB:
		if (mem->val.b) {
		    len += mem->val.b->b_len;
		}
		break;

	    case NFR_TYPE_INT:
		{
		    char buff[64];
		    sprintf( buff, "%d", mem->val.i );
		    len += strlen(buff);
		}
		break;

	    case NFR_TYPE_IPV4HOST:
		{
		    char buff[64];
		    sprintf( buff, "%s", util_ntoa( mem->val.h.s_addr ) );
		    len += strlen(buff);
		}
		break;

	    case NFR_TYPE_IPV4NET:
		{
		    struct in_addr mask;
		    char buff[64];

		    mask.s_addr = mem->val.n->ip_mask;

		    /* see if we can display this in cidr style notation. */
		    {
			int i = 32;
			while (i--) {
			    if (mem->val.n->ip_mask == htonl(0xffffffff << i)) {
				sprintf( buff, "%s/%d", 
					inet_ntoa( mem->val.n->ip_addr ), 32-i );
				break;
			    }
			}
			if (i == -1) {
			    sprintf(buff, "%s:", inet_ntoa( mem->val.n->ip_addr ) );
			    strcat( buff, util_ntoa( mem->val.n->ip_mask ) );
			}
		    }
		    len += strlen(buff);
		}
		break;

	    case NFR_TYPE_ETHMAC:
		{
		    char buff[64];
		    sprintf( buff, "%2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x",
			mem->val.e.e_addr[0],
			mem->val.e.e_addr[1],
			mem->val.e.e_addr[2],
			mem->val.e.e_addr[3],
			mem->val.e.e_addr[4],
			mem->val.e.e_addr[5] );
		    len += strlen(buff);
		}
		break;

	    case NFR_TYPE_LIST:
		len += liststrlen( mem->val.v );
		break;

	    default:
		break;
	}
    }
    /*
    ** tmpalloc the string's memory, then set it up.
    ** if the result gets used in an assignment, it will be malloced for real
    */
    {
	char * new = (char *)tmpalloc( len+1 );
	char * t = new;
	if (new == 0) {
	    m->m_mem[result].typ = NFR_TYPE_ERR;
	    m->m_mem[result].val.i = NFR_ERR_OOM;
	    return;
	} else {
	    for ( argi = 0 ; argi < argc; ++argi ) {
		MemVal * mem = &m->m_mem[firstarg+argi];
		switch ( mem->typ ) {
		    case NFR_TYPE_STR:
			if (mem->val.c) {
			    char * f = mem->val.c;
			    if (f) {
				while( *f ) {
				    *t++ = *f++;
				}
			    }
			}
			break;

		    case NFR_TYPE_BLOB:
			if (mem->val.b) {
			    char * f = mem->val.b->b_data;
			    int mLen = mem->val.b->b_len;

			    if (f) {
				while( *f && (mLen--) ) {
				    /* XXX only isprint(*f)? */
				    *t++ = *f++;
				}
			    }
			}
			break;

		    case NFR_TYPE_INT:
			{
			    char buff[64], *f;
			    sprintf( buff, "%d", mem->val.i );
			    f = buff;
			    while( *f ) {
				*t++ = *f++;
			    }
			}
			break;

		    case NFR_TYPE_LIST:
			t = liststrcat( mem->val.v, t );
			break;


		    case NFR_TYPE_IPV4HOST:
			{
			    char buff[64], *f;
			    sprintf( buff, "%s", util_ntoa( mem->val.h.s_addr ) );
			    f = buff;
			    while( *f ) {
				*t++ = *f++;
			    }
			}
			break;

		    case NFR_TYPE_IPV4NET:
			{
			    struct in_addr mask;
			    char buff[64];

			    mask.s_addr = mem->val.n->ip_mask;

			    /* see if we can display this in cidr style notation. */
			    {
				int i = 32;
				while (i--) {
				    if (mem->val.n->ip_mask == htonl(0xffffffff << i)) {
					sprintf( buff, "%s/%d", 
						inet_ntoa( mem->val.n->ip_addr ), 32-i );
					break;
				    }
				}
				if (i == -1) {
				    sprintf(buff, "%s:", inet_ntoa( mem->val.n->ip_addr ) );
				    strcat( buff, util_ntoa( mem->val.n->ip_mask ) );
				}
			    }
			    {
				char* f = buff;
				while( *f ) {
				    *t++ = *f++;
				}
			    }
			}
			break;

		    case NFR_TYPE_ETHMAC:
			{
			    char buff[64], *f;
			    sprintf( buff, "%2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x",
				mem->val.e.e_addr[0],
				mem->val.e.e_addr[1],
				mem->val.e.e_addr[2],
				mem->val.e.e_addr[3],
				mem->val.e.e_addr[4],
				mem->val.e.e_addr[5] );
			    f = buff;
			    while( *f ) {
				*t++ = *f++;
			    }
			}
			break;

		    default:
			break;
		}
	    }
	}
	*t = 0; /* null terminator */
	m->m_mem[result].typ = NFR_TYPE_STR;
	m->m_mem[result].val.c = new;

    }
}
/*
** nfr_blt_blobcat
**
** Cat's together the parameters.   Returns the concatenated blob.
** allows strings as well as blobs.  Allows lists to be strcat'd
** together as well... ignores ints, and other binary types.
**
** blobcat( parm, ... )
**
**
*/

void
nfr_blt_blobcat(Machine *m,int argc,int firstarg,int result)
{
    int argi;
    int len = 0;

    for ( argi = 0; argi < argc; ++argi ) {
	MemVal * mem = &m->m_mem[firstarg+argi];
	switch ( mem->typ ) {
	    case NFR_TYPE_BLOB:
		if (mem->val.b) {
		    len += mem->val.b->b_len;
		}
		break;

	    case NFR_TYPE_STR:
		/* only add a length if there's a string really there */
		if (mem->val.c) {
		    len += strlen(mem->val.c);
		}
		break;

	    case NFR_TYPE_LIST:
		len += listbloblen( mem->val.v );
		break;

	    default:
		break;
	}
    }
    /*
    ** We make a new blob, but set up the callback to release
    ** the blob when tmpfree is called.
    */
    {
	blob_t * b = blobNew( 0, len, 0 );
	char * t = b->b_data;

	tmpCallOnFree( blobDecRef, b );

	if ((b == 0) || (b->b_data == 0)) {
	    m->m_mem[result].typ = NFR_TYPE_ERR;
	    m->m_mem[result].val.i = NFR_ERR_OOM;
	    return;
	} else {
	    for ( argi = 0 ; argi < argc; ++argi ) {
		MemVal * mem = &m->m_mem[firstarg+argi];
		switch ( mem->typ ) {
		    case NFR_TYPE_BLOB:
			{
			    blob_t * b = mem->val.b;
			    if (b) {
				if (b->b_len) {
				    memcpy( t, b->b_data, b->b_len );
				    t += b->b_len;
				}
			    }
			}
			break;

		    case NFR_TYPE_STR:
			{
			    char * f = mem->val.c;
			    if (f) {
				while( *f ) {
				    *t++ = *f++;
				}
			    }
			}
			break;


		    case NFR_TYPE_LIST:
			{
			    t = listblobcat( mem->val.v, t );
			}
		    default:
			break;
		}
	    }
	}
	m->m_mem[result].typ = NFR_TYPE_BLOB;
	m->m_mem[result].val.b = b;

    }
}

/*
** nfr_blt_strndx
**
** strndx( parm, n )
** return a string starting at index <n>.  <n> is C style, first
** character is 0, last is strlen(parm)-1.
**
** strndx( parm, n, l )
**
** return a string starting at index <n> for <l> characters.
** length of the string truncates the length of the new string.
**
** incorrect number of parameters returns user errro, as does
** the first parameter not being a string, and the second and
** third parameters not being an int.
**
** The first parameter may also be a blob, in which case we 
** perform a conversion between a blob an a string.  In the
** case of blob's we hafta do some stuff a little different...
** for example we'll skip imbedded null's starting at index,
** but we'll copy till we either hit a null, or meet the users
** length, or run out of blob chars.
**
*/

void
nfr_blt_strndx(Machine *m,int argc,int firstarg,int result)
{
    MemVal * v;
    int index = 0;
    int len = 0x7fffffff; /* very high int */

    if ((argc < 1) ||  (argc > 3)) {
	userError:
	m->m_mem[result].typ = NFR_TYPE_ERR;
	m->m_mem[result].val.i = NFR_ERR_USER;
	return;
    }
    /*
    ** set up the index.
    */
    v = &m->m_mem[firstarg+1]; /* index */
    if (argc > 1) {
	if (v->typ != NFR_TYPE_INT) {
	    goto userError;
	}
	index = v->val.i;
    }

    if (argc == 3) {
	v =  &m->m_mem[firstarg+2]; /* length */
	if (v->typ != NFR_TYPE_INT) {
	    goto userError;
	}
	len = v->val.i;
	if (len < 0) {
	    len = 0;
	}
    }

    if (index < 0) {
	index = 0;
    }
    /*
    ** tmpalloc the string's memory. Note that we don't worry
    ** about allocing the string too long, if we have extra char's
    ** afterwards, it ought to not harm anything (kinda :^) )
    */
    v = &m->m_mem[firstarg]; /* string/blob */
    if ((v->typ == NFR_TYPE_STR) || (v->typ == NFR_TYPE_BLOB)) {
	char * new;
	int slen = 0;

	if (v->typ == NFR_TYPE_STR) {
	    if (v->val.v) {
		slen = strlen(v->val.c); 
	    }
	} else 
	if (v->typ == NFR_TYPE_BLOB) {
	    if (v->val.b) {
		slen = v->val.b->b_len;
	    }
	}
	if (index > slen) {
	    index = slen;
	}

	if (slen-index < len) {
	    len = slen - index;
	}

	if ((new = (char *)tmpalloc( len+1 )) == 0) {
	    m->m_mem[result].typ = NFR_TYPE_ERR;
	    m->m_mem[result].val.i = NFR_ERR_OOM;
	    return;
	} else {
	    char * t = new;
	    char * f;

	    if (v->typ == NFR_TYPE_STR) {
		f = &v->val.c[index];
	    } else 
	    if (v->typ == NFR_TYPE_BLOB) {
		if (v->val.b) {
		    f = &v->val.b->b_data[index];
		} else {
		    f = "";
		}
	    }

	    /*
	    ** copy the chars.
	    */
	    if ((len > 0) && *f) {
		while( len-- && *f ) {
		    *t++ = *f++;
		}
	    }

	    *t = 0; /* null terminator */
	    m->m_mem[result].typ = NFR_TYPE_STR;
	    m->m_mem[result].val.c = new;
	}
    } else {
	goto userError;
    }
}

/*
** nfr_blt_blobndx
**
** blobndx( parm, n )
** return a blob starting at index <n>.  <n> is C style, first
** character is 0, last is strlen(parm)-1.
**
** blobndx( parm, n, l )
**
** return a blob starting at index <n> for <l> characters.
** length of the blob truncates the length of the new blob.
**
** incorrect number of parameters returns user errro, as does
** the first parameter not being a string, and the second and
** third parameters not being an int.
**
** The first parameter may also be a blob, in which case we 
** perform a conversion between a blob an a string.  In the
** case of blob's we hafta do some stuff a little different...
** for example we'll skip imbedded null's starting at index,
** but we'll copy till we either hit a null, or meet the users
** length, or run out of blob chars.
**
*/

void
nfr_blt_blobndx(Machine *m,int argc,int firstarg,int result)
{
    MemVal * v;
    int index = 0;
    int len = 0x7fffffff; /* very high int */

    if ((argc < 1) ||  (argc > 3)) {
	userError:
	m->m_mem[result].typ = NFR_TYPE_ERR;
	m->m_mem[result].val.i = NFR_ERR_USER;
	return;
    }
    /*
    ** set up the index.
    */
    v = &m->m_mem[firstarg+1]; /* index */
    if (argc > 1) {
	if (v->typ != NFR_TYPE_INT) {
	    goto userError;
	}
	index = v->val.i;
    }

    if (argc == 3) {
	v =  &m->m_mem[firstarg+2]; /* length */
	if (v->typ != NFR_TYPE_INT) {
	    goto userError;
	}
	len = v->val.i;
	if (len < 0) {
	    len = 0;
	}
    }

    if (index < 0) {
	index = 0;
    }
    /*
    ** tmpalloc the string's memory. Note that we don't worry
    ** about allocing the string too long, if we have extra char's
    ** afterwards, it ought to not harm anything (kinda :^) )
    */
    v = &m->m_mem[firstarg]; /* string/blob */
    if ((v->typ == NFR_TYPE_STR) || (v->typ == NFR_TYPE_BLOB)) {
	blob_t * bn;
	char * new;
	int slen = 0;

	if (v->typ == NFR_TYPE_STR) {
	    if (v->val.v) {
		slen = strlen(v->val.c); 
	    }
	} else 
	if (v->typ == NFR_TYPE_BLOB) {
	    if (v->val.b) {
		slen = v->val.b->b_len;
	    }
	}
	if (index > slen) {
	    index = slen;
	}

	if (slen-index < len) {
	    len = slen - index;
	}

	{
	    char * f;

	    if (v->typ == NFR_TYPE_STR) {
		f = &v->val.c[index];
	    } else 
	    if (v->typ == NFR_TYPE_BLOB) {
		if (v->val.b) {
		    f = &v->val.b->b_data[index];
		} else {
		    len = 0;
		}
	    }

	    /*
	    ** make the blob, and arrange to have it released,
	    ** when everyone is done playing with it.
	    */
	    m->m_mem[result].typ = NFR_TYPE_BLOB;
	    if ((m->m_mem[result].val.b = blobNew( f, len, 0 )) == 0) {
		/* failed */
		m->m_mem[result].typ = NFR_TYPE_ERR;
		m->m_mem[result].val.i = NFR_ERR_OOM;
		return;
	    }
	    tmpCallOnFree( blobDecRef, m->m_mem[result].val.b );
	}
    } else {
	goto userError;
    }
}

/*
** nfr_blt_strstr
**
** strstr( big, little )
** Return the index of little inside big, -1 if its not in there.
**
**
*/

void
nfr_blt_strstr(Machine *m,int argc,int firstarg,int result)
{
    MemVal * big, *little ;
    char * index = 0;

    if (argc != 2) {
	userError:
	m->m_mem[result].typ = NFR_TYPE_ERR;
	m->m_mem[result].val.i = NFR_ERR_USER;
	return;
    }
    /*
    ** set up the index.
    */
    big = &m->m_mem[firstarg];
    if (big->typ != NFR_TYPE_STR) {
	goto userError;
    }

    little =  &m->m_mem[firstarg+1];
    if (little->typ != NFR_TYPE_STR) {
	goto userError;
    }

    if ((big->val.c == 0) || (little->val.c == 0)) {
	zer0:
	m->m_mem[result].typ = NFR_TYPE_INT;
	m->m_mem[result].val.i = -1;
	return;
    }
    if ((index = strstr( big->val.c, little->val.c )) == 0) {
	goto zer0;
    }
    m->m_mem[result].typ = NFR_TYPE_INT;
    m->m_mem[result].val.i = index - big->val.c;
    return;
}

/*
** nfr_blt_strcasecmp
**
** strcasecmp( a,b )
**
**
*/

void
nfr_blt_strcasecmp(Machine *m,int argc,int firstarg,int result)
{
    MemVal * big, *little ;
    char * index = 0;

    if (argc != 2) {
	userError:
	m->m_mem[result].typ = NFR_TYPE_ERR;
	m->m_mem[result].val.i = NFR_ERR_USER;
	return;
    }
    /*
    ** set up the index.
    */
    big = &m->m_mem[firstarg];
    if (big->typ != NFR_TYPE_STR) {
	goto userError;
    }

    little =  &m->m_mem[firstarg+1];
    if (little->typ != NFR_TYPE_STR) {
	goto userError;
    }

    if ((big->val.c == 0) || (little->val.c == 0)) {
	zer0:
	m->m_mem[result].typ = NFR_TYPE_INT;
	m->m_mem[result].val.i = -1;
	return;
    }
    m->m_mem[result].typ = NFR_TYPE_INT;
    m->m_mem[result].val.i = strcasecmp( big->val.c, little->val.c );
    return;
}

/*
** nfr_blt_byte
**
** byte( string|blob, int offset )
**
**
*/

void
nfr_blt_byte(Machine *m,int argc,int firstarg,int result)
{
    MemVal * from, * index;
    int ndx;

    if (argc != 2) {
	userError:
	m->m_mem[result].typ = NFR_TYPE_ERR;
	m->m_mem[result].val.i = NFR_ERR_USER;
	return;
    }
    from = &m->m_mem[firstarg];
    if ((from->typ != NFR_TYPE_STR) && (from->typ != NFR_TYPE_BLOB)) {
	goto userError;
    }
    index = &m->m_mem[firstarg+1];
    if (index->typ != NFR_TYPE_INT) {
	goto userError;
    }

    ndx = index->val.i;
    if (ndx < 0) {
	minusOne:
	    m->m_mem[result].typ = NFR_TYPE_INT;
	    m->m_mem[result].val.i = -1;
	    return;
    }

    if (from->typ == NFR_TYPE_BLOB) {
	if (ndx >= from->val.b->b_len) {
	    goto minusOne;
	}
	m->m_mem[result].val.i = from->val.b->b_data[ndx];
    } else
    if (from->typ == NFR_TYPE_STR) {
	if (ndx >= strlen(from->val.c)) {
	    goto minusOne;
	}
	m->m_mem[result].val.i = (from->val.c[ndx] & 0xff);
    }
    m->m_mem[result].typ = NFR_TYPE_INT;
    return;
}
/*
** nfr_blt_short
**
** short( string|blob, int offset )
**
**
*/

void
nfr_blt_short(Machine *m,int argc,int firstarg,int result)
{
    MemVal * from, * index;
    int ndx;

    if (argc != 2) {
	userError:
	m->m_mem[result].typ = NFR_TYPE_ERR;
	m->m_mem[result].val.i = NFR_ERR_USER;
	return;
    }
    from = &m->m_mem[firstarg];
    if ((from->typ != NFR_TYPE_STR) && (from->typ != NFR_TYPE_BLOB)) {
	goto userError;
    }
    index = &m->m_mem[firstarg+1];
    if (index->typ != NFR_TYPE_INT) {
	goto userError;
    }

    ndx = index->val.i;
    if (ndx < 0) {
	minusOne:
	    m->m_mem[result].typ = NFR_TYPE_INT;
	    m->m_mem[result].val.i = -1;
	    return;
    }

    if (from->typ == NFR_TYPE_BLOB) {
	if (ndx >= (from->val.b->b_len-1)) {
	    goto minusOne;
	}
	{
	    /*
	    ** doing a memcpy to move two bytes seems way
	    ** to silly.  Lets just use a union, preserve
	    ** to order, and then ntohs.
	    */
	    union {
		char	vc[2];
		short	value[1];
	    } u;
	    u.vc[0] = from->val.b->b_data[ndx];
	    u.vc[1] = from->val.b->b_data[ndx+1];
	    m->m_mem[result].val.i = ntohs(u.value[0]);
	}
    } else
    if (from->typ == NFR_TYPE_STR) {
	if (ndx >= strlen(from->val.c)) {
	    goto minusOne;
	}
	m->m_mem[result].val.i = (from->val.c[ndx] & 0xff);
    }
    m->m_mem[result].typ = NFR_TYPE_INT;
    return;
}
/*
** nfr_blt_long
**
** byte( string|blob, int offset )
**
**
*/

void
nfr_blt_long(Machine *m,int argc,int firstarg,int result)
{
    MemVal * from, * index;
    int ndx;

    if (argc != 2) {
	userError:
	m->m_mem[result].typ = NFR_TYPE_ERR;
	m->m_mem[result].val.i = NFR_ERR_USER;
	return;
    }
    from = &m->m_mem[firstarg];
    if ((from->typ != NFR_TYPE_STR) && (from->typ != NFR_TYPE_BLOB)) {
	goto userError;
    }
    index = &m->m_mem[firstarg+1];
    if (index->typ != NFR_TYPE_INT) {
	goto userError;
    }

    ndx = index->val.i;
    if (ndx < 0) {
	minusOne:
	    m->m_mem[result].typ = NFR_TYPE_INT;
	    m->m_mem[result].val.i = -1;
	    return;
    }

    if (from->typ == NFR_TYPE_BLOB) {
	if (ndx >= (from->val.b->b_len-3)) {
	    goto minusOne;
	}
	/*
	** doing a memcpy to fill two aligned bytes is silly.
	** Instead, lets just use a union to align the long.
	*/
	{
	    union {
		char vc[4];
		long value[1];
	    } u;
	    u.vc[0] = from->val.b->b_data[ndx];
	    u.vc[1] = from->val.b->b_data[ndx+1];
	    u.vc[2] = from->val.b->b_data[ndx+2];
	    u.vc[3] = from->val.b->b_data[ndx+3];
	    m->m_mem[result].val.i = ntohs(u.value[0]);
	}
    } else
    if (from->typ == NFR_TYPE_STR) {
	if (ndx >= strlen(from->val.c)) {
	    goto minusOne;
	}
	m->m_mem[result].val.i = (from->val.c[ndx] & 0xff);
    }
    m->m_mem[result].typ = NFR_TYPE_INT;
    return;
}

/*
** nfr_blt_blobbytes
**
** blobbytes( ... )
**
** for any integer type, insert the byte associated with the low
** order 8 bits of the expression.  For string or blob types,
** add these (as if we were doing blobcat's).  We can include
** lists, but integers inside those lists won't be included (yet)
** and inserted into 
**
*/

void
nfr_blt_blobbytes(Machine *m,int argc,int firstarg,int result)
{
    /*
    ** iterate across each of the operands, trying to figure out how much
    ** space to use.
    */
    int argi;
    int len = 0;

    for ( argi = 0; argi < argc; ++argi ) {
	MemVal * mem = &m->m_mem[firstarg+argi];
	switch ( mem->typ ) {
	    case NFR_TYPE_STR:
		if (mem->val.c) {
		    len += strlen(mem->val.c);
		}
		break;

	    case NFR_TYPE_BLOB:
		if (mem->val.b) {
		    len += mem->val.b->b_len;
		}
		break;

	    case NFR_TYPE_INT:
		len += 1;
		break;

	    case NFR_TYPE_LIST:
		/* hey! gotta do this one too! */
		len += listbyteslen( mem->val.v );
		break;

	    default:
		break;
	}
    }
    /*
    ** ok, make da blob, and fill it.
    */
    {
	blob_t * b = blobNew( 0, len, 0 );
	char * t = b->b_data;

	tmpCallOnFree( blobDecRef, b );

	for ( argi = 0; argi < argc; ++argi ) {
	    MemVal * mem = &m->m_mem[firstarg+argi];
	    switch ( mem->typ ) {
		case NFR_TYPE_INT:
		    *t++ = (mem->val.i & 0xff);
		    break;

		case NFR_TYPE_BLOB:
		    {
			blob_t * b = mem->val.b;
			if (b) {
			    if (b->b_len) {
				memcpy( t, b->b_data, b->b_len );
				t += b->b_len;
			    }
			}
		    }
		    break;

		case NFR_TYPE_STR:
		    {
			char * f = mem->val.c;
			if (f) {
			    while ( *f ) {
				*t++ = *f++;
			    }
			}
		    }
		    break;

		case NFR_TYPE_LIST:
		    {
			t = listbytescat( mem->val.v, t );
		    }

		default:
		    break;
	    }
	}
	m->m_mem[result].typ = NFR_TYPE_BLOB;
	m->m_mem[result].val.b = b;
    }
}
