/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/

#include <stdio.h>

#include "nfrd.h"

/*
** typeof.
**
** Returns either a string, describing the type of one parameter,
** or a list, describing the type of a buncha parameters.
**
**
** typeof( buncha expressions );
**
*/

char *
typeOf( int typ )
{
    switch( typ ) {
	case NFR_TYPE_ERR:
		return("error");
	case NFR_TYPE_INT:
		return("int");
	case NFR_TYPE_STR:
		return("str");
	case NFR_TYPE_LIST:
		return("list");
	case NFR_TYPE_RECORDER:
		return("recorder");
	case NFR_TYPE_SCHEMADATA:
		return("schemadata");
	case NFR_TYPE_IPV4HOST:
		return("ipv4host");
	case NFR_TYPE_IPV4NET:
		return("ipv4net");
	case NFR_TYPE_BLOB:
		return("blob");
	case NFR_TYPE_ETHMAC:
		return("ethmac");
	case NFR_TYPE_MAPBAG:
		return("array");
	case NFR_TYPE_MAP:
		return("map");
	case NFR_TYPE_INDIRECT:
		return("indirect");
	case NFR_TYPE_PATTERN:
		return("pattern");
	case NFR_TYPE_TRIGGER:
		return("trigger");
	default:
		return("unknown" );
    }
}


void
nfr_blt_typeof(Machine *m,int argc,int firstarg,int result)
{
	if (argc == 1) {
	    m->m_mem[result].val.c = typeOf(m->m_mem[firstarg].typ);
	    m->m_mem[result].typ = NFR_TYPE_STR;
	    return;
	}
	/*
	** otherwise we build a list.
	*/
	{
	    ListMem * new = 0;
	    ListMem * np;
	    int i;
	    for ( i = argc-1; i>= 0; --i ) {
		MemVal * mp = &m->m_mem[firstarg+i];
		np = (ListMem *)tmpcalloc( sizeof(*mp) );
		np->next = new;
		new = np;
		np->data.typ = NFR_TYPE_STR;
		np->data.val.c = typeOf(mp->typ);
	    }
	     m->m_mem[result].typ = NFR_TYPE_LIST;
	     m->m_mem[result].val.v = new;
	}
}
