/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/


#include <stddef.h>
#include <stdio.h>
#include <string.h>

#include "nfrd.h"


/*
** Master builtin function switch table. Builtin functions
** are in the "bltin" subdirectory. To add a builtin, place it in
** the table below, in approximate likelihood order in which it
** will be called. I.e.: frequently called builtins go near the
** beginning of the table.
**
** The interface to builtins is pretty grotesque, with
** each builtin passed a portion of the stack, which it is
** expected to modify and place its return values in.
**
** Marcus J. Ranum, Network Flight Recorder, Inc. 1996
**
*/


extern	void	nfr_blt_alert(Machine *, int, int, int);
extern	void	nfr_blt_display(Machine *, int, int, int);
extern	void	nfr_blt_echo(Machine *, int, int, int);
extern	void	nfr_blt_errno(Machine *, int, int, int);
extern	void	nfr_blt_maptosym(Machine *, int, int, int);
extern	void	nfr_blt_innet(Machine *, int, int, int);
extern	void	nfr_blt_strerror(Machine *, int, int, int);
extern	void	nfr_blt_strlen(Machine *, int, int, int);
extern	void	nfr_blt_strcat(Machine *, int, int, int);
extern	void	nfr_blt_blobcat(Machine *, int, int, int);
extern	void	nfr_blt_strndx(Machine *, int, int, int);
extern	void	nfr_blt_blobndx(Machine *, int, int, int);
extern	void	nfr_blt_strstr(Machine *, int, int, int);
extern	void	nfr_blt_listlen(Machine *, int, int, int);
extern	void	nfr_blt_sublist(Machine *, int, int, int);
extern	void	nfr_blt_listadd(Machine *, int, int, int);
extern	void	nfr_blt_lsort(Machine *, int, int, int);
extern	void	nfr_blt_luniq(Machine *, int, int, int);
extern	void	nfr_blt_recorderNew(Machine *, int, int, int);
extern	void	nfr_blt_recordWrite(Machine *, int, int, int);
extern	void	nfr_blt_typeof(Machine *, int, int, int);
extern	void	nfr_blt_strcasecmp(Machine *, int, int, int);
extern	void	nfr_blt_scope(Machine *, int, int, int);
extern	void	nfr_blt_scopes(Machine *, int, int, int);
extern	void	nfr_blt_regcomp(Machine *, int, int, int);
extern	void	nfr_blt_regexec(Machine *, int, int, int);
extern	void	nfr_blt_byte(Machine *, int, int, int);
extern	void	nfr_blt_short(Machine *, int, int, int);
extern	void	nfr_blt_long(Machine *, int, int, int);
extern	void	nfr_blt_blobbytes(Machine *, int, int, int);


/* here's where to add new functions */
static	Bltin	bltab[] = {
{ "alert",	nfr_blt_alert },		/* bltin/alert.c */
{ "byte",	nfr_blt_byte },			/* bltin/string.c */
{ "short",	nfr_blt_short },			/* bltin/string.c */
{ "long",	nfr_blt_long },			/* bltin/string.c */
{ "blobbytes",	nfr_blt_blobbytes },		/* bltin/string.c */
{ "display",	nfr_blt_display },		/* */
{ "echo",	nfr_blt_echo },
{ "errno",	nfr_blt_errno },
{ "innet",	nfr_blt_innet },		/* bltin/net.c */
{ "maptosym",	nfr_blt_maptosym },		/* bltin/net.c */
{ "recorder",	nfr_blt_recorderNew },		/* ./recorder.c */
{ "strerror",	nfr_blt_strerror },
{ "strlen",	nfr_blt_strlen },
{ "strcat",	nfr_blt_strcat },
{ "blobcat",	nfr_blt_blobcat },
{ "substr",	nfr_blt_strndx },
{ "subblob",	nfr_blt_blobndx },
{ "index",	nfr_blt_strstr },
{ "listadd",	nfr_blt_listadd },		/* bltin/add.c */
{ "listlen",	nfr_blt_listlen },
{ "sublist",	nfr_blt_sublist },
{ "strcasecmp", nfr_blt_strcasecmp },
{ "lsort",	nfr_blt_lsort },
{ "luniq",	nfr_blt_luniq },
{ "send",	nfr_blt_recordWrite },		/* ./recorder.c */
{ "typeof",	nfr_blt_typeof },		/* bltin-typeof.c */
{ "scope",	nfr_blt_scope },		/* bltin-scopes.c */
{ "scopes",	nfr_blt_scopes },		/* bltin-scopes.c */
{ "regcomp",	nfr_blt_regcomp },		/* bltin-regex.c */
{ "regexec",	nfr_blt_regexec },		/* bltin-regex.c */
{ "pattern",	nfr_blt_regcomp },		/* bltin-regex.c */
{ "match",	nfr_blt_regexec },		/* bltin-regex.c */
{ 0,		0 }
};


Bltin	*
bltlookup(char *s)
{
	Bltin	*ret;

	for(ret = bltab; ret->nam != (char *)0; ret++)
		if(!strcmp(s,ret->nam))
			return(ret);
	return((Bltin *)0);
}
