/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/


#include	"nfrd.h"

/*
** boot.c
**
** Tools to start up the engine...
**
**
*/

char *
strcats( char * s, ... )
{
    if (s) {
	int len = strlen(s);
	char * p;
	char * v;
	char * new;
	va_list ap;

	va_start( ap, s );

	while( p = va_arg(ap, char *) ) {
	    len += strlen(p);
	}
	va_end(ap);

	new = (char *)calloc( len+1, sizeof(*v) );
	v = new;
	if (v == 0) {
	    return 0;
	}

	va_start( ap, s );
	while( p = va_arg(ap, char *) ) {
		while(*p) {
		    *v++ = *p++;
		}
	}
	va_end(ap);
	return(new);
    } else {
	return(0);
    }
}


/*
** bootGlobals.
**
** ----------------------------------------
**
** Iterate through the different entries in
** the directory, and set the global variables
** based on file names.
**
** Basedir is the directory where we start.
** As we walk through each of the entries, if its
** a directory, we append the directory name
** to path, and continue.
*/

int
bootGlobals( baseDir, scope )
char * baseDir;
char * scope;
{
    DIR * dir;
    Filter * f = 0;

    dir = opendir( baseDir );

    /*
    ** if a scope is passed in, then we can create a structure
    ** (Filter *) to describe the scope... and then use it 
    ** to set the variable
    */
    if (scope) {
	f = (Filter *)calloc( 1, sizeof(*f) );
	/* assert f? */
	f->f_scope = strdup( scope );
	f->f_scopeNdx = strHash(f->f_scope);
    }

    if (dir) {
	struct dirent *entry;

	while (entry = readdir(dir)) {
	    /* The name of the file is the name of the global variable */
	    char * name = entry->d_name;
	    struct stat s[1];

	    if (isalpha(name[0])) {
		char * fname = nfrPath( baseDir, "/", name, 0 );
		if (fname) {
		    FILE * pkl;
		    if (fileType(fname) == 'd') {
			/*
			** directory.  we define a scope to read the 
			** directory contents, then fill the variables
			** based on that scope.
			*/
			bootGlobals( fname, name );
		    }
		    /*
		    ** need to see if the fname is a directory... if so,
		    ** then that's the scope of this variable...
		    ** For right now, just punt.
		    */
		    pkl = fopen( fname, "r" );
		    if (pkl) {
			MemVal * val = unpickle(pkl);
			/*
			** We don't bring back ERR types, or RECORDER
			** types.  We may decide to bring back recorder
			** types when the parameters to the recorder
			** builtin points to something which describes
			** the ENTIRE recorder, not just the path.
			*/
			if (val
			&& (val->typ != NFR_TYPE_ERR)
			&& (val->typ != NFR_TYPE_RECORDER)) {
			    if (f) {
				run_assign( f, name, NFR_OP_GASGN, val );
			    } else {
				run_globalAssign(name, val);
			    }
			}
			fclose(pkl);
		    } else {
			Alert_Args_Generate( NFR_SRC_NFRD, NFR_ERR_CANTOPENFILE,
			    entry->d_name, 0 );
		    }
		    free(fname);
		} else {
		    /*
		    ** can't malloc space for file name, we'll have big problems soon.
		    */
		    closedir(dir);
		    return;
		}
	    }
	}
	closedir(dir);
    }

    if (f) {
	free(f->f_scope);
	free(f);
    }
}
