/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/


/*
** Command processing...
**
** We have one inbound channel which awaits accepts..
**
*/

#include "nfrd.h"

#ifndef	DEFAULT_PACE_FIFO
#define	DEFAULT_PACE_FIFO "~/etc/run/pace.fifo"
#endif

/*
**
**
*/

int commandFd = 0;

commandInit()
{
    commandFd = 0;
}

/*
** ----------------------------------------
**
** commandReadCmd.
** Get the interpreter to read a file.
*/

/*
** need this to include parse.
*/
yy_nfrd_error(char *s)
{
    extern int yy_nfrd_lineno;
    parseFailed();
    uiCurrPrintf("line: %d, %s\n", yy_nfrd_lineno, s);
}

commandReadCmd( fd, argv )
int fd;
char * argv[];
{
    FILE * input = fopen( argv[0], "r" );
    int err;

    if (input == NULL) {
	Alert_Args_Generate( NFR_SRC_NFRD, NFR_ERR_CANTOPENFILE, argv[0], 0 );
	uiCurrPrintf( "Couldn't open file <%s>\n", argv[0] );
	return;
    }
    yy_nfrd_FileNameSet(argv[0]);
    setyy_nfrd_input(input);
    yy_nfrd_linenoReset();
    err = parse();
    if (err) {
	Alert_Args_Generate( NFR_SRC_NFRD, NFR_NFRD_SYNTAXERR, argv[0], 0 );
	uiCurrPrintf( "Parse error in <%s>\n", argv[0] );
    } else {
	Alert_Args_Generate( NFR_SRC_NFRD, NFR_NFRD_NEWNFR, argv[0], 0 );
	uiCurrPrintf( "Successfully compiled <%s>\n", argv[0] );
    }
    if (input) {
	fclose(input);
    }
}

/*
** ----------------------------------------
**
** commandTraceCmd
**
*/

commandTraceCmd( fd, argv )
int fd;
char * argv[];
{
    if (strcmp( argv[0], "off" ) == 0) {
	Userintf_t * ui = uiFind(fd);
	if (ui) {
	    uiSetTrace( ui, 0 );
	}
    } else
    if (strcmp( argv[0], "on" ) == 0) {
	Userintf_t * ui = uiFind(fd);
	if (ui) {
	    uiSetTrace( ui, 1 );
	}
    } else {
	uiCurrPrintf( "Usage: trace <on> <off>\n" );
    }
}

/*
** ----------------------------------------
**
** commandIntfCmd
**
*/

commandIntfCmd( fd, argv )
int fd;
char * argv[];
{
    Alert_Args_Generate( NFR_SRC_NFRD, NFR_NFRD_NEWINTF, argv[0], 0 );
    intfOpen( argv[0] );
}

/*
** ----------------------------------------
**
** commandPaceCmd
**
*/

commandPaceCmd( fd, argv )
int fd;
char * argv[];
{
    char * a = argv[0];
    char * f = 0;
    if(a == 0) {
	a = nfrPath( DEFAULT_PACE_FIFO, 0 );
	f = a;
    }
    paceOpen( a );
    if (f) {
	free(f);
    }
}


/*
** ----------------------------------------
**
** comandScopeCmd
**
*/

comandScopeCmd( fd, argv )
int fd;
char * argv[];
{
    char * oldScope = scopeSet(strdup(argv[0]));
    if (oldScope) {
	free(oldScope);
    }
}


/*
** ----------------------------------------
**
** comandScopesCmd
** print names of all the scopes.
**
*/

comandScopesCmd( fd, argv )
int fd;
char * argv[];
{
    Machine * m = machinenew();
    int argc = 0;

    if (argv[0]) {
	stack_push( m, memTmpNew( NFR_TYPE_STR, tmpstrdup(argv[0]) ) );
	argc++;
    }
    nfr_blt_scopes( m, argc, 0, 0 );

    blt_echo( m->m_mem[0] );
    uiCurrPrintf( "\n" );

    machinefree(m);
}


/*
** ----------------------------------------
**
** comandUnloadCmd
**
** Walk through all the filters, and look for
** ident names which match the unloaded filter.
*/

comandUnloadCmd( fd, argv )
int fd;
char * argv[];
{
    funcUnload( argv[0] );
}

/*
** ----------------------------------------
**
** comandVerboseCmd
*/

commandVerboseCmd( fd, argv )
int fd;
char * argv[];
{
    extern int vFlag;
    extern int rFlag;

    if (argv[0]) {
	if (strcmp( argv[0], "on" ) == 0) {
	    uiCurrPrintf( "Verbose turned on\n" );
	    vFlag = 1;
	} else if (strcmp( argv[0], "trace" ) == 0) {
	    uiCurrPrintf( "Verbose/Trace turned off\n" );
	    rFlag = 1;
	} else if (strcmp( argv[0], "off" ) == 0) {
	    uiCurrPrintf( "Verbose turned off\n" );
	    vFlag = 0;
	    rFlag = 0;
	} else {
	    uiCurrPrintf( "verbose [ on | off], don't understand <%s>\n", argv[0] );
	}

    } else {
	if (vFlag) {
	    uiCurrPrintf( "Verbose turned off\n" );
	    vFlag = 0;
	} else {
	    uiCurrPrintf( "Verbose turned on\n" );
	    vFlag = 1;
	}
    }
}

/*
** ----------------------------------------
**
** commandStats
*/

commandStats( fd, argv )
int fd;
char * argv[];
{
    if (argv[0] == 0) {
	intfStats();
	paceStats();
	packetStats();
	reassembleStats();
	tcpPatternStats();
	performStats();
	funcStats();
	recorderStats();
	tcpReassembleMem();
	pollStats();
    } else {
	while( argv[0] ) {
	    if (strcmp( argv[0], "intf" ) == 0) {
		intfStats();
	    } else if (strcmp( argv[0], "pace" ) == 0) {
		paceStats();
	    } else if (strcmp( argv[0], "packet" ) == 0) {
		packetStats();
	    } else if (strcmp( argv[0], "ip" ) == 0) {
		ipAssembleStats();
	    } else if (strcmp( argv[0], "perform" ) == 0) {
		performStats();
	    } else if (strcmp( argv[0], "tcp" ) == 0) {
		tcpReassembleStats();
		tcpPatternStats();
	    } else if (strcmp( argv[0], "func" ) == 0) {
		funcStats();
	    } else if (strcmp( argv[0], "recorder" ) == 0) {
		recorderStats();
	    } else if (strcmp( argv[0], "mem" ) == 0) {
		tcpReassembleMem();
	    } else if (strcmp( argv[0], "poll" ) == 0) {
		pollStats();
	    } else {
		uiCurrPrintf( "stats: usage stats [ intf ] [ pace ] [ packet ] [ ip ] [ perform ] [ tcp ] [ func ] [ recorder ] [ mem ] [ poll ]\n" );
	    }
	    argv++;
	}
    }
}


/*
** ----------------------------------------
** commandPrint
**
** print the value associated with the variable
** passed in... 
*/

commandPrint( fd, argv )
int fd;
char * argv[];
{
    while( argv[0] ) {
	symPrint( argv[0] );
	argv++;
    }
}


/*
** ----------------------------------------
** commandAlert
**
** currently the only thing you can do with
** the alert command is init.  (Maybe listing
** all the names would be nice)
*/

commandAlert( fd, argv )
int fd;
char * argv[];
{
    if (strcmp( argv[0], "init" ) == 0) {
	nfr_blt_alertInit();
    }
}

/*
** ----------------------------------------
**
** commandParse
**
** Read a command from the input, crack it, then parse it.
** 
** hey, we'll do a simple parse here, maybe later
** we'd somehow incorperate direct instruction parsing
** via the interpreter...
*/

commandParse( fd, argv )
char * argv[];
{

    if (argv[0] == 0) {
	return;
    }
    if (strcmp( "read", argv[0] ) == 0) {
	commandReadCmd( fd, &argv[1] );
    }
    else if (strcmp( "trace", argv[0] ) == 0) {
	commandTraceCmd( fd, &argv[1] );
    }
    else if (strcmp( "intf", argv[0] ) == 0) {
	commandIntfCmd( fd, &argv[1] );
    }
    else if (strcmp( "pace", argv[0] ) == 0) {
	commandPaceCmd( fd, &argv[1] );
    }
    else if (strcmp( "scope", argv[0] ) == 0) {
	comandScopeCmd( fd, &argv[1] );
    }
    else if (strcmp( "scopes", argv[0] ) == 0) {
	comandScopesCmd( fd, &argv[1] );
    }
    else if (strcmp( "unload", argv[0] ) == 0) {
	comandUnloadCmd( fd, &argv[1] );
    }
    else if (strcmp( "test", argv[0] ) == 0) {
    }
    else if (strcmp( "verbose", argv[0] ) == 0) {
	commandVerboseCmd( fd, &argv[1] );
    }
    else if (strcmp( "halt", argv[0] ) == 0) {
	nfrdHalt();
    }
    else if (strcmp( "stats", argv[0] ) == 0) {
	commandStats( fd, &argv[1] );
    }
    else if (strcmp( "print", argv[0] ) == 0) {
	commandPrint( fd, &argv[1] );
    }
    else if (strcmp( "alert", argv[0] ) == 0) {
	commandAlert( fd, &argv[1] );
    }
    else {
	uiCurrPrintf( "0 Command? %s\n", argv[0] );
	Alert_Args_Generate( NFR_SRC_NFRD, NFR_NFRD_COMMANSYNTAXERR, argv[0], 0 );
    }
    {
	char tbuf[64];
	time_t now = time(0);

	strftime( tbuf, sizeof(tbuf),
			    "[%D %T]", localtime( &now ) );
	uiCurrPrintf( "1 done %s %s\n", argv[0], tbuf );
    }
}

/*
** ----------------------------------------
**
** commandRead
**
** Read a command from the input, crack it, then parse it.
** Use the buffer mechanism which allows the newlines to
** cross packet boundaries, so that we can have partial
*/

commandRead( fd, rock )
int fd;
void * rock;
{
    buff_t * b = (buff_t *)rock;
    int err = fillBuff( b, fd );
    char * line;
    int len;

    if (err == -1) {
	buffFree(b);
	uiFdClose(fd);
	close(fd);
	pollUnCall(fd);
	return;
    }
    /*
    ** There may be some left over junk in the buffer
    */
    while( (line = getsBuff(b)) != 0) {
	char ** argv = crack(line);
	Userintf_t * oldUi = uiSetCurr( uiFind(fd) );

	if (argv) {
	    /*
	    ** now we've got an argv list of the buffer, 
	    ** lets parse it.
	    */
	    commandParse( fd, argv );
	    crackFree( argv );
	}
	uiSetCurr( oldUi );
    }
    if (err == -1) {
	buffFree(b) ;
    }
}


/*
** ----------------------------------------
**
** commandAccept
**
** Perform an accept on the socket, then 
** set up the polling routines to process the inbound
** command stream.
*/

commandAccept( fd, rock )
int fd;
void * rock;
{
    struct sockaddr sin;
    int len = sizeof(sin);
    int afd = accept( fd, (struct sockaddr *)&sin, &len );

    if (afd < 0) {
	Alert_Args_Generate( NFR_SRC_NFRD, NFR_NFRD_COMMANACCEPTERROR, 
		format( "%d", fd ), 0 );
	return;
    }
    uiNew( afd );

    pollCall( afd, commandRead, buffNew( 0, 4096 ) );
}

/*
** ----------------------------------------
**
** commandSockOpen
**
** create a single accept port to read commands.
**
*/

commandSockOpen( host, port )
char * host;
int port;
{
    struct hostent * h;
    struct sockaddr_in sin;

    commandFd = socket( AF_INET, SOCK_STREAM, 0 );
    if (commandFd < 0) {
	/* error */
	return(0);
    }

    /*
    ** Bind an address to the port...
    */
    memset( &sin, 0, sizeof(sin) );
    sin.sin_family = AF_INET;
    sin.sin_port = htons(port);

    if (strchr(host, '.') && isdigit(host[0])) {
	unsigned long ha;
	ha = inet_addr(host);
	memcpy( &sin.sin_addr.s_addr, &ha, sizeof(sin.sin_addr.s_addr) );
    } else {
	if ((h = gethostbyname( host )) == 0) {
	    Alert_Args_Generate( NFR_SRC_NFRD, NFR_ERR_CANTFINDHOST, 
		format( "%s/%d", host, port ), 0 );
	    return(0);
	}
	memcpy( &sin.sin_addr.s_addr, h->h_addr, sizeof(sin.sin_addr.s_addr) );
    }

    if (bind( commandFd, (struct sockaddr *)&sin, sizeof(sin) ) < 0) {
	Alert_Args_Generate( NFR_SRC_NFRD, NFR_ERR_CANTBINDIPHOST, host, 
		format( "%d", port ), 0 );
	return(0);
    }

    if ((listen( commandFd, 255 )) < 0) {
	Alert_Args_Generate( NFR_SRC_NFRD, NFR_ERR_LISTENERR, 
		format( "%s/%d", host, port ), 0 );
	return(0);
    }

    pollCall( commandFd, commandAccept, 0 );
    return(1);
}


/*
** ----------------------------------------
**
** commandFifoOpen
**
** create a single accept port to read commands.
**
*/

commandFifoOpen( name )
char * name;
{
    struct sockaddr_un un;

    commandFd = socket( AF_UNIX, SOCK_STREAM, 0 );
    if (commandFd < 0) {
	/* error */
	Alert_Args_Generate( NFR_SRC_NFRD, NFR_ERR_CANTOPENFIFO, name, 0 );
	return(0);
    }
    if (strlen(name)+1 > sizeof(un.sun_path)) {
	/* error */
	Alert_Args_Generate( NFR_SRC_NFRD, NFR_ERR_FIFONAMETOOLONG, name, 0 );
	return(0);
    }

    /* don't worry about failure... we just wanna clear this one out */

    /* XXX we may wanna stat the name, to see if its still a socket-type,
    ** and leave it alone if it is ... */

    unlink( name );

    /*
    ** Bind an address to the port...
    */
    un.sun_family = AF_UNIX;
    strcpy( un.sun_path, name );


    if (bind( commandFd, (struct sockaddr *)&un, sizeof(un) ) < 0) {
	Alert_Args_Generate( NFR_SRC_NFRD, NFR_ERR_CANTBINDFIFO, name, 0 );
	return(0);
    }

    if ((listen( commandFd, 255 )) < 0) {
	Alert_Args_Generate( NFR_SRC_NFRD, NFR_ERR_LISTENERR, name, 0 );
	return(0);
    }

    pollCall( commandFd, commandAccept, 0 );
    return(1);
}

/*
** ----------------------------------------
**
** commandClose
**
** shut down the command interface.
**
*/
commandClose()
{
    pollUnCall( commandFd );
    close( commandFd );
    commandFd = 0;
}
