/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/


/*
**
** Interface to read config files:
** 
**	config_t *x;
**	x= configRead( "filename" );
**	if (! x) you got an error;
**
** Now you have an in-core image of the config file.  use
**
**	char *s;
**	s=configGet( x, "token");
**
** to look up the config item named "token".
**
** When you don't need the config information any more:
**
**	configDispose(x);
** 
*/

#include "nfrd.h"

char *
configReadString( f )
FILE * f;
{
    /*
    ** Do what we need to do to unpickle the type contents...
    */
    if (f) {
	int len = 0; /* the end-of-string */
	int totLen = 64;
	char * data = (char *)malloc( totLen );
	int q;
	int c;

	/* We're looking for the lead in quote. */
	while( (c = fgetc( f )) != EOF ) {
	    if ((c == '\'') || (c == '"')) {
		q = c; /* this is the quote we're looking for... */

		/* 
		** add characters till we reach the other quote.
		*/
		while( (c = fgetc( f )) != EOF ) {
		    if (c == q) {
			we_are_done:

			if (len+1 != totLen)
			    data = (char *)realloc( data, len+1 );
			data[len] = 0;
			return(data);

		    } else switch (c) {
			case '\\':		/* literal next */
			    c = fgetc( f );
			    if (c == EOF) {
				goto we_are_done;  /* not sure what else to do */
			    } else if (c == 't') { /* replace with tab. */
				c = '\t';
			    } else if (c == 'n') { /* replace with newline. */
				c = '\n';
			    }

			case '\n':		/* should we issue a warning? */
			default:
			    data[len++] = c;
			    if (len >= totLen) {
				totLen += 32;
				data = (char *)realloc( data, totLen );
			    }
		    }
		}
	    }
	}
	goto we_are_done;
    } else {
	return(0);
    }

}

/*
** ----------------------------------------
**
** recorderConfigRead
**
** Read the file, finding variables names,
** and building an array of config_t's
** describing the contents of the file.
**
*/

config_t *
configRead( char * f )
{
    FILE * in = fopen( f, "r" );
    if (in) {
	int array_max = 10;
	config_t * array = (config_t *)calloc( sizeof(*array), array_max );
	int array_i = 0;
	int c;

	while ( (c = fgetc( in )) != EOF ) {
	    /* discard comments ... */
	    while(isspace(c)) {
		c = fgetc(in);
	    }
	    if (c == EOF) {
		break;
	    }

	    if (c == '#') {
		while( (c = fgetc(in)) != '\n' ) {
		}
	    } else {
		/* find the end of the first symbol... */
		int cmax = 32;
		char * cdata = (char *)malloc( cmax );
		int cndx = 0;
		char * vdata = 0;

		cdata[cndx++] = c;
		while ((c = fgetc(in)) != EOF) {
		    if (c == '=') {
			break;
		    }
		    if (c == '\n') {
			goto no_value;
		    }
		    if (isspace(c)) {
			goto no_more_symbol;
		    }
		    if (cndx >= cmax) {
			cmax += 32;
			cdata = (char *)realloc( cdata, cmax );
		    }
		    cdata[cndx++] = c;
		}
		no_more_symbol:
		cdata = (char *)realloc( cdata, cndx+1 );
		cdata[cndx] = 0;

		if (isspace(c)) {
		    while((c = fgetc(in)) != EOF) {
			if (!isspace(c)) {
				break;
			}
		    }
		}

		/*
		** We ought to be pointing at the '='
		*/
		if (c == '=') {
		    /*
		    ** now look for the value... if we see a quote
		    ** or a doublequote, then we'll use an unpickling
		    ** routine to 
		    */

		    while ( isspace(c = fgetc( in )) ) {
			if ((c == '\n') || (c == EOF)) {
			    goto no_value;
			}
		    }
		    if ((c == '"') || (c == '\'')) {
			ungetc( c, in );
			vdata = configReadString( in );
		    } else {
			int vmax = 32;
			int vndx = 0;

			vdata = (char *)malloc( vmax );
			/*
			** collect the value... till we hit a space...
			*/
			vdata[vndx++] = c;
			while( !isspace(c = fgetc( in )) ) {
			    if (vndx >= vmax) {
				vmax += 32;
				vdata = (char *)realloc( vdata, vmax );
			    }
			    vdata[vndx++] = c;
			}
			vdata = (char *)realloc( vdata, vndx+1 );
			vdata[vndx] = 0;
		    }
		} else {
		    while((c = fgetc(in)) != EOF) {
			if (c == '\n') {
			    break;
			}
		    }
		}
		no_value:

		/*
		** Add the name/value pair to the config list...
		*/
		if (array_i > array_max) {
		    array_max += 10;
		    array = (config_t *)realloc( array, array_max*(sizeof(*array)) );
		}
		array[array_i].c_name = cdata;
		array[array_i].c_value = vdata;
		array_i++;
	    }
	}
	fclose(in);

	array = (config_t *)realloc( array, (array_i+1)*(sizeof(*array)) );
	array[array_i].c_name = 0;
	array[array_i].c_value = 0;

	return(array);
    } else {
	return(0);
    }
}

/*
** ----------------------------------------
**
** recorderGetConfig
**
** Try to locate the value associated with some name.
*/

char *
configGet( c, name )
config_t * c;
char *	name;
{
    config_t * p;

    for ( p = c; p->c_name; ++p ) {
	if (strcmp( name, p->c_name ) == 0) {
		return( p->c_value );
	}
    }
    return(0);
}

/*
** ----------------------------------------
**
** free the stuff associated with the config file
*/

void
configDispose( config_t * c )
{
    while(c->c_name) {
	if (c->c_name) {
	    free(c->c_name);
	}
	if (c->c_value) {
	    free(c->c_value);
	}
	c++;
    }
    free(c);
}

