/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/

/*
** Since the nfrd has some of the tools we need to build the
** control interface, it makes sense to place it here, with the
** nfrd.
*/
#include "nfrd.h"

char * argv0;

int dFlag;	/* debug */
int eFlag;	/* echo commands before sent to engine */
int vFlag;	/* verbose */
int sFlag;	/* stay connected after input EOF */

#ifndef	DEFAULT_NFRD_FIFO
#define	DEFAULT_NFRD_FIFO "~/etc/run/nfrd.fifo"
#endif
#ifndef	SECOND_DEFAULT_NFRD_FIFO
#define	SECOND_DEFAULT_NFRD_FIFO "etc/run/nfrd.fifo"
#endif


/*
** We have to devices we can recieve output from,
** stdin, and the nfrd.
*/

/*
** ----------------------------------------
**
**
*/

int controlFd;

controlInit() {
    controlFd = 0;
}

/*
** ----------------------------------------
**
**
*/

userRead( fd, rock )
int fd;
void * rock;
{
    char * line;
    buff_t * b = (buff_t *)rock;
    int err = fillBuff( b, fd );

    if (err == -1) {
	pollUnCall( fd );
	close(fd);
	if (sFlag) {
	    if (vFlag) {
		log( FL, "Input End-of-File\n" );
	    }
	} else {
	    /*
	    ** no more input.  We'll only stay
	    ** connected if the user asked us to...
	    */
	    exit(0);
	}
    } else while ( (line = getsBuff( b )) != 0) {
	if (eFlag) {
	    log( FL, "Sent to engine: %s\n", line );
	}
	write( controlFd, line, strlen(line) );
	write( controlFd, "\n", 1 ); /* poor mans concat :( */
    }
}


/*
** ----------------------------------------
**
** Read fron the control fd... then display on stdout.
*/

controlRead( fd, rock )
int fd;
void * rock;
{
    char buffer[1024];
    int len;

    if ((len = read( fd, buffer, sizeof(buffer) )) <= 0) {
	/* close down the connection, and exit. */
	close(fd);
	exit(1);
    } else {
	write( 1, buffer, len );
    }
}

/*
** ----------------------------------------
**
*/

controlSockOpen( host, port )
char * host;
int port;
{
    struct hostent * h;
    struct sockaddr_in sin;

    controlFd = socket( AF_INET, SOCK_STREAM, 0 );
    if (controlFd < 0) {
	/* error */
	return(0);
    }

    /*
    ** create the other endpoint address.
    */
    memset( &sin, 0, sizeof(sin) );
    sin.sin_family = AF_INET;
    sin.sin_port = htons(port);

    if (strchr(host, '.') && isdigit(host[0])) {
	unsigned long ha;
	ha = inet_addr(host);
	memcpy( &sin.sin_addr.s_addr, &ha, sizeof(sin.sin_addr.s_addr) );
    } else {
	if ((h = gethostbyname( host )) == 0) {
	    log( FL, "controlSockOpen: can't find %s/%d\n", host, port );
	    return(0);
	}
	memcpy( &sin.sin_addr.s_addr, h->h_addr, sizeof(sin.sin_addr.s_addr) );
    }

    if (connect( controlFd, (struct sockaddr *)&sin, sizeof(sin) ) < 0) {
	log( FL, "commandSockOpen: can't connect %s/%d\n", host, port );
	perror("connect");
	return(0);
    }

    pollCall( controlFd, controlRead, 0 );
}

/*
** ----------------------------------------
**
*/

controlFifoOpen( name )
char * name;
{
    struct sockaddr_un un;

    controlFd = socket( AF_UNIX, SOCK_STREAM, 0 );
    if (controlFd < 0) {
	/* error */
	log( FL, "controlFifoOpen: can't create socket %s\n", name );
	return(-1);
    }
    if (strlen(name)+1 > sizeof(un.sun_path)) {
	/* error */
	log( FL, "controlFifoOpen: name too long %s\n", name );
	return(-1);
    }

    un.sun_family = AF_UNIX;
    strcpy( un.sun_path, name );

    if (connect( controlFd, (struct sockaddr *)&un, sizeof(un) ) < 0) {
	return(0);
    }

    pollCall( controlFd, controlRead, 0 );
    return(1);
}

/*
** --------------------
**
*/

controlClose()
{
    close(controlFd);
}

/*
** --------------------
**
** main.
**
*/

char *	nfrHost = 0;
int	nfrPort = 0;
char *	logFile;	/* we don't use this (yet) */
int	timeOut = -1;	/* we wait forever (maybe we ought to ping the nfrd?) */
int	restart = 0;	/* in case we want better signal support */



main( argc, argv )
int argc;
char * argv[];
{
    argv0 = argv[0];

    {
	int i;
	for (i=1; i<argc; ++i) {
	    if (argv[i][0] == '-') switch(argv[i][1]) {
		case 'l':	/* how to log. */
		    if (argv[i][2]) {
			logFile = &argv[i][2];
		    } else {
			logFile = argv[++i];
		    }
		    break;

		case 't':	/* timeout */
		    if (argv[i][2]) {
			timeOut = atoi( &argv[i][2] );
		    } else {
			timeOut = atoi( argv[++i] );
		    }
		    break;

		case 'd':	/* debug */
		    dFlag++;
		    break;
		
		case 'e':	/* echo commands */
		    eFlag++;
		    break;
		
		case 's':	/* stay */
		    sFlag++;
		    break;

		case 'V':	/* verbose */
		    vFlag++;
		    break;

		default:
		usage:
		    Usage:
		    fprintf( stderr, "usage: control { <fifo-path> | host port }\n" ); 
		    exit(1);
	    } else {
		if (nfrHost == 0) {
		    nfrHost = argv[i];
		}
		else if (nfrPort == 0) {
		    nfrPort = atoi(argv[i]);
		}

	    }
	}
    }
    logOpen( "-" );

    log( FL, "nfrd-control. $Header: /home/master/nfrbase/nfr/nfrd/control.c,v 1.4 1997/12/10 22:29:28 mts Exp $\n" );

    pollInit();
    pollOpen();

    if (nfrHost == 0) {
	nfrHost = DEFAULT_NFRD_FIFO;
    }

    /*
    ** Create the command/control processing stream.
    */
    {
	int ok = 0;

	if (nfrHost && nfrPort) {
	    controlInit();
	    ok = controlSockOpen( nfrHost, nfrPort );
	} else if (nfrHost) {
	    char * name;
	    controlInit();
	    if ((ok = controlFifoOpen( name = nfrPath( nfrHost, 0 )) ) == 0) {
		char * second;
		/*
		** try a second default...
		*/
		if ((ok = controlFifoOpen( second = nfrPath( SECOND_DEFAULT_NFRD_FIFO, 0 ))) == 0) {
		    log( FL, "controlFifoOpen: can't connect to %s or %s\n", name, second );
		    perror("connect");
		}
		free(second);
	    }
	    free(name);
	} else {
	    goto Usage;
	}
	if (ok == 0) {
	    printf( "Can't find connection to engine: %s nor %s\n",
		nfrHost, SECOND_DEFAULT_NFRD_FIFO );
	    exit(1);
	}
    }

    /*
    ** Create the standard input stream.
    */
    pollCall( fileno(stdin), userRead, buffNew( 0, 16*1024 ) );


    while (restart == 0) {
	pollPerform( timeOut );
    }

    pollClose();
    if (nfrHost && nfrPort) {
	controlClose();
    }

    /*
    ** We ought to save our global state once more, just before we exit...
    */

    /*
    ** normal exit.
    */
    exit(0);
}

mark() {}
