/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/

/*
** Since the nfrd has some of the tools we need to build the
** control interface, it makes sense to place it here, with the
** nfrd.
*/
#include "nfrd.h"

char * argv0;

int dFlag;
int vFlag;
int qFlag;

#define	NONE	0
#define	NONFRD	1
#define	READY	2
#define	WAIT	3
int	State = NONE;

#ifndef	DEFAULT_NFRD_FIFO
#define	DEFAULT_NFRD_FIFO "~/etc/run/nfrd.fifo"
#endif
#ifndef	SECOND_DEFAULT_NFRD_FIFO
#define	SECOND_DEFAULT_NFRD_FIFO "etc/run/nfrd.fifo"
#endif


/*
**
*/

char *	nfrFifo = 0;
int	nfrPort = 0;
char *	logFile;	/* we don't use this (yet) */
int	timeOut = -1;	/* we wait forever (maybe we ought to ping the nfrd?) */
int	restart = 0;	/* in case we want better signal support */


char ** requests;
int	requestsNdx = 0;	/* how many we found on the arg line */
int	requestsMax = 0;	/* max size of requests */
int	requestsDone = 0;	/* how many we've finished */


/*
** ----------------------------------------
**
**
*/

int controlFd;
FILE * controlFile;

controlInit() {
    controlFd = 0;
}

/*
** ----------------------------------------
**
**
*/

userRead( fd, rock )
int fd;
void * rock;
{
    char * line;
    buff_t * b = (buff_t *)rock;
    int err = fillBuff( b, fd );

    if (err == -1) {
	pollUnCall( fd );
    } else while ( (line = getsBuff( b )) != 0) {
	log( FL, "userRead: send %s\n", line );
	write( controlFd, line, strlen(line) );
	write( controlFd, "\n", 1 ); /* poor mans concat :( */
    }
}


/*
** ----------------------------------------
**
** Read fron the control fd... then display on stdout.
*/

controlRead( fd, rock )
int fd;
void * rock;
{
    buff_t * b = (buff_t *)rock;
    int err = fillBuff( b, fd );
    char * line;
    int len;

    while ( (line = getsBuff(b)) != 0) {
	/* see if the first character of the line is a digit.
	** if so, then its a prefix number indication some meta
	** info fo us.
	*/
	if (isdigit(line[0])) {
	    char * l = line;
	    int num = atoi(l);
	    while(isdigit(l[0])) {
		++l;
	    }
	    while(isspace(l[0])) {
		++l;
	    }
	    /*
	    ** Now change the global state, depending the 
	    ** number we got back.
	    **
	    ** Maybe we ought to have a routine which decides whether
	    ** to print the line depending on the value of num...
	    */

	    if (num == 1) {
		State = READY;
	    } else {
		puts( l );
	    }
	} else {
	    puts( line ); 
	}
    }
    if (err == -1) {
	pollUnCall( fd );
	buffFree(b);
    }
}

/*
** ----------------------------------------
**
*/

controlFifoOpen( name )
char * name;
{
    struct sockaddr_un un;

    controlFd = socket( AF_UNIX, SOCK_STREAM, 0 );
    if (controlFd < 0) {
	/* error */
	log( FL, "controlFifoOpen: can't create socket %s\n", name );
	return(0);
    }
    if (strlen(name)+1 > sizeof(un.sun_path)) {
	/* error */
	log( FL, "controlFifoOpen: name too long %s\n", name );
	return(0);
    }

    un.sun_family = AF_UNIX;
    strcpy( un.sun_path, name );

    if (connect( controlFd, (struct sockaddr *)&un, sizeof(un) ) < 0) {
	return(0);
    }

    pollCall( controlFd, controlRead, buffNew( 0, 4096 ) );

    controlFile = fdopen( controlFd, "w" );
    nfr_setlinebuf( controlFile );
    return(1);
}

/*
** --------------------
**
*/

controlClose()
{
    if (controlFile) {
	fclose(controlFile);
    } else {
	close(controlFd);
    }
}

/*
** engineMissing.
** in case we wanna do something about this in the future.
** the code is organized to possibly start the engine...
** but we've got to be sure its not running... (the fifo
** is a good clue).
*/

engineMissing()
{
    log( FL, "engineMissing: not connected\n" );
    exit(1);
}

/*
** requestStartup
**
** Wait for the engine to appear...
*/

requestStartup()
{
    /*
    ** wait a few seconds.  If the fifo doesn't appear
    ** then we've failed.  If it does appear, then we've
    ** started the nfrd.
    */

    {
	int count = 5;	/* how many times to try */
	while (count--) {
	    char * name;
	    if (controlFifoOpen( name = nfrPath( nfrFifo, 0 ) )) {
		/*
		** nfrd is now running.
		*/
		State = READY;
		log( FL, "ctlnfrd: nfrd ready\n" );
		break;
	    }
	    sleep(1);
	    if (name) {
		free(name);
	    }
	}
    }
    if (State != READY) {
	log( FL, "requestStart: couldn't start nfrd.\n" );
	exit(1);
    }
}


/*
** requestStart.
** Assume we can just start the nfrd.
** Don't use the system command, its too dangerous... instead
** fork a child off, and let it execvp.
**
*/

char * startNfrd[] = { "nfrd", "-l", "-", 0 };


requestStart()
{
    requestsDone++; /* in either case, we're done with this */

    if (State == NONFRD) {
	if (fork() == 0) {
	    /* child */

	    /*
	    ** need to think about what parameters
	    ** we ought to pass through.
	    */
	    execvp( startNfrd[0], startNfrd );
	    log( FL, "requestStart: couldn't start nfrd.\n" );
	    perror( "nfrd" );
	    exit(1);
	}
	requestStartup();
    }
}

/*
** packageLoad
** used by pkgLoad to load one component we've found.
*/

packageLoad( scope, s )
char * scope;
char * s;
{
    if (State == READY) {
	if (scope) {
	    fprintf( controlFile, "scope %s\n", scope );
	    State = WAIT;
	    while (State == WAIT) {
		pollPerform( timeOut );
	    }
	}
	if (s) {
	    fprintf( controlFile, "read %s\n", s );
	    State = WAIT;
	    while (State == WAIT) {
		pollPerform( timeOut );
	    }
	}
    }
    return(0);
}


/*
** requestPackage
**
** Load the engine with every .nfr file we find underneath
** the indicated path.
*/

requestPackage()
{

    if (State == NONFRD) {
	engineMissing();
    }
    if (State == READY) {
	requestsDone++;
	if (requestsDone == requestsNdx) {
	    fprintf( stderr, "start command: missing file name" );
	} else {
	    /*
	    ** This call indirectly calls descentRead with two
	    ** parameters, the scope, and the path of the .nfr
	    ** file to load.
	    **
	    */
	    pkgLoad( /* starting scope */0, requests[requestsDone++], packageLoad );
	}
    }
    if (State != READY) {
	log( FL, "requestPackage: couldn't find nfrd.\n" );
	exit(1);
    }
}


/*
** packageList
** used by pkgLoad to list one component we've found.
*/

packageList( scope, s )
char * scope;
char * s;
{
    printf( "Scope: %-20.20s Path: %s\n", scope, s );
    return(0);
}




/*
** requestLsPackage
**
** Load the engine with every .nfr file we find underneath
** the indicated path.
*/

requestLsPackage()
{

    if (State == NONFRD) {
	engineMissing();
    }
    if (State == READY) {
	requestsDone++;
	if (requestsDone == requestsNdx) {
	    fprintf( stderr, "start command: missing file name" );
	} else {
	    printf( "Package Scan starting at: %s\n", requests[requestsDone] );
	    pkgLoad( 0, requests[requestsDone++], packageList );
	}
    }
    if (State != READY) {
	log( FL, "requestLsPackage: couldn't find nfrd.\n" );
	exit(1);
    }
}



/*
** requestStats.
**
** Send the request for stats to the engine, then
** we go into a wait stats till we get the done reply.
*/

requestStats( s )
char * s;
{
    if (State == NONFRD) {
	engineMissing();
    }
    if (State == READY) {
	if (s == 0) {
	    requestsDone++;
	    fprintf( controlFile, "stats\n" );
	    State = WAIT;
	} else {
	    requestsDone++;
	    fprintf( controlFile, "stats %s\n", s );
	    State = WAIT;
	}
    }
}

/*
**
*/

requestHalt()
{
    if (State == NONFRD) {
	engineMissing();
    }
    if (State == READY) {
	requestsDone++;
	fprintf( controlFile, "halt\n" );
	State = WAIT;
    }
}

/*
** requestIntf.
**
** Use the next request word as the interface name.
**
** 
*/

requestIntf()
{
    if (State == NONFRD) {
	engineMissing();
    }
    if (State == READY) {
	requestsDone++;
	if (requestsDone == requestsNdx) {
	    fprintf( stderr, "intf command: missing interface name\n" );
	} else {
	    fprintf( controlFile, "intf %s\n", requests[requestsDone] );
	    requestsDone++;
	    State = WAIT;
	}
    }
}

/*
** requestPace.
**
** Maybe we ought to be responsible for actually starting the 
** command which also pumps the data into the engine?
**
*/

requestPace()
{
    if (State == NONFRD) {
	engineMissing();
    }
    if (State == READY) {
	requestsDone++;
	if (requestsDone == requestsNdx) {
	    fprintf( stderr, "pace command: assuming ~/etc/run/pace.fifo" );
	    fprintf( controlFile, "pace %s\n", requests[requestsDone] );
	    requestsDone++;
	    State = WAIT;
	} else {
	    fprintf( controlFile, "pace %s\n", requests[requestsDone] );
	    requestsDone++;
	    State = WAIT;
	}
    }
}

/*
** requestLoad.
**
**
*/

requestLoad()
{
    if (State == NONFRD) {
	engineMissing();
    }
    if (State == READY) {
	requestsDone++;
	if (requestsDone == requestsNdx) {
	    fprintf( stderr, "load command: missing file name" );
	} else {
	    /*
	    ** Its the read command, cause from the engine's point
	    ** of view, we're reading another .nfr file.
	    */
	    fprintf( controlFile, "read %s\n", requests[requestsDone] );
	    requestsDone++;
	    State = WAIT;
	}
    }
}

/*
** requestPrint.
**
**
*/

requestPrint()
{
    if (State == NONFRD) {
	engineMissing();
    }
    if (State == READY) {
	requestsDone++;
	if (requestsDone == requestsNdx) {
	    fprintf( stderr, "load command: missing variable name" );
	} else {
	    fprintf( controlFile, "print %s\n", requests[requestsDone] );
	    requestsDone++;
	    State = WAIT;
	}
    }
}

/*
** requestAlertInit.
**
**
*/

requestAlertInit()
{
    if (State == NONFRD) {
	engineMissing();
    }
    if (State == READY) {
	requestsDone++;
	fprintf( controlFile, "alert init\n", requests[requestsDone] );
	State = WAIT;
    }
}

/*
** requestVerbose.
**
**
*/

requestVerbose()
{
    if (State == NONFRD) {
	engineMissing();
    } 
    if (State == READY) {
	requestsDone++;
	if (requestsDone == requestsNdx) {
	    fprintf( controlFile, "verbose\n" );
	    requestsDone++;
	    State = WAIT;
	} else {
	    /*
	    ** Its the read command, cause from the engine's point
	    ** of view, we're reading another .nfr file.
	    */
	    fprintf( controlFile, "verbose %s\n", requests[requestsDone] );
	    requestsDone++;
	    State = WAIT;
	}
    }
}

/*
** performRequest.
**
** Multiplexor for the different operations we can perform.
*/

performRequest() {

    while (requestsDone < requestsNdx) {

	if (State == WAIT) {
	    if (qFlag == 0) {
		printf( "performRequest: waiting...\n" );
	    }
	    return;
	}

	if (qFlag == 0) {
	    printf( "Request: %s\n", requests[requestsDone] );
	}
	if (strcmp( requests[requestsDone], "stats" ) == 0) {
	    requestStats(0);
	} else if (strcmp( requests[requestsDone], "start" ) == 0) {
	    requestStart();
	} else if (strcmp( requests[requestsDone], "package" ) == 0) {
	    requestPackage();
	} else if (strcmp( requests[requestsDone], "lspackage" ) == 0) {
	    requestLsPackage();
	} else if (strcmp( requests[requestsDone], "stop" ) == 0) {
	    requestHalt();
	} else if (strcmp( requests[requestsDone], "halt" ) == 0) {
	    requestHalt();
	} else if (strcmp( requests[requestsDone], "intf" ) == 0) {
	    requestIntf();
	} else if (strcmp( requests[requestsDone], "pace" ) == 0) {
	    requestPace();
	} else if (strcmp( requests[requestsDone], "load" ) == 0) {
	    requestLoad();
	} else if (strcmp( requests[requestsDone], "print" ) == 0) {
	    requestPrint();
	} else if (strcmp( requests[requestsDone], "alertinit" ) == 0) {
	    requestAlertInit();
	} else if (strcmp( requests[requestsDone], "verbose" ) == 0) {
	    requestVerbose();
	} else if (strcmp( requests[requestsDone], "tcpstats" ) == 0) {
	    requestStats("tcp");
	} else if (strcmp( requests[requestsDone], "intfstats" ) == 0) {
	    requestStats("intf");
	} else if (strcmp( requests[requestsDone], "pacestats" ) == 0) {
	    requestStats("pace");
	} else if (strcmp( requests[requestsDone], "packetstats" ) == 0) {
	    requestStats("packet");
	} else if (strcmp( requests[requestsDone], "recorderstats" ) == 0) {
	    requestStats("recorder");
	} else if (strcmp( requests[requestsDone], "funcstats" ) == 0) {
	    requestStats("func");
	} else if (strcmp( requests[requestsDone], "memstats" ) == 0) {
	    requestStats("mem");
	} else if (strcmp( requests[requestsDone], "pollstats" ) == 0) {
	    requestStats("poll");
	} else {
	    fprintf( stderr, "Unknown request: `%s'\n",
		    requests[requestsDone] );
	    requestsDone++;
	}
    }
    if (requestsDone == requestsNdx) {
	if (State == READY) {
	    /* we're done.  Maybe we ought to keep track of success? */
	    exit(0);
	}
    }
}



/*
** --------------------
**
** main.
**
*/


main( argc, argv )
int argc;
char * argv[];
{
    argv0 = argv[0];

    {
	int i;
	for (i=1; i<argc; ++i) {
	    if (argv[i][0] == '-') switch(argv[i][1]) {
		case 'l':	/* how to log. */
		    if (argv[i][2]) {
			logFile = &argv[i][2];
		    } else {
			logFile = argv[++i];
		    }
		    break;

		case 't':	/* timeout */
		    if (argv[i][2]) {
			timeOut = atoi( &argv[i][2] );
		    } else {
			timeOut = atoi( argv[++i] );
		    }
		    break;

		case 'd':	/* debug */
		    dFlag++;
		    break;
		
		case 'V':	/* verbose */
		    vFlag++;
		    break;

		case 'q':	/* be very very quiet */
		    qFlag++;
		    break;

		case 'f':	/* name of the fifo. */
		    if (argv[i][2]) {
			nfrFifo = &argv[i][2];
		    } else {
			nfrFifo = argv[++i];
		    }
		    break;

		default:
		usage:
		    Usage:
		    fprintf( stderr, "usage: ctlnfrd -f fifo-path [ load fileName ] [ stats ] [ package packageName ]\n\t[ start ] [ stop ] [ pace paceFifoName ]\n\t[ verbose [ on | off ] ] [ tcpstats ] [ intfstats ]\n\t[ pacestats ] [ packetstats ] [ funcstats ] [alertinit]\n" ); 
		    exit(1);
	    } else {
		if (requestsNdx >= requestsMax) {
		    requestsMax += 10;
		    if (requestsNdx) {
			requests = (char **)realloc( requests,
				requestsMax * sizeof(*requests) );
		    } else {
			requests = (char **)calloc( sizeof(*requests), requestsMax );
		    }
		}
		requests[requestsNdx++] = argv[i];
	    }
	}
    }

    if (requestsNdx == 0) {
	goto Usage;
	exit(1);
    }

    logOpen(0);

    pollInit();
    pollOpen();

    /*
    ** Create the command/control processing stream.
    */
    controlInit();
    if (nfrFifo == 0) {
	nfrFifo = DEFAULT_NFRD_FIFO;
    }

    {
	char * name = 0;
	if (controlFifoOpen( name = nfrPath( nfrFifo, 0 ) ) == 0) {
	    char * second;
	    if (controlFifoOpen( second = nfrPath( SECOND_DEFAULT_NFRD_FIFO, 0 ) ) == 0) {
		/*
		** Don't give up until we get to command processing.
		** we have the tools to perform a recursive descent looking for
		** the fifo.  Its not a great idea, tho, cause someone could
		** have started ctlnfrd at the top of a very deep tree.
		*/
		fprintf( stderr, "Couldn't open connection to engine: %s or %s\n",
			name, second );
		State = NONFRD;
	    } else {
		State = READY;
	    }
	    free(second);
	} else {
	    State = READY;
	}
	free(name);
    }

    while (restart == 0) {
	performRequest();
	pollPerform( timeOut );
    }

    pollClose();
    if (nfrFifo) {
	controlClose();
    }

    /*
    ** We ought to save our global state once more, just before we exit...
    */

    /*
    ** normal exit.
    */
    exit(0);
}
