/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/


#include	"nfrd.h"


#define  ESC 0x1b

/*
** ----------------------------------------
** 
*/

int
dumpGetw( a )
unsigned char * a;
{
    return ( ((a[0]&0xff) << 24) | ((a[1]&0xff) << 16)
		| ((a[2]&0xff) << 8) | (a[3]&0xff));
}

int
dumpGets( a )
unsigned char * a;
{
    return ( ((a[0]&0xff) << 8) | (a[1]&0xff));
}


/*
** ----------------------------------------
** dump.
** n - offset. (what the left side column prints)
** a - address in memory.
** l - length.
*/

dump( int n, int * a, int l )
{
    if (l == 0) {
	    return;
    } else {
	char * base = (char *)a;
	int i;

	for (i=0; i<l; i += sizeof(*a) ) {
	    if ((i &0xf) == 0) {
		if (i) {
		    int j;
		    uiCurrPrintf( "\t" );
		    for (j=0; j < 16; j++) {
			    if (isprint(base[j]&0x7f)) {
				uiCurrPrintf( "%c", base[j]&0x7f );
			    } else {
				uiCurrPrintf( "." );
			    }
		    }
		    uiCurrPrintf( "\n" );
		}
		    base = (char *)a;
		    uiCurrPrintf( "%4.4x: ", n  );
		}
	    uiCurrPrintf( " %8.8x", dumpGetw(a++) );
	    n += sizeof(*a);
	}

	/*
	** Last line of output. Trailing spaces, and the characters.
	*/
	{
	    int j;
	    while((i &0xf) != 0) {
		uiCurrPrintf( "         " );
		i += sizeof(int);
	    }

	    uiCurrPrintf( "\t" );
	    for (j = 0; j < (l % 16); j++) {
		if (isprint(base[j]&0x7f) && (base[j] != ESC)) {
		    uiCurrPrintf( "%c", base[j]&0x7f);
		} else {
		    uiCurrPrintf( "." );
		}
	    }
	}
    }
}

/*
** ----------------------------------------
** displayTimeval
**
*/

displayTimeval( timeval_t * tv )
{
    uiCurrPrintf( "time: %d.%6.6d", tv->tv_sec, tv->tv_usec );
}


/*
** ----------------------------------------
** displayEnetMac
**
*/

displayEnetMac( unsigned char * cp )
{
    if (cp) {
	int i;
	uiCurrPrintf( "%2.2x", *cp++ );
	for (i = 1; i < 6; i++) {
	    uiCurrPrintf( ":%2.2x", *cp++ );
	}
    } else {
	uiCurrPrintf( "enet: none" );
    }
}

/*
** ----------------------------------------
** displayIpAddr.
**
** display the ip address of some entity (in dec/hex)
** Next: include machine name.
*/

displayIpAddr( unsigned char * cp )
{
    int i;
    unsigned char * tp = cp;
    uiCurrPrintf( "%d", *tp++ );
    for (i = 1; i < 4; i++) {
	uiCurrPrintf( ".%d", *tp++ );
    }
    uiCurrPrintf( " [%x", *cp++ );
    for (i = 1; i < 4; i++) {
	uiCurrPrintf( ".%x", *cp++ );
    }
    uiCurrPrintf( "] " );
}

/*
** ----------------------------------------
** displayEtherArp
**
*/
displayEtherArp( struct ether_arp * ep )
{
    uiCurrPrintf( " sha " ); displayEnetMac( (unsigned char *)&ep->arp_sha );
    uiCurrPrintf( " spa " ); displayIpAddr( (unsigned char *)&ep->arp_spa );
    uiCurrPrintf( " tha " ); displayEnetMac( (unsigned char *)&ep->arp_tha );
    uiCurrPrintf( " tpa " ); displayIpAddr( (unsigned char *)&ep->arp_tpa );
}

/*
** ----------------------------------------
** displayArp
**
*/

displayArp( struct ether_arp * ep )
{
    struct arphdr * ap = &ep->ea_hdr;
    uiCurrPrintf( "\nARP: " );
    switch( ap->ar_hrd ) {
	case ARPHRD_ETHER:
	    uiCurrPrintf( "ether " );
	    break;
	case 802:
	    uiCurrPrintf( "802 " );
	    break;
	default:
	    uiCurrPrintf( "form:%x ", ap->ar_hrd );
	    break;
    }
    uiCurrPrintf( "ar_pro %x ", ap->ar_pro );
    uiCurrPrintf( "ar_hln %d ", ap->ar_hln );
    uiCurrPrintf( "ar_pln %d ", ap->ar_pln );
    switch( ap->ar_op ) {
	case ARPOP_REQUEST:
	    displayEtherArp( ep );
	    uiCurrPrintf( "arpop_request" );
		break;
	case ARPOP_REPLY:
	    displayEtherArp( ep );
	    uiCurrPrintf( "arpop_reply" );
		break;

#ifdef REVARP_REQUEST
	case REVARP_REQUEST:
#endif
#ifdef ARPOP_REVREQUEST
	case ARPOP_REVREQUEST:
#endif
	    displayEtherArp( ep );
	    uiCurrPrintf( "revarp_request" );
	    break;
#ifdef REVARP_REPLY
	case REVARP_REPLY:
#endif
#ifdef ARPOP_REVREPLY
	case ARPOP_REVREPLY:
#endif
	    displayEtherArp( ep );
	    uiCurrPrintf( "revarp_reply" );
	    break;
	default:
	    uiCurrPrintf( "op:%x", ap->ar_op );
	    break;
    }
}

/*
** ----------------------------------------
** displayUdp
**
*/

displayUdp( int shost, int dhost, struct udphdr * up )
{
    uiCurrPrintf( "UDP " );
    uiCurrPrintf( "sport %d[%x] ", ntohs(up->uh_sport), ntohs(up->uh_sport) );
    uiCurrPrintf( "dport %d[%x] ", ntohs(up->uh_dport), ntohs(up->uh_dport) );
    uiCurrPrintf( "length %d[%x] ", ntohs(up->uh_ulen), ntohs(up->uh_ulen) );
    uiCurrPrintf( "cksum %d[%x] ", ntohs(up->uh_sum), ntohs(up->uh_sum) );
}

/*
** ----------------------------------------
** displayTcpFlags
**
*/

char *
displayTcpFlags( x )
{
    static char res[128];
    int some = 0;

    res[0] = 0;

    if (x & TH_FIN) {
	if (some++)
	    strcat( res, "," );
	strcat( res, "FIN" );
    }
    if (x & TH_SYN) {
	if (some++)
	    strcat( res, "," );
	strcat( res, "SYN" );
    }
    if (x & TH_RST) {
	if (some++)
	    strcat( res, "," );
	strcat( res, "RST" );
    }
    if (x & TH_PUSH) {
	if (some++)
	    strcat( res, "," );
	strcat( res, "PUSH" );
    }
    if (x & TH_ACK) {
	if (some++)
	    strcat( res, "," );
	strcat( res, "ACK" );
    }
    if (x & TH_URG) {
	if (some++)
	    strcat( res, "," );
	strcat( res, "URG" );
    }

    return(res);
}

/*
** ----------------------------------------
** displayTcp
**
*/
displayTcp( struct in_addr * shost, struct in_addr * dhost, struct tcphdr * tp )
{
    uiCurrPrintf( "TCP " );
    uiCurrPrintf( "src-port %d[%x] ", ntohs(tp->th_sport), ntohs(tp->th_sport) );
    uiCurrPrintf( "dst-port %d[%x] ", ntohs(tp->th_dport), ntohs(tp->th_dport) );
    uiCurrPrintf( "seq %u[%x] ", ntohl(tp->th_seq), ntohl(tp->th_seq) );
    uiCurrPrintf( "ack %u[%x] ", ntohl(tp->th_ack), ntohl(tp->th_ack) );
    uiCurrPrintf( "off %d[%x] ", tp->th_off, tp->th_off );
    uiCurrPrintf( "fls [%s] ", displayTcpFlags(tp->th_flags) );
    uiCurrPrintf( "win %d[%x] ", ntohs(tp->th_win), ntohs(tp->th_win) );
    uiCurrPrintf( "sum %d[%x] ", ntohs(tp->th_sum), ntohs(tp->th_sum) );
    uiCurrPrintf( "urp %d[%x] ", ntohs(tp->th_urp), ntohs(tp->th_urp) );
}

int	IP_ident;

struct ip ip_hdr[1];


/*
** ----------------------------------------
** displayIp
**
*/
displayIp( struct ip * ip )
{
    struct list * pairList;
    int hlen;
    int plen;

    memcpy(ip_hdr,ip,sizeof(struct ip));

    hlen = ip->ip_hl * 4;

    IP_ident = ip->ip_id;
    plen = ntohs(ip->ip_len) & 0xffff;

    uiCurrPrintf( "\nIP hl:%d(%d) v:%d ", ip->ip_hl,  hlen, ip->ip_v );
    uiCurrPrintf( "tos:%x length:%x id:%x off:%x(%x) ttl:%x sum:%x\n",
	    ip->ip_tos, plen, ip->ip_id,
	    ntohs(ip->ip_off) & 0x1fff, (ntohs(ip->ip_off) & 0x1fff) * 8,
	    ip->ip_ttl, ip->ip_sum );
    uiCurrPrintf( "src " ); displayIpAddr( (unsigned char *)&(ip->ip_src.s_addr ) );
    uiCurrPrintf( "dst " ); displayIpAddr( (unsigned char *)&(ip->ip_dst.s_addr ) );

    switch( ip->ip_p ) {
	case IPPROTO_IP:
	    uiCurrPrintf( "proto:ip " );
	    break;
	case IPPROTO_ICMP:
	    uiCurrPrintf( "proto:icmp " );
	    break;
	case IPPROTO_GGP:
	    uiCurrPrintf( "proto:ggp " );
	    break;
	case IPPROTO_TCP:
	    uiCurrPrintf( "proto:tcp " );
	    {
		struct tcphdr * tp;
		struct in_addr * shost = &ip->ip_src;
		struct in_addr * dhost = &ip->ip_dst;

		tp = (struct tcphdr *) (((char *)(ip)) + hlen);
		displayTcp( shost, dhost, tp );
	    }
	    break;
	case IPPROTO_EGP:
	    uiCurrPrintf( "proto:egp " );
	    break;
	case IPPROTO_PUP:
	    uiCurrPrintf( "proto:pup " );
	    break;
	case IPPROTO_UDP:
	    uiCurrPrintf( "proto:udp\n" );
	    {
		struct udphdr * up;
		int shost;
		int dhost;

		shost = ntohl( ip->ip_src.s_addr );
		dhost = ntohl( ip->ip_dst.s_addr );

		up = (struct udphdr *) (((char *)(ip)) + hlen);
		displayUdp( shost, dhost, up );
		uiCurrPrintf( "\n" );
	    }
	    break;
	case IPPROTO_IDP:
	    uiCurrPrintf( "proto:idp " );
	    break;
	case IPPROTO_RAW:
	    uiCurrPrintf( "proto:raw " );
	    break;
	}
}

/*
** This routine is called for each of the packets on the list.
**
*/

timeval_t  baseTime;
timeval_t  thisTime;

/*
**
**
*/
struct ether_header enet_hdr[1];

struct packet {
    struct  ether_header pk_eheader;
    union   {
	struct ether_arp pk_arphdr;
	    struct ip_pkt {
	    struct ip pk_iphdr ;
	} ip_struct ;
    } ether_data ;

};

struct eth8023 {
    char eth8023_src[6];
    char eth8023_dst[6];
    char eth8023_len[2];
    char eth8023_type[2];
};

/*
** ----------------------------------------
** display.
** 
*/
display( struct packet * pp, int pl, timeval_t * tp )
{
    struct ether_header * eth;
    if (!timerisset( &baseTime )) {
	/* set the timer to the base value */
	baseTime = *tp;
    }
    /*
    ** Figure out the difference in time...
    */
    thisTime = *tp;
    util_diffTv( &thisTime, &baseTime );

    uiCurrPrintf("%d.%3.3d[%d] d[", thisTime.tv_sec, thisTime.tv_usec / 1000, pl );

    eth = (struct ether_header *)pp;
    displayEnetMac( (unsigned char *)&(eth->ether_dhost) );

    uiCurrPrintf( "] s [" );
    displayEnetMac( (unsigned char *)&(eth->ether_shost) );


    /*
    ** copy the information from the about the ethernet addresses
    ** into the the header that we can pass the marshalling routines
    */


    memcpy( &(enet_hdr->ether_dhost), (char *)&(eth->ether_dhost),
	  sizeof(enet_hdr->ether_dhost));

    memcpy( &(enet_hdr->ether_shost), (char *)&(eth->ether_shost),
	  sizeof(enet_hdr->ether_shost));
    enet_hdr->ether_type = (eth->ether_type);

    uiCurrPrintf( "] " );
    if (dumpGets(&eth->ether_type) <= ETHERMTU) {
	/* 802.3 */
	struct eth8023 * e = (struct eth8023 *)eth;
	uiCurrPrintf( "802.3 " );
	switch (ntohs(e->eth8023_type[0] << 8 | e->eth8023_type[1])) {
	    case ETHERTYPE_PUP:
		uiCurrPrintf( "pup" );
		break;

	    case ETHERTYPE_IP:
		uiCurrPrintf( "ip" );
		displayIp( (struct ip *)((char *)&e->eth8023_type[2]) );
		break;

	    case ETHERTYPE_ARP:
		uiCurrPrintf( "arp" );
		displayArp( (struct ether_arp *)((char *)&e->eth8023_type[2]) );
		break;

#ifdef ETHERTYPE_REVARP	/* not present on all systems */
	    case ETHERTYPE_REVARP:
		uiCurrPrintf( "revarp" );
		break;
#endif

	    default:
		uiCurrPrintf( "type %x", dumpGets( &e->eth8023_type ) );
		break;
	}

    } else {
	switch (dumpGets(&eth->ether_type)) {
	    case ETHERTYPE_PUP:
		uiCurrPrintf( "pup" );
		break;

	    case ETHERTYPE_IP:
		    uiCurrPrintf( "ip" );
		    displayIp( (struct ip *)&(((short *)&eth->ether_type)[1]) );
		    break;

	    case ETHERTYPE_ARP:
		    uiCurrPrintf( "arp" );
		    displayArp( (struct ether_arp *)&(((short *)&eth->ether_type)[1]));
		    break;

#ifdef ETHERTYPE_REVARP	/* not present on all systems */
	    case ETHERTYPE_REVARP:
		    uiCurrPrintf( "revarp" );
		    break;
#endif

	    default:
		    uiCurrPrintf( "type %x", dumpGets(&eth->ether_type) );
		    break;
	}
    }
    uiCurrPrintf( "\n" );
    return(0);
}

/*
** ----------------------------------------
** Display.
** external inteface to view the current packet 
**
*/

extern char *                   packet_data;
extern int                      packet_len;
extern struct pcap_pkthdr *     packet_hdr;
extern int                      packet_nonce;

Display( include_hex, msg )
int include_hex;
char * msg;
{
    uiCurrPrintf(( "Display: %s\n", msg ));
    if (packet_data) {
	display( (struct packet *)packet_data, packet_len, &packet_hdr->ts );
	packetDisplay();
	ethDisplay();
	ipDisplay();
	udpDisplay();
	tcpDisplay();
	if (include_hex) {
	    uiCurrPrintf( "packet: nonce %d len %d\n", packet_nonce, packet_len );
	    dump( (int)0, ((int *)packet_data), packet_len );
	    uiCurrPrintf( "\n" );
	}
    } else {
	uiCurrPrintf(( "No packet data to display.\n" ));
    }
}
