/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/


/*
** home.
** Small cover to determine home path.
*/

#include "nfrd.h"

/*
** Pass in b.  If b is null, then
** we pass back null, if b is "~",
** or "~/<something>",
** then we return $NFRHOME.  If NFRHOME
** doesn't exist, we try HOME.  If HOME
** doesn't exist, we try ~ from passwd.
** If the entry doesn't exist in /etc/passwd,
** then return b.
*/

static char nfrHomeBuff[256];

char *
nfrHome( b )
char * b;
{
    if (b) {
	if ((b[0] == '~')) {
	    /* compute the new string. */
	    char * suff;
	    if ((suff = getenv("NFRHOME")) == 0) {
		if ((suff = getenv("HOME")) == 0) {
		    struct passwd * p;
		    if ((p = getpwuid( getuid() )) == 0) {
			/* now what? ... use "~" */
			return(b);
		    } else {
			if (b[1] == '/') {
			    b++;
			}
			if (b[1]) {
			    sprintf( nfrHomeBuff, "%s/%s", p->pw_dir, &b[1] );
			    return(nfrHomeBuff);
			} else {
			    return(p->pw_dir);
			}
		    }
		}
	    }
	    if (suff) {
		if (b[1] == '/') {
		    b++;
		}
		if (b[1]) {
		    sprintf( nfrHomeBuff, "%s/%s", suff, &b[1] );
		    return(nfrHomeBuff);
		} else {
		    return(suff);
		}
	    }
	} else {
	    return(b);
	}
    } else {
	return(0);
    }
}

/*
** nfrPath.
** 
** Pass in a path's to contatenate... the first parameter
** is the base path, which can include the '~' prefix,
** indicating home of the nfr.
**
** Return the catt-ed string, which is malloced.
** Make any of the directores
** in the path starting at the home directory.  Return
** null in failure.
** 
*/

char *
nfrPath( char * s, ... )
{
    va_list ap;
    char * home = nfrHome(s);
    int len = strlen(home);
    char * p;	/* path element we're working on */
    char * v;	/* value we'll return */
    char * a;	/* where we're appending into v */

    va_start( ap, s );
    while( p = va_arg(ap, char *) ) {
	len += strlen(p);
    }
    va_end(ap);


    /*
    ** Now construct the path.
    */
    v = (char *)calloc( len+1, sizeof(*v) );
    if (v == 0) {
	return(0);
    }

    a = v;
    p = home;	/* start appending using this string. */
    va_start( ap, s );
    do {
	/*
	** Add a character at a time. When we find '/',
	** then we try stating that dir.  If its not there,
	** we mkdir it... if that fails, we free 'v',
	** and return zero.
	*/
	while(*p) {
	    if (*p == '/') {
		/* time to try opening the dir. */
		struct stat s[1];

		if (v[0] && (stat( v, s ) < 0)) {
		    if (errno == ENOENT) {
			int fd;
			if ((fd = mkdir( v, 0755 )) < 0) {
			    Alert_Args_Generate( NFR_SRC_NFRD, NFR_ERR_CANTMKDIR, v, 0 );
			    free(v);
			    return(0);
			}
			close(fd);
		    } else {
			Alert_Args_Generate( NFR_SRC_NFRD, NFR_ERR_CANTSTAT, v, 0 );
			free(v);
			return(0);
		    }
		}
		*a++ = '/';
		while(*p == '/') {
		    p++;
		}
	    } else {
		*a++ = *p++;
	    }
	}

    } while( p = va_arg(ap, char *) );
    va_end(ap);
    return(v);
}

/*
** just like nfrPath, but without the mkdir's for interior
** directories....
*/


char *
nfrConcat( char * s, ... )
{
    va_list ap;
    char * home = nfrHome(s);
    int len = strlen(home);
    char * p;	/* path element we're working on */
    char * v;	/* value we'll return */
    char * a;	/* where we're appending into v */

    va_start( ap, s );
    while( p = va_arg(ap, char *) ) {
	len += strlen(p);
    }
    va_end(ap);


    /*
    ** Now construct the path.
    */
    v = (char *)calloc( len, sizeof(*v) );
    if (v == 0) {
	return(0);
    }
    a = v;
    while( *home ) {
	*a++ = *home++;
    }

    va_start( ap, s );
    while( p = va_arg(ap, char *) ) {
	/*
	** Add a character at a time. When we find '/',
	** then we try opening that dir.  If its not there,
	** we mkdir it... if that fails, we free 'v',
	** and return zero.
	*/
	while(*p) {
	    if (*p == '/') {
		*a++ = '/';
		while(*p == '/') {
		    p++;
		}
	    } else {
		*a++ = *p++;
	    }
	}

    }
    va_end(ap);
    return(v);
}
