/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/


#include "nfrd.h"

#include <syslog.h>

/*
**
** Cover for log file managent.
**
** Date stamp each of the output lines,
** mark each with the name of the application...
*/

extern char * argv0;		/* name of the application */

FILE * logFILE;
int	logSyslog = 0;


/*
** ----------------------------------------------------------------------
**
** logTimeStamp.
** Return the time in a form which we can use to mark every log message.
*/
 
static char logTS[128];         /* ought to be way way big */
 
char *
logTimeStamp()
{
    struct timeval tp[1];
    char tBuf[64];

    if (gettimeofday( tp, 0 ) != 0) {
	return( "<no time>" );
    }
    strftime(tBuf, sizeof(tBuf), "%D %T", localtime( &tp->tv_sec ) );
    sprintf( logTS, "%s.%2.2d", tBuf, tp->tv_usec / 10000 ); /* only to hundredths */
    return(logTS);
}

/*
** ----------------------------------------------------------------------
**
** log.
*/

log( char * file, int line, char * fmt, ... )
{
    va_list ap;
    char ** args;
    va_start(ap, fmt);

    if (logSyslog) {
	char buffer[1024];
	vsprintf( buffer, fmt, ap );
	if (vFlag) {
	    syslog( LOG_ERR, "%s:%s:%d: %s", argv0, file, line, buffer );
	} else {
	    syslog( LOG_ERR, "%s:%s", argv0, buffer );
	}
    } else {
	if (logFILE == 0) {
	    fprintf(stderr, "%s %s:%d: log: logFILE not open\n", logTimeStamp(), FL );
	    logFILE = stderr;
	}
	if (vFlag) {
	    fprintf( logFILE, "%s %s:%s:%d: ", logTimeStamp(), argv0, file, line );
	} else {
	    fprintf( logFILE, "%s ", logTimeStamp() );
	}
	vfprintf( logFILE, fmt, ap );
    }
}

/*
** ----------------------------------------------------------------------
**
** logCont.
**
** Although we get file/line info, we don't dump it :^(
*/

logCont( char * file, int line, char * fmt, ... )
{
va_list ap;
char ** args;
va_start(ap, fmt);

    if (logSyslog) {
	/*
	** we don't have any choice but to place each of the additional
	** data onto separate lines.
	*/
	char buffer[1024];
	vsprintf( buffer, fmt, ap );

	/* hey, no reason to just jam a null line */
	if ((buffer[0] == '\n') && (buffer[1] == 0)) {
	    return;
	}
	if (vFlag) {
	    syslog( LOG_ERR, "%s:%s:%d: %s", argv0, file, line, buffer );
	} else {
	    syslog( LOG_ERR, "%s:%s", argv0, buffer );
	}
    } else {
	if (logFILE == 0) {
	    fprintf(stderr, "%s:%d: log: logFILE not open\n", FL );
	}
	vfprintf( logFILE, fmt, ap );
    }
}

/*
** ----------------------------------------------------------------------
**
** logOpen.
**
** the file name is the only argument.
** if the file name is "syslog", we'll direct all the output for
** the log into syslog.
**
*/
logOpen( name )
char * name;
{
    if (name == 0) {
	name = "-";
    }

    if (strcmp(name, "syslog" ) == 0) {
	/*
	** add code to direct log output to syslog...
	*/
	logSyslog = 1;
    }
    if (strcmp(name, "-") == 0) {
	logFILE = stdout;
    }
    else {
	logFILE = fopen( name, "w" );
	if (logFILE == NULL) {
	    fprintf( stderr, "%s: can't open log, file :", argv0 );
	    perror( name );
	    return(1);
	}
	/*
	** We want to be able to tail -f the log.
	*/
	nfr_setlinebuf(logFILE);
    }
}

/*
** ----------------------------------------------------------------------
**
** logClose.
*/

logClose()
{
    if (logSyslog == 0) {
	if (logFILE) {
	    fclose(logFILE);
	    logFILE = 0;
	}
    }
}
