/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/

%{
#include	<stddef.h>
#include	<stdio.h>
#include	<ctype.h>
#include	<string.h>

#include	"nfrd.h"


/*
**	Compiler and lexical analyzer for the interpreter.
**  Some miscellaneous memory management routines related to
**  compilation are also here. This file uses standard (V7)
**  yacc format.
**
**	Marcus J. Ranum, Network Flight Recorder, Inc. 1996
*/


int		yylex();
static int	assemble(Filter *,int);
static Filter	*cfilt_init();
extern void	yyerror(char *);
u_int		yylineno;
char *		yyFileName;	/* convience, we don't do the opens */

/*
**
*/
yylinenoReset() {
    yylineno = 1;
}



/* voluminous compile/assemble time debugging
#define	NFR_COMPILEDEBUG
*/

#ifdef	NFR_COMPILEDEBUG
static void	assemble_printop(int);
#endif


static int	compiler_state = NFR_COMPILE_FILT;
static Filter	*c_filt = (Filter *)0;
static int	c_filt_memmax;		/* highwater compile memory space */
static int	c_filt_listmax;		/* highwater compile list space */

Machine *	performMac = 0;

/*
** scope name.
** Set specifically by using the scope name.
** Once set, the using name gets connected to
** any function set, so that an unload request can
** unload all related functions.
**
** Until the scope command is issued, the defined
** scope is global
*/

#define	GLOBALSCOPE	"global"

char*	scope;
int	scopeCnt = 0;

/*
** External interfaces to set and get scope.
*/
char *
scopeSet( s )
char * s;
{
    extern char * scope;
    char * os = scope;
    scope = s;
    return(os);
}
 
char *
scopeGet()
{
    return(scope);
}

scopeInit()
{
    scope = strdup(GLOBALSCOPE);
}


Filter *globalFilter;			/* initialized in globalFilterInit */
char *	globalScope = GLOBALSCOPE;
int	globalScopeNdx = 0;

globalFilterInit() {
    /*  
    ** we create a global filter.  Its not really a filter,
    ** but a sheel to hold some info, which we use to find
    ** symbols.
    */

    globalFilter = (Filter *)calloc( 1, sizeof(*globalFilter) );
    globalFilter->f_name = strdup("globalFilter");
    globalFilter->f_scope = strdup(globalScope);
    globalFilter->f_scopeNdx = strHash(globalFilter->f_scope);
    globalFilter->f_fromFile = strdup( "<none>" );
    globalScopeNdx = globalFilter->f_scopeNdx;
}

/*
** housekeeping routine to allow local execution
** of something jsut compiled.
**
*/
int
local_perform( Userintf_t * ui )
{
    Filter *nf;
    int err;

    nf = get_compiled_filter( ui );
    /* don't add the assignment, just set the global variables */
    if (performMac == 0) {
	performMac = machinenew();
    }
    machinereset(performMac);
    if (nf->f_indSymCnt) {
	symFixIndirect(nf->f_syms);
	nf->f_indSymCnt = 0;
    }
    err = machinerun(performMac, nf, 0 );
    free_compiled_filter(nf);
    return(err);
}

int
performInit()
{
    if (performMac == 0) {
	performMac = machinenew();
	if (performMac == 0) {
	    log( FL, "performInit: can't make machine\n" );
	    return;
	}
    }
}


/*
** performArgs
** This time, we pass arguments to the performing function.
** The arguments are MemVal *'s and usually come from
** memTmpNew...
*/


int
performArgs( char * func, char * scope, ... )
{
    Filter * funcp;

    funcp = funcfind( func, scope );
    if (funcp) {
	timeval_t run_start, run_done, run_time;
	Userintf_t * old;
	va_list args;
	MemVal * m;
	int cnt = 0;
	int err;

	va_start(args, scope);

	if (funcp->f_ui) {
	    old = uiSetCurr( funcp->f_ui );
	}

	if (performMac == 0) {
	    performMac = machinenew();
	    if (performMac == 0) {
		log( FL, "perform: can't make machine\n" );
		return;
	    }
	}
	machinereset(performMac);

	if (funcp->f_indSymCnt) {
	    symFixIndirect(funcp->f_syms);
	    funcp->f_indSymCnt = 0;
	}

	while( m = va_arg(args, MemVal *) ) {
	    stack_push( performMac, m );
	    cnt++;
	}

	frame_push( performMac, performMac->m_sp - cnt, cnt, 0 );
	funcp->f_useCount++;
	gettimeofday( &run_start, 0 );
	err = machinerun( performMac, funcp, 0 );
	gettimeofday( &run_done, 0 );
	run_time = util_diffTv( &run_done, &run_start );

	{
	    double diff = (util_tv2usec(&run_time)/MILLION);
	    diff *= diff;
	    funcp->f_sumSq += diff;
	    util_incrTv( &funcp->f_runTime, &run_time );
	    if (funcp->f_useCount > 1) {
		funcp->f_stdDev = util_stdDev(
		    util_tv2usec(&funcp->f_runTime)/funcp->f_useCount/MILLION,
		    funcp->f_sumSq, funcp->f_useCount );
	    }
	}

	if (funcp->f_ui) {
	    uiSetCurr( old );
	}

	va_end(args);

	tmpfree();

	return(err);
    } else {
	/*
	** We may need more work here.  This message could appear 
	** very often when some bad bugs happen.  Maybe a small
	** cache of the recent 20 or so failures would do the trick...
	*/
	log( FL, "performArgs: can't find %s:%s\n", scope, func );
	return(-1);
    }
}

/*
** perform
** return run-state (NFR_RUN_OK for success),
** -1 if we can't find the funcion.
*/


int 
perform( func )
char * func;
{
    return(performArgs( func, 0, 0 ));
}


/*
**
** A flag, to determine whether we are just parsing (cause we
** got a syntax error), or whether we can execute...
** 
*/
int	parseState = 0;

#define	PS_OK	1
#define	PS_ERR	2

int
parse() {
parseState = PS_OK;
return(yyparse());
}

int
parseOk() {
return(parseState == PS_OK);
}

parseFailed() {
parseState = PS_ERR;
}

parseReset() {
parseState = PS_OK;
}


/*
** How to decide if we execute the code...
*/

#define RS_RUN	1
#define	RS_DONT	2
#define	RS_INFO	3

int runState = RS_RUN;

runSetDont()
{
    runState = RS_DONT;
}

runSetInfo()
{
    runState = RS_INFO;
}

runOk()
{
return (parseOk() && (runState == RS_RUN));
}

runInfo()
{
return(parseOk() && (runState == RS_INFO));
}

/*
** for the yys struct/union which follows.
*/

#define	ival	yys.yyu.Ival
#define	cval	yys.yyu.Cval
#define	mval	yys.yyu.Mval
#define	tval	yys.yyu.Tval
#define part	yys.Part

#define	PARSE(x)	yylval.part = x;

/*
** make an op.
** We jam the line number at the high part of the op, and the actual
** op goes in the low part.  See nfr.h for more details.
*/

#define MAKE_OP(x)	(yylineno < 0xffff ? \
    ((x) | (yylineno << 16)) : ((x) | (((u_int)0xffff) << 16)) )

%}


%union	{
    struct yys {
	union {
	    int		Ival;
	    char	*	Cval;
	    ListMem	*	Mval;
	    trigger_t	*	Tval;
	} yyu;
	char *		Part;
    } yys;
}

%token	NUM STRING ETHMAC IDENT BLTIN
%token	IF ELSE FUNC FILTER RETURN
%token  RECORD TO ON OFF CALL
%token	DECLARE INSIDE GLOBAL
%token	FOREACH WHILE BREAK CONTINUE ISELEMENTOF

%right	ASGN
%left	OR AND BOR BAND LSHIFT RSHIFT XOR INV
%left	GT GTE LT LTE EQ NE ADD SUB MUL DIV OPNUL INSIDE
%right	UNARY
%left	NOT


%type	<mval> STRING IDENT ETHMAC listitems listitem
%type	<ival> NUM OR AND GT GTE LT LTE EQ NE ADD SUB MUL DIV INSIDE
%type	<ival> BOR XOR BAND LSHIFT RSHIFT
%type	<ival> expr asgn gasgn statement statements ifheader condition
%type	<ival> stop beginargs arglist arglistArgs exprlist exprargs
%type 	<ival> trigger triggerArg triggerArgs
%type   <ival> assemble_pc foreachhead while 
%type   <tval> filtertrigger 



%%

program:
    /* nothing - use to initialize our compile buffer */
    {
	if (parseOk()) {
	    /* uiCurrPrintf( "init\n" ); */
	    if(c_filt == (Filter *)0)
		    if((c_filt = cfilt_init()) == (Filter *)0)
			    return(NFR_COMPILE_SYSERR);
	    c_filt->f_memsiz = 0;
	    c_filt->f_listsiz = 0;
	}
    }
    |
    program gasgn ';'
    {
	if (parseOk()) {
	    /* global assignment. */
	    (void)assemble(c_filt,MAKE_OP(NFR_OP_POP));
	    (void)assemble(c_filt,MAKE_OP(NFR_OP_RETNV));
	    if (runOk()) {
		local_perform( uiCurr() );
	    }
	}
    }
    | program IDENT beginargs '(' arglist ')' ';'
    {

	if (parseOk()) {
	    int	x;
	    if((x = memlist_insert($2)) < 0)
		    compiler_state = NFR_COMPILE_SYSERR;
	    (void)assemble(c_filt,MAKE_OP(NFR_OP_CALL));
	    (void)assemble(c_filt,x);
	    (void)assemble(c_filt,$5);
	    (void)assemble(c_filt,MAKE_OP(NFR_OP_RETNV));
	    if (runOk()) {
		local_perform( uiCurr() );
	    }
	}
    }
    | program IDENT beginargs '(' error ';'
    {
	    yyerrok;
	    yyerror( "proc_call:syntax error in procedure call" );
    }
    | program funcdef
    {
	if (parseOk()) {
	    PARSE("program funcdef");
	}
    }
    | program error
    {
	    return(NFR_COMPILE_BAD);
    }
    ;



gasgn:
    IDENT ASGN expr
    {
	if (parseOk()) {
	    if (strchr( $1->data.val.c, ':' ) != 0) {
		uiCurrPrintf( "error: imbedded colon in variable name \"%s\"",
			$1->data.val.c );
		parseFailed();
	    } else {
		int	x;

		$$ = $3;
		if((x = memlist_insert($1)) < 0)
			compiler_state = NFR_COMPILE_SYSERR;
		(void)assemble(c_filt,MAKE_OP(NFR_OP_GASGN));
		(void)assemble(c_filt,x);
		PARSE("IDENT ASGN expr");
		if (runInfo()) {
		    printf( "Global: %s\n", $1->data.val.c );
		}
	    }
	}
    }
    |
    IDENT '[' expr ']' ASGN expr
    {
	if (parseOk()) {
	    if (strchr( $1->data.val.c, ':' ) != 0) {
		uiCurrPrintf( "error: imbedded colon in variable name \"%s\"",
			$1->data.val.c );
		parseFailed();
	    } else {
		int	x;
		$$ = $3;
		if((x = memlist_insert($1)) < 0)
		    compiler_state = NFR_COMPILE_SYSERR;
		(void)assemble(c_filt,MAKE_OP(NFR_OP_ARRAYGASGN));
		(void)assemble(c_filt,x);
		PARSE("IDENT '[' expr ']' ASGN expr");
	    }
	}
    }
    | '(' expr ')' '[' expr ']' ASGN expr
    {
	/*
	** in this case, $2 must evaluate to a MAPBAG,
	** but besides that, it ought to work.
	*/
	if (parseOk()) {
	    int	x;
	    $$ = $2;
	    (void)assemble(c_filt,MAKE_OP(NFR_OP_ARRAYEASGN));
	    PARSE("expr '[' expr ']' ASGN expr");
	}
    }
    ;


asgn:
    '$' IDENT ASGN expr
    {
	if (parseOk()) {
	    if (strchr( $2->data.val.c, ':' ) != 0) {
		uiCurrPrintf( "error: imbedded colon in variable name \"%s\"",
		    $2->data.val.c );
		parseFailed();
	    } else {
		int	x;

		$$ = $4;
		if((x = memlist_insert($2)) < 0)
			compiler_state = NFR_COMPILE_SYSERR;
		(void)assemble(c_filt,MAKE_OP(NFR_OP_LASGN));
		(void)assemble(c_filt,x);
		PARSE("'$' IDENT ASGN expr");
	    }
	}
    }
    | '$' IDENT '[' expr ']' ASGN expr
    {
        if (parseOk()) {
	    if (strchr( $2->data.val.c, ':' ) != 0) {
		uiCurrPrintf( "error: imbedded colon in variable name \"%s\"",
		    $2->data.val.c );
		parseFailed();
	    } else {
		int x;
 
		$$ = $4;
		if((x = memlist_insert($2)) < 0)
                    compiler_state = NFR_COMPILE_SYSERR;
		(void)assemble(c_filt,MAKE_OP(NFR_OP_ARRAYLASGN));
		(void)assemble(c_filt,x);
		PARSE("'$' IDENT '[' expr ']' ASGN expr");
	    }
        }
    }
    ;


statement:
    expr ';'
    {
	if (parseOk()) {
	    (void)assemble(c_filt,MAKE_OP(NFR_OP_POP));
	    PARSE("expr;");
	}
    }
    |
    expr error ';'
    {
	    yyerror( "expression:syntax error in expression statement" );
    }
    | RETURN ';'
    {
	if (parseOk()) {
	    $$ = assemble(c_filt,MAKE_OP(NFR_OP_RETNV));
	    PARSE("RETURN;");
	}
    }
    | RETURN expr ';'
    {
	if (parseOk()) {
	    $$ = $2;
	    (void)assemble(c_filt,MAKE_OP(NFR_OP_RETV));
	    PARSE("RETURN expr;");
	}
    }
    | ifheader condition statement stop
    {
	if (parseOk()) {
	    c_filt->f_mem[$1 + 1] = $3;	/* if part */
	    c_filt->f_mem[$1 + 2] = 0;	/* else part (none) */
	    c_filt->f_mem[$1 + 3] = $4;	/* continuation */
#ifdef	NFR_COMPILEDEBUG
uiCurrPrintf("IF@%d backpatch, ifpart:%d@%d, else:%d@%d, cont:%d@%d\n",
	    $1, c_filt->f_mem[$1+1],$1+1,c_filt->f_mem[$1+2],$1+2,
	    c_filt->f_mem[$1+3],$1+3);
	    PARSE("ifheader <condition> statement stop");
#endif
	}
    }
    | ifheader condition error ';'
    {
	    yyerror( "if_statement:syntax error in if statement" );
    }
    | ifheader condition statement stop ELSE statement stop
    {
	if (parseOk()) {
	    c_filt->f_mem[$1 + 1] = $3;	/* if part */
	    c_filt->f_mem[$1 + 2] = $6;	/* else part */
	    c_filt->f_mem[$1 + 3] = $7;	/* continuation */
#ifdef	NFR_COMPILEDEBUG
uiCurrPrintf("IF@%d backpatch, ifpart:%d@%d, else:%d@%d, cont:%d@%d\n",
	    $1, c_filt->f_mem[$1+1],$1+1,c_filt->f_mem[$1+2],$1+2,
	    c_filt->f_mem[$1+3],$1+3);
#endif
	    PARSE("ifheader condition statement stop ELSE statement stop");
	}
    }
    | ifheader condition statement stop ELSE error ';'
    {
	    yyerror( "if_else_statement:syntax error in if statement (with else clause)" );
    }
    | '{' statements '}'
    {
	if (parseOk()) {
	    $$ = $2;
	    PARSE("'{' statements '}'" );
	}
    }
    | '{' statements error 
    {
	if (parseOk()) {
	    yyerror( "missing_bracket:Statement error; missing end bracket" );
	    $$ = 0;
	}
    }
    | RECORD assemble_pc exprargs TO expr ';'
    {
	if (parseOk()) {
	    (void)assemble(c_filt,MAKE_OP(NFR_OP_RECORD));
	    (void)assemble(c_filt,$3+1); /* save number of expressions plus expra */
	    (void)assemble(c_filt,MAKE_OP(NFR_OP_POP));
	    $$ = $2;
	    PARSE("RECORD assemble_pc exprargs TO '$' IDENT ';'");

	}
    }
    | RECORD assemble_pc exprargs error ';'
    {
	    yyerror( "record:Record syntax error" );
	    $$ = 0;
    }
    | ON trigger CALL IDENT ';'
    {
	if (parseOk()) {
	    int x;
	    if ((x = memlist_insert($4)) < 0)
		    compiler_state = NFR_COMPILE_SYSERR;
	    $$ = assemble(c_filt,MAKE_OP(NFR_OP_SPUSH));
	    (void)assemble(c_filt,x);
	    assemble(c_filt, MAKE_OP(NFR_OP_ON ));
	    PARSE("ON trigger CALL IDENT ';'");
	}
    }
    | OFF expr ';'
    {
	if (parseOk()) {
	    assemble(c_filt, MAKE_OP(NFR_OP_OFF ));
	    $$ = $2;
	    PARSE("OFF expr ';'");
	}
    }
    | DECLARE '$' IDENT INSIDE expr ';'
    {
	if (parseOk()) {
	    int x;
	    if ((x = memlist_insert($3)) < 0)
		    compiler_state = NFR_COMPILE_SYSERR;
	    $$ = assemble(c_filt,MAKE_OP(NFR_OP_DECL_INSIDE));
	    (void)assemble(c_filt, x);
	    PARSE("DECLARE IDENT INSIDE expr ';'");
	}
    }
    | DECLARE '$' IDENT INSIDE expr error ';'
    {
	    yyerror( "declare:syntax error inside declare statement" );
    }
    | DECLARE '$' IDENT GLOBAL ';'
    {
	if (parseOk()) {
	    int x;
	    if ((x = memlist_insert($3)) < 0)
		    compiler_state = NFR_COMPILE_SYSERR;
	    $$ = assemble(c_filt,MAKE_OP(NFR_OP_DECL_GLOBAL));
	    (void)assemble(c_filt, x);
	    PARSE("DECLARE IDENT GLOBAL ';'");
	}
    }
    | FOREACH '$' IDENT INSIDE '(' expr ')'  foreachhead statement stop 
    {
	if (parseOk()) {
	    int x;
	    if ((x = memlist_insert($3)) < 0)
		    compiler_state = NFR_COMPILE_SYSERR;
	    /* backpatch ident index */
	    c_filt->f_mem[$8 + 1] = x;	/* index of the IDENT */
	    /* backpatch stop location */
	    c_filt->f_mem[$8 + 2] = $10;	/* location of instruction after stop */
	    PARSE("FOREACH '$' IDENT INSIDE '(' expr ')'  foreachhead statement stop" );
	}
    }
    | FOREACH '$' IDENT INSIDE '(' expr error ';'
    {
	    yyerror( "foreach:syntax error inside foreach statement" );
    }
    | while '(' expr ')' stop statement stop
    {
	if (parseOk()) {
	    $$ = $1;
	    /* backpatch statement to execute location. */
	    c_filt->f_mem[$1+1] = $6;
	    /* backpatch pass location. */
	    c_filt->f_mem[$1+2] = $7;
	    PARSE("while '(' expr ')' stop statement stop");
	}
    }
    | while '(' expr  error ';'
    {
	    yyerror( "while:error in while loop syntax" );
    }
    |
    BREAK ';'
    {
	if (parseOk()) {
	    $$ = assemble(c_filt, MAKE_OP(NFR_OP_BREAK ));
	    PARSE("BREAK;" );
	}
    }
    ;

foreachhead:
    {
	if (parseOk()) {
	    $$ = assemble(c_filt, MAKE_OP(NFR_OP_FOREACH ));
	    assemble(c_filt, MAKE_OP(NFR_OP_STOP ));	/* body */
	    assemble(c_filt, MAKE_OP(NFR_OP_STOP )); /* loop exit */
	    PARSE("foreachhead");
	}
    }
    ;

while:	WHILE
    {
	if (parseOk()) {
	    $$ = assemble( c_filt, MAKE_OP(NFR_OP_WHILE ));
	    assemble( c_filt, 0 );		/* backfill with statement to execute */
	    assemble( c_filt, 0 );		/* backfill with end of while */
	    PARSE("WHILE");
	}
    }
    ;

trigger:
    IDENT '(' triggerArgs  ')'
    {
	if (parseOk()) {
	    int     x;
	    if((x = memlist_insert($1)) < 0)
		    compiler_state = NFR_COMPILE_SYSERR;
	    (void)assemble(c_filt,MAKE_OP(NFR_OP_SPUSH));
	    (void)assemble(c_filt,x);
	    assemble(c_filt,MAKE_OP(NFR_OP_NPUSH));
	    (void)assemble(c_filt,0);	/* un-named trigger */
	    assemble(c_filt,MAKE_OP(NFR_OP_NPUSH));
	    (void)assemble(c_filt,$3);
	    PARSE("IDENT '(' triggerArgs  ')'" );
	}
    }
    | IDENT '(' triggerArgs  error
    {
	    yyerror( "trigger:trigger syntax error" );
	    $$ = 0;
    }
    | IDENT ASGN IDENT '(' triggerArgs  ')'
    {
	if (parseOk()) {
	    int     x;
	    if((x = memlist_insert($3)) < 0)
		    compiler_state = NFR_COMPILE_SYSERR;
	    (void)assemble(c_filt,MAKE_OP(NFR_OP_SPUSH));
	    (void)assemble(c_filt,x);
	    if((x = memlist_insert($1)) < 0)
		    compiler_state = NFR_COMPILE_SYSERR;
	    (void)assemble(c_filt,MAKE_OP(NFR_OP_SPUSH));
	    (void)assemble(c_filt,x);
	    assemble(c_filt,MAKE_OP(NFR_OP_NPUSH));
	    (void)assemble(c_filt,$5);
	    PARSE("IDENT ASGN IDENT '(' triggerArgs  ')'");
	}
    }
    | IDENT ASGN IDENT '(' triggerArgs error
    {
	    yyerror( "trigger:trigger/assignment syntax error" );
	    $$ = 0;
    }
    ;

triggerArgs:
    {
	if (parseOk()) {
	    $$ = 0;
	    PARSE("triggerArgs<-");
	}
    }
    | triggerArg
    {
	if (parseOk()) {
	    $$ = 1;
	    PARSE("triggerArg");
	}
    }
    | triggerArgs ',' triggerArg
    {
	if (parseOk()) {
	    $$ = $1 + 1;
	    PARSE("triggerArgs ',' triggerArg");
	}
    }
    ;

triggerArg:
    IDENT
    {
	if (parseOk()) {
	    int	x;

	    assemble(c_filt,MAKE_OP(NFR_OP_NPUSH));
	    (void)assemble(c_filt,1);

	    if((x = memlist_insert($1)) < 0)
		    compiler_state = NFR_COMPILE_SYSERR;
	    $$ = assemble(c_filt,MAKE_OP(NFR_OP_SPUSH));
	    (void)assemble(c_filt,x);
	    PARSE("IDENT" );
	}
    }
    |
    IDENT ':' expr
    {
	if (parseOk()) {
	    int	x;

	    if((x = memlist_insert($1)) < 0)
		    compiler_state = NFR_COMPILE_SYSERR;
	    $$ = assemble(c_filt,MAKE_OP(NFR_OP_SPUSH));
	    (void)assemble(c_filt,x);
	    PARSE("IDENT : expr" );
	}
    }
    ;

exprargs:
    {
	if (parseOk()) {
	$$ = 0;
	PARSE("exprargs<-");
	}
    }
    | exprlist
    {
	if (parseOk()) {
	$$ = $1;
	PARSE("exprlist");
	}
    }
	    

exprlist:
    expr
    {
	if (parseOk()) {
	$$ = 1;
	PARSE("exprlist<-");
	}
    }
    | exprlist ',' expr
    {
	if (parseOk()) {
	$$ = $1 + 1;
	PARSE(" exprlist ',' expr");
	}
    }
    ;


condition:
    '(' expr ')'
    {
	if (parseOk()) {
	    $$ = $2;
	    (void)assemble(c_filt,MAKE_OP(NFR_OP_IFCOND));
	    PARSE(" '(' expr ')'" );
	}
    }
    ;



stop:
    /* nothing */
    {
	if (parseOk()) {
	    (void)assemble(c_filt,MAKE_OP(NFR_OP_STOP));
	    $$ = c_filt->f_memsiz;
	    PARSE("stop<-");
	}
    }
    ;

assemble_pc:
    /* nothing */
    {
	if (parseOk()) {
	    $$ = assemble_pc(c_filt);
	    PARSE("assemble_pc<-");
	}
    }


statements:
    /* nothing */
    {
	if (parseOk()) {
	    $$ = assemble_pc(c_filt);
	    PARSE("statements<-");
	}
    }
    | statements statement
    {
	if (parseOk()) {
	    PARSE("statements statement");
	}
    }
    ;


expr:
    NUM
    {
	if (parseOk()) {
	    $$ = assemble(c_filt,MAKE_OP(NFR_OP_NPUSH));
	    (void)assemble(c_filt,$1);
	    PARSE("NUM");
	}
    }
    | NUM '.' NUM '.' NUM '.' NUM
    {
	if (parseOk()) {
	    if (($1 > 256) || ($3 > 256) || ($5 > 256) || ($7 > 256)) {
		uiCurrPrintf( "Too large of a number in dotted host notation\n" );
	    }
	    {
	    int n = htonl( ($1<<24)|($3<<16)|($5<<8)|($7) );
	    $$ = assemble(c_filt,MAKE_OP(NFR_OP_IPV4HOSTPUSH));
	    (void)assemble(c_filt,n);
	    }
	    PARSE("NUM  '.' NUM '.' NUM '.' NUM");
	}
    }
    | NUM '.' error
    {
	    yyerror( "host:syntax error in host address" );
    }
    | NUM '.' NUM '.' NUM '.' NUM ':'  NUM '.' NUM '.' NUM '.' NUM
    {
	if (parseOk()) {
	    if (($1 > 256) || ($3 > 256) || ($5 > 256) || ($7 > 256)) {
		uiCurrPrintf( "Too large of a number in dotted net notation\n" );
	    }
	    if (($9 > 256) || ($11 > 256) || ($13 > 256) || ($15 > 256)) {
		uiCurrPrintf( "Too large of a number in dotted mask notation\n" );
	    }
	    {
	    int n = htonl( ($1<<24)|($3<<16)|($5<<8)|($7) );
	    int m = htonl( ($9<<24)|($11<<16)|($13<<8)|($15) );

	    if ((n & (~m))) {
		uiCurrPrintf( "Warning: network %s:%8.8x, bits in mask\n",
			util_ntoa(n), ntohl(m) );
		n &= m;
	    }

	    $$ = assemble(c_filt,MAKE_OP(NFR_OP_IPV4NETPUSH));
	    (void)assemble(c_filt,n);
	    (void)assemble(c_filt,m);
	    }
	    PARSE("NUM  '.' NUM '.' NUM '.' NUM ':'  NUM '.' NUM '.' NUM '.' NUM" );
	}
    }
    | NUM '.' NUM '.' NUM '.' NUM ':'  error
    {
	yyerror( "net:netword address syntax error" );
	$$ = 0;
    }
    | NUM '.' NUM '.' NUM '.' NUM DIV NUM
    {
	if (parseOk()) {
	    if (($1 > 256) || ($3 > 256) || ($5 > 256) || ($7 > 256)) {
		uiCurrPrintf( "Too large of a number in cidr notation\n" );
	    }
	    if ($9 > 32) {
		uiCurrPrintf( "Too large of a number in cidr net notation\n" );
	    }
	    {
		int n = htonl( ($1<<24)|($3<<16)|($5<<8)|($7) );
		int m = htonl( 0xffffffff << (32-$9) );

		if ((n & (~m))) {
		    uiCurrPrintf( "Warning: network %s:%8.8x, bits in mask\n",
			util_ntoa(n), ntohl(m) );
		    n &= m;
		}

		$$ = assemble(c_filt,MAKE_OP(NFR_OP_IPV4NETPUSH));
		(void)assemble(c_filt,n);
		(void)assemble(c_filt,m);
	    }
	    PARSE("NUM  '.' NUM '.' NUM '.' NUM '/'  NUM " );
	}
    }
    | STRING
    {
	if (parseOk()) {
	    int	x;

	    if((x = memlist_insert($1)) < 0)
		    compiler_state = NFR_COMPILE_SYSERR;
	    $$ = assemble(c_filt,MAKE_OP(NFR_OP_SPUSH));
	    (void)assemble(c_filt,x);
	    PARSE("STRING");
	}
    }
    | OPNUL
    {
	if (parseOk()) {
	    $$ = assemble(c_filt,MAKE_OP(NFR_OP_NULPUSH));
	    PARSE("NUL");
	}
    }
    | ETHMAC
    {
	if (parseOk()) {
	    int x;
	    if((x = memlist_insert($1)) < 0) {
		compiler_state = NFR_COMPILE_SYSERR;
	    }
	    $$ = assemble(c_filt,MAKE_OP(NFR_OP_ETHMACPUSH));
	    assemble(c_filt, x );
	}
    }
    | '$' '#'
    {
	if (parseOk()) {
	    $$ = assemble(c_filt,MAKE_OP(NFR_OP_CPUSH));
	    PARSE("$#");
	}
    }
    | '$' NUM 
    {
	if (parseOk()) {
	    $$ = assemble(c_filt,MAKE_OP(NFR_OP_PPUSH));
	    (void)assemble(c_filt,$2);
	    PARSE("$num");
	}
    }
    | IDENT '.' IDENT
    {
	if (parseOk()) {
	    $$ = assemble(c_filt,MAKE_OP(NFR_OP_PACKET));
	    (void)assemble(c_filt,packet_var($1->data.val.c,$3->data.val.c));
	    listfree($1);
	    listfree($3);
	    PARSE("IDENT.IDENT");
	}
    }
    | '$' IDENT
    {
	if (parseOk()) {
	    int	x;
	    if((x = memlist_insert($2)) < 0)
		    compiler_state = NFR_COMPILE_SYSERR;
	    $$ = assemble(c_filt,MAKE_OP(NFR_OP_LEVAL));
	    (void)assemble(c_filt,x);
	    PARSE("$IDENT");
	}
    }
    | '(' expr ')' '[' expr ']'
    {
	if (parseOk()) {
	    $$ = assemble(c_filt,MAKE_OP(NFR_OP_ARRAYEREF));
	    PARSE("'(' expr ')' '[' expr ']''");
	}
    }
    | '$' IDENT '[' expr ']'
    {
	if (parseOk()) {
	    int	x;
	    if((x = memlist_insert($2)) < 0)
		    compiler_state = NFR_COMPILE_SYSERR;
	    $$ = assemble(c_filt,MAKE_OP(NFR_OP_ARRAYLREF));
	    (void)assemble(c_filt,x);
	    PARSE("'$' IDENT '[' expr ']''");
	}
    }
    | IDENT
    {
	if (parseOk()) {
	    int	x;
	    if((x = memlist_insert($1)) < 0)
		    compiler_state = NFR_COMPILE_SYSERR;
	    $$ = assemble(c_filt,MAKE_OP(NFR_OP_GEVAL));
	    (void)assemble(c_filt,x);
	    PARSE("IDENT");
	    if (runInfo()) {
		printf( "Global: %s\n", $1->data.val.c );
	    }
	}
    }
    | IDENT '[' expr ']'
    {
	if (parseOk()) {
	    int	x;
	    if((x = memlist_insert($1)) < 0)
		    compiler_state = NFR_COMPILE_SYSERR;
	    $$ = assemble(c_filt,MAKE_OP(NFR_OP_ARRAYGREF));
	    (void)assemble(c_filt,x);
	    PARSE("IDENT '[' expr ']''");
	}
    }
    | IDENT beginargs '(' arglist ')'
    {
	if (parseOk()) {
	    int	x;

	    if((x = memlist_insert($1)) < 0)
		    compiler_state = NFR_COMPILE_SYSERR;
	    (void)assemble(c_filt,MAKE_OP(NFR_OP_CALL));
	    (void)assemble(c_filt,x);
	    (void)assemble(c_filt,$4);
	    $$ = $2;
	    PARSE("IDENT  '(' arglist ')'" );
	}
    }
    | IDENT beginargs '(' error 
    {
	if (parseOk()) {
	    yyerror( "function_call:function call syntax error" );
	    $$ = 0;
	}
    }
    | '(' expr ')' beginargs '(' arglist ')'
    {
	if (parseOk()) {
	    (void)assemble(c_filt,MAKE_OP(NFR_OP_ECALL));
	    (void)assemble(c_filt,$6);
	    $$ = $2;
	    PARSE("IDENT  '(' arglist ')'" );
	}
    }
    | '[' listitems ']'
    {
	if (parseOk()) {
	    int	x;

#ifdef	NFR_COMPILEDEBUG
	    compile_printlist($2);
#endif
	    if((x = memlist_insert($2)) < 0)
		    compiler_state = NFR_COMPILE_SYSERR;

	    $$ = assemble(c_filt,MAKE_OP(NFR_OP_LPUSH));
	    (void)assemble(c_filt,x);
	    PARSE("'[' listitems ']'");
	}
    }
    | '[' listitems error
    {
	    yyerror( "list:list expression syntax error" );
    }
    | '(' expr ')'
    {
	if (parseOk()) {
	    $$ = $2;
	    PARSE("'(' expr ')'");
	}
    }
    | '(' expr error
    {
	    yyerror( "parenthesis:parenthesis error" );
    }
    | SUB expr %prec UNARY
    {
	if (parseOk()) {
	    $$ = $2;
	    (void)assemble(c_filt,MAKE_OP(NFR_OP_NEG));	/* negate top of stack */
	    PARSE("SUB expr" );
	}
    }
    | expr ADD expr
    {
	if (parseOk()) {
	    (void)assemble(c_filt,$2);
	    PARSE("expr op expr" );
	}
    }
    | expr SUB expr
    {
	if (parseOk()) {
	    (void)assemble(c_filt,$2);
	    PARSE("expr op expr" );
	}
    }
    | expr MUL expr
    {
	if (parseOk()) {
	    (void)assemble(c_filt,$2);
	    PARSE("expr op expr" );
	}
    }
    | expr DIV expr
    {
	if (parseOk()) {
	    (void)assemble(c_filt,$2);
	    PARSE("expr op expr" );
	}
    }
    | expr GT expr
    {
	if (parseOk()) {
	    (void)assemble(c_filt,$2);
	    PARSE("expr op expr" );
	}
    }
    | expr GTE expr
    {
	if (parseOk()) {
	    (void)assemble(c_filt,$2);
	    PARSE("expr op expr" );
	}
    }
    | expr LT expr
    {
	if (parseOk()) {
	    (void)assemble(c_filt,$2);
	    PARSE("expr op expr" );
	}
    }
    | expr LTE expr
    {
	if (parseOk()) {
	    (void)assemble(c_filt,$2);
	    PARSE("expr op expr" );
	}
    }
    | expr EQ expr
    {
	if (parseOk()) {
	    (void)assemble(c_filt,$2);
	    PARSE("expr op expr" );
	}
    }
    | expr NE expr
    {
	if (parseOk()) {
	    (void)assemble(c_filt,$2);
	    PARSE("expr op expr" );
	}
    }
    | expr OR expr
    {
	if (parseOk()) {
	    (void)assemble(c_filt,$2);
	    PARSE("expr op expr" );
	}
    }
    | expr BOR expr
    {
	if (parseOk()) {
	    (void)assemble(c_filt,$2);
	    PARSE("expr op expr" );
	}
    }
    | expr XOR expr
    {
	if (parseOk()) {
	    (void)assemble(c_filt,$2);
	    PARSE("expr op expr" );
	}
    }
    | expr AND expr
    {
	if (parseOk()) {
	    (void)assemble(c_filt,$2);
	    PARSE("expr op expr" );
	}
    }
    | expr BAND expr
    {
	if (parseOk()) {
	    (void)assemble(c_filt,$2);
	    PARSE("expr op expr" );
	}
    }
    | expr LSHIFT expr
    {
	if (parseOk()) {
	    (void)assemble(c_filt,$2);
	    PARSE("expr op expr" );
	}
    }
    | expr RSHIFT expr
    {
	if (parseOk()) {
	    (void)assemble(c_filt,$2);
	    PARSE("expr op expr" );
	}
    }
    | expr INSIDE expr
    {
	if (parseOk()) {
	    (void)assemble(c_filt,MAKE_OP(NFR_OP_INSIDE));
	    PARSE("expr INSIDE expr");
	}
    }
    | NOT expr %prec UNARY
    {
	if (parseOk()) {
	    $$ = $2;
	    (void)assemble(c_filt,MAKE_OP(NFR_OP_NOT));	/* invert top of stack */
	    PARSE("! expr");
	}
    }
    | INV expr %prec UNARY
    {
	if (parseOk()) {
	    $$ = $2;
	    (void)assemble(c_filt,MAKE_OP(NFR_OP_INV));	/* bit invert top of stack */
	    PARSE("~ expr");
	}
    }
    | asgn
    {
	if (parseOk()) {
	    PARSE("asgn");
	}
    }
    | gasgn
    {
	if (parseOk()) {
	    PARSE("gasgn");
	}
    }
    ;

ifheader:
    IF
    {
	if (parseOk()) {
	    $$ = assemble(c_filt,MAKE_OP(NFR_OP_IF));
	    (void)assemble(c_filt,MAKE_OP(NFR_OP_STOP));
	    (void)assemble(c_filt,MAKE_OP(NFR_OP_STOP));
	    (void)assemble(c_filt,MAKE_OP(NFR_OP_STOP));
	    PARSE("ifheader");
	}
    }
    ;



funcdef:
    FUNC IDENT statement
    {
	if (parseOk()) {
	    int	x;
	    Filter *nf;

	    if((x = memlist_insert($2)) < 0)
		    compiler_state = NFR_COMPILE_SYSERR;
	    c_filt->f_name = c_filt->f_list[x]->data.val.c;
	    compiler_state = NFR_COMPILE_FUNC;
	    PARSE("FUNC IDENT statement");
	    if (runInfo()) {
		printf( "Function: %s:%s\n",
			scopeGet(), c_filt->f_list[x]->data.val.c);
	    }
	    (void)assemble(c_filt,MAKE_OP(NFR_OP_RETNV));
	    nf = get_compiled_filter( uiCurr() );
	    if (funcadd(nf)) {
		    log( FL, "parse: funcadd failed.\n" );
	    }
	}
    }
    | filtertrigger statement
    {
	if (parseOk()) {
	    Filter *nf;
	    compiler_state = NFR_COMPILE_FILT;
	    PARSE("filtertrigger statement");

	    (void)assemble(c_filt,MAKE_OP(NFR_OP_RETNV));
	    nf = get_compiled_filter( uiCurr() );
	    /*
	    ** pass in the trigger from when we built it.
	    */
	    nf->f_trigger = $1;
	    if (funcadd(nf)) {
		    log( FL, "parse: funcadd failed.\n" );
	    }
	}
    }
    ;

filtertrigger:
    FILTER IDENT trigger
    {
	if (parseOk()) {
	    /*
	    ** evaluate the trigger, and return
	    ** the fitler structure.
	    */
	    int x;
	    if ((x = memlist_insert($2)) < 0)
		    compiler_state = NFR_COMPILE_SYSERR;
	    (void)assemble(c_filt,MAKE_OP(NFR_OP_SPUSH));
	    (void)assemble(c_filt,x);
	    c_filt->f_name = c_filt->f_list[x]->data.val.c;
	    (void)assemble(c_filt, MAKE_OP(NFR_OP_FUNCON ));
	    (void)assemble(c_filt,MAKE_OP(NFR_OP_RETNV));

	    { /*  local_perform( uiCurr() ); */
		Filter *nf;
		trigger_t * t;
		int err;

		nf = get_compiled_filter( uiCurr() );
		/* don't add the assignment, just set the global variables */
		if (performMac == 0) {
		    performMac = machinenew();
		}
		machinereset(performMac);
		if (nf->f_indSymCnt) {
		    symFixIndirect(nf->f_syms);
		    nf->f_indSymCnt = 0;
		}

		err = machinerun(performMac, nf, 0 );
		/*
		** extract the trigger from that function, we'll attach it
		** to the real function in a sec...
		*/
		$$ = nf->f_trigger;
		nf->f_trigger = 0;
		free_compiled_filter(nf);
	    }
	    PARSE("FILTER IDENT trigger");
	    if (runInfo()) {
		printf( "Filter: %s:%s\n",
			scopeGet(), c_filt->f_list[x]->data.val.c);
	    }
	}
    }
    | FILTER IDENT error
    {
	    yyerror( "filter:filter function declaration syntax error" );
	    $$ = 0; 
    }
    ;


beginargs:
    {
	if (parseOk()) {
	    /*
	    this is a dirty trick - we cache the beginning
	    instruction so we can correctly position the
	    pointer if called in an if()
	    */
	    $$ = c_filt->f_memsiz;
	    PARSE("beginargs");
	}
    }



arglist:
    /* nothing */
    {
	if (parseOk()) {
	    $$ = 0;
	    PARSE("arglist<-");
	}
    }
    | arglistArgs
    {
    }
    ;

arglistArgs:
    expr
    {
	if (parseOk()) {
	    $$ = 1;
	    PARSE("expr");
	}
    }
    | arglist ',' expr
    {
	if (parseOk()) {
	    $$ = $1 + 1;
	    PARSE("arglist ',' expr");
	}
    }
    ;




listitems:
    listitem
    {
	if (parseOk()) {
	    PARSE("listitem");
	}
    }
    | listitems ',' listitem
    {
	if (parseOk()) {
	    ListMem	*p;

	    /* append a new list item to end of list */
	    for(p = $1; p->next != (ListMem *)0; p = p->next)
		    ;
	    p->next = $3;
	    $3->next = (ListMem *)0;
	    PARSE("listitems ',' listitem");
	}
    }
    ;



listitem:
    STRING
    {
	if (parseOk()) {
	    PARSE("STRING");
	}
    }
    | NUM
    {
	if (parseOk()) {
	    ListMem	*ret;

	    if((ret = (ListMem *)malloc(sizeof(ListMem))) == (ListMem *)0) {
		$$ = (ListMem *)0; 
	    } else {
		ret->data.val.i = $1;
		ret->data.typ = NFR_TYPE_INT;
		ret->next = 0;
		$$ = ret;
	    }
	    PARSE("NUM");
	}
    }
    | SUB NUM
    {
	if (parseOk()) {
	    ListMem	*ret;

	    if((ret = (ListMem *)malloc(sizeof(ListMem))) == (ListMem *)0) {
		$$ = (ListMem *)0; 
	    } else {
		ret->data.val.i = -$2;
		ret->data.typ = NFR_TYPE_INT;
		ret->next = 0;
		$$ = ret;
	    }
	    PARSE("SUB NUM");
	}
    }
    | NUM '.' NUM '.' NUM '.' NUM
    {
	if (parseOk()) {

	    ListMem *ret;

	    if((ret = (ListMem *)malloc(sizeof(ListMem))) == (ListMem *)0) {
		    $$ = (ListMem *)0;
	    } else {
		if (($1 > 256) || ($3 > 256) || ($5 > 256) || ($7 > 256)) {
		    uiCurrPrintf( "Too large of a number in dotted host notation\n" );
		}
		{
		int n = ntohl( ($1<<24)|($3<<16)|($5<<8)|($7) );
		ret->data.val.h.s_addr = n;
		ret->data.typ = NFR_TYPE_IPV4HOST;
		ret->next = 0;
		$$ = ret;
	    }
	    }
	    PARSE("NUM '.' NUM '.' NUM '.' NUM");
	}
    }
    | NUM '.' error
    {
	    yyerror( "host:Host address syntax error" );
	    $$ = 0;
    }
    | NUM '.' NUM '.' NUM '.' NUM ':'  NUM '.' NUM '.' NUM '.' NUM
    {
	if (parseOk()) {
	    ListMem *ret;
	    ipv4net_t *net;

	    ret = (ListMem *)tmpcalloc(sizeof(ListMem));
	    net = (ipv4net_t *)tmpalloc(sizeof(ipv4net_t));

	    if((ret == (ListMem *)0) || (net == 0)) {
		    $$ = (ListMem *)0;
	    } else {
		if (($1 > 256) || ($3 > 256) || ($5 > 256) || ($7 > 256)) {
		    uiCurrPrintf( "Too large of a number in dotted net notation\n" );
		}
		if (($9 > 256) || ($11 > 256) || ($13 > 256) || ($15 > 256)) {
		    uiCurrPrintf( "Too large of a number in dotted mask notation\n" );
		}
		{
		net->ip_addr.s_addr = ntohl( ($1<<24)|($3<<16)|($5<<8)|($7) );
		net->ip_mask = ntohl( ($9<<24)|($11<<16)|($13<<8)|($15) );

		if ((net->ip_addr.s_addr & (~net->ip_mask))) {
		    uiCurrPrintf( "Warning: network %s:%8.8x, bits in mask\n",
			util_ntoa(net->ip_addr.s_addr), ntohl(net->ip_mask) );
		    net->ip_addr.s_addr &= net->ip_mask;
		}

		ret->data.typ = NFR_TYPE_IPV4NET;
		ret->data.val.n = net;

		$$ = ret;
		}
	    }
	    PARSE("NUM '.' NUM '.' NUM '.' NUM ':'  NUM '.' NUM '.' NUM '.' NUM");
	}
    }
    | NUM '.' NUM '.' NUM '.' NUM DIV NUM
    {
	if (parseOk()) {
	    ListMem *ret;
	    ipv4net_t *net;

	    ret = (ListMem *)tmpcalloc(sizeof(ListMem));
	    net = (ipv4net_t *)tmpalloc(sizeof(ipv4net_t));

	    if (($1 > 256) || ($3 > 256) || ($5 > 256) || ($7 > 256)) {
		uiCurrPrintf( "Too large of a number in cidr net notation\n" );
	    }
	    if ($9 > 32) {
		uiCurrPrintf( "Too large of a number in cidr net notation\n" );
	    }
	    {
		net->ip_addr.s_addr = htonl( ($1<<24)|($3<<16)|($5<<8)|($7) );
		net->ip_mask = htonl( 0xffffffff << (32 - $9) );

		if ((net->ip_addr.s_addr & (~net->ip_mask))) {
		    uiCurrPrintf( "Warning: network %s:%8.8x, bits in mask\n",
			util_ntoa(net->ip_addr.s_addr), ntohl(net->ip_mask) );
		    net->ip_addr.s_addr &= net->ip_mask;
		}

		ret->data.typ = NFR_TYPE_IPV4NET;
		ret->data.val.n = net;

		$$ = ret;
	    }
	    PARSE("NUM  '.' NUM '.' NUM '.' NUM '/'  NUM " );
	}
    }
    | NUM '.' NUM '.' NUM '.' NUM ':'  error
    {
	    yyerror( "net:netword address syntax error" );
	    $$ = 0;
    }
    | ETHMAC
    {
	if (parseOk()) {
	}
    }
    ;
%%

/*
** File input control.
**
** setyyFile is like setyyinput, but takes a 'char *'
**
** Its used by test-nfrd to read the ENTIRE file contents into
** memory, then if an error occurs, it spits out html reporting
** the location of the error...
*/

/* these are for conventional yacc usage */
FILE	*yyin;


void
setyyinput(FILE *f)
{
    yyin = f;
}


/*
** the compile filter buffer is "magic" -- we make it stretch. all other
** filters, as they are compiled, are cloned with precise size from this
** compile filter buffer. that way we don't waste memory.
*/
static	Filter	*
cfilt_init()
{
    Filter	*r;

    /* these values will grow if necessary so keep them reasonable */
    if((r = (Filter *)malloc(sizeof(Filter))) == (Filter *)0)
	    return((Filter *)0);

    /* set up memory space */
    if((r->f_mem = (int *)malloc(sizeof(int) * 2048)) == (int *)0) {
	    free(r);
	    return((Filter *)0);
    }
    c_filt_memmax = 2048;

    /* set up list space */
    if((r->f_list = (ListMem **)malloc(sizeof(ListMem *) * 512)) == (ListMem **)0) {
	    free(r->f_mem);
	    free(r);
	    return((Filter *)0);
    }
    c_filt_listmax = 512;

    /* zap symbol table */
    r->f_syms = (Sym *)0;
    return(r);
}




/* called after compiler error to zeroize various junk */
void
clear_compiler_error()
{
    int	x;

    for(x = 0; x < c_filt->f_listsiz; x++) {
	    listfree(c_filt->f_list[x]);
	    c_filt->f_list[x] = (ListMem *)0;
    }
    c_filt->f_memsiz = 0;
    c_filt->f_listsiz = 0;
}



/* retrieve a copy of the compiled filter */
Filter	*
get_compiled_filter( Userintf_t * ui )
{
    Filter	*ret;

    if((ret = (Filter *)malloc(sizeof(Filter))) == (Filter *)0)
	    return((Filter *)0);
    ret->f_name = c_filt->f_name;

    /* copy program memory */
    if((ret->f_mem = (int *)malloc(sizeof(int) * c_filt->f_memsiz)) == (int *)0) {
	    free(ret);
	    return((Filter *)0);
    }
    memcpy(ret->f_mem,c_filt->f_mem,sizeof(int) * c_filt->f_memsiz);
    ret->f_memsiz = c_filt->f_memsiz;

    /* copy external memory/list space */
    if(c_filt->f_listsiz == 0)
	    ret->f_list = (ListMem **)0;
    else {
	    ret->f_list = (ListMem **)malloc(sizeof(ListMem *) * c_filt->f_listsiz);
	    if(ret->f_list == (ListMem **)0) {
		    free(ret->f_mem);
		    free(ret);
		    return((Filter *)0);
	    }
	    /*
	    ** we do NOT need to copy lists because they are already allocated.
	    */
	    memcpy(ret->f_list,c_filt->f_list,sizeof(ListMem *) * c_filt->f_listsiz);
	    /*
	    ** we DO need to zero any of the list elements in cfilt.
	    ** just for housekeeping.
	    */
	    memset( c_filt->f_list, 0, sizeof(ListMem *) * c_filt->f_listsiz);
    }
    ret->f_listsiz = c_filt->f_listsiz;


    /* zap symbol table */
    ret->f_syms = (Sym *)0;
    ret->f_indSymCnt = 0;

    /* now clear out the current c_filt */
    c_filt->f_memsiz = 0;
    c_filt->f_listsiz = 0;

    /* associate output stream */
    ret->f_ui = uiIncr(ui);

    /* associate filter with a scope */
    if (scope) {
	ret->f_scope = strdup(scope);
	ret->f_scopeNdx = strHash(scope);
    } else {
	ret->f_scope = 0;
    }
    ret->f_fromFile = 0;
    if (yyFileName) {
	ret->f_fromFile = strdup(yyFileName);
    }

    /*
    ** zero any additional fields.
    */
    ret->f_useCount = 0;
    memset( &ret->f_runTime, 0, sizeof(ret->f_runTime) );
    ret->f_sumSq = 0;
    ret->f_stdDev = 0;
    ret->f_trigger = 0;

    return(ret);
}


/* XXX  what an unsafe routine :^( */

free_compiled_filter(f)
Filter * f;
{
    if (f == 0) {
	    return(0);
    }

    if (f->f_mem) {
	free(f->f_mem);
	f->f_mem = 0;
    }

    /* free all memory lists. */
    if (f->f_listsiz) {
	    free(f->f_list);
    }

    if (f->f_scope) {
	free(f->f_scope);
	f->f_scope = 0;
    }
    if (f->f_fromFile) {
	free(f->f_fromFile);
	f->f_fromFile = 0;
    }

    free(f);
}


static int
assemble_pc()
{
return(c_filt->f_memsiz);
}




static	int
assemble(Filter *p,int op)
{
    /* if we're out of space, try to grow */
    if(c_filt->f_memsiz >= c_filt_memmax) {
	    c_filt_memmax = c_filt_memmax * 2;
	    p->f_mem = (int *)realloc(p->f_mem,sizeof(int) * c_filt_memmax);
	    if(p->f_mem == (int *)0) {
		    yyerror("program_size:program too big\n");
		    compiler_state = NFR_COMPILE_SYSERR;
		    return(0);
	    }
    }
#ifdef	NFR_COMPILEDEBUG
    printf("[%d]",c_filt->f_memsiz);
    assemble_printop(op);
    printf("\n");
#endif
    p->f_mem[c_filt->f_memsiz] = op;
    c_filt->f_memsiz += 1;
    return(c_filt->f_memsiz - 1);
}



/* store data in the external string/memory space */
static	int
memlist_insert(ListMem *n)
{

    if(n == (ListMem *)0)
	    return(-1);
    /* need to grow the external store table */
    if(c_filt->f_listsiz + 1 >= c_filt_listmax) {
	    c_filt_listmax = c_filt_listmax * 2;
	    c_filt->f_list = (ListMem **)realloc(c_filt->f_list,sizeof(ListMem *) * c_filt_listmax);
	    if(c_filt->f_list == (ListMem **)0) {
		    yyerror("store_table:external store table too big\n");
		    compiler_state = NFR_COMPILE_SYSERR;
		    return(-1);
	    }
    }
    c_filt->f_list[c_filt->f_listsiz] = n;
    c_filt->f_listsiz++;
    return(c_filt->f_listsiz - 1);
}



static	ListMem	*
memlist_fromstring(char *s)
{
    ListMem	*ret;

    if((ret = (ListMem *)malloc(sizeof(ListMem))) == (ListMem *)0)
	    return((ListMem *)0);
    ret->data.val.c = (char *)malloc(strlen(s) + 1);
    ret->next = 0;
    if(ret->data.val.c == (char *)0) {
	    free(ret);
	    return((ListMem *)0);
    }
    strcpy(ret->data.val.c,s);
    ret->data.typ = NFR_TYPE_STR;
    return(ret);
}



/* locate keywords */
static	int
lookup(char *s)
{
    int	start	= 0;
    int	ret;

    static	struct	kwordz{
	    char	*kw;
	    int	rval;
    } keyz[] = {
    /* MUST BE IN LEXICAL SORT ORDER !!!!!! */
    { "NULL",			OPNUL },
    { "break",			BREAK },
    { "call",			CALL },
    { "continue",		CONTINUE },
    { "declare",			DECLARE },
    { "else",			ELSE },
    { "filter",			FILTER },
    { "foreach",			FOREACH },
    { "func",			FUNC },
    { "global",			GLOBAL },
    { "if",				IF },
    { "inside",			INSIDE },
    { "off",			OFF },
    { "on",				ON },
    { "record",			RECORD },
    { "return",			RETURN },
    { "to",				TO },
    { "while",			WHILE },
    { 0,0 }
    };

    int	end	= (sizeof(keyz)/sizeof(struct kwordz)) - 2;
    int	p	= end/2;

    while(start <= end) {
	    ret = strcmp(s,keyz[p].kw);
	    if(ret == 0)
		    return(keyz[p].rval);
	    if(ret > 0)
		    start = p + 1;
	    else
		    end = p - 1;

	    p = start + ((end - start)/2);
    }
    return(-1);
}


/*
** ------------------------------
** lex routines start here...
*/

ishex(c) {
    return( isdigit(c) ||
	((c >= 'a') && (c <= 'f')) ||
	((c >= 'A') && (c <= 'F')) );
}


int
makehex(c) {
    if (isdigit(c)) {
	return(c-'0');
    }
    if ((c >= 'a') && (c <= 'f')) {
	return(c-'a'+10);
    }
    if ((c >= 'A') && (c <= 'F')) {
	return(c-'A'+10);
    }
}

/*
** return a reference to a single ethmac address.
*/

struct ethMac *
makeEthMac( a, b, d, e, f, g, h, i, j, k, l, m )
char a, b, d, e, f, g, h, i, j, k, l, m;
{
    static struct ethMac lclEthMac[1];

    memset( lclEthMac, 0, sizeof(lclEthMac) );
    if ((ishex(a) == 0) || (ishex(b) == 0) 
    ||  (ishex(d) == 0) || (ishex(e) == 0)
    ||  (ishex(f) == 0) || (ishex(g) == 0)
    ||  (ishex(h) == 0) || (ishex(i) == 0)
    ||  (ishex(j) == 0) || (ishex(k) == 0)
    ||  (ishex(l) == 0) || (ishex(m) == 0) ) {
	return(0);
    }

    lclEthMac->e_addr[0] = (makehex(a) << 4) | (makehex(b));
    lclEthMac->e_addr[1] = (makehex(d) << 4) | (makehex(e));

    lclEthMac->e_addr[2] = (makehex(f) << 4) | (makehex(g));
    lclEthMac->e_addr[3] = (makehex(h) << 4) | (makehex(i));

    lclEthMac->e_addr[4] = (makehex(j) << 4) | (makehex(k));
    lclEthMac->e_addr[5] = (makehex(l) << 4) | (makehex(m));
    return(lclEthMac);
}


/*
** Used by the lexer to set the current token to an ethmac.
** but check everything...
*/

ListMem * 
ethmac( a, b, d, e, f, g, h, i, j, k, l, m )
char a, b, d, e, f, g, h, i, j, k, l, m;
{
    ListMem * ret;

    if ((ret = (ListMem *)malloc(sizeof(ListMem))) == (ListMem *)0) {
	return 0;
    }

    ret->data.typ = NFR_TYPE_ETHMAC;
    ret->next = 0;

    ret->data.val.e.e_addr[0] = (makehex(a) << 4) | (makehex(b));
    ret->data.val.e.e_addr[1] = (makehex(d) << 4) | (makehex(e));

    ret->data.val.e.e_addr[2] = (makehex(f) << 4) | (makehex(g));
    ret->data.val.e.e_addr[3] = (makehex(h) << 4) | (makehex(i));

    ret->data.val.e.e_addr[4] = (makehex(j) << 4) | (makehex(k));
    ret->data.val.e.e_addr[5] = (makehex(l) << 4) | (makehex(m));

    return(ret);
}

/*
** input character routines.
**
** Provide an easy way to direct the input either from
** file's or from other things.  test-main (the syntax
** checker of the engine), will redirect input from memory,
** after reading all the input from the files...
**
** Note that since yysetinput is called to set the FILE*,
** and that this is just a cookie used by yygetc/yyungetc,
** we can use that value as a rock, property of the
** implementation ** of the target getc_t/ungetc_t.
*/

typedef int (*getc_t)( void * );
getc_t yyGetcFtn = 0;

typedef int (*ungetc_t)( int c, void * );
ungetc_t yyUngetcFtn = 0;

/* should returned values of ftell_t be off_t's? */
typedef long (*ftell_t)( void * );
ftell_t yyFtellFtn = 0;

typedef int (*feof_t)( void * );
feof_t yyFeofFtn;


getc_t
yyGetcSet( getc_t ftn )
{
getc_t oFtn = yyGetcFtn;
yyGetcFtn = ftn;
return(oFtn);
}

ungetc_t
yyUngetcSet( ungetc_t ftn )
{
ungetc_t oFtn = yyUngetcFtn;
yyUngetcFtn = ftn;
return(oFtn);
}

ftell_t
yyFtellSet( ftell_t ftn )
{
ftell_t oFtn = yyFtellFtn;
yyFtellFtn = ftn;
return(oFtn);
}

feof_t
yyFeofSet( feof_t ftn )
{
feof_t oFtn = yyFeofFtn;
yyFeofFtn = ftn;
return(oFtn);
}

char *
yyFileNameSet( char * s )
{
char * os = yyFileName;
yyFileName = s;
return(os);
}

/*
** ------------------------------
*/

int
yygetc( FILE * f )
{
if (yyGetcFtn) {
    return(yyGetcFtn( (void *)f ));
}
return(fgetc(f));
}

void
yyungetc( int c, FILE * f)
{
if (yyUngetcFtn) {
    (void)yyUngetcFtn( c, (void *)f );
} else {
    (void)ungetc(c,f);
}
}

long
yyftell( FILE * f)
{
if (yyFtellFtn) {
    return( yyFtellFtn( (void *)f ) );
}
return(ftell(f));
}

int
yyfeof( FILE * f )
{
if (yyFeofFtn) {
    return( yyFeofFtn( (void *)f ) );
}
return(feof(f));
}

/*
** Cover for the lexer.
** ------------------------------
**
** Here we keep a short stack of the last token... we
** just keep the last token processed beginnning address..
** This provides us with a way of finding the start of the
** last token processed.  In the even of a syntax error,
** we can tell where the last token processed was...
**
*/

long yyLastTokenCp = 0;			/* typeof returned value of ftell_t */

int
yylex() {
int token;
yyLastTokenCp = yyftell(yyin);
token = lex();
return(token);
}

long
yyLastToken() {
return(yyLastTokenCp);
}

/*
**
** The lexer...
** --------------------------------------------------
**
*/

int
lex()
{
    char	in[BUFSIZ];
    char	*p = in;
    int	c;
    static	int lastToken;		/* sigh. */

    again:
    /* handle whitespace */
    while(isspace(c = yygetc(yyin))) {
	if(c == '\n') {
	    yylineno++;
	}
    }
    
    if (c  == '#' && (lastToken != '$')) {
	/* comment.  skip all characters till end of line, then try again.  */
	while( (c = yygetc(yyin)) != '\n')
	    /* nutin */;
	yylineno++;
	goto again;
    }
    lastToken = 0;
    
    /* handle EOF */
    if(c == EOF || yyfeof(yyin)) {
	return(0);
    }

    /* save current char - it is valuable */
    *p++ = c;

    /*
    ** try to get ethernet mac addresses... look for two hex digits
    ** followed by a ':'.  We may hafta get alot of yyungetc's if we miss...
    ** cause of all the single chars, we'll name them:
    ** ab:de:fg:hi:jk:lm (but we don't use 'm', cause it'l be in the variable c).
    */
    {
	int a,b,d,e,f,g,h,i,j,k,l,m, x;

	if (ishex(c)) {
	    a = c;
	    b = yygetc(yyin);
	    if (ishex(b)) {
		c = yygetc(yyin);
		if (c == ':') {

		    /* first pair */
		    d = yygetc(yyin);
		    if (ishex(d)) {
			e = yygetc(yyin);
			if (ishex(e)) {
			    c = yygetc(yyin);
			    if (c == ':') {
							    
				/* second pair */
				f = yygetc(yyin);
				if (ishex(f)) {
				    g = yygetc(yyin);
				    if (ishex(g)) {
					c = yygetc(yyin);
					if (c == ':') {
							    
					    /* third pair */
					    h = yygetc(yyin);
					    if (ishex(h)) {
						i = yygetc(yyin);
						if (ishex(i)) {
						    c = yygetc(yyin);
						    if (c == ':') {

							/* fourth pair */
							j = yygetc(yyin);
							if (ishex(j)) {
							    k = yygetc(yyin);
							    if (ishex(k)) {
								c = yygetc(yyin);
								if (c == ':') {

								    /* fifth pair */
								    l = yygetc(yyin);
								    if (ishex(l)) {
									m = yygetc(yyin);
									if (ishex(m)) {
									    /* THAT IT! */
									    if((yylval.mval = 
										ethmac(
										    a,b,
										    d,e,
										    f,g,
										    h,i,
										    j,k,
										    l,m ))==0) {
										return(-1);
									    }
									    return(ETHMAC);
									}
									yyungetc(m,yyin);
								    }
								    yyungetc(l,yyin);
								    yyungetc(':',yyin);
								} else {
								    yyungetc(c,yyin);
								}
							    }
							    yyungetc(k,yyin);
							}
							yyungetc(j,yyin);
							yyungetc(':',yyin);
						    } else {
							yyungetc(c,yyin);
						    }
						}
						yyungetc(i,yyin);
					    }
					    yyungetc(h,yyin);
					    yyungetc(':',yyin);
					} else {
					    yyungetc(c,yyin);
					}
				    }
				    yyungetc(g,yyin);
				}
				yyungetc(f,yyin);
				yyungetc(':',yyin);
			    } else {
				yyungetc(c,yyin);
			    }
			}
			yyungetc(e,yyin);
		    }
		    yyungetc(d,yyin);
		    yyungetc(':',yyin);
		} else {
		    yyungetc( c, yyin );
		}
	    }
	    yyungetc(b,yyin);

	c = a;
	}
    }

    /* handle NUM */
    if(isdigit(c)) {
	    int	num;

	    Number:
	    num = c - '0';

	    if (num == 0) {
		    c = yygetc(yyin);
		    if (c == 'x') {
			/* hex */
			while((c = yygetc(yyin)) > 0) {
			    if (isdigit(c)) {
				num *= 16;
				num += c - '0';
			    } else if ((c >= 'a') && (c <= 'f')) {
				num *= 16;
				num += (c - 'a') + 10;
			    } else if ((c >= 'A') && (c <= 'F')) {
				num *= 16;
				num += (c - 'A') + 10;
			    } else {
				break;
			    }
			}
		    }
		    (void)yyungetc(c,yyin);
		    yylval.ival = num;
		    return(NUM);
	    }
	    while(isdigit(c = yygetc(yyin))) {
		    num = (num * 10) + (c - '0');
	    }

	    (void)yyungetc(c,yyin);
	    if(c == '\n')
		    yylineno--;

	    yylval.ival = num;
	    return(NUM);
    }

    /* handle keywords or idents/builtins */
    if(isalnum(c) || c == '_') {
	    int	cnt = 0;
	    int	rv;

	    while((c = yygetc(yyin)) != EOF && (isalnum(c) || c == '_' || c == ':')) {
		    if (c == ':') {
			c = yygetc(yyin);
			if ((c != EOF) && (!isalnum(c) && (c != '_'))) {
			    yyungetc( c, yyin );
			    c = ':'; /* yyungetc later. */
			    break;
			} else {
			    *p++ = ':';
			    *p++ = c;
			}
		    } else {
			if(++cnt + 1 >= 128)
				yyerror("ident:identifier too long");
			*p++ = c;
		    }
	    }

	    (void)yyungetc(c,yyin);
	    *p = '\0';
	    if((rv = lookup(in)) != -1)
		    return(rv);
    
	    if((yylval.mval = memlist_fromstring(in)) == (ListMem *)0)
		    return(-1);
	    return(IDENT);
    }

    /* handle quoted strings */
    if(c == '"' || c == '\'') {
	    int	cnt = 0;
	    int	quot = c;

	    /* strip start quote by resetting ptr */
	    p = in;

	    /* match quoted strings */
	    while((c = yygetc(yyin)) != EOF && c != quot) {
		    /*
		    ** We used to only allow ascii chars here...
		    */
		    /* if(!isascii(c))
			    continue; */

		    if(++cnt + 1 >= sizeof(in))
			    yyerror("string:string too long");

		    /* we have to guard the line count */
		    if(c == '\n')
			    yylineno++;

		    if(c == '\\') {
			    int	b;
			    switch(b = yygetc(yyin)) {
				    case	EOF:
					    yyerror("string:EOF in quoted string");
					    break;

				    case	'r':
					    *p++ = '\r';
					    break;

				    case	't':
					    *p++ = '\t';
					    break;

				    case	'n':
					    *p++ = '\n';
					    break;

				    default:
					    *p++ = b;
			    }
		    } else {
			    *p++ = c;
		    }
	    }

	    if(c == EOF)
		    yyerror("string:EOF in quoted string");

	    *p = '\0';
    
	    if((yylval.mval = memlist_fromstring(in)) == (ListMem *)0)
		    return(-1);
	    return(STRING);
    }

    switch(c) {
	    case	'=':
		    {
			    int	t;
			    t = yygetc(yyin);
			    if(t != '=') {
				    (void)yyungetc(t,yyin);
				    yylval.ival = 0;
				    return(ASGN);
			    }
		    }
		    yylval.ival = MAKE_OP(NFR_OP_EQ);
		    return(EQ);

	    case	'>':
		    {
			    int	t;
			    t = yygetc(yyin);
			    if(t == '=') {
				yylval.ival = MAKE_OP(NFR_OP_GTE);
				return(GTE);
			    } else
			    if(t == '>') {
				yylval.ival = MAKE_OP(NFR_OP_RSHIFT);
				return(RSHIFT);
			    } else
			    if(t != '=') {
				    (void)yyungetc(t,yyin);
				    yylval.ival = MAKE_OP(NFR_OP_GT);
				    return(GT);
			    }
		    }

	    case	'<':
		    {
			    int	t;
			    t = yygetc(yyin);
			    if(t == '=') {
				yylval.ival = MAKE_OP(NFR_OP_LTE);
				return(LTE);
			    } else
			    if(t == '<') {
				yylval.ival = MAKE_OP(NFR_OP_LSHIFT);
				return(LSHIFT);
			    } else
			    {
				    (void)yyungetc(t,yyin);
				    yylval.ival = MAKE_OP(NFR_OP_LT);
				    return(LT);
			    }
		    }

	    case	'!':
		    {
			    int	t;
			    t = yygetc(yyin);
			    if(t != '=') {
				    (void)yyungetc(t,yyin);
				    yylval.ival = MAKE_OP(NFR_OP_NOT);
				    return(NOT);
			    }
		    }
		    yylval.ival = MAKE_OP(NFR_OP_NE);
		    return(NE);

	    case	'&':
		    {
			    int	t;
			    t = yygetc(yyin);
			    if(t != '&') {
				    (void)yyungetc(t,yyin);
				    yylval.ival = MAKE_OP(NFR_OP_BAND);
				    return(BAND);
			    }
		    }
		    yylval.ival = MAKE_OP(NFR_OP_AND);
		    return(AND);

	    case	'|':
		    {
			    int	t;
			    t = yygetc(yyin);
			    if(t != '|') {
				    (void)yyungetc(t,yyin);
				    yylval.ival = MAKE_OP(NFR_OP_BOR);
				    return(BOR);
			    }
		    }
		    yylval.ival = MAKE_OP(NFR_OP_OR);
		    return(OR);

	    case	'/':
		    yylval.ival = MAKE_OP(NFR_OP_DIV);
		    return(DIV);

	    case	'*':
		    yylval.ival = MAKE_OP(NFR_OP_MUL);
		    return(MUL);

	    case	'-':
		    yylval.ival = MAKE_OP(NFR_OP_SUB);
		    return(SUB);

	    case	'+':
		    yylval.ival = MAKE_OP(NFR_OP_ADD);
		    return(ADD);

	    case	'^':
		    yylval.ival = MAKE_OP(NFR_OP_XOR);
		    return(XOR);

	    case	'~':
		    yylval.ival = MAKE_OP(NFR_OP_INV);
		    return(INV);

	    case	'$':
		    lastToken = '$';
		    return('$');
    }

    /* punt */
    if(c == '\n')
	    yylineno++;

    return(c);
}



void
parseError(char *s)
{
    parseFailed();
    uiCurrPrintf("line: %d, %s\n", yylineno, s);
#ifdef	YYYDEBUG
    /*
    ** attempt a stack backtrace.
    */
    { /* hey, howcome andrew is compiling this part? */
	short * base = yyss;
	short * curr = yyssp;
	int i;

	printf( "line: %d Partial parse: ", yylineno );
	for ( i=0; base <= curr; ++base, ++i ) {
	    if (yyvs[i].part) {
		/* printf( "Str: %s\n", yyvs[i].part ); */
	    }
	    if (base > yyss) {
		parse_info( *base, base[-1] );
	    }
	}

	printf( "| " );
	if (yychar < 256) {
	    if (isprint(yychar)) {
		printf( "`%c' |\n", yychar );
	    } else {
		printf( "`0x%x' |\n", yychar );
	    }
	} else {
	    if (yychar < YYMAXTOKEN) {
		printf( "`%s' |\n", yyname[yychar] );
	    } else {
		printf( " ?? %d |\n", yychar );
	    }
	}
	parse_want( yylineno, *yyssp);
    }
#endif
}



#ifdef	NFR_COMPILEDEBUG
void
compile_printmem(ListMem *lp)
{
    switch(lp->data.typ) {
    case NFR_TYPE_ERR:	uiCurrPrintf(" ERR="); blt_echo(lp); return;
    case NFR_TYPE_INT:	uiCurrPrintf(" INT="); blt_echo(lp); return;
    case NFR_TYPE_STR:	uiCurrPrintf(" STR=\""; blt_echo(lp); uiCurrPrintf("\""); return;
    case NFR_TYPE_LIST:	uiCurrPrintf(" LIST="); blt_echo(lp); return;
    case NFR_TYPE_RECORDER: uiCurrPrintf(" RECORDER="); blt_echo(lp); return;
    case NFR_TYPE_SCHEMADATA:uiCurrPrintf(" SCHEMA="); blt_echo(lp); return;
    case NFR_TYPE_IPV4HOST: uiCurrPrintf(" HOST="); blt_echo(lp); return;
    case NFR_TYPE_IPV4NET:  uiCurrPrintf(" NET=" ); blt_echo(lp); return;
    case NFR_TYPE_BLOB:	uiCurrPrintf(" BLOB=" ); blt_echo(lp); return;
    case NFR_TYPE_ETHMAC:	uiCurrPrintf(" ENET=" ); blt_echo(lp); return;
    case NFR_TYPE_PATTERN:	uiCurrPrintf(" PATTERN=" ); blt_echo(lp); return;
    case NFR_TYPE_TRIGGER:	uiCurrPrintf(" TRIGGER=" ); blt_echo(lp); return;
    default:
			    uiCurrPrintf(" Unknown!(%d)",lp->data.typ);
    }
}



void
compile_printlist(ListMem *lp)
{
    ListMem	*p;

    uiCurrPrintf("list:");
    for(p = lp; p != (ListMem *)0; p = p->next) {
	    compile_printmem(p);
	    if(p->next != (ListMem *)0)
		    uiCurrPrintf(", ");
    }
    uiCurrPrintf("\n");
}



static	void
assemble_printop(int op)
{
    switch(NFR_OP(op)) {
    case NFR_OP_STOP:	uiCurrPrintf("STOP"); break;
    case NFR_OP_POP:	uiCurrPrintf("POP"); break;
    case NFR_OP_NPUSH:	uiCurrPrintf("NPUSH"); break;
    case NFR_OP_SPUSH:	uiCurrPrintf("SPUSH"); break;
    case NFR_OP_CPUSH:	uiCurrPrintf("CPUSH"); break;
    case NFR_OP_PPUSH:	uiCurrPrintf("PPUSH"); break;
    case NFR_OP_LPUSH:	uiCurrPrintf("LPUSH"); break;
    case NFR_OP_NULPUSH:	uiCurrPrintf("NULPUSH"); break;
    case NFR_OP_LASGN:	uiCurrPrintf("LASGN"); break;
    case NFR_OP_CALL:	uiCurrPrintf("CALL"); break;
    case NFR_OP_RETNV:	uiCurrPrintf("RETNV"); break;
    case NFR_OP_RETV:	uiCurrPrintf("RETV"); break;
    case NFR_OP_ADD:	uiCurrPrintf("ADD"); break;
    case NFR_OP_SUB:	uiCurrPrintf("SUB"); break;
    case NFR_OP_MUL:	uiCurrPrintf("MUL"); break;
    case NFR_OP_DIV:	uiCurrPrintf("DIV"); break;
    case NFR_OP_NEG:	uiCurrPrintf("NEG"); break;
    case NFR_OP_EQ:		uiCurrPrintf("EQ"); break;
    case NFR_OP_NE:		uiCurrPrintf("NE"); break;
    case NFR_OP_LT:		uiCurrPrintf("LT"); break;
    case NFR_OP_LTE:	uiCurrPrintf("LTE"); break;
    case NFR_OP_GT:		uiCurrPrintf("GT"); break;
    case NFR_OP_GTE:	uiCurrPrintf("GTE"); break;
    case NFR_OP_NOT:	uiCurrPrintf("NOT"); break;
    case NFR_OP_AND:	uiCurrPrintf("NOT"); break;
    case NFR_OP_OR:		uiCurrPrintf("NOT"); break;
    case NFR_OP_IF:		uiCurrPrintf("IF"); break;
    case NFR_OP_IFCOND:	uiCurrPrintf("IFCOND"); break;
    case NFR_OP_LEVAL:	uiCurrPrintf("LEVAL"); break;
    case NFR_OP_GASGN:	uiCurrPrintf("GASGN"); break;
    case NFR_OP_GEVAL:	uiCurrPrintf("GEVAL"); break;
    case NFR_OP_IPV4HOSTPUSH: uiCurrPrintf("IPV4HOSTPUSH"); break;
    case NFR_OP_IPV4NETPUSH: uiCurrPrintf("IPV4NETPUSH"); break;
    case NFR_OP_ON:		uiCurrPrintf("ON"); break;
    case NFR_OP_OFF:	uiCurrPrintf("OFF"); break;
    case NFR_OP_DECL_INSIDE:uiCurrPrintf("DECL/INSIDE"); break;
    case NFR_OP_DECL_GLOBAL:uiCurrPrintf("DECL/GLOBAL"); break;
    case NFR_OP_FOREACH:	uiCurrPrintf("FOREACH"); break;
    case NFR_OP_WHILE:	uiCurrPrintf("WHILE"); break;
    case NFR_OP_BREAK:	uiCurrPrintf("BREAK"); break;

    default:
	    uiCurrPrintf("?"); break;
    }
    uiCurrPrintf(":%d",op);
}
#endif
