/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/



/*
** pkgLoad.
** ------------------------------
**
** This file contains some routines which
** are used by both the engine and ctlnfrd.
**
** These routines have code to walk directories,
** which are intended to have packages in them,
** find, and automatically load them.  
**
** The policy is as follows:
**
** review the directory passed in, and look for .cfg
** files.   If one exists, review that configuration file
** for an 'enable' variable.  If its set, then check
** for a cooresponding .nfr file.  If its there, load
** it.  If there's a directory with the same basename
** then reviewits contents, and apply the same rule.
** as we recurse, append scope names by using the basenames
** concatenated with "_"s between them.
**
*/


#include "nfrd.h"

/*
** cfgSelect.
** used by scandir to exclude dotFiles,
** and to only notice .cfg files.
*/

cfgSelect( dir )
struct dirent * dir;
{
    if (dir->d_name[0] == '.') {
	return 0;
    }
    {
	int len = strlen(dir->d_name);
	if (strcmp( &dir->d_name[len-4], ".cfg" ) == 0) {
		return 1;
	}
    }
    return 0;
}

/*
** pkgLoad.
**
** walk through a directory.  If it is a directory,
** call recursivly.  If its a .nfr, then find the associated
** .cfg.  If there is a .cfg, then Cf_Read it and look
** for the 'enabled' variable.  If the value of 'enabled'
** is 'true' then try issuing the load command for this
** component.
** 
*/

pkgLoad( scope, path, perform )
char * scope;
char * path;
int (*perform)( char *, char * );
{
    struct dirent ** names;
    int cnt = nfrScandir( path, &names, cfgSelect, nfrAlphasort );
    int len = strlen(path);
    if (cnt > 0) {
	struct dirent **namei = names;
	while(cnt--) {
	    int dLen = strlen(namei[0]->d_name);
	    int newLen = len + dLen + 2;
	    char * cfgPath = (char *)malloc( newLen );
	    char * newPath, * nfrPath, * base;
	    sprintf( cfgPath, "%s/%s", path, namei[0]->d_name );

	    /* create the scope component to be appended */
	    base = strdup(namei[0]->d_name);
	    base[strlen(base)-4] = 0;

	    /* create the path of the .nfr file */
	    nfrPath = strdup(cfgPath);
	    strcpy( &nfrPath[strlen(nfrPath)-4], ".nfr" );

	    /* create the path of any directory to descend */
	    newPath = strdup(cfgPath);
	    newPath[strlen(newPath)-4] = 0;

	    /*
	    ** see if the .cfg file has been enabled.
	    */
	    {
		/* found config file? */
		TCf * cf = Cf_New(32);
		if ((Cf_Read( cfgPath, cf )) == LIBNFR_ERR_UNDEFINED) {
		} else {
		    /* see if we can find the enabled variable. */
		    char * enabled = Cf_GetVal( "enabled", cf );
		    if (enabled && (strcmp(enabled, "true") == 0)) {
			/* load it */
			char * newScope;
			if (scope) {
			    newScope = (char *)malloc( strlen(scope)+strlen(base)+2 );
			    sprintf( newScope, "%s_%s", scope, base );
			} else {
			    newScope = (char *)malloc( strlen(base)+1 );
			    sprintf( newScope, "%s", base );
			}

			if (fileType(nfrPath) == '-') {
			    (*perform)( newScope, nfrPath );
			}
			if (fileType(newPath) == 'd') {
			    pkgLoad( newScope, newPath, perform );
			}
			if (newScope) {
			    free(newScope);
			}
		    }
		    Cf_Delete(cf);
		}
	    }

	    if (base) {
		free(base);
	    }
	    if (cfgPath) {
		free(cfgPath);
	    }
	    if (nfrPath) {
		free(nfrPath);
	    }
	    if (newPath) {
		free(newPath);
	    }

	    /*
	    ** now free components created by scandir
	    **/
	    if (namei[0]) {
		free(namei[0]);
	    }
	    namei++;
	}
	if (names) {
	    free(names);
	}
	return(0);
    } else if (cnt < 0) {
	return(-1);
    }
}

