/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/


/*
** ------------------------------
**
** poll.
**
**
*/

#include "nfrd.h"

#include "pcap.h"

char * argv0;
  

/*
** poll management.
** --------------------
**
** poll management structures.
**
*/

struct pollfd * pollFds;
int	nPollFds;		/* number of fd's we're managing */

pollCall_t *	pollCalls = 0;	/* vector of routines to call when we have data. */
void	** 	pollRocks = 0;	/* vector of related rocks for each call. */


/*
** --------------------
**
** pollCall.
**
*/

int
pollCall( fd, call, rock )
int fd;
pollCall_t call;
void * rock;
{
    if (pollCalls == 0) {
	pollFds  = (struct pollfd *)malloc( sizeof(*pollFds) );
	pollCalls = (pollCall_t *)malloc( sizeof(*pollCalls) );
	pollRocks = malloc( sizeof(pollRocks) );
    } else {
	int nCnt = nPollFds+1;
	pollFds  = (struct pollfd *)realloc( pollFds, sizeof(*pollFds)*nCnt );
	pollCalls = (pollCall_t *) realloc( pollCalls, sizeof(*pollCalls)*nCnt );
	pollRocks = realloc( pollRocks, sizeof(pollRocks)*nCnt );
    }

    pollCalls[nPollFds] = call;
    pollRocks[nPollFds] = rock;

    /* fill the poll fd structure. */
    pollFds[nPollFds].fd = fd;
    pollFds[nPollFds].events = POLLIN;
    pollFds[nPollFds].revents = 0;

    nPollFds++;
}

/*
** --------------------
**
** pollUnCall.
**
** Remove `fd' from the poll list (input stream has shut down).
** this doesn't happen often, so just copy down, and resize the structs.
** 
*/

int
pollUnCall( fd )
int fd;
{
    int i, j;

    for (i=0, j=0; i<nPollFds; ) {
	if (fd != pollFds[i].fd) {
	    j++;
	} else {
	}
	++i;
	if ((i != j) && (i != nPollFds)) {
	    /* move all the data about an entry down */
	    pollFds[j].fd = pollFds[i].fd;
	    pollFds[j].events = pollFds[i].events;
	    pollCalls[j] = pollCalls[i];
	    pollRocks[j] = pollRocks[i];
	}
    }
    /*
    ** if i==j, then we didn't find the fd.
    */
    if (i == j) {
	return(1);	/* failure */
    } else {
	nPollFds--;
	pollFds  = (struct pollfd *)realloc( pollFds, sizeof(*pollFds)*nPollFds );
	pollCalls = (pollCall_t *) realloc( pollCalls, sizeof(*pollCalls)*nPollFds );
	pollRocks = (void *)realloc( pollRocks, sizeof(*pollRocks)*nPollFds );
	return(0); /* success */
    }
}


/*
** --------------------
**
** pollInit.
**
*/

pollInit()
{
}

/*
** --------------------
**
** pollOpen.
**
*/

pollOpen()
{
}

/*
** --------------------
**
** pollClose.
**
*/

pollClose()
{
}

/*
** --------------------
**
** pollPerform.
** 
** See what's active, and callout for each active fd. The only parameter
** describes how long to wait.
**
*/

pollPerform( timeOut )
int timeOut;
{
    int nfd = poll( pollFds, nPollFds, timeOut );

    if (nfd < 0) {
	/* error. */
	return 0;
    } else
    if (nfd == 0) {
	/* timed out */
	return 0;
    } else {
	int i;

	for (i=0; i<nPollFds; ++i) {
	    if (pollFds[i].revents) {
		/* return code? XXX */
		(*pollCalls[i])( pollFds[i].fd, pollRocks[i] );
	    }
	}
	return 1;
    }
}

/*
** --------------------
** pollStats.
** display some info about all the current poll connections.
*/

pollStats()
{
    int i;
    uiCurrPrintf( "poll: nPollFds %d\n", nPollFds );
    for (i=0; i<nPollFds; ++i) {
	uiCurrPrintf( "poll: %d fd %d revent %d\n",
		i, pollFds[i].fd, pollFds[i].events, pollFds[i].revents  );
	uiCurrPrintf( "poll: %d pollCall %x pollRock %x\n",
		i, pollCalls[i], pollRocks[i] );
    }
}
