/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/


#include	"nfrd.h"

/*
** doubly linked list queue management.
**
*/

que_t *
queNew(d)
void * d;
{
    que_t * q = (que_t *)malloc( sizeof(*q) );
    /* assert q? */
    q->q_forw = q;
    q->q_back = q;
    q->q_data = d;

    return(q);
}

/*
** Add n after q.
**
** Its assumed that `n' doesn't have any interesting
** forward or backward pointers.
** Its assumed that q does.
** 
** returns a.
*/

que_t *
queAdd(q,n)
que_t * q;
que_t * n;
{
    if ((q == 0) || (n == 0)) {
	return(0);
    }

    n->q_forw = q->q_forw;
    n->q_back = q;

    n->q_forw->q_back = n;
    q->q_forw = n;

    return(n);
}

/*
** There's no before add, cause to add before, you
** just add q->q_back.
*/


/*
** remove it from the list, reinit the pointers, and return it.
*/

que_t *
queDel(d)
que_t * d;
{
    /*
    ** remove it from the current list
    */
    if (d->q_forw && d->q_back) {
	d->q_forw->q_back = d->q_back;
	d->q_back->q_forw = d->q_forw;
    }

    /*
    ** init the pointers.
    */
    d->q_forw = d;
    d->q_back = d;

    return(d);
}

/*
**
** reclaim storage.
*/

void
queFree(f)
que_t * f;
{
    if (f->q_forw && f->q_back) {
	queDel(f);
    }
    free(f);
}

/*
**
*/

int 
queEmpty(q)
que_t * q;
{
    return(q->q_forw == q->q_back);
}

/*
** return 0 if the next element is THIS element.
** return the next element if its something different.
**
** The forward pointer in the queue element should
** point to this element when there's nothing left
** on the list.  This routine is used to see if
** the header pointer ought to be zero-ed.
*/

que_t *
queForw(q)
que_t * q;
{
    return(q->q_forw == q ? 0 : q->q_forw );
}


que_t *
queBack(q)
que_t * q;
{
    return(q->q_back == q ? 0 : q->q_back );
}
