/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/



/*
** recorder.
**
** interface to the recorder children.
*/

#include "nfrd.h"

/*
** Global list of all recorders.
*/
recorder_t * recorders;

/*
** what uid to set the children up as...
*/

int	recorderUid;

/*
** crack.
**
** Crack the line... return a vector of the tokens
** (argv style)
**
*/

char **
crack( line )
char * line;
{
    char ** argv;
    int argc = 0;
    int argi = -1;
    char * lp;

    while (line[0] == ' ') {
	line++;
    }
    if (line[0] == 0) {
	return(0);
    }

    /*
    ** by first scanning past the spaces, the
    ** first argv entry is the same as the malloc'd
    ** value, which makes it possible to free(argv[0])
    */

    lp = strdup( line ); 

    /*
    ** realloc the memory when we run out of argv's.
    */
    argc = 10;
    argv = (char **)malloc( sizeof(*argv)*argc );

    while(lp[0]) {
	/*
	** skip white space in the front.
	*/
	while ((lp[0] == ' ') || (lp[0] == '\t') || (lp[0] == '\r') || (lp[0] == '\n')) {
	    if (lp[0] == 0) {
		break;
	    }
	    lp++;
	}
	if (lp[0] == 0) {
	    break;
	}
	if (argi >= (argc-1)) {
	    argc += 10;
	    argv = realloc( (void *)argv, sizeof(*argv)*argc );
	}

	argv[++argi] = lp;

	/*
	** move to the first space...
	*/
	while ((lp[0] != ' ') && (lp[0] != '\t') && (lp[0] != '\r') && (lp[0] != '\n')) {
	    if (lp[0] == 0) {
		break;
	    }
	    lp++;
	}
	if (lp[0] == 0) {
	    break;
	}
	lp[0] = 0;
	lp++;
    }

    /*
    ** need to zero the end of the vector...
    */
    ++argi;
    argv = realloc( (void *)argv, sizeof(*argv)*(argi+1) );
    argv[argi] = 0;

    /* note: we don't realloc to the right size. */

    return(argv);
}

/*
** crackFree.
**
** return the storage associated with the crack call.
**
**
*/

crackFree( argv )
char ** argv;
{
    if (argv) {
	free( argv[0] );		/* the string we strdup'd */
	free( argv );		/* the vector */
    }
}

/*
** fromFile.
**
** Try to figure out where this filter came from.
**
*/
char *
fromFile(  )
{
    Filter * f = runCurrentGet();
    if (f && f->f_fromFile) {
	return(
	    format( "file: %s:%d", f->f_fromFile, NFR_OP_LINE(runCurrentOp) ));
    } else {
	return(format( "line: %d", NFR_OP_LINE(runCurrentOp) ));
    }
}

/*
** ----------------------------------------
**
** recorderDied()
**
** what to do when a recorder object dies.
*/

void
recorderDied( sig )
{
    int pid;
    int status;

    while ((pid = waitpid( -1, &status, WNOHANG )) > 0) {
	Alert_Args_Generate( NFR_SRC_NFRD, NFR_NFRD_RECORDERDIED, 
		format( "%d", pid ), 0 );
	{
	    recorder_t * r = recorders;
	    while(r) {
		if (pid == r->r_pid) {
		    /*
		    ** only try resurecting this iff it epoch
		    ** time is a minute (or so)
		    */
		    if (r->r_file) {
			fclose(r->r_file);
			r->r_file = 0;
		    }
		    close(r->r_fd); /* overkill? */
		    r->r_fd = 0;
		    r->r_pid = 0;

		    if (time(0) - r->r_epoch > 60) {
			/*
			** XXX do we need an alert to mention we restarted it?
			*/

			if ((r->r_pid = recorderResurrect(r)) == 0) {
			    /* failed to restart the recorder */
			}
		    } else {
			/* he's dead jim */
			/* XXX should we retry in a minute, or not? */
		    }
		    return;
		}
		r = r->r_next;
	    }
	    Alert_Args_Generate( NFR_SRC_NFRD, NFR_NFRD_INTERNALERROR, 
		format( "recorderDied: couldn't find pid %d on recorders list\n", pid ), 0 );
	    /* couldn't find the recorder :^( */
	}
    }
}

/*
** ----------------------------------------
**
** recorderUidSet()
**
**
*/

recorderUidSet( uid )
int uid;
{
    int oldUid = recorderUid;
    recorderUid = uid;
    return(oldUid);
}


/*
** ----------------------------------------
**
** recorderResurrect()
**
** If a recorder has died, we can call this routine, pointing
** at the recorder, and restart it.  Return the pid of the
** child (or 0 if the fork failed).
**
*/

int
recorderResurrect( r )
recorder_t * r;
{
    int pid;
    int sv[2];
    int err;

    Alert_Args_Generate( NFR_SRC_NFRD, NFR_NFRD_RECORDERSTART, 
	r->r_argv[0], util_str(r->r_schema), fromFile(), 0 );
    r->r_restarts++;

    err = socketpair( AF_UNIX, SOCK_STREAM, 0, sv );
    if (err) {
	Alert_Args_Generate( NFR_SRC_NFRD, NFR_ERR_SOCKETPAIR, 0 );
	return(0);
    }
    if ((pid = fork()) == 0) {
	/* child */
	close(sv[1]);
	close(0);
	if (dup(sv[0]) != 0) {
	    /* XXX error, didn't get stdin */
	}
	close(1);
	if (dup(sv[0]) != 1) {
	    /* XXX error, didn't get stdout */
	}

	/*
	** leave fd[2] (stderr) alone,
	** but close down every other fd.
	*/
	{
	    int i;
	    static int maxNumOfFds = 0;
	    if (maxNumOfFds == 0) {
		maxNumOfFds = getMaxFds();
	    }

	    for (i=3; i<maxNumOfFds; ++i) {
		close(i);
	    }
	}

	/*
	** should we be concerned if we cant set the 
	** child's credentials?
	*/
	if (recorderUid) {
	    nfrSetuid( recorderUid );
	}

	{
	    char * path;
	    if (path = nfrCfGetVal( r->r_argv[0] )) {
		/*
		** exec a different path from the one the recorder
		** was created with.
		*/
		execvp( path, r->r_argv );
	    }
	}
	execvp( r->r_argv[0], r->r_argv );
	Alert_Args_Generate( NFR_SRC_NFRD, NFR_ERR_EXEC, r->r_argv[0], 0 );
	exit(1);
    } else 
    if (pid < 0) {
	Alert_Args_Generate( NFR_SRC_NFRD, NFR_ERR_FORK, 
		format( "%d", errno ), 0 );
	return(0);
    }

    /* parent */
    close(sv[0]);
    r->r_fd = sv[1];
    r->r_epoch = time(0);

    return(pid);
}

/*
** ----------------------------------------
**
** recorderNew
** pid is left to be filled out later... as
** children are managed... epoch is left for
** later too, its set as children run...
*/

recorder_t *
recorderNew( argv, schema )
char ** argv;
char * schema;
{
    recorder_t * rp = (recorder_t *)calloc( sizeof(*rp), 1 );

    /* XXX assert(rp) ? */

    rp->r_refCnt = 1;
    rp->r_argv = argv;

    if (schema) {
	rp->r_schema = strdup( schema );
    }

    rp->r_next = recorders;
    recorders = rp;

    return(rp);
}

/*
** ----------------------------------------
** Tell the other end we're shutt'n down.
** then free all our data.
*/

recorderFree( r )
recorder_t * r;
{
    if (r) {
	/*
	** first, take it off the list.
	*/
	if (recorders == r) {
	    recorders = r->r_next;
	} else {
	    recorder_t * l = recorders;
	    while(l) {
		if (l->r_next = r) {
		    l->r_next = r->r_next;
		    break;
		}
		l = l->r_next;
	    }
	    if (l == 0) {
		/* didn't find it on the list :^( */
		Alert_Args_Generate( NFR_SRC_NFRD, NFR_NFRD_INTERNALERROR, 
			format( "recorderFree: r %x not on recorders list\n", r ), 0 );
	    } else {
		if (l->r_next == 0) {
		    l->r_next = 0;
		} else {
		    l->r_next = l->r_next->r_next; /* remove r from the list */
		}
	    }
	}
	/*
	** now clean up `r'
	*/
	if (r->r_file) {
	    fclose(r->r_file);
	    r->r_file = 0;
	    r->r_fd = 0;
	} else if (r->r_fd) {
	    close(r->r_fd);
	    r->r_fd  = 0;
	}
    }
}

/*
** ----------------------------------------
*/
recorderIncRef(r)
recorder_t * r;
{
    if (r) {
	r->r_refCnt++;
    }
}

/*
** ----------------------------------------
*/

void 
recorderDecRef(v)
void * v;
{
    recorder_t * r = (recorder_t *)v;
    if (--r->r_refCnt == 0) {
	recorderFree(r);
    }
}

/*
** ----------------------------------------
**
** recorderName
**
** Given a recorder_t, return a string describing
** that recorder_t.
**
*/

static char recorderNameBuffer[256];		/* XXX */

char *
recorderName( r )
recorder_t * r;
{
    if (r) {
	sprintf( recorderNameBuffer,
		"%s schema(%s) filter(%s)",
		util_str(r->r_argv[0]),
		util_str(r->r_schema) );
    } else {
	sprintf( recorderNameBuffer, "<recorder null>", r->r_argv[0], r->r_schema );
    }
    return( recorderNameBuffer );
}


/*
** ----------------------------------------
**
** recordChild
**
** pass in a string to exec, return a cookie,
** which gets passed back when we wanna write
** to that child.
*/

void *
recordChild( v, schema )
char * v;
char * schema;
{
    /*
    ** crack the string into argv list, then
    ** get the interface into the child, and
    ** fork to let it spin...
    */
    recorder_t * new;
    char ** argv = crack(v);

    new = recorderNew( argv, schema );
    new->r_pid = recorderResurrect(new);

    return( (void *)new );
}


/*
** ----------------------------------------
**
** recordWrite ... interval version, which
** is passed the schemaName.
**
*/


void
nfr_blt_recordWrite_int( m, n, f, result, schemaName )
Machine * m;
int n;		/* number of expressions */
int f;		/* first expression */
int result;	/* result. */
char * schemaName;
{
    MemVal * mv = &(m->m_mem[f+n-1]);	/* target of the record expression */

    /*
    ** type mv, ought to be a recorder object.
    ** If so, then we get the schema out of mv's type,
    ** and then walk its type, match it with the list
    ** in m1, and write the associated schema into
    ** out to the recorder. 
    */
    n--; /* we already know about the last (recorder) parameter. */

    {
	recorder_t * r;
	Sym * schemaSym;
	int schemaUnique;
	ListMem * lp = 0;

	if (mv->typ != NFR_TYPE_RECORDER) {
	    Alert_Args_Generate( NFR_SRC_NFRD, NFR_NFRD_INTERNALERROR, 
		format("recordWrite: bad last parameter type <need RECORDER>, got %d(%s)\n",
			mv->typ, util_type2str(mv->typ) ), 0 );
	    return;
	}

	/*
	** look up the schema... start the type matchine.
	** if the types match, then write the schema number,
	** and then each of the entries on the list.
	*/
	if ((r = (recorder_t *)(mv->val.v)) == 0) {
	    Alert_Args_Generate( NFR_SRC_NFRD, NFR_NFRD_INTERNALERROR, 
		"recordWrite: no recorder_t value.", 0 );
	    return;
	}

	/*
	** If there are no parameters, this is a flush...
	*/

	if (n == 0) {
	    if (r->r_file) {
		fflush(r->r_file);
	    }
	    return;
	}

	if (r->r_schema == 0) {
	    /*
	    ** If there isn't a schemas associated with the recorder,
	    ** see if the first parameter is a list... if so, then
	    ** we'll try using that as a schema...
	    */
	    MemVal * fst = &(m->m_mem[f]);
	    if (fst->typ != NFR_TYPE_LIST) {
		Alert_Args_Generate( NFR_SRC_NFRD, NFR_NFRD_INTERNALERROR,
			"recordWrite: no schema associated with recorder_t.", 0 );
		return;
	    } else {
		lp = fst->val.v;
		f++;
		n--;
	    }
	} else {
	    /*
	    ** Look for the schema definition in the scope of the procedure.
	    */
	    schemaSym = run_lookup( runCurrentGet(), schemaName, NFR_OP_GEVAL );
	    if (schemaSym == 0) {
		Alert_Args_Generate( NFR_SRC_NFRD, NFR_NFRD_INTERNALERROR,
		    format("recordWrite: can't find schema %s:%s\n",
			runCurrentGet()->f_scope, schemaName ), 0 );
		return;
	    }

	    if (schemaSym->s_data.typ != NFR_TYPE_LIST) {
		Alert_Args_Generate( NFR_SRC_NFRD, NFR_NFRD_INTERNALERROR,
		    format("recordWrite: schema %s has wrong type %d/%s\n",
			r->r_schema, schemaSym->s_data.typ,
			util_type2str(schemaSym->s_data.typ) ), 0 );
		return;
	    }

	    lp = (ListMem *)schemaSym->s_data.val.v;
	}

	/*
	** The first entry on the list is the uniqueSchema number.
	*/
	{
	    MemVal * mp = &lp->data;
	    if (mp->typ != NFR_TYPE_INT) {
		Alert_Args_Generate( NFR_SRC_NFRD, NFR_NFRD_INTERNALERROR,
			format( "recordWrite: %s, schema number missing...",
				fromFile() ), 0 );
		return;
	    }
	    schemaUnique = mp->val.i;
	    lp = lp->next;
	}

	/*
	** Now try to match the types...
	**
	** If we are gonna make the schema, make sure we make
	** one COMPLETE schema..
	*/
	{
	    ListMem * slp = lp; 	/* schema list */
	    int cnt = 0;

	    int anyErrors = 0;

	    while(slp && (cnt <= n)) {
		if (slp->data.val.i != m->m_mem[f+cnt].typ) {

		    /*
		    ** XXX a temporary hack to deal with an odd problem
		    ** which we'll fix shortly.
		    */
		    if (alertsSet()) {
		    Alert_Args_Generate( NFR_SRC_NFRD, NFR_NFRD_INTERNALERROR,
			format( "recordWrite: %s type mismatch...arg %d "
			    	"(want %s (%x), got %s (%x) ) in %s/%s: %x\n",
			    fromFile(),
			    cnt+1, util_type2str(slp->data.val.i), slp->data.val.i,
			    util_type2str(m->m_mem[f+cnt].typ), m->m_mem[f+cnt].typ,
			    recorderName(r), r->r_schema), fromFile(), 0 );

		    uiCurrPrintf( "recordWrite: type mismatch... %s arg %d (want %s (%x), got %s (%x) ) in %s/%s: %s\n",
			fromFile(),
			cnt+1,
			util_type2str(slp->data.val.i), slp->data.val.i,
			util_type2str(m->m_mem[f+cnt].typ), m->m_mem[f+cnt].typ,
			recorderName(r), r->r_schema, fromFile() );
		    Display(1, "recordWrite: type mismatch" );
		    }
		    anyErrors++;
		}
		cnt++;
		slp = slp->next;
	    }
	    if (anyErrors) {
		/* don't continue if we get parameter type mismatches. */
		r->r_errCnt++;
		return;
	    }
	    if (cnt != n) {
		/* don't do this if we don't have enough parameters */
		if (n < cnt) {
		    Alert_Args_Generate( NFR_SRC_NFRD, NFR_NFRD_INTERNALERROR,
			format("recordWrite: insufficient number of "
			"parameters, %s: actual %d wanted %d\n",
			    fromFile(), n, cnt ), 0 );
		    return;
		}
		{
		    Alert_Args_Generate( NFR_SRC_NFRD, NFR_NFRD_INTERNALERROR,
			format( "recordWrite: mismatched number of parameters,"
				"%s: actual %d wanted %d\n",
				    fromFile(), n, cnt ), 0 );
		}
		n = cnt;
	    }
	}

	/*
	** Now try to write the types.
	*/
	if (r->r_file == 0 && (r->r_fd > 0)) {
	    r->r_file = fdopen( r->r_fd, "w" );
	}
	if (r->r_file == 0) {
	    /* sorry, can't write, but don't incr the r_cnt. */
	    r->r_errCnt++;
	    return;
	}

	r->r_dirtyCnt++;
	r->r_cnt++;


	{
	    int cnt = fwrite( &schemaUnique, sizeof(schemaUnique), 1, r->r_file );
	    if (cnt != 1) {
		if (ferror(r->r_file)) {
		    /* we could get told the connection has dropped. */
		    if (errno == EPIPE) {
			/* the connection is shut down (do we get valid errno here?) */
			Alert_Args_Generate( NFR_SRC_NFRD, NFR_NFRD_INTERNALERROR, 
			    format( "recordWrite: errno/EPIPE connect lost to %s\n",
				recorderName(r) ), 0 );
			uiCurrPrintf( FL,
			    "recordWrite: errno/EPIPE connect lost to %s\n",
			    recorderName(r) );

			fclose(r->r_file);
			r->r_file = 0;
			r->r_fd = 0;
		    }
		}
	    }
	    r->r_bytes = counter_add( r->r_bytes, counter_from_int(sizeof(schemaUnique)) );
	}

	/* XXX error codes? test return from fwrites? */
	if (r->r_file) {
	    int i;
	    int cnt;

	    for (i=0; i<n; ++i) {
		MemVal * mp = &(m->m_mem[f+i]);
		switch (mp->typ) {
		    case NFR_TYPE_IPV4HOST:
		    case NFR_TYPE_INT:
			cnt = fwrite( &mp->val.i, sizeof(mp->val.i), 1, r->r_file );
			cnt *= sizeof(mp->val.i);
			break;

		    case NFR_TYPE_IPV4NET:
			cnt = fwrite( mp->val.n, sizeof(*mp->val.n), 1, r->r_file );
			cnt *= sizeof(*mp->val.n);
			break;

		    case NFR_TYPE_STR:
			{
			int len = strlen(mp->val.c);

			cnt = fwrite( &len, sizeof(len), 1, r->r_file );
			cnt *= sizeof(len);
			cnt += fwrite( mp->val.c, sizeof(char), len, r->r_file );
			}
			break;

		    case NFR_TYPE_BLOB:
			if (mp->val.b) {
			    cnt = fwrite( &mp->val.b->b_len,
				sizeof(mp->val.b->b_len), 1, r->r_file );
			    cnt *= sizeof(mp->val.b->b_len);
			    cnt += fwrite( mp->val.b->b_data, sizeof(char),
				mp->val.b->b_len, r->r_file );
			} else {
			    int zero = 0;
			    cnt = fwrite( &zero, sizeof(zero), 1, r->r_file );
			}
			break;

		    case NFR_TYPE_ETHMAC:
			/*
			* DO NOT use sizeof(mp->val.e) here!  It will be
			* the size of the union, not the 6 bytes of the
			* ethernet address.
			*/
			cnt = fwrite( &mp->val.e, sizeof(mp->val.e.e_addr), 1, r->r_file );
			cnt *= sizeof(mp->val.e.e_addr);
			break;

		    /*
		    ** note: seems that writing a recorder value out
		    ** to a recorder isn't such a hot idea (yet).
		    ** maybe a schema would be a good idea tho
		    ** (we may be able to write a schema type, to
		    ** tell the recorder what's in the schema).
		    */
		    default:
			{
			    Alert_Args_Generate( NFR_SRC_NFRD, NFR_NFRD_INTERNALERROR, 
				format( "recordWrite: parm %d can't record basic type: %d/%s\n",
				i+1, mp->typ, util_type2str(mp->typ) ) , 0 );
			}
			break;
		}

	    r->r_bytes = counter_add( r->r_bytes, counter_from_int(cnt) );
	    }
	}
    }
}

/*
** ----------------------------------------
**
** recordWrite
**
*/

void
nfr_blt_recordWrite( m, n, f, result )
Machine * m;
int n;		/* number of expressions */
int f;		/* first expression */
int result;	/* result. */
{
    MemVal * mv = &(m->m_mem[f+n-1]);   /* target of the record expression */
    /* find the schemaName associated with the recorder, hand it over */
    recorder_t * r;

    if (mv->typ != NFR_TYPE_RECORDER) {
	Alert_Args_Generate( NFR_SRC_NFRD, NFR_NFRD_INTERNALERROR, 
	    format( "recordWrite: bad last parameter type <need RECORD>, got %d(%s)\n",
		    mv->typ, util_type2str(mv->typ) ), 0 );
	return;
    }

    /*
    ** look up the schema... start the type machine.
    ** if the types match, then write the schema number,
    ** and then each of the entries on the list.
    */
    if ((r = (recorder_t *)(mv->val.v)) == 0) {
	Alert_Args_Generate( NFR_SRC_NFRD, NFR_NFRD_INTERNALERROR,
	    "recordWrite: no recorder_t value.", 0 );
	return;
    }

    nfr_blt_recordWrite_int( m, n, f, result, r->r_schema );

}


/*
** ----------------------------------------
** recorderNew.
**
** Builtin function which returns a recorer type.
** Usage:
**
** n = recorderNew( name );
*/

void
nfr_blt_recorderNew(Machine *m,int argc,int firstarg,int result)
{
    MemVal * v = &m->m_mem[firstarg];			/* thing to run */
    MemVal * s = &m->m_mem[firstarg+1];			/* schema name */
    recorder_t * r = 0;
    char * schemaName = 0;

    if (argc > 1) {
	switch (s->typ) {
	    case NFR_TYPE_STR:
		schemaName = strdup(s->val.c);
		break;
	    
	    default:
		break;
	}
    }

    switch(v->typ) {
	case	NFR_TYPE_STR:
	    r = recordChild( v->val.c, schemaName );
	    break;

	default:
	case	NFR_TYPE_ERR:
	    break;
    }

    if (r) {
	m->m_mem[result].typ = NFR_TYPE_RECORDER;
	m->m_mem[result].val.v = (void *)r;	
    } else {
	m->m_mem[result].typ = NFR_TYPE_ERR;
	m->m_mem[result].val.i = NFR_ERR_ARGUMENT;
    }
}

/*
** ----------------------------------------
** recorderSync.
** 
** To each recorder with an open file descriptor,
** send down a sync command... This happens infrequently.
*/


void
recorderSync( Machine * mac  ) {
    recorder_t * r = recorders;
    time_t now;

    now=time(0);

    while(r) {
	/*
	** to do this, we gotta push some junk on the stack
	** then call the builtin to write the schema.
	*/
	if (mac) {
	    char * schemaName = "timestampSchema";
	    static int timestampSchemaExists = 0; /* 0:uninit, 1:ok, -1:error */

	    switch(timestampSchemaExists) {
		case 0:	/* we look to see if it exists. */
		    timestampSchemaExists = 1;
		    if (run_lookup( globalFilter, schemaName, NFR_OP_GEVAL ) == 0) {
			ListMem tsSchemaInt[1], tsSchema[1];
			MemVal value[1];
			/*
			** creat the schema the way we think
			** its supposed to look:
			** timestampSchema  = [ 57007, 1 ];
			*/
			tsSchemaInt->data.typ = NFR_TYPE_INT;
			tsSchemaInt->data.val.i = 1;
			tsSchemaInt->next = 0;

			tsSchema->data.typ = NFR_TYPE_INT;
			tsSchema->data.val.i = 57007;
			tsSchema->next = tsSchemaInt;

			value->typ = NFR_TYPE_LIST;
			value->val.v = tsSchema;

			run_globalAssign( schemaName, value );

			if (run_lookup( globalFilter, schemaName, NFR_OP_GEVAL ) == 0) {
			    /* just log one error about the missing schema. */
			    Alert_Args_Generate( NFR_SRC_NFRD, NFR_NFRD_INTERNALERROR,
				format("recordWrite: can't find schema %s\n", schemaName ), 0 );
			    timestampSchemaExists = -1;
			    break;
				    
			} else {
			    /* fall through */
			    log( FL, "recorderSync: created timestampSchema\n" );
			}
		    }
		    /* fall through */
		    
		case 1: /* the schema exists. */
		    {
		    MemVal recMem, command, pops;
		    Filter * saveCurrent = runCurrentSet(globalFilter);

		    command.typ = NFR_TYPE_INT;
		    command.val.i = now;
		    stack_push( mac, &command );

		    recMem.typ = NFR_TYPE_RECORDER;
		    recMem.val.v = (void *)r; /* XXX */
		    stack_push( mac, &recMem );

		    nfr_blt_recordWrite_int(mac, 2,
			mac->m_sp-2, mac->m_sp-2, schemaName );
		    runCurrentSet(saveCurrent);


		    stack_pop( mac, &pops );
		    stack_pop( mac, &pops );
		    }
		    break;

		case -1: /* the timestampSchema doesn't exist */
		    break;

		default:
		    Alert_Args_Generate( NFR_SRC_NFRD, NFR_NFRD_INTERNALERROR,
			format("recordWrite: bad value of timestampSchemaExists: %d\n",
				timestampSchemaExists ), 0 );
	    }
	}

	if (r->r_file && r->r_dirtyCnt) {
	    /*
	    ** If no machine was passed in, then we'll just flush
	    ** any data associted with the recorders.
	    */
	    int err = fflush( r->r_file );
	    if (err != 0) {
		if (errno == EPIPE) {
		    /* other end has shut down.
		    ** We ought to disable more stuff, but for right
		    ** now, we'll just sever the connection.
		    */
		    fclose( r->r_file );
		    r->r_file = 0;
		    r->r_fd = 0;

		}
	    }
	    r->r_dirtyCnt = 0;
	}

	/*
	*/
	r = r->r_next;
    }
}

/*
** ----------------------------------------
**
** recorderInit()
**
** recorder specific initalization.
** 
** set signal processing routine for children.
*/

void
recorderInit( sec )
int sec;
{
    struct sigaction act, oldAct;
    sigset_t sigs;
    sigemptyset( &act.sa_mask );

    act.sa_handler  = recorderDied;
    act.sa_flags    = SA_NOCLDSTOP | SA_RESTART;

    sigaction( SIGCHLD, &act, &oldAct );
    /* XXX return values */
}


/*
**
** ----------------------------------------
** recorderStat.
**
** Display details about a recorder
*/

void
recorderStat( r )
recorder_t * r;
{
    char cbuf[COUNTER_TEXT_SIZE];
    char epoch[64];

    counter_itoa( cbuf, r->r_bytes );

    strftime(epoch, sizeof(epoch), "%D %T", localtime( &r->r_epoch ) );

    uiCurrPrintf( "recorder: %s epoch %s restarts %d fd %d pid %d FILE* %x\n",	
	    r->r_argv[0], epoch, r->r_restarts, r->r_fd, r->r_pid, r->r_file );
    uiCurrPrintf( "recorder: %s dirtyCnt %d cnt %d bytes %s errCnt %d\n",
	r->r_argv[0], r->r_dirtyCnt, r->r_cnt, cbuf, r->r_errCnt );
    uiCurrPrintf( "recorder: %s schema %s\n",
	r->r_argv[0], util_str(r->r_schema) );
}

/*
**
** ----------------------------------------
** recorderStats.
**
** Display all the currently running recorder, and some small
** amount of info about each.
*/

void
recorderStats()
{
    recorder_t * r = recorders;
    while(r) {
	recorderStat(r);
	r = r->r_next;
    }
}

/*
** ----------------------------------------
**
** recorderConfigRead
**
** Read the file, finding variables names,
** and building an array of config_t's
** describing the contents of the file.
**
*/

config_t **
recorderConfigRead( char * f )
{
    FILE * in = fopen( f, "r" );
    if (in) {
	int array_max = 10;
	config_t ** array = (config_t **)calloc( sizeof(*array), array_max );
	int array_i = 0;
	int c;

	while ( (c = fgetc( in )) != EOF ) {
	    /* discard comments ... */
	    while(isspace(c)) {
		c = fgetc(in);
	    }
	    if (c == EOF) {
		break;
	    }

	    if (c == '#') {
		while( (c = fgetc(in)) != '\n' ) {
		}
	    } else {
		/* find the end of the first symbol... */
		int cmax = 32;
		char * cdata = (char *)malloc( cmax );
		int cndx = 0;
		char * vdata = 0;

		cdata[cndx++] = c;
		while (c = fgetc(in) != EOF) {
		    if (c == '=') {
			break;
		    }
		    if (cndx >= cmax) {
			cmax += 32;
			cdata = (char *)realloc( cdata, cmax );
		    }
		    cdata[cndx++] = c;
		}
		cdata = (char *)realloc( cdata, cndx+1 );
		cdata[cndx] = 0;

		/*
		** We ought to be pointing at the '='
		*/
		if (c == '=') {
		    /*
		    ** now look for the value... if we see a quote
		    ** or a doublequote, then we'll use an unpickling
		    ** routine to 
		    */

		    while ( isspace(c = fgetc( in )) ) {
			if ((c == '\n') || (c == EOF)) {
			    goto no_value;
			}
		    }
		    if ((c == '"') || (c == '\'')) {
			/*
			** unpickle the string.
			*/
			extern int stringUnpickle( FILE *, char** ); /* XXX not quite */
			int err = stringUnpickle( in, &vdata );
		    } else {
			int vmax = 32;
			int vndx = 0;

			vdata = (char *)malloc( vmax );
			/*
			** collect the value... collect chars
			** till we hit a space...
			** XXX
			*/
			vdata[vndx++] = c;
			while( !isspace(c = fgetc( in )) ) {
			    if (vndx >= vmax) {
				vmax += 32;
				vdata = (char *)realloc( vdata, vmax );
			    }
			    vdata[vndx] = c;
			}
			vdata = (char *)realloc( vdata, vndx+1 );
			vdata[vndx] = 0;
		    }
		} else {
		    no_value:
		    /*
		    ** no values... just set the name of the symbol
		    */
		    vdata = 0;
		}

		/*
		** Add the name/value pair to the config list...
		*/
		if (array_i > array_max) {
		    array_max += 10;
		    array = (config_t **)realloc( array, array_max*(sizeof(*array)) );
		}
		array[array_i++]->c_name = cdata;
		array[array_i++]->c_value = vdata;
	    }
	}
	fclose(in);

	array = (config_t **)realloc( array, (array_i+1)*(sizeof(*array)) );
	array[array_i]->c_name = 0;
	array[array_i]->c_value = 0;

	return(array);
    } else {
	return(0);
    }
}

/*
** ----------------------------------------
**
** recorderGetConfig
**
** Try to locate the value associated with some name.
*/

char *
recorderGetConfig( c, name )
config_t ** c;
char *	name;
{
    config_t ** p;

    for ( p = c; p[0]->c_name; ++p ) {
	if (strcmp( name, p[0]->c_name ) == 0) {
		return( p[0]->c_value );
	}
    }
    return(0);
}
