/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/


/*
** ------------------------------
**
** select.
**
** similar to poll,
** we just do it a different way.
** 
** we provide the same external interface, tho.
**
*/

#include "nfrd.h"

#include "pcap.h"

char * argv0;
  

/*
** poll management.
** --------------------
**
** poll management structures.
**
*/

fd_set pollIns;
fd_set pollOuts;
fd_set pollXepts;

int	nPollFds;		/* number of fd's we're managing */
int	pollHighFd = 0;		/* highest fd we've seen */

int	*	pollFds = 0;	/* vector of file descriptors */
pollCall_t *	pollCalls = 0;	/* vector of routines to call when we have data. */
void	** 	pollRocks = 0;	/* vector of related rocks for each call. */


int pollAnyInput = 0;
int pollAnyOutput = 0;
int pollAnyXepts = 0;


/*
** --------------------
**
** pollCall.
**
** Use pollCall only for inbound data.
** For fd's which you'd like to watch outbound (or except)
** traffic, create new interfaces.
** It may be quite a chore to write those routines,
** cause select doesn't use three nFd's as parameters,
** but has one collective count.  Cause of that,
** there'd likely have to be lotsa realloc's...
**
** There's this assumption, cause of the way the code
** is written... that we won't be adding or removing
** lotsa outbound calls all the time... if that isn't true
** these routines ought to be re-written.
**
*/

int
pollCall( fd, call, rock )
int fd;
pollCall_t call;
void * rock;
{
    if (pollCalls == 0) {
	pollFds  = (int *)malloc( sizeof(*pollFds) );
	pollCalls = (pollCall_t *)malloc( sizeof(*pollCalls) );
	pollRocks = malloc( sizeof(pollRocks) );
    } else {
	int nCnt = nPollFds+1;
	pollFds  = (int *)realloc( pollFds, sizeof(*pollFds)*nCnt );
	pollCalls = (pollCall_t *) realloc( pollCalls, sizeof(*pollCalls)*nCnt );
	pollRocks = realloc( pollRocks, sizeof(pollRocks)*nCnt );
    }

    pollCalls[nPollFds] = call;
    pollRocks[nPollFds] = rock;
    pollFds[nPollFds] = fd;

    /* fill the poll fd structure. */
    if (fd > pollHighFd) {
	pollHighFd = fd;
    }

    FD_SET( fd, &pollIns );

    nPollFds++;
    pollAnyInput++;
}

/*
** --------------------
**
** pollUnCall.
**
** Remove `fd' from the poll list (input stream has shut down).
** this doesn't happen often, so just copy down, and resize the structs.
** 
*/

int
pollUnCall( fd )
int fd;
{
    int i, j;

    for (i=0, j=0; i<nPollFds; ) {
	if (fd != pollFds[i]) {
	    j++;
	} else {
	    FD_CLR( fd, &pollIns );
	}
	++i;
	if (i != j) {
	    pollFds[j] = pollFds[i];
	    pollCalls[j] = pollCalls[i];
	    pollRocks[j] = pollRocks[i];
	}
    }
    /*
    ** if i==j, then we didn't find the fd.
    */
    if (i == j) {
	printf( "pollUnCall: can't find fd %d\n", fd );
	return(1);	/* failure */
    } else {
	nPollFds--;
	pollAnyInput--;
	pollFds  = (int *)realloc( pollFds, sizeof(*pollFds)*nPollFds );
	pollCalls = (pollCall_t *) realloc( pollCalls, sizeof(*pollCalls)*nPollFds );
	pollRocks = (void *)realloc( pollRocks, sizeof(*pollRocks)*nPollFds );

	/*
	** Reset the highest number fd...
	{
	int i;
	pollHighFd = -1;

	for (i=0; i<nPollFds; ++i) {
	    if (pollFds[i] > pollHighFd) {
		pollHighFd = pollFds[i];
	    }
	}

	}
	return(0); /* success */
    }
}


/*
** --------------------
**
** pollInit.
**
*/

pollInit()
{
	FD_ZERO( &pollIns );
	FD_ZERO( &pollOuts );
	FD_ZERO( &pollXepts );
}

/*
** --------------------
**
** pollOpen.
**
*/

pollOpen()
{
}

/*
** --------------------
**
** pollClose.
**
*/

pollClose()
{
}

/*
** --------------------
**
** pollFindi.
** 
** pass in the fd, return the index (or -1 if missing)
**
*/

int
pollFindi( fd )
{
    int i;
    for (i=0; i<nPollFds; ++i) {
    	if (fd == pollFds[i]) {
	    return (i);
	}
    }
    return(-1);
}

/*
** --------------------
**
** pollPerform.
** 
** use select to find out what fd's need help...
**
*/

pollPerform( timeOut )
int timeOut;
{
    int maxFd = pollHighFd+1;
    int nfd;
    fd_set tPollIns;
    fd_set tPollOuts;
    fd_set tPollXepts;
    struct timeval tv;
    struct timeval *tvp = 0;
    
    /*
    ** copy the fd_set into the targets, and get ready for
    ** selecting...
    */

    tPollIns = pollIns;
    tPollOuts = pollOuts;
    tPollXepts = pollXepts;

    if (timeOut != -1) {
	/*
	** Set up the timeout.
	*/
	tv.tv_sec = timeOut / 1000;
	tv.tv_usec = (timeOut % 1000) * 1000;
	tvp = &tv;
    }

    LOGV( 2, ( FL, "doing select: %d\n", maxFd ));
    nfd = select( maxFd, &tPollIns, &tPollOuts, &tPollXepts, tvp );
    LOGV( 2, ( FL, "select done: %d\n", nfd ));

    if (nfd < 0) {
	/* error. */
	return 0;
    } else
    if (nfd == 0) {
	/* timed out */
	return 0;
    } else {
	int i;

	/*
	** Check in input fd's
	*/
	if (pollAnyInput) {
	    for (i=0; i<maxFd; ++i) {
		if (FD_ISSET( i, &tPollIns ) ) {
		    int x = pollFindi( i );
		    /* return code? XXX */
		    LOGV( 2, ( FL, "pollPerform: read i %d x %d fd %d\n",
			    i, x, pollFds[x] ));
		    (*pollCalls[x])( pollFds[x], pollRocks[x] );
		}
	    }
	}
	/*
	** Check the output fd's
	*/
	if (pollAnyOutput) {
	    for (i=0; i<maxFd; ++i) {
		if (FD_ISSET( i, &tPollOuts ) ) {
		    int x = pollFindi( i );
		    /* return code? XXX */
		    LOGV( 2, ( FL, "pollPerform: write i %d x %d fd %d\n",
			    i, x, pollFds[x] ) );
		    (*pollCalls[x])( pollFds[x], pollRocks[x] );
		}
	    }
	}
	/*
	** Check the exception fd's
	*/
	if (pollAnyXepts) {
	    for (i=0; i<maxFd; ++i) {
		if (FD_ISSET( i, &tPollOuts ) ) {
		    int x = pollFindi( i );
		    /* return code? XXX */
		    LOGV( 2, ( FL, "pollPerform: xept i %d x %d fd %d\n",
			    i, x, pollFds[x] ) );
		    (*pollCalls[x])( pollFds[x], pollRocks[x] );
		}
	    }
	}
	return 1;
    }
}

/*
** --------------------
**
** pollStats.
** 
** Display some of the internal state used to
** perform the polling.
*/

pollStats()
{
    int maxFd = pollHighFd+1;
    uiCurrPrintf( "poll: nPollFds %d pollHighFd %d\n", nPollFds, pollHighFd ); 
    {
	int i;
	for (i=0; i<nPollFds; ++i) {
	    uiCurrPrintf( "poll: %d fd %d pollCalls %x pollRock %x\n",
		    i, pollFds[i], pollCalls[i], pollRocks[i] );
	}
    }
    /*
    ** display the select vectors.
    */
    {
	int i;
	uiCurrPrintf( "poll: input select: " );
	for (i=0; i<maxFd; ++i) {
		if (FD_ISSET( i, &pollIns )) {
			uiCurrPrintf( "%d ", i );
		}
	}
	uiCurrPrintf( "\n" );
	/* -------------------- */
	uiCurrPrintf( "poll: output select: " );
	for (i=0; i<maxFd; ++i) {
		if (FD_ISSET( i, &pollOuts )) {
			uiCurrPrintf( "%d ", i );
		}
	}
	uiCurrPrintf( "\n" );
	/* -------------------- */
	uiCurrPrintf( "poll: exception select: " );
	for (i=0; i<maxFd; ++i) {
		if (FD_ISSET( i, &pollOuts )) {
			uiCurrPrintf( "%d ", i );
		}
	}
	uiCurrPrintf( "\n" );
    }
}
