/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/


/*
**
*/

#include "nfrd.h"

void sigTermHandler( sig )
int sig;
{
    /* 
    ** clean up.
    */
    nfrdUninit();
    nfrdHalt();
    return;
}

void sigLog( sig )
int sig;
{
    {
	char sigNo[32];
	sprintf( sigNo, "%d", sig );
	Alert_Args_Generate( NFR_SRC_NFRD, NFR_ERR_SIGNAL, sigNo, 0 );
    }
    return;
}

/*
** child death. generic child death.
*/
void sigChild( sig )
int sig;
{
    int pid;
    int status;
	     
    sigLog( SIGCHLD );
    while ((pid = waitpid( -1, &status, WNOHANG )) > 0) {
	/*
	** XXX needs to be connected to recorder death.
	*/
	if (vFlag) {
	    log( FL, "sigChild pid %d\n", pid );
	}
    }
}

/*
**
*/
sigInit()
{
    struct sigaction act, oldAct;
    sigset_t sigs;

    sigemptyset( &sigs );
    sigaddset( &sigs, SIGHUP );
    sigaddset( &sigs, SIGINT );
    sigaddset( &sigs, SIGQUIT );
    sigaddset( &sigs, SIGSEGV );
    act.sa_handler  = sigLog;
    act.sa_mask     = sigs;
    act.sa_flags    = SA_NOCLDSTOP | SA_RESTART;

    {
	extern int dFlag;

	if (dFlag == 0) {
	    /* point all signals at the sigLog handler. */
	    int i;
	    for (i=1; i<NSIG; ++i) {
		switch(i) {
		    case SIGILL:
		    case SIGBUS:
		    case SIGSEGV:
		    case SIGPIPE:
		    case SIGIOT:	/* assert failures */
			break;

		    default:
			sigaction( i, &act, &oldAct );
			break;
		}
	    }
	}
    }

    /*
    ** handler for child death.
    */
    act.sa_handler  = sigChild;
    sigaction( SIGCHLD, &act, &oldAct );

    /*
    ** SIGTERM handler.
    */
    sigemptyset( &sigs );
    sigaddset( &sigs, SIGTERM );
    act.sa_handler  = sigTermHandler;
    act.sa_mask     = sigs;
    act.sa_flags    = SA_NOCLDSTOP | SA_RESTART;
    sigaction( SIGTERM, &act, &oldAct );

    /*
    ** SIGPIPE.
    ** ignore sigpipe.
    */
    sigemptyset( &sigs );
    sigaddset( &sigs, SIGPIPE );
    act.sa_handler    =  SIG_IGN;
    act.sa_mask     = sigs;
    act.sa_flags    = SA_NOCLDSTOP | SA_RESTART;
    sigaction( SIGPIPE, &act, &oldAct );

}
