/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/


#include "nfrd.h"

/*
** tcpip data reassembly.
**
** we use the routines in packet.c to aid us in our
** reconstruction goals...
*/


extern char *                   packet_data;
extern int                      packet_len;
extern struct pcap_pkthdr *     packet_hdr;
extern int                      packet_nonce;

int	vFlag;

/*
** ----------------------------------------
*/

#define	PASTLEN	16

typedef struct tcpPast {
	int	tp_seq;
	int	tp_len;
} tcpPast_t;

#define	TCP_STREAMS	2

typedef struct tcpAssemble {
	struct in_addr	ta_src;
	struct in_addr	ta_dst;
	u_short		ta_sport;
	u_short		ta_dport;

	time_t		ta_epoch;	/* creation time */
	int		ta_hash;	/* built once at connection time */

	int		ta_flags;
	time_t		ta_time;	/* for garbage collection */

	MapBag *	ta_map;		/* associations */

	int		ta_syn;		/* number of syns this conn saw */
	int		ta_synAck;	/* number of syn/acks this conn saw */
	int		ta_rst;		/* number of rsts this conn saw */
	int		ta_rstAck;	/* number of rst/acks this conn saw */
	int		ta_fin;		/* number of fins this conn saw */
	int		ta_finAck;	/* number of fins/acks this conn saw */

	struct	stream  {
	    int		ta_OrigSeq;	/* first sequence number */
	    que_t *	ta_Packets;	/* backward packets. */
	    int		ta_PackNo;	/* lowest sequence number in packets. */
	    timeval_t	ta_Arriv;	/* last packet arrival time. */
	    timeval_t	ta_ArrDiff;	/* diff's of arrival times. */
	    double	ta_ArrSumSq;	/* sum of square of arrival time */
	    double	ta_ArrStdDv;	/* std dev of packet arrival rate */
	    u_int	ta_SeqNo;	/* backward sequence numbers. */
	    int		ta_Bytes;	/* how much data. */
	    int		ta_Lost;	/* how much data discarded. */
	    int		ta_Cnt;		/* total packets added */
	    tcpPast_t	ta_Past[PASTLEN];/* short history */
	    int		ta_PastI;	/* current index into history */

	    que_t *	ta_Matches;	/* list of matches working on */
	    time_t	ta_SeqGc;	/* sequence number garbage collection */
	    short	ta_info;	/* */
	    short	ta_holeCnt;	/* XXX :( */
	}		ta_stream[TCP_STREAMS];

} tcpAssemble_t;


tcpAssemble_t *	tcpConnTa;

/*
** routines to get int32's int16's out of unaligned memory.
*/
static short endian = 0x1234;

/*
** retain inaddr's in network order
*/

int
inaddr( p )
u_char * p;
{
    if ( ((u_char *)&endian)[0] == 0x12) { /* big endian */
	return( (p[0] << 24) | (p[1] << 16) | (p[2] << 8) | (p[3]) );
    } else if ( ((u_char *)&endian)[0] == 0x34) { /* little endian */
	return( (p[0]) | (p[1] << 8) | (p[2] << 16) | (p[3] << 24) );
    }
    /* trouble */
    return(0);
}

int
int32( p )
u_char * p;
{
    return( (p[0] << 24) | (p[1] << 16) | (p[2] << 8) | (p[3]) );
}

unsigned short
int16( p )
u_char * p;
{
    return( (p[0] << 8) | (p[1]) );
}


/*
** memInt
** If the MemVal is an int, then return the value,
** otherwise, return -1...
**
** take care in using this routine: it can return
** values which 
*/

#define memInt32(m) ((m) ? (((m)->typ == NFR_TYPE_INT) ? (m)->val.i : -1) : -1 )
#define TRUE 1

void reassembleIp();
void reassembleTcp();

#ifndef IP_OFFMASK
#define	IP_OFFMASK 0x1fff
#endif


/*
** ------------------------------------------------------------
**
*/
reassemble()
{
    MemVal m;

    if ((ipIsIp(&m) == 0) && (memInt32(&m) == TRUE)) {
	/*
	** ip reassembly..
	** see if we need to reassemble this thing...
	*/
	struct ip * ip = ipHdr();
	int off = int16( &ip->ip_off );

	ipTriggerDispatch();

	if ( ((off & IP_MF) != 0) || (off & IP_OFFMASK)) {
	    /*
	    ** need to reassemble this packet...
	    */
	    reassembleIp(ip);
	} else {
	    /*
	    ** The packet is complete.  See what protocol we
	    ** ought to tell...
	    */
	    switch ( ip->ip_p ) {
		case IPPROTO_TCP:
		    reassembleTcp();
		    break;
		case IPPROTO_UDP:
		    udpRecv();	/* in packet.c */
		    break;
		case IPPROTO_ICMP:
		    icmpRecv(); /* in packet.c */
		    break;
	    }
	}
    } else {
	/* any other protocols which need reassembly? */
    }
}

/*
** reassemble garbage collect.
** procedure definition compatable with a timeout function call. 
*/

void
ipReassembleGC( void * v )
{
    void reassembleTcpGC();

    /* reassembleIpGC(); */
    reassembleTcpGC();
}

/*
** ip reAssembly
**
**
** -------------------------------------------------------
**
*/

typedef struct ipData {
	struct ip *	id_header;		/* just the header of the packet */
	int		id_hLen;		/* header length */
	char *		id_data;		/* reference to the data */
	int		id_dLen;		/* length of the data */
} ipData_t;

int	ipDataBytes = 0;
int	ipDataCnt = 0;

/*
**
**
*/

ipData_t *
ipDataNew( ipHdr, ipHLen, data, dLen )
struct ip * ipHdr;
int ipHLen;
char * data;
int dLen;
{
    ipData_t * ipd = (ipData_t *)malloc( sizeof(*ipd) );
    /* assert ipd? */

    if (ipHLen) {
	ipd->id_header = (struct ip *)malloc( ipHLen ); 
	/* assert ipd->id_header? */
	memcpy( ipd->id_header, ipHdr, ipHLen );
	ipd->id_hLen = ipHLen;
	ipDataBytes += ipHLen;
    } else {
	ipd->id_header = 0;
	ipd->id_hLen = 0;
    }

    if (dLen) {
	ipd->id_data = (char *)malloc( dLen );
	/* assert ipd->id_data */
	memcpy( ipd->id_data, data, dLen );
	ipd->id_dLen = dLen;
	ipDataBytes += dLen;
    } else {
	ipd->id_data = 0;
	ipd->id_dLen = 0;
    }
    ipDataCnt++;
    return( ipd );
}

/*
** 
*/

void
ipDataFree( ipData_t * d )
{
    if (d) {
	if (vFlag) {
	    printf( "ipDataFree: %x hdr %x/%x data %x/%x\n",	
		d, d->id_hLen, d->id_header, d->id_dLen, d->id_data );
	}
	if (d->id_header && d->id_hLen) {
	    free(d->id_header);
	    d->id_header = 0;
	    ipDataBytes -= d->id_hLen;
	}
	if (d->id_data && d->id_dLen) {
	    free(d->id_data);
	    d->id_data = 0;
	    ipDataBytes -= d->id_dLen;
	}
	ipDataCnt--;
	free(d);
    }
}

/*
**
*/

typedef struct ipAssemble {

	struct in_addr	ia_src;		/* source */
	struct in_addr	ia_dst;		/* destination */
	u_short		ia_id;		/* ident */
	u_char		ia_p;		/* protocol */

	int		ia_flags;	/* did we get a last packet? */
	int		ia_time;	/* seconds, first reassemble packet */

	int		ia_pktCnt;	/* count of the number of packets */

	que_t *		ia_packets;	/* list of packets we've got. */

} ipAssemble_t;

#define	IA_FLAGS_DOREASS	1	/* bit on when we've seen the last-frag bit */


que_t * ipsList;

int ipAssembleByes;
int ipAssembleCnt;
int ipReassembled = 0;
int ipReassembledTcp = 0;

/*
** ipAssembleNew.
** cons up a new one.
*/

ipAssemble_t *
ipAssembleNew( src, dst, id, p )
struct in_addr * src;
struct in_addr * dst;
u_short id;
u_char p;
{
    ipAssemble_t * n = (ipAssemble_t *)malloc( sizeof(*n) );
    /* XXX assert? */
    n->ia_src.s_addr = src->s_addr;
    n->ia_dst.s_addr = dst->s_addr;
    n->ia_id = id;
    n->ia_p = p;

    n->ia_time = 0;
    n->ia_flags = 0;

    n->ia_packets = 0;
    n->ia_pktCnt = 0;

    ipAssembleCnt++;
    return(n);
}

void
ipAssembleFree( ipAssemble_t * ia )
{
    if (ia) {
	que_t * q;
	while (q = ia->ia_packets) {
	    ipData_t * id = (ipData_t *)q->q_data;
	    ipDataFree(id);
	    ia->ia_packets = queForw(q);
	    queFree(q);
	}
	free(ia);
	ipAssembleCnt--;
    }
}

/*
** note that it doesn't return an ipAssemble
*/

que_t *
findIpAssemble( src, dst, id, protocol )
struct in_addr * src;
struct in_addr * dst;
u_short id;
u_char protocol;
{
    que_t * p = ipsList;
    if (p) {
	do {
	    ipAssemble_t * a = (ipAssemble_t *)p->q_data;
	    if ((a->ia_src.s_addr == src->s_addr)
	     && (a->ia_dst.s_addr == dst->s_addr)
	     && (a->ia_id = id) && (a->ia_p == protocol)) {
		/*
		** found the matching entry.
		*/
		return(p);
	    }
	    p = p->q_forw;
	} while ( p != ipsList );
    }
    return(0);
}

#define	IP_OFFSET(ip)	(int16( &((ip)->ip_off) ) & IP_OFFMASK)

/*
** ------------------------------
**
** ipTrigger.
**
*/

typedef struct ipTrigger {
    trigger_t *	it_trigger;
    MemVal *	it_src;
    MemVal *	it_dst;
    MemVal *	it_proto;
    int		it_flags;
} ipTrigger_t;

#define IT_FLAGS_PROTO	0x1
#define IT_FLAGS_SRC	0x2
#define IT_FLAGS_DST	0x4
#define IT_FLAGS_HOST	0x8		/* use src */

ipTrigger_t *
ipTriggerNew( src, dst, proto, flags, t )
MemVal * src;
MemVal * dst;
MemVal * proto;
int	flags;
trigger_t * t;
{
    ipTrigger_t * new = (ipTrigger_t *)malloc( sizeof(*new) );
    new->it_src = src;
    new->it_dst = dst;
    new->it_proto = proto;
    new->it_flags = flags;
    new->it_trigger = t;
    return(new);
}

void
ipTriggerFree( n )
ipTrigger_t * n;
{
    if (n->it_trigger) {
	trigger_t * t = n->it_trigger;
	n->it_trigger = 0;
	triggerDecRef(t);
    }
    free(n);
}

int ipTotalTriggers;
int ipTriggerReap;
que_t * ipTriggers;

void
ipTriggerOff(t)
trigger_t * t;
{
    que_t * q = ipTriggers;
    if (q) {
	do {
	    if (((void *)q->q_data == (void *)t->t_rock) && (q->q_data)) {
		ipTrigger_t * it = (ipTrigger_t *)q->q_data;
		q->q_data = 0;
		ipTriggerFree(it);
		ipTriggerReap++;

		if (q == ipTriggers) {
		    ipTriggers = queForw(q);
		}
		queFree(q);

		return;
	    }
	    q = q->q_forw;
	} while (q != ipTriggers);
    }
}

ipTriggerSet( t )
trigger_t * t;
{
    int i;
    int err = 0;
    int flags = 0;
    MemVal * src = 0, * dst = 0, * proto = 0;

    for (i = 0; i<t->t_ndx; ++i) {
	if (strcmp( t->t_whom[i], "ip" ) == 0) {
	    Sym * l = t->t_nv[i];
	    {
		for ( l = t->t_nv[i]; l ; l = l->s_next ) {
		    MemVal * m = &l->s_data;

		    if (symCmp( l, "src" ) == 0) {
			if (flags & (IT_FLAGS_SRC|IT_FLAGS_HOST)) {
			    uiCurrPrintf( "ipTriggerSet: can't set src twice\n" );
			    err = 1;
			}
			flags |= IT_FLAGS_SRC;
			src = m;
		    } else
		    if (symCmp( l, "host" ) == 0) {
			if (flags & (IT_FLAGS_SRC|IT_FLAGS_DST|IT_FLAGS_HOST)) {
			    uiCurrPrintf( "ipTriggerSet: can't set src/dst/host twice\n" );
			    err = 1;
			}
			flags |= IT_FLAGS_HOST;
			src = m;
		    } else
		    if (symCmp( l, "dst" ) == 0) {
			if (flags & (IT_FLAGS_DST|IT_FLAGS_HOST)) {
			    uiCurrPrintf( "ipTriggerSet: can't set dst twice\n" );
			    err = 1;
			}
			flags |= IT_FLAGS_DST;
			dst = m;
		    } else
		    if (symCmp( l, "proto" ) == 0) {
			if (flags & IT_FLAGS_PROTO) {
			    uiCurrPrintf( "ipTriggerSet: can't set proto twice\n" );
			    err = 1;
			}
			flags |= IT_FLAGS_PROTO;
			proto = m;
		    } else {
			uiCurrPrintf( "ipTriggerSet: unknown keyword `%s'\n",
				symName(l) );
			err = 1;
		    }	
		}
	    }
	}
    }
    if (err == 0) {
	ipTrigger_t * it = ipTriggerNew( src, dst, proto, flags, t );
	que_t * n = queNew(it);

	t->t_free = ipTriggerOff;
	t->t_rock = (void *)it;

	if (queAdd( ipTriggers, n ) == 0) {
	    ipTriggers = n;
	}
	return( (int)n );
    } else {
	/* alert here? XXX */
	uiCurrPrintf( "ipTriggerSet: error in trigger\n" );
    }
    return(0);
}

/*
**
*/

ipTriggerDispatch( )
{
    que_t * q = ipTriggers;
    int any = 0;

    if (q) {
	do {
	    que_t * qn = q->q_forw;
	    MemVal m[1];
	    ipTrigger_t * it = (ipTrigger_t *)q->q_data;

	    if ((it->it_flags == 0) /* no flags */
	    ||  ((((it->it_flags & (IT_FLAGS_SRC|IT_FLAGS_DST|IT_FLAGS_HOST)) == 0)
		|| ((it->it_flags & (IT_FLAGS_HOST))
			&& (tEqual( it->it_src, (ipSrc(m), m) ) ||
			    tEqual( it->it_src, (ipDst(m), m) ) ) )
		|| (((it->it_flags & (IT_FLAGS_SRC|IT_FLAGS_DST)) == IT_FLAGS_SRC)
			&& (tEqual( it->it_src, (ipSrc(m), m) )) )
		|| (((it->it_flags & (IT_FLAGS_DST|IT_FLAGS_SRC)) == IT_FLAGS_DST)
			&& (tEqual( it->it_dst, (ipDst(m), m) )) )
		|| (((it->it_flags & (IT_FLAGS_DST|IT_FLAGS_SRC)) == (IT_FLAGS_DST|IT_FLAGS_SRC))
			&& (tEqual( it->it_dst, (ipDst(m), m) ))
			&& (tEqual( it->it_src, (ipSrc(m), m) ))  ) )

		&& ( ((it->it_flags & IT_FLAGS_PROTO) == 0)  
		   ||((it->it_flags & IT_FLAGS_PROTO) &&
			(tEqual( it->it_proto, (ipProto(m), m)) ) ) ) )
	    )  {
		/*
		** perform the trigger.
		*/
		performArgs( it->it_trigger->t_func, it->it_trigger->t_scope,
			memTmpNew( NFR_TYPE_TRIGGER, it->it_trigger ), 0 );
		ipTotalTriggers++;
		any++;
	    }

	    q = qn;
	} while (ipTriggers && (q != ipTriggers));
    }
    /*
    **
    */
    reapTriggers:

    q = ipTriggers;
    if (ipTriggerReap && q) {
	do {
	    que_t * qn = q->q_forw;
	    if (q->q_data == 0) {
		if (q == ipTriggers) {
		    ipTriggers = queForw(q);
		}
		queFree(q);
	    }

	} while (ipTriggers && (q != ipTriggers));
	ipTriggerReap = 0;
    }
}

/*
**
*/
void
reassembleIp()
{
    /*
    ** Find the matching reassembly structure.
    ** walk through all the packets we've got, and see
    ** if we have enough to reconstruct the entire
    ** packet contents.
    */
    struct ip * ip = ipHdr();
    struct in_addr src;
    struct in_addr dst;
    que_t * r;
    ipAssemble_t * ia;

    src.s_addr = inaddr( &ip->ip_src );
    dst.s_addr = inaddr( &ip->ip_dst );

    if ((r = findIpAssemble( &src, &dst, ip->ip_id, ip->ip_p )) == 0) {
	/* make the first one, add the current packet to the queue. */
	ia = ipAssembleNew( &src, &dst, ip->ip_id, ip->ip_p );
	if (queAdd( ipsList, r = queNew(ia) ) == 0) {
	    ipsList = r;
	}
    } else {
	ia = (ipAssemble_t *)r->q_data;
    }
    ia->ia_time = packet_sec();
    /*
    ** Add another packet to `r'. 
    ** sort the list of ipData's by the offset...
    ** so that we reassemble by comparing adjacent packets...
    */
    {
	struct ip * ip = ipHdr();
	int ipLen = ip->ip_hl * 4;
	char * data = &((char *)ip)[ipLen];
	int dLen = max( min( int16( &ip->ip_len) , packet_len)-ipLen, 0 );

	ipData_t * ipd = ipDataNew( ip, ipLen, data, dLen );
	que_t * n = queNew( (void *)ipd );
	que_t * p = ia->ia_packets;

	int off = int16( &(ip->ip_off ) );

	ia->ia_pktCnt++;

	if ( ((off & IP_MF) == 0) && (IP_OFFSET(ip) != 0) ) {
	    ia->ia_flags |= IA_FLAGS_DOREASS;
	}

	if (p = ia->ia_packets) {
	    do {
		ipData_t * ipd = (void *)p->q_data;
		if (IP_OFFSET(ip) < IP_OFFSET(ipd->id_header)) {
		    /* insert at end */
		    queAdd( p->q_back, n );
		    goto semble;
		}
		p = p->q_forw;
	    } while (p != ia->ia_packets);
	    /*
	    ** add at the end.
	    */
	    queAdd( p->q_back, n );
	} else {
	    ia->ia_packets = (void *)n;
	}
    }

    semble:

    if (ia->ia_flags & IA_FLAGS_DOREASS) {
	/*
	** We have *every* packet associated ... even duplicates...
	*/
	int overlap = 0;
	int duplicate = 0;
	int dup_protocols = 0; 
	int missing_data = 0;

	int protocol = -1;
	int maxLen = 0;

	/*
	** find how much data we've got in the packet.
	*/
	que_t * p = ia->ia_packets;
	if (p) {
	    do {
		ipData_t * ipd = (ipData_t *)p->q_data;
		int ipdOffset = IP_OFFSET(ipd->id_header)*8;
		maxLen = max( maxLen, ipd->id_dLen+ipdOffset );
		p = p->q_forw;
	    } while (p != ia->ia_packets);
	    /*
	    ** Get memory for the data packet we're going to construct,
	    ** then move the components into the data packet.
	    */
	    {
		struct ip * ip = ipHdr();
		char * end = ipEnd();
		int hOff = ipEnd()-packet_data;
		char * data = (char *)malloc( maxLen+hOff );

		/*
		** Note: the current ip header may NOT be representative
		** of all the options associated with this fragmented
		** packet.
		*/
		memcpy( data, packet_data, hOff );

		p = ia->ia_packets;
		do {
		    ipData_t * ipd = (ipData_t *)p->q_data;
		    int ipdOffset = IP_OFFSET(ipd->id_header)*8;
		    /*
		    ** review the different protocols in each packet.
		    ** make sure they're all the same.
		    */
		    if (protocol < 0) {
			/* since its a octet, we can just reference it */
			protocol = ipd->id_header->ip_p;
		    } else {
			if (protocol != ipd->id_header->ip_p) {
			    dup_protocols++;
			}
		    }
		    /*
		    ** XXX note: bad packets could have bad length's
		    */
		    memcpy( &data[hOff+ipdOffset], ipd->id_data, ipd->id_dLen );
		    /*
		    ** Check for duplicates, or for overlapping ip fragments.
		    ** Since the list is sorted, we can compare adjacent packets.
		    */
		    {
			que_t * b = p->q_back;
			ipData_t * ipdb = (ipData_t *)b->q_data;
			int ipdbOffset = IP_OFFSET(ipdb->id_header) * 8;

			if (b != p) {
			    if ((ipdbOffset+ipdb->id_dLen) > ipdOffset) {
				if (p != ia->ia_packets) {
				    overlap++;
				}
			    }
			    if ((ipdbOffset == ipdOffset)
			     && (ipdb->id_dLen == ipd->id_dLen)) {
				if (p != ia->ia_packets) {
				    duplicate++;
				}
			    }
			    if ((ipdbOffset+ipdb->id_dLen) < ipdOffset) {
				if (p != ia->ia_packets) {
				    missing_data++;
				}
			    }
			}
		    }
		    p = p->q_forw;
		} while (p != ia->ia_packets);

		if (missing_data == 0) {
		    /* we have constructed a complete packet.  */

		    ipReassembled++;
		    /*
		    ** see if there's any higher layer to accept it...
		    */
		    switch (protocol) {
			case IPPROTO_TCP:
			    /*
			    ** Pretend there's a synthetic packet 
			    ** which we received, which is the 
			    ** catenation of the ip fragments.
			    */
			    packet_data = data;
			    packet_len = maxLen+hOff;
			    packet_nonce++;
			    /*
			    ** now mod the current packet.
			    */
			    packet_hdr->len = packet_len;
			    {
				struct ip * i = (struct ip *)ethTail();
				i->ip_len =
					htons( packet_len - hOff + (ip->ip_hl * 4) );
			    }
			    reassembleTcp();
			    ipReassembledTcp++;
			    break;

			case IPPROTO_UDP:
			    /*
			    ** as in the tcp case.
			    */
			    packet_data = data;
			    packet_len = maxLen+hOff;
			    packet_nonce++;
			    packet_hdr->len = packet_len;
			    {
				struct ip * i = (struct ip *)ethTail();
				i->ip_len =
					htons( packet_len - hOff + (ip->ip_hl * 4) );
			    }
			    udpRecv();	/* in packet.c */
			    break;

			case IPPROTO_ICMP:
			    icmpRecv(); /* in packet.c */
			    break;

			default:
			    break;
		    }

		    /*
		    ** Since we've delivered the data
		    ** time to discard all the packets. XXX
		    ** (really?)
		    */
		    if (r == ipsList) {
			ipsList = queForw(r);
		    }
		    queFree(r);
		    ipAssembleFree(ia);
		}
		/*
		** discard the packet_data we've mumbled up.
		** Also discard the ip packet data we've been assembling...
		*/
		free(data);
		/*
		** at most only one will get reassembled.
		*/
	    }
	} else {
	    /* internal error */

	    /* we got here by noticing IA_FLAGS_DOREASS was set,
	    ** but the queue of packets is NULL, so even tho we've
	    ** set the bit cause we've seen packets, there aren't any
	    ** packets.
	    */
	}

    }
}

/*
** ipAssembleStats
**
*/

ipAssembleStats()
{
    uiCurrPrintf( "ip: ipDataBytes %d\n", ipDataBytes );
    uiCurrPrintf( "ip: ipDataCnt %d\n", ipDataCnt );
    uiCurrPrintf( "ip: ipReassembled %d\n", ipReassembled );
    uiCurrPrintf( "ip: ipAssembleByes %d \n", ipAssembleByes );
    uiCurrPrintf( "ip: ipAssembleCnt %d\n", ipAssembleCnt );
    uiCurrPrintf( "ip: ipReassembledTcp %d\n", ipReassembledTcp );
    uiCurrPrintf( "ip: ipReassembledTcp %d\n", ipReassembledTcp );
    if (ipsList) {
	que_t * a = ipsList;

	do {
	    ipAssemble_t * ia = a->q_data;
	    uiCurrPrintf( "ip: src %s ", inet_ntoa( ia->ia_src ) );
	    uiCurrPrintf( "dst %s ", inet_ntoa( ia->ia_dst ) );
	    uiCurrPrintf( "id %x proto 0x%x flags 0x%x ",
		    ia->ia_id, ia->ia_p, ia->ia_flags );
	    uiCurrPrintf( "\nip: cnt %d packets %x: ",
		    ia->ia_pktCnt, ia->ia_packets );
	    uiCurrPrintf( "\n" );
	    if (ia->ia_packets) {
		que_t * q = ia->ia_packets;
		if (q) {
			    
		    do {
			ipData_t * id = (ipData_t *)q->q_data;
			uiCurrPrintf( "ip: hLen %d offset 0x%x [0x%x,%d] dLen %d\n",
			    id->id_hLen,
			    IP_OFFSET( id->id_header ) ,
			    IP_OFFSET( id->id_header ) * 8 ,
			    IP_OFFSET( id->id_header ) * 8 ,
			    id->id_dLen );
			uiCurrPrintf( "ip: len 0x%x,%d off %x\n",
			    int16( &((id->id_header)->ip_len ) ),
			    int16( &((id->id_header)->ip_len ) ),
			    int16( &((id->id_header)->ip_off ) ) );
			q = q->q_forw;
		    } while (q!=ia->ia_packets);
		}
	    }
	    a = a->q_forw;
	} while (a != ipsList);
    }
}


/*
** reassembleIpGC
**
** Garbage collect ip.
**
** Walk through every ipAssemble structure, and look for old
** assemble data.  Discard stuff which hasn't assembled in a
** bunch of seconds...  
**
** Perhaps we need to send an alert when it doesn't assemble.
*/

#define	IPASS_TIMEOUT	10

reassembleIpGC()
{
    que_t * r;
    restart:
    if (r = ipsList) {
	int now = packet_sec();
	if (now == 0) {
	    now = time(0);
	}

	do {
	    /*
	    ** How old is this one?
	    */
	    ipAssemble_t * ia = (ipAssemble_t *)r->q_data;
	    if (ia->ia_time + IPASS_TIMEOUT < now ) {
		/* garbage collect this one */
		if (r == ipsList) {
		    ipsList = queForw(r);
		    goto restart; /* exit condition for while may not work */
		}
		queFree(r);
		ipAssembleFree(ia);
	    }
	    r = r->q_forw;
	} while (r != ipsList);
    }
}


/*
** -------------------------------------------------------
**
** Tcp reassembly generates a stream of data... which
** we can apply to various filters...
**
** Notice that two streams will be created, one which
** describes the outgoing data, and one which describes
** the incomming data.
**
*/

typedef	struct tcpData {
	u_int	td_seq;			/* sequence number */
	char *	td_data;		/* data of the stream */
	int	td_len;			/* amount of data in the stream. */
	int	td_nonce;		/* copied from the packet */
	time_t	td_sec;			/* data created when? */
	char	td_flags;		/* temporary */
	char	td_client;		/* client or server? */
} tcpData_t;

int	tcpDataBytes = 0;
int	tcpDataCnt = 0;
int	tcpDataDel = 0;

tcpData_t *
tcpDataNew( seq, data, len, flags, client, sec )
int	seq;
char *	data;
int	len;
int	flags;
int	client;
int	sec;
{
    tcpData_t * tcpd = (tcpData_t *)malloc( sizeof(*tcpd) );
    /* assert tcpd ? */

    tcpd->td_seq = seq;
    tcpd->td_len = len;
    tcpd->td_nonce = packet_nonce;
    tcpd->td_flags = flags;
    tcpd->td_client = client;
    tcpd->td_sec = sec;

    if (len) {
	tcpd->td_data = (char *)malloc( len );
	memcpy( tcpd->td_data, data, len );
	tcpDataBytes += len;
    } else {
	tcpd->td_data = 0;
    }

    tcpDataCnt++;
    return(tcpd);
}

tcpDataFree( tcp )
tcpData_t * tcp;
{
    if (tcp) {
	if (tcp->td_data) {
	    free(tcp->td_data);
	    tcp->td_data = 0;
	    tcpDataBytes -= tcp->td_len;
	}
	free(tcp);
	tcpDataCnt--;
	tcpDataDel++;
    }
}

/*
** ----------------------------------------
** tcpPatterns describe the search patterns.
** We currently only support very simple 
** match expressions.
**
** tcpPatter's describe the things we are trying to
** match.  The tcpMatch strcuture describes the
** current patterns in process of being matched.
**
** We use MemVal's borrowed from the trigger_t to perform
** the matches... we use the engines own equal routine
** to perform the test, using MemVal's populated from
** the packet routines.
**
**
*/

typedef struct tcpPattern {
    trigger_t	*	tp_trigger;		/* function is in the trigger */
    char *		tp_pattern;		/* pattern to match */
    char *		tp_patEnd;		/* end pattern. */
    MemVal *		tp_src;			/* match for ip source address */
    MemVal *		tp_dst;			/* match for ip dest address */
    MemVal *		tp_dport;		/* zero is any port */
    MemVal *		tp_sport;		/* zero is any port */
    short		tp_saveLen;		/* how much to save in a buffer */
    short		tp_refCnt;		/* # of matches pointing to pattern */
    int			tp_flags;
} tcpPattern_t;

#define	TP_FLAGS_REFFREE	1	/* when refCnt == 0, then free this */

#define	TP_FLAGS_CLIENT		2	/* client direction match */
#define	TP_FLAGS_SERVER		4	/* client direction match */

#define	TP_FLAGS_STARTSTOP	0x10	/* start/stop patterns */
#define	TP_FLAGS_MATCH		0x20	/* match pattern */

#define	TP_FLAGS_PORT		0x40	/* only one port mentioned */
#define	TP_FLAGS_HOST		0x80	/* only one host/net mentioned */

#define	TP_FLAGS_ADDREITHER	0x100	/* src/dst in src/dst (both ways) */
#define	TP_FLAGS_PORTEITHER	0x200	/* sport/dport in src/dst */

#define	TP_FLAGS_NOTICESESSION	0x400	/* matched when a new session is found */
#define	TP_FLAGS_DISCARDSESSION	0x800	/* matched when a new session is found */

#define	TP_FLAGS_FREESRC	0x1000	/* free src MemVal when done */
#define	TP_FLAGS_FREEDST	0x2000	/* free src MemVal when done */
#define	TP_FLAGS_FREESPORT	0x4000	/* free src MemVal when done */
#define	TP_FLAGS_FREEDPORT	0x8000	/* free src MemVal when done */

#define	TP_FLAGS_OPENSESSION	0x10000	/* event -- */
#define	TP_FLAGS_CLOSESESSION	0x20000	/* event -- */

/*
** i don't have a good name for these triggers.  but they're the
** ones associated with meta events... not every packet, but events
** based on the stream, like NOTICE, DISCARD, OPEN and CLOSE
*/
#define	TP_FLAGS_META	(TP_FLAGS_NOTICESESSION|TP_FLAGS_DISCARDSESSION|\
			TP_FLAGS_OPENSESSION|TP_FLAGS_CLOSESESSION)


que_t * tcpPatterns;			/* full ipSrc/ipDst/sport/dport matches */
que_t * tcpOpenClose;			/* session creation-deletion */

int tcpPatternBytes = 0;
int tcpPatternCnt = 0;

void tcpPatternTest( tcpPattern_t *,void *, char *,int, int forw, int flags );

/*
** we don't fill in barely any of the values in the pattern
** structure... there's just too much of a chance in getting
** them wrong.
**
*/
tcpPattern_t *
tcpPatternNew( pattern )
char * pattern;
{
    tcpPattern_t * p = (tcpPattern_t *)calloc( sizeof(*p), 1 );
    /* assert p? */

    p->tp_pattern = pattern;

    tcpPatternCnt++;
    return(p);
}

void
tcpPatternFree(t)
tcpPattern_t * t;
{
    if (t) {
	if (t->tp_refCnt == 0) {
	    tcpPatternCnt--;
	    if ((t->tp_flags & TP_FLAGS_FREESRC) && (t->tp_src)) {
		free(t->tp_src);
		t->tp_src = 0;
	    }
	    if ((t->tp_flags & TP_FLAGS_FREEDST) && (t->tp_dst)) {
		free(t->tp_dst);
		t->tp_dst = 0;
	    }
	    if ((t->tp_flags & TP_FLAGS_FREESPORT) && (t->tp_sport)) {
		free(t->tp_sport);
		t->tp_sport = 0;
	    }
	    if ((t->tp_flags & TP_FLAGS_FREEDPORT) && (t->tp_dport)) {
		free(t->tp_dport);
		t->tp_dport = 0;
	    }
	    free(t);
	} else {
	    t->tp_flags |= TP_FLAGS_REFFREE;
	}
    }
}

que_t *
tcpPatternAdd(t)
tcpPattern_t * t;
{
    que_t * q = queNew(t);
    if (queAdd( tcpPatterns, q ) == 0) {
	tcpPatterns = q;
    }
    return(q);
}

void *
tcpPatternDel(q)
que_t * q;
{
    if (q->q_data) {
	tcpPatternFree( (void *)q->q_data );
    }

    if (tcpPatterns = q) {
	tcpPatterns = queForw(q);
    }
    queFree(q);
}

void
tcpPatternIncr(t)
tcpPattern_t * t;
{
    t->tp_refCnt++;
}

void
tcpPatternDecr(t)
tcpPattern_t * t;
{
    t->tp_refCnt--;
    if (t->tp_flags & TP_FLAGS_REFFREE) {
	/*
	** Let the free routine test the reference count.
	*/
	tcpPatternFree(t);
    }
}

/*
** tcpPatternOff
** If we find this thing, then we nix it...  If we don't
** find it, we return 0.... XXX More Work! XXX
** what about triggers?
**
*/

int
tcpPatternOff(t)
tcpPattern_t * t;
{
    que_t * q = tcpPatterns;
    if (q) {
	do {
	    tcpPattern_t * p = (tcpPattern_t *)q->q_data;
	    if (t == p) {
		/*
		** Off this theng.
		*/
		if (q == tcpPatterns) {
		    tcpPatterns = queForw(q);
		}
		queFree(q);
		tcpPatternFree(p);
		return(1);
	    }
	    q = q->q_forw;
	} while (tcpPatterns && (q != tcpPatterns));
    }

    /*
    ** We have patterns on multiple lists.
    */
    q = tcpOpenClose;
    if (q) {
	do {
	    tcpPattern_t * p = (tcpPattern_t *)q->q_data;
	    if (t == p) {
		/*
		** Off this theng.
		*/
		if (q == tcpOpenClose) {
		    tcpOpenClose = queForw(q);
		}
		queFree(q);
		tcpPatternFree(p);
		return(1);
	    }
	    q = q->q_forw;
	} while (tcpOpenClose && (q != tcpOpenClose));
    }

    return(0);
}

/*
** Iterate through every pattern which matches ... calling 
** a function for each pattern which we ought to consider...
**
** Matching is done against the current packet...
**
** When we match, we'll test the first character we got
** against each new character we're adding to the stream.
** Each time we match, we'll create one matching context...
** (if one doesn't exist)
**
** forw... 1 if we're testing the forward direction,
** 0, if we're testing the backward direction.
** We also have two other flags which we turn on in
** the FORW parameter... CLIENT/SERVER.  These are
** clues about the data stream ...
**
** flags ... passed to the callout... 
**
** In the case of tcpPatternTest, the flags are used to describe
** what gets done when the pattern matches... currently there are
** two choices -- either perorm the indicated action, or remove
** the match (the match is removed when we find the end: match)
**
*/

#define	SERVER	4
#define	CLIENT	2
#define	FORW	1
#define	BACK	0


tcpPattern_t *
tcpPatternsCall( callout, rock, data, len, forw, flags )
void (*callout)();
tcpAssemble_t * rock;
char * data;
int len;
int forw;
int flags;
{
    struct ip * ip = ipHdr();
    struct tcphdr * tcp = tcpHdr();
    que_t * q = tcpPatterns;

    if (q) {
	do {
	    tcpPattern_t * p = (void *)q->q_data;
	    MemVal m[1];

	    /*
	    * Here theer be band-aids.  m is a MemVal used as a temp
	    * variable in these comparisons.  Any of tcpConn*() require
	    * tcpConnTa to be set correctly, or they can't fill in
	    * values in m.  When you then use m, it crashes on bogus
	    * data.
	    * My band-aid is to set tcpConnTa just before calling
	    * tcpConn*().  Normally you do this through tcpStreamSet()
	    * but I don't really follow all the implications of that,
	    * so I won't try.  Instead, I will preserve tcpConnTa, set it
	    * and restore it after I've finished.
	    *
	    * b.t.w. These 2 if statements are ripe for replacement with
	    * something simpler.
	    *
	    *  - Mark S.
	    */

	    tcpAssemble_t * save_tcpConnTa ;
	    int restore_needed = 1;

	    save_tcpConnTa = tcpConnTa;
	    tcpConnTa = rock;

	    if (((p->tp_flags & TP_FLAGS_HOST) /* either src/dst */
		 && ( tEqual( p->tp_src, (tcpConnSrc(m), m) )
		    ||tEqual( p->tp_src, (tcpConnDst(m), m) )))
						/* exactly src/dst */
	     || ( ((p->tp_src == 0) || (tEqual( p->tp_src, (tcpConnSrc(m), m) )))
		&&((p->tp_dst == 0) || (tEqual( p->tp_dst, (tcpConnDst(m), m) ))))
	     || ((p->tp_flags & TP_FLAGS_ADDREITHER) /* src as src/dst, dst as src/dst */
		&&( (  tEqual( p->tp_src, (tcpConnSrc(m),m))
		    || tEqual( p->tp_src, (tcpConnDst(m),m)) ) )
		&&( (  tEqual( p->tp_dst, (tcpConnSrc(m),m))
		    || tEqual( p->tp_dst, (tcpConnDst(m),m)) ) ) ) )   {

		    /* host match... now try port */
		    if (((p->tp_flags & TP_FLAGS_PORT) /* either src/dst port */
			 && ( tEqual( p->tp_sport, (tcpConnSport(m), m) )
			    ||tEqual( p->tp_sport, (tcpConnDport(m), m) )))
		     || ( ((p->tp_sport == 0) || (tEqual( p->tp_sport, (tcpConnSport(m), m))))
			&&((p->tp_dport == 0) || (tEqual( p->tp_dport, (tcpConnDport(m), m)))))
		     || ( (p->tp_flags & TP_FLAGS_PORTEITHER)
			&&(  tEqual( p->tp_sport, (tcpConnSport(m), m))
			   ||tEqual( p->tp_sport, (tcpConnDport(m), m)) )
			&&(  tEqual( p->tp_dport, (tcpConnSport(m), m))
			   ||tEqual( p->tp_dport, (tcpConnDport(m), m)) ) ) ) {
			if (restore_needed)
				{ tcpConnTa=save_tcpConnTa; restore_needed=0; }
				

			if (vFlag) {
			    printf( "Call: %x forw: %x client %x server %x\n",
				p, forw, (p->tp_flags & TP_FLAGS_CLIENT), (p->tp_flags & TP_FLAGS_SERVER) );
			}
			/* port matched.  now test to see if its the right stream. */
			if (p->tp_flags & TP_FLAGS_CLIENT) {
			    if (forw & CLIENT)  {
				(*callout)( p, rock, data, len, FORW & forw, flags );
			    }
			} else
			if (p->tp_flags & TP_FLAGS_SERVER) {
			    if (forw & SERVER) {
				(*callout)( p, rock, data, len, FORW & forw, flags );
			    }
			} else {
			    (*callout)( p, rock, data, len, FORW & forw, flags );
			}
		    }
	    if (restore_needed)
		{ tcpConnTa=save_tcpConnTa; restore_needed=0; }

	    }
	    q = q->q_forw;
	} while (q != tcpPatterns);
    }
}

/*
** 
** tcpPatternStat.
**
** Display status about the patterns we're matching.
**
*/
tcpPatternStat( l )
que_t * l;
{
    que_t * q = l;
    if (l) {
	do {
	    tcpPattern_t * tp = (tcpPattern_t *)q->q_data;
	    if (tp) {
		uiCurrPrintf( "tp: %x trigger %x pattern %s patEnd %s\n",
		    tp, tp->tp_trigger,
		    util_str(tp->tp_pattern), util_str(tp->tp_patEnd) );
		if (tp->tp_trigger) {
		    triggerStat(tp->tp_trigger);
		}
		uiCurrPrintf( "tp: src %x <", tp->tp_src );
		if (tp->tp_src) {
		    blt_echo( tp->tp_src );
		}
		uiCurrPrintf( "> dst %x <", tp->tp_dst );
		if (tp->tp_dst) {
		    blt_echo( tp->tp_dst );
		}
		uiCurrPrintf( "> sport %x <", tp->tp_sport );
		if (tp->tp_sport) {
		    blt_echo( tp->tp_sport );
		}
		uiCurrPrintf( ">: dport %x <", tp->tp_dport );
		if (tp->tp_dport) {
		    blt_echo( tp->tp_dport );
		}
		uiCurrPrintf( ">\ntp: refCnt %d flags %d\n",
			tp->tp_refCnt, tp->tp_flags );
	    }
	    q = q->q_forw;
	} while (q != l);
    }
}

/*
** tcpPatternStats()
*/

tcpPatternStats()
{
    tcpPatternStat(tcpOpenClose);
    tcpPatternStat(tcpPatterns);
}


/*
** ----------------------------------------
** tcpMatch.
** When we discover a new end-end connection,
** begin running this match.
**
*/

typedef struct tcpMatch {
	tcpPattern_t *	tm_tp;		/* reference to pattern we're woring on */
	int		tm_seq;		/* location in stream last matched */
	char *		tm_pattern;	/* pattern we're matching */
	int		tm_flags;	/* Important stuff. */
	int		tm_refCnt;	/* reference count */
	char *		tm_save;	/* characters we're saving */
	short		tm_saveLen;	/* max len of tm_save */
	short		tm_saveNdx;	/* current index inside tm_save */
	que_t *    	tm_free;        /* other match besides this one to free */
} tcpMatch_t;

#define	TM_FLAGS_STARTSTOP	0x1	/* make the data available to the action. */
#define	TM_FLAGS_MATCH		0x2	/* we're matching. */

#define	TM_FLAGS_START		0x10	/* matched start pattern */
#define	TM_FLAGS_STOP		0x20	/* matched stop pattern */
#define TM_FLAGS_UNANCHORED	0x40	/* we haven't started matching the pattern */


int tcpMatchCnt = 0;

tcpMatch_t *
tcpMatchNew( seq, pattern, tp, flags )
int	seq;
char * pattern;
tcpPattern_t * tp;
int	flags;
{
    tcpMatch_t * m = (tcpMatch_t *)malloc( sizeof(*m) );
    if (vFlag) {
	printf( "tcpMatchNew: match <%s>\n", pattern );
    }
    /* assert m? */
    m->tm_tp = tp;
    if (tp) {
	tcpPatternIncr(tp);
    }
    m->tm_seq = seq;
    m->tm_pattern = pattern;
    m->tm_flags = flags;
    /*
    ** propogate some flags from the pattern to the match...
    */
    if (tp) {
	if (tp->tp_flags & TP_FLAGS_MATCH) {
	    m->tm_flags |= TM_FLAGS_MATCH;
	}
	if (tp->tp_flags & TP_FLAGS_STARTSTOP) {
	    m->tm_flags |= TM_FLAGS_STARTSTOP;
	}
    }

    /*
    ** if the pattern says to save characters, record how much
    ** to save...
    */
    m->tm_saveLen = tp->tp_saveLen;
    m->tm_saveNdx = 0;
    m->tm_save = 0;
    if (m->tm_saveLen) {
	m->tm_save = (char *)calloc( 1, tp->tp_saveLen );
    }
    m->tm_free = 0;
    tcpMatchCnt++;
    return(m);
}

void
tcpMatchIncRef( m )
tcpMatch_t * m;
{
    if (m) {
	m->tm_refCnt++;
    }
}

void
tcpMatchFree( f )
tcpMatch_t * f;
{
    if (f) {
	/* patterns aren't our property to free... we just reference them */
	tcpMatchCnt--;
	if ((f->tm_free) && (f->tm_refCnt-- <=0)) {
	    tcpMatch_t * m = (tcpMatch_t *)(f->tm_free->q_data);
	    tcpMatchFree(m);
	    queFree(f->tm_free);
	    f->tm_free = 0;
	}
	if (f->tm_save) {
	    free(f->tm_save);
	    f->tm_save = 0;
	}
	if (f->tm_tp) {
	    tcpPatternDecr(f->tm_tp);
	}
	free(f);
    }
}

/*
** tcpMatchSave.
**
** Save data from some stream into the match structure...
*/
void
tcpMatchSave( m, data, len )
tcpMatch_t * m;
char * data;
int len;
{
    if (vFlag) {
	log( FL, "tcpMatchSave: m %x data %x len %x\n", m, data, len );
    }
    if (m->tm_save) {
	/*
	** pour the data...
	*/
	int mlen = len;
	int more = 0;

	if (len + m->tm_saveNdx > m->tm_saveLen) {
	    mlen = m->tm_saveLen - m->tm_saveNdx;
	    more = len - mlen; /* should always be > 0 */
	}
	/* if there's space in the buffer... */
	if (mlen > 0) {
	    memcpy( &m->tm_save[m->tm_saveNdx], data, mlen );
	    m->tm_saveNdx += mlen;
	}
	/* see if we're supposed to truncate at the front or rear */
	if (m->tm_saveNdx >= m->tm_saveLen) {
	}
	/* see if we're supposed to move data into the front of the buffer */
    }
}

/*
**
** tcpTriggerOff.
** A cover to get to tcpPatternOff.
*/

void
tcpTriggerOff( t )
trigger_t * t;
{
    if (t->t_rock) {
	tcpPatternOff( t->t_rock );
	t->t_rock = 0;
    }
}

/*
** tcpTriggerSet.
** Review all the name value pairs to see what's requested.
**
** All the trigger tags:
** port: match against the port.
** sport: match against the source port.
** dport: match against the source port.
** src: match against the source address.
** dest: match against the dest address.
** addr: match against the address ...
**
** match: string match against the stream.
** start: string match against the stream, trigger is called
**	for all data in the stream starting at start.
** end: string match against the stream, trigger is called
**	between data matching start, and ending the end pattern.
**	it ought to be legal to only match the end token.
**
** tags:
** client: when matching, compare against the requestor stream.
**	the requestor path is the one performing the SYN.
** server: when matching, compare against the server stream.
**	(which is the destination port address)
**
*/

	/*
	** describe the types we'll allow for the MemVal * for the
	** match'es for ip addresses.
	*/
#define	IS_ADDR_TYPES(x) (((x) == NFR_TYPE_LIST) || \
	((x) == NFR_TYPE_IPV4HOST) || ((x) == NFR_TYPE_IPV4NET))

	/*
	** describe the types we'll allow for the MemVal * for
	** the matches for port values.
	*/
#define IS_PORT_TYPE(x) (((x) == NFR_TYPE_LIST) || ((x) == NFR_TYPE_INT))

trigger_t *
tcpTriggerSet( t )
trigger_t * t;
{
    int i;
    
    MemVal * src = 0;
    MemVal * dst = 0;
    int	eitherAddr = 0;
    
    MemVal * sport = 0;
    MemVal * dport = 0;
    int eitherPort = 0;

    int client = 0;
    int server = 0;

    int saveLength = 0; /* by default, we don't save any characters from the session */

    char * start = 0;
    char * stop = 0;
    int	match = 0;

    int flags = 0;

    int err = 0;
    char * errStr = 0;

    for (i = 0; i<t->t_ndx; ++i) {
	if (strcmp( t->t_whom[i], "tcp" ) == 0) {
	    Sym * l = t->t_nv[i];
	    for( l = t->t_nv[i]; l ; l = l->s_next ) {
		MemVal * m = &l->s_data;
		if (symCmp( l, "currentsession" ) == 0) {
		    if (t->t_flags & T_FLAGS_FUNC) {
			/* sorry, can' use currentsession in the filter clause */
		    } else
		    if (packet_data == 0) {
			/* sorry, doesn't seem to be a current session */
		    } else {
			MemVal m[1];
			src = memTmpNew(0,0);
			memvalclonedata( src, (ipSrc(m),m) );
			dst = memTmpNew(0,0);
			memvalclonedata( dst, (ipDst(m),m) );
			sport = memTmpNew(0,0);
			memvalclonedata( sport, (tcpSport(m),m) );
			dport = memTmpNew(0,0);
			memvalclonedata( dport, (tcpDport(m),m) );
			flags |= (TP_FLAGS_FREESRC|TP_FLAGS_FREEDST|
				TP_FLAGS_FREESPORT|TP_FLAGS_FREEDPORT|
				TP_FLAGS_ADDREITHER|TP_FLAGS_PORTEITHER);
		    }
		} else
		if (symCmp( l, "src" ) == 0) {
		    /* value is src net. */
		    if (src) {
			uiCurrPrintf( "tcpTriggerSet: %s\n",
				errStr = "src already set" );
			err = 1;
		    } else
		    if ( !IS_ADDR_TYPES(m->typ)) {
			uiCurrPrintf( "tcpTriggerSet: %s: %s\n",
				errStr = "bad src type", util_type2str( m->typ ) );
			err = 1;
		    } else {
			src = m;
		    }
		} else
		if (symCmp( l, "dst" ) == 0) {
		    /* value is dest net. */
		    if (dst) {
			uiCurrPrintf( "tcpTriggerSet: %s\n",
				errStr = "dst already set" );
			err = 1;
		    } else
		    if ( !IS_ADDR_TYPES(m->typ)) {
			uiCurrPrintf( "tcpTriggerSet: %s: %s\n",
				errStr = "bad dst type", util_type2str( m->typ ) );
			err = 1;
		    } else {
			dst = m;
		    }
		} else
		if (symCmp( l, "net" ) == 0) {
		    /* value is a net address */
		    if (src) {
			if (dst) {
			    uiCurrPrintf( "tcpTriggerSet: %s\n",
				    errStr = "src already set" );
			    err = 1;
			} else {
			    /*
			    ** two nets described.  Turn off eitherAddr.
			    */
			    eitherAddr = 0;
			    dst = m;
			    flags |= TP_FLAGS_ADDREITHER;
			}
		    } else
		    if ( !IS_ADDR_TYPES(m->typ)) {
			uiCurrPrintf( "tcpTriggerSet: %s: %s\n",
				errStr = "bad net type", util_type2str( m->typ ) );
			err = 1;
		    } else {
			src = m;
			eitherAddr = 1;
		    }
		} else
		if (symCmp( l, "host" ) == 0) {
		    /* value is a host (net with exact match) address */
		    if (src) {
			if (dst) {
			    uiCurrPrintf( "tcpTriggerSet: %s\n",
				    errStr = "src already set" );
			    err = 1;
			} else {
			    /*
			    ** two hosts described, turn off eitherAddr,
			    ** turn on either src/dst.
			    */
			    dst = m;
			    eitherAddr = 0;
			    flags |= TP_FLAGS_ADDREITHER;
			}
		    } else 
		    if ( !IS_ADDR_TYPES(m->typ)) {
			uiCurrPrintf( "tcpTriggerSet: %s: %s\n",
				errStr = "bad host type", util_type2str( m->typ ) );
			err = 1;
		    } else {
			src = m;
			eitherAddr = 1;
		    }
		} else
		if (symCmp( l, "sport" ) == 0) {
		    /* source port */
		    if (sport) {
			uiCurrPrintf( "tcpTriggerSet: %s\n",
				    errStr = "sport already set" );
			err = 1;
		    } else 
		    if ( !IS_PORT_TYPE(m->typ)) {
			uiCurrPrintf( "tcpTriggerSet: %s\n",
				    errStr = "bad sport type" );
			err = 1;
		    } else {
			sport = m;
		    }
		} else
		if (symCmp( l, "dport" ) == 0) {
		    /* destination port */
		    if (sport) {
			uiCurrPrintf( "tcpTriggerSet: %s\n",
				    errStr = "dport already set" );
			err = 1;
		    } else
		    if ( !IS_PORT_TYPE(m->typ)) {
			uiCurrPrintf( "tcpTriggerSet: %s\n",
				    errStr = "bad dport type" );
			err = 1;
		    } else {
			dport = m;
		    }
		} else
		if (symCmp( l, "port" ) == 0) {
		    /* notice port */
		    if (sport) {
			if (dport) {
			    uiCurrPrintf( "tcpTriggerSet: %s\n",
					errStr = "sport/dport already set" );
			    err = 1;
			} else {
			    dport = m;
			    eitherPort = 0;
			    flags |= TP_FLAGS_PORTEITHER;
			}
		    } else
		    if ( !IS_PORT_TYPE(m->typ)) {
			uiCurrPrintf( "tcpTriggerSet: %s\n",
				    errStr = "bad port type" );
			err = 1;
		    } else {
			sport = m;
			eitherPort = 1;
		    }
		} else
		if (symCmp( l, "client" ) == 0) {
		    /* identify client stream */
		    flags |= TP_FLAGS_CLIENT;
		    client = 1;
		} else
		if (symCmp( l, "server" ) == 0) {
		    /* identify server stream */
		    flags |= TP_FLAGS_SERVER;
		    server = 1;
		} else
		if (symCmp( l, "match" ) == 0) {
		    /* only single match, only allow strings for matches. */
		    if (start) {
			uiCurrPrintf( "tcpTriggerSet: %s\n",
				    errStr = "start already set" );
			err = 1;
		    } else
		    if (match) {
			uiCurrPrintf( "tcpTriggerSet: %s\n",
				    errStr = "match already set" );
			err = 1;
		    } else
		    if (m->typ != NFR_TYPE_STR) {
			uiCurrPrintf( "tcpTriggerSet: %s\n",
				    errStr = "bad match type" );
			err = 1;
		    } else {
			start = m->val.c;
			flags |= TP_FLAGS_MATCH;
			match = 1;
		    }
		} else
		if (symCmp( l, "start" ) == 0) {
		    /* start pattern match */
		    if (start) {
			uiCurrPrintf( "tcpTriggerSet: %s\n",
				    errStr = "start already set" );
			err = 1;
		    } else
		    if (m->typ != NFR_TYPE_STR) {
			uiCurrPrintf( "tcpTriggerSet: %s\n",
				    errStr = "bad start type" );
			err = 1;
		    } else {
			start = m->val.c;
			flags |= TP_FLAGS_STARTSTOP;
		    }
		} else
		if (symCmp( l, "stop" ) == 0) {
		    /* stop pattern match */
		    if (stop) {
			uiCurrPrintf( "tcpTriggerSet: %s\n",
				    errStr = "stop already set" );
			err = 1;
		    } else
		    if (m->typ != NFR_TYPE_STR) {
			uiCurrPrintf( "tcpTriggerSet: %s\n",
				    errStr = "bad stop type" );
			err = 1;
		    } else {
			stop = m->val.c;
			flags |= TP_FLAGS_STARTSTOP;
		    }
		} else
		if (symCmp( l, "noticesession" )== 0) {
		    flags |= TP_FLAGS_NOTICESESSION;
		} else
		if (symCmp( l, "opensession" )== 0) {
		    flags |= TP_FLAGS_OPENSESSION;
		} else
		if (symCmp( l, "discardsession" ) == 0) {
		    flags |= TP_FLAGS_DISCARDSESSION;
		} else
		if (symCmp( l, "save_length" ) == 0) {
		    if (m->typ != NFR_TYPE_INT) {
			/* we currently only allow sizes to be ints */
			uiCurrPrintf( "tcpTriggerSet: %s: %s\n",
				    errStr = "bad type for save_length",
				    util_type2str( m->typ ) );
		    } else {
			saveLength = m->val.i;
		    }
		} else {
		    /* don't know what this is.  won't build the pattern. */
		    uiCurrPrintf( "tcpTriggerSet: %s: %s\n",
				    errStr = "bad keyword", symName(l) );
		    err = 1;
		}

	    }
	}
    }

    /*
    ** First, try for some conflicting choices.
    */
    if (client && server) {
	/* sorry, not both client and server */
	uiCurrPrintf( "trigger: tcp: can't include both client and server\n" );
	err = 1;
    }
    if (err) {
	Alert_Args_Generate( NFR_SRC_NFRD, NFR_NFRD_TRIGGERERROR,
		    "tcp", errStr ? errStr : "" , 0 );
	return(0);
    }

    /*
    ** Now see if we have enough stuff ...
    ** What's the mininum?  likely tcp ( )? All the data, all the time?
    */
    {
	tcpPattern_t * tp = tcpPatternNew(0);

	/* record how to free this one */
	t->t_free = tcpTriggerOff;
	t->t_rock = (void *)tp;

	tp->tp_flags |= flags; /* if malloc'ind the MemVals */

	tp->tp_pattern = start;
	tp->tp_patEnd = stop;

	tp->tp_src = src;
	tp->tp_dst = dst;
	if (eitherAddr) {
	    tp->tp_flags |= TP_FLAGS_HOST;
	}

	tp->tp_sport = sport;
	tp->tp_dport = dport;
	if (eitherPort) {
	    tp->tp_flags |= TP_FLAGS_PORT;
	}
	tp->tp_saveLen = saveLength;

	/*
	** Add this thing to the correct list of patterns... 
	*/
	if (flags & TP_FLAGS_META) {
	    que_t * q = queNew(tp);
	    if (queAdd( tcpOpenClose, q ) == 0) {
		tcpOpenClose = q;
	    }
	    tp->tp_trigger = t;
	    return( t );
	} else {
	    que_t * q = tcpPatternAdd(tp);
	    tp->tp_trigger = t;
	    return( t );
	}
    }
}

/* use FORW/BACK as the two indicies into the structure.
** We can also pass around FORW/BACK (usually as the parameter forw)
** to use as an index into ta_stream.  Note that this makes
** some routines very *very* common, tho possibly a bit more
** difficult to understand.  More comments will follow over time...
*/

#define ta_forwOrigSeq	ta_stream[FORW].ta_OrigSeq
#define	ta_forwPackets	ta_stream[FORW].ta_Packets
#define	ta_forwPackNo	ta_stream[FORW].ta_PackNo
#define	ta_forwArriv	ta_stream[FORW].ta_Arriv
#define	ta_forwArrDiff	ta_stream[FORW].ta_ArrDiff
#define	ta_forwSumSq	ta_stream[FORW].ta_ArrSumSq
#define	ta_forwArrStdDv	ta_stream[FORW].ta_ArrStdDv
#define	ta_forwSeqNo	ta_stream[FORW].ta_SeqNo
#define	ta_forwBytes	ta_stream[FORW].ta_Bytes
#define	ta_forwLost	ta_stream[FORW].ta_Lost
#define	ta_forwCnt	ta_stream[FORW].ta_Cnt
#define	ta_forwPast	ta_stream[FORW].ta_Past
#define	ta_forwPastI	ta_stream[FORW].ta_PastI
#define	ta_forwMatches	ta_stream[FORW].ta_Matches
#define	ta_forwInfo	ta_stream[FORW].ta_info

#define ta_backOrigSeq	ta_stream[BACK].ta_OrigSeq
#define	ta_backPackets	ta_stream[BACK].ta_Packets
#define	ta_backPackNo	ta_stream[BACK].ta_PackNo
#define	ta_backArriv	ta_stream[BACK].ta_Arriv
#define	ta_backArrDiff	ta_stream[BACK].ta_ArrDiff
#define	ta_backSumSq	ta_stream[BACK].ta_ArrSumSq
#define	ta_backArrStdDv	ta_stream[BACK].ta_ArrStdDv
#define	ta_backSeqNo	ta_stream[BACK].ta_SeqNo
#define	ta_backBytes	ta_stream[BACK].ta_Bytes
#define	ta_backLost	ta_stream[BACK].ta_Lost
#define	ta_backCnt	ta_stream[BACK].ta_Cnt
#define	ta_backPast	ta_stream[BACK].ta_Past
#define	ta_backPastI	ta_stream[BACK].ta_PastI
#define	ta_backMatches	ta_stream[BACK].ta_Matches
#define	ta_backInfo	ta_stream[BACK].ta_info

/* char * direction[2] = { "back", "forw" }; */
char * direction[2] = { "fromDst", "fromSrc" };


tcpPast( ta, seq, len, forw )
tcpAssemble_t * ta;
{
    ta->ta_stream[forw].ta_PastI++;
    if (ta->ta_stream[forw].ta_PastI == (PASTLEN)) {
	ta->ta_stream[forw].ta_PastI = 0;
    }
    ta->ta_stream[forw].ta_Past[ ta->ta_stream[forw].ta_PastI ].tp_seq = seq;
    ta->ta_stream[forw].ta_Past[ ta->ta_stream[forw].ta_PastI ].tp_len = len;
}

#define tcpForwPast( ta, seq, len) (tcpPast( ta, seq, len, FORW ))
#define tcpBackPast( ta, seq, len) (tcpPast( ta, seq, len, BACK ))

/*
** forward declaration...
*/
void tcpStreamSet( tcpAssemble_t * ta, char * d, int l, tcpMatch_t * m );




int tcpAssembleCnt = 0;

/*
** all the data we've got on all the packets.
*/
que_t * tcpsList;

/*
** A few flags.
** We keep the tcp_flags in the low order byte...
*/

#define	TA_FLAGS_ORIG_SRC	0x100	/* originator of session is src. */
#define	TA_FLAGS_ORIG_DST	0x200	/* originator of session is dst. */

#define	TA_FLAGS_FLIP		0x400	/* src/dst flipped from original packet */
#define	TA_FLAGS_RSTACK		0x800	/* saw rst/rst-ack */
#define	TA_FLAGS_FINACK		0x1000	/* saw fin/fin-ack */
#define	TA_FLAGS_SYNACK		0x1000	/* saw syn/syn-ack */


/*
** tcpOpenCloseTest
**
** Call the trigger if we find an open or close event.
** The flags parameter in this case is either
** TP_FLAGS_NOTICESESSION, or TP_FLAGS_DISCARDSESSION,
** when 'and'ed with the trigger flags, will tell us
** whether to perform the trigger.
*/

void
tcpOpenCloseTest( p, rock, data, len, forw, flags )
tcpPattern_t * p;
void * rock;	/* actually tcpAssemble_t * */
char * data;
int len;
int forw;	/* which direction we're testing. */
int flags;	/* flags to associate with any tcpMatch structures we new */
{
    tcpAssemble_t * ta = (tcpAssemble_t *)rock;

    if (flags & p->tp_flags) {
	/*
	** Perform the trigger.
	*/
	if (p->tp_trigger) {
	    trigger_t * t = p->tp_trigger;
	    tcpStreamSet( ta, 0, 0, 0 );
	    performArgs( t->t_func, t->t_scope, memTmpNew( NFR_TYPE_TRIGGER, t ), 0 );
	    tcpStreamSet( ta, 0, 0, 0 );
	}

    }
}

/*
** tcpOpenCloseCall
**
** Very similar to tcpPatternCall, but we look up the
** port/address information in the tcpAssemble, not in the
** current packet... we also traverse a different pattern list...
** tcpOpenClose
*/

tcpPattern_t *
tcpOpenCloseCall( callout, ta, data, len, forw, flags )
void (*callout)();
tcpAssemble_t * ta;
char * data;
int len;
int forw;
int flags;
{
    struct ip * ip = ipHdr();
    struct tcphdr * tcp = tcpHdr();
    que_t * q = tcpOpenClose;
    MemVal src[1];
    MemVal dst[1];
    MemVal sport[1];
    MemVal dport[1];

    /*
    ** Since the source and dest come from the ta*, we need
    ** be sensitive to the possible inversion of src/dst
    ** due to the ip lookup mechanism.
    */

    src->typ = NFR_TYPE_IPV4HOST;
    dst->typ = NFR_TYPE_IPV4HOST;
    sport->typ = NFR_TYPE_INT;
    dport->typ = NFR_TYPE_INT;

    if ((ta->ta_flags & TA_FLAGS_FLIP) == 0) {
	src->val.h = ta->ta_src;
	dst->val.h = ta->ta_dst;
	sport->val.i = ta->ta_sport;
	dport->val.i = ta->ta_dport;
    } else {
	src->val.h = ta->ta_dst;
	dst->val.h = ta->ta_src;
	sport->val.i = ta->ta_dport;
	dport->val.i = ta->ta_sport;
    }

    if (q) {
	do {
	    tcpPattern_t * p = (void *)q->q_data;
	    MemVal m[1];

	    if (((p->tp_flags & TP_FLAGS_HOST) /* either src/dst */
		 && ( tEqual( p->tp_src, src )
		    ||tEqual( p->tp_src, dst )))
						/* exactly src/dst */
	     || ( ((p->tp_src == 0) || (tEqual( p->tp_src, src )))
		&&((p->tp_dst == 0) || (tEqual( p->tp_dst, dst ))))
	     || ((p->tp_flags & TP_FLAGS_ADDREITHER) /* src as src/dst, dst as src/dst */
		&&( (  tEqual( p->tp_src, src )
		    || tEqual( p->tp_src, dst ) ) )
		&&( (  tEqual( p->tp_dst, src )
		    || tEqual( p->tp_dst, dst ) ) ) ) )   {

		    /* host match... now try port */
		    if (((p->tp_flags & TP_FLAGS_PORT) /* either src/dst port */
			 && ( tEqual( p->tp_sport, sport )
			    ||tEqual( p->tp_sport, dport )))
		     || ( ((p->tp_sport == 0) || (tEqual( p->tp_sport, sport)))
			&&((p->tp_dport == 0) || (tEqual( p->tp_dport, dport))))
		     || ( (p->tp_flags & TP_FLAGS_PORTEITHER)
			&&(  tEqual( p->tp_sport, sport)
			   ||tEqual( p->tp_sport, dport) )
			&&(  tEqual( p->tp_dport, sport)
			   ||tEqual( p->tp_dport, dport) ) ) ) {
				
			if (vFlag) {
			    printf( "tcpOpenCloseCall: %x forw: %x client %x server %x\n",
				p, forw, (p->tp_flags & TP_FLAGS_CLIENT), (p->tp_flags & TP_FLAGS_SERVER) );
			}
			/* port matched.  now test to see if its the right stream. */
			if (p->tp_flags & TP_FLAGS_CLIENT) {
			    if (forw & CLIENT)  {
				(*callout)( p, ta, data, len, FORW & forw, flags );
			    }
			} else
			if (p->tp_flags & TP_FLAGS_SERVER) {
			    if (forw & SERVER) {
				(*callout)( p, ta, data, len, FORW & forw, flags );
			    }
			} else {
			    (*callout)( p, ta, data, len, FORW & forw, flags );
			}
		    }

	    }
	    q = q->q_forw;
	} while (q != tcpOpenClose);
    }
}




tcpAssemble_t *
tcpAssembleNew( src, dst, sport, dport, flags )
struct in_addr * src;
struct in_addr * dst;
u_short sport;
u_short dport;
int	flags;
{
    tcpAssemble_t * ta = (tcpAssemble_t *)calloc( sizeof(*ta), 1 );

    ta->ta_src.s_addr = src->s_addr;
    ta->ta_dst.s_addr = dst->s_addr;

    ta->ta_sport = sport;
    ta->ta_dport = dport;

    ta->ta_flags = flags;

    ta->ta_epoch = packet_sec();
    ta->ta_stream[BACK].ta_SeqGc = 0;
    ta->ta_stream[FORW].ta_SeqGc = 0;
    ta->ta_hash = packet_nonce;	/* well, its ok for now, but i'd like a hash */

    /*
    ** look to see if we have an event for this...
    */
    tcpOpenCloseCall( tcpOpenCloseTest, ta, 0, 0, 0, TP_FLAGS_NOTICESESSION );

    tcpAssembleCnt++;
    return(ta);
}

void
tcpAssembleFree( tcpd )
tcpAssemble_t * tcpd;
{
    if (tcpd) {
	/*
	** look to see if we have an event for this...
	*/
	tcpOpenCloseCall( tcpOpenCloseTest, tcpd, 0, 0, 0, TP_FLAGS_DISCARDSESSION );

	/*
	** Free the two queue's of packets describing data...
	*/
	{
	    que_t * q;
	    int forw;
	    for (forw=BACK; forw <= FORW; ++forw) {
		while (q = tcpd->ta_stream[forw].ta_Packets) {
		    tcpDataFree( (tcpData_t *)q->q_data );
		    tcpd->ta_stream[forw].ta_Packets = queForw(q);
		    queFree(q);
		}
	    }
	}
	/*
	** Free two queues of match structures...
	*/
	{
	    que_t * q;
	    int forw;
	    for (forw=BACK; forw <= FORW; ++forw) {
		while(q = tcpd->ta_stream[forw].ta_Matches) {
		    tcpMatchFree( (tcpMatch_t *)q->q_data );
		    tcpd->ta_stream[forw].ta_Matches = queForw(q);
		    queFree(q);
		}
	    }
	}
	tcpAssembleCnt--;

	/*
	** discard any name/value pairs...
	*/
	if ( tcpd->ta_map ) {
	    mapBagFree( tcpd->ta_map );
	    tcpd->ta_map = 0;
	}
	free(tcpd);
    }
}


que_t *
findTcpAssemble( src, dst, sport, dport )
struct in_addr * src;
struct in_addr * dst;
u_short sport;
u_short dport;
{
    que_t * p = tcpsList;
    if (p) {
	do {
	    tcpAssemble_t * a = (tcpAssemble_t *)p->q_data;

	    if ((a->ta_src.s_addr == src->s_addr)
	     && (a->ta_dst.s_addr == dst->s_addr)
	     && (a->ta_sport == sport)
	     && (a->ta_dport == dport)) {
		return(p);
	    }
	    p = p->q_forw;
	} while ( p != tcpsList );
    }
    return(0);
}

/*
** tcpForgetMatch.
** Convience routine to cover some housekeeping for freeing tcpMatch's.
*/
void
tcpForgetMatch( ta, forw, q )
tcpAssemble_t * ta;
int forw;
que_t * q;
{
    if (q == ta->ta_stream[forw].ta_Matches) {
	ta->ta_stream[forw].ta_Matches = queForw(q);
    }
    if (q->q_data) {
	tcpMatchFree( (tcpMatch_t *)q->q_data );
	q->q_data = 0;
    }
    queFree(q);
}

/*
** tcpPatternMatch.
** Called with an assemble, and some data... and which stream.
**
** It has one of three roles:
**
** -completing a match started in the stream.
** -pushing the data up to the interpreted code
** -matching the end pattern match.
**
** We are also responsible for calling the trigger body
** when we find a match, calling the trigger body when
** we are sending data between start/stop, switching between
** start and stop patterns, finding the stop pattern,
** etc.  In other words, this performs most of the core
** operations associated with trigger/matching operations.
**
*/

void
tcpPatternMatch( ta, q, data, len, forw )
tcpAssemble_t * ta;
que_t * q;
char * data;
int len;
{
    tcpMatch_t * m = (tcpMatch_t *)q->q_data;
    char * d;
    int l;

    char * pat = m->tm_pattern;
    d = data;

    if (vFlag) {
    uiCurrPrintf( "tcpPatternMatch: %s nonce %d len %d MATCH pat %s que %x\n",
	direction[forw], packet_nonce, len, pat, q );
    dump( 0, data, len );
    uiCurrPrintf( "\n" );
    }

    /*
    ** If there isn't a pattern to match, then we still may have work to do.
    */
    if (m->tm_pattern) {
	/*
	** If we're running unanchored, it means if we miss a character
	** in the pattern match, that just fine... we're looking for
	** the pattern, we don't mind not finding it.
	*/
	if (m->tm_flags & TM_FLAGS_UNANCHORED) {
	    if (len) {
		char * start = data;
		int s_len = len;
		/*
		** scan the input string to see if we can
		** TRY to match the pattern.
		*/
		while(s_len > 0) {
		    char * d = start;
		    int l = s_len;
		    char * pat = m->tm_pattern;

		    while(*pat && l) {
			if (*pat != *d) {
			    break;
			}
			d++;
			l--;
			pat++;
		    }
		    if (*pat == 0) {
			/* complete match... */
			if (m->tm_flags & TM_FLAGS_MATCH) {
			    if (m->tm_tp && m->tm_tp->tp_trigger) {
				trigger_t * t = m->tm_tp->tp_trigger;

				tcpMatchSave( m, data, len-l );
				tcpStreamSet( ta, data, len-l, m );
				performArgs( t->t_func, t->t_scope,
					memTmpNew( NFR_TYPE_TRIGGER, t ), 0 );
				tcpStreamSet( 0, 0, 0, 0 );
			    }
			} else {
			    /*
			    ** If its the stop pattern, send the data to any trigger
			    */
			    if ((m->tm_flags & TM_FLAGS_STARTSTOP)
			      &&(m->tm_flags & TM_FLAGS_START)) {
				if (m->tm_tp && m->tm_tp->tp_trigger) {
				    trigger_t * t = m->tm_tp->tp_trigger;
				    /* from data->d */
				    tcpMatchSave( m, data, len-l );
				    tcpStreamSet( ta, data, len-l, m );
				    performArgs( t->t_func, t->t_scope,
					memTmpNew( NFR_TYPE_TRIGGER, t ), 0 );
				    tcpStreamSet( 0, 0, 0, 0 );
				    data = &data[len-l];
				    len -= l;
				    m->tm_flags |= TM_FLAGS_STOP;
				    m->tm_pattern = 0; /* done matching end pattern */
				}
			    }
			}
			/*
			** we're done with this match, but we may need to
			** review the rest of the packet for more of the 
			** same patterns (when we're doing a start pattern,
			** and and end pattern).
			*/
			if (m->tm_tp && (m->tm_tp->tp_flags & TP_FLAGS_STARTSTOP)
			 &&(m->tm_tp->tp_pattern)) {
			    tcpForgetMatch( ta, forw, q );
			    tcpPatternTest( m->tm_tp, ta, d, l, forw, 0 );
			}
			return;
		    } else if (l == 0) {
			/* ran out of characters */
			/* if we modify m, then we could miss the acutal match.
			** instead, we hafta make another match, n, and remove
			** both m, and the new n when `n' matches completely
			*/
			tcpMatch_t * n = tcpMatchNew( 0, pat, m->tm_tp, TM_FLAGS_START );
			que_t * p = queNew(n);
			n->tm_free = q;
			m->tm_refCnt++;
			if (queAdd( ta->ta_stream[forw].ta_Matches, q ) == 0) {
			    ta->ta_stream[forw].ta_Matches = q;
			}

		    }
		    start++;
		    s_len--;
		}
		/*
		** Here, we have data/len left... 
		** if we're doing start/stop, we hafta deliver the data...
		*/
		if (m->tm_flags & TM_FLAGS_STARTSTOP) {
		    if (m->tm_tp && m->tm_tp->tp_trigger) {
			trigger_t * t = m->tm_tp->tp_trigger;
			/* from data->d */
			tcpMatchSave( m, data, len );
			tcpStreamSet( ta, data, len, m );
			performArgs( t->t_func, t->t_scope,
				memTmpNew( NFR_TYPE_TRIGGER, t ), 0 );
			tcpStreamSet( 0, 0, 0, 0 );
			}
		}
	    }
	} else {
	    /*
	    ** When running ANCHORED, we must match every next charater.
	    */
	    l = len;
	    while(m->tm_pattern[0] == d[0]) {
		m->tm_pattern++;
		m->tm_seq++;
		d++;
		if (--l == 0) {
		    /* out of characters yet? */
		    break;
		}
	    }
	    if (m->tm_pattern[0] == 0) {
		/* we matched...  we ran out of pattern. */
		if (vFlag) {
		    log( FL, "tcpPatternMatch: %s MATCHED! flags %x\n",
			direction[forw], m->tm_flags );
		}
		/*
		** Are we finishing the match for the start string,
		** or finishing the match for the stop string?
		*/
		if (m->tm_flags & TM_FLAGS_STARTSTOP) {
		    if (m->tm_flags & TM_FLAGS_START) {
			/*
			** We've already the START pattern,
			** we've now matched the STOP pattern.
			** And if we have any more
			** data left, we need to see if the same
			** pattern could start up again.
			*/
			if (m->tm_tp && m->tm_tp->tp_trigger) {
			    trigger_t * t = m->tm_tp->tp_trigger;
			    tcpStreamSet( ta, data, len-l, m );
			    performArgs( t->t_func, t->t_scope,
				memTmpNew( NFR_TYPE_TRIGGER, t ), 0 );
			    tcpStreamSet( 0, 0, 0, 0 );
			}
			tcpForgetMatch( ta, forw, q );
			tcpPatternTest( m->tm_tp, ta, d, l, forw, 0 );
		    } else {
			/*
			** we haven't matched the start pattern yet...
			** so this is THAT start pattern match.
			** if there's a second string, move it to
			** the first... if there isn't a second
			** string, perform the action (!)
			*/
			m->tm_flags |= TM_FLAGS_START;
			if (m->tm_tp->tp_patEnd) {
			    m->tm_pattern = m->tm_tp->tp_patEnd;
			    m->tm_flags |= TM_FLAGS_UNANCHORED;
			}
		    }
		} else 
		if (m->tm_flags & TM_FLAGS_MATCH) {
		    /*
		    ** call the trigger, if there is one...
		    */
		    if (vFlag) {
			log( FL, "tcpPatternMatch: %s trigger\n", direction[forw] ); 
		    }
		    if (m->tm_tp && m->tm_tp->tp_trigger) {
			trigger_t * t = m->tm_tp->tp_trigger;
			if (vFlag) {
			    log( FL, "tcpPatternMatch: %s trigger, calling %x\n", 
				direction[forw], t->t_func );
			}
			tcpStreamSet( ta, data, len-l, m );
			performArgs( t->t_func, t->t_scope,
			    memTmpNew( NFR_TYPE_TRIGGER, t ), 0 );
			tcpStreamSet( 0, 0, 0, 0 );
		}
		tcpForgetMatch( ta, forw, q );
	    }
	} else if (l) {
	    /* we didn't match, and we've got data left.
	    ** if we've already seen the START string, then
	    ** we need to hand the data to a trigger.
	    */
	    if (m->tm_flags & TM_FLAGS_START) {
		if (m->tm_tp && m->tm_tp->tp_trigger) {
		    trigger_t * t = m->tm_tp->tp_trigger;
		    tcpStreamSet( ta, data, len, m );
		    performArgs( t->t_func, t->t_scope,
			memTmpNew( NFR_TYPE_TRIGGER, t ), 0 );
		    tcpStreamSet( 0, 0, 0, 0 );
		    }
		}
		tcpForgetMatch( ta, forw, q );
	    }
	}
    } else {
	/*
	** there's no pattern to match.
	** Could be we've already matched the start pattern,
	** and there's no end pattern.  If so, then
	** we hand the data to the trigger...
	*/
	if (((m->tm_flags & TM_FLAGS_STARTSTOP) == 0)
	  && (m->tm_flags & TM_FLAGS_START)
	  &&((m->tm_flags & TM_FLAGS_STOP) == 0)){
	    /* add additional characters to the save buffer */
	    if (m->tm_save) {
		tcpMatchSave( m, data, len );
	    }
	    /* hand all the data up... */
	    if (m->tm_tp && m->tm_tp->tp_trigger) {
		trigger_t * t = m->tm_tp->tp_trigger;
		tcpStreamSet( ta, data, len, m );
		performArgs( t->t_func, t->t_scope,
			memTmpNew( NFR_TYPE_TRIGGER, t ), 0 );
		tcpStreamSet( 0, 0, 0, 0 );
	    }
	}
    }
}

/*
** tcpAssembleMatch... iterating across all the
** matchines currently associated with the tcpAssemble,
** and try to move some of them forward...
*/

void
tcpAssembleMatch( ta, data, len, forw )
tcpAssemble_t * ta;
char * data;
int len;
{
    que_t * q = ta->ta_stream[forw].ta_Matches;
    if (q) {
	/*
	** see if we have any in progress...
	*/
	do {
	    que_t * qf = q->q_forw;
	    tcpPatternMatch( ta, q, data, len, forw );
	} while( ta->ta_stream[forw].ta_Matches && q != ta->ta_stream[forw].ta_Matches );
    }
}

/*
** tcpPatternTest.
** Try to apply the pattern to the stream.
**
** as soon as the first character matches, we 
** create a match structure, and hand the rest
** of the matching process over to tcpPatternMatch.
** forw is 1 if we're Testing in the forw direction,
** 0, if we testing in the backward direction.
*/

void
tcpPatternTest( p, rock, data, len, forw, flags )
tcpPattern_t * p;
void * rock;	/* actually tcpAssemble_t * */
char * data;
int len;
int forw;	/* which direction we're testing. */
int flags;	/* flags to associate with any tcpMatch structures we new */
{
    tcpAssemble_t * ta = (tcpAssemble_t *)rock;

    /*
    ** Before we decide to run this pattern in this ta,
    ** lets see if its already running... look in the
    ** list we'd be adding it to.
    */
    if (ta->ta_stream[forw].ta_Matches) {
	que_t * q = ta->ta_stream[forw].ta_Matches;
	do {
	    tcpMatch_t * m = (tcpMatch_t *)q->q_data;
	    if (m && (m->tm_tp == p)) {
		/* this pattern is currently in progress
		** in this direction on this stream
		** Don't start it again (option in the future?)
		*/
		if (vFlag) {
		    printf( "tcpPatternTest: early return\n" );
		}
		return;
	    }
	    q = q->q_forw;
	} while (q != ta->ta_stream[forw].ta_Matches);
    }

    if (len) {
	int startStop = 0;

	while (len) {
	    char * d = data;

	    if (p->tp_pattern == 0) {
		if (p->tp_flags & TP_FLAGS_STARTSTOP) {
		    /*
		    ** Keep state about the connection.
		    ** We don't worry about client/server, cause something else 
		    ** has tested for those cases.
		    */
		    tcpMatch_t * m = tcpMatchNew( 0, p->tp_patEnd, p, flags );
		    que_t * q = queNew(m);
		    m->tm_flags |=
			(TM_FLAGS_UNANCHORED|TM_FLAGS_START);
		    if (queAdd( ta->ta_stream[forw].ta_Matches, q ) == 0) {
			ta->ta_stream[forw].ta_Matches = q;
		    }
		    if (p->tp_patEnd == 0) {
			/*
			** deliver any data we've got no start, no stop... no match...
			*/
			if (p->tp_trigger) {
			    trigger_t * t = p->tp_trigger;
			    tcpStreamSet( ta, data, len, m );
			    if (vFlag) {
				uiCurrPrintf( "CALLING <%s> t %x\n", t->t_func, t );
			    }
			    performArgs( t->t_func, t->t_scope,
				    memTmpNew( NFR_TYPE_TRIGGER, p->tp_trigger ), 0 );
			    tcpStreamSet( 0, 0, 0, 0 );
			}
		    } else {
			/* 
			** match any of the remaining data against the stop pattern.
			*/
			tcpPatternMatch( ta, q, data, len, forw );
		    }
		    return;
		} else if (p->tp_flags & TP_FLAGS_MATCH) {
		    panic( format( "%s:%d match without pattern", FL ) );
		} else {
		    /* nothing else set, but the pattern matched... */
		    if (p->tp_trigger) {
			trigger_t * t = p->tp_trigger;
			tcpStreamSet( ta, data, len, 0 );
			if (vFlag) {
			    uiCurrPrintf( "CALLING <%s> t %x\n", t->t_func, t );
			}
			performArgs( t->t_func, t->t_scope,
				memTmpNew( NFR_TYPE_TRIGGER, p->tp_trigger ), 0 );
			tcpStreamSet( 0, 0, 0, 0 );
		    }
		    return;
		}
	    } else
	    if (p->tp_pattern[0] == d[0]) {
		char * pat = &p->tp_pattern[1];
		int l = len-1; 
		d++;

		/* first charcter has matched. */
		while (*pat && l) {
		    if (*pat != *d) {
			break;
		    }
		    d++;
		    l--;
		    pat++;
		}

		/* why'd we exit the loop? */

		if (*pat == 0) {
		    /* we matched the pattern... even if we no `l' left.  */

		    if (vFlag) {
			log( FL, "tcpPatternTest: MATCHED! %s nonce %d\n", p->tp_pattern, packet_nonce );
			Display( 1, "tcpPatternTest" );
			uiCurrPrintf( "What? %x\n", p->tp_trigger );
		    }

		    if (p->tp_flags & TP_FLAGS_MATCH) {
			/*
			** if we're TP_FLAGS_MATCHing, perform the trigger...
			*/
			if (p->tp_trigger) {
			    trigger_t * t = p->tp_trigger;
			    if (vFlag) {
				uiCurrPrintf( "CALLING <%s> t %x\n", t->t_func, t );
			    }
			    tcpStreamSet( ta, data, len-l, 0 );
			    performArgs( t->t_func, t->t_scope,
				    memTmpNew( NFR_TYPE_TRIGGER, p->tp_trigger ), 0 );
			    tcpStreamSet( 0, 0, 0, 0 );
			}
		    } else {
			/*
			** We're sending start-to-stop pattern data....
			** we call tcpPatternMatch to deal with the
			** stop part of the pattern...
			*/
			tcpMatch_t * m = tcpMatchNew( 0, p->tp_patEnd, p, 
				TM_FLAGS_START|TM_FLAGS_UNANCHORED );
			que_t * q = queNew(m);
			tcpPatternMatch( ta, q, d, l, forw );
		    }
		    return;
		} else if (l == 0) {
		    /* we ran out of data to match. */
		    /* create a match structure, leave the partial match inside it */
		    tcpMatch_t * m = tcpMatchNew( 0, pat, p, flags );
		    que_t * q = queNew(m);
		    if (queAdd( ta->ta_stream[forw].ta_Matches, q ) == 0) {
			ta->ta_stream[forw].ta_Matches = q;
		    }
		    return;
		}
		/* we didn't match the pattern... try again... */
	    }
	    data++;
	    len--;
	}
    }
}

/*
** display some statistics about the tcpAssemble strucure...
*/

void
tcpAssembleStats( ta, taCnt, taBytes )
tcpAssemble_t * ta;
int * taCnt;
int * taBytes;
{

    {
	char tbuf[64];
	strftime( tbuf, sizeof(tbuf),
	    "%D %T", localtime( &ta->ta_epoch ) );
	uiCurrPrintf( "ta: %x epoch %d %s hash %d\n",
		ta, ta->ta_epoch, tbuf, ta->ta_hash );
    }
    uiCurrPrintf( "ta: src %s/%d ",
	    inet_ntoa( ta->ta_src ), ta->ta_sport );
    uiCurrPrintf( "dst %s/%d ",
	    inet_ntoa( ta->ta_dst ), ta->ta_dport );

    uiCurrPrintf( "\nta: " );

    {
	char tbuf[64];

	strftime( tbuf, sizeof(tbuf),
	    "%D %T", localtime( &ta->ta_time ) );
	uiCurrPrintf( "time %d %s flags %-d",
		ta->ta_time, tbuf, ta->ta_flags );
	{
	    int cnt;
	    cnt = util_tcpFlags( ta->ta_flags );
	    if (ta->ta_flags & TA_FLAGS_ORIG_SRC) {
		if (cnt) {
		    uiCurrPrintf( "," );
		}
		uiCurrPrintf( "origSrc" );
		cnt++;
	    }
	    if (ta->ta_flags & TA_FLAGS_ORIG_DST) {
		if (cnt) {
		    uiCurrPrintf( "," );
		}
		uiCurrPrintf( "origDst" );
		cnt++;
	    }
	    if (ta->ta_flags & TA_FLAGS_FLIP) {
		if (cnt) {
		    uiCurrPrintf( "," );
		}
		uiCurrPrintf( "flip" );
		cnt++;
	    }
	    if (ta->ta_flags & TA_FLAGS_RSTACK) {
		if (cnt) {
		    uiCurrPrintf( "," );
		}
		uiCurrPrintf( "rstack" );
		cnt++;
	    }
	    if (ta->ta_flags & TA_FLAGS_FINACK) {
		if (cnt) {
		    uiCurrPrintf( "," );
		}
		uiCurrPrintf( "finack" );
		cnt++;
	    }
	    if (ta->ta_flags & TA_FLAGS_SYNACK) {
		if (cnt) {
		    uiCurrPrintf( "," );
		}
		uiCurrPrintf( "synack" );
		cnt++;
	    }
	}
    }
    uiCurrPrintf( " syn %d synAck %d rst %d rstAck %d fin %d finAck %d",
	ta->ta_syn, ta->ta_synAck, ta->ta_rst, ta->ta_rstAck, ta->ta_fin, ta->ta_finAck );

    {
	int forw;	/* back = 0, forw = 1 */
	for (forw = 0; forw < TCP_STREAMS; forw ++ ) {
	    char tbuf[64];
	    que_t * p;
	    int seq = ta->ta_stream[forw].ta_SeqNo;

	    uiCurrPrintf( "\nta: " );
	    uiCurrPrintf( "%s %x seqNo %u orig %u packNo %u bytes %d lost %d cnt %d info %c hole %d",
		direction[forw],
		ta->ta_stream[forw].ta_Packets, seq,
		ta->ta_stream[forw].ta_OrigSeq,
		ta->ta_stream[forw].ta_PackNo,
		ta->ta_stream[forw].ta_Bytes,
		ta->ta_stream[forw].ta_Lost,
		ta->ta_stream[forw].ta_Cnt,
		ta->ta_stream[forw].ta_info,
		ta->ta_stream[forw].ta_holeCnt );
	    uiCurrPrintf( "\nta: " );
	    strftime( tbuf, sizeof(tbuf),
		"[%D %T]", localtime( &ta->ta_stream[forw].ta_Arriv.tv_sec ) );
	    uiCurrPrintf( "last arriv %s sec.usec %d.%6.6d Diff %d.%6.6d\n",
		tbuf,
		ta->ta_stream[forw].ta_Arriv.tv_sec,
		ta->ta_stream[forw].ta_Arriv.tv_usec,
		ta->ta_stream[forw].ta_ArrDiff.tv_sec,
		ta->ta_stream[forw].ta_ArrDiff.tv_usec );
	    if (ta->ta_stream[forw].ta_Cnt) {
		double r = (util_tv2usec( &ta->ta_stream[forw].ta_ArrDiff ) /
		    ta->ta_stream[forw].ta_Cnt) / MILLION;
		double d = util_tv2usec( &ta->ta_stream[forw].ta_ArrDiff )/MILLION;
		int cnt = ta->ta_stream[forw].ta_Cnt;
		uiCurrPrintf( "ta: Packet Arriv Rate sec per packet %g Bytes Arriv Rate %g SumSq %g",
			r,
			r / ta->ta_stream[forw].ta_Cnt * ta->ta_stream[forw].ta_Bytes,
			ta->ta_stream[forw].ta_ArrSumSq );
		if (r > 0) {
		    uiCurrPrintf( "\nta: Packet Arriv Rate packet/sec %g Bytes/sec %g", 
			1/r,
			(1/r) / ta->ta_stream[forw].ta_Cnt * ta->ta_stream[forw].ta_Bytes
			);
		}
		if (cnt > 1) {
		    uiCurrPrintf( "\nta: Packet stdDev %g",
			ta->ta_stream[forw].ta_ArrStdDv );
		}
	    }

	    uiCurrPrintf( "\nta: " );
	    if (p = ta->ta_stream[forw].ta_Packets) {
		do {
		    char tbuf[64];

		    tcpData_t * d = (tcpData_t *)p->q_data;
		    if (taCnt) {
			(*taCnt)++;
		    }
		    if (taBytes) {
			*taBytes += d->td_len;
		    }

		    strftime( tbuf, sizeof(tbuf),
			"[%D %T]", localtime( &d->td_sec ) );

		    uiCurrPrintf( "\nta: %s packets seq:len ", direction[forw] );
		    uiCurrPrintf( "%dD:%d[%dD]#%d %s f%x=",
			d->td_seq-seq, d->td_len, d->td_seq-seq+d->td_len,
			d->td_nonce, tbuf, d->td_flags );
		    util_tcpFlags( d->td_flags );
		    switch( (d->td_client & (CLIENT|SERVER) ) ) {
			case 0:
			    uiCurrPrintf( " " );
			    break;
			case CLIENT:
			    uiCurrPrintf( ",client " );
			    break;
			case SERVER:
			    uiCurrPrintf( ",server " );
			    break;
			case CLIENT|SERVER:
			    uiCurrPrintf( ",client/server " );
			    break;
		    }

		    p = p->q_forw;
		} while (p != ta->ta_stream[forw].ta_Packets);
	    } else {
		uiCurrPrintf( "no %s packets ", direction[forw] );
	    }

	    uiCurrPrintf( "\nta: %s Past... ", direction[forw] );
	    {
		int i = ta->ta_stream[forw].ta_PastI;
		do {
		    if (ta->ta_stream[forw].ta_Past[i].tp_seq == 0) {
			break;
		    }
		    uiCurrPrintf( "[%dD:%u]",
			    ta->ta_stream[forw].ta_Past[i].tp_seq-seq,
			    ta->ta_stream[forw].ta_Past[i].tp_len );
		    i = ((i == 0) ? (PASTLEN-1) : (i-1));
		} while( i != ta->ta_stream[forw].ta_PastI );
	    }
	}
    }
    uiCurrPrintf( "\n" );
}

/*
** tcpAssembleMem
**
** print info about tcpAssemble memory usage.
** return the number of bytes consumed.
*/

int
tcpAssembleMem( ta )
tcpAssemble_t * ta;
{
    int packets[TCP_STREAMS];
    int matches[TCP_STREAMS];
    int maps = mapBagMem( ta->ta_map );
    int forw;

    for ( forw=0; forw<TCP_STREAMS; ++forw) {
	packets[forw] = 0;
	matches[forw] = 0;
    }

    for ( forw=0; forw<TCP_STREAMS; ++forw) {
	que_t * p;
	if (p = ta->ta_stream[forw].ta_Packets) {
	    do {
		tcpData_t * d = (tcpData_t *)p->q_data;
		packets[forw] += sizeof(*d) + d->td_len;
		p = p->q_forw;
	    } while (p != ta->ta_stream[forw].ta_Packets);
	}
	if (p = ta->ta_stream[forw].ta_Matches) {
	    do {
		tcpMatch_t * m = (tcpMatch_t *)p->q_data;
		matches[forw] += sizeof(*m) + m->tm_saveLen;
		p = p->q_forw;
	    } while (p != ta->ta_stream[forw].ta_Matches);
	}
    }
    uiCurrPrintf( "ta: %x %d %d sizeof %d packets forw %d back %d matches forw %d back %d symbols %d\n",
	ta, ta->ta_hash,
	sizeof(*ta) + packets[0] + packets[1] + maps + matches[0] + matches[1],
	sizeof(*ta), packets[0], packets[1], matches[0], matches[1], maps );

    return( sizeof(*ta) + packets[0] + packets[1] + maps + matches[0] + matches[1] );
}

/*
** tcpAssemblePrint
** used to iterate through all the the connections, and display the values
** of variables.
**
*/

int
tcpAssemblePrint( callout, rock )
mapCallout_t callout;
void * rock;
{
    if (tcpsList) {
	que_t * q = tcpsList;
	do {
	    tcpAssemble_t * ta = (tcpAssemble_t *)q->q_data;
	    MapBag * mb = ta->ta_map;
	    char info[128];

	    sprintf( info, "tcp.conn[%s,%s]",
		inet_ntoa( ta->ta_src ), inet_ntoa( ta->ta_dst ) );
	    /*
	    ** look in every one of the streams for variables.
	    ** currently, we don't pass any new details about which
	    ** connection the variable is associated with.
	    ** We could do that by passing a 'char *' with 
	    ** some context associated with the variable.
	    */
	    if (mb) {
		mapBagIterate( mb, callout, rock, info );
	    }

	    q = q->q_forw;
	} while (q != tcpsList);
    }
	
}

/*
** tcpReassemble.
** Review packets in the tcpAssemble structure, and try to
** move the data forward...
**
*/

int tcpForwZero = 0;
int tcpForwLost = 0;
int tcpBackZero = 0;
int tcpBackLost = 0;


tcpReassemble( ta )
tcpAssemble_t * ta;
{
    /*
    ** Walk the forward and backward queue's, looking to 
    ** see if there's data we can deliver to the higher level... 
    **
    ** Instead of sorting the saved packets, instead, we'll
    ** just try to find one that's supposed to be there, then
    ** deliver it. 
    **
    ** Any packets which should have already been delivered
    ** ought to be alerted upon (but only if they're not
    ** duplicates)
    */

    moreForward:
    if (ta->ta_forwPackets) {
	que_t * p = ta->ta_forwPackets;

	do {
	    tcpData_t * d = (tcpData_t *)p->q_data;

	    if (d->td_seq == ta->ta_forwSeqNo) {
		ta->ta_forwSeqNo = d->td_seq + d->td_len;
		if (d->td_len) {
		    /*
		    ** Add this to the forward direction.
		    */
		    ta->ta_forwBytes += d->td_len;
		    tcpAssembleMatch( ta, d->td_data, d->td_len, FORW );
		    tcpPatternsCall( tcpPatternTest, ta,
			d->td_data, d->td_len, d->td_client | FORW, 0 );
		}
		if (p == ta->ta_forwPackets) {
		    tcpForwZero++;
		    ta->ta_forwPackets = queForw(p);
		    if (ta->ta_forwPackets == 0) {
			ta->ta_forwPackNo = 0;
		    }
		}
		queFree(p);
		tcpDataFree(d);
		goto moreForward;
	    }
	    if (d->td_seq < ta->ta_forwSeqNo) {
		/* hey, we should have already delivered this... */
		if (d->td_seq+d->td_len > ta->ta_forwSeqNo) {
		    /* deliver bypes between ta->ta_forwSeqNo and
		    ** d->td_seq+d->td_len.
		    */
		    int ndx = ta->ta_forwSeqNo - d->td_seq;
		    if (vFlag) {
			printf( "FORW %d retrans %d seq %u pack seq %u len %u\n",
			    ta->ta_hash, ndx, ta->ta_forwSeqNo, d->td_seq, d->td_len );
		    }
		    ta->ta_forwLost += ndx;
		    ta->ta_forwSeqNo = d->td_seq+d->td_len;
		    ta->ta_forwBytes += ta->ta_forwSeqNo - d->td_seq;

		    tcpAssembleMatch( ta, d->td_data, d->td_len, FORW );
		    tcpPatternsCall( tcpPatternTest, ta,
			&d->td_data[ndx], ta->ta_forwSeqNo - d->td_seq,
			d->td_client | FORW, 0 );
		} else {
		    ta->ta_forwLost += d->td_len;
		}
		/* XXX alert? XXX */
		if (p == ta->ta_forwPackets) {
		    /* no more data. */
		    tcpForwLost++;
		    ta->ta_forwPackets = queForw(p);
		    if (ta->ta_forwPackets == 0) {
			ta->ta_forwPackNo = 0;
		    }
		}
		queFree(p);
		tcpDataFree(d);
		goto moreForward;
	    }
	    p = p->q_forw;
	} while (p != ta->ta_forwPackets);
    }

    moreBackward:
    if (ta->ta_backPackets) {
	que_t * p = ta->ta_backPackets;

	do {
	    tcpData_t * d = (tcpData_t *)p->q_data;

	    if (d->td_seq == ta->ta_backSeqNo) {
		ta->ta_backSeqNo = d->td_seq + d->td_len;
		/* deliver the data. */
		if (d->td_len) {
		    /*
		    ** Add this to the back direction.
		    */
		    ta->ta_backBytes += d->td_len;
		    tcpAssembleMatch( ta, d->td_data, d->td_len, BACK );
		    tcpPatternsCall( tcpPatternTest, ta,
			d->td_data, d->td_len, d->td_client | BACK, 0 );
		}
		if (p == ta->ta_backPackets) {
		    tcpBackZero++;
		    ta->ta_backPackets = queForw(p);
		    if (ta->ta_backPackets == 0) {
			ta->ta_backPackNo = 0;
		    }
		}
		queFree(p);
		tcpDataFree(d);
		goto moreBackward;
	    }

	    /*
	    ** We may have duplicate data...
	    ** in fact, it may even overlapp...
	    ** In which we we have to be senseitive
	    ** to the difference
	    */
	    if (d->td_seq < ta->ta_backSeqNo) {
		/* hey, we should have already delivered this ... */
		if (d->td_seq+d->td_len > ta->ta_backSeqNo) {
		    /* deliver bypes between ta->ta_backSeqNo and
		    ** d->td_seqNo+d->td_len.
		    */
		    int ndx = ta->ta_backSeqNo - d->td_seq;
		    if (vFlag) {
			printf( "BACK %d retrans %d seq %u pack seq %u len %u\n",
			    ta->ta_hash, ndx, ta->ta_backSeqNo, d->td_seq, d->td_len );
		    }
		    ta->ta_backLost += ndx;
		    ta->ta_backSeqNo = d->td_seq+d->td_len;
		    ta->ta_backBytes += ta->ta_backSeqNo - d->td_seq;
		    tcpAssembleMatch( ta, d->td_data, d->td_len, BACK );
		    tcpPatternsCall( tcpPatternTest, ta,
			&d->td_data[ndx], ta->ta_backSeqNo - d->td_seq,
			d->td_client | BACK, 0 );
		} else {
		    ta->ta_backLost += d->td_len;
		}
		if (p == ta->ta_backPackets) {
		    tcpBackLost++;
		    ta->ta_backPackets = queForw(p);
		    if (ta->ta_backPackets == 0) {
			ta->ta_backPackNo = 0;
		    }
		}
		queFree(p);
		tcpDataFree(d);
		goto moreBackward;
	    }
	    p = p->q_forw;
	} while ( p != ta->ta_backPackets );
    }
}

/*
** reassembleTcp.
** Adds a new packet, then trys to tcpReassemble.
*/

void
reassembleTcp()
{
    MemVal m;

    if ((tcpIsTcp(&m) == 0) && (memInt32(&m) == TRUE)) {
	/*
	** reassemble the tcp stream.
	*/
	struct ip * ip = ipHdr();
	struct tcphdr * tcp = tcpHdr();

	if (tcp) {
	    que_t * a = 0;
	    tcpAssemble_t * ta = 0;
	    struct in_addr src[1];
	    struct in_addr dst[1];
	    int sport;
	    int dport;
	    u_int seq;
	    u_int ack;
	    int len;
	    int hlen;
	    int comming = 1;		/* is the data comming or going? */
	    int client = 0;		/* client sent it (2)? or the server (4)? */

	    src->s_addr = inaddr( &ip->ip_src );
	    dst->s_addr = inaddr( &ip->ip_dst );

	    sport = int16( &tcp->th_sport );
	    dport = int16( &tcp->th_dport );

	    seq = int32( &tcp->th_seq );
	    ack = int32( &tcp->th_ack );

	    hlen = (tcp->th_off * 4);
	    len = int16( &ip->ip_len ) - (hlen) - ipHLen();

	    if (len < 0) {
		/*
		** no use trying to get this packet sequenced  
		*/

		Alert_Args_Generate( NFR_SRC_NFRD, NFR_NFRD_TCPNEGATIVELEN, 
		    format( "%d (%d,%d,%d)",
			len, int16( &ip->ip_len ), sizeof(struct tcphdr), ipHLen() ), 0 );
		return;
	    }

	    /*
	    ** We canonicalize the addresses... Cause then
	    ** we only need to lookup once... 
	    **
	    ** if we lookup using src/dst, then we leave
	    ** comming alone.
	    **
	    ** if we lookup using dst/src, then we set comming
	    ** to zero, meaning the data is intended.
	    */
	    if (src->s_addr < dst->s_addr) {
		if (a = findTcpAssemble( src, dst, sport, dport )) {
		    ta = (tcpAssemble_t *)a->q_data;
		}
	    } else {
		if (a = findTcpAssemble( dst, src, dport, sport)) {
		    ta = (tcpAssemble_t *)a->q_data;
		    comming = 0;
		}
	    }

	    if (a == 0) {
		int flags = 0;
		/*
		** start recording now... even if we don't have a SYN
		** we'll just hafta believe what we've heard so far.
		*/
		if (src->s_addr < dst->s_addr) {
		    if (tcp->th_flags & TH_SYN) {
			flags |= TA_FLAGS_ORIG_SRC;
		    }
		    ta = tcpAssembleNew( src, dst, sport, dport, flags );
		    ta->ta_forwSeqNo = seq;
		    ta->ta_forwInfo = 's';
		    ta->ta_backSeqNo = ack;
		    ta->ta_backInfo = 'a';
		    ta->ta_forwOrigSeq = seq;
		    ta->ta_backOrigSeq = ack;
		} else {
		    flags |= TA_FLAGS_FLIP;
		    if (tcp->th_flags & TH_SYN) {
			flags |= TA_FLAGS_ORIG_DST;
		    }
		    ta = tcpAssembleNew( dst, src, dport, sport, flags );
		    ta->ta_forwSeqNo = ack;
		    ta->ta_forwInfo = 'b';
		    ta->ta_backSeqNo = seq;
		    ta->ta_backInfo = 't';
		    ta->ta_forwOrigSeq = ack;
		    ta->ta_backOrigSeq = seq;
		    comming = 0;
		}
		a = queNew( (void *)ta );
		if (queAdd( tcpsList, a ) == 0) {
		    tcpsList = a;
		}
	    } else {
		/* mark `comming' when we found assemble on the list */
		if (src->s_addr > dst->s_addr) {
		    comming= 0;
		}
	    }
	    ta->ta_time = packet_sec();

	    /*
	    ** Detect the opening of the connection as the client/server
	    ** would notice it.  Only thing is, this isn't perfect.
	    ** we need lots more tests to get the right condition here.
	    ** We're simply following the SYN-ACK from the server,
	    ** then noticing the reply ACK.  We ought to test to make
	    ** sure the ACK is comming from the client end.
	    */
	    if (tcp->th_flags & TH_ACK) {
		if (ta->ta_flags & TA_FLAGS_SYNACK) {
		    ta->ta_flags &= ~TA_FLAGS_SYNACK;
		    tcpOpenCloseCall( tcpOpenCloseTest,
				    ta, 0, 0, 0, TP_FLAGS_OPENSESSION );
		}
	    }
	    /*
	    ** Now use SYN to sync the sequence numbers.
	    ** (SYN|ACK ought to have both)
	    */
	    if (tcp->th_flags & TH_SYN) {
		ta->ta_syn++;
		if (comming) {
		    ta->ta_forwSeqNo = seq;
		    ta->ta_forwInfo = 'S';
		    ta->ta_backSeqNo = ack;
		    ta->ta_backInfo = 'A';
		    if ((tcp->th_flags & TH_ACK) == 0) {
			ta->ta_flags |= TA_FLAGS_ORIG_SRC;
		    } else {
			ta->ta_forwSeqNo++;
			ta->ta_synAck++;
			ta->ta_flags |= TA_FLAGS_SYNACK;
		    }
		} else {
		    ta->ta_forwSeqNo = ack;
		    ta->ta_forwInfo = 'B';
		    ta->ta_backSeqNo = seq;
		    ta->ta_backInfo = 'T';
		    if ((tcp->th_flags & TH_ACK) == 0) {
			ta->ta_flags |= TA_FLAGS_ORIG_DST;
		    } else {
			ta->ta_backSeqNo++;
			ta->ta_synAck++;
			ta->ta_flags |= TA_FLAGS_SYNACK;
		    }
		}
	    }

	    /*
	    ** If this is the ack for the fin, then we may jiggle the
	    ** sequence numbers.
	    */
	    if ((tcp->th_flags & TH_ACK) && (ta->ta_flags & TH_FIN)) {
		if (comming) {
		    if (seq - ta->ta_forwSeqNo == 1) {
			ta->ta_forwSeqNo++;
		    }
		} else {
		    if (seq - ta->ta_backSeqNo == 1) {
			ta->ta_backSeqNo++;
		    }
		}
	    }

	    /*
	    ** leave the RST flag sticking in the ta
	    ** structure... then later we'll GC it...
	    */
	    ta->ta_flags |= (tcp->th_flags & (TH_RST|TH_FIN));

	    if ((tcp->th_flags & (TH_RST|TH_ACK)) == (TH_RST|TH_ACK)) {
		ta->ta_rstAck++;
	    }
	    if ((tcp->th_flags & (TH_FIN|TH_ACK)) == (TH_FIN|TH_ACK)) {
		ta->ta_finAck++;
	    }

	    /*
	    ** we don't monitor the connections close enough to
	    ** really determine the window size, which is important
	    ** for RST processing... the sequence numbers 
	    ** are supposed to be in the window.
	    ** But for right now, just try something simple.
	    */
	    if (tcp->th_flags & TH_RST) {
		ta->ta_rst++;
		if (comming) {
		    if (seq - ta->ta_forwSeqNo == 1) {
			ta->ta_forwSeqNo++;
		    }
		} else {
		    if (seq - ta->ta_backSeqNo == 1) {
			ta->ta_backSeqNo++;
		    }
		}
	    }
	    if (tcp->th_flags & TH_FIN) {
		ta->ta_fin++;
	    }


	    /*
	    ** Figure out whether the packet was sent by the client or the server.
	    ** There are three values possible:
	    ** 0 == we don't know.
	    ** 2 == the client. (CLIENT == 2)
	    ** 4 == the server. (SERVER == 4)
	    ** These values are choosen to be values easily
	    ** extracted by tcpPatternsCall.
	    */
	    if (ta->ta_flags & TA_FLAGS_ORIG_SRC) {
		if (comming == 1) {
		    client = CLIENT;
		} else {
		    client = SERVER;
		}
	    }
	    if (ta->ta_flags & TA_FLAGS_ORIG_DST) {
		if (comming == 0) {
		    client = CLIENT;
		} else {
		    client = SERVER;
		}
	    }


	    /*
	    ** Take the data from the current packet, and
	    ** either add it to the list of packet comming
	    ** towards src, or going towards dest
	    */
	    {
		char * data = tcpEnd();
		tcpData_t * tcpd = tcpDataNew( seq, data, len, tcp->th_flags,
			client, packet_sec() );
		que_t * n = queNew( tcpd );

		if (vFlag) {
		    Display( 1, "Packet:" );
		    uiCurrPrintf("Packet Data: ta_flags %x comming %x client %x <%d>\n", 
			ta->ta_flags, comming, client, len );
		    dump( 0, data, len );
		    uiCurrPrintf("\n" );
		}

		if (comming) {
		    if (ta->ta_forwPackets) {
			queAdd( ta->ta_forwPackets->q_back, n );
		    } else {
			ta->ta_forwPackets = n;
		    }
		    tcpForwPast( ta, seq, len );
		    if (ta->ta_forwCnt > 0) {
			int cnt = ta->ta_forwCnt;
			double sq;
			timeval_t d = util_diffTv( &packet_hdr->ts, &ta->ta_forwArriv );
			util_incrTv( &ta->ta_forwArrDiff, &d );
			sq = util_tv2usec( &d )/MILLION;
			ta->ta_forwSumSq += (sq*sq);

			if (cnt > 1) {
	 		    double r =
				(util_tv2usec( &ta->ta_forwArrDiff ) / cnt) / MILLION;
			    ta->ta_forwArrStdDv = util_stdDev( r, ta->ta_forwSumSq, cnt );
			}
		    }
		    ta->ta_forwArriv = packet_hdr->ts;
		    if (ta->ta_forwPackNo == 0) {
			ta->ta_forwPackNo = seq;
		    } else {
			if (seq < ta->ta_forwPackNo) {
			    ta->ta_forwPackNo = seq;
			}
		    }
		    ta->ta_forwCnt++;
		} else {
		    if (ta->ta_backPackets) {
			queAdd( ta->ta_backPackets->q_back, n );
		    } else {
			ta->ta_backPackets = n;
		    }
		    tcpBackPast( ta, seq, len );
		    if (ta->ta_backCnt > 0) {
			int cnt = ta->ta_backCnt;
			double sq;
			timeval_t d = util_diffTv( &packet_hdr->ts, &ta->ta_backArriv );
			util_incrTv( &ta->ta_backArrDiff, &d );
			sq = util_tv2usec( &d )/MILLION;
			ta->ta_backSumSq += (sq*sq);

			if (cnt > 1) {
	 		    double r =
				(util_tv2usec( &ta->ta_backArrDiff ) / cnt) / MILLION;
			    ta->ta_backArrStdDv = util_stdDev( r, ta->ta_backSumSq, cnt );
			}
		    }
		    ta->ta_backArriv = packet_hdr->ts;
		    if (ta->ta_backPackNo == 0) {
			ta->ta_backPackNo = seq;
		    } else {
			if (seq < ta->ta_backPackNo) {
			    ta->ta_backPackNo = seq;
			}
		    }
		    ta->ta_backCnt++;
		}
	    }
	    tcpReassemble( ta );
	} else {
	    /*
	    ** Hey, somethings really goofed up.
	    */
	}
    }
}

/*
** tcpLowSeq
** Find the lowest sequence number of all the recorded packets.
** return the packet (tcpData_t) associated with that sequence number.
**
*/
tcpData_t *
tcpLowSeq( list )
que_t * list;
{
    u_int low = HIGH_UINT;
    que_t * q = list;
    tcpData_t * tdLow = 0;

    if (q) {
	do {
	    tcpData_t * td = (tcpData_t *)q->q_data;
	    if (td && (td->td_seq < low)) {
		low = td->td_seq;
		tdLow = td;
	    }
	    q = q->q_forw;
	} while (q != list);
    }
    return(tdLow);
}

/*
** tcpPacketCnt
** return the number of packets enqueud on this stream.
**
*/
int
tcpPacketCnt( list )
que_t * list;
{
    u_int cnt = 0;
    que_t * q = list;

    if (q) {
	do {
	    tcpData_t * td = (tcpData_t *)q->q_data;
	    if (td) {
		cnt++;
	    }
	    q = q->q_forw;
	} while (q != list);
    }
    return(cnt);
}


/*
** reassembleTcpGC
**
*/

#define	TCP_TIMEOUT	600
int tcpsListForw = 0;

void
reassembleTcpGC()
{
    int cnt = 0;
    int now = packet_sec();
    if (now == 0) { /* any packet? */
	now = time(0);
    }

    moreGarbage:
    {
	que_t * q = tcpsList;
	if (q) {
	    do {
		tcpAssemble_t * ta = (tcpAssemble_t *)q->q_data;
		double stdDv = ta->ta_stream[0].ta_ArrStdDv;
		double rate = 10 * 600; /* kinda default timeout XXX */

		if (stdDv == 0) {
		    if (ta->ta_stream[1].ta_ArrStdDv > 0) {
			stdDv = ta->ta_stream[1].ta_ArrStdDv;
			rate = (util_tv2usec( &ta->ta_stream[1].ta_ArrDiff ) /
				    ta->ta_stream[1].ta_Cnt) / MILLION;
		    } 
		} else {
		    if ((ta->ta_stream[1].ta_ArrStdDv > 0) &&
		        (ta->ta_stream[1].ta_ArrStdDv > stdDv)) {
			    stdDv = ta->ta_stream[1].ta_ArrStdDv;
			    rate = (util_tv2usec( &ta->ta_stream[1].ta_ArrDiff ) /
				    ta->ta_stream[1].ta_Cnt) / MILLION;
		    } else {
			rate = (util_tv2usec( &ta->ta_stream[0].ta_ArrDiff ) /
				ta->ta_stream[0].ta_Cnt) / MILLION;
		    }
		}
		if (stdDv) {
		    stdDv *= 4; /* four standard deviations */

		    if ((ta->ta_flags & (TH_RST|TH_FIN))
		     && (stdDv > 0) && ((ta->ta_time + rate + stdDv) < now)) {
			if (vFlag) {
			    printf( "reassembleTcpGC: stdDv wait %s/%d %g (%g)\n",
				inet_ntoa( ta->ta_src ), ta->ta_sport, stdDv, stdDv/4 );
			}
			if (q == tcpsList) {
			    tcpsList = queForw(q);
			    tcpsListForw++;
			}
			queFree(q);

			if (vFlag) {
			    printf( "reassembleTcpGC: freeing %x %d time %d rate %g stdDv %g now %d %s/%d ",
				ta, ta->ta_hash,
				ta->ta_time, rate, stdDv, now, 
				inet_ntoa( ta->ta_src ), ta->ta_sport );
			    printf( " %s/%d\n",
				inet_ntoa( ta->ta_dst ), ta->ta_dport );
			    /* tcpAssembleStats(ta, 0, 0); */
			}
			tcpAssembleFree(ta);
			cnt++;
			goto moreGarbage;
		    }
		}


		if (((ta->ta_time + TCP_TIMEOUT) < now)
		 && (ta->ta_flags & (TH_RST|TH_FIN))) {
		    GarbageCollect:

		    if (q == tcpsList) {
			tcpsList = queForw(q);
			tcpsListForw++;
		    }
		    queFree(q);

		    if (vFlag) {
			printf( "reassembleTcpGC: freeing %s/%d ",
			    inet_ntoa( ta->ta_src ), ta->ta_sport );
			printf( " %s/%d\n",
			    inet_ntoa( ta->ta_dst ), ta->ta_dport );
			tcpAssembleStats(ta, 0, 0);
		    }
		    tcpAssembleFree(ta);
		    cnt++;
		    goto moreGarbage;
		} else {
		    /*
		    ** see if we should move the sequence number
		    ** forward (on either stream).
		    **
		    ** This code needs more thought.
		    ** In particular, we ought to keep track of how
		    ** fast the sequence number is moving forward
		    ** and if we have packets, and if we keep collecting
		    ** packets, we may wanna move the sequence number forward.
		    */
		}

		/*
		** Deal with packet loss ...
		**
		** There are two different kinda issues here, one
		** is true packet loss, which everybody has some
		** understanding about, and sequence numbers which
		** we assumed were correct, but aren't.
		**
		** WE can (and do) get connections where we never
		** saw the SYN/SYN-ACK.  We record the sequence number
		** of the session info we saw, and "pretend" thats
		** the SYN/SYN-ACK of the end to end connection.
		** ONLY thing is, we started seeing this stream
		** in the "middle" of the conversation, and may have
		** MISSED a previous packed with MORE data, which
		** COULD move the sequence number HIGHER than the
		** one we're syncing on....  For that reason,
		** we're more lenient with sequence nubmers on
		** connections which we never saw SYN/SYN-ACK's on.
		**
		** oh, btw.  Leave alone connections which are shutting down.
		*/
		{
		    int dir;
		    for (dir = 0; dir < 2; dir ++ ) {
			if ((ta->ta_stream[dir].ta_Packets) 
			&&  ((ta->ta_flags & (TH_RST|TH_FIN)) == 0) )  {
			    double stdDv = ta->ta_stream[dir].ta_ArrStdDv;
			    double rate = 10 * 600; /* kinda default timeout XXX */
			    if (stdDv && ta->ta_stream[dir].ta_Cnt) {
				rate = (util_tv2usec( &ta->ta_stream[dir].ta_ArrDiff ) /
					    ta->ta_stream[dir].ta_Cnt) / MILLION;
			    }

			    if (stdDv) {
				char * reason = "possible packet loss";
				tcpData_t * td =
				    tcpLowSeq( ta->ta_stream[dir].ta_Packets);

				if (((ta->ta_flags & (TA_FLAGS_ORIG_SRC|TA_FLAGS_ORIG_DST)) == 0)
				&& (ta->ta_stream[dir].ta_SeqGc == 0)) {
				    /*
				    ** we never saw the SYN/SYN-ACK.
				    ** do we have packets lying around?
				    */
				    stdDv *= 2; /* two standard deviations */
				    reason = "missed SYN/SYN-ACK";
				} else {
				    stdDv *= 12; /* lets be patient */
				}

				if ((stdDv > 0) && (td)
				&& ((td->td_sec + rate + stdDv) < now)) {
				    /*
				    ** find the lowest sequence number, display
				    ** the "hole" we found.
				    */
				    int diff = td->td_seq - ta->ta_stream[dir].ta_SeqNo;

				    if (diff > 0) {
					/* bleech. bad hack */
					ta->ta_stream[dir].ta_holeCnt++;

					log( FL, "%s: %d: Skipping sequence numbers %s %s %s cnt %d\n",
					    reason, ta->ta_stream[dir].ta_holeCnt,
					    format( "%d", diff ),
					    format( "%u", ta->ta_stream[dir].ta_SeqNo ),
					    format( "%u (hash %d direction %s)", td->td_seq, ta->ta_hash, direction[dir] ),
						tcpPacketCnt( ta->ta_stream[dir].ta_Packets)
					    );
					/* tcpAssembleStats( ta, 0, 0 ); */
					/*Alert_Args_Generate( NFR_SRC_NFRD,
					    NFR_NFRD_TCPSKIPSEQNO,
					    format( "%d", diff ),
					    format( "%d", ta->ta_stream[dir].ta_SeqNo ),
					    format( "%d (hash %d direction %s)", td->td_seq, ta->ta_hash, direction[dir] ), 0 ); */

					ta->ta_stream[dir].ta_SeqGc = time(0);
					ta->ta_stream[dir].ta_SeqNo = td->td_seq;

					/*
					** XXX We need to tell the tcp layer
					** what packet we're working on...
					*/
					packetAlertsOff();
					tcpReassemble( ta ); /* :^( how sad */
					packetAlertsOn();
					/*
					** how many packets are left now
					*/
					log( FL, "skipped packets left: %d\n",
					    tcpPacketCnt( ta->ta_stream[dir].ta_Packets) );
				    }
				}
			    }
			}
		    }
		}
		/*
		** ok, we've got another test to make.
		** If there's a zero count in one direction, but not in the
		** other, and if we havn't received any data in a long time,
		** from the one direction, then that means the sender gave
		** up... no ACK ever received on the connection...
		*/
		{
		    if ( ((ta->ta_forwCnt == 0) && (ta->ta_backCnt != 0))
		    ||   ((ta->ta_forwCnt != 0) && (ta->ta_backCnt == 0)) ) {
			/* candidate */
			if  ((ta->ta_time + 600) < now) {
			    /* no traffic for ten minutes... */
			    /* no acks either... one data in one direction */
			    goto GarbageCollect;
			}
		    }
		}


		q = q->q_forw;
	    } while (q != tcpsList);
	}
    }
}

/*
**
*/

tcpReassembleStats()
{
	uiCurrPrintf( "tcp: tcpDataBytes %d\n", tcpDataBytes );
	uiCurrPrintf( "tcp: tcpDataCnt %d\n", tcpDataCnt );
	uiCurrPrintf( "tcp: tcpDataDel %d\n", tcpDataDel );
	uiCurrPrintf( "tcp: tcpPatternBytes %d\n", tcpPatternBytes );
	uiCurrPrintf( "tcp: tcpPatternCnt %d\n", tcpPatternCnt );
	uiCurrPrintf( "tcp: tcpMatchCnt %d\n", tcpMatchCnt );
	uiCurrPrintf( "tcp: tcpAssembleCnt %d\n", tcpAssembleCnt );
	uiCurrPrintf( "tcp: tcpForwZero %d\n", tcpForwZero );
	uiCurrPrintf( "tcp: tcpBackZero %d\n", tcpBackZero );
	uiCurrPrintf( "tcp: tcpForwLost %d\n", tcpForwLost );
	uiCurrPrintf( "tcp: tcpBackLost %d\n", tcpBackLost );
	/*
	** Give some more details about the tcpAssemble lists...
	*/
	if (tcpsList) {
	    que_t * q = tcpsList;
	    int tcpDCnt[1] = { 0 };
	    int tcpDBytes[1] = { 0 };
	    int tcpCnt = 0;

	    tcpDCnt[0] = 0;
	    tcpDBytes[0] = 0;
	    uiCurrPrintf( "taList ==================== nonce %d ====================\n", 
		packet_nonce );

	    do {
		uiCurrPrintf( "\n" );
		tcpAssembleStats( (tcpAssemble_t *)q->q_data, tcpDCnt, tcpDBytes  );
		q = q->q_forw;
		tcpCnt++;
	    } while (q != tcpsList);

	    uiCurrPrintf( "tcp: Counted tcpDCnt %d\n", tcpDCnt[0] );
	    uiCurrPrintf( "tcp: Counted tcpDBytes %d\n", tcpDBytes[0] );
	    uiCurrPrintf( "tcp: tcpDataCnt %d [%d]\n", tcpDataCnt, tcpDataCnt-tcpDCnt[0] );
	    uiCurrPrintf( "tcp: tcpDataBytes %d [%d]\n", tcpDataBytes, tcpDataBytes-tcpDBytes[0] );
	    uiCurrPrintf( "tcp: tcpsListForw %d\n", tcpsListForw );
	    uiCurrPrintf( "taList ==================== end-of-data ====================\n" );
	}
}

/*
** tcpReassembleMem
**
*/

int
tcpReassembleMem()
{
    int total = 0;
    int cnt = 0;

    if (tcpsList) {
	que_t * q = tcpsList;
	do {
	    total += tcpAssembleMem( (tcpAssemble_t *)q->q_data );
	    cnt++;
	    q = q->q_forw;
	} while (q != tcpsList);
    }
    uiCurrPrintf( "tcpReassembleMem: cnt %d total %d\n", cnt, total );
}

/*
** reassembleStats.
*/

reassembleStats()
{
    ipAssembleStats();
    tcpReassembleStats();
}


/*
** -------------------------------------------------------
**
** tcpStream. tcpBytes.  tcpBlob.  tcpConn details...
**
** nfr code interface to collect data made available from reassembly.
** Note that we keep info about the data found in globals,
** moving to a radically different programming model would be hard.
*/

/*  This moved up the file to allow earlier references:
tcpAssemble_t *	tcpConnTa;
*/

char *		tcpStreamData = 0;
int		tcpStreamLen = 0;

tcpMatch_t 	*tcpConnMatch;

/*
** ------------------------------
** tcpStreamSet.
**
** tcpStreamSet is used to mark the current stream data, and
** current match structures, to allow the .nfr code to fetch
** info (like tcp.connHash, or tcp.connSave)
*/

void
tcpStreamSet( ta, d, l, m )
tcpAssemble_t * ta;
char * d;
int l;
tcpMatch_t * m;
{
    tcpConnTa = ta;
    tcpStreamData = d;
    tcpStreamLen = l;

    tcpConnMatch = m;
}

/*
** ------------------------------
** tcpBlob.
**
** Return the blob associated with the bytes
** of some tcp packet.
*/

blob_t tcpBlobVar[1];

int
tcpBlob( m )
MemVal * m;
{
    if (tcpStreamData && tcpStreamLen) {
	m->typ = NFR_TYPE_BLOB;
	m->val.b = blobSet( tcpBlobVar,
		tcpStreamData, tcpStreamLen, 1, BLOB_FLAGS_STATIC );
	return(0);
    } else {
	return(1);
    }
}

/*
** ------------------------------
** tcpBytes.
**
** tcpBytes is poorly implemented.
** it uses a local string, which it keeps hanging around.
** It'd be better if something understood to free this
** some later time.
*/

int
tcpBytes( m )
MemVal * m;
{
    static char * tcpBytesData = 0;	/* not good for threads :( */
    static int tcpBytesLen = 0;

    if (tcpStreamData && tcpStreamLen) {
	m->typ = NFR_TYPE_STR;
	if (tcpBytesLen < (tcpStreamLen+1)) {
	    if (tcpBytesLen) {
		free(tcpBytesData);
	    }
	    tcpBytesData = (char *)malloc( tcpStreamLen+1 );
	    tcpBytesLen = tcpStreamLen+1;
	}
	m->val.c = tcpBytesData;
	memcpy( tcpBytesData, tcpStreamData, tcpStreamLen );
	tcpBytesData[tcpStreamLen] = 0;
	return(0);
    } else {
	return(1);
    }
}

/*
** ------------------------------
** tcpStream.
**
** Another name for tcpBytes...
*/

int
tcpStream( m )
MemVal * m;
{
    if (tcpStreamData && tcpStreamLen) {
	tcpBytes(m);
	return(0);
    } else {
	return(1);
    }
}

/*
** ------------------------------
** tcpLength.
**
** question: is it possible for the data to be delivered to have
** zero length? Does it matter?
*/

int
tcpLength( m )
MemVal * m;
{
    if (tcpStreamData) {
	m->typ = NFR_TYPE_INT;
	m->val.i = tcpStreamLen;
	return(0);
    } else {
	return(1);
    }
}

/*
** ------------------------------
** tcpConn.
**
** returns the memory location of the
** structure which represents the connection.
** Unique while the connects exists.
**
*/

tcpConn( m )
MemVal * m;
{
    m->typ = NFR_TYPE_INT;
    m->val.i = (int)tcpConnTa;		/* we just want a non-negative number */
    return(0);
}

/*
** ------------------------------
** tcpConnEpoch.
**
** time (sec) when the connection was noticed.
*/

tcpConnEpoch( m )
MemVal * m;
{
    if (tcpConnTa) {
	m->typ = NFR_TYPE_INT;		/* ought to be marked as time */
	m->val.i = tcpConnTa->ta_epoch;
	return(0);
    }
    return(1);
}

/*
** ------------------------------
** tcpConnPktSec.
**
** time the last packed arrived into this connection.
** either direction (forw/back), in seconds.
*/

tcpConnPktSec( m )
MemVal * m;
{
    if (tcpConnTa) {
	m->typ = NFR_TYPE_INT;		/* ought to be marked as time */
	m->val.i = tcpConnTa->ta_time;
	return(0);
    }
    return(1);
}

/*
** ------------------------------
** tcpConnHash
**
** this is intended to be a unique value
** for the duration of the operation of the
** engine.  We can make it more unique
** (over the lifetime of NFR) with some
** more work, but that's all we were asked
** for right now.  
*/

tcpConnHash( m )
MemVal * m;
{
    if (tcpConnTa) {
	m->typ = NFR_TYPE_INT;
	m->val.i = tcpConnTa->ta_hash;
	return(0);
    }
    return(1);
}

/*
** ------------------------------
** tcpConnSrc.
**
** the source of the connection.  Best
** described as one of the two ends, 
** associated with the requestor when
** we know who issued the SYN.
** When (tcp.connSyn-tcp.connSynAck) is zero,
** then this could be either connection.
*/

tcpConnSrc( m )
MemVal * m;
{
    if (tcpConnTa) {
	m->typ = NFR_TYPE_IPV4HOST;
	if (tcpConnTa->ta_flags & TA_FLAGS_FLIP) {
	    m->val.h = tcpConnTa->ta_dst;
	} else {
	    m->val.h = tcpConnTa->ta_src;
	}
	return(0);
    }
    return(1);
}

/*
** ------------------------------
** tcpConnDst.
**
** See the note under tcpConnSrc.
*/

tcpConnDst( m )
MemVal * m;
{
    if (tcpConnTa) {
	m->typ = NFR_TYPE_IPV4HOST;
	if (tcpConnTa->ta_flags & TA_FLAGS_FLIP) {
	    m->val.h = tcpConnTa->ta_src;
	} else {
	    m->val.h = tcpConnTa->ta_dst;
	}
	return(0);
    }
    return(1);
}

/*
** ------------------------------
**
** tcpConnSport.
**
** Source port.  See the note under tcpConnSrc.
*/

tcpConnSport( m )
MemVal * m;
{
    if (tcpConnTa) {
	m->typ = NFR_TYPE_INT;
	if (tcpConnTa->ta_flags & TA_FLAGS_FLIP) {
	    m->val.i = tcpConnTa->ta_dport;
	} else {
	    m->val.i = tcpConnTa->ta_sport;
	}
	return(0);
    }
    return(1);
}

/*
** ------------------------------
** tcpConnDport
**
** Destination port.  See the note under tcpConnSrc.
*/

tcpConnDport( m )
MemVal * m;
{
    if (tcpConnTa) {
	m->typ = NFR_TYPE_INT;
	if (tcpConnTa->ta_flags & TA_FLAGS_FLIP) {
	    m->val.i = tcpConnTa->ta_sport;
	} else {
	    m->val.i = tcpConnTa->ta_dport;
	}
	return(0);
    }
    return(1);
}

/*
** ------------------------------
** tcpConnForwBytes.
**
** Number of bytes xmitted from...
*/

tcpConnForwBytes( m )
MemVal * m;
{
    if (tcpConnTa) {
	m->typ = NFR_TYPE_INT;
	if (tcpConnTa->ta_flags & TA_FLAGS_FLIP) {
	    m->val.i = tcpConnTa->ta_backBytes;
	} else {
	    m->val.i = tcpConnTa->ta_forwBytes;
	}
	return(0);
    }
    return(1);
}

/*
** ------------------------------
** tcpConnBackBytes.
**
** number of bytes xmitted from the other side.
*/

tcpConnBackBytes( m )
MemVal * m;
{
    if (tcpConnTa) {
	m->typ = NFR_TYPE_INT;
	if (tcpConnTa->ta_flags & TA_FLAGS_FLIP) {
	    m->val.i = tcpConnTa->ta_forwBytes;
	} else {
	    m->val.i = tcpConnTa->ta_backBytes;
	}
	return(0);
    }
    return(1);
}

/*
** ------------------------------
** tcpConnForwLost.
**
** number of bytes retransmitted in the 'forw' direction.
*/

tcpConnForwLost( m )
MemVal * m;
{
    if (tcpConnTa) {
	m->typ = NFR_TYPE_INT;
	if (tcpConnTa->ta_flags & TA_FLAGS_FLIP) {
	    m->val.i = tcpConnTa->ta_backLost;
	} else {
	    m->val.i = tcpConnTa->ta_forwLost;
	}
	return(0);
    }
    return(1);
}

/*
** ------------------------------
** tcpConnBackLost.
**
** retransmitted characters in the 'back'ward direction.
**
*/

tcpConnBackLost( m )
MemVal * m;
{
    if (tcpConnTa) {
	m->typ = NFR_TYPE_INT;
	if (tcpConnTa->ta_flags & TA_FLAGS_FLIP) {
	    m->val.i = tcpConnTa->ta_forwLost;
	} else {
	    m->val.i = tcpConnTa->ta_backLost;
	}
	return(0);
    }
    return(1);
}

/*
** ------------------------------
** tcpConnSave.
** saved characters from match or start-stop sequences.
**
*/

tcpConnSave( m )
MemVal * m;
{
    if (tcpConnMatch) {
	if (tcpConnMatch->tm_save) {
	    m->typ = NFR_TYPE_STR;
	    m->val.c = tcpConnMatch->tm_save;
	    return(0);
	}
    }
    return(1);
}


/*
** ------------------------------
** tcpConnSyn
** Number of SYN's seen on this connection.
**
*/

tcpConnSyn( m )
MemVal * m;
{
    if (tcpConnTa) {
	m->typ = NFR_TYPE_INT;
	m->val.i = tcpConnTa->ta_syn;
	return(0);
    }
    return(1);
}


/*
** ------------------------------
** tcpConnSynAck.
** Number of SYN/ACK's seen on this connection.
**
*/

tcpConnSynAck( m )
MemVal * m;
{
    if (tcpConnTa) {
	m->typ = NFR_TYPE_INT;
	m->val.i = tcpConnTa->ta_synAck;
	return(0);
    }
    return(1);
}

/*
** ------------------------------
** tcpConnRst.
** Number of RST's seen on this connection.
**
*/

tcpConnRst( m )
MemVal * m;
{
    if (tcpConnTa) {
	m->typ = NFR_TYPE_INT;
	m->val.i = tcpConnTa->ta_rst;
	return(0);
    }
    return(1);
}

/*
** ------------------------------
** tcpConnFin
** Number of FIN's seen on this connection.
**
*/

tcpConnFin( m )
MemVal * m;
{
    if (tcpConnTa) {
	m->typ = NFR_TYPE_INT;
	m->val.i = tcpConnTa->ta_fin;
	return(0);
    }
    return(1);
}

/*
** ------------------------------
** tcpConnMap.
**
** return the whole map bag.
*/

tcpConnMap( m )
MemVal * m;
{
    if (tcpConnTa) {
	m->typ = NFR_TYPE_MAPBAG;
	/* width of zero, passed in means mapBag picks the size */
	if (tcpConnTa->ta_map == 0) {
	    tcpConnTa->ta_map = mapBagNew( 0 );
	}
	m->val.v = tcpConnTa->ta_map;
	mapBagIncRef(tcpConnTa->ta_map); /* cause our caller will use it */
	tmpCallOnFree( mapBagDecRef, m->val.v );
	return(0);
    }
    return(1);
}

/*
** ------------------------------
** tcpConnSym.
**
** This is an abbreviation for symofmap( tcpConnMap, index );
** Where the index is scope index.  That means the variables
** will be shared by everything withing the same scope.
*/

tcpConnSym( m )
MemVal * m;
{
    if (tcpConnTa) {
	m->typ = NFR_TYPE_MAP;
	if (tcpConnTa->ta_map == 0) {
	    tcpConnTa->ta_map = mapBagNew( 0 );
	}
	m->val.v = mapBagFind( tcpConnTa->ta_map,
		runCurrentGet()->f_scopeNdx, runCurrentGet()->f_scope );
	if (m->val.v == 0) {
	    /* create one */
	    Map * map = mapNew( 0,
		runCurrentGet()->f_scopeNdx, runCurrentGet()->f_scope, 0 );
	    mapBagAdd( tcpConnTa->ta_map, map );
	    tmpCallOnFree( mapDecRef, m->val.v );
	    m->val.v = map;
	    mapIncRef(map);	/* cause our caller will use it. */
	}
	return(0);
    }
    return(1);
}

/*
** ipInit.
**
** The void parameter is really a timeout_t, but
** its hard to get the recursive delcaration correct.
*/

void
ipInit( )
{
    timeval_t when = util_makeTv( /* sec */ 2, /* usec */ 0 );
    to_t * to = toNew( &when, TO_FLAGS_SPECIAL | TO_FLAGS_REPEAT, 0, ipReassembleGC );
}
