/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/

#include <stdio.h>

#include "nfrd.h"

char * argv0;

int cFlag = 0;
int eFlag = 0;
int iFlag = 0;
int lFlag = -1;
int sFlag = 0;
int vFlag = 0;
int VFlag = 0;
int nFlag = 0;
int pFlag = 0;
int rFlag = 0;

struct pcap_stat ps;

/*
** trivial little command to test an interface to make sure
** we can open it.  Usefull to verify that the interface opens
** without having to run nfrd.
*/

time_t	totalStart;
int	totalPackets;
counter	totalBytes;		/* maybe this ought to be a long long? */
counter	totalCapture;		/* maybe this ought to be a long long? */

time_t	lastPrint;
int	lastPackets;
counter	lastBytes;
counter	lastCapture;

/*
** this could be very fancy, but not for right now.
*/
#define	RA_NUM		2048
int	readArray[RA_NUM];
int	readArrayI;


/*
**
*/
someStats( pcap_t * p )
{
    lastPrint = time(0);
    {
	char cbuf[COUNTER_TEXT_SIZE];
	char dbuf[COUNTER_TEXT_SIZE];
	char ebuf[COUNTER_TEXT_SIZE];
	char fbuf[COUNTER_TEXT_SIZE];
	int diff = lastPrint - totalStart;

	counter_itoa( cbuf, totalBytes );
	counter_itoa( dbuf, lastBytes );
	counter_itoa( ebuf, totalCapture );
	counter_itoa( fbuf, lastCapture );
	if (eFlag) {
	    printf( "\n" );
	}
	printf( "packets: %d (%d)\tchars: %s (%s)\tcap: %s (%s) s: %d\n",
	    totalPackets, lastPackets, cbuf, dbuf, ebuf, fbuf, diff );
	/*
	** display pcap stats.
	*/
	if (pFlag) {
	    struct pcap_stat l;
	    pcap_stats( p, &l );
	    printf( "Pcap packets %d (%d) drops %d (%d)\n",
		l.ps_recv, l.ps_recv-ps.ps_recv,
		l.ps_drop, l.ps_drop-ps.ps_drop );
	    ps = l;
	}

	/*
	** display read stats.
	*/
	if (readArrayI) {
	    int i;
	    int reCnt = 0;
	    if (rFlag) {
		if (readArrayI == RA_NUM) {
		    printf( "Reads: possible overflow (try smaller -i) %d: ", readArrayI );
		} else {
		    printf( "Reads: %d: ", readArrayI );
		}
		for (i=0; i<readArrayI; ++i) {
		    printf( "%d ", readArray[i] );
		    reCnt += readArray[i];
		}
		printf( "-> %d\n", reCnt );
	    }
	    readArrayI = 0;
	}

	lastBytes = counter_from_int(0);
	lastCapture = counter_from_int(0);
	lastPackets = 0;
    }
}

/*
**
*/
done( pcap_t * ps ) {
    pcap_close( ps );
    /*
    ** print stats.
    */
    someStats( ps );
    exit(0);
}


/*
** called for each packet read.
*/

void
performRead( u_char * v, const struct pcap_pkthdr * pkt, const u_char * pk )
{
    pcap_t * p = (pcap_t *)v;
    pcap_stats( p, &ps );

    if (eFlag) {
	if (vFlag) {
	    fprintf(stderr, "%d ", pkt->len );
	} else {
	    fprintf(stderr, ".");
	}
    }
    /*
    ** keep track of the total packets read, and the total
    ** bytes read.
    */
    totalPackets++;
    totalBytes = counter_add( totalBytes, counter_from_int(pkt->len) );
    lastPackets++;
    lastBytes = counter_add( lastBytes, counter_from_int(pkt->len) );

    totalCapture = counter_add( totalCapture, counter_from_int(pkt->caplen) );
    lastCapture = counter_add( lastCapture, counter_from_int(pkt->caplen) );


    if (cFlag && (cFlag <= totalPackets)) {
	done( p );
    }
    if (nFlag && (counter_cmp( totalBytes, counter_from_int(nFlag) ) > 0)) {
	done( p );
    }
    if (iFlag && (lastPrint+iFlag <= time(0))) {
	someStats( p );
    }
}

/*
**
*/
pcapTest( pcap_t * p )
{
    int n;

    totalStart = time(0);
    lastPrint = time(0);

    totalBytes = counter_from_int(0);

    for (;;) {
	n = pcap_dispatch( p, -1, performRead, (u_char *)p );
	if (readArrayI < RA_NUM) {
	    readArray[readArrayI++] = n;
	}
	if (VFlag) {
	    fprintf( stderr, "dispatch: %d\n", n );
	}
	/*
	** see if we exit early cause of nsec... sFlag.
	*/
	if (sFlag) {
	    if ((sFlag+totalStart) < time(0)) {
		done(p);
	    }
	}

    }
}

char * intf;

pcap_t * p;

main( argc, argv )
int argc;
char * argv[];
{

    argv0 = argv[0];

    {
	int i;
	for (i=1; i<argc; ++i) {
	    if (argv[i][0] == '-') {
		switch(argv[i][1]) {
		    case 'c':	/* # of packets to read */
			if (argv[i][2]) {
			    cFlag = atoi(&argv[i][2]);
			} else {
			    cFlag = atoi(argv[++i]);
			}
			break;

		    case 'e':	/* every packet split out a dot. */
			eFlag++;
			break;

		    case 'i':	/* interval... spit out data every n seconds */
			if (argv[i][2]) {
			    iFlag = atoi(&argv[i][2]);
			} else {
			    iFlag = atoi(argv[++i]);
			}
			break;

		    case 'l':	/* length of packets to read */
			if (argv[i][2]) {
			    lFlag = atoi(&argv[i][2]);
			} else {
			    lFlag = atoi(argv[++i]);
			}
			break;

		    case 'n':	/* total bytes to read */
			if (argv[i][2]) {
			    nFlag = atoi(&argv[i][2]);
			} else {
			    nFlag = atoi(argv[++i]);
			}
			break;

		    case 'p':	/* display pcap stats */
			pFlag++;
			break;

		    case 'r': /* read information */
			rFlag++;
			break;

		    case 's': /* number of seconds to read */
			if (argv[i][2]) {
			    sFlag = atoi(&argv[i][2]);
			} else {
			    sFlag = atoi(argv[++i]);
			}
			break;

		    case 'v': /* little verbose */
			vFlag++;
			break;

		    case 'V': /* big verbose */
			VFlag++;
			break;

		    default:
			break;
		}
	    } else {
		intf = argv[i];
	    }
	}
    }

    if (intf == 0) {
	usage:
	fprintf( stderr, "usage: test-intf [ -e ] [ -v ] [ -V ] [ -p ] [ -r ]\n\t[ -s nsec ] [ -c cnt ] [ -i nsec ] [ -l len ] [ -n cnt ] intf\n" );
	fprintf( stderr, "Where:\n" );
	fprintf( stderr, "\t-e print something every time you get a packet\n" );
	fprintf( stderr, "\t-v a bit verbose, with -e print packet length every time you get a packet\n" );
	fprintf( stderr, "\t-V more verbose\n" );
	fprintf( stderr, "\t-p display pcap stats\n" );
	fprintf( stderr, "\t-r display reads/readCnt data.\n" );
	fprintf( stderr, "\t-s nsec: sniff for nseconds, then exit\n" );
	fprintf( stderr, "\t-c cnt: sniff for cnt packet, then exit\n" );
	fprintf( stderr, "\t-i nsec: every nsec, print a stats message\n" );
	fprintf( stderr, "\t-l len: sniff only <len> chars per packet\n" );
	fprintf( stderr, "\t-n len: exit when you read <len> characters\n" );
	exit(0);
    }

    {
	char error[1024];
	p = pcap_open_live( intf,
		/* man len (-1 == complete packet) */ lFlag,
		/* promisc */           1,
		/* read timeout */      100,
		/* error buffer */      error );
	if (p == 0) {
	    printf( "pcap_open_live: intf %s error %s\n", intf, error );
	    exit(1);
	} else {
	    if (lFlag > -1) {
		printf( "successfully opened %s: snap len: %d \n", intf, lFlag );
	    }
	    else {
		printf( "successfully opened %s: read full packets\n", intf );
	    }
	}
    }
    pcapTest( p );

    exit(0);
}
