/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/


#include "nfrd.h"

/*
** test-nfrd
** ------------------------------------------------------------
**
** The syntax testing arm of the engine...
** attempts to generate syntax errors in great detail.
** uses input interfaces of parse.y.
** possibly over-complex input data management.
**
*/

/*
** necessry, since we include nfrd, but unused.
*/

char *           	packet_data = 0;
int              	packet_len = 0;
struct pcap_pkthdr *	packet_hdr = 0;
int			packet_nonce = 1;

char *			varDir;		/* this ought to be fixed */

/*
** all-about-town variables.
*/

Machine	*m;

char * argv0;

int vFlag;
int dFlag;
int hFlag;
int cFlag;

char * hPrefix = "syntax.html#";	/* html prefix. */

int highestError = 0;


/*
**
** we need to implelment a list of blocks which describes the input.
** We only need this do deal with the syntax errors of `stdin`, but
** since its needed there, we'll do it the right way.
**
** [If we didn't read from stdin, we could just malloc the size of
** the file, and stamp the file inta memory, then look for syntax errors]
**
** We use list of blob's, since we have that stuff lying around anyway
** and they meet our needs very closely...
*/

que_t * input;
#define	QUE2BLOB(q)	((blob_t *)((q)->q_data))

/*
** This routine cracks a blob into two, at the cp passed in
** (which must be inside `blob_t *b', and returns the new
** blob_t *... which we can then do an intert onto.
*/

blob_t *
blobInsert( blob_t * b, char * cp )
{
    return(0);
}

/*
** Return the que_t with the cp imbedded in the blob of the que.
** The que_t passed in is a list of blob_t's.
*/
que_t *
blobFind( que_t * q, char * cp )
{
    que_t * p = q;
    do {
	blob_t * b = (blob_t *)p->q_data;
	if (cp >= b->b_data && (cp < &b->b_data[b->b_len])) {
	    return(p);
	}
	p = p->q_forw;
    } while (p != q);

    return(0);
}

/*
** ------------------------------------------------------------
**
** inputState tracks the input stream... its passed around instead
** of a FILE* to some of the input routines ...
*/

typedef struct inputState {
    char *	last;
    que_t *	first;
    char *	error;
} inputState_t;

int thisLineno = 1;

/*
** This ungetc is VERY inefficient, but since its not part of the
** mainline, and is used primarily for testing the
** syntax of .nfr code, i'm not sure that's important.
*/


int
thisGetc( void * rock )
{
    inputState_t * is = (inputState_t *)rock;
    que_t * q = blobFind(is->first, is->last);
    blob_t * b;
    if (q) {
	b = QUE2BLOB(q);
	{
	    int ndx = is->last - b->b_data;

	    if (ndx < b->b_len) {
		int c = b->b_data[ndx++];
		is->last ++;
		if (ndx >= b->b_len) {
		    if (q->q_forw && (q != is->first->q_back)) {
			q = q->q_forw;
			b = QUE2BLOB(q);
			is->last = b->b_data;
		    }
		}
		return(c);
	    } 
	}
    }
    return(-1);
}

/*
** See if there's a previous character.
** if so, then see if its the *same* character.
** We don't need to do this, but it seems wise,
** just cause we're not supposed to ungetc any character we
** didn't getc (at least in the current lex-er)
*/
thisUngetc( int c, void * rock )
{
    inputState_t * is = (inputState_t *)rock;
    que_t * q = blobFind(is->first, is->last);
    blob_t * b = QUE2BLOB(q);

    int ndx = is->last - b->b_data;

    ndx--;
    if (ndx >= 0) {
	if (c == b->b_data[ndx]) {
	    is->last --;
	    return(b->b_data[ndx]);
	} else {
	    /*
	    ** we can't overwrite the internal buffer,
	    ** but we don't wanna worry about cracking the
	    ** current blob into pieces, and injecting 
	    ** this unget'c char
	    */
	    panic( "test-main: thisUngetc\n" );
	}
    } 
    if (q->q_back && (is->first != q)) {
	/*
	** test to see if the last character in the
	** previous blob matches this one before
	** moving everything back.
	*/
	que_t * prev = q->q_back;
	blob_t * pb = QUE2BLOB(prev);

	if (pb->b_data[pb->b_len-1] == c) {
	    q = q->q_back;
	    b = QUE2BLOB(q);
	    is->last = &b->b_data[b->b_len-1];
	    ndx = is->last - b->b_data;
	    return(b->b_data[ndx]);
	}
    }
    /*
    ** otherwise add another entry in front of q.
    */
    {
	char d = c;
	blob_t * b = blobNew( &d, 1, 0 );
	que_t * q = queNew( b );

	if (is->first) {
	    queAdd( is->first->q_back, q );
	}
	is->first = q;
    }
    return(0);
}

long
thisFtell( void * rock )
{
    inputState_t * is = (inputState_t *)rock;
    return( (long)is->last );
}

int
thisFeof( void * rock )
{
    inputState_t * is = (inputState_t *)rock;
    que_t * q = blobFind(is->first, is->last);
    if (q) {
	blob_t * b = QUE2BLOB(q);

	int ndx = is->last - b->b_data;
	if ((ndx > b->b_len) && (q != is->first->q_back)) {
	    return(1);
	}
    }
    return(0);
}


/*
** --------------------------------------------------
**
**
*/

inputState_t is;
char * inputName;

testNfrd( f )
FILE * f;
{

    is.last = 0;
    is.first = 0;

    if (m == 0) {
	m = machinenew();
    }
    yy_nfrd_linenoReset();
    /*
    ** First read till we get an end-of-file.
    ** Use a blob's to collect the characters together.
    */

    /*
    ** Read the input, creating a list of blobs.
    */
    {
	int	readVal;
	char	buff[2048];

	while( (readVal = fread( buff, 1, sizeof(buff), f )) > 1 ) {
	    blob_t * b  = blobNew( buff, readVal, 0 );
	    que_t * q = queNew(b);
	    if (is.first) {
		queAdd( is.first->q_back, q );
	    } else {
		is.first = q;
	    }
	}
    }
    /*
    ** Set up the input to read the blob data.
    */
    setyy_nfrd_input( (FILE *)&is );	/* XXX ought to be void */
    yy_nfrd_GetcSet( thisGetc );
    yy_nfrd_UngetcSet( thisUngetc );
    yy_nfrd_FtellSet( thisFtell );
    yy_nfrd_FeofSet( thisFeof );

    is.last = (QUE2BLOB(is.first))->b_data;

    parseReset();

    /*
    ** Parse/execute it.
    **
    ** The parse knows not to execute if there are any syntax errors,
    ** BUT it doesn't know if there will be any syntax errors in the
    ** future stream, so it executes code as it finds it.
    **
    */
    while(1) {
	int retVal;

	if((retVal = yy_nfrd_parse()) == NFR_COMPILE_EOF) {
	    break;
	}
	if (vFlag) {
	    log( FL, "testNfrd: Parse Return: %d\n", retVal );
	}
	if (retVal > highestError) {
	    highestError = retVal;
	}

    }

    /*
    ** If there were any errors, the syntax errors are displayed.
    ** But we need to splat the rest of the data.
    */
    if (parseOk() == 0) {
	yy_nfrd_error( 0 );
	if (hFlag) {
	    printf( "</pre>\n" );
	}
    }
    /*
    ** cleanup.  Get rid of da blobs.
    */
    do {
	blob_t * b = QUE2BLOB(is.first);
	free(b->b_data);
	free(b);
	queFree(is.first);
    } while(is.first->q_forw != is.first->q_back);

    is.first = 0;
    is.last = 0;
    is.error = 0;
}

/*
** ------------------------------------------------------------
*/

/*
** Display each character...
** AND see if we can print the message we've been asked to include in
** the stream... sigh.
*/
int
inerror( char * message, int didMsg, int inErr, int more, char c )
{
    extern int yy_nfrd_lineno;



    if (c == '\n') {
	/* new line. */
	thisLineno++;
	if (inErr && hFlag) {
	    printf( "</strong></u>\n" );
	} else {
	    printf( "\n" );
	}
	if (yy_nfrd_lineno == thisLineno) {
	    if (message && (didMsg == 0)) {
		if (hFlag) {
		    printf( "<strong><u>Error: %s</strong></u>\n", message );
		} else {
		    printf( "Error: %s\n", message );
		}
		didMsg = 1;
	    }
	}
	if (more) {
	    printf( "%5d: ", thisLineno );
	    if (inErr && hFlag) {
		printf( "<strong><u>" );
	    }
	}
    } else {
	if (hFlag) {
	    /*
	    ** If we're generating html, we have to quote some characters.
	    */
	    switch(c) {
		case '<':
		    printf( "&lt" );
		    break;

		case '>':
		    printf( "&gt" );
		    break;

		case '&':
		    printf( "&amp" );
		    break;

		case '"':
		    printf( "&quot" );
		    break;

		default:
		    printf( "%c", c );
		    break;
	    }
	} else {
	    printf( "%c", c );
	}
    }
    return(didMsg);
}

/*
**
** splat out the stuff we've seen so far, then
** display the token we've seen, then reset the
** start-of-display in case of other errors.
*/


int
yy_nfrd_error( char * message )
{
    int didMsg = 0;
    char * msg = message;
    extern int yy_nfrd_lineno;
    char * href = "syntax_error";
    int hrefLen = strlen(href);

    if (parseOk()) {
	if (hFlag) {
	    printf( "<p>Parse Errors in input <strong>\"%s\".</strong> "
		"Region bolded and underscored\n<pre>", inputName);
	}
    }
    parseFailed();
    if (msg) {
	char * h;
	if ((h = strchr( msg, ':' )) != 0) {
	    href = msg;
	    hrefLen = h-msg;
	    msg = &h[1];
	}
	fprintf( stderr, "line: %d, %s\n", yy_nfrd_lineno, msg);
    } else {
	/* flushing */
    }

    /*
    ** Print starting at is.error, up to the character in
    ** the blob representing just before the last token read.
    ** Then reset the error pointer.
    */
    {
	que_t * s;
	int sndx;
	que_t * e;
	int endx;
	char * lastToken = (char *)yy_nfrd_LastToken();

	if (is.error) {
	    s = blobFind( is.first, is.error );
	    if (s) {
		sndx = is.error - QUE2BLOB(s)->b_data;
	    } else {
		return;	/* no more */
	    }
	} else {
	    printf( "%5d: ", thisLineno );
	    s = is.first;
	    sndx = 0;
	}

	e = blobFind( is.first, lastToken ); 
	endx = lastToken - QUE2BLOB(e)->b_data;

	/*
	*/
	while(1) {
	    if (s == e) {
		/* print the last info in the blob and break. */
		for ( ; sndx < endx; ++sndx ) {
		    didMsg = inerror( msg, didMsg, 0,
			(endx - sndx) > 1, QUE2BLOB(s)->b_data[sndx] );
		}
		break;
	    }
	    {
		/* print the whole blob */
		int p;
		for ( p = sndx; p < QUE2BLOB(s)->b_len; ++p ) {
		    didMsg = inerror( msg, didMsg, 0, 
			(QUE2BLOB(s)->b_len - p) > 1, QUE2BLOB(s)->b_data[p] );
		}
	    }
	    s = s->q_forw;
	    sndx = 0;
	}
	/*
	** Now we print a marker indicating where the error started.
	** but only if an error message was passed in...
	*/
	if (msg) {
	    if (hFlag) {
		printf( "<strong><u><a href=\"%s%*.*s\">",
			hPrefix, hrefLen, hrefLen, href );
	    } else {
		printf( "<< " );
	    }
	}

	/*
	** Now we print from where we just stopped to where the
	** current token is...
	*/
	e = blobFind( is.first, is.last );
	if (e == 0) {
	    /*
	    ** the current message may be just `one char' behind...
	    ** After all, when we're done scanning, we ought to point
	    ** at the end of the input.
	    */
	    e = blobFind( is.first, is.last-1 );
	    if (e == 0) {
		return;
	    }
	}
	endx = is.last - QUE2BLOB(e)->b_data;
	/*
	*/
	while(1) {
	    if (s == e) {
		/* print the last info in the blob and break. */
		for ( sndx; sndx < endx; ++sndx ) {
		    didMsg = inerror( msg, didMsg, 1,
			(endx - sndx) > 1, QUE2BLOB(s)->b_data[sndx] );
		}
		break;
	    }
	    {
		/* print the whole blob */
		int p;
		for ( p = sndx; p < QUE2BLOB(s)->b_len; ++p ) {
		    didMsg = inerror( msg, didMsg, 1,
			( QUE2BLOB(s)->b_len - p) > 1, QUE2BLOB(s)->b_data[p] );
		}
	    }
	    s = s->q_forw;
	    sndx = 0;
	}
	if (msg) {
	    if (hFlag) {
		printf( "</a></u></strong>" );
	    } else {
		printf( ">> " );
	    }
	} else {
	    /* flush done */
	}

	/*
	** now we move the error info forward...
	*/
	is.error = is.last;
    }
}


int
main( argc, argv )
int argc;
char * argv[];
{

    argv0 = argv[0];

    logOpen("-");
    scopeInit();
    globalFilterInit();

    if (argc > 0) {
	int i;

	for( i = 1; i < argc ; ++i ) {
	    if (argv[i][0] == '-') {
		switch(argv[i][1]) {
		    case 0:	/* read stdin. */
			inputName = "(standard input)";
			testNfrd(stdin);
			break;

		    case 's':	/* scope */
			{
			    char * s;
			    if (argv[i][2]) {
				s = &argv[i][2];
			    } else {
				s = argv[++i];
			    }
			    s = scopeSet(strdup(s));
			    if (s) {
				free(s);
			    }
			}
			break;

		    case 'v':	/* verbose */
			vFlag++;
			break;

		    case 'd':	/* debug */
			dFlag++;
			break;

		    case 'h':	/* html errors-output */
			hFlag++;
			break;

		    case 'i':	/* info about the .nfr code, but don't run */
			runSetInfo();
			break;

		    case 'p':	/* html path prefix. */
			if (argv[i][2]) {
			    hPrefix = &argv[i][2];
			} else {
			    hPrefix = argv[++i];
			}
			break;

		    case 'c':	/* cFlag.  check syntax, don't run. */
			cFlag++;
			runSetDont();/* tell the parser not to execute code */
			break;

		    case 'r': /* runFlag. */
			runSetDebug();
			break;

		    default:
		    Usage:
			fprintf( stderr, "test-nfrd [ -v ] [ -d ] [ -p htmlPrefix ]"
				" [ -r ] [ -i ] [ -s scope ] [ files ]\n" );
			fprintf( stderr, "If no files are given, then stdin is read\n" );
			fprintf( stderr, "Using \"-\" as a file name, read stdin\n" );
			exit(1);
		}
	    } else {
		char t;
		if ((t = fileType(argv[i])) == '-') {
		    FILE * in = fopen( argv[i], "r" );
		    if (in) {
			int retVal;
			if (vFlag) {
			    fprintf( stderr, "Reading: %s\n", argv[i] );
			}
			inputName = argv[i];
			retVal = testNfrd(in);
			if (vFlag) {
			    fprintf( stderr, "Done Reading: %d <- (%s)\n", retVal, argv[i] );
			}
			
			fclose(in);
			tmpfree();
		    }
		} else {
		    switch (t) {
			case 'd':
			    /* hey, should we iterate through all of these? */
			    fprintf( stderr, "Sorry, %s is a directory\n", argv[i] );
			    break;
			case 'c':
			case 'b':
			case 's':
			case 'f':
			    fprintf( stderr, "Sorry, %s is a not a regular file\n", argv[i] );
			    break;
			default:
			    perror( argv[i] );
			    break;
		    }
		}
	    }
	}
    } else {
	testNfrd(stdin);
    }
    tmpfree();
    logClose();

    exit(highestError);
}

recvPacket()
{
}

char * 
nfrCfGetVal( name )
char * name;
{
	return 0;
}
