/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/


/*
** test-user.c
**
** Routines to support writing info back to a telnetted
** session.  Allows debugging of components...
** info is dropped when the connetion is torn down.
**
** This file contains versios of user.c intended for use
@@ with the test-nfrd.
*/

#include	"nfrd.h"

/*
**
** user session issues.
**
** we'd like to be able to associate logging output,
** parse errors, etc with a telnet session... that way
** we can forward info to that session as long as its
** connected.   When it does die, we don't have to
** relinquish resources
**
** To meet those goals, we have a Userintf_t structure,
** which describes all the info to get some data to
** a user.  We keep track of the fd's and map them to
** the Userintf_t's in a simple way...
*/

/*
** ----------------------------------------
**
** External state... describes current user interface...
*/

static Userintf_t * uiCurrent;

Userintf_t *
uiCurr() {
    return(uiCurrent);
}

Userintf_t *
uiSetCurr( Userintf_t * ui )
{
    /* XXX check to see if its on the list? */
    Userintf_t * old = uiCurrent;
    uiCurrent = ui;

    return( old );
}

Userintf_t *
uiSetTrace( Userintf_t * ui, int flag )
{
    ui->ui_trace = flag;
    return(ui);
}
 
/*
** ----------------------------------------
*/

uiPrintf( Userintf_t * ui, char * fmt, ... )
{
    {
    int rtn;
    va_list ap;
    va_start(ap, fmt);
    rtn = vfprintf( stdout, fmt, ap );
    return(rtn);
    /* XXX flush? */
    }

}

/*
** same as uiPrintf, but uses current user...
*/
uiCurrPrintf( char * fmt, ... )
{
    {
    int rtn;
    va_list ap;
    va_start(ap, fmt);

    rtn = vfprintf( stdout, fmt, ap );
    return(rtn);
    /* XXX flush? */
    }
}

Userintf_t *
uiIncr( Userintf_t * ui )
{
    return(ui);
}

Userintf_t *
uiDecr( Userintf_t * ui )
{
    return(ui);
}

Userintf_t *
uiNew( int fd )
{
    Userintf_t * ui = (Userintf_t *)calloc( sizeof(*ui), 1 );
    ui->ui_fd = fd;
    return(ui);
}

uiFree( Userintf_t * ui )
{
    free(ui);
}
