/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/

/*
** timeouts... to's
**
** Some code to support triggers for routines based on
** timeouts.   The support 
**
*/

#include	"nfrd.h"

to_t * toAll;

/*
** ----------------------------------------
** toTop.
**
** return the top timeout... we'll actually
** decrement the value on that timeout.
*/

timeval_t *
toTop()
{
    if (toAll) {
	return(&toAll->to_tv);
    } else {
	return(0);
    }
}


/*
** ----------------------------------------
** toCmp.
**
** compare two tv's ...
** toCmp(a,b) < 0 iff a < b
** toCmp(a,b) == 0 iff a == b
** toCmp(a,b) > 0 iff a > b
*/

int
toCmp( a, b )
timeval_t * a, * b;
{
    if (a->tv_sec < b->tv_sec) {
	return(-1);
    } else
    if (a->tv_sec > b->tv_sec) {
	return(1);
    } else {
	if (a->tv_usec < b->tv_usec) {
	    return(-1);
	} else
	if (a->tv_usec > b->tv_usec) {
	    return(1);
	}
	return(0); /* unlikely */
    }
}

/*
** ----------------------------------------
** toInsert.
**
** sort the to_t passed in inta toAll.
** we don't check to make sure new isn't already
** in the list.  If it was, that'd be trouble.
*/

to_t *
toInsert( new )
to_t * new;
{
    if ((toAll == 0) || toCmp( &new->to_tv, &toAll->to_tv ) < 0) {
	/* before the front. */
	new->to_next = toAll;
	toAll = new;
    } else {
	/*
	** sort it into toAll;
	*/
	to_t * afta = toAll;
	while( afta->to_next && (toCmp( &new->to_tv, &afta->to_next->to_tv ) >= 0)) {
	    afta = afta->to_next;
	}
	/* it goes afta afta */
	new->to_next = afta->to_next;
	afta->to_next = new;
    }
    return(new);
}

/*
** toList.
** handy routine to display all the current timeouts.
** used for debugging timeouts.
*/

toList()
{
    to_t * list = toAll;
    printf( "toList: \n" );
    while(list) {
	printf( "%x next %x flags %x time %d.%6.6d disp %d.%6.6d xx %x\n",
		list, list->to_next, list->to_flags,
		list->to_tv.tv_sec, list->to_tv.tv_usec,
		list->to_tvOrig.tv_sec, list->to_tvOrig.tv_usec,
		list->to_special );
	list = list->to_next;
    }
    printf( "toList: done\n" );
}

/*
** ----------------------------------------
** toNew.
**
** We only have one list of timeVals.  On that
** list we have every one set.  The timeval_t's are
** sorted... from the earliest to the most far away.
** 
**
*/

to_t *
toNew( tv, flags, t, s )
timeval_t * tv;
int flags;
trigger_t * t; 
to_special_t s;
{
    to_t * new = calloc( 1, sizeof(*new) );
    /* assert new? */

    new->to_tv = *tv;
    new->to_tvOrig = *tv;
    new->to_flags = flags;
    new->to_trigger = t;
    if (flags & TO_FLAGS_SPECIAL) {
	new->to_special = s;
    }

    toRearm( new );

    return(new);
}

/*
** ----------------------------------------
** toRemove.
**
** Scan down the list, find the to_t we've passed
** in, and remove it from the list.  Return the parameter
** passed in...
*/

to_t *
toRemove( out )
to_t * out;
{
    to_t * afta = toAll;
    if (toAll) {
	while ( afta->to_next && (afta->to_next != out) ) {
	    afta = afta->to_next;
	}
	if (afta->to_next == out) {
	    afta->to_next = afta->to_next->to_next;
	    return(out);
	} else {
	    /* hey, it wasn't on the list. */ 
	    return(out);
	}
    } else {
	/* well, the list is zero. can't be on the list. */
	return(out);
    }
}

/*
** ----------------------------------------
** toFree.
**
** Remove the thing from the list (make sure its not there),
** remove any associated trigger, then free it...
*/

to_t *
toFree( to )
to_t * to;
{
    toRemove(to);
    /*
    ** if its not a special function call, and a trigger is still there..
    ** then reset the trigger.
    */
    if ( ((to->to_flags & TO_FLAGS_SPECIAL) == 0) && to->to_trigger) {
	trigger_t * trigger = to->to_trigger;
	to->to_trigger = 0;
	triggerDecRef( trigger );
    }
    free(to);
    return(0);
}

/*
** ----------------------------------------
** toTriggerOff.
**
*/

void
toTriggerOff( t )
trigger_t * t;
{
    if (t->t_rock) {
	to_t * to = (to_t *)t->t_rock;
	t->t_rock = 0;
	toFree(to);
    }
}

/*
** ----------------------------------------
** toTriggerSet.
**
** Review the trigger which is set, and figure
** out what the timeout ought to be.  Then
** call routines to create it, and don't forget
** to return a reference to the new to_t.
**
** If you say sec, then the time interval component
** of seconds is included.  You can still say msec,
** or usec along with the other parameters.
**
*/

void *
toTriggerSet( trigger_t * trigger )
{
    /*
    ** check out each of the user's requests.
    */
    {
	timeval_t tv;
	int flags = 0;
	int i;

	tv.tv_sec = 0;
	tv.tv_usec = 0;

	for (i = 0; i<trigger->t_ndx; ++i) {
	    Sym * l;

	    if (strcmp( trigger->t_whom[i], "timeout") == 0) {
		for( l = trigger->t_nv[i]; l ; l = l->s_next ) {
		    MemVal * m = &l->s_data;
		    if (symCmp( l, "sec" ) == 0) {
			/* the parameter must be an int. */

			tv.tv_sec += m->val.i;
		    }
		    else
		    if (symCmp( l, "msec" ) == 0) {
			tv.tv_usec += (m->val.i * 1000);
		    }
		    else
		    if (symCmp( l, "usec" ) == 0) {
			tv.tv_usec += m->val.i;
		    }
		    else
		    if (symCmp( l, "once" ) == 0) {
			if (flags & TO_FLAGS_REPEAT) {
			    /* hey! */
			} else {
			    flags |= TO_FLAGS_ONCE;
			}
		    }
		    else
		    if (symCmp( l, "repeat" ) == 0) {
			if (flags & TO_FLAGS_ONCE) {
			    /* hey! */
			} else {
			    flags |= TO_FLAGS_REPEAT;
			}
		    }
		}
	    }
	}
	/*
	** create the new timeout request...
	** and return it... hey, we'll even allow zero time :)
	*/
	{
	    to_t * new = toNew( &tv, flags, trigger, 0 );

	    trigger->t_free = toTriggerOff;
	    trigger->t_rock = new;
	}
    }

    return( (void *)trigger );
}

/*
** ----------------------------------------
** toRearm.
*/

toRearm( to )
to_t * to;
{
    timeval_t now;
    gettimeofday( &now, 0 );
    to->to_tv = to->to_tvOrig;
    util_incrTv( &to->to_tv, &now );
    toInsert( to );
}

/*
** ----------------------------------------
** toDispatch.
**
** Oddly enough, the only trigger which can
** fire is the one on the top of the toAll list.
** tho the one under it may also need to fire...
**
** We've noticed its time to fire the trigger.
** after its fired, we'll want to see if we
** need to rearm the trigger, or remove it.
*/

toDispatch( )
{
    if (toAll) {
	timeval_t when = toAll->to_tv;

	while( toAll
	    && (toAll->to_tv.tv_sec == when.tv_sec)
	    && (toAll->to_tv.tv_usec == when.tv_usec)) {

		trigger_t * trigger = toAll->to_trigger;
		to_t * this = toAll;

		toAll = this->to_next;
		this->to_next = 0;

		if (this->to_flags & TO_FLAGS_SPECIAL) {
		    (this->to_special)(this);
		    if (this->to_flags & TO_FLAGS_REPEAT) {
			/* re-arm the time, reInsert it. */
			toRearm( this );
		    } else {
			/* assume it was only once. */
			toFree( this );
		    }
		} else 
		if (trigger) {
		    /*
		    ** fire this thing, then see what else we need
		    ** to do with it.
		    */
		    performArgs( trigger->t_func, trigger->t_scope,
			    memTmpNew( NFR_TYPE_TRIGGER, trigger ), 0 );

		    if (this->to_flags & TO_FLAGS_REPEAT) {
			/* re-arm the time, reInsert it. */
			toRearm( this );
		    } else {
			/* assume it was only once. */
			trigger_t * trigger = this->to_trigger;
			this->to_trigger = 0;
			triggerOff( trigger );
		    }
		} else {
		    /*
		    ** the trigger has been offed, but still has a dangling ref
		    ** somewhere.  When that ref dies, so will the trigger
		    ** structure.
		    */
		    toFree(this);
		}
	}
    } else {
	/* huh? */
    }
}
