/*
** Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
** All rights reserved.
**
** Use and distribution of this software and its source code
** are governed by the terms and conditions of the
** Network Flight Recorder Software License ("LICENSE.TXT")
*/


#include <stddef.h>
#include <stdio.h>

#include "nfrd.h"


/*
**
**	Herein is the stuff which fit nowhere else.
**		Marcus J. Ranum, Network Flight Recorder, Inc. 1996
**
*/


void
panic( char * fmt, ... )
{
    char buff[2048];
    va_list ap;

    va_start( ap, fmt );

    vfprintf( stderr, fmt, ap );
    abort();
}

char *
util_val2str( s )
int s;
{
    switch(s) {
	case NFR_RUN_OK:
	    return( "state: ok" );
	    break;

	case NFR_RUN_STACK:
	    return( "state: stack overflow" );
	    break;

	case NFR_RUN_FRAME:
	    return( "state: frame overflow" );
	    break;

	case NFR_RUN_CPU:
	    return( "state: ticks max" );
	    break;

	case NFR_RUN_ILLEGAL:
	    return( "state: ill instruction" );
	    break;

	case NFR_RUN_BREAK:
	    return( "state: break (internal state)" );
	    break;

	case NFR_RUN_RETURN:
	    return( "state: return (internal state)" );
	    break;

    }
    return("<unkown state>");
}

char *
util_type2str( t )
int t;
{
    switch (t) {
	case NFR_TYPE_ERR:
	    return("type: error");
	    break;

	case NFR_TYPE_INT:
	    return("type: int");
	    break;

	case NFR_TYPE_STR:
	    return("type: str");
	    break;

	case NFR_TYPE_LIST:
	    return("type: list");
	    break;

	case NFR_TYPE_RECORDER:
	    return( "type: recorder" );
	    break;

	case NFR_TYPE_SCHEMADATA:
	    return( "type: schemadata" );
	    break;

	case NFR_TYPE_IPV4HOST:
	    return( "type: ipv4host" );
	    break;

	case NFR_TYPE_IPV4NET:
	    return( "type: ipv4net" );
	    break;

	case NFR_TYPE_PATTERN:
	    return( "type: pattern" );
	    break;

	case NFR_TYPE_MAP:
	    return( "type: map" );
	    break;

	case NFR_TYPE_MAPBAG:
	    return( "type: mapbag (array)" );
	    break;

	default:
	    return("type: unknown");
	    break;
    }
}

char *
util_err2str( e )
int e;
{
    switch(e) {
	    case NFR_ERR_NONE:
		return( "no error" );
		break;
	    
	    case NFR_ERR_USER:
		return( "user error" );
		break;
	    
	    case NFR_ERR_STACK:
		return( "stack error" );
		break;
	    
	    case NFR_ERR_FRAME:
		return( "frame shortage error" );
		break;
	    
	    case NFR_ERR_ZDIV:
		return( "divide by zero error" );
		break;
	    
	    case NFR_ERR_NONUM:
		return( "not-a-number error" );
		break;
	    
	    case NFR_ERR_OOM:
		return( "out-of-memory error" );
		break;
	    
	    case NFR_ERR_NOVAR:
		return( "undefined variable error" );
		break;
	    
	    case NFR_ERR_NOFUNC:
		return( "no such variable (or builtin) error" );
		break;
	    
	    case NFR_ERR_BADVAL:
		return( "can't retrieve packet data" );
		break;
	    
	    case NFR_ERR_ARGUMENT:
		return( "bad argument to function/builtin" );
		break;
	    
	    case NFR_ERR_PROTOCOL:
		return( "not the correct protocol " );
		break;

	    case NFR_ERR_NULL:
		return("null");
		break;
    }
    return( "unknown error" );
}

int
util_tcpFlags( f )
int f;
{
    int cnt = 0;
    if (f & TH_FIN) {
	uiCurrPrintf( "FIN" );
	cnt++;
    }
    if (f & TH_SYN) {
	if (cnt) {
	    uiCurrPrintf( "," );
	}
	uiCurrPrintf( "SYN" );
	cnt++;
    }
    if (f & TH_RST) {
	if (cnt) {
	    uiCurrPrintf( "," );
	}
	uiCurrPrintf( "RST" );
	cnt++;
    }
    if (f & TH_PUSH) {
	if (cnt) {
	    uiCurrPrintf( "," );
	}
	uiCurrPrintf( "PUSH" );
	cnt++;
    }
    if (f & TH_ACK) {
	if (cnt) {
	    uiCurrPrintf( "," );
	}
	uiCurrPrintf( "ACK" );
	cnt++;
    }
    if (f & TH_URG) {
	if (cnt) {
	    uiCurrPrintf( "," );
	}
	uiCurrPrintf( "URG" );
	cnt++;
    }
    return(cnt);
}

/*
** util_makeTv.
** just a way to improve code readability when
** where making timeval's.
*/

timeval_t
util_makeTv( sec, usec )
{
    timeval_t new;
    new.tv_sec = sec;
    new.tv_usec = usec;
    return new;
}

/*
**
*/

timeval_t
util_diffTv( a, b )
timeval_t * a;
timeval_t * b;
{
    timeval_t diff;
    diff.tv_sec = a->tv_sec - b->tv_sec;
    if ((diff.tv_usec = (a->tv_usec - b->tv_usec)) < 0) {
	diff.tv_sec--;
	diff.tv_usec += MILLION;
    }
    return(diff);
}

/*
** util_incrTv
**
*/

void
util_incrTv( a, b )
timeval_t * a;
timeval_t * b;
{
    a->tv_sec += b->tv_sec;
    if ((a->tv_usec += b->tv_usec) > MILLION) {
	a->tv_usec -= MILLION;
	a->tv_sec++;
    }
}

/*
** util_tv2usec.
**
*/

double
util_tv2usec( tv )
timeval_t * tv;
{
   return(tv->tv_usec + ((double)(tv->tv_sec) * MILLION));

}

/*
** util_tv2msec.
** since sometimes you wanna have msec, we can try to
** shove em inta an int.
*/

int
util_tv2msec( tv )
timeval_t * tv;
{
    return( (int) (tv->tv_sec * 1000) + (tv->tv_usec/1000) );
}

/*
** format.
**
** use sprintf to make a string to return...
** Only use small amounts of data... we use a static
** buffer allocated with about 2k of data. 
**
** Some systems support vasprintf, but not all that many.
**
*/

#ifdef	VASPRINTF

char *
format( char * fmt, ... )
{
    char * bp;
    va_list ap;

    va_start( ap, fmt );

    vasprintf( &bp, fmt, ap );
    if (bp != 0) {
	char * ret = (char *)tmpalloc(strlen(bp)+1);
	strcpy(ret, bp);
	free(bp);
	return(ret);
    } else {
	return "<error>";
    }
}

#else	/* !VASPRINTF */

char *
format( char * fmt, ... )
{
    char buff[2048];
    va_list ap;

    va_start( ap, fmt );

    vsprintf( buff, fmt, ap );
    {
	char * ret = (char *)tmpalloc(strlen(buff)+1);
	strcpy(ret, buff);
	return(ret);
    }
}

#endif

/*
** stdDev.
**
** Return the standard deviation.
*/

double
util_stdDev( sum, sumSq, cnt )
double sum;
double sumSq;
int cnt;
{
    if (cnt < 2) {
	return(0);
    }
    {
	double sSq = ((sumSq) - ((sum*sum)/cnt))/(cnt-1);
	if (sSq < 0) {
	    sSq = -sSq;
	}
	return(sqrt(sSq));
    }
}

char *
util_str( s )
char * s;
{
    if (s == 0) {
	return("<null>");
    }
    return(s);
}

char *
util_ntoa( n )
int n;
{
	struct in_addr a;
	a.s_addr = n;
	return(inet_ntoa(a));
}
