#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int isspace();

/* M K A R G V
 *
 * mkargv() is used to divide a supplied command line string into its 
 * components and create an argument vector (argv) ready for exec usage.
 *
 * Called as:
 *     args = mkargv(&argc,&argv,"/some/cmd -some-options other options");
 *
 * Returns:
 *      argc - number of arguments
 *      NULL - malloc failed
 */

char **mkargv(int *argc, char **argv[], char *str)
{
    char *p, *cp, *savedstr;
    int i;

    savedstr = strdup(str);  /* Save it so it can be passed as literals */

    /* 
    ** Determine how many args there are so 
    ** that you know how much to allocate for.
    */

    i = 0;
    p = savedstr;
    while (*p) {
        while (*p && (!isspace(*p)))
              p++;
        i++;
        while (*p && (isspace(*p)))
              p++;
    }

    ++i;

    /* Allocate the storage vector */

    *argv = (char **) malloc( sizeof(char *) * (i+1));
    if (*argv == (char **) NULL) 
        return((char **)NULL);

    /*
    ** Time to parse the string and save the elements into the vector.
    */

    p = savedstr;
    cp = p;
    *argc = 0;

    while (*p) {
        while (*p && (!isspace(*p)))    /* Get off the argument   */
              p++;
        
	if (*p)
  	      *p++ = '\0';              /* terminate the argument */

        if (*cp) {                /* Is there something to save ? */
            /* Guess so... */
            *(*argv+*argc) = strdup(cp);
            if (*(*argv+*argc) == (char *) NULL)
                return((char **) NULL);
            *argc += 1;
            cp = p;
        }

        while (*p && (isspace(*p)))    /* Get off the white space */
              p++;
    }
    *(*argv+*argc) = (char *) NULL;    /* Terminate the array     */
    return(*argv);                     /* return to the caller... */
}

#ifdef MKARGVTEST

int usage(char *progname)
{
   fprintf(stderr,"usage: %s -s \"string\"\n", progname);
   return(1);
}

int eargc;
char **eargv;

int main(int argc, char **argv)
{
   int getopt();
   extern char *optarg;

   char *string = NULL;
   int c;

   if (argc > 1) {
      while ((c = getopt(argc, argv, "s:")) != EOF) {
         switch (c) {
             case 's':   /* debugging stuff */
                 string = optarg;
		 if (string) {
		       mkargv(&eargc, &eargv, string);

		       (void) printf("Argc == %d\n", eargc);
		       for (c = 0; c < eargc; c++) 
			   (void) printf("%d: %s\n", c, eargv[c]);
		 }
                 break;
             default:
                 return(usage(argv[0]));
         }
      }   
   }   

   if (!string)
      return(usage(argv[0]));

  return(0);
}

#endif
