/******************************************************************** 
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT") 

  File:   testprog.c 
  Author: Kent Landfield <kent@nfr.net>
 
********************************************************************/ 

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <pwd.h>
#include <stdarg.h>
#include <unistd.h>
#include <setjmp.h>

#define NFRPIDDIR "etc/run"

char pidfile[BUFSIZ];

uid_t uid;
gid_t gid;
pid_t pid;

jmp_buf env; 

int setnfrpid(char *pidfile); 

extern char *optarg;
int getopt();

void printenv(char **envp)
{
   char *cp;

   cp = *envp;

   while(cp) {
       fprintf(stderr,"%s\n", cp);
       cp = *envp++;
   }
}

/*
** usage()
**
** Print the command's usage message
*/

int usage(char *progname)
{
   fprintf(stderr,"usage: %s -p pidfile\n", progname);
   fprintf(stderr,"\t-p pidfile : pidfile to read from/write to\n");
   return(1);
}

static void signal_handler(sig)
int sig;
{
     switch (sig)     {
          case SIGTERM: longjmp(env,sig);    
                        break;               /* break never reached */
          case SIGINT:  longjmp(env,sig);
                        break;               /* break never reached */
          default:        exit(sig);
     }
}

/*
** Ye Olde Main.
**
*/

int main(int argc, char **argv, char **envp)
{
   char *progname;

   pid_t pid;
   pid_t fpid;
   int c;
   int termination = 0;
   int server = 0;

   pidfile[0] = '\0';

   if ((progname = strrchr(argv[0], '/')) == NULL)
        progname = argv[0];
   else
        progname++;

  if (strcmp(progname,"chldprog") != 0) 
      server = 1;

   if (argc > 1) {
      while ((c = getopt(argc, argv, "p:")) != EOF) {
         switch (c) {
             case 'p':   /* pidfile */
                 sprintf(pidfile,"%s",optarg);
                 break;
             default:   
                 return(usage(argv[0]));
         }
      }
   }

   if (pidfile[0] == '\0')
       return(usage(argv[0]));
   
   if (server) {
       switch(fork()) {
           case 0:   /* child */
                     break;
           case -1:  /* error occurred. */
                     perror(argv[0]);     /* tell us why */
                     return(-127);        /* Eject, eject! */
                     /*NOTREACHED*/
           default:  return(0);  /* parent -  nothing to be done here */
                     /*NOTREACHED*/
       }
   }
   
   pid = getpid();      /* get the processid */
   uid = getuid();       /* get the process uid */
   gid = getgid();       /* get the process gid */

   fpid = setnfrpid(pidfile);  /* Create the process id file */
   switch(fpid) {
       case 0 : fprintf(stderr,"%s: [%s] created successfully\n", 
                                   argv[0],pidfile);
                break;
       case 1 : fprintf(stderr,"%s: Another process already running: %d\n", 
                                   argv[0], fpid); 
                return(-1);
                /*NOTREACHED*/
       case -1: fprintf(stderr,"%s: Pidfile open read failed: %s\n", 
                                   argv[0],pidfile);
                return(-1);
                /*NOTREACHED*/
       case -2: fprintf(stderr,"%s: Pidfile format error: %s\n", 
                                   argv[0],pidfile);
                return(-1);
                /*NOTREACHED*/
       case -3: fprintf(stderr,"%s: Pidfile open write failed: %s\n", 
                                   argv[0],pidfile);
                return(-1);
                /*NOTREACHED*/
       default: fprintf(stderr,"%s: ??? setnfrpid returned a %d\n", 
                                   argv[0],fpid);
                return(-1);
                /*NOTREACHED*/
   }
      
   for(;;) {
       switch (setjmp(env)) {
          case SIGTERM:
              termination++;
              fprintf(stderr,"SIGTERM received: %s terminating\n",argv[0]);
              break;
          case SIGINT:
              fprintf(stderr,"SIGINT received: Dumping process variables\n");
              fprintf(stderr,"%s: Process id  == %d\n",argv[0],pid);
              fprintf(stderr,"%s: Process UID == %d\n",argv[0],uid);
              fprintf(stderr,"%s: Process GID == %d\n",argv[0],gid);
              fprintf(stderr,"%s: Process pidifle = %s\n",argv[0],pidfile);
              fprintf(stderr,"%s Environment:\n",argv[0]);
              printenv(envp);
              break;
          case 0: break;
       }
    
       if (termination) 
           break;

       (void) signal(SIGTERM, signal_handler);
       (void) signal(SIGINT,  signal_handler);

       for(;;) 
           sleep(360);
       /* NOTREACHED */
   }
   unlink(pidfile);
   return(0);
}
