/***************************************************************************** 
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT") 

  File:   main.c 
  Author: Kent Landfield <kent@nfr.net>
 
 *****************************************************************************/ 

#include "sys-include.h"

#include "spaceman.h"
#include "cf.h"

int getopt();

int verbose = 0;
int debug = 0;

extern FILE *stream;
extern char *optarg;
extern int optind, opterr, optopt;

extern struct node *Phead[];

unsigned int alarmtime = 900;
int smack_spacemand(void);

int FifoFdW;    /* Write Input fifo file descriptor */
int FifoFdR;    /* Read Input fifo file descriptor */

char *Fifopath = SPACEMAN_FIFO;   /* Disk path to input fifo    */
char *Configpath = SPACEMAN_CONFIG;  /* config file to use */

int usage(char *progname)
{
   fprintf(stderr,"%s [-cdrv] [ [-f configfile] [-t seconds]\n", progname);
   fprintf(stderr,"\t-c             : Check syntax of the config file\n");
   fprintf(stderr,"\t-d             : Turn on debugging\n");
   fprintf(stderr,"\t-f configfile  : Path to spaceman configuration file\n");
   fprintf(stderr,"\t-r             : Tell spacemand to re-read configuration file\n");
   fprintf(stderr,"\t-v             : Turn on verbose output\n");
   fprintf(stderr,"\t-t seconds     : Space check period\n");
   fprintf(stderr,"\nDefault configuration file: $NFRHOME/%s\n", SPACEMAN_CONFIG);
   fprintf(stderr,"Default Space Check period: %d\n", alarmtime);
   return(1);
}

int main(int argc, char **argv)
{
   int c;
   int check_syntax = 0;
   PDaemon pDaemon = NULL;

   stream = stderr;

   if (argc > 1) {
      while ((c = getopt(argc, argv, "cdf:rt:v?")) != EOF) {
         switch (c) {
             case 'c':   /* syntax check the config file */
                 check_syntax++;
                 debug++;
                 break;
             case 'd':   /* debugging stuff */
                 debug++;
                 break;
             case 'f':   /* specify the config file */
                 Configpath = strdup(optarg);
                 break;
             case 'r':   /* re-read config file */
                 return(smack_spacemand());
             case 't':   /* alarmtime */
                 alarmtime = atoi(optarg);
                 break;
             case 'v':   /* verbose */
                 verbose++;
                 break;
             case '?':   
             default:   
                 return(usage(argv[0]));
         }
      }
   }

   if (check_syntax) {   /* simply check the syntax of the config file? */
       if ((c = read_config())) {
            if (verbose)
                print_list(Phead);
        }
        free_list(Phead);
        return(c == 1 ? 0 : 1);       /* terminate this process */
   }

   /* Space management for here on out */

   pDaemon = Daemon_New();
   pDaemon->Main = spaceman_main;
   pDaemon->OnStop = spaceman_stop;
   pDaemon->OnStart = NULL;
 
   Daemon_Start(pDaemon, 30000);
 
   /* parent only from here down */
 
   /* free and memory allocated in parent and say bye - bye */
 
   Daemon_Delete(pDaemon);
   return(0);
}
