/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT") 

  File:   procfile.c 
  Author: Kent Landfield <kent@nfr.net>
 
 *****************************************************************************/ 

#include "sys-include.h"

#include "alert.h"
#include "nfr_res.h"

void report(char *format, ... );

extern int debug;

int is_executable(char *cmd)
{
     /* Will need to redo this for MS */

     struct stat stbuf;
     char cbuf[BUFSIZ];
     char *cp;

     strcpy(cbuf, cmd);
     cp = cbuf;

     while (*cp && (!isspace(*cp)))
          cp++;

     if (*cp)
         *cp = '\0';

     if (stat(cbuf, &stbuf) != 0) {   /* Does the command exist ? */
         if (debug)
             report ("command %s: not found\n", cbuf);
         else
             Alert_Args_Generate(NFR_SRC_SPACEMAND, NFR_SPACEMAND_CMD_MISSING,
                                  cbuf, 0);
         return(0);
     }

     if ((stbuf.st_mode & S_IFMT) != S_IFREG) { /* Is it the right type */
         if (debug)
             report ("command %s: not a regular file\n", cbuf);
         else
             Alert_Args_Generate(NFR_SRC_SPACEMAND, NFR_SPACEMAND_CMD_BADFILE,
                                  cbuf, 0);
         return(0);
     }
     if (access(cbuf, X_OK) != 0) {   /* Is the command executable ? */
         if (debug)
             report ("command %s: not executable\n", cbuf);
         else
             Alert_Args_Generate(NFR_SRC_SPACEMAND, NFR_SPACEMAND_CMD_NO_RUN,
                                  cbuf, 0);
         return(0);
     }
     return(1);
}

int process_archive(char *filename, char *command)
{
     register int i;
     register char *cp;
     char *cmd;
     char pbuf[BUFSIZ];
     FILE *fp;
     FILE *pfp;

     if (debug)
         report("process_archive(%s, %s)\n", filename, command);

     /* 
     ** Need to determine if it is necessary to simply execute a command line
     ** and be done with it or do we need to open the process as a filter and
     ** pipe the data being archived to it ?  The indicator here is the presense
     ** or absense of the '|' character at the start of the process command.
     */
     
     if (*command !=  '|') {

     if (debug)
         report("system(%s)\n", command);

     if (!is_executable(command)) /* Let's catch bad executables */
         return (-1);

         system(command);
         return(0);
     }

     cmd = command;
     while (*cmd && (isspace(*cmd) || *cmd == '|'))
          cmd++;

     if (!*cmd) {
          if (debug)
              report ("missing command: %s\n", command);
          else
             Alert_Args_Generate(NFR_SRC_SPACEMAND, NFR_SPACEMAND_CMD_EMPTY,
                                  command, 0);
          return (-1);
     }

     /* Let's do some setup work to catch bad executables */
     
     if (!is_executable(cmd)) 
         return (-1);

     /* open the file to be sent to the external command */

     if ((fp = fopen(filename,"r")) == NULL) {
          if (debug)
              report ("process command: can not open %s\n", filename);
          else
             Alert_Args_Generate(NFR_SRC_SPACEMAND, NFR_SPACEMAND_RKIVE_OPEN,
                                  filename, 0);
          return (-1);
     }

     if (debug)
         report("popen(%s, w)\n", cmd);

     /* open a pipe to the specified command */

     if ((pfp = popen(cmd, "w")) == NULL) {
          if (debug)
              report ("process command: can not open pipe to %s\n", cmd);
          else
             Alert_Args_Generate(NFR_SRC_SPACEMAND, NFR_SPACEMAND_PIPE_OPEN,
                                  cmd, 0);
          fclose(fp);
          return (-1);
     }

     /* 
     ** zero out the buffer that will be used to read
     ** data into it.  This is done so that on EOF we
     ** can determine any partial reads that need to
     ** be written out.
     */

     for (i = 0; i < BUFSIZ; i++)
         pbuf[i] = '\0';

     while ((i = fread(pbuf, BUFSIZ, 1, fp))) {
         fwrite(pbuf, BUFSIZ, 1, pfp);
         for (i = 0; i < BUFSIZ; i++)
             pbuf[i] = '\0';
     }

     if (feof(fp)) {
         /* Find out how many were read */
         cp = pbuf+BUFSIZ-1;

         while(*cp == '\0' && cp != pbuf)
               cp--;
      
         if (cp != pbuf) {
             fwrite(pbuf, cp-pbuf+1, 1, pfp);
         }
     }

     fflush(pfp);
     pclose(pfp);
     fclose(fp);
     return (0);
}

#ifdef TEST
int debug = 0;

int usage(char *progname)
{
   fprintf(stderr,"%s [-dv] [ [-f fifo]\n", progname);
   fprintf(stderr,"\t-d       : Turn on debugging\n");
   fprintf(stderr,"\t-f fifo  : Path to input fifo\n");
   return(1);
}

int main(int argc, char **argv)
{
   int c;

   if (argc > 1) {
      while ((c = getopt(argc, argv, "dv")) != EOF) {
         switch (c) {
             case 'd':   /* debugging stuff */
                 debug++;
                 break;
             default:   
                 return(usage(argv[0]));
         }
      }
   }

   report("argc == [%d]\n", argc);
   report("optind == [%d]\n", optind);

   if (optind+2 == argc)
       process_archive(argv[optind], argv[optind+1]);

   return(0);
}
#endif
