/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT") 

  File:   rdheader.c 
  Author: Kent Landfield <kent@nfr.net>
 
 *****************************************************************************/ 

#include "sys-include.h"

#include "archive.h"

#define USAGE  "usage: %s [ -dv ]\n"

char *progname;                 /* name of executable              */

int index_record = 0;
int verbose = 0;
int show_blob = 0;

char *atime(time_t sometime);

void read_blob(Archive *rkive)
{
    int pos;
    int reclen;
    char *buf;

    pos = ftell(rkive->fp);    /* Save current position of file pointer */

    reclen = rkive->head.datastart - sizeof(struct header);
    /*
    ** Go to rec1 and read it into the recinfo structure
    */
    fseek(rkive->fp, sizeof(struct header), SEEK_SET);
    buf = (char *)calloc(1, reclen+1);
    fread(buf, reclen, 1, rkive->fp);

    if (rkive->head.datastart == sizeof(struct header))
        puts("NO Recorder Specific Information");
    else 
        printf("Recorder Specific Information: %s\n", buf);

    fseek(rkive->fp, pos, SEEK_SET);
}

int read_header(char *file)
{
    Archive *rkive;
    struct stat st;

    void free(void *ptr);

    /* Need to allocate memory for the Archive structure */

    if ((rkive = (Archive *)calloc(1, sizeof(Archive))) == NULL) 
         return(-1);

    if ((stat(file, &st) == 0) && ((st.st_mode & S_IFMT) != S_IFREG)) {
        (void) fprintf(stderr,"%s: %s not a regular file\n",progname, file);
        return(1);
    }

    if ((rkive->fp = fopen(file,"rb")) == NULL) {
        (void) fprintf(stderr,"%s: Cant open %s\n",progname, file);
        return(1);
    }

    fread(&rkive->head, sizeof(struct header), 1, rkive->fp);

    if (index_record) {
       printf("%s %s %s %ld\n", rkive->head.backend,
              (*rkive->head.ctype ? rkive->head.ctype : "None"),
               file, rkive->head.starttime);
    }
    else {
        printf("Backend:       %s\n", rkive->head.backend);
        printf("Cf Version:    %s\n", rkive->head.cfversion);
        printf("Content Type:  %s\n", 
            (*rkive->head.ctype ? rkive->head.ctype : "None Specified"));
        printf("Archive Size:  %ud\n", rkive->head.archive_size);
        printf("Archive Start: %s\n", atime(rkive->head.starttime));
        printf("Archive End:   %s\n", 
            rkive->head.endtime == 0 ? "Not Set" : atime(rkive->head.endtime));
        printf("Datastart:     %ld\n", rkive->head.datastart);
        printf("Size of Header: %d\n", sizeof(struct header));

        if (show_blob)
             read_blob(rkive);
    }

    archive_close(rkive);
    free(rkive);
    return(0);
}


int main(int argc, char **argv)
{
   int c;
   extern int optind;
   extern int opterr;
   int getopt();

   opterr = 0;
   progname = argv[0];

   if (argc > 1) {
      while ((c = getopt(argc, argv, "ivr")) != EOF) {
         switch (c) {
             case 'v':
                 verbose++;
                 break;
             case 'i':
                 index_record++;
                 break;
             case 'r':
                 show_blob++;
                 break;
             default:
                 (void) fprintf(stderr,USAGE, progname);
                 return(1);
         }
      }
   }

   c = 0;

   if ((optind >= argc) || (argc == 1)) {  /* file from stdin */
        char buf[BUFSIZ];
        while (fgets(buf,sizeof buf,stdin) != NULL) {
               buf[strlen(buf) -1] = '\0';
               c += read_header(buf);
        }
   }
   else { 
        for (; optind < argc; optind++)    /* process files to print */
             c += read_header(argv[optind]);
   }
   return(c);                              /* terminate this process */
}
