/***************************************************************************** 
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT") 

  File:   ssmain.c 
  Author: Kent Landfield <kent@nfr.net>
 
 *****************************************************************************/

#include "sys-include.h"

#include <setjmp.h>
#include "spaceman.h"
#include "archive.h"
#include "alert.h"
#include "nfr_res.h"

jmp_buf env; 

unsigned int indxlen = sizeof(struct indx_rec);

extern struct node *Phead[]; 
extern int FifoFdR; 
extern unsigned int alarmtime;
extern int debug;

int terminate = 0;

void print_list(struct node **phead);
void Rereadconfig(int sig);
void Runlist(void);
int spaceman_start(void);
time_t runnode(struct node *t,struct tm *now,time_t crnt_time);
char *itoa(int i);

char *tmstring(time_t tstamp)
{
    struct tm *now;
    static char dtstr[80];

    now = localtime(&tstamp); 
    sprintf(dtstr,"%.2d/%.2d/%.2d %.2d:%.2d:%.2d",now->tm_mon,now->tm_mday,
                        now->tm_year,now->tm_hour,now->tm_min,now->tm_sec);
    return(dtstr);
}

void dump_index_record(struct indx_rec *Irec)
{
    report("\tBackend: %s\n", Irec->backend);
    report("\tContent type: %s\n", Irec->ctype);
    report("\tPath to file: %s\n", Irec->path);
    report("\tStarttime: %ld - %s\n", 
                    Irec->starttime, tmstring(Irec->starttime));
}


static void signal_handler(sig)
int sig;
{
     switch (sig)     {
          case SIGTERM:                        /* process for interrupt */
                          longjmp(env,sig);
                          break;               /* break never reached */
          case SIGHUP:    Rereadconfig(sig);   /* process for interrupt */
                          Runlist();
                          longjmp(env,sig);
                          break;               /* break never reached */
          case SIGINT:                         /* process for interrupt */
                          longjmp(env,sig);
                          break;               /* break never reached */
          case SIGALRM:   Runlist();           /* process for alarm */
                          longjmp(env,sig);
                          break;               /* break never reached */
          default:        exit(sig);
     }
}

int spaceman_main(PDaemon pDaemon)
{
    char *rp;
    struct node *t;
    struct tm *now;
    time_t crnt_time;
    struct indx_rec Irec;
    int len = 0;
    int rc = 0;
    int processing = 1;

    /*
    ** Set things up, fifos, pidfiles etc. 
    ** If an error is encountered, exit.
    */

    if ((rc = spaceman_start()) != 0) 
        return(rc);
          
    if (debug)
        report("I AM IN MAIN!!!!!!! %s\n", "YEAH!!!!");

    for(;;) {
        switch (setjmp(env))     {
            case SIGTERM:
                    terminate++;
                    if (debug) {
                        report("longjumped from SIGTERM: %d\n",SIGTERM);
                        report("Rip Van Winkle is done. %s\n", "Good...");
                    }
                    break;
            case SIGINT:
                    if (debug)
                        report("longjumped from SIGINT: %d\n",SIGINT);
                    break;
            case SIGHUP:
                    if (debug) {
                        print_list(Phead);
                        report("longjumped from SIGHUP: %d\n",SIGHUP);
                    }
                    break;
            case SIGALRM:
                    if (debug)
                        report("longjumped from SIGALRM %d\n",SIGALRM);
                    break;
            case 0: break;
        }
    
        if (terminate) 
            break;

        (void) signal(SIGTERM, signal_handler);
        (void) signal(SIGINT, signal_handler);
        (void) signal(SIGALRM, signal_handler);
        (void) signal(SIGHUP, signal_handler);

        alarm(alarmtime);

        while (processing) {
             if (debug)
                 print_list(Phead);
 
             if ((len=read(FifoFdR, &Irec,indxlen)) != indxlen) {
                 if (debug)
                     report("READ ERROR!!!!! len == %d Should have been %d\n",
                          len, indxlen);
                 else
                     Alert_Args_Generate(NFR_SRC_SPACEMAND, 
                                         NFR_SPACEMAND_FIFO_READ_ERR,
                                         itoa(len), itoa(indxlen), 0);
             }   

             /*
             ** Find the node to process and process just that node
             ** First separate the backend/recorder info.
             ** Loop through the list until a dual match is found.
             ** Run the found node.
             */
             
             if ((rp = strchr(Irec.backend,'/')) != NULL)
                  *rp++ = '\0';
             else
                  rp = NULL;

             crnt_time = time((time_t *)0);  /* get current runtime */ 
             now = localtime(&crnt_time);    /* Convert to a tm struct */

             t = *Phead;
             while(t != NULL) {
                  if ((strcmp(Irec.backend, t->backend) == 0) &&
                      (strcmp(rp, t->recorder) == 0))
                      runnode(t,now,crnt_time);
                  t = t->next;
             }

             if (debug)
                 dump_index_record(&Irec);

        }
    }

    if (debug) {
        report("I AM LEAVING MAIN!!!!!!! %s\n", "YEAH!!!!");
        report("Falling out the bottom... %s\n", "Yes!!...");
    }

    return(spaceman_stop(pDaemon));
}
