/***************************************************************************** 
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT") 

  File:   sstop.c 
  Author: Kent Landfield <kent@nfr.net>
 
 *****************************************************************************/ 

#include "sys-include.h"

#include "spaceman.h"
#include "alert.h"
#include "nfr_res.h"

extern int FifoFdR;
extern int FifoFdW;
extern char *Fifopath;
extern int debug;

void free_list(struct node **phead);
void Fifo_Destroy(char *name);

extern struct node *Phead[];

/*
**  Stop:
**          1. close open files
**          2. Remove input FIFO
**          3. free memory allocated
**          4. Remove pid file
**          5. Log termination
**          6. Exit process
*/  

int spaceman_stop(PDaemon pDaemon)
{
  /*
  ** Close any open files
  */

  close(FifoFdR);
  close(FifoFdW);

  /*
  ** Delete the fifo
  */

  if (debug)
      report("Removing Fifopath == %s\n", Fifopath);
  else 
      Alert_Args_Generate(NFR_SRC_SPACEMAND, NFR_SPACEMAND_RMFIFO, Fifopath, 0);

  Fifo_Destroy(Fifopath);

  /*
  **  Remove the runtime pidfile
  */

  if (debug)
      report("Removing SPACEMAN_PIDFILE == %s\n", SPACEMAN_PIDFILE);
  else 
      Alert_Args_Generate(NFR_SRC_SPACEMAND, NFR_SPACEMAND_RMFIFO, Fifopath, 0);

  remove(SPACEMAN_PIDFILE);

  /*
  ** Free allocated memory 
  */

  if (debug)
      report("%s freeing allocated memory\n", "Now");
  else 
      Alert_Generate(NFR_SRC_SPACEMAND, NFR_SPACEMAND_CLEANUP);

  free_list(Phead);

  /*
  ** put out a logging alert message
  */
  if (debug)
      report("%s\n","spaceman: normal termination");
  else 
      Alert_Generate(NFR_SRC_SPACEMAND, NFR_SPACEMAND_TERMINATION);

  /*
  ** terminate the process
  */
  return 0;
}
