/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT") 

  File:   utils.c 
  Author: Kent Landfield <kent@nfr.net>
 
 *****************************************************************************/ 

#include "sys-include.h"

#include "spaceman.h"
#include "alert.h"
#include "nfr_res.h"

extern int debug;

void report(char *, ... );
int makedirs(char *dirpath);
char *itoa(int i);

int rmfile(char *dbname, char *filename)
{
    int rc;

    if ((rc = unlink(filename)) != 0) {
        if (debug)
            report ("%s - %s: unlink failed - errno %d\n", dbname,filename,errno);
        else
            Alert_Args_Generate(NFR_SRC_SPACEMAND, NFR_SPACEMAND_RMFILE_ERR,
                             dbname,filename, itoa(errno), 0);
    }
    else {
        if (debug)
            report ("%s - %s: Purged\n", dbname,filename);
        else
            Alert_Args_Generate(NFR_SRC_SPACEMAND, NFR_SPACEMAND_FILE_PURGED,
                             dbname,filename, 0);
    }
    return(rc);
}

int copyfile(char *src, char *dest)
{
    char *cp;
    char fbuf[BUFSIZ];
    int from, to, ct;

    /* make the parent directories... */

    strcpy(fbuf, dest);
    if ((cp = strrchr(fbuf, '/')) != NULL) {
         *cp = '\0';
         makedirs(fbuf);
    }
     
    /* Open the source file from which to copy */

    if ((from = open(src, O_RDONLY)) == -1) {
        if (debug)
            report ("copyfile - %s: open failed - errno %d\n", src,errno);
        else
            Alert_Args_Generate(NFR_SRC_SPACEMAND, NFR_SPACEMAND_COPYFILE_OPEN,
                             src,itoa(errno), 0);
        return (-1);
    }

    /* Create the destination file to be copied to. */

    if ((to = creat(dest, 0666)) == -1) {
        if (debug)
            report ("copyfile - %s: creat failed - errno %d\n", dest,errno);
        else
            Alert_Args_Generate(NFR_SRC_SPACEMAND, NFR_SPACEMAND_COPYFILE_CREAT,
                             dest,itoa(errno), 0);
        (void) close(from);
        return (-1);
    }

    /* copy the file */

    while ((ct = read(from, fbuf, BUFSIZ)) != 0) {
        if (ct < 0 || write(to, fbuf, ct) != ct) {
            if (debug)
                report ("copyfile - %s: write failed - errno %d\n", dest,errno);
            else
                Alert_Args_Generate(NFR_SRC_SPACEMAND, NFR_SPACEMAND_COPYFILE_WRITE,
                             dest,itoa(errno), 0);
            (void) close(from);
            (void) close(to);
            (void) unlink(dest);
            return (-1);
        }
    }
    (void) close(from);
    (void) close(to);

    if (debug)
        report ("%s saved to %s\n", src, dest);
    else
        Alert_Args_Generate(NFR_SRC_SPACEMAND, NFR_SPACEMAND_FILE_COPIED,
                             src,dest, 0);
    return(0);
}


#ifdef TEST
int verbose = 0;
int debug = 0;

extern FILE *stream;

int usage(char *progname)
{
   fprintf(stderr,"%s [-dv] [ [-f fifo]\n", progname);
   fprintf(stderr,"\t-d       : Turn on debugging\n");
   fprintf(stderr,"\t-v       : Turn on verbose output\n");
   fprintf(stderr,"\t-f fifo  : Path to input fifo\n");
   return(1);
}

int main(int argc, char **argv)
{
   int c;

   stream = stderr;

   if (argc > 1) {
      while ((c = getopt(argc, argv, "dv")) != EOF) {
         switch (c) {
             case 'd':   /* debugging stuff */
                 debug++;
                 break;
             case 'v':   /* verbose */
                 verbose++;
                 break;
             default:   
                 return(usage(argv[0]));
         }
      }
   }

#ifdef COPYTEST

   if (optind+2 == argc)
       copyfile(argv[optind], argv[optind+1]);
   else
       report("copyfile: Invalid number of parameters\n");
#else
 
   for (; optind < argc; optind++)    /* process files to print */
        rmfile("weblint",argv[optind]);
#endif

   return(0);
}
#endif
