/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   PackagesPanel.java
  Author: Eric Wall (eric@nfr.net)

 *****************************************************************************/



import java.awt.*;
import java.io.*;
import java.net.*;
import java.util.*;



import nfr.misc.*;



public class PackagesPanel extends Panel {

  Vector backendVec; 
  List mainList;
  Panel buttonPanel;
  Button query, configure;
  String string, guiType;
  String backendTab = "     ";
  Cf st;
  BackendStruct back = null;



  public PackagesPanel() {

    // Bug: Don't know what Cf does if the file can't be opened
    st = new Cf(NFRInfo.document_base,"etc/javastrings/PackagesPanel.str");
    st.Read();

    this.setLayout(new GridBagLayout());

    // Create the list
    mainList = new List();
    mainList.setBackground(Color.white);
    mainList.setForeground(Color.black);

    fillList(mainList);


    // Create query/disable button panel
    buttonPanel = new Panel();
    buttonPanel.setLayout(new GridBagLayout());

    query = new Button(st.textorname("Query"));
    configure = new Button(st.textorname("Configure"));

    query.disable();
    configure.disable();

    gbadd(buttonPanel,query,0,0,1,1,
          GridBagConstraints.BOTH,
          GridBagConstraints.CENTER,
          0.75,0.5,0,0,0,0);
    gbadd(buttonPanel,configure,1,0,1,1,
          GridBagConstraints.BOTH,
          GridBagConstraints.CENTER,
          0.25,0.5,0,0,0,0); 


    // Add list and button panel to this
    gbadd(this,mainList,0,0,1,1,
          GridBagConstraints.BOTH,
          GridBagConstraints.NORTH,
          0.5,1.0,1,1,1,1);

    gbadd(this,buttonPanel,0,1,1,1,
          GridBagConstraints.BOTH,
          GridBagConstraints.CENTER,
          0.5,0.0,1,1,1,1);
  }



  private void fillList(List targetList) {

    String line;
    backendVec = new Vector();
    BackendStruct backend;

    try {
      DataInputStream d = cgi_post.get(NFRInfo.document_base,	
				       "ls_packages", "");
      while ((line = d.readLine()) != null)
	{

	  // Don't let error messages get into the ListBox
	  String tmp = line.substring(0,3);
	  if(tmp.equals("***")) {
	    System.out.println("Found an error string.");
	    continue;
	  }

	  // Handle Package lines differently
	  tmp = line.substring(0,8);
	  if(tmp.equals("Package:")) {
	    String pname = line.substring(line.indexOf(";") + 1);
	    if (pname.substring(0,1).compareTo("*")==0)
			continue;
	    targetList.addItem(" ");
	    targetList.addItem(pname);
	    continue;
	  }


	  // Compute offsets into string, bail out if they look
	  // screwed up
	  int slash = line.indexOf('/');
	  if (slash < 0) continue;
	  
	  int dot = line.indexOf('.');
	  if (dot < 0) continue;	// how ?
	  
	  int equal = line.indexOf('=');
	  if (equal < 0) continue;	// how?
	  
	  int colon = line.indexOf(':');
	  if (colon < 0) continue; // how?

	  // create and fill in a new BackendStruct
	  backend = new BackendStruct();
	  backend.packageName = line.substring(0,slash);
	  backend.name = line.substring(slash+1,dot);
	  backend.description = line.substring(colon+1);
	  backend.guiType = line.substring(equal+1,colon);

	  // if the title marks it as hidden, suppress it.
	  if (backend.description.substring(0,1).compareTo("*") == 0)
		continue;

	  // Toss it into backendVec for safe keeping
	  backendVec.addElement(backend);

	  // Add it to the list box and save off its index
	  targetList.addItem(backendTab + backend.description);
	  backend.listIndex = targetList.countItems() - 1;
	}
    } catch (IOException e) {
      System.out.println("ended with "+e.toString());
    }
  }
  


  public boolean action(Event e, Object arg) {

    int selectedIndex = mainList.getSelectedIndex();
    Enumeration backendVecEnum = backendVec.elements();
    while(backendVecEnum.hasMoreElements()) {
      back = (BackendStruct) backendVecEnum.nextElement();
      if(selectedIndex == back.listIndex) {
	break;
      }
    }
    
    
    if((e.target == query) || (e.target == mainList)) {
      
      // Debugging
      System.out.println("Backend: " + back.toString());
      // Debugging
      if (NFRInfo.dancyThing != null) NFRInfo.dancyThing.startDancing();


      Frame f = new Frame(st.textorname("Query") + " " + 
			  back.packageName + "/" + back.name);
      repTile r = new repTile();
      
      f.setLayout(r);
      r.setStretchy(0,0);
      Component  h; 
      if ((back.guiType).compareTo("histogram") == 0) {
      	h=new histogram_query(back.packageName + "/" + back.name, 
      			      NFRInfo.document_base , this);
      } else {
	if ((back.guiType).compareTo("list") == 0) {
	  h=new list_query(back.packageName + "/" + back.name,
			   NFRInfo.document_base , this);
	} else {

	  if(back.guiType.equals("")) { 
	    MsgBox.show(st.textorname("internalError"),
			st.textorname("nullGuiType"));
	  } else {
	    MsgBox.show(st.textorname("internalError"),
			st.textorname("unknown"));
	  }
          if (NFRInfo.dancyThing != null) NFRInfo.dancyThing.stopDancing();
	  return(true);
	}
      }

      f.add(h);
      r.setConstraints(0,0,0,0,"",h);
      f.resize(710,545);
      f.show();

      if (NFRInfo.dancyThing != null) NFRInfo.dancyThing.stopDancing();
      
      return(true);
    }
    if(e.target == configure) {
      if (NFRInfo.dancyThing != null) NFRInfo.dancyThing.startDancing();
      
      Frame f = new Frame(st.textorname("Configure") + " " + 
			  back.packageName + "/" + back.name);
      repTile r = new repTile();
      
      f.setLayout(r);
      r.setStretchy(0,0);
      Component  h; 
      System.out.println("type is -" + back.guiType + "-");
      if ((back.guiType).compareTo("histogram") == 0) {
	h=new histogram_configure(back.packageName + "/" + back.name, 
				  NFRInfo.document_base , this);
      } else { 
	if ((back.guiType).compareTo("list") == 0) {
	  h=new list_configure(back.packageName + "/" + back.name,
			       NFRInfo.document_base , this);
	} else {
	  if(back.guiType.equals("")) { 
	    MsgBox.show(st.textorname("internalError"),
			st.textorname("nullGuiType"));
	  } else {
	    MsgBox.show(st.textorname("internalError"),
			st.textorname("unknown"));
	  }
          if (NFRInfo.dancyThing != null) NFRInfo.dancyThing.stopDancing();
	  return(true);
	}
      }
      
      f.add(h);
      r.setConstraints(0,0,0,0,"",h);
      
      f.resize(600,590);
      f.show();

      if (NFRInfo.dancyThing != null) NFRInfo.dancyThing.stopDancing();
      return(true);
    }
    
    return(false);
  }
  

  
  public boolean handleEvent(Event e) {

    if (e.target instanceof List) {
      string = mainList.getSelectedItem();

      switch (e.id) {
      case Event.LIST_SELECT:
	
	// Bug: This is a Kludge

	// A backend
	if(string.startsWith(backendTab)) {
	  query.enable();
	  configure.enable();
	} else {
	  // A Package or a blank
	  query.disable();
	  configure.disable();
	}
	break;

      case Event.LIST_DESELECT:
	query.disable();
	configure.disable();
	break;
      }
    }
    return(super.handleEvent(e));
  }
  
  
  
  // Wraper function for filling out the GridBagConstraints object
  private void gbadd(Container container, Component component, 
		     int grid_x, int grid_y, int grid_width, int
		     grid_height,
		     int fill, int anchor, double weight_x, double
		     weight_y,
		     int top, int left, int bottom, int right)
  {
    GridBagConstraints c = new GridBagConstraints();
    c.gridx = grid_x; c.gridy = grid_y;
    c.gridwidth = grid_width; c.gridheight = grid_height;
    c.fill = fill; c.anchor = anchor;
    c.weightx = weight_x; c.weighty = weight_y;
    if (top+bottom+left+right > 0)
      c.insets = new Insets(top, left, bottom, right);
    
    ((GridBagLayout)container.getLayout()).setConstraints(component, c);
    container.add(component);
  }
  
  
  public static void main(String[] args) {
    System.out.println("PackagesPanel.class");
  }
  
}


