/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   QueryLoader.java
  Author: Eric Wall (eric@nfr.net)

 *****************************************************************************/



import java.awt.*;
import java.util.*;


import nfr.misc.*;


public class QueryLoader extends Frame {
  

  private Label promptLabel;
  private List queryList;
  private Panel buttonPanel;
  private Button load,cancel;
  private String fileName;
  private Cf st;
  private Cf edit;
  private int queryType = 0;

  // Pointers to GUI components for Histogram stuff
  private Checkbox timeAxis,queryTime,queryAllTime,nukeCount;
  private TextField cutOff;
  private TextField[] axis_inputs;
 
  // Pointers for List Stuff
  private TextField[] column_inputs;

  // Pointers for both List and Histogram Stuff
  private AbsoluteTime minTime, maxTime;
  private Vector checkVec, primaryVec, secondaryVec;
  private Checkbox suppressMapping;
  
  // inUse flag to prevent multiple instantiation
  public static boolean inUse;

  public static final int LIST = 1;
  public static final int HISTOGRAM = 2;



  public QueryLoader(String fileName) {

    inUse = true;
    this.fileName = fileName;

    st = new Cf(NFRInfo.document_base,"etc/javastrings/QueryLoader.str");
    st.Read();

    this.setLayout(new GridBagLayout());
    NFRInfo.do_style(this);

    promptLabel = new Label(st.textorname("prompt"));
    gbadd(this,promptLabel,0,0,1,1,
          GridBagConstraints.NONE,
          GridBagConstraints.CENTER,
          0.0,0.1,0,0,0,0);
    
    queryList = new List(5,false);
    queryList.setBackground(Color.white); 
    queryList.setForeground(Color.black);
    fillList(this.fileName);
    gbadd(this,queryList,0,1,1,1,
          GridBagConstraints.NONE,
          GridBagConstraints.CENTER,
          0.0,0.1,0,0,0,0);
    
    buttonPanel = new Panel();
    buttonPanel.setLayout(new GridBagLayout());
    load = new Button(st.textorname("load"));
    load.disable();
    cancel = new Button(st.textorname("cancel"));
    
    gbadd(buttonPanel,load,0,0,1,1,
          GridBagConstraints.NONE,
          GridBagConstraints.CENTER,
          0.0,0.0,0,0,0,0);
    gbadd(buttonPanel,cancel,1,0,1,1,
          GridBagConstraints.NONE,
          GridBagConstraints.CENTER,
          0.0,0.0,0,0,0,0);
    gbadd(this,buttonPanel,0,2,1,1,
          GridBagConstraints.NONE,
          GridBagConstraints.CENTER,
          0.0,0.1,0,0,0,0);
  }



  private void fillList(String fileName) {
    edit = new Cf(NFRInfo.document_base,fileName);
    edit.Read();
    if(edit.isEmpty()) {
      promptLabel.setText(st.textorname("empty"));
    } else {
      Enumeration Keys = edit.keys();
      while(Keys.hasMoreElements()) {
	queryList.addItem((String) Keys.nextElement());
      }
    }
  }

 

  public void loadList(AbsoluteTime minTime,AbsoluteTime maxTime,
		       Checkbox suppressMapping,
		       TextField[] column_inputs, Vector checkVec,
		       Vector primaryVec, Vector secondaryVec)
  {
    this.queryType = QueryLoader.LIST;

    this.minTime = minTime;
    this.maxTime = maxTime;
    this.suppressMapping = suppressMapping;
    this.column_inputs = column_inputs;
    this.checkVec = checkVec;
    this.primaryVec = primaryVec;
    this.secondaryVec = secondaryVec;
    
    this.setTitle(st.textorname("title"));
    this.resize(400,300);
    this.show(); 
  }



  public void loadHistogram(Checkbox timeAxis, Checkbox queryTime,
			    Checkbox queryAllTime, Checkbox nukeCount,
			    TextField cutOff, AbsoluteTime minTime,
			    AbsoluteTime maxTime,
			    Checkbox suppressMapping,
			    TextField[] axis_inputs,
			    Vector checkVec,
			    Vector primaryVec, Vector secondaryVec)
  {
    this.queryType = QueryLoader.HISTOGRAM;

    this.timeAxis = timeAxis;
    this.queryTime = queryTime;
    this.queryAllTime = queryAllTime;
    this.nukeCount = nukeCount;
    this.cutOff = cutOff;
    this.minTime = minTime;
    this.maxTime = maxTime;
    this.suppressMapping = suppressMapping;
    this.axis_inputs = axis_inputs;
    this.checkVec = checkVec;
    this.primaryVec = primaryVec;
    this.secondaryVec = secondaryVec;

    this.setTitle(st.textorname("title"));
    this.resize(400,300);
    this.show(); 
  }



  public void dispose() {
    super.dispose();
    inUse = false;
  }

  

  public static boolean inUse() {
    return(inUse);
  }



  private void loadItList(String name) {
    String tmp;
    Enumeration tmpEnum;
    TextField tmpField;
    Checkbox tmpBox;
    StringTokenizer strTok = new StringTokenizer(edit.getProperty(name),
						 QuerySaver.delimChar);
    // times
    minTime.setEditable(QueryLoader.BVal(strTok.nextToken()));
    minTime.set_value(Long.parseLong(strTok.nextToken()));
    
    maxTime.setEditable(QueryLoader.BVal(strTok.nextToken()));
    maxTime.set_value(Long.parseLong(strTok.nextToken()));

    // suppress mapping
    tmp = strTok.nextToken();
    suppressMapping.setState(QueryLoader.BVal(tmp));

    // Fist page fields
    for(int i=0; i<column_inputs.length; i++) {
      tmp = strTok.nextToken();
      if(tmp.equals("_")) {
	tmp = "";
      }
      column_inputs[i].setText(tmp);
    }

    // Checkboxes for first page
    tmpEnum = checkVec.elements();
    while(tmpEnum.hasMoreElements()) {
      tmpBox = (Checkbox) tmpEnum.nextElement();
      tmp = strTok.nextToken();
      tmpBox.setState(QueryLoader.BVal(tmp));
    }

    // Second page fields
    tmpEnum = primaryVec.elements();
    while(tmpEnum.hasMoreElements()) {
      tmp = strTok.nextToken();
      if(tmp.equals("_")) {
	tmp = "";
      }
      tmpField = (TextField) tmpEnum.nextElement();
      tmpField.setText(tmp);
    }
    
    // Third page fields
    tmpEnum = secondaryVec.elements();
    while(tmpEnum.hasMoreElements()) {
      tmp = strTok.nextToken();
      if(tmp.equals("_")) {
	tmp = "";
      }
      tmpField = (TextField) tmpEnum.nextElement();
      tmpField.setText(tmp);
    }
  }



  private void loadItHistogram(String name) {
    String tmp;
    Enumeration tmpEnum;
    TextField tmpField;
    Checkbox tmpBox;
    StringTokenizer strTok = new StringTokenizer(edit.getProperty(name),"{");

    timeAxis.setState(QueryLoader.BVal(strTok.nextToken()));
    queryTime.setState(QueryLoader.BVal(strTok.nextToken()));
    queryAllTime.setState(QueryLoader.BVal(strTok.nextToken()));
    nukeCount.setState(QueryLoader.BVal(strTok.nextToken()));
    
    tmp = strTok.nextToken();
    if(tmp.equals("_")) {
      tmp = "";
    }
    cutOff.setText(tmp);

    // times
    minTime.setEditable(QueryLoader.BVal(strTok.nextToken()));
    minTime.set_value(Long.parseLong(strTok.nextToken()));

    maxTime.setEditable(QueryLoader.BVal(strTok.nextToken()));
    maxTime.set_value(Long.parseLong(strTok.nextToken()));
    
    // suppress mapping
    tmp = strTok.nextToken();
    suppressMapping.setState(QueryLoader.BVal(tmp));
    
    // First page fields
    for(int i=0; i<axis_inputs.length; i++) {
      tmp = strTok.nextToken();
      if(tmp.equals("_")) {
	tmp = "";
      }
      axis_inputs[i].setText(tmp);
    }

    // first page checkboxes
    tmpEnum = checkVec.elements();
    while(tmpEnum.hasMoreElements()) {
      tmpBox = (Checkbox) tmpEnum.nextElement();
      tmp = strTok.nextToken();
      tmpBox.setState(QueryLoader.BVal(tmp));
    }
    
    // second page fields
    tmpEnum = primaryVec.elements();
    while(tmpEnum.hasMoreElements()) {
      tmp = strTok.nextToken();
      if(tmp.equals("_")) {
	tmp = "";
      }
      tmpField = (TextField) tmpEnum.nextElement();
      tmpField.setText(tmp);
    }
    
    // third page fields
    tmpEnum = secondaryVec.elements();
    while(tmpEnum.hasMoreElements()) {
      tmp = strTok.nextToken();
      if(tmp.equals("_")) {
	tmp = "";
      }
      tmpField = (TextField) tmpEnum.nextElement();
      tmpField.setText(tmp);
    }
  }



 private static boolean BVal(String value) {
    if(value.equals("true")) {
      return(true);
    } else {
      return(false);
    }
  }


  
  public boolean action(Event e, Object arg) {
    if(e.target == load) {

      // Should check for either List or Histogram and throw an error
      // if it is neither
      if(queryType == QueryLoader.LIST) {
	loadItList(queryList.getSelectedItem());
      } else {
	loadItHistogram(queryList.getSelectedItem());
      }

      this.dispose();
      return(true);
    }
    if(e.target == cancel) {
      this.dispose();
      return(true);
    }
    return(false);
  }



  public boolean handleEvent(Event e) {
    if (e.target instanceof List) {
      switch (e.id) {

      case Event.LIST_SELECT:
	load.enable();
        break;
	
      case Event.LIST_DESELECT:
        load.disable();
        break;

      }
    }
    return super.handleEvent(e);
  } 


  
  // Wraper function for filling out the GridBagConstraints object
  private void gbadd(Container container, Component component, 
		     int grid_x, int grid_y, int grid_width, int
		     grid_height,
		     int fill, int anchor, double weight_x, double
		     weight_y,
		     int top, int left, int bottom, int right)
  {
    GridBagConstraints c = new GridBagConstraints();
    c.gridx = grid_x; c.gridy = grid_y;
    c.gridwidth = grid_width; c.gridheight = grid_height;
    c.fill = fill; c.anchor = anchor;
    c.weightx = weight_x; c.weighty = weight_y;
    if (top+bottom+left+right > 0)
      c.insets = new Insets(top, left, bottom, right);
    
    ((GridBagLayout)container.getLayout()).setConstraints(component, c);
    container.add(component);
  }
  
  
  public static void main(String[] args) {
    System.out.println("QueryLoader.class");
  }
  
}


