/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   QuerySaver.java
  Author: Eric Wall (eric@nfr.net)

 *****************************************************************************/



import java.awt.*;
import java.util.*;



import nfr.misc.*;



public class QuerySaver extends Frame {


  private Label promptLabel;
  private TextField nameField;
  private Panel buttonPanel;
  private Button save,cancel;
  private Thread parent;
  private Cf st;
  private String fileName;
  private Vector dataStrings;

  private static boolean inUse;

  public static final String delimChar= "{";


  public QuerySaver(String fileName) {
 
    inUse = true;
    this.fileName = "packages/" + fileName + ".sq";
       
    st = new Cf(NFRInfo.document_base,"etc/javastrings/QuerySaver.str");
    st.Read();
    
    this.setLayout(new GridBagLayout());
    NFRInfo.do_style(this);
    
    promptLabel = new Label(st.textorname("prompt"));
    gbadd(this,promptLabel,0,0,1,1,
          GridBagConstraints.NONE,
          GridBagConstraints.CENTER,
          0.0,0.1,0,0,0,0);

    nameField = new TextField(50);
    gbadd(this,nameField,0,1,1,1,
          GridBagConstraints.NONE,
          GridBagConstraints.CENTER,
          0.0,0.1,0,0,0,0);

    buttonPanel = new Panel();
    buttonPanel.setLayout(new GridBagLayout());
    save = new Button(st.textorname("save"));
    cancel = new Button(st.textorname("cancel"));
    
    gbadd(buttonPanel,save,0,0,1,1,
          GridBagConstraints.NONE,
          GridBagConstraints.CENTER,
          0.0,0.0,0,0,0,0);
    gbadd(buttonPanel,cancel,1,0,1,1,
          GridBagConstraints.NONE,
          GridBagConstraints.CENTER,
          0.0,0.0,0,0,0,0);
    gbadd(this,buttonPanel,0,2,1,1,
          GridBagConstraints.NONE,
          GridBagConstraints.CENTER,
          0.0,0.1,0,0,0,0);
  }



  public void save(Vector dataStrings) {
    int i=0;
    this.dataStrings = dataStrings;

    this.setTitle(st.textorname("title"));
    this.resize(500,200);
    this.show();
  }



  public void dispose() {
    super.dispose();
    inUse = false;
  }



  public static boolean inUse() {
    return(inUse);
  }



  private void saveIt(String name) {
    Cf edit = new Cf(NFRInfo.document_base,fileName);
    String info = "";
    //edit.Read();
    
    // Build Save String
    Enumeration e = dataStrings.elements();
    while(e.hasMoreElements()) {
      String tok = (String) e.nextElement();
      if(tok.equals("")) {
	info += "_" + QuerySaver.delimChar;
      } else {
	info += tok  + QuerySaver.delimChar;
      }
    }

    System.out.println("Saving: " + name + "=" + info);

    edit.put(name,info);
    edit.Merge();
  }



  public boolean action(Event e, Object arg) {
    if(e.target == save) {
      
      // Get name
      String name = nameField.getText();

      if(name.equals("")) {
	MsgBox.show(st.textorname("saveError"),
		    st.textorname("saveErrorMsg"));
	return(true);
      }

      saveIt(name);

      this.dispose();
      return(true);
    }
    if(e.target == cancel) {
      
      // Possibly clean up anything an attempted save has mussed up
      
      this.dispose();
      return(true);
    }
    return(false);
  }


  
  // Wraper function for filling out the GridBagConstraints object
  private void gbadd(Container container, Component component, 
		     int grid_x, int grid_y, int grid_width, int
		     grid_height,
		     int fill, int anchor, double weight_x, double
		     weight_y,
		     int top, int left, int bottom, int right)
  {
    GridBagConstraints c = new GridBagConstraints();
    c.gridx = grid_x; c.gridy = grid_y;
    c.gridwidth = grid_width; c.gridheight = grid_height;
    c.fill = fill; c.anchor = anchor;
    c.weightx = weight_x; c.weighty = weight_y;
    if (top+bottom+left+right > 0)
      c.insets = new Insets(top, left, bottom, right);
    
    ((GridBagLayout)container.getLayout()).setConstraints(component, c);
    container.add(component);
  }
  
  
  public static void main(String[] args) {
    System.out.println("QuerySaver.class");
  }
  
}


