/*
        Copyright(C) 1997 Network Flight Recorder, Inc.
        All rights reserved.

        Use and distribution of this software and its source code
        are governed by the terms and conditions of the
        Network Flight Recorder Software License ("LICENSE.TXT")

*/

package graph;



import java.applet.*;
import java.awt.*;
import java.io.*;
import java.net.URL;
import java.util.*;

// BarGraphViewer:  a type of "viewer" that views bar graphs.  

// the "run" routine that executes on the data input thread which reads
// in chunks of data and merges them into the burgeoning bar graph.  after
// each new chunk of data is merged in, the bar graph is redisplayed.

public class BarGraphViewer extends Viewer
{

  public BarGraphViewer(URL filename) {
    super(filename);
  }

  public BarGraphViewer() {
    super();
  }

  public Graph initGraph(EOFInputStream file) 
       throws EOFException, IOException
  {
    return new BarGraph(file, nfr.misc.NFRInfo.default_font, 
		this.getFontMetrics(nfr.misc.NFRInfo.default_font));
  }

  // run:  the main job of this thread is to read in the rest
  // of the data and update the display in "chunks"
  public void run() {

    //  we happen to know the "graph" is a bar graph
    BarGraph bgraph = null;

	while (bgraph == null)
		{
System.out.println("sleeping waiting for bgraph to be non-null");
		Thread.yield();
		bgraph = (BarGraph)graph;
		}

    System.out.println("run: reading");
    while (file != null) {
System.out.println("bar graph while loop");

      // if we're not done reading in data, go get some more

      try {
System.out.println("bar graph while loop");

	BGDataList list = new BGDataList(file, MAX_CHUNK);

	System.out.println("run: merging");
	bgraph.merge(list);

	display.repaint();

	Thread.yield();

      } catch (java.io.EOFException e) {

	System.out.println("run: done");
	display.repaint();

	// these values are changed by repainting, which calculates
	// the actual number of bars visible
	setScroll(bgraph);

	file = null;
      } catch (java.io.IOException e) {
	System.out.println("ERROR: I/O error or malformed data");
	stop();
      }
	Thread.yield();
    }
  }


/*

These functions work without interfering with the other
event handling.  The question still remains whether we can have a
popup window that follows the cursor around, or if we have to draw 
little squares ourselves.

	public boolean mouseEnter(Event e, int x, int y)
	{
	System.out.println("Mouse Enter");
	return false;
	}

	public boolean mouseExit(Event e, int x, int y)
	{
	System.out.println("Mouse Leave");
	return false;
	}

	public boolean mouseMove(Event e, int x, int y)
	{
	System.out.println("Mouse Move");
	return false;
	}
*/

}
