/*
        Copyright(C) 1997 Network Flight Recorder, Inc.
        All rights reserved.

        Use and distribution of this software and its source code
        are governed by the terms and conditions of the
        Network Flight Recorder Software License ("LICENSE.TXT")

*/

package graph;


import java.awt.*;
import java.util.*;


//  Graph:  abstract class that describes a "graph" a "viewer" wishes to 
//  display.  the specifics of the graph are defined in a derived class, but
//  graphs share certain constant dimensions, are capable of returning a 
//  canvas that displays their contents, and will tell their clients if
//  they are capable of scrolling, and if so will define methods that
//  the clients can use to scroll with.


public abstract class Graph {

  public static final int BORDER = 20;	//  pixel border around graph

  // the x,y axis offsets define the space for axis labels and
  // coordinates
  public static final int GAP = 5;	   // pixel gap between data rows/cols
  public static final int X_AXIS_OFF=80;   // x axis y coord
  public static final int Y_AXIS_OFF=80;   // y axis x coord
  public static final int LAB_HEIGHT=20;   // height of max column label
  public static final int LAB_WIDTH=70;    // height of max column label
  public static final int MAX_LAB_LEN = 20;	// count of label chars 


  /****************************************

   abstract graph methods

   ***************************************/

  // return a canvas representing the graphical representation of
  // this data
  public abstract Canvas getCanvas();

  public abstract Vector getTitles();

  // scroll: scroll the list left or right
  public boolean vScroll (int value) {
    return false;
  }

  // scroll: scroll the list up or down
  public boolean hScroll (int value) {
    return false;
  }

  // hasVscroll: true if this graph has a vertical scroll region
  public boolean hasVscroll() {
    return false;
  }

  // hasHscroll: true if this graph has a horizontal scroll region
  public boolean hasHscroll() {
    return false;
  }

  // getHCount: range of horizontal scroll area
  public int getHCount() {
    return 0;
  }

  // getVCount: range of vertical scroll area
  public int getVCount() {
    return 0;
  }

  // getHVisible: amount of horizontal area visible
  public int getHVisible() {
    return 0;
  }

  // getHVisible: amount of vertical area visible
  public int getVVisible() {
    return 0;
  }


}
