/*
        Copyright(C) 1997 Network Flight Recorder, Inc.
        All rights reserved.

        Use and distribution of this software and its source code
        are governed by the terms and conditions of the
        Network Flight Recorder Software License ("LICENSE.TXT")

*/

package graph;



// PCCanvas: a canvas that holds a pie chart; the pie chart will
// be regenerated whenever the image needs to be refreshed or the
// screen is resized

import java.awt.*;

class PCCanvas extends Canvas {

  private Image i;
  private PieChart pc;
    
  PCCanvas(PieChart pc) {
    // keep track of the plot to which this refers 
    this.pc = pc;
  }

  public Dimension minimumSize() {
    Container parent = getParent();
    Dimension size = parent.size();

    return new Dimension(75, 75);
  }

  public Dimension preferredSize() {
    Container parent = getParent();
    Dimension size = parent.size();

    return new Dimension(100, 100);
  }

  public void update(Graphics g) {
    // since we're doing the whole bitmap, don't clear
    // the background
    paint(g);
  }

  public void paint(Graphics g) {

    // check to see if we need new data in our image
    boolean regen = pc.updated();

    int w = size().width;
    int h = size().height;

    if (i == null || 
	(i.getWidth(null) != w || 
	 i.getHeight(null) != h)) {

      if (i != null)
	i.flush();

      // create a new cached image
      i = createImage(w, h);

      regen = true;
    }

    if (regen) {

      // size has changed, or contents... regenerate our
      // cached bitmap
      Graphics ig = i.getGraphics();

      ig.clearRect(0, 0, w, h);
      pc.display(ig, this);

      ig.dispose();
    }

    g.drawImage(i, 0, 0, null);
  }
}
