/*
        Copyright(C) 1997 Network Flight Recorder, Inc.
        All rights reserved.

        Use and distribution of this software and its source code
        are governed by the terms and conditions of the
        Network Flight Recorder Software License ("LICENSE.TXT")

*/

package graph;



import java.awt.*;
import java.io.*;
import java.net.URL;
import java.util.*;

// PieViewer:  a type of "viewer" that views pie chart graphs.  

// the "run" routine that executes on the data input thread which reads
// in chunks of data and merges them into the burgeoning pie chart.  after
// each new chunk of data is merged in, the pie chart is redisplayed.

public class PieViewer extends Viewer
{

  public PieViewer(URL filename) {
    super(filename);
  }

  public PieViewer() {
    super();
  }

  public Graph initGraph(EOFInputStream file) 
       throws EOFException, IOException
  {
    return new PieChart(file, MAX_CHUNK);
  }

  // run:  the main job of this thread is to read in the rest
  // of the data and update the display in "chunks"
  public void run() {

    //  we happen to know the "graph" is a pie chart
    PieChart pchart = null;

	while (pchart == null)
		{
		Thread.yield();
		pchart = (PieChart)graph;
		}

    System.out.println("run: reading");
    while (file != null) {

      // if we're not done reading in data, go get some more

      try {

	System.out.println("run: merging");
	BGDataList list = new BGDataList(file, MAX_CHUNK);

	pchart.merge(list);
	display.repaint();

	Thread.yield();

      } catch (java.io.EOFException e) {

	System.out.println("run: done");
	display.repaint();

	file = null;
      } catch (java.io.IOException e) {
	System.out.println("ERROR: I/O error or malformed data");
	stop();
      }
	Thread.yield();
    }
  }

}
