/*
        Copyright(C) 1997 Network Flight Recorder, Inc.
        All rights reserved.

        Use and distribution of this software and its source code
        are governed by the terms and conditions of the
        Network Flight Recorder Software License ("LICENSE.TXT")

*/

package graph;


import java.awt.*;

// PCCanvas: a canvas that holds a scatter plot; the scatter plot will
// be regenerated whenever the image needs to be refreshed or the
// screen is resized

class SPCanvas extends Canvas {

  private Image i;
  private ScatterPlot sp;
    
  SPCanvas(ScatterPlot sp) {
    // keep track of the plot to which this refers 
    this.sp = sp;
  }

  public Dimension minimumSize() {

    return new Dimension(75, 75);
  }

  public Dimension preferredSize() {
    Container parent = getParent();
    Dimension size = parent.size();

    return new Dimension(100, 100);
  }

  public void update(Graphics g) {
    // since we're doing the whole bitmap, don't clear
    // the background
    paint(g);
  }

  public void paint(Graphics g) {

    // check to see if we need new data in our image
    boolean regen = sp.updated();
    boolean resize = false;

    int w = size().width;
    int h = size().height;

    if (i == null || 
	(i.getWidth(null) != w || 
	 i.getHeight(null) != h)) {

      if (i != null)
	i.flush();

      // create a new cached image
      i = createImage(w, h);

      // force regeneration of x and y label font metrics
      sp.label_fm = null;
      sp.yidx = 0;

      regen = true;
    }

    if (regen) {

      // size has changed, or contents... regenerate our
      // cached bitmap
      Graphics ig = i.getGraphics();

      ig.clearRect(0, 0, w, h);
      sp.display(ig, this);

      ig.dispose();
    }

    g.drawImage(i, 0, 0, null);
  }

  public void resize(Dimension d) {
	super.resize(d);
System.out.println("resize d");
  }

  public void resize(int w, int h) {
	super.resize(w,h);
System.out.println("resize int");
  }

  public void reshape(int x, int y, int w, int h ) {
	super.reshape(x,y,w,h);
System.out.println("reshape "+x+" "+y+" "+w+" "+h);

	repaint();
  }

}

