/*
        Copyright(C) 1997 Network Flight Recorder, Inc.
        All rights reserved.

        Use and distribution of this software and its source code
        are governed by the terms and conditions of the
        Network Flight Recorder Software License ("LICENSE.TXT")

*/

package graph;



import java.awt.*;

// ScaledTextCanvas: a canvas whose job it is to hold onto some scaled
// text that needs to be redrawn every time we repaint

class ScaledTextCanvas extends Canvas {

  private final int ST_MIN_HEIGHT=20;
  private final int ST_MIN_WIDTH=20;

  ScaledTextVector sv;
  Image i;
  Dimension size;
  int xoff;

  ScaledTextCanvas(ScaledTextVector v) {
    this.sv = v;
  }

  public Dimension minimumSize() {
    Container parent = getParent();
    Dimension size = parent.size();

    return new Dimension(size.width, size.height/7);
  }

  public Dimension preferredSize() {
    Container parent = getParent();
    Dimension size = parent.size();

    return new Dimension(size.width, size.height/6);
  }

  public void update(Graphics g) {
    // don't clear background, since we're just
    // going to write over it
    paint(g);
  }

  public void paint(Graphics g) {

    Dimension cur_size = size();
    boolean redo_image = (i == null) || 
      (size == null) ||
      !(size.width == cur_size.width && size.height == cur_size.height);

    // if we haven't changed size, use the cached image
    if (redo_image) {

      Font f_save = g.getFont();

      // calculate a new scaled image from our string vector
      Image ni = sv.getImage(this, g, cur_size.width, cur_size.height);
      if (i != null)
	i.flush();
      i = ni;

      size = new Dimension(cur_size.width, cur_size.height);
      xoff = (cur_size.width - sv.getWidth()) / 2;
      g.setFont(f_save);
    }

    // update the image on the screen
    g.drawImage(i, xoff, 0, null);
  }
}

