/*
        Copyright(C) 1997 Network Flight Recorder, Inc.
        All rights reserved.

        Use and distribution of this software and its source code
        are governed by the terms and conditions of the
        Network Flight Recorder Software License ("LICENSE.TXT")

*/

package graph;



import java.applet.*;
import java.awt.*;
import java.io.*;
import java.net.URL;
import java.util.*;

// ScatterViewer:  a type of "viewer" that views scatter plots.  

// the "run" routine that executes on the data input thread which reads
// in chunks of data and merges them into the burgeoning scatter plot.  after
// each new chunk of data is merged in, the scatter plot is redisplayed.


public class ScatterViewer extends Viewer
{

  public ScatterViewer(URL filename) {
    super(filename);
  }

  public ScatterViewer() {
    super();
  }

  public Graph initGraph(EOFInputStream file) 
       throws EOFException, IOException
  {
    return new ScatterPlot(file, MAX_CHUNK);
  }

  // run:  the main job of this thread is to read in the rest
  // of the data and update the display in "chunks"
  public void run() {

    //  we happen to know the "graph" is a scatter plot
    ScatterPlot splot = null;

	while (splot == null)
		{
		Thread.yield();
		splot = (ScatterPlot)graph;
		}

    System.out.println("run: reading");
    while (file != null) {

      // if we're not done reading in data, go get some more

      try {

	System.out.println("run: merging");
System.out.println("ScatterViewer: splot = "+splot);
	splot.merge(file);
	display.repaint();

	// these values are changed by repainting, which calculates
	// the actual number of bars visible
	setScroll(splot);

	Thread.yield();

      } catch (java.io.EOFException e) {

	System.out.println("run: done");
	display.repaint();
	setScroll(splot);

	file = null;
      } catch (java.io.IOException e) {
	System.out.println("ERROR: I/O error or malformed data");
	stop();
      }
	Thread.yield();
    }
  }

}
