/*
        Copyright(C) 1997 Network Flight Recorder, Inc.
        All rights reserved.

        Use and distribution of this software and its source code
        are governed by the terms and conditions of the
        Network Flight Recorder Software License ("LICENSE.TXT")

By Mark Sienkiewicz / NFR

*/

import java.awt.*;
import nfr.misc.*;
import java.util.*;
import nfr.rc.*;

public class histogram_configure extends Panel {
	Button submit_button,reload_button,close_button;
	TextField title;
	TextArea n_code, description;
	Panel axis_list_parent;
	Panel axis_list_widget;
	CardLayout the_card;
	Panel card_panel;
	Checkbox recorder_enabled;
	Choice new_axis_record;
	Choice data_selection;
	Font subtitlefont;
	TextField data_label;
	TextField count_label;
	Checkbox do_alerts;
	IntervalTime rollover_s;
	Label originlabel,modified_label;
	boolean ever_inited= false;
	String my_name;
        static Cf st;
	Cf hist_conf;
	String n_code_string, description_string;

	//
	// all this is about the alert on new cell
	AlertRes alert_object;
	ResConfigurator alert_configurator;
	Panel alert_panel;
	repTile alert_reptile;
	int alert_row;
	Label must_save_alert_label;

        type_mapping type_map;


        RollPanel sp;



/************************************************************
* this is the constructor.  pass in the name of the recorder
* to configure, the document base, and the ancestor window
* that you will eventually add this to.
*/
public histogram_configure(java.lang.String s, java.net.URL docbase, 
	Component ancestor) {

		my_name = s;

		NFRInfo.do_style(this);

	// load the string table

		if (st == null)
			{
			Cf tmp = new Cf(NFRInfo.document_base
				,"etc/javastrings/histogramc");
			tmp.Read();
			st=tmp;
			}

	// load the type mapping (recordable:primary type:secondary type:rep
		type_map = new type_mapping(NFRInfo.document_base
				);

		int row;
		Label l;
		Panel p,p1,q;
		repTile w1,w3;

	// start building the window

		// get a bigger font
		subtitlefont = ancestor.getFont();
		subtitlefont = new Font (subtitlefont.getName(), Font.BOLD, 
				subtitlefont.getSize()+2);

		w1=new repTile();

		setLayout(w1);	// make this widget the top level of the applet

		w1.setborder(4,4,4,4); // a little bit of border around window

// row 0
		row=0;
	// histogram name label
		l = new Label(st.textorname("name"));
		add(l);
		w1.setConstraints(row,0,row,0,"",l);

	// histogram name field
		l=new Label(my_name);
		add(l);
		w1.setConstraints(row,1,row,2,"",l);

// row
		row++;
	// histogram title label
		l = new Label(st.textorname("title"));
		add(l);
		w1.setConstraints(row,0,row,0,"",l);

	// histogram title field
		title=new TextField("This is the title");
		add(title);
		w1.setConstraints(row,1,row,4,"",title);

// row
		row++;
	// origin label
		l = new Label(st.textorname("origin"));
		add(l);
		w1.setConstraints(row,0,row,0,"",l);

		originlabel = new Label("");
		add(originlabel);
		w1.setConstraints(row,1,row,1,"",originlabel);

		modified_label = new Label("");
		add(modified_label);
		modified_label.setForeground(Color.red);
		w1.setConstraints(row,2,row,2,"",modified_label);

// row
		row++;

		recorder_enabled = new Checkbox(st.textorname("enabled"));
		add(recorder_enabled);
		w1.setConstraints(row,0,row,2,"fl",recorder_enabled);

// row
		row ++;
		w1.setStretchy(row,1);

		tabthing tabbie = new tabthing(10);
		add(tabbie);
		w1.setConstraints(row,0,row,2,"",tabbie);

	// first card - description
		q = new Panel();
		tabbie.addThing(q,st.textorname("descriptab"));
		w3=new repTile();
		q.setLayout(w3);

		// label
		l = new Label(st.textorname("descriptitle"));
		l.setFont(subtitlefont);
		q.add(l);
		w3.setConstraints(0,0,0,1,"ch",l);

		// text area in rest of card
		description = new TextArea();
		q.add(description);
		w3.setConstraints(2,0,2,1,"",description);

		w3.setStretchy(2,1);

	// a card - data to record

                q = new Panel();
		tabbie.addThing(q,st.textorname("datatab"));
                w3=new repTile();
                q.setLayout(w3);

                // title
                l=new Label(st.textorname("datatitle"));
                l.setFont(subtitlefont);
                q.add(l);
                w3.setConstraints(0,0,0,3,"ch",l);

		// card row 1

		// label of it's name
		l=new Label(st.textorname("sumlabel"));
		q.add(l);
		w3.setConstraints(1,2,1,2,"",l);

		//  sum
		data_label=new TextField("");
		q.add(data_label);
		w3.setConstraints(1,3,1,3,"",data_label);
		
		// label of count label
		l=new Label(st.textorname("countlabel"));
		q.add(l);
		w3.setConstraints(2,2,2,2,"",l);

		// count labels name
		count_label=new TextField("");
		q.add(count_label);
		w3.setConstraints(2,3,2,3,"",count_label);

		w3.setStretchy(-1,3);

		// rollover period
		l = new Label(st.textorname("rollover"));
		q.add(l);
		w3.setConstraints(3,2,3,2,"",l);

		// rollover select box:
		rollover_s = new IntervalTime(true, 60, 3600);
		q.add(rollover_s);
		w3.setConstraints(3,3,3,3,"fl",rollover_s);

	// a card - N code
		q = new Panel();
		tabbie.addThing(q,st.textorname("filtertab"));
		w3=new repTile();
		q.setLayout(w3);

		// label
		l = new Label(st.textorname("ncodepage"));
		l.setFont(subtitlefont);
		q.add(l);
		w3.setConstraints(0,0,0,1,"ch",l);

		// text area in rest of card
		n_code = new TextArea();
		q.add(n_code);
		w3.setConstraints(2,0,2,1,"",n_code);

		w3.setStretchy(2,1);

	// a card - axes
		q = new Panel();
		tabbie.addThing(q,st.textorname("axestab"));
		w3=new repTile();
		q.setLayout(w3);

		// title
		l=new Label(st.textorname("axistitle"));
		l.setFont(subtitlefont);
		q.add(l);
		w3.setConstraints(0,0,0,1,"ch",l);

		axis_list_parent = q;
		axis_list_widget = new Panel();
		q.add(axis_list_widget);
		w3.setConstraints(1,0,1,1,"",axis_list_widget);

		w3.setStretchy(1,1);


	// a card - alerts
		q = new Panel();
		tabbie.addThing(q,st.textorname("alert"));
		w3 = new repTile();
		q.setLayout(w3);

		// title
		l=new Label(st.textorname("alerttitle"));
		l.setFont(subtitlefont);
		q.add(l);
		w3.setConstraints(0,0,0,4,"ch",l);

		// alert on new cell message:
		do_alerts = new Checkbox(st.textorname("alertcheck"));
		q.add(do_alerts);
		w3.setConstraints(1,0,1,3,"fl",do_alerts);

		alert_panel = q;
		alert_reptile = w3;
		alert_row = 3;

		must_save_alert_label = new Label(" ");
		q.add(must_save_alert_label);
		w3.setConstraints(alert_row+2,0,alert_row+2,3,"ch",
			must_save_alert_label);

        // a card - Roll Over Config Stuffs
		sp = new RollPanel();
		tabbie.addThing(sp,st.textorname("rollpanel"));

	// end of tab cards
		tabbie.compute();


// row - submit button
		row++;

	// submit
		submit_button = new Button(st.textorname("submit"));
		add(submit_button);
		w1.setConstraints(row,0,row,2,"fl",submit_button);

	// close
		close_button = new Button(st.textorname("close"));
		add(close_button);
		w1.setConstraints(row,0,row,2,"ch",close_button);

	// reload
		reload_button = new Button(st.textorname("reload"));
		add(reload_button);
		w1.setConstraints(row,1,row,2,"fr",reload_button);


	act_on_load();
	}



/****
* no special paint instructions yet.
        public void paint(Graphics gc) {
	}
*/

/****
* all events come here
*/
	public boolean handleEvent (Event e) {
		if (e.id == Event.ACTION_EVENT) {
			if ( e.target == submit_button) {
				act_on_submit();
				System.out.println("submit!");
				return true;
			} else if ( e.target == close_button ) {
				destroy_parent();
				return true;
			} else if ( e.target == reload_button ) {
				act_on_load();
				System.out.println("reload!");
				return true;
			}

		return false;
		} else if (e.id == Event.WINDOW_DESTROY) {
			System.out.println("destroy event: "+e.toString());
			destroy_parent();
			return true;
		} else {
			return false;
		}
	}


/****
* if close button hit, we walk up to the top level frame and
* delete it.
*/
	public void destroy_parent() {
		Component c;
		c=this;
		while ( (c != null) && (! (c instanceof Frame)) )
			c= c.getParent();
		if (c != null)
			((Frame)c).dispose();
	}

/****
* set a value in the configuration file, return true if it is changed
* from previous value
*/
	boolean conf_update(String name, String value) {
		String s;
		s=hist_conf.getProperty(name);
		if (s == null) {
			hist_conf.put(name,value);
			return true;
		}
		if (s.compareTo(value) != 0) {
			hist_conf.put(name,value);
			return true;
		}
		return false;
	}

/****
* handle the submit button.
* for each object in the ui, update the associated configuration file entry
* set the modified entry if anything changed.
*/
	private void act_on_submit() {
		boolean change=false;
                if (NFRInfo.dancyThing != null) NFRInfo.dancyThing.startDancing();

		// COPY
		change |= conf_update("rollover_size_val",sp.rtValField.getText());
		change |= conf_update("rollover_time_val",sp.rsValField.getText());
		change |= conf_update("archive_path",sp.archivePathField.getText());
		
		if(sp.rtCB1.getState()) {
		  change |= conf_update("rollover_size","YES");
		} else {
		  change |= conf_update("rollover_size","NO");
		}

		if(sp.rsCB1.getState()) {
		  change |= conf_update("rollover_time","YES");
		} else {
		  change |= conf_update("rollover_time","NO");
		}
		// COPY

		change |= conf_update("title",title.getText());
		change |= conf_update("data_label",data_label.getText());
		change |= conf_update("count_label",count_label.getText());
		change |= conf_update("new_cell_alerts", do_alerts.getState()
			? "true" : "false" );
		int t;

		t=(int) rollover_s.get_value();

		change |= conf_update("rollover", Integer.toString(t));

	// enable/disable does not count as a change
		hist_conf.put("enabled", recorder_enabled.getState()
			? "true" : "false" );

		String tmp = n_code.getText();
		if (tmp.compareTo(n_code_string) != 0)
			{
			change = true;
			remote_file_io.save_file( NFRInfo.document_base ,
			"packages/"+my_name+".nfr", tmp);
			}

		tmp = description.getText();
		if (tmp.compareTo(description_string) != 0)
			{
			change = true;
			remote_file_io.save_file( NFRInfo.document_base ,
			"packages/"+my_name+".desc", tmp);
			}

		alert_configurator.write(alert_object);
		if (alert_object.getState() != alert_object.RESID_STATE_STABLE)
			{
			change = true;
			alert_object.save();
			}

		if (change) {
		  hist_conf.put("modified","true");
		  sp.cfversion++;
		  change |= conf_update("cfversion",Integer.toString(sp.cfversion));
		}

		hist_conf.Merge();
		System.out.println("submit finished - reloading");
		act_on_load();
                if (NFRInfo.dancyThing != null) NFRInfo.dancyThing.stopDancing();
	}

/****
* load the configuration file from the server.  set all the values
* in the objects on the display
*/
	private void act_on_load() {
		String s;
                if (NFRInfo.dancyThing != null) NFRInfo.dancyThing.startDancing();

	// read the recorder config file
		hist_conf = new Cf(NFRInfo.document_base
				,"packages/"
			+my_name+".cfg");
		if (hist_conf.Read())
			System.out.println("load cf true");
		else
			System.out.println("load cf false");

	// set about stuffing gui objects

		title.setText(hist_conf.getProperty("title"));

		s=hist_conf.getProperty("origin");
		if (s == null) s="unknown";
		originlabel.setText(s);

		s=hist_conf.getProperty("modified");
		if ((s == null) || (s.compareTo("false") != 0))
			s=st.textorname("modified");
		else
			s="";
		modified_label.setText(s);

		// COPY
		s=hist_conf.getProperty("cfversion");
		if(s == null) s="-1";
		try {
		  sp.cfversion = Integer.parseInt(s);
		} catch (Exception e) {
		  System.out.println("Caught an exception in");
		  System.out.println("histogram_configure.act_on_load()");
		  sp.cfversion = -1;
		}

		s=hist_conf.getProperty("rollover_size");
		if(s != null) { 
		  if(s.equals("YES")) {
		    sp.rsCB1.setState(true);
		    sp.rsValField.enable();
		  } else {
		    sp.rsCB2.setState(true);
		    sp.rsValField.disable();
		  }
		}
		
		s=hist_conf.getProperty("rollover_size_val");
		if(s == null) s="Unknown";
		sp.rsValField.setText(s);
		
		s=hist_conf.getProperty("rollover_time");
		if(s != null) { 
		  if(s.equals("YES")) {
		    sp.rtCB1.setState(true);
		    sp.rtValField.enable();
		  } else {
		    sp.rtCB2.setState(true);
		    sp.rtValField.disable();
		  }
		}

		s=hist_conf.getProperty("rollover_time_val");
		if(s == null) s="Unknown";
		sp.rtValField.setText(s);

		s=hist_conf.getProperty("archive_path");
		if(s == null) s="Unknown";
		sp.archivePathField.setText(s);
		// COPY

		// data_selection - what choices of data are

		data_label.setText(hist_conf.getProperty("data_label"));

		count_label.setText(hist_conf.getProperty("count_label"));

		

		// do_alerts - whether to alert
		do_alerts.setState(cfg_true("new_cell_alerts"));

		{
		/*
		* convert name into valid resource name
		*/
		int slash = my_name.indexOf("/");
		String tname = my_name.substring(0,slash) +
			"_" + my_name.substring(slash+1);
		
		/*
		* load the alert from the config file
		*/
		alert_object = AlertRes.load(
			"packages_"+tname+"_new_alert",
			NFRInfo.usr_resource_file_name);
		System.out.println(" new alert object - " + alert_object.getState());

		/*
		* If there is no resource for this object, set it from
		* the config file.  This means we might display an alert
		* that isn't actually possible until after the user 
		* hits "save".  This only affects imported packages.
		* bug: fix this to be less confusing to the user, 
		* maybe in the next release if we don't get to it.
		*/
System.out.println("ALERT STUFF: "+alert_object.getState());
		if (alert_object.getState() == alert_object.RESID_STATE_ADDED)
			{
			String alstr = hist_conf.getProperty("new_alert_text");
			if (alstr != null)
				alert_object.setText(alstr);
			must_save_alert_label.setText(st.textorname
				("must_save_alert"));
			}
		else
			must_save_alert_label.setText(" ");
			
		/*
		* delete the old GUI objects
		*/
		if (alert_configurator != null)
			{
			alert_panel.remove(alert_configurator);
			alert_reptile.removeLayoutComponent(alert_configurator);
			}

		/*
		* create/add new GUI objects
		*/
		alert_configurator = alert_object.getConfigurator();
		alert_panel.add(alert_configurator);
		alert_reptile.setConstraints(alert_row,0,alert_row,3, "",alert_configurator);
		alert_reptile.setStretchy(alert_row, 2);
		alert_panel.invalidate();
		alert_panel.validate();
		}

		rollover_s.set_value(hist_conf.getProperty("rollover"));
		rollover_s.displayTime();

		recorder_enabled.setState(cfg_true("enabled"));

		load_axis_list();
/*
n_code - actual n code

axis_list - list of current axes; not right form yet

*/

		n_code_string = remote_file_io.load_file( NFRInfo.document_base
				, "packages/"+my_name+".nfr");
		n_code.setText(n_code_string);

		description_string = remote_file_io.load_file( 
			NFRInfo.document_base , "packages/"+my_name+".desc");
		description.setText(description_string);

	invalidate();
	validate();
                if (NFRInfo.dancyThing != null) NFRInfo.dancyThing.stopDancing();
	}

/****
* get a true/false value from the config file, return what it is
*/
	boolean cfg_true(String s) {
		s=hist_conf.getProperty(s);
		if ((s == null) || (s.compareTo("true") != 0))
			return false;
		else
			return true;
	}

/****
* make the axis list widget look right
*/
	void load_axis_list() {
		String s=hist_conf.getProperty("num_columns");
		int num=0;
		try {
			num = Integer.parseInt(s);
		} catch (Exception e) {
			num = 0;
System.out.println("load_axis_list - exception: " + e.toString() );
		}
		// remove old panel 
		axis_list_parent.remove(axis_list_widget);

		// create new panel
		axis_list_widget = new Panel();

		// add new panel to old card
		axis_list_parent.add(axis_list_widget);

		// set new panels constraints in the old card's layout
		repTile r = (repTile) axis_list_parent.getLayout();
		r.setConstraints(1,0,1,1,"",axis_list_widget);

		// make layout for new panel
		r = new repTile();
		axis_list_widget.setLayout(r);
		Label lnum, lnam, lrec, ltyp;
		lnum = new Label(st.textorname("axnum"));
		lnam = new Label(st.textorname("axname"));
		lrec = new Label(st.textorname("axrec"));
		ltyp = new Label(st.textorname("axtyp"));

			axis_list_widget.add(lnum);
			axis_list_widget.add(lnam);
			axis_list_widget.add(lrec);
			axis_list_widget.add(ltyp);

		r.setConstraints(0,0,0,0,"",lnum);
		r.setConstraints(0,1,0,1,"",lrec);
		r.setConstraints(0,2,0,2,"",ltyp);
		r.setConstraints(0,3,0,3,"",lnam);

		r.setStretchy(-1,3);	// name field is stretchy

		for (int x = 1; x <= num; x++) {
			// get names of config file keywords
			String type_name = "column_"+x+"_type";
			String label_name = "column_"+x+"_label";
			String recordable_name = "column_"+x+"_recordable";
			
			// get values from config file
			String type = hist_conf.getProperty(type_name);
			String recordable = hist_conf
					.getProperty(recordable_name);
			String name = hist_conf.getProperty(label_name);

			// make labels for each column
			lnum = new Label(""+x);
			lrec = new Label(recordable);
			ltyp = new Label(type);
			lnam = new Label(name);

			// add the labels
			axis_list_widget.add(lnum);
			axis_list_widget.add(lrec);
			axis_list_widget.add(ltyp);
			axis_list_widget.add(lnam);
			
			// stick each label in the right row/column
			r.setConstraints(x,0,x,0,"",lnum);
			r.setConstraints(x,1,x,1,"",lrec);
			r.setConstraints(x,2,x,2,"",ltyp);
			r.setConstraints(x,3,x,3,"",lnam);
		}
		axis_list_parent.invalidate();
		axis_list_parent.validate();
	}

}

