/*
        Copyright(C) 1997 Network Flight Recorder, Inc.
        All rights reserved.

        Use and distribution of this software and its source code
        are governed by the terms and conditions of the
        Network Flight Recorder Software License ("LICENSE.TXT")

By Mark Sienkiewicz / NFR

*/

import java.io.*;
import java.awt.*;
import nfr.misc.*;
import java.util.*;
import java.net.*;
import graph.*;

public class histogram_query extends Panel {
	// buttons at the bottom of the screen
	Button load_button, save_button, close_button;
	Button text_button, bar_button, pie_button, scatter_button;
	Button excel_button, html_button;

	Checkbox nuke_count;

	//
	// this object is a bunch of panels placed into a tabthing
	//

        Checkbox suppress_mapping;
        Checkbox query_time;
        Checkbox query_all_time;

	//
	// axis_list_parent is a panel that will be the tab that lists
	//	the axes.  it contains a lable and a panel with the list
	// axis_list_widget is the actual panel with the list.  it is
	//	destroyed, regenerated, and re-added to axis_list_parent
	//	by load_axis_list()

	Panel axis_list_parent;
	Panel axis_list_widget;

	//
	// axis_inputs[] is the input field for the user to type query
	//	values for each axis
	//
	TextField axis_inputs[];	// text fields for each axis

	//
	// these are two panel-like-objects that contain a list of
	// primary types and secondary types the user can query
	//
	types_panel primary_widget, secondary_widget;

	//
	// how many axes there are
	//
	int num_axes;


	//
	// tables mapping axis number (represented as a string) to
	//	- a TextField for the users query
	//	- a Checkbox whether to display this axis in the results
	//	- the primary type of the axis given in the config file
	ordered_table axis_text_list;
	ordered_table axis_check_list;

	String [] axis_types;

	//
	// font to use at the top of a particular card of the tabthing
	//
	Font subtitlefont;

	//
	// name of the recorder we are querying

	String my_name;

	//
	// string table, for multi-language ability
static	Cf st = null;

	//
	// config file for this instance of the recorder
	//
	Cf hist_conf;

	//
	// URL we were invoked as
	//
	java.net.URL my_docbase;

	//
	// recordable/primary/secondary/representation type table
	//
	type_mapping type_map;

	//
	// use time as an axis?
	//
	Checkbox time_checkbox;

	//
	// nukeage box - what values to discard
	//
	TextField nukeage_box;

  //
  // Time input components
  //
  AbsoluteTime min_time, max_time;

  //
  // Object to handle query saving
  //
  QuerySaver saver;

  //
  // Object to handle query loading
  //
  QueryLoader loader;



/************************************************************
* this is the constructor.  pass in the name of the recorder
* to configure, the document base, and the ancestor window
* that you will eventually add this to.
*/
public histogram_query(java.lang.String s, java.net.URL docbase, 
	Component ancestor) {

		my_name = s;

		NFRInfo.do_style(this);

	// load the string table
		if (docbase == null)
			System.out.println("docbase is null");
		my_docbase=docbase;
		if (st == null)
			{
			st = new Cf(docbase,"etc/javastrings/histogramq1");
			st.Read();
			}

	// load the type mapping (recordable:primary type:secondary type:rep)
		type_map = new type_mapping(docbase);

		int row;
		Label l;
		Panel p1,q;
		repTile w1,w3;

	// start building the window

	System.out.println("in constructor - name is " + my_name);

		// get a bigger font
		subtitlefont = ancestor.getFont();
		subtitlefont = new Font (subtitlefont.getName(), Font.BOLD, 
				subtitlefont.getSize()+2);

		w1=new repTile();

		setLayout(w1);	// make this widget the top level of the applet

		w1.setStretchy(3,1);

		w1.setborder(4,4,4,4); // a little bit of border around window

	// read the recorder config file
		hist_conf = new Cf(my_docbase,"packages/"
			+my_name+".cfg");
		if (hist_conf.Read())
			System.out.println("load cf true");
		else
			System.out.println("load cf false");

// row 0
		row=0;
	// histogram name label
		l = new Label(st.textorname("name"));
		add(l);
		w1.setConstraints(row,0,row,0,"",l);

	// histogram name field
		l=new Label(my_name);
		add(l);
		w1.setConstraints(row,1,row,2,"",l);

// row 1
		row=1;
	// histogram title label
		l = new Label(st.textorname("title"));
		add(l);
		w1.setConstraints(row,0,row,0,"",l);

	// histogram title field
		l=new Label(hist_conf.getProperty("title"));
		add(l);
		w1.setConstraints(row,1,row,4,"",l);

// row 2
		row=2;

// row 3 - a tab panel that is the big blob in the middle.

		row=3;

		tabthing tabbie = new tabthing(10);
		add(tabbie);
		w1.setConstraints(row,0,row,2,"",tabbie);

	// a card - axis values to match
		q = new Panel();
		tabbie.addThing(q,st.textorname("axistab"));
		w3=new repTile();
		q.setLayout(w3);

		// label
		l = new Label(st.textorname("axistitle"));
		l.setFont(subtitlefont);
		q.add(l);
		w3.setConstraints(0,0,0,3,"ch",l);

		axis_list_parent = q;

		axis_list_widget = new Panel();
		add(axis_list_widget);
		w3.setConstraints(1,0,1,3,"",axis_list_widget);

		time_checkbox = new Checkbox(st.textorname("timeaxis"));
		q.add(time_checkbox);
		w3.setConstraints(2,0,2,3,"fl",time_checkbox);

		load_axis_list();

		{
		Panel p = new Panel();
		q.add(p);
		w3.setConstraints(3,0,3,5,"",p);
		repTile r = new repTile();
		p.setLayout(r);

		l = new Label(st.textorname("mintime"));
		p.add(l);
		r.setConstraints(3,0,3,0,"ft",l);
		min_time = new AbsoluteTime(false,60,st.textorname("minearly"),
			st.textorname("mintimeuse"));
		p.add(min_time);
		r.setConstraints(3,1,3,1,"",min_time);

		l = new Label(st.textorname("maxtime"));
		p.add(l);
		r.setConstraints(4,0,4,0,"ft",l);

		max_time = new AbsoluteTime(false,60, st.textorname("maxlate"),
			st.textorname("maxtimeuse"));
		max_time.set_value(1262322000); /* jan 1, 2010 ; arbitrary */
		max_time.displayDate();
		p.add(max_time);
		r.setConstraints(4,1,4,1,"",max_time);
		}


		{
		Panel p = new Panel();
		q.add(p);
		w3.setConstraints(5,0,5,5,"",p);
		repTile r = new repTile();
		p.setLayout(r);

		l = new Label(st.textorname("nukeage"));
		p.add(l);
		r.setConstraints(0,0,0,0,"",l);

		nukeage_box = new TextField(10);
		p.add(nukeage_box);
		r.setConstraints(0,1,0,1,"",nukeage_box);

		l = new Label(st.textorname("nukageof"));
		p.add(l);
		r.setConstraints(0,2,0,2,"",l);

		CheckboxGroup cbg = new CheckboxGroup();

		// you can always know which is checked by looking at one
		// of them.  we only let nuke_count be vibible to the rest
		// of the program because sometimes the sum isn't there

		nuke_count = new Checkbox(hist_conf.getProperty("count_label"),
			cbg,true);
		p.add(nuke_count);
		r.setConstraints(0,3,0,3,"",nuke_count);

		if (hist_conf.getProperty("data") != null)
			{
			Checkbox nuke_sum = 
			new Checkbox(hist_conf.getProperty("data_label"),
				cbg, false);
			p.add(nuke_sum);
			r.setConstraints(0,4,0,4,"",nuke_sum);
			}
		}

                suppress_mapping = new Checkbox(
                        st.textorname("suppressmapping"), null, false );

                q.add(suppress_mapping);
                w3.setConstraints(6,0,6,0,"",suppress_mapping);

		w3.setStretchy(-1,1);

		{
		Panel p = new Panel();
		q.add(p);
		w3.setConstraints(7,0,7,5,"",p);
		repTile r = new repTile();
		p.setLayout(r);
		boolean use = true;

		CheckboxGroup cbg = new CheckboxGroup();

		query_time = new Checkbox(st.textorname("by_time"), cbg,use);
		if (hist_conf.getProperty("suppress_current") == null)
		  {
		    use= false;
		    p.add(query_time);
		    r.setConstraints(0,0,0,0,"",query_time);
		  }
		
		query_all_time = new Checkbox(st.textorname("all_time"),cbg,use);
		if (hist_conf.getProperty("suppress_all_time") == null)
		  {
		    use= false;
		    p.add(query_all_time);
		    r.setConstraints(0,1,0,1,"",query_all_time);
		  }
		
		if (use) 
		  {
		    /* impossible to have data in configuration... */
		    /* what to do here? */
		  }
		
		}

	// a card - primary types to match
		q = new Panel();
		tabbie.addThing(q,st.textorname("primtab"));
		w3=new repTile();
		q.setLayout(w3);

		// label
		l = new Label(st.textorname("primtitle"));
		l.setFont(subtitlefont);
		q.add(l);
		w3.setConstraints(0,0,0,1,"ch",l);

		primary_widget = new types_panel();
		q.add(primary_widget);
		w3.setConstraints(1,0,1,1,"",primary_widget);
		w3.setStretchy(1,1);

	// a card - secondary types to match
		q = new Panel();
		tabbie.addThing(q,st.textorname("sectab"));
		w3=new repTile();
		q.setLayout(w3);

		// label
		l = new Label(st.textorname("sectitle"));
		l.setFont(subtitlefont);
		q.add(l);
		w3.setConstraints(0,0,0,1,"ch",l);

		secondary_widget = new types_panel();
		q.add(secondary_widget);
		w3.setConstraints(1,0,1,1,"",secondary_widget);


		// load types into primary_widget and secondary_widget
		//
		// for each axis:
		//	look up primary type, add it
		//	look up associated secondary type, add it
		for (int y=0; y < num_axes; y++)
			{
			String tmp;
			if (axis_types[y].length() > 0)
				primary_widget.label_textfield.put(
						axis_types[y],null);
			tmp = (String)type_map.primary_type.get(axis_types[y]);
			if (tmp != null)
				secondary_widget.label_textfield.put(tmp,null);
			}
		primary_widget.compute();
		secondary_widget.compute();

		w3.setStretchy(1,1);


	// end of tab cards
		tabbie.compute();

// row 4 - empty

// row 5 - submit button
		row=5;

		q = new Panel();
		add(q);
		w1.setConstraints(row,0,row,2,"",q);

		w3=new repTile();
		q.setLayout(w3);
		
		int col;
		col = 0;

	// submit - text
		text_button = new Button(st.textorname("text"));
		q.add(text_button);
		w3.setConstraints(0,col,0,col,"",text_button);
		col++;

	// submit - excel
		excel_button = new Button(st.textorname("excel"));
		q.add(excel_button);
		w3.setConstraints(0,col,0,col,"",excel_button);
		col++;
	
	// submit - html
		html_button = new Button(st.textorname("html"));
		q.add(html_button);
		w3.setConstraints(0,col,0,col,"",html_button);
		col++;
		
	// submit - bar
		bar_button = new Button(st.textorname("bar"));
		q.add(bar_button);
		w3.setConstraints(0,col,0,col,"",bar_button);
		col++;
		
	// submit - pie
		pie_button = new Button(st.textorname("pie"));
		q.add(pie_button);
		w3.setConstraints(0,col,0,col,"",pie_button);
		col++;
		
	// submit - scatter
		scatter_button = new Button(st.textorname("scatter"));
		q.add(scatter_button);
		w3.setConstraints(0,col,0,col,"",scatter_button);
		col++;

	// save
		save_button = new Button(st.textorname("save"));
		q.add(save_button);
		w3.setConstraints(0,col,0,col,"",save_button);
		col++;

	// load
		load_button = new Button(st.textorname("load"));
		q.add(load_button);
		w3.setConstraints(0,col,0,col,"",load_button);
		col++;

	// close
		close_button = new Button(st.textorname("close"));
		q.add(close_button);
		w3.setConstraints(0,col,0,col,"fr",close_button);

	}

/****
* no special paint instructions yet.
*/
        public void paint(Graphics gc) {
                System.out.println("in paint function");
	}

/****
* all events come here
*/
	public boolean handleEvent (Event e) {
		if (e.id == Event.ACTION_EVENT) {
			if ( e.target == text_button) {
				submit_query("text");
				return true;
			}
			if ( e.target == excel_button) {
				submit_query("excel");
				return true;
			}
			if ( e.target == html_button) {
				submit_query("html");
				return true;
			}
			if ( e.target == bar_button) {
				submit_query("bar");
				return true;
			}
			if ( e.target == pie_button) {
				submit_query("pie");
				return true;
			}
	  		if ( e.target == scatter_button) {
			  submit_query("scatter");
			  return true;
			}
			if(e.target == save_button) {
			  if(QuerySaver.inUse()) {
			    System.out.println("QuerySaver already in use.");
			  } else {
			    Vector stringVec = new Vector();
			    Enumeration tmpEnum;
			    String tmpS;
			    Checkbox tmpBox;
			    
			    // Collect information common to all
			    // histogrms
			    System.out.println("Collecting information to Save");
			    
			    // Time as an axis
			    stringVec.addElement((time_checkbox.getState())
						 ? "true" : "false");
			    // query time 
			    stringVec.addElement((query_time.getState())
						 ? "true" : "false");
			    
			    // query all time
			    stringVec.addElement((query_all_time.getState())
						 ? "true" : "false");

			    // Transactions
			    stringVec.addElement((nuke_count.getState())
						 ? "true" : "false");
			    
			    // Cut off percent
			    stringVec.addElement(nukeage_box.getText());

			    // Min and Max time
			    stringVec.addElement((min_time.usedGui())
						 ? "true" : "false");
			    stringVec.addElement(Long.toString(min_time.get_value()));
			    
			    stringVec.addElement((max_time.usedGui())
						 ? "true" : "false");
			    stringVec.addElement(Long.toString(max_time.get_value()));
			    
		 	    // suppress translations
			    tmpS = (suppress_mapping.getState()) ? "true" : "false";
			    stringVec.addElement(tmpS);

			    // First page fields
			    for(int i=0; i<axis_inputs.length; i++) {
			      stringVec.addElement(axis_inputs[i].getText());
			    }
			    
			    // Buttons for first page fields
			    tmpEnum = axis_check_list.keys();
			    while(tmpEnum.hasMoreElements()) {
			      tmpS = (String) tmpEnum.nextElement();
			      tmpBox = (Checkbox) axis_check_list.get(tmpS);
			      tmpS = (tmpBox.getState()) ? "true" : "false";
			      stringVec.addElement(tmpS);
			    }

			    // Second Page fields
			    tmpEnum = (primary_widget.strings()).elements();
			    while(tmpEnum.hasMoreElements()) {
			      stringVec.addElement((String) tmpEnum.nextElement());
			    }
			    // Third Page Fields
			    tmpEnum = (secondary_widget.strings()).elements();
			    while(tmpEnum.hasMoreElements()) {
			      stringVec.addElement((String) tmpEnum.nextElement());
			    }
			    saver = new QuerySaver(my_name);
			    saver.save(stringVec);
			  }
			  return(true);
			}
			if(e.target == load_button) {
			  Vector checkVec = new Vector();
			  Enumeration tmpEnum;
			  Checkbox tmpBox;
			  String tmpS;

			  // Button refrences for first page fields
			  tmpEnum = axis_check_list.keys();
			  while(tmpEnum.hasMoreElements()) {
			    tmpS = (String) tmpEnum.nextElement();
			    tmpBox = (Checkbox) axis_check_list.get(tmpS);
			    checkVec.addElement(tmpBox);
			  }

			  if(QueryLoader.inUse()) {
			    System.out.println("QuerLoader already in use.");
			  } else {
			    loader = new QueryLoader("packages/"+my_name+".sq");
			    loader.loadHistogram(time_checkbox,query_time,
						 query_all_time, nuke_count,
						 nukeage_box,min_time,max_time,
						 suppress_mapping,
						 axis_inputs, checkVec, 
						 primary_widget.fields(),
						 secondary_widget.fields());
			  }
			  return(true);
			}
			if ( e.target == close_button ) {
			  destroy_parent(this);
			  return true;
			}
		return false;
		} else if (e.id == Event.WINDOW_DESTROY) {
			System.out.println("destroy event: "+e.toString());
			destroy_parent(this);
			return true;
		} else {
			return false;
		}
	}


/****
* if close button hit, we walk up to the top level frame and
* delete it.
*/
	public void destroy_parent(Component c) {
		while ( (c != null) && (! (c instanceof Frame)) )
			c= c.getParent();
		if (c != null)
			((Frame)c).dispose();
	}

/****
* get a true/false value from the config file, return what it is
*/
	boolean cfg_true(String s) {
		s=hist_conf.getProperty(s);
		if ((s == null) || (s.compareTo("true") != 0))
			return false;
		else
			return true;
	}

/****
* make the axis list widget look right
*/
	void load_axis_list() {
System.out.println("enter load_axis_list");
		String s=hist_conf.getProperty("num_columns");
		int num=0;
		try {
			num = Integer.parseInt(s);
		} catch (Exception e) {
			num = 0;
System.out.println("load_axis_list - exception: " + e.toString() );
		}
System.out.println("load_axis_list - axes: " + num + " string: " + s);

		num_axes = num;

		if (num < 1)
			{
			// bug: display message that this is not queryable
			// because there are no axes
			}

		axis_text_list = new ordered_table();
		axis_check_list = new ordered_table();

		axis_types = new String[num_axes];

		axis_inputs = new TextField [num];

		// remove old panel 
		axis_list_parent.remove(axis_list_widget);

		// create new panel
		axis_list_widget = new Panel();

		// add new panel to old card
		axis_list_parent.add(axis_list_widget);

		// set new panels constraints in the old card's layout
		repTile r = (repTile) axis_list_parent.getLayout();
		r.setConstraints(1,0,1,1,"",axis_list_widget);

		// make layout for new panel
		r = new repTile();
		axis_list_widget.setLayout(r);
		Label lcbox, lnum, lnam, linput;
		lcbox = new Label(st.textorname("cbox"));
		lnum = new Label(st.textorname("axnum"));
		lnam = new Label(st.textorname("axname"));
		linput = new Label(st.textorname("axinput"));

			axis_list_widget.add(lcbox);
			axis_list_widget.add(lnum);
			axis_list_widget.add(lnam);
			axis_list_widget.add(linput);

		r.setConstraints(0,0,0,0,"",lcbox);
		r.setConstraints(0,1,0,1,"",lnum);
		r.setConstraints(0,2,0,2,"",lnam);
		r.setConstraints(0,3,0,3,"",linput);

		r.setStretchy(-1,3);

		TextField tinput;
		Checkbox cbox;

		for (int x = 1; x <= num; x++) {
			String xstr = "" + x;
			String label_name = "column_"+xstr+"_label";
			String name = hist_conf.getProperty(label_name);

			// make labels for each column
			lnum = new Label(""+x);
			lnam = new Label(name);
			tinput = new TextField();
			cbox = new Checkbox();
			cbox.setState(true);

			axis_inputs[x-1] = tinput;


			axis_list_widget.add(cbox);
			axis_list_widget.add(lnum);
			axis_list_widget.add(lnam);
			axis_list_widget.add(tinput);
			
			r.setConstraints(x,0,x,0,"",cbox);
			r.setConstraints(x,1,x,1,"",lnum);
			r.setConstraints(x,2,x,2,"",lnam);
			r.setConstraints(x,3,x,3,"",tinput);

			String type_name = "column_"+xstr+"_type";
			name = hist_conf.getProperty(type_name);
			if ((name != null) && (name.length() >= 2))
					axis_types[x-1]=name.substring(2);
			else
					axis_types[x-1]="";
			axis_check_list.put(xstr,cbox);
			axis_text_list.put(xstr,tinput);
		}
		axis_list_parent.invalidate();
		axis_list_parent.validate();
	}


	public void submit_query(String display_type) 
	{
		String axis, primary, secondary, misc, axis_map;
		int uncollapsed = 0;
		if (NFRInfo.dancyThing != null) NFRInfo.dancyThing.startDancing();

		// The eric-1 change may screw up the if statement below.

		if ((query_all_time != null) && query_all_time.getState())
			misc="query_type=3";
		else
			misc="query_type=1";
		misc = misc + "&recorder_name="+URLEncoder.encode(my_name);

		axis="";
		axis_map = "";
		if (time_checkbox.getState())
			{
			axis_map = "&time_checkbox=on";
			uncollapsed++;
			}
		if (min_time.usedGui())
			{
System.out.println("min_time used gui");
			min_time.compileDate(true);
			misc = misc + "&min_time="+
				min_time.get_value();
			}
		if (max_time.usedGui())
			{
System.out.println("max_time used gui");
			max_time.compileDate(true);
			misc = misc + "&max_time="+
				max_time.get_value();
			}

                if (suppress_mapping.getState())
                        misc = misc + "&suppress_mapping=y";

		for (int x = 1; x<= num_axes; x++)
			{
			String t1=((TextField)axis_text_list.get(""+x))
				.getText();
			if (t1.length() > 0)
				axis = axis + "&axis_" + (x-1) + "=" +
					URLEncoder.encode(t1);
			if (((Checkbox)axis_check_list.get(""+x)).getState()
				== false)
				axis_map = axis_map +
					"&collapse_" + x + "=on";
			else
				uncollapsed++;
			}
		primary = primary_widget.build_url_frag("p_");
		secondary = secondary_widget.build_url_frag("s_");
		String query = misc+axis_map + 
				axis+primary+secondary;
		System.out.println(query);

		String s =
		"/cgi-bin/histogram_query?"
			+ "mode=" + display_type
			+ "&nukepercent=" + nukeage_box.getText()
			+ "&" + query
			;
		if (nuke_count.getState())
			s = s + "&nuke_count";
		else
			s = s + "&nuke_sum";
		URL u = null;
		try {
			u = new URL(NFRInfo.document_base,s);
		} catch (Exception e ) {
			System.out.println("making url: "+e.toString());
		}
		if ((display_type == "text") 
			|| (display_type == "html") ||
			(display_type == "excel"))
			{
			System.out.println("Executing showDocument");
			try {
				NFRInfo.applet_context.showDocument(u,
					"_blank");
				System.out.println("Finished showDocument");
			} catch (Exception e) {
			    System.out.println("shoDocument: "+e.toString());
			}
			}
		else if (display_type == "bar") 
			{
			if (uncollapsed == 1)
				new graph.View("b",u,"title",st.textorname(
					"closegraph"));
			else
				new popup_bummer("You can only have a "+
				"bar graph if you only check one column "+
				"for display","Bummer");
			}
		else if (display_type == "pie")
			{
			if (uncollapsed == 1)
				new graph.View("p",u,"title",st.textorname(
					"closegraph"));
			else
				new popup_bummer("You can only have a "+
				"pie chart if you only check one column "+
				"for display","Bummer");
			}
		else if (display_type == "scatter")
			{
			if (uncollapsed == 2)
				new graph.View("s",u,"title",st.textorname(
					"closegraph"));
			else
				new popup_bummer("You can only have a "+
				"scatter plot if you check exactly two "+
				"columns for display","Bummer");
			}
		if (NFRInfo.dancyThing != null) NFRInfo.dancyThing.stopDancing();
	}
}





