/*
        Copyright(C) 1997 Network Flight Recorder, Inc.
        All rights reserved.

        Use and distribution of this software and its source code
        are governed by the terms and conditions of the
        Network Flight Recorder Software License ("LICENSE.TXT")

By Mark Sienkiewicz / NFR

*/

import java.io.*;
import java.awt.*;
import nfr.misc.*;
import java.util.*;
import java.net.*;

public class list_query extends Panel {
	// buttons at the bottom of the screen
	Button load_button, save_button, close_button;
	Button text_button, html_button, excel_button;

	//
	// this object is a bunch of panels placed into a tabthing
	//

	Checkbox suppress_mapping;

	//
	// column_list_parent is a panel that will be the tab that lists
	//	the columns.  it contains a lable and a panel with the list
	// column_list_widget is the actual panel with the list.  it is
	//	destroyed, regenerated, and re-added to column_list_parent
	//	by load_column_list()

	Panel column_list_parent;
	Panel column_list_widget;

	//
	// column_inputs[] is the input field for the user to type query
	//	values for each column
	//
	TextField column_inputs[];	// text fields for each column

	//
	// these are two panel-like-objects that contain a list of
	// primary types and secondary types the user can query
	//
	types_panel primary_widget, secondary_widget;

	//
	// how many columns there are
	//
	int num_columns;


	//
	// tables mapping column number (represented as a string) to
	//	- a TextField for the users query
	//	- a Checkbox whether to display this column in the results
	//	- the primary type of the column given in the config file
	ordered_table column_text_list;
	ordered_table column_check_list;

	String [] column_types;

	//
	// font to use at the top of a particular card of the tabthing
	//
	Font subtitlefont;

	//
	// name of the recorder we are querying

	String my_name;

	//
	// string table, for multi-language ability
static	Cf st = null;

	//
	// config file for this instance of the recorder
	//
	Cf hist_conf;

	//
	// URL we were invoked as
	//
	java.net.URL my_docbase;

	//
	// recordable/primary/secondary/representation type table
	//
	type_mapping type_map;

  //
  // Time input components
  //
  AbsoluteTime min_time, max_time;

  //
  // Object to handle query saving
  //
  QuerySaver saver;

  //
  // Object to handle query loading
  //
  QueryLoader loader;
  


/************************************************************
* this is the constructor.  pass in the name of the recorder
* to configure, the document base, and the ancestor window
* that you will eventually add this to.
*/
public list_query(java.lang.String s, java.net.URL docbase, 
	Component ancestor) {

		my_name = s;

		NFRInfo.do_style(this);

	// load the string table
		if (docbase == null)
			System.out.println("docbase is null");
		my_docbase=docbase;
		if (st == null)
			{
			st = new Cf(docbase,"etc/javastrings/listq1");
			st.Read();
			}

	// load the type mapping (recordable:primary type:secondary type:rep)
		type_map = new type_mapping(docbase);

		int row;
		Label l;
		Panel p,p1,q;
		repTile w1,w3;

	// start building the window

		// get a bigger font
		subtitlefont = ancestor.getFont();
		subtitlefont = new Font (subtitlefont.getName(), Font.BOLD, 
				subtitlefont.getSize()+2);

		w1=new repTile();

		setLayout(w1);	// make this widget the top level of the applet

		w1.setStretchy(3,1);

		w1.setborder(4,4,4,4); // a little bit of border around window

	// read the recorder config file
		hist_conf = new Cf(my_docbase,"packages/"
			+my_name+".cfg");
		if (hist_conf.Read())
			System.out.println("load cf true");
		else
			System.out.println("load cf false");

// row 0
		row=0;
	// list name label
		l = new Label(st.textorname("name"));
		add(l);
		w1.setConstraints(row,0,row,0,"",l);

	// list name field
		l=new Label(my_name);
		add(l);
		w1.setConstraints(row,1,row,2,"",l);

// row 1
		row=1;
	// list title label
		l = new Label(st.textorname("title"));
		add(l);
		w1.setConstraints(row,0,row,0,"",l);

	// list title field
		l=new Label(hist_conf.getProperty("title"));
		add(l);
		w1.setConstraints(row,1,row,4,"",l);

// row 2
		row=2;

// row 3 - a tab panel that is the big blob in the middle.

		row=3;

		tabthing tabbie = new tabthing(10);
		add(tabbie);
		w1.setConstraints(row,0,row,2,"",tabbie);

	// a card - column values to match
		q = new Panel();
		tabbie.addThing(q,st.textorname("columntab"));
		w3=new repTile();
		q.setLayout(w3);

		// label
		l = new Label(st.textorname("columntitle"));
		l.setFont(subtitlefont);
		q.add(l);
		w3.setConstraints(0,0,0,3,"ch",l);

		column_list_parent = q;

		column_list_widget = new Panel();
		add(column_list_widget);
		w3.setConstraints(1,0,1,3,"",column_list_widget);

		load_column_list();

                {
                Panel px = new Panel();
                q.add(px);
                w3.setConstraints(3,0,3,5,"",px);
                repTile r = new repTile();
                px.setLayout(r);

		l = new Label(st.textorname("mintime"));
		px.add(l);
		r.setConstraints(3,0,3,0,"",l);
                min_time = new AbsoluteTime(false,60,st.textorname("minearly"),
                        st.textorname("mintimeuse"));
		px.add(min_time);
		r.setConstraints(3,1,3,1,"",min_time);

		l = new Label(st.textorname("maxtime"));
		px.add(l);
		r.setConstraints(4,0,4,0,"",l);

                max_time = new AbsoluteTime(false,60, st.textorname("maxlate"),
                        st.textorname("maxtimeuse"));
                max_time.set_value(1262322000); /* jan 1, 2010 ; arbitrary */
                max_time.displayDate();

		px.add(max_time);
		r.setConstraints(4,1,4,1,"",max_time);
		}

		w3.setStretchy(-1,1);

		suppress_mapping = new Checkbox(
			st.textorname("suppressmapping"), null, false );

		q.add(suppress_mapping);
		w3.setConstraints(5,0,5,0,"",suppress_mapping);

	// a card - primary types to match
		q = new Panel();
		tabbie.addThing(q,st.textorname("primtab"));
		w3=new repTile();
		q.setLayout(w3);

		// label
		l = new Label(st.textorname("primtitle"));
		l.setFont(subtitlefont);
		q.add(l);
		w3.setConstraints(0,0,0,1,"ch",l);

		primary_widget = new types_panel();
		q.add(primary_widget);
		w3.setConstraints(1,0,1,1,"",primary_widget);
		w3.setStretchy(1,1);

	// a card - secondary types to match
		q = new Panel();
		tabbie.addThing(q,st.textorname("sectab"));
		w3=new repTile();
		q.setLayout(w3);

		// label
		l = new Label(st.textorname("sectitle"));
		l.setFont(subtitlefont);
		q.add(l);
		w3.setConstraints(0,0,0,1,"ch",l);

		secondary_widget = new types_panel();
		q.add(secondary_widget);
		w3.setConstraints(1,0,1,1,"",secondary_widget);


		// load types into primary_widget and secondary_widget
		//
		// for each column:
		//	look up primary type, add it
		//	look up associated secondary type, add it
		for (int y=0; y < num_columns; y++)
			{
			String tmp;
			if (column_types[y].length() > 0)
				primary_widget.label_textfield.put(
						column_types[y],null);
			tmp = (String)type_map.primary_type.get(column_types[y]);
			if (tmp != null)
				secondary_widget.label_textfield.put(tmp,null);
			}
		primary_widget.compute();
		secondary_widget.compute();

		w3.setStretchy(1,1);


	// end of tab cards
		tabbie.compute();

// row 4 - empty

// row 5 - submit button
		row=5;

		q = new Panel();
		add(q);
		w1.setConstraints(row,0,row,2,"",q);

		w3=new repTile();
		q.setLayout(w3);
		
	// submit - text
		text_button = new Button(st.textorname("text"));
		q.add(text_button);
		w3.setConstraints(0,0,0,0,"",text_button);
		
	// submit - html
		html_button = new Button(st.textorname("html"));
		q.add(html_button);
		w3.setConstraints(0,1,0,1,"",html_button);

	// submit - excel
		excel_button = new Button(st.textorname("excel"));
		q.add(excel_button);
		w3.setConstraints(0,2,0,2,"",excel_button);

	// save
		save_button = new Button(st.textorname("save"));
		q.add(save_button);
		w3.setConstraints(0,3,0,3,"",save_button);

	// load
		load_button = new Button(st.textorname("load"));
		q.add(load_button);
		w3.setConstraints(0,4,0,4,"",load_button);
					     
		
	// close
		close_button = new Button(st.textorname("close"));
		q.add(close_button);
		w3.setConstraints(0,5,0,5,"fr",close_button);

	}

/****
* no special paint instructions yet.
*/
        public void paint(Graphics gc) {
	}

/****
* all events come here
*/
	public boolean handleEvent (Event e) {
		if (e.id == Event.ACTION_EVENT) {
			if ( e.target == text_button ) {
				submit_query("text");
				return true;
			}
			if ( e.target == html_button ) {
				submit_query("html");
				return true;
			}
			if ( e.target == excel_button ) {
				submit_query("excel");
				return true;
			}
			if(e.target == save_button) {
			  if(QuerySaver.inUse()) {
			    System.out.println("QuerySaver already in use.");
			  } else {
			    Vector stringVec = new Vector();
			    Enumeration tmpEnum;
			    String tmpS;
			    Checkbox tmpBox;

			    // Collect information common to all lists
			    System.out.println("Collecting information to Save");
			    
			    // Min and Max time
			    stringVec.addElement((min_time.usedGui())
						 ? "true" : "false");
			    stringVec.addElement(Long.toString(min_time.get_value()));

			    stringVec.addElement((max_time.usedGui())
						 ? "true" : "false");
			    stringVec.addElement(Long.toString(max_time.get_value()));
			    
			    // suppress translations
			    tmpS = (suppress_mapping.getState()) ? "true" : "false";
			    stringVec.addElement(tmpS);

			    // First page fields
			    for(int i=0; i<column_inputs.length; i++) {
			      stringVec.addElement(column_inputs[i].getText());
			    }

			    // Buttons for first page fields
			    tmpEnum = column_check_list.keys();
			    while(tmpEnum.hasMoreElements()) {
			      tmpS = (String) tmpEnum.nextElement();
			      tmpBox = (Checkbox) column_check_list.get(tmpS);
			      tmpS = (tmpBox.getState()) ? "true" : "false";
			      stringVec.addElement(tmpS);
			    }

			    tmpEnum = (primary_widget.strings()).elements();
			    while(tmpEnum.hasMoreElements()) {
			      stringVec.addElement((String) tmpEnum.nextElement());
			    }
			    tmpEnum = (secondary_widget.strings()).elements();
			    while(tmpEnum.hasMoreElements()) {
			      stringVec.addElement((String) tmpEnum.nextElement());
			    }

			    saver = new QuerySaver(my_name);
			    saver.save(stringVec);
			  }
			  return(true);
			}
			if(e.target == load_button) {
			  Vector checkVec = new Vector();
			  Enumeration tmpEnum;
			  Checkbox tmpBox;
			  String tmpS;

			  // Button refrences for first page fields
			  tmpEnum = column_check_list.keys();
			  while(tmpEnum.hasMoreElements()) {
			    tmpS = (String) tmpEnum.nextElement();
			    tmpBox = (Checkbox) column_check_list.get(tmpS);
			    checkVec.addElement(tmpBox);
			  }

			  if(QueryLoader.inUse()) {
			    System.out.println("QuerLoader already in use.");
			  } else {
			    loader = new QueryLoader("packages/"+my_name+".sq");
			    loader.loadList(min_time,max_time,
					    suppress_mapping,
					    column_inputs,checkVec,
					    primary_widget.fields(),
					    secondary_widget.fields());
			  }
			  return(true);
			}
			if ( e.target == close_button ) {
				destroy_parent(this);
				return true;
			}
		return false;
		} else if (e.id == Event.WINDOW_DESTROY) {
			System.out.println("destroy event: "+e.toString());
			destroy_parent(this);
			return true;
		} else {
			return false;
		}
	}


/****
* if close button hit, we walk up to the top level frame and
* delete it.
*/
	public void destroy_parent(Component c) {
		while ( (c != null) && (! (c instanceof Frame)) )
			c= c.getParent();
		if (c != null)
			((Frame)c).dispose();
	}

/****
* get a true/false value from the config file, return what it is
*/
	boolean cfg_true(String s) {
		s=hist_conf.getProperty(s);
		if ((s == null) || (s.compareTo("true") != 0))
			return false;
		else
			return true;
	}

/****
* make the column list widget look right
*/
	void load_column_list() {
		String s=hist_conf.getProperty("num_columns");
		int num=0;
		try {
			num = Integer.parseInt(s);
		} catch (Exception e) {
			num = 0;
System.out.println("load_column_list - exception: " + e.toString() );
		}

		num_columns = num;

		if (num < 1)
			{
			// bug: display message that this is not queryable
			// because there are no columns
			}

		column_text_list = new ordered_table();
		column_check_list = new ordered_table();

		column_types = new String[num_columns];

		column_inputs = new TextField [num];

		// remove old panel 
		column_list_parent.remove(column_list_widget);

		// create new panel
		column_list_widget = new Panel();

		// add new panel to old card
		column_list_parent.add(column_list_widget);

		// set new panels constraints in the old card's layout
		repTile r = (repTile) column_list_parent.getLayout();
		r.setConstraints(1,0,1,1,"",column_list_widget);

		// make layout for new panel
		r = new repTile();
		column_list_widget.setLayout(r);
		Label lcbox, lnum, lnam, linput;
		lcbox = new Label(st.textorname("cbox"));
		lnum = new Label(st.textorname("axnum"));
		lnam = new Label(st.textorname("axname"));
		linput = new Label(st.textorname("axinput"));

			column_list_widget.add(lcbox);
			column_list_widget.add(lnum);
			column_list_widget.add(lnam);
			column_list_widget.add(linput);

		r.setConstraints(0,0,0,0,"",lcbox);
		r.setConstraints(0,1,0,1,"",lnum);
		r.setConstraints(0,2,0,2,"",lnam);
		r.setConstraints(0,3,0,3,"",linput);

		r.setStretchy(-1,3);

		TextField tinput;
		Checkbox cbox;

		for (int x = 1; x <= num; x++) {
			String xstr = "" + x;
			String label_name = "column_"+xstr+"_label";
			String name = hist_conf.getProperty(label_name);

			// make labels for each column
			lnum = new Label(""+x);
			lnam = new Label(name);
			tinput = new TextField();
			cbox = new Checkbox();
			cbox.setState(true);

			column_inputs[x-1] = tinput;


			column_list_widget.add(cbox);
			column_list_widget.add(lnum);
			column_list_widget.add(lnam);
			column_list_widget.add(tinput);
			
			r.setConstraints(x,0,x,0,"",cbox);
			r.setConstraints(x,1,x,1,"",lnum);
			r.setConstraints(x,2,x,2,"",lnam);
			r.setConstraints(x,3,x,3,"",tinput);

			String type_name = "column_"+xstr+"_type";
			name = hist_conf.getProperty(type_name);
			if ((name != null) && (name.length() >= 2))
					column_types[x-1]=name.substring(2);
			else
					column_types[x-1]="";
			column_check_list.put(xstr,cbox);
			column_text_list.put(xstr,tinput);
		}
		column_list_parent.invalidate();
		column_list_parent.validate();
	}


	public void submit_query(String display_type) {
		String column, primary, secondary, misc, column_map;
		int uncollapsed = 0;
		if (NFRInfo.dancyThing != null) NFRInfo.dancyThing.startDancing();

		misc="query_type=1&recorder_name="+URLEncoder.encode(my_name)
			+ "&mode="+display_type;

		column="";
		column_map = "";

		min_time.compileDate(true);
		max_time.compileDate(true);

		misc = misc + "&min_time="+
			URLEncoder.encode(""+min_time.get_value())
			+ "&max_time=" + 
			URLEncoder.encode(""+max_time.get_value());

		if (suppress_mapping.getState())
			misc = misc + "&suppress_mapping=y";

		for (int x = 1; x<= num_columns; x++)
			{
			String t1=((TextField)column_text_list.get(""+x))
				.getText();
			if (t1.length() > 0)
				column = column + "&column_" + (x-1) + "=" +
					URLEncoder.encode(t1);
			if (((Checkbox)column_check_list.get(""+x)).getState()
				== false)
				column_map = column_map +
					"&collapse_" + x + "=on";
			else
				uncollapsed++;
			}
		primary = primary_widget.build_url_frag("p_");
		secondary = secondary_widget.build_url_frag("s_");
		String query = misc+column_map + 
				column+primary+secondary;
		try {
		NFRInfo.applet_context.showDocument(
                                new URL(NFRInfo.document_base
					,"/cgi-bin/list_query?"
					+query
					),"_blank");
		} catch (IOException e) {
			new popup_bummer(e.toString(),
				st.textorname("databummer"));
		}
		if (NFRInfo.dancyThing != null) NFRInfo.dancyThing.stopDancing();
	}

}

