/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   AckCommentator.java
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

package nfr.alerts;

import nfr.rc.*;
import nfr.misc.*;

import java.util.*;
import java.net.*;
import java.io.*;
import java.awt.*;



public class AckCommentator extends GoodFrame
{
  AlertRecord mAlertRecord = null;
  boolean mDead = false;
  TextArea mCommentTxt = null;
  Button mOkBtn = null;
  Button mCancelBtn = null;
  Thread mCommentThd = null;

public AckCommentator(AlertRecord ar, Window w)
  {
    super(w);
    mAlertRecord = ar;
    int row = 0;

    repTile l = new repTile();
    setLayout(l);

    Label lbl = new Label(String.valueOf(ar));
    add(lbl);
    l.setConstraints(row,0,row,0,"",lbl);
    row++;

    // spacer
    lbl = new Label(" ");
    add(lbl);
    l.setConstraints(row,0,row,0,"",lbl);
    row++;

    Label commentLbl = new Label("Comment:");
    add(commentLbl);
    l.setConstraints(row,0,row,0,"",commentLbl);
    row++;

    mCommentTxt = new TextArea(3,24);
    mCommentTxt.setEditable(true);
    add(mCommentTxt);
    l.setConstraints(row,0,row,0,"",mCommentTxt);
    l.setStretchy(row,0);
    row++;

    Panel btns = new Panel();

    mOkBtn = new Button("done");
    btns.add(mOkBtn);

    mCancelBtn = new Button("cancel");
    btns.add(mCancelBtn);

    add(btns);
    l.setConstraints(row,0,row,0,"ch",btns);
    row++;

    l.setborder(CtoP.Width(1,this), CtoP.Width(1,this),
		CtoP.Height(1,this)/2, CtoP.Height(1,this)/2);

    resize(CtoP.Width(100,this),CtoP.Height(20, this));
    setTitle("NFR Alert response comment");

  }

public boolean startWriteComment()
  {
    if(mCommentThd != null)
      mCommentThd.stop();
    mCommentThd = new Thread(new CommentThread(this));
    mCommentThd.start();
    return true;
  }

public boolean writeComment()
  {
    DataInputStream In = null;
    DataOutputStream Out = null;

    StringBuffer sbuf = new StringBuffer();
    String line = null;

    CgiScript cgi = new CgiScript("/cgi-bin/ackcomment");
    switch(mAlertRecord.AckState)
      {
      case AlertRecord.ACKSTATE_NA:
	System.out.println("bailing cause there's no AckState");
	return false;
	
      case AlertRecord.ACKSTATE_UNACKED:
	System.out.println("this has ACKSTATE_UNACKED");
	cgi.addDataNVPair("hist_file", "data/alerts/" + 
			  mAlertRecord.getLogName() +
			  "/unacked/" + 
			  String.valueOf(mAlertRecord.Serial));
	break;
	
      case AlertRecord.ACKSTATE_ACKED:
	System.out.println("this has ACKSTATE_ACKED");
	cgi.addDataNVPair("hist_file", "data/alerts/" + 
			  mAlertRecord.getLogName() +
			  "/acked/" + 
			  String.valueOf(mAlertRecord.Serial));
	break;
      }

    cgi.addDataNVPair("comment", mCommentTxt.getText());

    if(cgi.run() != 0)
      {
	MsgBox.show("Error", cgi.getRelevantMsg());
	return false;
      }

    mAlertRecord.showHistory(NFRInfo.top);
    return true;
  }

public boolean action(Event e, Object arg)
  {
    //System.out.println("EVENT: " + e);
    if(e.target == mOkBtn)
      {
	startWriteComment();
	//dispose();
	return true;
      }
    if(e.target == mCancelBtn)
      {
	mAlertRecord.killCommentator();
	return true;
      }
    else
      return false;
  }

}

class CommentThread implements Runnable
{
  AckCommentator mAckCommentator = null;

  CommentThread(AckCommentator a)
  {
    mAckCommentator = a;
  }

public void run()
  {
    if(!mAckCommentator.writeComment())
      MsgBox.show("Error", "can't make comment.");
    mAckCommentator.mCommentThd = null;
    mAckCommentator.mAlertRecord.killCommentator();
  }
}

