/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   AckHistory.java
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

package nfr.alerts;

import nfr.rc.*;
import nfr.misc.*;

import java.util.*;
import java.net.*;
import java.io.*;
import java.awt.*;


public class AckHistory extends GoodFrame
{
  AlertRecord mAlertRecord = null;
  TextArea mHistoryTxt = null;
  Button mOkBtn = null;
  Thread mHistoryThd = null;

public AckHistory(AlertRecord ar, Window w)
  {
    super(w);

    mAlertRecord = ar;

    repTile l = new repTile();
    setLayout(l);

    Label histLbl = new Label("History:");
    add(histLbl);
    l.setConstraints(0,0,0,0,"",histLbl);

    mHistoryTxt = new TextArea(3,24);
    mHistoryTxt.setEditable(false);
    add(mHistoryTxt);
    l.setConstraints(1,0,1,0,"",mHistoryTxt);

    Panel btns = new Panel();

    mOkBtn = new Button("done");
    btns.add(mOkBtn);

    add(btns);
    l.setConstraints(2,0,2,0,"ch",btns);

    l.setStretchy(1,0);
    l.setborder(CtoP.Width(1,this), CtoP.Width(1,this),
		CtoP.Height(1,this)/2, CtoP.Height(1,this)/2);

    resize(CtoP.Width(100,this),CtoP.Height(20, this));
    setTitle("NFR Alert response history");

  }

public boolean startUpdate()
  {
    if(mHistoryThd != null)
      mHistoryThd.stop();
    mHistoryThd = new Thread(new HistoryThread(this));
    mHistoryThd.start();
    return true;
  }

public boolean update()
  {
    DataInputStream In = null;
    DataOutputStream Out = null;

    StringBuffer sbuf = new StringBuffer();
    String line = null;
    String histfile = null;

    CgiScript cgi = new CgiScript("/cgi-bin/ackhistory");

    switch(mAlertRecord.AckState)
      {
      case AlertRecord.ACKSTATE_NA:
	System.out.println("bailing cause there's no AckState");
	return false;
	
      case AlertRecord.ACKSTATE_UNACKED:
	System.out.println("this has ACKSTATE_UNACKED");
	cgi.addDataNVPair("hist_file", "data/alerts/" + 
			  mAlertRecord.getLogName() +
			  "/unacked/" + 
			  String.valueOf(mAlertRecord.Serial));
	break;
	
      case AlertRecord.ACKSTATE_ACKED:
	System.out.println("this has ACKSTATE_ACKED");
	cgi.addDataNVPair("hist_file", "data/alerts/" + 
			  mAlertRecord.getLogName() +
			  "/acked/" + 
			  String.valueOf(mAlertRecord.Serial));
	break;
      }

    if(cgi.sendData() != 0)
      {
	MsgBox.show("Error", cgi.getRelevantMsg());
	return false;
      }

    if(cgi.getResponse() != 0)
      {
	MsgBox.show("Error", cgi.getRelevantMsg());
	return false;
      }

    In = cgi.getInputStream();
    try
      {
	System.out.println(In.available() + " bytes available");
	boolean True = true;
	line = In.readLine();
	while(line != null)
	  {
	    sbuf.append(line);
	    sbuf.append("\n");
	    line = In.readLine();
	  }
      }
    catch(EOFException e)
      {
	System.out.println("eof");
	sbuf.append(line);
	//'sok, we were expecting this
      }

    catch(Exception e)
      {
	e.printStackTrace();
	return false;
      }

    mHistoryTxt.setText(sbuf.toString());
    return true;
  }

public boolean action(Event e, Object arg)
  {
    //System.out.println("EVENT: " + e);
    if(e.target == mOkBtn)
      {
	mAlertRecord.killHistory();
	return true;
      }
    else
      return false;
  }

}


class HistoryThread implements Runnable
{
  AckHistory mAckHistory = null;

  HistoryThread(AckHistory a)
  {
    mAckHistory = a;
  }

public void run()
  {
    if(!mAckHistory.update())
      MsgBox.show("Error", "can't load history.");
    mAckHistory.mHistoryThd = null;
  }
}

