/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   AlertRecord.java
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

package nfr.alerts;

import java.util.*;

import nfr.misc.*;
import nfr.rc.*;
import java.awt.*;

public class AlertRecord implements ResConsts, Formattable
{
// public members
public int Serial = 0;
public int Timestamp = 0;
public AlertSrcRes AlertSrc = null;
public AlertRes Alert = null;

public static final int ACKSTATE_NA = 0;
public static final int ACKSTATE_ACKED = 1;
public static final int ACKSTATE_UNACKED = -1;

public int AckState = ACKSTATE_NA;

  // non-public members
  String[] mFields = null;
  String[] mSevStrings = null;
  AckCommentator mCommentator = null;
  AckHistory mHistory = null;
  Date mDate = null;
  String mHostname = null;

  static final int SEV_FIELD_WIDTH = 10;
  static final int HOST_FIELD_WIDTH = 20;
  static final int SOURCE_FIELD_WIDTH = 14;
  static final int ALERT_FIELD_WIDTH = 0;


public AlertRecord(int s, int t, String hostname, 
		   AlertSrcRes aSrc, AlertRes a, 
		   int state)
  {
    mFields = new String[5];

    Serial = s;
    Timestamp = t;
    AlertSrc = aSrc;
    Alert = a;
    AckState = state;
    mDate = new Date((long)Timestamp*1000);
    mHostname = new String(hostname);
    mSevStrings = new String[4];
    mSevStrings[SEV_NONE]    = "<none>";
    mSevStrings[SEV_INFO]    = "Info";
    mSevStrings[SEV_WARNING] = "Warning";
    mSevStrings[SEV_ERROR]   = "Error";   

    mFields[0] = new String(mSevStrings[Alert.getSev()]);
    mFields[1] = new String(mDate.toLocaleString());
    mFields[2] = new String(mHostname);
    mFields[3] = new String(AlertSrc.getName());
    mFields[4] = new String(Alert.getText());
  }

public String getLogName()
  {
    return getLogName(mHostname, mDate);
  }

public static String getLogName(String h, Date d)
  {
    return new String(h + "/" + getLogDateStr(d));
  }

public String getLogDateStr()
  {
    return getLogDateStr(mDate);
  }

public static String getLogDateStr(Date d)
  {
    return new String(DateThing.getMonthString(d.getMonth()) + "_" +
		      String.valueOf(d.getDate()) + "_" +
		      String.valueOf(d.getYear() + 1900) );
  }

public static int getSevWidth()
  { return SEV_FIELD_WIDTH; }

public static int getHostWidth()
  { return HOST_FIELD_WIDTH; }

public static int getTimeWidth()
  { return ((new Date()).toLocaleString()).length() + 2; }

public static int getSourceWidth()
  { return SOURCE_FIELD_WIDTH; }

public static int getAlertWidth()
  { return ALERT_FIELD_WIDTH; }

public String toString()
  {
    return new String(mFields[0] + "    " +
		      mFields[1] + "    " +
		      mFields[2] + "    " +
		      mFields[3] + ":   " + 
		      mFields[4]);
  }

public String getHostname()
  {
    return mHostname;
  }

public String[] getFields()
  {
    //System.out.println("in getFields()");
    //System.out.println("mFields[0] = " + mFields[0]);
    //System.out.println("mFields[1] = " + mFields[1]);
    //System.out.println("mFields[2] = " + mFields[2]);
    //System.out.println("mFields[3] = " + mFields[3]);
    return mFields;
  }

public void showCommentator(Window w)
  {
    if(mCommentator == null)
      mCommentator = new AckCommentator(this, w);
    mCommentator.show();
  }

public void killCommentator()
  {
    if(mCommentator != null)
      {
	mCommentator.dispose();
	mCommentator = null;
      }
  }

public void showHistory(Window w)
  {
    if(mHistory == null)
      mHistory = new AckHistory(this, w);
    mHistory.startUpdate();
    mHistory.show();
  }

public void updateHistory()
  {
    showHistory(NFRInfo.top);
    mHistory.update();
  }

public void killHistory()
  {
    if(mHistory != null)
      {
	mHistory.dispose();
	mHistory = null;
      }
  }

}
