/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   AlertViewer.java
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

package nfr.alerts;

import nfr.rc.*;
import nfr.misc.*;

import java.util.*;
import java.net.*;
import java.io.*;
import java.awt.*;


public class AlertViewer extends Panel implements ResConsts
{
  Panel mViewPanel = null;
  SortView mAllView = null;
  SortView mAckView = null;
  SortView mUnAckView = null;
  Button mAckHistoryBtn = null;
  Button mAckCommentBtn = null;
  Button mUnAckHistoryBtn = null;
  Button mUnAckCommentBtn = null;
  Button mAckBtn = null;
  DateThing mStartDate = null;
  DateThing mEndDate = null;
  ResTab mResTab = null;
  Checkbox mAllListCkx = null;
  Checkbox mAckListCkx = null;
  Checkbox mUnAckListCkx = null;
  CheckboxGroup mAckCkxGroup = null;
  int mCurStart = 0;
  int mCurEnd = 0;
  int mChunkSize = 0;
  Thread mLoadThd = null;
  Thread mAckThd = null;
  Thread mAutoLoadThd = null;
  boolean mLoaded = false;

public AlertViewer()
  {
    setFont(NFRInfo.font);

    mResTab = new ResTab();

    repTile l = new repTile();
    setLayout(l);
    l.setborder(CtoP.Width(1,this), CtoP.Width(1,this),
		CtoP.Height(1,this), CtoP.Height(1,this)/2);


    mViewPanel = new Panel();
    CardLayout vpl = new CardLayout();
    mViewPanel.setLayout(vpl);

    mAllView = new SortView(CtoP.Height(4, this), CtoP.Width(20, this));
    mAllView.addView("severity", new ListView_Sev(), 
		     AlertRecord.getSevWidth());
    mAllView.addView("time", new ListView_Time(), 
		     AlertRecord.getTimeWidth(),
		     true); // select this initially
    mAllView.addView("host", new ListView_Host(), 
		     AlertRecord.getHostWidth());
    mAllView.addView("source", new ListView_Source(), 
		     AlertRecord.getSourceWidth());
    mAllView.addView("alert", new ListView_Alert(),
		     AlertRecord.getAlertWidth());
    mAllView.build();

    Panel btns = new Panel();
    btns.add(new Button("reload"));
    btns.add(new Button("stop"));

    Panel lvp = new Panel();
    repTile lvpl = new repTile();
    lvp.setLayout(lvpl);
    lvp.add(mAllView);
    lvpl.setConstraints(0,0,0,0,"",mAllView);
    lvp.add(btns);
    lvpl.setConstraints(1,0,1,0,"ch",btns);
    lvpl.setStretchy(0,0);

    mViewPanel.add("All", lvp);

    mUnAckView = new SortView();
    mUnAckView.setListColors(Color.red, Color.white);
    mUnAckView.addView("severity", new ListView_Sev(), 
		       AlertRecord.getSevWidth());
    mUnAckView.addView("time", new ListView_Time(),
		       AlertRecord.getTimeWidth(),
		       true); // select this initially
    mUnAckView.addView("host", new ListView_Host(), 
		     AlertRecord.getHostWidth());
    mUnAckView.addView("source", new ListView_Source(),
		       AlertRecord.getSourceWidth());
    mUnAckView.addView("alert", new ListView_Alert(),
		       AlertRecord.getAlertWidth());
    mUnAckView.build();

    btns = new Panel();
    mUnAckHistoryBtn = new Button("history");
    btns.add(mUnAckHistoryBtn);
    mUnAckCommentBtn = new Button("comment");
    btns.add(mUnAckCommentBtn);
    mAckBtn = new Button("acknowledge");
    btns.add(mAckBtn);
    btns.add(new Button("reload"));
    btns.add(new Button("stop"));

    Panel avp = new Panel();
    repTile avpl = new repTile();
    avp.setLayout(avpl);
    avp.add(mUnAckView);
    avpl.setConstraints(0,0,0,0,"",mUnAckView);
    avp.add(btns);
    avpl.setConstraints(1,0,1,0,"ch",btns);
    avpl.setStretchy(0,0);

    mViewPanel.add("Unacknowledged", avp);

    mAckView = new SortView();
    mAckView.addView("severity", new ListView_Sev(),
		     AlertRecord.getSevWidth());
    mAckView.addView("time", new ListView_Time(),
		     AlertRecord.getTimeWidth(),
		     true); // select this initially
    mAckView.addView("host", new ListView_Host(), 
		     AlertRecord.getHostWidth());
    mAckView.addView("source", new ListView_Source(),
		     AlertRecord.getSourceWidth());
    mAckView.addView("alert", new ListView_Alert(),
		     AlertRecord.getAlertWidth());
    mAckView.build();
    avp = new Panel();

    btns = new Panel();
    mAckHistoryBtn = new Button("history");
    btns.add(mAckHistoryBtn);
    mAckCommentBtn = new Button("comment");
    btns.add(mAckCommentBtn);
    btns.add(new Button("reload"));
    btns.add(new Button("stop"));

    avp = new Panel();
    avpl = new repTile();
    avp.setLayout(avpl);
    avp.add(mAckView);
    avpl.setConstraints(0,0,0,0,"",mAckView);
    avp.add(btns);
    avpl.setConstraints(1,0,1,0,"ch",btns);
    avpl.setStretchy(0,0);

    mViewPanel.add("Acknowledged", avp);


    add(mViewPanel);
    l.setConstraints(0,0,0,0,"", mViewPanel);
    l.setStretchy(0,0);
    


    Panel bp = new Panel();

    repTile bpl = new repTile();
    bp.setLayout(bpl);
    
    Panel lbp = new Panel();
    repTile lbpl = new repTile();
    lbp.setLayout(lbpl);
    
    Label lbl = new Label("Entries no earlier than: ");
    lbp.add(lbl);
    lbpl.setConstraints(0,0,0,0,"", lbl);

    mStartDate = new DateThing();
    lbp.add(mStartDate);
    lbpl.setConstraints(0,1,0,1,"", mStartDate);

    lbl = new Label("and no later than: ");
    lbp.add(lbl);
    lbpl.setConstraints(1,0,1,0,"", lbl);

    mEndDate = new DateThing();
    lbp.add(mEndDate);
    lbpl.setConstraints(1,1,1,1,"", mEndDate);

    lbpl.setStretchy(-1,-1);

    bp.add(lbp);
    bpl.setConstraints(0,0,0,0,"", lbp);


    Panel mbp = new Panel();
    // nothing here currently
    
    bp.add(mbp);
    bpl.setConstraints(0,1,0,1,"", mbp);


    mAckCkxGroup = new CheckboxGroup();

    Panel rbp = new Panel();
    repTile rbpl = new repTile();
    rbp.setLayout(rbpl);
    mAllListCkx = new Checkbox("All", mAckCkxGroup, true);
    rbp.add(mAllListCkx);
    rbpl.setConstraints(0,0,0,0,"fl", mAllListCkx);

    mUnAckListCkx = new Checkbox("Unacknowledged", mAckCkxGroup, false);
    rbp.add(mUnAckListCkx);
    rbpl.setConstraints(1,0,1,0,"fl", mUnAckListCkx);

    mAckListCkx = new Checkbox("Acknowledged", mAckCkxGroup, false);
    rbp.add(mAckListCkx);
    rbpl.setConstraints(2,0,2,0,"fl", mAckListCkx);
    
    bp.add(rbp);
    bpl.setConstraints(0,2,0,2,"", rbp);


    bpl.setStretchy(-1,1);
    
    add(bp);
    l.setConstraints(1,0,1,0,"", bp);
    
  }         


public void show() 
  { 
    showHack();
    super.show();
  }

public void show(boolean b)
  {
    if(b)
      showHack();
    super.show(b);
  }

void showHack()
  {
    // bug: the following is to work around a problem where netscape
    // blows up if startLoad() is called before constructor for
    // toplevel completes.  this is most likely a bug in the load()
    // method or the BarMeter.repaint() method - fix later
    if(!mLoaded)
      {
	startLoad();
	mLoaded = true;
      }
  }

public boolean startLoad()
  {
    if(mLoadThd != null)
      mLoadThd.stop();
    mLoadThd = new Thread(new LoadThread(this));
    mLoadThd.start();
    return true;
  }

public boolean stopLoad()
  {
    if(mLoadThd != null)
      mLoadThd.stop();
    mStartDate.enable();
    mEndDate.enable();
    NFRInfo.dancyThing.stopDancing();
    return true;
  }

public boolean startAck(Object[] ars)
  {
    if(mAckThd != null)
      mAckThd.stop();
    mAckThd = new Thread(new AckThread(this, ars));
    mAckThd.start();
    return true;
  }

public boolean acknowledge(AlertRecord ar)
  {
    CgiScript cgi = new CgiScript("/cgi-bin/acknowledge");
    cgi.setType(cgi.POST);
    cgi.addDataNVPair("serial",  String.valueOf(ar.Serial));
    cgi.addDataNVPair("hostname",  ar.getHostname());
    cgi.addDataNVPair("date",  ar.getLogDateStr());
    if(cgi.run() != 0)
      {
	MsgBox.show("Error", cgi.getRelevantMsg());
	return false;
      }
    return true;
  }

public String[] getAvailLogs()
  {
    CgiScript cgi = new CgiScript("/cgi-bin/ls_logs");
    if(cgi.sendData() != 0)
      {
	MsgBox.show("Error", cgi.getRelevantMsg());
	return null;
      }

    if(cgi.getResponse() != 0)
      {
	MsgBox.show("Error", cgi.getRelevantMsg());
	return null;
      }

    DataInputStream In = cgi.getInputStream();
    Vector strings = new Vector();
    try
      {
	boolean hellNotFrozen = true;
	while(hellNotFrozen) // stupid javace won't compile while(true);
	  {
	    int strLen = In.readInt();
	    System.out.println("string length: " + strLen);
	    if(strLen > 0)
	      {
		char sbuf[] = new char[strLen];
		int i = 0;
		while(i < strLen)
		  sbuf[i++] = (char) In.readByte();
		String s = new String(sbuf);
		strings.addElement(s);
		System.out.println("got a host: " + s);
	      }
	  }
      }
    catch(EOFException e)
      {
	System.out.println("eof");
	String[] sa = new String[strings.size()];
	strings.copyInto(sa);
	return sa;
	//'sok, we were expecting this
      }
    
    catch(Exception e)
      {
	System.out.println("wiped after " + strings.size());
	e.printStackTrace();
	return null;
      }

    return null; // not reaced, just shut up javac
  }

public boolean load()
  {
    boolean result = true;
    mAllView.clear();
    mAckView.clear();
    mUnAckView.clear();
    String[] logs = getAvailLogs();

    long t = mStartDate.getTime();
    while(mStartDate.getTime() <= mEndDate.getTime())
      {
	int i = 0;
	while(i < logs.length)
	  {
	    System.out.println("trying to load log: " + logs[i]);
	    result &= loadHostLog(logs[i++],
				  AlertRecord.getLogDateStr(new Date(mStartDate.getTime())) );
	  }
	mStartDate.incDay();
      }
    mStartDate.setTime(t);
    return result;
  }

public boolean loadHostLog(String hostname, String dateString)
  {
    //System.out.println("AlertViewer.load()");
    
    NFRInfo.dancyThing.startDancing();

    int ResultCode = 0;
    boolean result = true;

    DataInputStream In = null;
    DataOutputStream Out = null;

    // first load a contiguous chunk of the alert log
    System.out.println("loading alert log");
    CgiScript cgi = new CgiScript("/cgi-bin/alertdump");
    cgi.addDataNVPair("log_name", "data/alerts/" + 
		      hostname + "/" + dateString + "/log.dat");
    cgi.addDataNVPair("idx_name", "data/alerts/" + 
		      hostname  + "/" + dateString + "/log.idx");

    if(cgi.sendData() != 0)
      {
	MsgBox.show("Error", cgi.getRelevantMsg());
	NFRInfo.dancyThing.stopDancing();
	return false;
      }

    ResultCode = cgi.getResponse();
    if(ResultCode == 7) // just no data
      {
	NFRInfo.dancyThing.stopDancing();
	return true;
      }
    else if(ResultCode != 0)
      {
	MsgBox.show("Error", cgi.getRelevantMsg());
	NFRInfo.dancyThing.stopDancing();
	return false;
      }

    In = cgi.getInputStream();

    int count = readAlerts(In, mAllView, hostname,
			   AlertRecord.ACKSTATE_NA);
    if(count < 0)
      result = false;

    System.out.println("done loading alert log");

    cgi.close();

    if(!result)
      {
	NFRInfo.dancyThing.stopDancing();
	return false;
      }

    result = true;
    // now load a chunk of any alerts that are unacknowledged
    cgi = new CgiScript("/cgi-bin/ackdump");
    cgi.addDataNVPair("log_name", "data/alerts/" + 
		      hostname + "/" + dateString + "/log.dat");
    cgi.addDataNVPair("idx_name", "data/alerts/" + 
		      hostname + "/" + dateString + "/log.idx");
    cgi.addDataNVPair("target_dir", "data/alerts/" + 
		      hostname + "/" + dateString + "/unacked");

    if(cgi.sendData() != 0)
      {
	MsgBox.show("Error", cgi.getRelevantMsg());
	result = false;
      }

    ResultCode = cgi.getResponse();
    if(ResultCode == 7) // just no data
      result = false;
    else if(ResultCode != 0)
      {
	MsgBox.show("Error", cgi.getRelevantMsg());
	result = false;
      }

    In = cgi.getInputStream();

    if(result)
      {
	count = readAlerts(In, mUnAckView, hostname,
			   AlertRecord.ACKSTATE_UNACKED);
	if(count < 0)
	  result = false;
      }
    if(mUnAckView.count() > 0)
      {
	mUnAckListCkx.setForeground(Color.red);
	mUnAckListCkx.show(); // netscpe4 doesn't do this automatically :(
      }
    else
      {
	mUnAckListCkx.setForeground(Color.black);
	mUnAckListCkx.show(); // netscpe4 doesn't do this automatically :(
      }

    cgi.close();

    // now load a chunk of any alerts that are acknowledged
    cgi = new CgiScript("/cgi-bin/ackdump");
    cgi.addDataNVPair("log_name", "data/alerts/" + 
		      hostname + "/" + dateString + "/log.dat");
    cgi.addDataNVPair("idx_name", "data/alerts/" + 
		      hostname + "/" + dateString + "/log.idx");
    cgi.addDataNVPair("target_dir", "data/alerts/" + 
		      hostname + "/" + dateString + "/acked");

    result = true;

    if(cgi.sendData() != 0)
      {
	MsgBox.show("Error", cgi.getRelevantMsg());
	result = false;
      }

    ResultCode = cgi.getResponse();
    if(ResultCode == 7) // just no data
      result = false;
    else if(ResultCode != 0)
      {
	MsgBox.show("Error", cgi.getRelevantMsg());
	result = false;
      }

    In = cgi.getInputStream();

    if(result)
      {
	count = readAlerts(In, mAckView, hostname, 
			   AlertRecord.ACKSTATE_ACKED);
	if(count < 0)
	  result = false;
      }

    NFRInfo.dancyThing.stopDancing();

    return true;
  }


  int readAlerts(DataInputStream In, SortView targetView, 
		 String hostname, int state)
  {
    System.out.println("in readAlerts()");
    int total = 0;
    try
      {
	int serial = In.readInt();
	boolean True = true;
	while(True) //stupid javac won't compile while(true), ^%$@#@!!
	  {
	    int timestamp = In.readInt();
	    int rc = In.readInt();
	    AlertSrcRes aSrc = new AlertSrcRes(rc, In, mResTab);
	    rc = In.readInt();
	    AlertRes a = new AlertRes(rc, In, mResTab);

	    targetView.insert(new AlertRecord(serial, timestamp, hostname,
					      aSrc, a, 
					      state));
	    total++;
	    serial = In.readInt();	    
	  }
      }

    catch(EOFException e)
      {
	System.out.println("eof");
	return total;
	//'sok, we were expecting this
      }

    catch(Exception e)
      {
	System.out.println("in readAlerts, wiped after " + total);
	e.printStackTrace();
	return -1;
      }
    return total; // not reached, just shut up javac
  }

public boolean action(Event e, Object arg)
  {
    System.out.println("got an action in AlertViewer: " + e);
    if(e.target == mAllListCkx)
      {
	((CardLayout)mViewPanel.getLayout()).show(mViewPanel,"All");
	return true;
      }
    else if(e.target == mAckListCkx)
      {
	((CardLayout)mViewPanel.getLayout()).show(mViewPanel,"Acknowledged");
	return true;
      }
    else if(e.target == mUnAckListCkx)
      {
	((CardLayout)mViewPanel.getLayout()).show(mViewPanel,"Unacknowledged");
	return true;
      }

    else if (e.target == mUnAckCommentBtn)
      {
	Object[] ars = mUnAckView.getCurSelections();
	if(ars != null)
	  {
	    for(int i = 0; i < ars.length; i++)
	      ((AlertRecord)ars[i]).showCommentator(NFRInfo.top);
	  }
	return true;
      }
    else if ((e.target == mUnAckHistoryBtn)  || (e.target == mUnAckView))
      {
	Object[] ars = mUnAckView.getCurSelections();
	if(ars != null)
	  {
	    for(int i = 0; i < ars.length; i++)
	      ((AlertRecord)ars[i]).showHistory(NFRInfo.top);
	  }
	return true;
      }
    else if (e.target == mAckBtn)
      {
	Object[] ars = mUnAckView.getCurSelections();
	if(ars != null)
	  {
	    for(int i = 0; i < ars.length; i++)
	      startAck(ars);
	  }
	return true;
      }
    else if (e.target == mAckCommentBtn)
      {
	Object[] ars = mAckView.getCurSelections();
	if(ars != null)
	  {
	    for(int i = 0; i < ars.length; i++)
	      ((AlertRecord)ars[i]).showCommentator(NFRInfo.top);
	  }
	return true;
      }
    else if ((e.target == mAckHistoryBtn) || (e.target == mAckView))
      {
	System.out.println("gonna show history");
	Object[] ars = mAckView.getCurSelections();
	if(ars != null)
	  {
	    for(int i = 0; i < ars.length; i++)
	      ((AlertRecord)ars[i]).showHistory(NFRInfo.top);
	  }
	return true;
      }

    else if (("reload").equals(arg))
      {
	startLoad();
	return true;
      }
    else if (("stop").equals(arg))
      {
	stopLoad();
	return true;
      }
    else
      return false;
  }


}


class AckThread implements Runnable
{
  Object[] mAlertRecords = null;
  AlertViewer mAlertViewer = null;

  AckThread(AlertViewer a, Object[] ars)
  {
    mAlertViewer = a;
    mAlertRecords = ars;
  }

public void run()
  {
    boolean Result = true;
    for(int i = 0; i < mAlertRecords.length; i++)
      {
	if(!mAlertViewer.acknowledge( ((AlertRecord)mAlertRecords[i]) )  )
	  Result = false;
      }
    if(!Result)
      MsgBox.show("Error", "can't ack.");
    mAlertViewer.mAckThd = null;
    mAlertViewer.startLoad();
  }
}

class LoadThread implements Runnable
{
  AlertViewer mAlertViewer = null;

  LoadThread(AlertViewer a)
  {
    mAlertViewer = a;
  }

public void run()
  {
    mAlertViewer.mStartDate.disable();
    mAlertViewer.mEndDate.disable();
    if(!mAlertViewer.load())
      MsgBox.show("Error", "can't load alert log");
    mAlertViewer.mLoadThd = null;
    mAlertViewer.mStartDate.enable();
    mAlertViewer.mEndDate.enable();
  }
}


class ListView_Time extends ListView
{
public int compare(Object one, Object two)
  {
    return ( ((AlertRecord)one).Timestamp - ((AlertRecord)two).Timestamp );
  }
}

class ListView_Host extends ListView
{
public int compare(Object one, Object two)
  {
    String nameOne = ((AlertRecord)one).getHostname();
    String nameTwo = ((AlertRecord)two).getHostname();
    return nameOne.compareTo(nameTwo);
  }
}

class ListView_Sev extends ListView
{
public int compare(Object one, Object two)
  {
    // do these in reverse order
    return ( ((AlertRecord)two).Alert.getSev() - 
	     ((AlertRecord)one).Alert.getSev() );
  }
}

class ListView_Alert extends ListView
{
public int compare(Object one, Object two)
  {
    return ( ((AlertRecord)one).Alert.getResNo() - 
	     ((AlertRecord)two).Alert.getResNo() );
  }
}

class ListView_Source extends ListView
{
public int compare(Object one, Object two)
  {
    String nameOne = ((AlertRecord)one).AlertSrc.getName();
    String nameTwo = ((AlertRecord)two).AlertSrc.getName();
    return nameOne.compareTo(nameTwo);
  }
}


