//  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
//  All rights reserved.
//
//  Use and distribution of this software and its source code
//  are governed by the terms and conditions of the
//  Network Flight Recorder Software License ("LICENSE.TXT")
//
//  File:   FacEditor.java 
//  Author: Eric Wall <eric@nfr.net>
//

package nfr.facilities;

// base class for panels that display and update cf files

import java.awt.*;
import java.util.*;
import java.net.*;

import nfr.misc.*;

public class FacEditor extends Panel
{
  Cf mCf;
  //static Cf guiCf;
  Component mFields[][];
  int mCount;
  int rowCount;

public FacEditor() {}

public FacEditor(Cf guiCf, URL BaseURL, String Path)
  {
    // Debug
    String guiStyle = (String) guiCf.get("gui_style");
    if(guiStyle == null) { guiStyle = "null"; }
    System.out.println("Info from guiCf: " + guiStyle);
    // Debug

    this.setLayout(new GridBagLayout());

    // Read the queue.cf file
    mCf=new Cf(BaseURL, Path);
    mCf.Read();

    // Map of this array:
    //
    // mFields[i][0] = 
    // mFields[i][1] = 
    // mFields[i][2] = 
    mFields = new Component[mCf.size()][3];
    Enumeration Keys = mCf.keys();

    // First unused row
    rowCount = 6;
    
    while(Keys.hasMoreElements())
      {

	// Pull the information out of the hash tables
	//
	// Key is the internal (ugly) system name
	// goodLabel is the (pretty) user label
	// value is the value from this name=value pair
	String Key = (String) Keys.nextElement();
	String goodLabel = (String) guiCf.get(Key);
	String value = (String) mCf.get(Key);

	// Pick the right label, make the components
	if((goodLabel == null) || (goodLabel.equals(""))) {
	  mFields[mCount][0] = new Label(Key);
	} else {
	  System.out.println("Using goodLabel: " + goodLabel);
	  mFields[mCount][0] = new Label(goodLabel);
	}
	//mFields[mCount][1] = new TextField(((String) mCf.get(Key)), 30);
	String tmp = (String) mCf.get(Key);
	mFields[mCount][1] = new TextField( tmp , 30);
	
	// Space wasting Kludge, because I don't want to convert
	// mFields to a more complex data structure.
	mFields[mCount][2] = new Label(Key); 

	// Some logic to determine where to put certain components
	System.out.println("Working on: " + Key);
	
	if(Key.equals("Exec_Path")) {
	  gbadd(this,mFields[mCount][0],0,0,1,1,
		GridBagConstraints.NONE,
		GridBagConstraints.NORTHWEST,
		0.0,0.0,5,5,5,5);
	  gbadd(this,mFields[mCount][1],1,0,1,1,
		GridBagConstraints.NONE,
		GridBagConstraints.NORTHWEST,
		0.0,0.0,5,5,5,5);
	  System.out.println("Exec_Path");
	}
	
        else if(Key.equals("Exec_Args")) {
	  gbadd(this,mFields[mCount][0],0,1,1,1,
		GridBagConstraints.NONE,
		GridBagConstraints.NORTHWEST,
		0.0,0.0,5,5,5,5);
	  gbadd(this,mFields[mCount][1],1,1,1,1,
		GridBagConstraints.NONE,
		GridBagConstraints.NORTHWEST,
		0.0,0.0,5,5,5,5);
	  System.out.println("Exec_Args");
	}
	
        else if(Key.equals("Exec_Terminator")) {
	  gbadd(this,mFields[mCount][0],0,2,1,1,
		GridBagConstraints.NONE,
		GridBagConstraints.NORTHWEST,
		0.0,0.1,5,5,5,5);
	  gbadd(this,mFields[mCount][1],1,2,1,1,
		GridBagConstraints.NONE,
		GridBagConstraints.NORTHWEST,
		0.0,0.1,5,5,5,5);
	  System.out.println("Exec_Terminator");
	}

	else if(Key.equals("Flush_Count")) {
	  gbadd(this,mFields[mCount][0],0,3,1,1,
		GridBagConstraints.NONE,
		GridBagConstraints.NORTHWEST,
		0.0,0.0,5,5,5,5);
	  gbadd(this,mFields[mCount][1],1,3,1,1,
		GridBagConstraints.NONE,
		GridBagConstraints.NORTHWEST,
		0.0,0.0,5,5,5,5);
	  System.out.println("Flush_Count");
	}

	else if(Key.equals("Flush_Time")) {
	  gbadd(this,mFields[mCount][0],0,4,1,1,
		GridBagConstraints.NONE,
		GridBagConstraints.NORTHWEST,
		0.0,0.0,5,5,5,5);
	  gbadd(this,mFields[mCount][1],1,4,1,1,
		GridBagConstraints.NONE,
		GridBagConstraints.NORTHWEST,
		0.0,0.0,5,5,5,5);
	  System.out.println("Flush_Time");
	}
	
	else if(Key.equals("Flush_Method")) {
	  gbadd(this,mFields[mCount][0],0,5,1,1,
		GridBagConstraints.NONE,
		GridBagConstraints.NORTHWEST,
		0.0,0.1,5,5,5,5);
	  gbadd(this,mFields[mCount][1],1,5,1,1,
		GridBagConstraints.NONE,
		GridBagConstraints.NORTHWEST,
		0.0,0.1,5,5,5,5);
	  System.out.println("Flush_Method");
	}
	
	else {
	  gbadd(this,mFields[mCount][0],0,rowCount,1,1,
		GridBagConstraints.NONE,
		GridBagConstraints.NORTHWEST,
		0.0,0.0,5,5,5,5);
	  gbadd(this,mFields[mCount][1],1,rowCount,1,1,
		GridBagConstraints.NONE,
		GridBagConstraints.NORTHWEST,
		0.0,0.0,5,5,5,5);
	  rowCount++;
	  
	  System.out.println("Key: " + Key);
	}

	mCount++;
      } /* End While */
    /******
    Button save = new Button("Save");
    gbadd(this,save,0,rowCount,1,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.SOUTHWEST,
	  0.0,0.1,5,5,5,5);

    Button reload = new Button("Reload");
    gbadd(this,reload,1,rowCount,1,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.SOUTHWEST,
	  0.0,0.1,5,5,5,5);
	  ****/
  } /* End Constructor */

  // Wraper function for filling out the GridBagConstraints object
  private void gbadd(Container container, Component component, 
		     int grid_x, int grid_y, int grid_width, int grid_height,
		     int fill, int anchor, double weight_x, double weight_y,
		     int top, int left, int bottom, int right)
  {
    GridBagConstraints c = new GridBagConstraints();
    c.gridx = grid_x; c.gridy = grid_y;
    c.gridwidth = grid_width; c.gridheight = grid_height;
    c.fill = fill; c.anchor = anchor;
    c.weightx = weight_x; c.weighty = weight_y;
    if (top+bottom+left+right > 0)
      c.insets = new Insets(top, left, bottom, right);
    
    ((GridBagLayout)container.getLayout()).setConstraints(component, c);
    container.add(component);
  }


  // 6. create a static method in FacEditPanel called getEditor 
  // which takes a facility name and returns a FacEditPanel
  // object appropriate for the named facility.  For now this 
  // will just return an instance of FacEditPanel in the same way 
  // as the 
  //
  //    CfEditor = new CfEditPanel(BaseURL, 
  // 				"etc/alertd/facilities/" + FacName + 
  //				"/queue.cf");
  //
  // call in FacEditor.java does.  Later we can subclass 
  // FacEditPanel and have this function return different 
  // classes for different facilities.


  public static FacEditor getEditor(URL BaseURL, String facName)
  {

    // open the gui.cf file and read it
    Cf guiCf = new Cf(BaseURL,"etc/alertd/facilities/" + facName + "/gui.cf");

    System.out.println(" *** Opening: etc/alertd/facilities/" + facName + "/gui.cf");

    if(!guiCf.Read())
      return(new FacEditor(guiCf, BaseURL,"etc/alertd/facilities/" + facName + "/queue.cf"));
    String guiStyle = (String) guiCf.get("gui_style");

    System.out.println(" *** Found a gui_style: " + guiStyle);
    
    if(guiStyle == null) {
      System.out.println("guiType: null");
      return(new FacEditor(guiCf, BaseURL,"etc/alertd/facilities/" + facName + "/queue.cf"));
    }

    if(guiStyle.equals("mail")) {
      System.out.println("guiType: mail");
      return(new MailEditor(guiCf, BaseURL,"etc/alertd/facilities/" + facName + "/queue.cf"));
    }

    if(guiStyle.equals("page")) {
      System.out.println("guiType: page");
      return(new PageEditor(guiCf, BaseURL,"etc/alertd/facilities/" + facName + "/queue.cf"));
    }

    if(guiStyle.equals("fax")) {
      System.out.println("guiType: fax");
      return(new FaxEditor(guiCf, BaseURL,"etc/alertd/facilities/" + facName + "/queue.cf"));
    }

    // if(guiStyle.equals("log")) {
    //   System.out.println("guiType: log");
    //   return(new LogEditor(guiCf, BaseURL,"etc/alertd/facilities/" + facName + "/queue.cf"));
    // }
      
    // Allow a guiType of generic to be handeled by the catch all return
    // at the end of this method.  
    // if(guiStyle.equals("generic")) {
    //   System.out.println("guiType: generic");
    //   return(new FacEditor(guiCf, BaseURL,"etc/alertd/facilities/" + facName + "/queue.cf"));
    // }
    
    // If all else fails return a Generic Editor    
    System.out.println("guiType: generic");
    return(new FacEditor(guiCf, BaseURL,"etc/alertd/facilities/" + facName + "/queue.cf"));
  }

  /*** leave this out, we'll provide buttons in the container
  public boolean action(Event e, Object arg)
  {
    String CfName = null;
    if(arg == "Save") {
      this.Merge();
      System.out.println("You hit Save");
      return true;
    }
    if(arg == "Reload") {
      this.Reload();
      System.out.println("You hit Reload");
      return true;
    }
    return false;
  }
  ***/
  
  // The werid error is most likely here.
  public boolean Merge()
  {
    int i = mCount;
    while (i-- > 0)
      {
	mCf.put( ( (Label) mFields[i][2] ).getText(), 
		 ( (TextField) mFields[i][1] ).getText() );
      }
    return mCf.Merge();
  }
  

  public boolean Reload()
  {
    mCf.Read();
    Enumeration Keys = mCf.keys();
    int i = 0;
    while( (Keys.hasMoreElements()) && (i < mCount) ) // don't add more elments
      {
	String Key = (String) Keys.nextElement();
	( (Label) mFields[i][0] ).setText(Key);
	( (TextField) mFields[i++][1] ).setText( (String)mCf.get(Key));
      }
    // bug: handle underflow here
    return true;
  }

  public static void main(String[] args)
  {
    System.out.println("FacEditor.class");
  }
}
