//  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
//  All rights reserved.
//
//  Use and distribution of this software and its source code
//  are governed by the terms and conditions of the
//  Network Flight Recorder Software License ("LICENSE.TXT")
//
//  File:   MailEditor.java 
//  Author: Eric Wall <eric@nfr.net>
//


package nfr.facilities;

// base class for panels that display and update cf files

import java.awt.*;
import java.util.*;
import java.net.*;

import nfr.misc.*;

public class MailEditor extends FacEditor
{

  Cf mCf;
  Label recipLabel;
  Label fTimeLabel, fCountLabel, fMethodLabel;
  TextField recipField;
  TextField fTimeField, fCountField, fSubjectField;
  // fMethodField;

  public MailEditor(Cf guiCf, URL BaseURL, String Path) 
  {
    String value;
    String goodLabel;

    // Set layout manager and read queue.cf file
    this.setLayout(new GridBagLayout());
    mCf=new Cf(BaseURL, Path);
    mCf.Read();
    
    // Recipient
    goodLabel = (String) guiCf.get("Recipient");
    if((goodLabel == null) || (goodLabel.equals(""))) {
      recipLabel = new Label("Recipient");
    } else {
      recipLabel = new Label(goodLabel);
    }
    gbadd(this,recipLabel,0,0,1,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.NORTHWEST,
	  0.0,0.0,5,5,5,5);

    value = (String) mCf.get("Recipient");
    recipField = new TextField(value, 30);
    gbadd(this,recipField,1,0,1,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.NORTHWEST,
	  0.0,0.0,5,5,5,5);
    
    
    // Flush Time
    goodLabel = (String) guiCf.get("Flush_Time");
    if((goodLabel == null) || (goodLabel.equals(""))) {
      fTimeLabel = new Label("Flush_Time");
    } else {
      fTimeLabel = new Label(goodLabel);
    }
    gbadd(this,fTimeLabel,0,1,1,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.NORTHWEST,
	  0.0,0.0,5,5,5,5);

    value = (String) mCf.get("Flush_Time");
    fTimeField = new TextField(value, 30);
    gbadd(this,fTimeField,1,1,1,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.NORTHWEST,
	  0.0,0.0,5,5,5,5);
    

    // Flush Count
    goodLabel = (String) guiCf.get("Flush_Count");
    if((goodLabel == null) || (goodLabel.equals(""))) {
      fCountLabel = new Label("Flush_Count");
    } else {
      fCountLabel = new Label(goodLabel);
    }
    gbadd(this,fCountLabel,0,2,1,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.NORTHWEST,
	  0.0,0.0,5,5,5,5);
    
    value = (String) mCf.get("Flush_Count");
    fCountField = new TextField(value, 30);
    gbadd(this,fCountField,1,2,1,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.NORTHWEST,
	  0.0,0.0,5,5,5,5);


    /*******
    // Flush Method
    goodLabel = (String) guiCf.get("Flush_Method");
    if((goodLabel == null) || (goodLabel.equals(""))) {
      fMethodLabel = new Label("Flush_Method");
    } else {
      fMethodLabel = new Label(goodLabel);
    }
    gbadd(this,fMethodLabel,0,3,1,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.NORTHWEST,
	  0.0,0.1,5,5,5,5);
    
    value = (String) mCf.get("Flush_Method");
    fMethodField = new TextField(value, 30);
    gbadd(this,fMethodField,1,3,1,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.NORTHWEST,
	  0.0,0.1,5,5,5,5);
    *****/

    // Subject
    fMethodLabel = new Label(guiCf.textorname("Subject"));
    gbadd(this,fMethodLabel,0,3,1,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.NORTHWEST,
	  0.0,0.1,5,5,5,5);
    
    value = (String) mCf.get("Subject");
    fSubjectField = new TextField(value, 30);
    gbadd(this,fSubjectField,1,3,1,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.NORTHWEST,
	  0.0,0.1,5,5,5,5);

  }

  // Wraper function for filling out the GridBagConstraints object
  private void gbadd(Container container, Component component, 
		     int grid_x, int grid_y, int grid_width, int grid_height,
		     int fill, int anchor, double weight_x, double weight_y,
		     int top, int left, int bottom, int right)
  {
    GridBagConstraints c = new GridBagConstraints();
    c.gridx = grid_x; c.gridy = grid_y;
    c.gridwidth = grid_width; c.gridheight = grid_height;
    c.fill = fill; c.anchor = anchor;
    c.weightx = weight_x; c.weighty = weight_y;
    if (top+bottom+left+right > 0)
      c.insets = new Insets(top, left, bottom, right);
    
    ((GridBagLayout)container.getLayout()).setConstraints(component, c);
    container.add(component);
  }

  public boolean Merge()
  {
    System.out.println("In MailEditor.Merge() method");

    mCf.put("Recipient",recipField.getText()); 
    mCf.put("Flush_Time",fTimeField.getText());     
    mCf.put("Flush_Count",fCountField.getText());    
    mCf.put("Subject",fSubjectField.getText());    
    // mCf.put("Flush_Method",fMethodField.getText()); 

    return mCf.Merge();
  }

  
  public boolean Reload()
  {
    mCf.Read();
    System.out.println("In MailEditor.Reload() method");

    recipField.setText((String) mCf.get("Recipient"));
    fTimeField.setText((String) mCf.get("Flush_Time")); 
    fCountField.setText((String) mCf.get("Flush_Count"));
    fSubjectField.setText((String) mCf.get("Subject"));
    // fMethodField.setText((String) mCf.get("Flush_Method"));

    return true;
  }

  public static void main(String[] args) {
    System.out.println("MailEditor.class");
  }
} 

