//  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
//  All rights reserved.
//
//  Use and distribution of this software and its source code
//  are governed by the terms and conditions of the
//  Network Flight Recorder Software License ("LICENSE.TXT")
//
//  File:   PageEditor.java 
//  Author: Eric Wall <eric@nfr.net>
//

package nfr.facilities;

// base class for panels that display and update cf files

import java.awt.*;
import java.util.*;
import java.net.*;

import nfr.misc.*;

public class PageEditor extends FacEditor
{
  
  Cf mCf;
  Label pagenumLabel, pinLabel;
  Label fTimeLabel, fCountLabel, fMethodLabel;
  TextField pagenumField, pinField; 
  TextField fTimeField, fCountField;
  // fMethodField;

  public PageEditor(Cf guiCf, URL BaseURL, String Path) 
  {
    String value;
    String goodLabel;

    // Set layout manager and read queue.cf file
    this.setLayout(new GridBagLayout());
    mCf=new Cf(BaseURL, Path);
    mCf.Read();

    // PagerNum
    goodLabel = (String) guiCf.get("PagerNum");
    if((goodLabel == null) || (goodLabel.equals(""))) {
      pagenumLabel = new Label("PagerNum");
    } else {
      pagenumLabel = new Label(goodLabel);
    }
    gbadd(this,pagenumLabel,0,0,1,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.NORTHWEST,
	  0.0,0.0,5,5,5,5);

    value = (String) mCf.get("PagerNum");
    pagenumField = new TextField(value, 30);
    gbadd(this,pagenumField,1,0,1,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.NORTHWEST,
	  0.0,0.0,5,5,5,5);

    // PIN
    goodLabel = (String) guiCf.get("PIN");
    if((goodLabel == null) || (goodLabel.equals(""))) {
      pagenumLabel = new Label("PIN");
    } else {
      pagenumLabel = new Label(goodLabel);
    }
    gbadd(this,pagenumLabel,0,1,1,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.NORTHWEST,
	  0.0,0.0,5,5,5,5);

    value = (String) mCf.get("PIN");
    pinField = new TextField(value, 30);
    gbadd(this,pinField,1,1,1,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.NORTHWEST,
	  0.0,0.0,5,5,5,5);

    // Flush Time
    goodLabel = (String) guiCf.get("Flush_Time");
    if((goodLabel == null) || (goodLabel.equals(""))) {
      fTimeLabel = new Label("Flush_Time");
    } else {
      fTimeLabel = new Label(goodLabel);
    }
    gbadd(this,fTimeLabel,0,2,1,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.NORTHWEST,
	  0.0,0.0,5,5,5,5);

    value = (String) mCf.get("Flush_Time");
    fTimeField = new TextField(value, 30);
    gbadd(this,fTimeField,1,2,1,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.NORTHWEST,
	  0.0,0.0,5,5,5,5);

      // Flush Count
    goodLabel = (String) guiCf.get("Flush_Count");
    if((goodLabel == null) || (goodLabel.equals(""))) {
      fCountLabel = new Label("Flush_Count");
    } else {
      fCountLabel = new Label(goodLabel);
    }
    gbadd(this,fCountLabel,0,3,1,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.NORTHWEST,
	  0.0,0.0,5,5,5,5);
    
    value = (String) mCf.get("Flush_Count");
    fCountField = new TextField(value, 30);
    gbadd(this,fCountField,1,3,1,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.NORTHWEST,
	  0.0,0.0,5,5,5,5);

    /****
      // Flush Method
    goodLabel = (String) guiCf.get("Flush_Method");
    if((goodLabel == null) || (goodLabel.equals(""))) {
      fMethodLabel = new Label("Flush_Method");
    } else {
      fMethodLabel = new Label(goodLabel);
    }
    gbadd(this,fMethodLabel,0,4,1,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.NORTHWEST,
	  0.0,0.1,5,5,5,5);
    
    value = (String) mCf.get("Flush_Method");
    fMethodField = new TextField(value, 30);
    gbadd(this,fMethodField,1,4,1,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.NORTHWEST,
	  0.0,0.1,5,5,5,5);
    ****/
  }

  // Wraper function for filling out the GridBagConstraints object
  private void gbadd(Container container, Component component, 
		     int grid_x, int grid_y, int grid_width, int grid_height,
		     int fill, int anchor, double weight_x, double weight_y,
		     int top, int left, int bottom, int right)
  {
    GridBagConstraints c = new GridBagConstraints();
    c.gridx = grid_x; c.gridy = grid_y;
    c.gridwidth = grid_width; c.gridheight = grid_height;
    c.fill = fill; c.anchor = anchor;
    c.weightx = weight_x; c.weighty = weight_y;
    if (top+bottom+left+right > 0)
      c.insets = new Insets(top, left, bottom, right);
    
    ((GridBagLayout)container.getLayout()).setConstraints(component, c);
    container.add(component);
  }

  public boolean Merge()
  {
    System.out.println("In PageEditor.Merge() method");

    mCf.put("PagerNum",pagenumField.getText()); 
    mCf.put("PIN",pinField.getText()); 
    mCf.put("Flush_Time",fTimeField.getText());     
    mCf.put("Flush_Count",fCountField.getText());    
    // mCf.put("Flush_Method",fMethodField.getText()); 

    return mCf.Merge();
  }
  
  
  public boolean Reload()
  {
    System.out.println("In PageEditor.Reload() method");

    mCf.Read();

    pagenumField.setText((String) mCf.get("PagerNum"));
    pinField.setText((String) mCf.get("PIN"));
    fTimeField.setText((String) mCf.get("Flush_Time")); 
    fCountField.setText((String) mCf.get("Flush_Count"));
    // fMethodField.setText((String) mCf.get("Flush_Method"));

    return true;
  }
    
  static int i;
  private void p() {
    i++;
    System.out.println("Tag " + i);
  }

  public static void main(String[] args) {
    System.out.println("PageEditor.class");
  }
} 

