//  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
//  All rights reserved.
//
//  Use and distribution of this software and its source code
//  are governed by the terms and conditions of the
//  Network Flight Recorder Software License ("LICENSE.TXT")
//
//  File:   AbsoluteTime.java
//  Author: Eric Wall <eric@nfr.net>
//


package nfr.misc;

import java.awt.*;
import java.util.*;

public class AbsoluteTime extends Panel {
  private Date date;
  private int resolution;

  private Choice monthChoice, ampmChoice;
  private TextField yearField, dayField, hourField, minuteField, secondField;

  private CheckboxGroup cbg;
  private Checkbox cb1, cb2;

  Cf st;


  public AbsoluteTime (boolean editable, int min_resolution)
  {

    st = new Cf(NFRInfo.document_base,"etc/javastrings/AbsoluteTime.str");
    st.Read();

    this.setLayout(new GridBagLayout());
    

    resolution = min_resolution;
    date = new Date(80,0,1,0,0,0);

    Panel datePanel = new Panel();
    datePanel.setLayout(new GridBagLayout());
    Panel timePanel = new Panel();
    timePanel.setLayout(new GridBagLayout());

    // Date Components
    dayField = new TextField("1",2);
    Label comma = new Label(",");
    yearField = new TextField("1980",4);
    monthChoice = new Choice();

    monthChoice.addItem(st.textorname("Jan"));
    monthChoice.addItem(st.textorname("Feb"));
    monthChoice.addItem(st.textorname("Mar"));
    monthChoice.addItem(st.textorname("Apr"));
    monthChoice.addItem(st.textorname("May"));
    monthChoice.addItem(st.textorname("Jun"));
    monthChoice.addItem(st.textorname("Jul"));
    monthChoice.addItem(st.textorname("Aug"));
    monthChoice.addItem(st.textorname("Sep"));
    monthChoice.addItem(st.textorname("Oct"));
    monthChoice.addItem(st.textorname("Nov"));
    monthChoice.addItem(st.textorname("Dec"));

    gbadd(datePanel,monthChoice,0,0,1,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.CENTER,
	  0.1,0.0,0,0,0,0);

    if(min_resolution <= 86400) {

      gbadd(datePanel,dayField,1,0,1,1,
	    GridBagConstraints.NONE,
	    GridBagConstraints.CENTER,
	    0.1,0.0,0,0,0,0);
      
      gbadd(datePanel,comma,2,0,1,1,
	    GridBagConstraints.NONE,
	    GridBagConstraints.CENTER,
	    0.1,0.0,0,0,0,0);
    }
    
    gbadd(datePanel,yearField,3,0,1,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.CENTER,
	  0.1,0.0,0,0,0,0);


    // Time Components
    hourField = new TextField("12",2);
    minuteField = new TextField("00",2);
    secondField = new TextField("00",2);

    Label colon = new Label(":");
    Label colon2 = new Label(":");
    Label zeros = new Label("00");
    Label col1with0 = new Label(": 00");
    Label col2with0 = new Label(": 00");

    ampmChoice = new Choice();
    ampmChoice.addItem("AM");
    ampmChoice.addItem("PM");

    if(min_resolution <= 3600) {

      // First Blob
      gbadd(timePanel,hourField,0,0,1,1,
	    GridBagConstraints.NONE,
	    GridBagConstraints.CENTER,
	    0.5,0.0,0,0,0,0);
      
      gbadd(timePanel,ampmChoice,5,0,1,1,
	    GridBagConstraints.REMAINDER,
	    GridBagConstraints.CENTER,
	    0.5,0.0,0,0,0,0);

      if(min_resolution <= 60) {
	// Second Blob
	gbadd(timePanel,colon,1,0,1,1,
	      GridBagConstraints.NONE,
	      GridBagConstraints.CENTER,
	      0.1,0.0,0,0,0,0);
	
	gbadd(timePanel,minuteField,2,0,1,1,
	      GridBagConstraints.NONE,
	      GridBagConstraints.CENTER,
	      0.5,0.0,0,0,0,0);
	
	if(min_resolution <= 1) {
	  // Third Blob
	  gbadd(timePanel,colon2,3,0,1,1,
		GridBagConstraints.NONE,
		GridBagConstraints.CENTER,
		0.1,0.0,0,0,0,0);
	  
	  gbadd(timePanel,secondField,4,0,1,1,
		GridBagConstraints.NONE,
		GridBagConstraints.CENTER,
		0.5,0.0,0,0,0,0);
	} else {
	  gbadd(timePanel,col2with0,4,0,1,1,
		GridBagConstraints.NONE,
		GridBagConstraints.CENTER,
		0.5,0.0,0,0,0,0);
	}
      } else {
	gbadd(timePanel,col2with0,4,0,1,1,
	      GridBagConstraints.NONE,
	      GridBagConstraints.CENTER,
	      0.5,0.0,0,0,0,0);

	gbadd(timePanel,col1with0,2,0,1,1,
	      GridBagConstraints.NONE,
	      GridBagConstraints.CENTER,
	      0.5,0.0,0,0,0,0);
      }
      // Adding the timePanel to the main panel should happen
      // only if you add the hours or lower
      gbadd(this,timePanel,2,0,2,1,
	    GridBagConstraints.REMAINDER,
	    GridBagConstraints.SOUTH,
	    0.1,0.0,0,5,0,5);
    }
    gbadd(this,datePanel,0,0,2,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.SOUTH,
	  0.1,0.0,0,5,0,5);

    this.setEditable(editable); 
  }


  public AbsoluteTime (boolean editable, int min_resolution, String l1, String l2)
  {
    st = new Cf(NFRInfo.document_base,"etc/javastrings/AbsoluteTime.str");
    st.Read();

    this.setLayout(new GridBagLayout());
    
    resolution = min_resolution;
    date = new Date(80,0,1,0,0,0);
    
    Panel datePanel = new Panel();
    datePanel.setLayout(new GridBagLayout());
    Panel timePanel = new Panel();
    timePanel.setLayout(new GridBagLayout());
    Panel cbPanel = new Panel();
    cbPanel.setLayout(new GridBagLayout());

    // Checkbox Stuffs
    cbg = new CheckboxGroup();
    cb1 = new Checkbox(l1,cbg,false);
    cb2 = new Checkbox(l2,cbg,false);

    gbadd(cbPanel,cb1,0,0,1,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.WEST,
	  0.1,0.0,0,0,0,0);

    gbadd(cbPanel,cb2,0,1,1,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.WEST,
	  0.1,0.0,0,0,0,0);
    
    // Date Components
    dayField = new TextField("1",2);
    Label comma = new Label(",");
    yearField = new TextField("1980",4);
    monthChoice = new Choice();
    
    monthChoice.addItem(st.textorname("Jan"));
    monthChoice.addItem(st.textorname("Feb"));
    monthChoice.addItem(st.textorname("Mar"));
    monthChoice.addItem(st.textorname("Apr"));
    monthChoice.addItem(st.textorname("May"));
    monthChoice.addItem(st.textorname("Jun"));
    monthChoice.addItem(st.textorname("Jul"));
    monthChoice.addItem(st.textorname("Aug"));
    monthChoice.addItem(st.textorname("Sep"));
    monthChoice.addItem(st.textorname("Oct"));
    monthChoice.addItem(st.textorname("Nov"));
    monthChoice.addItem(st.textorname("Dec"));
    
    gbadd(datePanel,monthChoice,0,0,1,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.CENTER,
	  0.1,0.0,0,0,0,0);
    
    if(min_resolution <= 86400) {
      
      gbadd(datePanel,dayField,1,0,1,1,
	    GridBagConstraints.NONE,
	    GridBagConstraints.CENTER,
	    0.1,0.0,0,0,0,0);
      
      gbadd(datePanel,comma,2,0,1,1,
	    GridBagConstraints.NONE,
	    GridBagConstraints.CENTER,
	    0.1,0.0,0,0,0,0);
    }
    
    gbadd(datePanel,yearField,3,0,1,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.CENTER,
	  0.1,0.0,0,0,0,0);
    
    
    // Time Components
    hourField = new TextField("12",2);
    minuteField = new TextField("00",2);
    secondField = new TextField("00",2);
    
    Label colon = new Label(":");
    Label colon2 = new Label(":");
    Label zeros = new Label("00");
    Label col1with0 = new Label(": 00");
    Label col2with0 = new Label(": 00");
    
    ampmChoice = new Choice();
    ampmChoice.addItem("AM");
    ampmChoice.addItem("PM");
    
    if(min_resolution <= 3600) {
      
      // First Blob
      gbadd(timePanel,hourField,0,0,1,1,
	    GridBagConstraints.NONE,
	    GridBagConstraints.CENTER,
	    0.5,0.0,0,0,0,0);
      
      gbadd(timePanel,ampmChoice,5,0,1,1,
	    GridBagConstraints.REMAINDER,
	    GridBagConstraints.CENTER,
	    0.5,0.0,0,0,0,0);
      
      if(min_resolution <= 60) {
	// Second Blob
	gbadd(timePanel,colon,1,0,1,1,
	      GridBagConstraints.NONE,
	      GridBagConstraints.EAST,
	      0.1,0.0,0,0,0,0);
	
	gbadd(timePanel,minuteField,2,0,1,1,
	      GridBagConstraints.NONE,
	      GridBagConstraints.CENTER,
	      0.5,0.0,0,0,0,0);
	
	if(min_resolution <= 1) {
	  // Third Blob
	  gbadd(timePanel,colon2,3,0,1,1,
		GridBagConstraints.NONE,
		GridBagConstraints.CENTER,
		0.1,0.0,0,0,0,0);
	  
	  gbadd(timePanel,secondField,4,0,1,1,
		GridBagConstraints.NONE,
		GridBagConstraints.CENTER,
		0.5,0.0,0,0,0,0);
	} else {
	  gbadd(timePanel,col2with0,4,0,1,1,
		GridBagConstraints.NONE,
		GridBagConstraints.CENTER,
		0.5,0.0,0,0,0,0);
	}
      } else {
	gbadd(timePanel,col2with0,4,0,1,1,
	      GridBagConstraints.NONE,
	      GridBagConstraints.CENTER,
	      0.5,0.0,0,0,0,0);
	
	gbadd(timePanel,col1with0,2,0,1,1,
	      GridBagConstraints.NONE,
	      GridBagConstraints.CENTER,
	      0.5,0.0,0,0,0,0);
      }
      // Adding the timePanel to the main panel should happen
      // only if you add the hours or lower
      gbadd(this,timePanel,3,0,2,1,
	    GridBagConstraints.REMAINDER,
	    GridBagConstraints.EAST,
	    0.1,0.0,0,5,0,5);
    }
    gbadd(this,datePanel,1,0,2,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.CENTER,
	  0.1,0.0,0,5,0,5);

    gbadd(this,cbPanel,0,0,1,1,
	  GridBagConstraints.NONE,
	  GridBagConstraints.WEST,
	  0.1,0.0,0,5,0,5);
    
    this.setEditable(editable);  
  }


  
  // Bug: I have no idea what this code will do when I sink it into a larger
  // applet.  Watch it carefully.
  // Bug: I'm catching keys based on their ASCII value.  How will this break
  // when tossed into an international environment?
  // Bug: This messes up the cursor.  It stops blinking and seems to alternate
  // on and off as you press keys.
  // You should look to see if some other event that you don't
  // know about is coming here.  Maybe there is an event that makes
  // the cursor flash or something...
  public boolean keyDown(Event e, int key) {
    // Digit
    if((key >= 48) && (key <=  57)) {
      return(false);
    }
    // Tab
    if(key == 9) {
      return(false);
    }
    // Backspace
    if(key == 8) {
      return(false);
    }
    // Arrow Keys
    // bug: These should be symbolic references to the key names in the
    // Event object.  i.e. Event.DOWN, Event.UP, Event.LEFT, Event.RIGHT
    // bug: left out Event.PGDN, Event.PGUP, Event.HOME, Event.END
    if((key == 1004) || (key == 1005) || (key == 1006) || (key == 1007)) {
      return(false);
    }
    // Else Hijack it!
    return(true);
  }

  public boolean action(Event e, Object arg) {
    if(e.target == cb1) {
      this.setEditable(false);
      return(true);
    }
    if(e.target == cb2) {
      this.setEditable(true);
      return(true);
    }
    return(false);
  }
  
  public void displayDate() {
    // Bug: When this function is executed the Month Choice disappears
    // Moving a window over it and then revealing it seems to make it appear
    // So something is not getting painted correctly.  I may simply have to
    // call monthChoice.show(); to get it to force a repaint.
    //
    // Ok, it wasn't as easy as calling monthChoice.show();
    //
    // Well the combination hide/show call does the trick.  The choice 
    // box disappears for a split second but then re-appears with out the
    // user interfearing.  Not perfect but better.
    try {
      monthChoice.select(date.getMonth());
        monthChoice.hide();
        monthChoice.show();
	//monthChoice.repaint();
    }
    catch(Exception e) {
      // Bug: Should probably do something more intelligent with this error
      System.out.print("Caught Exception in");
      System.out.println(" AbsoluteTime.displayDate()");
      System.out.println("Exception: "+ e);
    }
    dayField.setText(Integer.toString(date.getDate()));
    yearField.setText(Integer.toString(1900 + date.getYear()));

    int hour = date.getHours();
    if(hour > 11) {
      hour -= 12;
      ampmChoice.select(1);  // PM
    } else {
      ampmChoice.select(0);  // AM
    }
    ampmChoice.hide();
    ampmChoice.show();
    if(hour == 0) { hour = 12; }
    hourField.setText(Integer.toString(hour));

    minuteField.setText(Integer.toString(date.getMinutes()));
    secondField.setText(Integer.toString(date.getSeconds()));
  }

  public boolean usedGui() {
    if(cb2 != null) {
      return(cb2.getState());
    } else {
      return(true);
    }
  }

  public boolean compileDate() {
    return this.compileDate(false);
  }
  
  public boolean compileDate(boolean doit_anyway) {
    String tmp;
    boolean error = false;
    int year, day, hour, minute, second;
    
    // Parse Year
    tmp = yearField.getText();
    if(tmp.equals("")) {
      year = 1980; 
    } else {
      try { year = Integer.parseInt(tmp); }
      catch(Exception e) {
	System.out.println("Caught Exception in AbsoluteTime.compileDate()");
	System.out.println("  Exception: " + e);
	error = true;
	year = 1980;
      }
      if(year < 100) { year += 1900; }
    }
    if((year < 1970) || (year > 2037)) {
      // bug:? what to do about this?  The 2038 problem does not apply
      // to Java code (which has 64 bits of milliseconds in a time),
      // but it *does* apply to the time_t on our host system.
      System.out.println("Year out of range.");
      error=true;
	/* first off, don't bounce the year back to 1970... */
    }
    
    // Parse Month
    int month = monthChoice.getSelectedIndex();
	monthChoice.repaint();
    
    // Parse Day
    tmp = dayField.getText();
    if(tmp.equals("")) {
      day = 1;
    } else {
      try { day = Integer.parseInt(tmp); }
      catch(NumberFormatException e) {
	error = true;
	day = 1;
      }
    }
    switch(month) {
    case 3:
    case 5:
    case 8:
    case 10:
      // Apr, Jun, Sep, Nov
      if((day < 1) || (day > 30)) {
	System.out.println("Date out of range. (30)");
	error=true;
	day = 1;
      }
      break;
    case 0:
    case 2:
    case 4:
    case 6:
    case 7:
    case 9:
    case 11:
      // Jan, Mar, May, Jul, Aug, Oct, Dec
      if((day < 1) || (day > 31)) {
	System.out.println("Date out of range. (31)");
	error=true;
	day = 1;
      }
      break;
    case 1:
      // Feb
      if((year % 4) == 0) {
	if((day < 1) || (day > 29)) {
	  System.out.println("Date out of range. (29)");
	  error=true;
	  day = 1;
	}
      } else {
	if((day < 1) || (day > 28)) {
	  System.out.println("Date out of range. (28)");
	  error=true;
	  day = 1;
	}
      }
      break;
    default:
      System.out.println("Should never get here.");
      error=true;
      day = 1;
    }
    
    // Parse Hour
    tmp = hourField.getText();
    if(tmp.equals("")) {
      hour = 12;
    } else {
      try { hour = Integer.parseInt(tmp); }
      catch(NumberFormatException e) {
	error = true;
	hour = 12;
      }
    }
    if((ampmChoice.getSelectedItem()).equals("PM") )
      {
	if ((hour >= 1) && (hour <= 11))
	  hour = hour + 12;
	else if (hour == 12)
	  hour = hour;  // I'm calling "12:00 pm" "noon"
	else
	  {
	    System.out.println("Hour out of range.");
	    error=true;
	  }
      }
    else
      {
	if ((hour >= 1) && (hour <= 11))
	  hour = hour;
	else if (hour == 12)
	  hour = 0;	// I'm calling "12:00 am" "midnight"
	else
	  {
	    System.out.println("Hour out of range.");
	    error=true;
	  }
      }
    
    // Parse Minute
    tmp = minuteField.getText();
    if(tmp.equals("")) {
      minute = 0;
    } else {
      try { minute = Integer.parseInt(tmp); }
      catch(NumberFormatException e) {
	error = true;
	minute = 0;
      }
    }
    if((minute < 0) || (minute > 59)) {
      System.out.println("Minute out of range.");
      error=true;
      minute = 0;
    }
    
    // Parse Second
    tmp = secondField.getText();
    if(tmp.equals("")) {
      second = 0; 
    } else {
      try { second = Integer.parseInt(tmp); }
      catch(NumberFormatException e) {
	error = true;
	second = 0;
      }
    }
    if((second < 0) || (second > 59)) {
      System.out.println("Second out of range.");
      error=true;
      second = 0;
    }
    
    if ( !error || doit_anyway )
      {
	try {
	  date.setYear(year - 1900);
	  date.setMonth(month);
	  date.setDate(day);
	  date.setHours(hour);
	  date.setMinutes(minute);
	  date.setSeconds(second);
	  date.setTime(date.getTime());	// normalize the date
	}
	catch(Exception e) {
	  // Bug: Should probably do something more intelligent with this error
	  System.out.print("Caught Exception in");
	  System.out.println(" AbsoluteTime.compileDate()");
	  System.out.println("  Exception: " + e);
          date.setTime(0);
	  return(error = true);
	}
	displayDate();
      }
    
    return(error);
  }
  
  public void setEditable(boolean state) {
    if(state) {
      yearField.enable();
      monthChoice.enable();
      dayField.enable();
      hourField.enable();
      minuteField.enable();
      secondField.enable(); 
      ampmChoice.enable();

      if(cb1 != null) {
	cb1.setState(false);
	cb2.setState(true);
      }
      
    } else {
      yearField.disable();
      monthChoice.disable();
      dayField.disable();
      hourField.disable();
      minuteField.disable();
      secondField.disable();
      ampmChoice.disable();
      
      if(cb1 != null) {
	cb1.setState(true);
	cb2.setState(false);
      }
    }
  }

  public void set_value(long time) 
  {
    try {
      date.setTime(time * 1000);
      this.displayDate();
    } catch(Exception e) {
      // Bug: Should probably do something more intelligent with this error
      System.out.print("Caught Exception in");
      System.out.println(" AbsoluteTime.set_value(long)");
      System.out.println("  Exception: " + e);
      System.out.println("  setting time to 0");
      date.setTime(0);
      //return(-1);
    }
    //return(0);
  }
  
  public void set_value(String time)
  {
    try
      {
	int x = Integer.parseInt(time);
	this.set_value(x);
	this.displayDate();
      }
    catch (Exception e)
      {
	System.out.print("Caught Exception in");
	System.out.println(" AbsoluteTime.set_value(String)");
	System.out.println("  Exception: " + e);
	System.out.println("  setting time to 0");
	this.set_value(0);
	//return(-1);
      }
    //return(0);
  }
  
  public long get_value() 
  {
    // Changed this to return seconds not miliseconds since the epoch
    try {
      this.compileDate();
      return(date.getTime() / 1000);
    } catch(Exception e) {
      // Bug: Should probably do something more intelligent with this error
      System.out.print("Caught Exception in");
      System.out.println(" AbsoluteTime.get_value()");
      System.out.println(" Exception: " + e);
      return(-1);
    }
  }

  // Wraper function for filling out the GridBagConstraints object
  private void gbadd(Container container, Component component, 
		     int grid_x, int grid_y, int grid_width, int grid_height,
		     int fill, int anchor, double weight_x, double weight_y,
		     int top, int left, int bottom, int right)
  {
    GridBagConstraints c = new GridBagConstraints();
    c.gridx = grid_x; c.gridy = grid_y;
    c.gridwidth = grid_width; c.gridheight = grid_height;
    c.fill = fill; c.anchor = anchor;
    c.weightx = weight_x; c.weighty = weight_y;
    if (top+bottom+left+right > 0)
      c.insets = new Insets(top, left, bottom, right);
    
    ((GridBagLayout)container.getLayout()).setConstraints(component, c);
    container.add(component);
  }
  
  public String toString() {
    this.compileDate();
    return(date.toString());
  }
  
  public static void main(String[] args) {
    System.out.println("AbsoluteTime.class");
  }
}

