/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   Cf.java
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

package nfr.misc;

import java.util.*;
import java.net.*;
import java.io.*;

public class Cf extends Properties
{
  String mProto;
  String mURLEPath;
  String mPath;
  String mHostName;
  int mPort;
  URL mURL;

// Constructors

public Cf(String HostName, String Path) 
  {
    SetProto(new String("http"));
    SetPort(-1);
    SetHostName(HostName);
    SetPath(Path);
  }

public Cf(String HostName) 
  {
    SetProto(new String("http"));
    SetPort(-1);
    SetHostName(HostName);
  }

public Cf(URL BaseURL, String Path)
  {
    SetProto(BaseURL.getProtocol());
    SetPort(BaseURL.getPort());
    SetHostName(BaseURL.getHost());    
    SetPath(Path);
  }

public Cf(URL BaseURL)
  {
    SetProto(BaseURL.getProtocol());
    SetPort(BaseURL.getPort());
    SetHostName(BaseURL.getHost());    
  }


// Accessors & Mutators
  // declared as "final" so that compiler may inline them
public final void SetHostName(String HostName) { mHostName = HostName; }

public final String GetHostName() { return mHostName; }

public final void SetPort(int Port) { mPort = Port; }

public final int GetPort() { return mPort; }

public final void SetPath(String Path)
  {
    mPath = Path;
    mURLEPath = URLEncoder.encode(Path);
  }

public final String GetPath() { return mPath; }

public final void SetProto(String Proto) { mProto = Proto; }

public final String GetProto(String Proto) { return mProto; }


// Read retrieves mName from mHostName using mProto, and then parses
// it, adding all name=value pairs to a hash table.
public boolean Read(String Path) 
  {
    SetPath(Path);
    return Read();
  }

public boolean Read()
  {
    try
      {
	URL CfUrl = new URL(mProto, mHostName, mPort, "/cgi-bin/cfstrip");
	URLConnection CfConn = CfUrl.openConnection();
	CfConn.setDoOutput(true);
		
	DataOutputStream Out = new DataOutputStream(CfConn.getOutputStream());
	Out.writeBytes("cf_name=" + mURLEPath + "\r\n\r\n");
	
	DataInputStream In = new DataInputStream(CfConn.getInputStream());
	
	// we only have to handle strings of the exact form: 
	//    <name>=<value>\n
	// cfstrip ensures that there is no extraneous whitespace,
	// any other form of input can be considered an error, and 
	// for now, is ignored.
	String Line = In.readLine();
	while(Line != null)
	  {
	    int Idx = Line.indexOf('=');
	    if(Idx > 0)
	      {
		String Name = Line.substring(0,Idx);
		String Val = Line.substring(Idx+1, Line.length());
		put(Name, Val);
	      }
	    Line = In.readLine();
	  }
	In.close();
	Out.close();
      }

    catch(MalformedURLException e)
      {
	// bug: add something of substance here or throw it 
	System.out.println(e);
	MsgBox.show("Error", String.valueOf(e));
	return false;
      }

    catch(IOException e)
      {
	// bug: add something of substance here or throw it
	System.out.println(e);
	MsgBox.show("Error", String.valueOf(e));
	return false;
      }

    return true;
  }


// Merge() 
public boolean Merge()
  {
    Enumeration Keys = keys();
    StringBuffer Buf = new StringBuffer();
    Buf.append("cf_name=" + mURLEPath);
    while(Keys.hasMoreElements())
      {
	String Key = (String) Keys.nextElement();
	Buf.append("&" + URLEncoder.encode(Key) + "=" + 
		   URLEncoder.encode((String)get(Key)) );
      }
    try
      {
	URL CfUrl = new URL(mProto, mHostName, mPort, "/cgi-bin/cfmerge");
	URLConnection CfConn = CfUrl.openConnection();
	CfConn.setDoOutput(true);
	DataOutputStream Out = new DataOutputStream(CfConn.getOutputStream());
	Out.writeBytes(Buf + "\r\n\r\n");
	
	InputStream In = CfConn.getInputStream();

	In.close();
	Out.close();
      }
    catch(MalformedURLException e)
      {
        // bug: add something of substance here or throw this
	System.out.println(e);
	return false;
      }
    catch(IOException e)
      {
        // bug: add something of substance here or throw this
	System.out.println(e);
	return false;
      }
    return true;    
  }

public String textorname(String name) {
	String s = (String) get(name);
	if (s == null)
		return name;
	else
		return s;
	}

}


