/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   CgiScript.java
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

// this is a class used to interface with cgi programs that use
// Cgi_Response() to communicate with the client.

package nfr.misc;

import java.awt.*;
import java.util.*;
import java.net.*;
import java.io.*;

public class CgiScript 
{
public static final int GET = 0;
public static final int POST = 1;

  int mType = POST;

  String mScriptName = null;
  String mData = null;
  String mPathInfo = null;

  int mBytesSent = 0;
  int mBytesRecvd = 0;

  int mRemResult = 0;
  String mRemMsg = null;

  int mLocResult = 0;
  String mLocMsg = null;

  Exception mLastException = null;

  DataOutputStream mOut = null;
  DataInputStream mIn = null;
  URL mURL = null;
  URLConnection mConn = null;

public CgiScript()
  {
    mScriptName = "";
    mData = "";
    mPathInfo = "";
  }

public CgiScript(String scriptName)
  {
    setName(scriptName);
    mData = "";
    mPathInfo = "";
  }

public void setType(int t)
  {
    mType = t;
  }

public void setName(String scriptName)
  {
    if(scriptName == null)
      mScriptName = "";
    else
      mScriptName = new String(scriptName);
  }

public void setData(String d)
  {
    if(d != null)
      mData = new String(d);
  }

public void clearData()
  { mData = new String(""); }

public void setPathInfo(String pi)
  {
    if(pi == null)
      mPathInfo = "";
    mPathInfo = new String("/" + pi);
  }

public String getRemMsg()
  { return new String(mRemMsg); }

public String getLocMsg()
  { return new String(mLocMsg); }

public String getRelevantMsg()
  {
    if(mLocResult != 0)
      return getLocMsg();
    return getRemMsg();
  }

public int getBytesSent()
  { return mBytesSent; }

public int getBytesRecvd()
  { return mBytesRecvd; }

public void addDataNVPair(String n, String v)
  {
    mData = new String(mData + "&" + 
		       URLEncoder.encode(n) + "=" + 
		       URLEncoder.encode(v));
  }

public int run()
  {
    int msgLen = 0;

    mLocMsg = new String("success");
    mLocResult = 0;
    if(sendData() != 0)
      return getResult();
    if(getInputStream() == null)
      return getResult();
    if(getResponse() != 0)
      return getResult();
    return close();
  }

public int sendData()
  {
    if(mType == POST)
      {
	if(getOutputStream() == null)
	  return getResult();
	try
	  { mOut.writeBytes(mData + "\r\n\r\n"); }
	catch(Exception e)
	  { onException(e); }
	return getResult();
      }
    else
      return 0;
  }

public DataOutputStream getOutputStream()
  {
    try
      {
	if(mOut == null)
	  {
	    mURL = new URL(NFRInfo.document_base, mScriptName + mPathInfo);
	    System.out.println("POSTing: " + mScriptName + mPathInfo);
	    mConn = mURL.openConnection();
	    mConn.setDoOutput(true);
	    mConn.setDoInput(true);
	    mOut = new DataOutputStream(mConn.getOutputStream());
	  }
	return mOut;
      }
    catch(Exception e)
      { onException(e); }
    return null;
  }

public DataInputStream getInputStream()
  {
    try
      {
	if(mIn == null)
	  {
	    if(mType == GET)
	      {
		mURL = new URL(NFRInfo.document_base, 
			       mScriptName + mPathInfo + 
			       "?" + mData);
		System.out.println("GETting: " + 
				   mScriptName + mPathInfo +
				   "?" + mData);
		mConn = mURL.openConnection();		
		mConn.setDoOutput(false);
		mConn.setDoInput(true);
	      }
	    mIn = new DataInputStream(mConn.getInputStream());
	  }
	return mIn;
      }
    catch(Exception e)
      { onException(e); }
    return null;
  }

public int getResponse()
  {
    if(getInputStream() == null)
      return getResult();
    try
      {
	mRemResult = mIn.readInt();
	System.out.println("result from script: " + mRemResult);
	int msgLen = mIn.readInt();
	System.out.println("msg length: " + msgLen);
	if(msgLen > 0)
	  {
	    char sbuf[] = new char[msgLen];
	    int i = 0;
	    while(i < msgLen)
	      sbuf[i++] = (char) mIn.readByte();
	    mRemMsg = new String(sbuf);
	  }
	else
	  mRemMsg = "<no response from server>";
	System.out.println("msg from server: " + mRemMsg);
      }
    catch(Exception e)
      { onException(e); }
    return getResult();
  }

public int close()
  {
    try
      {
	if(mIn != null)
	  {
	    mIn.close();
	    mIn = null;
	  }
	if(mOut != null)
	  {
	    mOut.close();
	    mOut = null;
	  }
      }
    catch(Exception e)
      { onException(e); }
    return getResult();
  }

public int getResult()
  {
    if(mLocResult < 0)
      return mLocResult;
    return mRemResult;
  }

  void onException(Exception e)
  {
    e.printStackTrace();
    mLocMsg = new String(e.toString());
    mLocResult = -1;
    mLastException = e;
  }

}
