/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   ColoredBarMeter.java
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

package nfr.misc;

import java.awt.*;
import java.util.*;

class ColorEntry
{
  public Color color = null;
  public int value = 0;
  
public ColorEntry(int val, Color c)
  {
    color = c;
    value = val;
  }
}

public class ColoredBarMeter extends BarMeter
{
  Vector colorVec = null;

public ColoredBarMeter(boolean logarithmic, int max_mode) 
  {
    super(logarithmic, max_mode);
    colorVec = new Vector();
  }

public void set_colorAt(int v, Color c)
  {
    int val = v;
    if(logarithmic) // expand the scale to make it move smoothly
      val = (int) (java.lang.Math.log(v) * LOG_SCALING_FACTOR); 

    //first see if the Vector is empty
    if(colorVec.size() == 0)
      {
	colorVec.addElement(new ColorEntry(val, c));
	return;
      }

    // now check if it goes at the end
    if( val > ((ColorEntry)colorVec.lastElement()).value )
      {
	colorVec.addElement(new ColorEntry(val, c));
	return;
      }

    // now just go through the vector 'til we get to the right
    // place to insert it
    for(int i = 0; i < (colorVec.size()); i++)
      {
	if( val == ((ColorEntry)colorVec.elementAt(i)).value )
	  {
	    colorVec.setElementAt(new ColorEntry(val, c), i);
	    return;
	  }
	if( val < ((ColorEntry)colorVec.elementAt(i)).value )
	  {
	    colorVec.insertElementAt(new ColorEntry(val, c), i);
	    return;
	  }
      }
  }

public void draw_contents(Graphics g)
  {
    Dimension d = size();

    if(current < prev)
      {
	int left = (current * d.width / max) + (border*2);
	int right = d.width - (border*2);
	g.setColor(bg_color);
	g.fillRect(left, border*2,
		   right - left,
		   d.height - (border*4));
      }

    else if(current > prev)
      {
	Enumeration colors = colorVec.elements();
	
	boolean done = false;
	int left = 0;
	int right = 0;
	g.setColor(fg_color);
	while(!done)
	  {
	    ColorEntry ce = null;
	    right = current * (d.width - border*4) / max;
	    if(colors.hasMoreElements())
	      {
		ce = (ColorEntry)colors.nextElement();
		if(right > ce.value * (d.width - border*4) / max)
		  right = ce.value * (d.width - border*4) / max;
		else
		  done = true;
	      }
	    else
	      done = true;
	    
	    g.fillRect(border*2 + left, border*2, 
		       right - left, 
		       d.height - (border*4));
	    left = right;
	    if(!done)
	      g.setColor(ce.color);
	  }
      }
  }
  
}
