/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   DateThing.java
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

package nfr.misc;

import java.awt.*;
import java.util.*;

public class DateThing extends Panel
{
  TextField mMonth = null;
  TextField mDay = null;
  TextField mYear = null;
  Date mDate = null;
  static String[] mMonths = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", 
			      "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

public DateThing()
  {
    mDate = new Date();
    mMonth = new TextField(3);
    mDay = new TextField(2);
    mYear = new TextField(4);
    add(mMonth);
    add(mDay);
    add(mYear);
    /***
    mMonths = new String[12];
    mMonths[0] = new String("Jan");
    mMonths[1] = new String("Feb");
    mMonths[2] = new String("Mar");
    mMonths[3] = new String("Apr");
    mMonths[4] = new String("May");
    mMonths[5] = new String("Jun");
    mMonths[6] = new String("Jul");
    mMonths[7] = new String("Aug");
    mMonths[8] = new String("Sep");
    mMonths[9] = new String("Oct");
    mMonths[10] = new String("Nov");
    mMonths[11] = new String("Dec");
    ***/
    display();
  }

public void display()
  {
    mMonth.setText(mMonths[mDate.getMonth()]);
    mDay.setText(String.valueOf(mDate.getDate()));
    mYear.setText(String.valueOf(mDate.getYear() + 1900));    
  }

public static String getMonthString(int m)
  {
    return new String(mMonths[m]);
  }

public void disable()
  { enable(false); }

public void enable()
  { enable(true); }

public void enable(boolean b)
  {
    mMonth.enable(b);
    mDay.enable(b);
    mYear.enable(b);    
  }

public void incDay()
  {
    getTime();
    mDate.setDate(mDate.getDate() + 1);
    display();
    getTime();
    display(); // do twice to get rid of stuff like Dec 32
  }

public void incMonth()
  {
    getTime();
    int m = (mDate.getMonth() + 1) % 12;
    int y = (mDate.getMonth() + 1) / 12;
    mDate.setMonth(m);
    setYear(mDate.getYear() + y);
    display();
  }

public void incYear()
  {
    getTime();
    setYear(mDate.getYear() + 1);
    display();
  }

public void decDay()
  {
    getTime();
    mDate.setDate(mDate.getDate() - 1);
    display();
    getTime();
    display(); // do twice to get rid of stuff like Jan 0
  }

public void decMonth()
  {
    getTime();
    if(mDate.getMonth() == 0)
      setYear(mDate.getYear() - 1);
    mDate.setMonth((mDate.getMonth() + 11) % 12);
    display();
  }

public void decYear()
  {
    getTime();
    setYear(mDate.getYear() - 1);
    display();
  }

public void setYear(int y)
  {
    // have to do this to keep from getting stuck in 2038 or 1969
    if(y > 137)
      mDate.setYear(137);
    else if(y < 70)
      mDate.setYear(70);
    else
      mDate.setYear(y);
  }

public long getTime()
  {
    try
      {
	long l = Date.parse(mDay.getText() + " " + 
			    mMonth.getText() + " " +
			    mYear.getText());
	mDate.setTime(l);
	return (l);
      }
    catch(IllegalArgumentException e)
      {
	return mDate.getTime();
      }
  }

public void setTime(long t)
  {
    mDate.setTime(t);
    display();
  }

public boolean keyDown(Event e, int key) 
  {
    System.out.println("key pressed: " + key);

    if((key == 43) || (key == 1004)) // '+' or UP_ARROW
      {
	if(e.target == mDay)
	  {
	    incDay();
	    return true;
	  }
	else if(e.target == mMonth)
	  {
	    incMonth();
	    return true;
	  }
	else if(e.target == mYear)
	  {
	    incYear();
	    return true;
	  }
      }

    else if((key == 45) || (key == 1005)) // '-' or DOWN_ARROW
      {
	if(e.target == mDay)
	  {
	    decDay();
	    return true;
	  }
	else if(e.target == mMonth)
	  {
	    decMonth();
	    return true;
	  }
	else if(e.target == mYear)
	  {
	    decYear();
	    return true;
	  }
      }

    else if(key == 10) // Enter
      {
	getTime();
	display();
	return true;
      }

    return false;
  }

}

