/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   FifoRdr.java
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

package nfr.misc;

import java.net.*;
import java.io.*;

import nfr.misc.*;

/*****************************************************************************
 FifoRdr is an abstract class which provides the capability to
 create and read from a fifo on the server.  Subclasses must override
 OnIdle() and OnData().  OnIdle() is called every time an "idle tick"
 is recieved from the server.  An idle tick is sent roughly every
 mIdleTime microseconds. OnData() is called whenever data arrives from
 the server.
*****************************************************************************/

public abstract class FifoRdr implements Runnable
{
  String mProto = null;
  String mHostName = null;
  int mPort = -1;
  String mFifoName = null;
  String mURLEFifoName = null;
  DataInputStream mIn = null;
  DataOutputStream mOut = null;
  int mIdleGrain = 1000;
  int mIdleTimeout = 10;	// there should be an exception thrown
				// if this many IdleGrains go by without an
				// idle tick or data.


// abstract methods
public abstract boolean OnIdle();

public abstract boolean OnData(DataInputStream In, int Type);

  //public abstract boolean OnErr(Exception);


// constructors
public FifoRdr(URL BaseURL, String FifoName)
  {
    setFifoName(FifoName);
    mProto = BaseURL.getProtocol();
    mPort = BaseURL.getPort();
    mHostName = BaseURL.getHost();    
  }

// accessors and mutators
  // silly ones declared "final" so the compiler can inline them

public final String getFifoName() { return mFifoName; }

public final void setFifoName(String FifoName)
  {
    mFifoName = FifoName;
    mURLEFifoName = URLEncoder.encode(FifoName);
  }

public final void setIdleGrain(int g) { mIdleGrain = g; }

public final void setIdleTimeout(int g) { mIdleTimeout = g; }

  
public void run() 
  {
    try
      {
	URL u = new URL(mProto, mHostName, mPort, "/cgi-bin/nph-fifordr");
	URLConnection Conn = u.openConnection();
	Conn.setDoOutput(true);
	
	mOut = new DataOutputStream(Conn.getOutputStream());
	mOut.writeBytes("fifo_name=" + mURLEFifoName + "\n");
	mIn = new DataInputStream(Conn.getInputStream());
	int Type = 0;
	while(Type > -1)
	  {
	    Type = mIn.readInt();
	    if (Type == 0)
	      {
		//resetIdleTimer();
		OnIdle();
	      }
	    else 
	      {
		//resetIdleTimer();
		OnData(mIn, Type);
	      }
	  }
	closeConnection();
      }
    catch(Exception e)
      {
	e.printStackTrace();
      }
  }
  

public void closeConnection() throws IOException
  {
    if(mIn != null)
      {
	mIn.close();
	mIn = null;
      }
    if(mOut != null)
      {
	mOut.close();
	mOut = null;
      }
  }
}
