//  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
//  All rights reserved.
//
//  Use and distribution of this software and its source code
//  are governed by the terms and conditions of the
//  Network Flight Recorder Software License ("LICENSE.TXT")
//
//  File:   eric.java
//  Author: Eric Wall <eric@nfr.net>
//


package nfr.misc;

import java.awt.*;

public class IntervalTime extends Panel {

  private long time = 0;
  private TextField dayField, hourField, minuteField, secondField;
  private Label colon, colon2;
  private Label zeros, col1with0, col2with0;
  
  public IntervalTime (boolean editable, int min, int max) {
    
    if((min < 1) || (max < 1) || (max < min)) {
      System.out.println("Invalid Minimum or Maximum Resolution values.");
      System.out.println("Using Min of 1 second and Max of 86400 seconds.");
      // Bug:  Do something here, set error condition exit something.
      // General question.  What do you do when new fails?
      min = 1;
      max = 86400;
    }
    

    this.setLayout(new GridBagLayout());

    dayField = new TextField("0",10);
    hourField = new TextField("00",2);
    colon = new Label(":");
    minuteField = new TextField("00",2);
    colon2 = new Label(":");
    secondField = new TextField("00",2);
    zeros = new Label("00");
    col1with0 = new Label(": 00");
    col2with0 = new Label(": 00");

    if(max >= 86400) {
      gbadd(this,dayField,0,0,1,1,
	    GridBagConstraints.NONE,
	    GridBagConstraints.CENTER,
	    0.1,0.0,0,0,0,10);
    }
     
    if((max >= 3600) && (min <= 3600)) {
      gbadd(this,hourField,1,0,1,1,
	    GridBagConstraints.NONE,
	    GridBagConstraints.CENTER,
	    0.1,0.0,0,0,0,0);
    } else {
      gbadd(this,zeros,1,0,1,1,
	    GridBagConstraints.NONE,
	    GridBagConstraints.CENTER,
	    0.1,0.0,0,0,0,0);
    }

    if((max >= 60) && (min <= 60)) {
      gbadd(this,colon,2,0,1,1,
	    GridBagConstraints.NONE,
	    GridBagConstraints.CENTER,
	    0.1,0.0,0,0,0,0);
      
      gbadd(this,minuteField,3,0,1,1,
	    GridBagConstraints.NONE,
	    GridBagConstraints.CENTER,
	    0.1,0.0,0,0,0,0);
    } else {
      gbadd(this,col1with0,3,0,1,1,
	    GridBagConstraints.NONE,
	    GridBagConstraints.CENTER,
	    0.1,0.0,0,0,0,0);
    }
    
    if(min <= 1) {
      gbadd(this,colon2,4,0,1,1,
	    GridBagConstraints.NONE,
	    GridBagConstraints.CENTER,
	    0.1,0.0,0,0,0,0);
      
      gbadd(this,secondField,5,0,1,1,
	    GridBagConstraints.NONE,
	    GridBagConstraints.CENTER,
	    0.1,0.0,0,0,0,0);
    } else {
      gbadd(this,col2with0,5,0,1,1,
	    GridBagConstraints.NONE,
	    GridBagConstraints.CENTER,
	    0.1,0.0,0,0,0,0);
    }
  }
  
  // Wraper function for filling out the GridBagConstraints object
  private void gbadd(Container container, Component component, 
                     int grid_x, int grid_y, int grid_width, int
		     grid_height,
                     int fill, int anchor, double weight_x, double
		     weight_y,
                     int top, int left, int bottom, int right)
  {
    GridBagConstraints c = new GridBagConstraints();
    c.gridx = grid_x; c.gridy = grid_y;
    c.gridwidth = grid_width; c.gridheight = grid_height;
    c.fill = fill; c.anchor = anchor;
    c.weightx = weight_x; c.weighty = weight_y;
    if (top+bottom+left+right > 0)
      c.insets = new Insets(top, left, bottom, right);
    
    ((GridBagLayout)container.getLayout()).setConstraints(component, c);
    container.add(component);
  }

  public void displayTime() {
    long q = 0;
    long r = 0;
    

    q = time / 86400;
    r = time % 86400; 
    dayField.setText(Long.toString(q));

    q = r / 3600;
    r = r % 3600;
    hourField.setText(Long.toString(q));

    q = r / 60;
    r = r % 60;
    minuteField.setText(Long.toString(q));
    secondField.setText(Long.toString(r));
  }

  public boolean compileTime() {
    String tmp;
    boolean error = false;
    long day, hour, minute, second;
    this.time = 0;
    
    // Parse Day
    tmp = dayField.getText();
    if(tmp.equals("")) {
      day = 0;
    } else {
      try { day = Long.parseLong(tmp); }
      catch(NumberFormatException e) {
	System.out.println("Catch Day");
	error = true;
	day = 0;
      }
    }
    if(day < 0) {
      System.out.println("Negative day argument.");
      error = true;
      day = 0;
    }
    this.time = day * 86400;
    System.out.println("Time: " + this.time);
    
    // parse Hour
    tmp = hourField.getText();
    if(tmp.equals("")) {
      hour = 0;
    } else {
      try { hour = Long.parseLong(tmp); }
      catch(NumberFormatException e) {
	System.out.println("Catch Hour");
	error = true;
	hour = 0;
      }
    }
    if(hour < 0) {
      System.out.println("Negative hour argument.");
      error = true;
      hour = 0;
    }
    this.time = this.time + (hour * 3600);
    System.out.println("Time: " + this.time);
    
    // parse Minute
    tmp = minuteField.getText();
    if(tmp.equals("")) {
      minute = 0;
    } else {
      try { minute = Long.parseLong(tmp); }
      catch(NumberFormatException e) {
	System.out.println("Catch Minute");
	error = true;
	minute = 0;
      }
    }
    if(minute < 0) {
      System.out.println("Negative minute argument.");
      error = true;
      minute = 0;
    }
    this.time = this.time + (minute * 60);
    System.out.println("Time: " + this.time);
    
    // parse Second
    tmp = secondField.getText();
    if(tmp.equals("")) {
      second = 0;
    } else {
      try { second = Long.parseLong(tmp); }
      catch(NumberFormatException e) {
	System.out.println("Catch Second");
	error = true;
	second = 0;
      }
    }
    if(second < 0) {
      System.out.println("Negative second argument.");
      error = true;
      second = 0;
    }
    this.time = this.time + second;
    System.out.println("Time: " + this.time);
    
    return(error);
  }

  public void set_value(long in_time) {
    time = in_time;
  }
  
  public void set_value(String in_time) {
    try {
      set_value(Integer.parseInt(in_time));
    } catch (NumberFormatException e) {
      set_value(0);
    }
  }
  
  public long get_value() {
    return(time);
  }
  
  public String toString() {
    return("IntervalTime.class");
  }
  
  // Bug: I have now idea what this code will do when I sink it into a larger
  // applet.  Watch it carefully.
  // Bug: I'm catching keys based on their ASCII value.  How will this break
  // when tossed into an international environment?
  // Bug: This messes up the cursor.  It stops blinking and seems to alternate
  // on and off as you press keys.
  public boolean keyDown(Event e, int key) {
    // Digit
    System.out.println("Key Hit: " + key);
    if((key >= 48) && (key <=  57)) {
      return(false);
    }
    // Tab
    if(key == 9) {
      return(false);
    }
    // Backspace
    if(key == 8) {
      return(false);
    }
    // Arrow Keys
    if((key == 1004) || (key == 1005) || (key == 1006) || (key == 1007)) {
      return(false);
    }
    // Else Hijack it!
    return(true);
  }

  public static void main(String[] args) {
    System.out.println("IntervalTime.class");
  }
}

