/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   ListView.java
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

// this class was written for use with the SortView class.  it probably
// isn't very useful by itself.

package nfr.misc;

import java.awt.*;
import java.util.*;

public class ListView extends Panel
{
  int mLastTouchedItem = 0;
  Vector mMembers = null;
  List mListbox = null;

public ListView()
  {
    mMembers = new Vector();
    mListbox = new List();
    mListbox.setMultipleSelections(true);
    repTile l = new repTile();
    setLayout(l);
    add(mListbox);
    l.setConstraints(0,0,0,0,"",mListbox);
    l.setStretchy(0,0);
  }

public int count()
  {
    return mMembers.size();
  }

public void setMinSize(int w, int h)
  {
    ((repTile)getLayout()).setMinSize(w,h,mListbox);
  }

public void clear()
  {
    mMembers = new Vector();
    if(mListbox.countItems() > 0)
      mListbox.clear();
  }

public void insert(Object newThing)
  {
    insert(newThing, String.valueOf(newThing));
  }

public void insert(Object newThing, String newThingString)
  {
    if(!filter(newThing))
      return;

    // numbskull linear insertion sort - fix later if necessary

    //first see if the Vector is empty
    if(mMembers.size() == 0)
      {
	mMembers.addElement(newThing);
	mListbox.addItem(newThingString);
	return;
      }

    // now check if it goes at the end
    int result = compare(newThing, mMembers.lastElement());
    if(result >= 0)
      {
	mMembers.addElement(newThing);
	mListbox.addItem(newThingString);
	return;
      }

    // now just go through the vector 'til we get to the right
    // place to insert it
    for(int i = 0; i < (mMembers.size()); i++)
      {
	result = compare(newThing, mMembers.elementAt(i));
	if(result <= 0)
	  {
	    mMembers.insertElementAt(newThing, i);
	    mListbox.addItem(newThingString, i);
	    return;
	  }
      }
  }

public int compare(Object first, Object second)
  {
    // new insertions are added to the end of mMembers, unless this is
    // overridden
    return 1;
  }

public void remove(int i) { mListbox.delItem(i); }

public void remove(Object o)
  {
    for(int i = 0; i < (mMembers.size()); i++)
      {
	if(o == mMembers.elementAt(i))
	  {
	    System.out.println("removing item #" + i);
	    mListbox.delItem(i);
	    return;
	  }
      }    
  }

public boolean filter(Object thing)
  {
    // new insertions are always succeed, unless this is overridden
    return true;
  }


public int getSelectedIndex() { return mListbox.getSelectedIndex(); }

public String getSelectedItem() { return mListbox.getSelectedItem(); }

public Object getSelectedObject()
  {
    return mMembers.elementAt(mListbox.getSelectedIndex());
  }

public int[] getSelectedIndexes() { return mListbox.getSelectedIndexes(); }

public String[] getSelectedItems() { return mListbox.getSelectedItems(); }

public Object[] getSelectedObjects()
  {
    int[] selectionIdxs = mListbox.getSelectedIndexes();
    Object[] selections = new Object[selectionIdxs.length];
    for(int i = 0; i < selectionIdxs.length; i++)
      {
	selections[i] = mMembers.elementAt(selectionIdxs[i]);
      }
    return selections;
  }

public void select(int i) { mListbox.select(i); }

public void select(Object o)
  {
    for(int i = 0; i < (mMembers.size()); i++)
      {
	if(o == mMembers.elementAt(i))
	  {
	    mListbox.select(i);
	    //mListbox.makeVisible(i);

	    // bug: netscape on bsdi won't reset the position of the
	    // List scrollbar to show the selected item, even with the
	    // above call to makeVisible() :(
	    return;
	  }
      }    
  }

public boolean handleEvent(Event e)
  {
    // System.out.println("handleEvent() EVENT: " + e);

    // the following is some pretty gross muckery, we set the target
    // to reference this ListView object so that the parent (a
    // SortView) can call getSelectedObjects on the reference in
    // order to find out what is actually selected, when it gets
    // the Event since we pass it on up from here after this
    // tampering
    if(e.id == Event.LIST_SELECT)
      {
	System.out.println("handleEvent - LIST_SELECT: resetting event target to a ListView");
	mLastTouchedItem = ((Integer)e.arg).intValue();
	e.target = this;
      }
    else if(e.id == Event.LIST_DESELECT)
      {
	System.out.println("handleEvent - LIST_DESELECT: resetting event target to a ListView");
	mLastTouchedItem = ((Integer)e.arg).intValue();
	e.target = this;
      }
    else if(e.id == Event.ACTION_EVENT)
      {
	System.out.println("handleEvent - ACTION_EVENT: resetting event target to a ListView");
	// need to select the last touched item, since for some reason this action deselects it
	mListbox.select(mLastTouchedItem);
	e.target = this;
      }
    else
      return super.handleEvent(e);
    return false;
  }

}
