/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   Meter.java
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

package nfr.misc;

import java.awt.*;


public abstract class Meter extends Canvas implements ProgressThing
{
public final static int BORDER_SUNKEN = -1;
public final static int BORDER_FLAT = 0;
public final static int BORDER_RAISED = 1;
public final static int LOG_SCALING_FACTOR = 1000;
public final static int FIXED_MAX = 0;
public final static int INFINITE_MAX = 3;
public final static int ADAPTIVE_MAX = 2;
  int adaptiveScalingFactor = 20;
  int max = 1; 
  int initialMax = 1; 
  int prev = 0;
  int current = 0;
  int currentPercent = 0;
  int max_mode = 0;
  boolean logarithmic = false;
  Dimension psize = null;
  int border = 2;  
  Color fg_color = null;
  Color bg_color = null;
  Color border_color = null;
  int border_style = BORDER_SUNKEN;

  int danceShoeSize = 5;
  int danceCard = 0;
  int danceLength = 100;
  boolean dancing = false;
  Thread danceThread = null;
  
public Meter(boolean logarithmic, int max_mode) 
  {
    super();
    this.max_mode = max_mode;
    this.logarithmic = logarithmic;
    set_max(100);
    fg_color = Color.black;
    bg_color = Color.lightGray;
    border_color = Color.lightGray;
  }

// MSBS mode - aka logarithmic
public final void setMSBS(boolean b) { logarithmic = b; }

// fancy shmancy stuff
public final void set_foreground(Color c) { fg_color = c; }
public final void set_background(Color c) { bg_color = c; }
public final void set_borderColor(Color c) { border_color = c; }
public final void set_border(int i) { border = i; }
public final void set_borderStyle(int i) { border_style = i; }
public final void set_size(Dimension d) { psize = d; }


// implement the ProgressThing interface
public void set_max(int m) 
  {
    if(logarithmic) // expand the scale to make it move smoothly
      max = (int) (java.lang.Math.log(m) * LOG_SCALING_FACTOR); 
    else
      max = m;
    initialMax = max;
  }

public void set_value(int v) 
  {
    prev = current;

    if(logarithmic) // expand the scale to make it move smoothly
      current = (int) (java.lang.Math.log(v) * LOG_SCALING_FACTOR);
    else
      current = v;


    switch (max_mode)
      {
      case FIXED_MAX:		// fixed max
	if(current > max)
	  current = max;
	break;
      case INFINITE_MAX:	// only increasing
	if (current > max)
	  max = current;
	break;
      case ADAPTIVE_MAX:	// adaptive
	//max = (max * 9 + current) / 10;
	if (current >= max)
	  {
	    max = (current * (100 + adaptiveScalingFactor)) / 100 ;
	    prev = max; // force things to redraw
	  }
	break;
      }

    if(max == 0)
      currentPercent = 100;
    else
      currentPercent = current * 100 / max;

    if(prev != current)
      update();
  }


public void set_percentage(int i)
  {
    currentPercent = i;
    current = max * currentPercent / 100;
  }

public final int get_max() { return max; }

public final int get_value() { return current; }

public final int get_percentage() { return currentPercent; }

public final void progress() { set_value(current + 1); }

public final void progress(int i) { set_value(current + i); }

public final void unprogress() { set_value(current - 1); }

public final void unprogress(int i) { set_value(current - i); }

public void reset() 
  { 
    set_value(0); 
    max = initialMax;
  }

public void complete() { set_value(max); }

public void stall() {;}

public void idle() {;}

public void startDancing() 
  {
    System.out.println("now is time on shprockets ven ve dance!");
    danceCard++;
    if(danceThread == null)
      {
	dancing = true;
	danceThread = new Thread(new DanceThread(this, 
						 danceShoeSize,
						 danceLength));
	danceThread.start();
      }
  }

public void stopDancing() 
  {
    System.out.println("touch my monkey");
    danceCard--;
    if(danceCard <= 0)
      {
	System.out.println("Dieter has left the building");
	dancing = false;
	danceCard = 0;
	if (danceThread != null)
	  {
	    danceThread.stop();
	    danceThread = null;
	  }
	reset();
      }
      
  }

  ////////////////////////////////////////////////////////////////////
  // the rest of these are used to draw the thing.  Subclasses must
  // override draw_contents() and should override getPrefferredSize().

public abstract void draw_contents(Graphics g);

public void danceStep(Graphics g) {;}
  
public void draw_border(Graphics g)
  {
    // this works well at thicknesses of 1 or 2 pixels
    Dimension d = size();
    int thick = 2;
    
    d.width = d.width - border - border;
    d.height = d.height - border - border;
    int i = 0;
    switch(border_style)
      {
      case BORDER_FLAT:
	while(i < border)
	  {
	    g.drawRect(border+i, border+i, d.width-border, d.height-border);
	    i++;
	  }
	break;

      case BORDER_RAISED:
	while(i < border)
	  {
	    g.draw3DRect(border+i, border+i, d.width-border, d.height-border,
			 true);
	    i++;
	  }
	break;

      case BORDER_SUNKEN:
	while(i < border)
	  {
	    g.draw3DRect(border+i, border+i, d.width-border, d.height-border,
			 false);
	    i++;
	  }
	break;

      }
  }  

public void update()
  {
    // just draw the contents so the border doesn't flicker
    Graphics g = getGraphics();
    if(dancing)
      danceStep(g);
    else
      draw_contents(g);
  }


public void paint(Graphics g) 
  {
    g.setColor(border_color);
    draw_border(g);
    g.setColor(fg_color);    
    draw_contents(g);
  }
 
public Dimension preferredSize() 
  {
    return getPreferredSize();
  }
  
public Dimension getPreferredSize() 
  {
    if (psize == null)
      {
	psize= new Dimension(CtoP.Width(16,this), CtoP.Height(2,this));
      }

    return psize;
  }
  
}

class DanceThread implements Runnable
{
  Meter mMeter = null;
  int mDanceShoeSize = 0;
  int mDanceLength = 0;
  int mCur = 0;

  DanceThread(Meter m, int danceShoeSize, int danceLength)
  {
    mMeter = m;
    mDanceShoeSize = danceShoeSize;
    mDanceLength = danceLength;
  }

public void run()
  {
    mCur = mDanceShoeSize;
    mMeter.set_max(mDanceLength + mDanceShoeSize);
    mMeter.set_value(mCur);
    boolean True = true;
    while(True) // stupid javac won't compile while(true)
      {
	// System.out.println("gonna move Right");
	if(!moveRight())
	  return;
	// System.out.println("gonna move Left");
	if(!moveLeft())
	  return;
      }
  }
  
  boolean moveLeft()
  {
    // System.out.println("moving Left");
    while(mCur > mDanceShoeSize)
      {
	mCur -= mDanceShoeSize;
	if(mCur < mDanceShoeSize)
	  {
	    // System.out.println("done decreasing");
	    mCur = mDanceShoeSize;
	    break;
	  }
	if(go())
	  return false;
      }
    return true;
  }

  boolean moveRight()
  {
    // System.out.println("moving Right");
    while(mCur < mDanceLength)
      {
	mCur += mDanceShoeSize;
	if(mCur >=  mDanceLength)
	  {
	    mCur = mDanceLength;
	    // System.out.println("done increasing");
	  }
	else if(go())
	  return false;
      }
    return true;
  }

  boolean go()
  {
    // System.out.println("going");
    mMeter.set_value(mCur);
    try { Thread.sleep(100); }
    catch(InterruptedException X)
      { 
	System.out.println("insomnia"); 
	return true;
      }
    return false;
  }
}
