/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   MsgBox.java
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

package nfr.misc;

import java.awt.*;
import java.util.*;
///////////////////////////////////////////////////////////////////////////
// If you just want to show a dialog box with a message and an "Ok"
// button, use the static method MsgBox.show(String title, String msg)
//
// If you just want to show a dialog box with a message and an "Ok"
// button, but you want to block until the user has pressed the "Ok"
// button, use the static method MsgBox.wait(String title, String msg)
//
// If you'd like to specify some buttons and then find out which one
// the user pushed, create a MsgBox and then add buttons using the
// addButton() method.  When you've added all the buttons, pass the
// new MsgBox object to the static method MsgBox.wait(MsgBox mb).  The
// function will block until the user has pressed a button. The return
// value is the label of the button pressed
// The one problem is that if you try to wait on this from an event 
// handler without spawning a thread first, you'll deadlock.
///////////////////////////////////////////////////////////////////////////

//public class MsgBox extends Dialog implements Runnable
//{
//  String mResult = null;
//
//  public MsgBox(String title, String msg, boolean modal)
//  {
//    super(null, title, modal);

// it seems that netscape 3 doesn't support dialogs :(
public class MsgBox extends Frame implements Runnable
{
  String mResult = null;
  Vector mButtons = null;
  String mMsg = null;
  boolean mAlive = true;

  public MsgBox(String title, String msg, boolean modal)
  {
    mMsg = msg;
    setTitle(title);
    setFont(NFRInfo.font);
    mButtons = new Vector();
  }

public void addButton(String s)
  {
    mButtons.addElement(s);
  }

public void run() { show(); }

public void show()
  {
    if(mButtons.size() == 0)
      mButtons.addElement("Ok");
    repTile l = new repTile();
    setLayout(l);
    Label lbl = new Label(mMsg);
    add(lbl);
    l.setConstraints(0,0,0,0,"ch",lbl);
    lbl = new Label("");
    add(lbl);
    l.setConstraints(1,0,1,0,"ch",lbl);
    Panel bp = new Panel();
    for(int i = 0; i < mButtons.size(); i++)
      {
	Button b = new Button( (String)mButtons.elementAt(i) );
	bp.add(b);
      }
    add(bp);
    l.setConstraints(2,0,2,0,"ch",bp);
    l.setborder(CtoP.Width(3, this), CtoP.Width(3, this),
		CtoP.Height(1, this), CtoP.Height(1, this));
    l.setStretchy(0,0);
    int w = mMsg.length() + 8;
    if(w < 55)
      w = 55;
    resize(CtoP.Width(w, this), CtoP.Height(11, this));
    super.show();
  }

public boolean isAlive() { return mAlive; }

public static void show(String title, String msg)
  {
    MsgBox mb = new MsgBox(title, msg, false);
    mb.show();
  }

public static String wait(String title, String msg)
  {
    MsgBox mb = new MsgBox(title, msg, false);
    return MsgBox.wait(mb);
  }

public static String wait(MsgBox mb)
  {
    Thread mbThd = new Thread(mb);
    mbThd.start();
    

    //try { mbThd.join(); } 

    // bug: should be able to avoid the polling below with the join()
    // call above, but trying to call stop() in the action() method
    // isn't working - fix later

    try 
      {
	while(mb.isAlive())
	  Thread.sleep(250);
      }
    
    catch(InterruptedException e) { System.out.println("interrupted"); }

    return mb.getResult();
  }

public final String getResult() { return mResult; }

public boolean action(Event e, Object arg) 
  {
    if(e.target instanceof Button)
      {
	dispose(); // any of the buttons make it go away
	mResult = new String((String)arg);
	mAlive = false;
	return true;
      }
    return false;
  }
}

