/*
        Copyright(C) 1997 Network Flight Recorder, Inc.
        All rights reserved.

        Use and distribution of this software and its source code
        are governed by the terms and conditions of the
        Network Flight Recorder Software License ("LICENSE.TXT")

By Mark Sienkiewicz / NFR

*/


/*

extabthing - extended tabthing

How to use:

	tabthing t = new extabthing(max_number_of_tabs);
		// you don't *have* to use them all.  it makes an array
		// that big for internal use

	t.addThing(object,tab_object");
		// each added object gets a new page.  the tab_object
		// is displayed in the tab area

	t.compute();
		// tell it you are done adding things

It will use all the space given to it by the parent layout manager.

You only get one row of tabs.


Don't be surprised if this looks REMARKABLY like tabthing. :)  The two
should probably be merged into a single object with options for all
the curvy lines and stuff, but that would be a lot of work just so I
could feel satisfied that I'm being a purist.

*/

package nfr.misc;

import java.awt.*;
import java.util.Hashtable;

class extabtop extends Panel {

	public extabtop ( Component c ) {
		super();
		add(c);
	}

	public boolean action (Event e, Object arg) 
	{
		Event e1 = new Event(this, Event.ACTION_EVENT, this );
		this.getParent().handleEvent(e1);
		return true;
	}

	public boolean mouseDown(Event e, int x, int y ) 
	{
		Event e1 = new Event(this, Event.ACTION_EVENT, this );
		this.getParent().handleEvent(e1);
		return true;
	}

}

public class extabthing extends Panel 
{
	int max_tabs;			// max tabs we can have
	int cur_tabs;			// current number of tabs
	int cur_selected=0;
	static int borderline = 20;

	// objects at the bottom of the tabthing; logos, or maybe
	// panels containing several objects
	Component c_foot, a_foot, b_foot;

	// thing across the top
	Component top_part;

// g1,  zce, zco, vpad, hpad
	int g1 = 10, zce = 10, zco = 15, vpad = 10, hpad = 20;

	int g5 = 20, g6 = 10, g7 = 10;

	/*
	* data on each tab:
	*  a name/label - displayed on the tab bar
	*  a component to put on the card - probably a composite type,
	*	but it's up to you to build it.
	*  a tabtop widget - the actual clickable creature.
	*/
	
	String [] names;		// label on each tab (must be unique)
	Component [] carray;		// component on card
	extabtop [] barray;		// button at the top

	Point [] box_loc;
	Point [] box_size;
	/*
	* the body of the tab panel is a card layout on a panel
	*/
	Panel cardpanel;
	CardLayout card;


	Color brighter = null, darker = null;

	// You construct the tab thing with
	// tabthing t = new tabthing( max_number_of_tabs )

	public extabthing ( int n_tabs ) {
		names = new String[n_tabs];
		carray = new Component[n_tabs];
		barray = new extabtop[n_tabs];
		box_loc = new Point [ n_tabs ];
		box_size = new Point [ n_tabs ];
		max_tabs = n_tabs;
		cur_tabs = 0;
		cardpanel = new Panel();
		add(cardpanel);


		this.setLayout(null);

		//this.setBackground(Color.black);
		//this.setForeground(Color.white);

		// here theer be lameness - right now I only allow
		// one row of tabs at the top; splitting them into
		// multiple rows will not affect the interface, but
		// it will affect this code:

		card = new CardLayout();
		cardpanel.setLayout(card);
		// cardpanel.reshape(0,50,200,200);
	}

	public void addThing( Component c, String s)
	{
		int x;
		for (x=0; x< cur_tabs; x++) 
			{
			if (c == carray[x])
				break;
			}
		if (x < max_tabs)
			{
			carray[x]=c;
			names[x]=s;
			cur_tabs=x+1;
			}
		else
			{
			System.out.println("tabthing: added too many tabs:" + " name is " + s);
			// should throw an exception, but then everybody
			// has to write try/catch around this function...
			}
	}

	public void addThing( Component c, Component tab)
	{
		int x;
		for (x=0; x< cur_tabs; x++) 
			{
			if (c == carray[x])
				break;
			}
		if (x < max_tabs)
			{
			carray[x]=c;
			names[x]= null;
			cur_tabs=x+1;
			barray[x] = new extabtop(tab);
			repTile r = new repTile();
			barray[x].setLayout(r);
			r.setConstraints(0,0,0,0,"",tab);
			}
		else
			{
			System.out.println("tabthing: added too many tabs with object: " + tab );
			// should throw an exception, but then everybody
			// has to write try/catch around this function...
			}
	}
	

	public void set_common_things( Component c, Component a, Component b,
		Component top_part)
	{
	c_foot = c;
	a_foot = a;
	b_foot = b;
	this.top_part = top_part;
	add(c_foot);
	add(a_foot);
	add(b_foot);
	if (top_part != null) add(top_part);
	}

	/*
	* call compute() after you have added all the tab elements you
	* want to have.
	*/

	public void compute() {
		int x;
		extabtop b;
		Component c;
		for (x=0; x<cur_tabs; x++)
			{
			c=carray[x];
			if ( barray[x] == null)
				barray[x] = new extabtop(new Button(names[x]) );
			this.add(barray[x]);
			cardpanel.add(c);
			card.addLayoutComponent(""+x,c);
			}
		select_tab(1);
		select_tab(0);
	}

	/*
	*/
        public void paint(Graphics gc) 
	{
		super.paint(gc);
		gc.setColor(getForeground());
// System.out.println("enter paint function");

		Point ul_card = new Point ( 0, 0);
		Point br_card = new Point ( 0, 0);

	// this function is broken up into several blocks to get
	// around bugs in the java compiler.

		Point wmax = new Point (this.size().width, this.size().height);

	// compute stuff at bottom of screen
		{

		Dimension a_size = a_foot.preferredSize();
		Dimension b_size = b_foot.preferredSize();
		Dimension c_size = c_foot.preferredSize();

// System.out.println("sizes "+a_size+" "+b_size+" "+c_size);

		if (c_size.height > b_size.height)
			b_size.height = c_size.height;

		if (c_size.height > b_size.height)
			b_size.height = c_size.height;

		if (b_size.height < a_size.height + 2 * zce + 2 * zco )
			b_size.height = a_size.height + 2 * zce + 2 * zco;

		Point bul = new Point ( wmax.x - b_size.width - hpad - zco,
				wmax.y - b_size.height - vpad - zco );

		Point bur = new Point ( bul.x + b_size.width, bul.y );
		Point bbl = new Point ( bul.x, bul.y + b_size.height );
		Point bbr = new Point ( bul.x + b_size.width,
						bul.y + b_size.height );

			b_foot.reshape(bul.x, bul.y, bbr.x-bul.x, bbr.y-bul.y);
// System.out.println("b at "+ b_foot.location() +" " + b_foot.size());

		Point aul = new Point ( bul.x - a_size.width - g1 - zco,
				bul.y );

		Point abl = new Point ( aul.x, aul.y + a_size.height );
		Point abr = new Point ( aul.x + a_size.width,
						aul.y + a_size.height );
		Point aur = new Point ( aul.x + a_size.width, aul.y );

			a_foot.reshape(aul.x, aul.y, abr.x-aul.x, abr.y-aul.y);
// System.out.println("a at "+ a_foot.location() +" " + a_foot.size());

		Point cll = new Point ( aul.x - zco,
				aul.y + a_size.height / 2);

		Point clr = new Point ( wmax.x - hpad, cll.y);

		Point clre = new Point ( wmax.x, cll.y );

		Point aulc = new Point ( aul.x + zce , aul.y + zce );

		Point aul1 = new Point ( aul.x - zco , aul.y + zce );

		Point aul2 = new Point ( aul.x + zce , aul.y - zco );

		Point ablc = new Point ( abl.x + zce , abl.y - zce );

		Point abl1 = new Point ( abl.x - zco , abl.y - zce );

		Point abl2 = new Point ( abl.x + zce , abl.y + zco );

		Point coc = new Point ( abr.x + g1 - zce - zco ,
				abr.y + zco + zco + zce );

		Point coc1 = new Point ( coc.x + zce + zco , coc.y );

		Point coc2 = new Point ( coc.x, coc.y - zce - zco );

		Point clb = new Point ( 0, cll.y );

		Point cccl = new Point ( hpad, cll.y );

		Point bblc = new Point ( bbl.x + zce , bbl.y -zce );
		Point bbl1 = new Point ( bbl.x - zco , bbl.y -zce );
		Point bbl2 = new Point ( bbl.x + zce , bbl.y + zco );

		Point cccr = new Point ( cccl.x + 2 * zco + c_size.width
			, cccl.y );

		Point burc = new Point ( bur.x - zce , bur.y + zce );
		Point bur1 = new Point ( bur.x - zce , bur.y - zco );
		Point bur2 = new Point ( bur.x + zco , bur.y + zce );

		Point bbrc = new Point ( bbr.x - zce , bbr.y - zce );
		Point bbr1 = new Point ( bbr.x + zco , bbr.y - zce );
		Point bbr2 = new Point ( bbr.x - zce , bbr.y + zco );

		Point cul = new Point ( hpad + zco , bur1.y );

		Point cbl = new Point ( cul.x, cul.y + c_size.height );
		Point cbr = new Point ( cul.x + c_size.width,
						cul.y + c_size.height );
		Point cur = new Point ( cul.x + c_size.width, cul.y );

		c_foot.reshape(cul.x, cul.y, c_size.width, c_size.height);
// System.out.println("c at "+ c_foot.location() +" " + c_foot.size());

		Point cblc = new Point ( cbl.x + zce , cbl.y -zce );
		Point cbl1 = new Point ( cbl.x - zco , cbl.y -zce );
		Point cbl2 = new Point ( cbl.x + zce , cbl.y + zco );

		Point cbrc = new Point ( cbr.x - zce , cbr.y - zce );
		Point cbr1 = new Point ( cbr.x + zco , cbr.y - zce );
		Point cbr2 = new Point ( cbr.x - zce , cbr.y + zco );


	// draw stuff at bottom of screen

	// center lines
		gc.drawLine(clb.x,clb.y,cccl.x,cccl.y);
		gc.drawLine(cccl.x,cccl.y,cbl1.x,cbl1.y);
		gc.drawLine(cbl2.x,cbl2.y,cbr2.x,cbr2.y);
		gc.drawLine(cccr.x,cccr.y,cll.x,cll.y);
		gc.drawLine(aul1.x,aul1.y,abl1.x,abl1.y);
		gc.drawLine(abl2.x,abl2.y,coc2.x,coc2.y);
		gc.drawLine(coc1.x,coc1.y,bbl1.x,bbl1.y);

		gc.drawLine(bbl2.x,bbl2.y,bbr2.x,bbr2.y);
		gc.drawLine(bbr1.x,bbr1.y,bur2.x,bur2.y);
		gc.drawLine(clr.x,clr.y,clre.x,clre.y);
		gc.drawLine(aul2.x,aul2.y,bur1.x,bur1.y);

		gc.drawLine(cccr.x, cccr.y, cbr1.x, cbr1.y);

	// for horizontal lines between parts of the image, the lower & 
	// higher horizontal lines don't reach quite as far as the primary 
	// line.  this gives a sculpted look without making new colors 
	// and doing area fills  (area fills of parts of this mess would
	// be pretty ugly to implement.)

		// lower lines

		gc.drawLine(clb.x, clb.y+2, cccl.x - 2, cccl.y+2);
		gc.drawLine(cccl.x - 2,cccl.y + 2,cbl1.x - 2,cbl1.y);
		gc.drawLine(cccr.x + 2, cccr.y+2, cll.x - 2 , cll.y+2);
		gc.drawLine(clr.x + 2, clr.y + 2, clre.x, clre.y + 2);
		gc.drawLine(cccr.x - 2 , cccr.y + 2 , cbr1.x - 2, cbr1.y);

		// higher lines
		gc.drawLine(clb.x , clb.y-2, cccl.x - 2 , cccl.y-2);
		gc.drawLine(cccl.x + 2,cccl.y + 2 ,cbl1.x + 2,cbl1.y);
		gc.drawLine(cccr.x + 2, cccr.y-2, cll.x - 2 , cll.y-2);
		gc.drawLine(clr.x + 2, clr.y - 2, clre.x, clre.y - 2);
		gc.drawLine(cccr.x + 2 , cccr.y + 2, cbr1.x + 2, cbr1.y);

	// around box on left

		gc.drawLine(cbl2.x , cbl2.y+2, cbr2.x , cbr2.y+2);
		gc.drawLine(cbl2.x , cbl2.y-2, cbr2.x , cbr2.y-2);

	// around box on right

		gc.drawLine(aul2.x, aul2.y - 2, bur1.x, bur1.y - 2);
		gc.drawLine(aul2.x, aul2.y + 2, bur1.x, bur1.y + 2);

		gc.drawLine(abl2.x, abl2.y - 2, coc2.x, coc2.y - 2);
		gc.drawLine(abl2.x, abl2.y + 2, coc2.x, coc2.y + 2);

		gc.drawLine(bbl2.x, bbl2.y - 2, bbr2.x, bbr2.y - 2);
		gc.drawLine(bbl2.x, bbl2.y + 2, bbr2.x, bbr2.y + 2);

		gc.drawLine(bbr1.x - 2, bbr1.y, bur2.x - 2, bur2.y );
		gc.drawLine(bbr1.x + 2, bbr1.y, bur2.x + 2, bur2.y );

		gc.drawLine(aul1.x - 2, aul1.y, abl1.x - 2, abl1.y );
		gc.drawLine(aul1.x + 2, aul1.y, abl1.x + 2, abl1.y );

		gc.drawLine(coc1.x - 2, coc1.y, bbl1.x -2, bbl1.y );
		gc.drawLine(coc1.x + 2, coc1.y, bbl1.x + 2, bbl1.y );

	// draw bunches of arcs

		do_arc(gc,burc,1);
		do_arc(gc,bbrc,2);
		do_arc(gc,bblc,3);
		do_arc(gc,coc,1);
		do_arc(gc,ablc,3);
		do_arc(gc,aulc,4);
		do_arc(gc,cbrc,2);
		do_arc(gc,cblc,3);

		br_card.x = wmax.x - hpad;
		br_card.y = bur1.y - vpad;

		}


	// compute stuff at top of screen
		{
		Dimension top_part_size ;
		if (top_part != null)
			top_part_size = top_part.preferredSize();
		else
			top_part_size = new Dimension(0,0);

		int bh = 0;
		for (int x = 0; x < cur_tabs; x++ )
			{
			bh = java.lang.Math.max(bh,barray[x].preferredSize().height);
			}



		Point ta = new Point ( hpad + zco, zco+g6+top_part_size.height);
		Point tb = new Point ( ta.x, ta.y + bh );

		Point td = new Point( ta.x, ta.y ) ;

		{
		int x = 0;
		while ( x < cur_tabs)
			{
			td.x = td.x + barray[x].preferredSize().width+g7;
			x++;
			}
		td.x = td.x - g7;
		}

// bug: should place limit on lower size of bh, here

		Point tc = new Point ( td.x, tb.y );

		Point tac = new Point ( ta.x + zce, ta.y + zce );
		Point ta1 = new Point ( ta.x - zco, ta.y + zce );
		Point ta2 = new Point ( ta.x + zce, ta.y - zco );

		Point tbc = new Point ( tb.x + zce, tb.y - zce );
		Point tb1 = new Point ( tb.x - zco, tb.y - zce );
		Point tb2 = new Point ( tb.x + zce, tb.y + zco );

		Point tcc = new Point ( tc.x - zce, tc.y - zce );
		Point tc1 = new Point ( tc.x + zco, tc.y - zce );
		Point tc2 = new Point ( tc.x - zce, tc.y + zco );

		Point tdc = new Point ( td.x + g5 - zce, td.y + zce);
		Point td1 = new Point ( td.x + g5 + zco, td.y + zce);
		Point td2 = new Point ( td.x + g5 - zce, td.y - zco);

	//
		Point tclb = new Point ( 0, ta.y + bh / 2 );
		Point tccl = new Point ( hpad, tclb.y );
		Point tclr1 = new Point ( tc1.x , tclb.y );
		Point tclr2 = new Point ( td1.x, tclb.y );
		Point tclre = new Point ( wmax.x, tclb.y);

	// draw the item above the tabs
		if (top_part != null)
			{
			Dimension d = top_part.preferredSize();
			top_part.reshape(( wmax.x - d.width ) / 2, 0,
				d.width, d.height );
			}

	// draw lines around the tabs
		gc.drawLine(tclb.x, tclb.y, tccl.x, tccl.y);
		gc.drawLine(ta1.x, ta1.y, tb1.x, tb1.y);
		gc.drawLine(tclr1.x, tclr1.y, tclre.x, tclre.y );
		gc.drawLine(tb2.x, tb2.y, tc2.x, tc2.y );
		gc.drawLine(ta2.x, ta2.y, td2.x, td2.y );
	 	gc.drawLine(td1.x, td1.y, tclr2.x, tclr2.y );
		gc.drawLine(tc1.x, tc1.y, tclr1.x, tclr1.y );

	// upper line
		gc.drawLine(tclb.x, tclb.y - 2, tccl.x - 2, tccl.y - 2);
		gc.drawLine(ta1.x - 2, ta1.y, tb1.x - 2 , tb1.y);
		gc.drawLine(tclr1.x + 2, tclr1.y - 2, tclre.x, tclre.y - 2);
		gc.drawLine(tb2.x, tb2.y - 2, tc2.x, tc2.y - 2 );
		gc.drawLine(ta2.x, ta2.y - 2, td2.x, td2.y - 2 );
	 	gc.drawLine(td1.x - 2, td1.y, tclr2.x - 2, tclr2.y - 2);
		gc.drawLine(tc1.x - 2, tc1.y, tclr1.x - 2, tclr1.y + 2 );

	// lower line
		gc.drawLine(tclb.x, tclb.y + 2, tccl.x - 2, tccl.y + 2);
		gc.drawLine(ta1.x + 2, ta1.y, tb1.x + 2, tb1.y);
		gc.drawLine(tclr1.x + 2, tclr1.y + 2, tclre.x, tclre.y + 2);
		gc.drawLine(tb2.x, tb2.y + 2 , tc2.x, tc2.y + 2 );
		gc.drawLine(ta2.x, ta2.y + 2, td2.x, td2.y + 2);
	 	gc.drawLine(td1.x + 2, td1.y, tclr2.x + 2, tclr2.y - 2 );
		gc.drawLine(tc1.x + 2, tc1.y, tclr1.x + 2, tclr1.y + 2);
	
	// draw arcs
		do_arc(gc, tac, 4);
		do_arc(gc, tbc, 3);
		do_arc(gc, tcc, 2);
		do_arc(gc, tdc, 1);
	
		ul_card.x = hpad;
		ul_card.y = tb2.y + vpad;

		int hpos = ta.x;
		for (int x = 0; x < cur_tabs; x++)
			{
			Dimension d = barray[x].preferredSize();
			box_loc[x] = new Point(hpos, ta.y);
			box_size[x] = new Point(d.width, d.height );
// System.out.println("reshape tab "+x+" to "+hpos+" " +ta.y);
			barray[x].reshape(hpos, ta.y, 
				d.width, d.height);
			barray[x].repaint();
			hpos += d.width + g7;
			}
		}

// System.out.println("cardpanel.reshape "+ ul_card + " "+ br_card);

	// have to reshape the panel that has the card layout
	// on it to fit into the new space

	cardpanel.reshape(ul_card.x, ul_card.y, br_card.x - ul_card.x, 
		br_card.y - ul_card.y );

	cardpanel.validate();

	// you *also* have to reshape the object INSIDE the card panel
	// because it can't get the resize right!

	carray[cur_selected].reshape(0, 0, br_card.x - ul_card.x, 
		br_card.y - ul_card.y );

// System.out.println("end cardpanel.reshape ");

	select_tab(cur_selected);

// System.out.println("done ex tabthing paint()\n");
	// try { Thread.sleep(3000); } catch (Exception e) { }
// System.out.println("done sleeping\n");

	}

	public void do_arc (Graphics gc, Point where, int quadrant )
	{
		// Damn!  Do you think the Java guys could have made
		// the drawArc() primitive harder to use?

		// Don't even ask how I came up with these formula.  I
		// read the java 1.0 and java 1.1 specs, tried to figure
		// out what drawArc does from the two specs, then started
		// drawing arcs and jiggling them around until they got
		// about where I wanted them...

		int x,y;
		int as;

		x = where.x - zce - zco ; y = where.y  - zce - zco;

		as=0;
		switch (quadrant)
			{
		case 1:	as=0   ; 
			break;
		case 2: as=270 ; 
			break;
		case 3: as=180 ; 
			break;
		case 4: as=90  ; 
			break;
			}
		gc.drawArc(x, y, 2 * (zce + zco) , 2 * (zce + zco), as, 90);
		gc.drawArc(x - 2 , y - 2 , 4+ 2 * (zce + zco) , 4+ 2 * (zce + zco) , as, 90);
		gc.drawArc(x + 2 , y + 2 , 2 * (zce + zco) -4 , 2 * (zce + zco) - 4 , as, 90);

	}

	/*
	* java 1.0 event function...
	*/

	public boolean action (Event e, Object arg) 
	{
		if (e.target instanceof extabtop)
			for (int x = 0; x< cur_tabs; x++)
				{
				if (barray[x] == e.target)
					{
					select_tab(x);
					return true;
					}
				}
	return false;
	}

	public void select_tab(int which) 
	{
	card.show(cardpanel,""+which);

	Graphics gc = this.getGraphics();

	if (gc == null)
		{
		System.out.println("gc null in extabthing select_tab()");
		return;
		}

	gc.setColor(cardpanel.getBackground());

	gc.drawRect(box_loc[cur_selected].x-		4,
		box_loc[cur_selected].y-		4,
		box_size[cur_selected].x-1 + 2 *	4,
		box_size[cur_selected].y-1 + 2 *	4  );

	gc.drawRect(box_loc[cur_selected].x -		3,
		box_loc[cur_selected].y -		3,
		box_size[cur_selected].x-1 + 2 * 	3,
		box_size[cur_selected].y-1 + 2 *	3  );

	cur_selected=which;

	gc.setColor(Color.red);

	gc.drawRect(box_loc[cur_selected].x-		4,
		box_loc[cur_selected].y-		4,
		box_size[cur_selected].x-1 + 2 *	4,
		box_size[cur_selected].y-1 + 2 *	4  );

	gc.drawRect(box_loc[cur_selected].x -		3,
		box_loc[cur_selected].y -		3,
		box_size[cur_selected].x-1 + 2 * 	3,
		box_size[cur_selected].y-1 + 2 *	3  );

	}


}
