/*
        Copyright(C) 1997 Network Flight Recorder, Inc.
        All rights reserved.

        Use and distribution of this software and its source code
        are governed by the terms and conditions of the
        Network Flight Recorder Software License ("LICENSE.TXT")

By Mark Sienkiewicz / NFR

*/

/*
* You wouldn't think it would be so hard to put a picture on
* the screen in a language that is being hyped so intensely for
* web applications.  The provided interfaces for handling images
* is extremely complex.  
*
* This object is a Component that has an image in it.
*
* You need an Applet object to get an image from somewhere.  You
* can pass either a URL or a string to identify the image to fetch
* from the web server.  The image works if the URL or string identifies
* a GIF file.  I don't know if it knows other file formats.
*
* I believe this is almost the simplest it is possible make this
* object.  Simpler versions are welcome.
*/

package nfr.misc;

import java.applet.*;
import java.awt.*;
import java.awt.image.*;
import java.net.*;

public class jafp extends Canvas
{
	Image img;
	boolean loaded = false;
	MediaTracker tracker;

	public jafp ( Applet a, URL u )
	{
		try {
			img = a.getImage(u);
		} catch (Exception e) {
			img = null;
			System.out.println("choke loading image "+e);
			return;
		}
		mkjafp();
	}

	public jafp ( Applet a, String name )
	{
		try {
			URL u = new URL(a.getDocumentBase(),name);
			img = a.getImage(u);
		} catch (Exception e) {
			img = null;
			System.out.println("choke loading image "+e);
			return;
		}
		mkjafp();
	}
			

	public void mkjafp()
	{
		try {
			tracker = new MediaTracker(this);
			tracker.addImage(img, 0);
			tracker.checkAll(true);	// make it start loading
			tracker.waitForAll();
		} catch (Exception e) {
			System.out.println("choke on media tracker "+e);
			e.printStackTrace();
		}
	//
	// you could get a small win in loading time by removing the above
	// call to tracker.waitForAll(), but then this resize would fail
	// because we would not know the image size here.
	// If you want to implement an ImageObserver, you might be
	// able to do an asynchronous load.  If it works as well as the
	// rest of AWT, that will probably be an uphill battle.

		this.resize(img.getWidth(null), img.getHeight(null));
	}


/*
* Provide a way to send ACTION_EVENT so you can use this image 
* as a button.  Unlike X windows, the mouseDown event will be eaten
* by this object whether it uses it or not, so you can't just jam this
* image into a parent that is sensitive to mouse events.
*/
        public boolean mouseDown(Event e, int x, int y ) 
	{
	Event e1 = new Event(this, Event.ACTION_EVENT, this );
	this.getParent().handleEvent(e1);
	return true;
	}


/*
* you don't really need reshape or move functions at all, but sometimes
* some FlowLayout somewhere tries to give us a negative X coordinate.
* This ends up with most of the picture drawn outside our window, and
* therefore clipped.
*
* I've been unable to figure out whose FlowLayout is doing this, so I'm
* just going to ignore it because the layout manager that is doing this
* is obviously BROKEN.
*/
	public void reshape (int x, int y, int w, int h )
	{
	if (x < 0)
		return;
	super.reshape(x,y,w,h);
	}

	public void move (int x, int y )
	{
	if (x < 0)
		return;
	super.move(x,y);
	}

/*
* real work occurs here.  Just draw the image.
*/

	public void paint (Graphics gc)
	{
	if (loaded == false)
		{
		// this code is all about making sure the image is
		// fully loaded before we try to draw it.  See the
		// comment in mkjafp() before you try to do asynch
		// image loading.

		try {
			tracker.waitForAll();
		} catch (Exception e) {
			// what should I do?
		}
		loaded = true;
		tracker=null;	// free the tracker
		}
	gc.drawImage(img, 0, 0, null);
	}
}

