/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   AlertConfigurator.java
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

package nfr.rc;

import nfr.misc.*;
import java.awt.*;
import java.util.*;

/*****************************************************************************
 *****************************************************************************/

class SevSelector extends Panel implements ResConsts
{
  Checkbox mCBoxes[];
  CheckboxGroup mCBoxGroup = null;

  SevSelector()
  {
    repTile l = new repTile();
    setLayout(l);
    mCBoxGroup = new CheckboxGroup();
    mCBoxes = new Checkbox[4];

    mCBoxes[SEV_INFO] = new Checkbox("Information");
    add(mCBoxes[SEV_INFO]);
    l.setConstraints(0,0,0,0,"fl",mCBoxes[SEV_INFO]);
    mCBoxes[SEV_INFO].setCheckboxGroup(mCBoxGroup);

    mCBoxes[SEV_WARNING] = new Checkbox("Warning");
    add(mCBoxes[SEV_WARNING]);
    l.setConstraints(1,0,1,0,"fl",mCBoxes[SEV_WARNING]);
    mCBoxes[SEV_WARNING].setCheckboxGroup(mCBoxGroup);

    mCBoxes[SEV_ERROR] = new Checkbox("Error");
    add(mCBoxes[SEV_ERROR]);
    l.setConstraints(2,0,2,0,"fl",mCBoxes[SEV_ERROR]);
    mCBoxes[SEV_ERROR].setCheckboxGroup(mCBoxGroup);
  }

  void setThis(int sev)
  {
    mCBoxGroup.setCurrent(mCBoxes[sev]);
  }

  int getSelection()
  {
    int i = 3;
    while(!mCBoxes[i].getState() && i > 0)
      i--;
    return i;
  }

public void enable(boolean b) { setEnabled(b); }
public void enable() { enable(true); }
public void disable() { enable(false); }
public void setEnabled(boolean b)
  {
    int i = 3;
    while(i > 0)
      {
	i--;
	mCBoxes[i].enable(b);
      }
  }

public void clear()
  {
    int i = 3;
    while(i > 0)
      {
	i--;
	mCBoxes[i].setState(false);
      }
  }
}

class FacSelector extends Panel
{
  Vector mCheckboxes = null;
  Vector mAllFacIds = null;
  Checkbox mAckCkx = null;
  Checkbox mLogCkx = null;
  Checkbox mTraceCkx = null;

  FacSelector(ResTab rt)
  {
    Panel p = new Panel();
    repTile l = new repTile();
    p.setLayout(l);
    add(p);

    mAllFacIds = rt.cloneFacVector();
    mCheckboxes = new Vector(mAllFacIds.size());
    mCheckboxes.setSize(mAllFacIds.size());
    int i = 0;
    for (i = 1; i < mAllFacIds.size(); i++) // this is a 1-based array!
      {
 	ResId fId = (ResId) mAllFacIds.elementAt(i);
	if (fId == null)
	  break;
	Checkbox cBox = new Checkbox(fId.getDispName());
	mCheckboxes.setElementAt(cBox, i);
	p.add(cBox);
	l.setConstraints(i,0,i,0,"fl",cBox);
	
	// these have special relationships, so we need to keep
	// track of them.
	if("NFR_FAC_NFRLOG".equals(fId.getId()))
	  mLogCkx = cBox;
	else if("NFR_FAC_TRACE".equals(fId.getId()))
	  mTraceCkx = cBox;
	else if("NFR_FAC_ACK".equals(fId.getId()))
	  {
	    mAckCkx = cBox;
	    mAckCkx.disable();
	    mAckCkx.setState(false);

	    // stupid netscape on windoze doesn't change color of
	    // disabled checkbox
	    mAckCkx.setForeground(Color.gray); 
	  }
      }
    mCheckboxes.setSize(i);
    //System.out.println("*** made " + i + " checkboxen");
  }
  
  void setThese(Vector facIds)
  {
    if(facIds == null)
      return;

    for (int i = 1; i < mCheckboxes.size(); i++)
      {
	((Checkbox)mCheckboxes.elementAt(i)).setState(false);
      }

    for (int i = 0; i < facIds.size(); i++)
      {
	ResId fId = (ResId) facIds.elementAt(i);
	Checkbox cBox = (Checkbox) mCheckboxes.elementAt(fId.getResNo());
	cBox.setState(true);
      }
    handleSpecials();
  }
  
  Vector getSelections()
  {
    Vector selFacs = new Vector();
    for (int i = 1; i < mCheckboxes.size(); i++)
      {
	if(((Checkbox)mCheckboxes.elementAt(i)).getState())
	  selFacs.addElement((ResId) mAllFacIds.elementAt(i));
      }  
    return selFacs;
  }

  void handleSpecials()
  {
    // handle special relationships here

    // the Ack facility depends on the log facility to work, so we
    // ensure that ack can't be on unless log is, too.

    if((mLogCkx != null) && (mAckCkx != null))
      {
	if(mLogCkx.getState())
	  {
	    mAckCkx.enable();

	    // stupid netscape on windoze doesn't change color of
	    // disabled checkbox
	    mAckCkx.setForeground(Color.black);
	    mAckCkx.show(); // netscpe4 doesn't do this automatically :(
	  }
	else
	  {
	    mAckCkx.disable();
	    mAckCkx.setState(false);
	    mAckCkx.setForeground(Color.gray); 
	    mAckCkx.show(); // netscpe4 doesn't do this automatically :(
	  }
      }
      
  }

public boolean action(Event e, Object arg)
  {
    // the Trace facility is just a special case of Log, so we make
    // these two exclusive
    if((mLogCkx != null) && (mTraceCkx != null))
      {
	if((e.target.equals(mLogCkx)) && (mLogCkx.getState()))
	  mTraceCkx.setState(false);
	else if((e.target.equals(mTraceCkx)) && (mTraceCkx.getState()))
	  mLogCkx.setState(false);
      }
    handleSpecials();
    return false; // let this event go on up the hierarchy
  }

public void enable(boolean b) { setEnabled(b); }
public void enable() { enable(true); }
public void disable() { enable(false); }
public void setEnabled(boolean b)
  {
    for (int i = 1; i < mCheckboxes.size(); i++)
      ((Checkbox)mCheckboxes.elementAt(i)).enable(b);
  }

public void clear()
  {
    for (int i = 1; i < mCheckboxes.size(); i++)
      ((Checkbox)mCheckboxes.elementAt(i)).setState(false);
  }

}

public class AlertConfigurator extends ResConfigurator
{
  TextField mTextVal = null;
  FacSelector mFacSelector = null;
  SevSelector mSevSelector = null;

public AlertConfigurator(ResId rid, ResTab rt)
  {
    super(rt);

    repTile l = new repTile();
    setLayout(l);

    Label lbl = new Label("Message format string");
    add(lbl);
    l.setConstraints(0,0,0,1, "", lbl);
    mTextVal = new TextField();
    add(mTextVal);
    l.setConstraints(1,0,1,1, "", mTextVal);

    lbl = new Label("Message severity");
    add(lbl);
    l.setConstraints(2,0,2,0, "", lbl);
    mSevSelector = new SevSelector();
    add(mSevSelector);
    l.setConstraints(3,0,3,0, "", mSevSelector);

    lbl = new Label("Message destinations");
    add(lbl);
    l.setConstraints(2,1,2,1, "", lbl);
    mFacSelector = new FacSelector(mResTab);
    add(mFacSelector);
    l.setConstraints(3,1,3,1, "", mFacSelector);

    l.setStretchy(-1,0);

    read(rid);
  }

public boolean read(ResId rid)
  {
    mTextVal.setText( ((AlertRes)rid).getText() );
    mSevSelector.setThis( ((AlertRes)rid).getSev()); 
    mFacSelector.setThese( ((AlertRes)rid).getFacIds()); 
    return true;
  }

public boolean write(ResId rid)
  {
    String text = mTextVal.getText();
    String errMsg = checkQuotes(text);
    if(errMsg != null)
      {
	MsgBox.show("Error", errMsg);
	return false;
      }

    ((AlertRes)rid).setText(text);
    ((AlertRes)rid).setSev(mSevSelector.getSelection());
    ((AlertRes)rid).setFacIds(mFacSelector.getSelections());
    return true;
  }

public void setEnabled(boolean b)
  {
    mTextVal.enable(b);
    mFacSelector.enable(b);
    mFacSelector.enable(b);
  }

public void clear()
  {
    mTextVal.setText("");
    mFacSelector.clear();
    mFacSelector.clear();
  }

}
