/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   AlertRes.java
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

package nfr.rc;

import nfr.misc.*;

import java.net.*;
import java.io.*;
import java.util.*;
import java.awt.*;


public class AlertRes extends ResId implements Testable
{
  int mSeverity = 1;
  String mText = null;
  private Vector mFacIds = null;

public AlertRes(ResTab rt)
  {
    super(rt);
    setType(RES_ALERT);
    //System.out.println("in AlertRes(), ResCode=" + mCode);
  }

public AlertRes(String Id, ResTab rt)
  {
    super(Id, rt);
    setType(RES_ALERT);
    setText("<empty>");
  }

public AlertRes(int rc, DataInputStream In, ResTab rt)
  {
    super(rc, In, rt);
    try
      {
	mSeverity = In.readInt();
	int i = 0;
	int len = In.readInt();
	char sbuf[] = new char[len];
	while(i < len)
	  sbuf[i++] = (char) In.readByte();
	mText = new String(sbuf);

	mFacIds = new Vector();
	i = 0;
	len = In.readInt();
	while(i++ < len)
	  {
	    byte facCode = In.readByte();
	    mFacIds.addElement( (ResId)mResTab.getFacId((int)facCode) );
	  }

      }
    catch(Exception e)
      {
	e.printStackTrace();
      }
  }

public final String getText() { return mText; }

public void setText(String t) 
  { 
    if((mText != null) && (mText.equals(t)))
      return;

    //System.out.println("Text changing for: " + getId());
    mText = new String(t); 
    if(mState == RESID_STATE_STABLE)
      mState = RESID_STATE_ALTERED;
  }

public final int getSev() { return mSeverity; }

public final void setSev(int s) 
  { 
    if(mSeverity == s)
      return;

    //System.out.println("Severity changing for: " + getId());
    mSeverity = s; 
    if(mState == RESID_STATE_STABLE)
      mState = RESID_STATE_ALTERED;
  }

public final Vector getFacIds() { return mFacIds; }

public void setFacIds(Vector v) 
  { 
    boolean different = false;

    if(mFacIds == null)
      mFacIds = new Vector();

    if(mFacIds.size() != v.size())
      {
	//System.out.println(String.valueOf(mFacIds.size()) + " != " + 
	//	   String.valueOf(v.size()) );
	different = true;
      }
    else
      {
	String newId = null;
	String oldId = null;
	for(int i = 0; i < mFacIds.size(); i++)
	  {
	    oldId = ((ResId)mFacIds.elementAt(i)).getId();
	    different = true;

	    for(int j = 0; j < v.size(); j++)
	      {
		newId =  ((ResId)v.elementAt(j)).getId(); 
		if(oldId.equals(newId))
		  {
		    different = false;
		    break;
		  }
	      }

	    if(different)
	      {
		//System.out.println(oldId + " != " + newId);
		break;
	      }
	  }
      }
    

    if (!different)
      return;

    //System.out.println("FacIds changing for: " + getId());
    mFacIds = v; 
    if(mState == RESID_STATE_STABLE)
      mState = RESID_STATE_ALTERED;
  }


public ResConfigurator getConfigurator()
  {
    return new AlertConfigurator(this, mResTab);
  }

public ResConfigurator getConfigurator(ResTab rt)
  {
    return new AlertConfigurator(this, rt);
  }

public boolean dump(DataOutputStream Out)
  {
    super.dump(Out);

    try
      {
	Out.writeInt(mSeverity);

	if(mText == null)
	  Out.writeInt(0);
	else
	  {
	    Out.writeInt(mText.length());
	    Out.writeBytes(mText);
	  }

	if(mFacIds == null)
	  Out.writeInt(0);
	Out.writeInt(mFacIds.size());
	for(int i = 0; i < mFacIds.size(); i++)
	  {
	    byte facCode = (byte) ((ResId)mFacIds.elementAt(i)).getResNo();
	    Out.writeByte(facCode);
	  }

	// bug: remove this block
	// now write the fac strings
	//for(int i = 0; i < mFacIds.size(); i++)
	//{
	//  String s = ((ResId)mFacIds.elementAt(i)).getId();
	//  Out.writeInt(s.length());
	//  Out.writeBytes(s);
	//}
      }	
    catch(Exception e)
      {
	e.printStackTrace();
	return false;
      }
    return true;
  }

public boolean save()
  {
    return save("_default_alerts");
  }

public boolean save(String Range)
  {
    RangeRes range = (RangeRes) mResTab.get(Range);
    range.insert(this);
    return mResTab.dump();
  }

public static AlertRes load(String Id, String Resfile)
  {
    ResTab rt = new ResTab();
    rt.load(NFRInfo.ThisApplet.getDocumentBase(),
	    Resfile, 
	    "items=" + Id + "&types=" + String.valueOf(ID_FAC) + "," +
	    String.valueOf(RES_RANGE));
    AlertRes ar = (AlertRes) rt.get(Id);
    if (ar == null)
      {
	//System.out.println(Id + " wasn't there :(");
	ar = new AlertRes(Id, rt);
      }
    else
      {
	//System.out.println(Id + " found :)");
      }

    rt.insert(ar);

    return ar;
  }

public boolean test()
  {
    if((mState == RESID_STATE_ALTERED) || (mState == RESID_STATE_ADDED))
      {
	MsgBox.show("Warning", "You must apply the changes before testing");
	return true;
      }

    CgiScript cgi = new CgiScript("/cgi-bin/testalert");
    if(isUser())
      cgi.addDataNVPair("alertId", mId);
    else
      cgi.addDataNVPair("alertnum", String.valueOf(mCode));
    int Result = cgi.run();
    
    System.out.println("Result of cgi-bin/testalert: " + Result);
    if(Result < 0)
      MsgBox.show("Error", cgi.getLocMsg());
    else if (Result > 0)
      MsgBox.show("Error", cgi.getRemMsg());
    else
      MsgBox.show("Success", cgi.getRemMsg());

    return true;   
  }

}
