/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   FacIdConfigurator.java
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

package nfr.rc;

import nfr.misc.*;
import nfr.facilities.*;

import java.awt.*;

/*****************************************************************************
  just a simple class to wrap eric's fac configuration stuff
*****************************************************************************/

public class FacIdConfigurator  extends ResConfigurator
{
  TextField mDispNameTxt = null;
  Button mPropBtn = null;
  ResId mCurResId = null;
  Choice mFacTypeChooser = null;

public FacIdConfigurator(ResId rid, ResTab rt)
  {
    super(rt);

    mCurResId = rid;
    repTile l = new repTile();
    setLayout(l);

    Label lbl = null;
    int row = 0;

    lbl = new Label("Display name");
    add(lbl);
    l.setConstraints(row,0,row,0, "", lbl);
    row++;

    mDispNameTxt = new TextField(rid.getDispName() );
    add(mDispNameTxt);
    l.setConstraints(row,0,row,0, "", mDispNameTxt);
    row++;

    lbl = new Label("");
    add(lbl); // spacer
    l.setConstraints(row,0,row,0,"", lbl);
    row++;

    Panel p = new Panel();

    mFacTypeChooser = new Choice();

    if(rid.getState() == RESID_STATE_ADDED)
      {
	lbl = new Label("Facility type:");
	p.add(lbl);

	// this is a new facility so we need to allow the user to
	// specify a type

	// HEY_ERIC: here's where you'll add a choice item for each
	// type of facility that we know about.  Try to do this in a
	// way that will allow you to dynamically create the list of
	// available types so that we can support user defined types
	// in the future. probably the best way to do this is with a
	// static function in FacEditor that returns a vector of
	// possible types, then add each item from the vector to
	// mFacTypeChooser

	// bug:  Didn't make it dymanic yet.
	
	mFacTypeChooser.addItem("mail");
	mFacTypeChooser.addItem("pager");
	mFacTypeChooser.addItem("fax");
	mFacTypeChooser.addItem("generic");
	
	p.add(mFacTypeChooser);
      }
    else
      {
	mPropBtn = new Button("Properties");
	p.add(mPropBtn);
      }

    add(p);
    l.setConstraints(row,0,row,0,"ch", p);

    l.setStretchy(-1, 0);
  }

public boolean read(ResId rid)
  {
    mCurResId = rid;
    mDispNameTxt.setText( rid.getDispName() );

    return true;
  }

public boolean write(ResId rid)
  {
    //System.out.println("in FacIdConfigurator.write()");
    //System.out.println("rid = " + rid);
    //System.out.println("mDispNameTxt = " + mDispNameTxt);

    String dispName = mDispNameTxt.getText();
    String errMsg = checkQuotes(dispName);
    if(errMsg != null)
      {
	MsgBox.show("Error", errMsg);
	return false;
      }

    rid.setDispName(dispName);
    //System.out.println("set DispName to: " + rid.getDispName());

    // if we're in the ResAdder we need to set this so that FacEditWin
    // will know what type to create
    if(rid.getState() == RESID_STATE_ADDED)
      ((FacId)rid).setGuiType(mFacTypeChooser.getSelectedItem());

    // System.out.println("set GuiType to: " + ((FacId)rid).getGuiType());
    
    return true;
  }

  public boolean action(Event e, Object arg)
  {
    if(e.target == mPropBtn)
      {
	Window parent = mResTab.getCfgWindow();
	if(parent == null)
	  parent = NFRInfo.top;
	
	if(mCurResId.getState() == RESID_STATE_ADDED)
	  {
	    MsgBox.show("Error", "You must apply the current settings first");
	    return true;
	  }
	else
	  {
	    for(int i = 0; i < FIXED_FAC_ARRAY.length; i++)
	      {
		if(FIXED_FAC_ARRAY[i].equals(mCurResId.getId()))
		  {
		    MsgBox.show("Info", "There are no configurable properties for this item");
		    return true;
		  }
	      }
	  }

	( new FacEditWin((FacId)mCurResId, parent) ).show();
	return true;
      }
    
    return false;
  }
  
public void setEnabled(boolean b)
  {
    mPropBtn.enable(b);
    mFacTypeChooser.enable(b);
    mDispNameTxt.enable(b);
  }

public void clear() 
  {
    mDispNameTxt.setText("");
  }

}
