/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   RangeConfigurator.java
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

package nfr.rc;

import java.awt.*;

import nfr.misc.*;
import nfr.facilities.*;


public class RangeConfigurator extends ResConfigurator
{
  RangeRes mThisRange = null;
  List mMemberList = null;
  Button mDeleteBtn = null;
  Button mAddBtn = null;
  Button mTestBtn = null;
  ResConfigurator mMemberConfigurator = null;
  ResId mCurSelection = null;
  ResAdder mResAdder = null;
  Window mParentWin = null;


public RangeConfigurator(RangeRes rid, ResTab rt)
  {
    super(rt);

    mThisRange = rid;

    repTile l = new repTile();
    setLayout(l);

    mMemberList = new List();
    mMemberList.setBackground(Color.white);
    add(mMemberList);
    l.setConstraints(0,0,0,0,"",mMemberList);
    l.setMinSize(CtoP.Width(30, this), CtoP.Height(8, this), mMemberList);
    for(int i = 0; i < mThisRange.Members.size(); i++)
      {
	if(rt.isUser() == ((ResId) mThisRange.Members.elementAt(i)).isUser())
	  {
	    ResId res = (ResId) mThisRange.Members.elementAt(i);
	    mMemberList.addItem(res.getId());
	  }
      }

    ResId r = (ResId)mThisRange.Members.firstElement();
    mMemberConfigurator = r.getConfigurator(mResTab);

    mMemberConfigurator.clear();
    mMemberConfigurator.disable();

    Label lbl = new Label("  "); // spacer
    add(lbl);
    l.setConstraints(0,1,0,1,"",lbl);
    add(mMemberConfigurator);
    l.setConstraints(0,2,0,2,"ch",mMemberConfigurator);
    l.setMinSize(CtoP.Width(48, this), CtoP.Height(8, this), 
		 mMemberConfigurator);

    Panel btnPanel = new Panel();
    btnPanel.setLayout(new FlowLayout());
    if (rt.isUser())
      {
	mDeleteBtn = new Button("Delete");
	btnPanel.add(mDeleteBtn);
	mAddBtn = new Button("New");
	btnPanel.add(mAddBtn);
      }
    if(mThisRange.Members.firstElement() instanceof Testable)
      {
	mTestBtn = new Button("Test");
	btnPanel.add(mTestBtn);	    
      }
    add(btnPanel);

    l.setConstraints(1,0,1,0,"ch",btnPanel);
    l.setborder(CtoP.Width(0, this), CtoP.Width(0, this),
		CtoP.Height(1, this), CtoP.Height(0, this));

    l.setStretchy(0,0);

    mMemberList.select(0);
    OnSelection();
  }

public boolean read(ResId rid)
  {
    if(mCurSelection == null)
      return true;
    return mMemberConfigurator.read(mCurSelection);
  }

public boolean write()
  {
    if(mCurSelection == null)
      return true;
    return mMemberConfigurator.write(mCurSelection);
  }

public boolean write(ResId rid)
  {
    if(mCurSelection == null)
      return true;
    return mMemberConfigurator.write(mCurSelection);
  }

public void setEnabled(boolean b)
  {
    mMemberList.enable(b);
    mDeleteBtn.enable(b);
    mAddBtn.enable(b);
    mTestBtn.enable(b);
    mMemberConfigurator.enable(b);
  }

public void clear()
  {
    mMemberList.clear();
    mMemberConfigurator.clear();
  }

public boolean action(Event e, Object eArg)
  {
    // System.out.println(eArg);
    // System.out.println(e);

    if (e.target instanceof List)
      {
	OnSelection();
	return true;
      }

    else if(e.target instanceof Button)
      {
	// System.out.println("button");
	if(eArg == "Delete")
	  {
	    return OnDelete();
	  }

	else if(eArg == "New")
	  {
	    mResAdder = new ResAdder(mThisRange.getMemberType(), mResTab, 
				     this);
	    add(mResAdder);
	    mResAdder.show();
	    return true;
	  }

	if(eArg == "Test")
	  {
	    if(OnSelection())
	      ((Testable)mCurSelection).test();
	  }

	// this one gets passed up from mResAdder
	if(String.valueOf(eArg).equals("Add"))
	  {
	    // System.out.println("got Add");
	    return OnAdd();
	  }
      }
    return false;
  }

public boolean handleEvent(Event e)
  {
    if(e.id == Event.LIST_SELECT)
      {
	OnSelection();
	return true;
      }
    return super.handleEvent(e);
  }

boolean OnAdd()
  {
    // System.out.println("in OnAdd()");
    ResId rid = mResAdder.getResId();
    if(rid == null)
      return true; // there was something wrong with the user input
    // System.out.println("new id: " + rid.getId());
    mResAdder.dispose();
    mMemberList.addItem(rid.getId());
    mThisRange.insert(rid);
    //if(rid.getType() == ID_FAC)
    //( new FacEditWin((FacId)rid, mParentWin) ).show();
    return true;
  }

boolean OnDelete()
  {
    mCurSelection.setState(RESID_STATE_DELETED);
    int i = mMemberList.getSelectedIndex();
    mThisRange.Members.removeElementAt(i);
    mMemberList.delItem(i);
    
    if(i == 0)
      mMemberList.select(i);
    else
      mMemberList.select(i-1);
    OnSelection();
    return true;
  }

boolean OnSelection()
  {
    if(mMemberList.countItems() <= 0)
      {
	mCurSelection = null;
	mMemberConfigurator.clear();
	mMemberConfigurator.disable();
      }
    else
      {
	if(mCurSelection != null)
	  {
	    if(!mMemberConfigurator.write(mCurSelection))
	      return false;
	  }
	int idx = mMemberList.getSelectedIndex();
	mCurSelection = (ResId)mThisRange.Members.elementAt(idx);
	mMemberConfigurator.enable();
	mMemberConfigurator.read(mCurSelection);

	if(mDeleteBtn != null)
	  mDeleteBtn.enable();
	
	if(mCurSelection.getType() == ID_FAC)
	  {
	    for(int i = 0; i < FIXED_FAC_ARRAY.length; i++)
	      {
		if(FIXED_FAC_ARRAY[i].equals(mCurSelection.getId()))
		  {
		    if(mDeleteBtn != null)
		      mDeleteBtn.disable();
		    break;
		  }
	      }
	  }
      }

    return true;
  }
}
