/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   ResAdder.java
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

package nfr.rc;

import nfr.misc.*;

import java.awt.*;


public class ResAdder extends GoodFrame implements ResConsts
{
  ResTab mResTab = null;
  ResId mResId = null;
  ResConfigurator mIdConfigurator = null;
  TextField mIdField = null;
  Thread mMainThread = null;
  boolean mDone = false;

public ResAdder(int type, ResTab rt, Component parent)
  {
    super(parent);
    setFont(NFRInfo.font);

    Button b = null;
    Label lbl = null;
    repTile l = new repTile();
    setLayout(l);
    
    mResTab = rt;

    mResId = ResId.create(type, rt);

    lbl = new Label("Id String:");
    add(lbl);
    l.setConstraints(0,0,0,0,"", lbl);
    mIdField = new TextField();
    add(mIdField);
    l.setConstraints(1,0,1,0,"", mIdField);
    mIdConfigurator = mResId.getConfigurator();
    add(mIdConfigurator);
    l.setConstraints(2,0,2,0,"", mIdConfigurator);
    Panel p = new Panel();
    b = new Button("Add");
    p.add(b);
    b = new Button("Cancel");
    p.add(b);
    add(p);
    l.setConstraints(3,0,3,0,"",p);
    l.setborder(CtoP.Width(3,this), CtoP.Width(3,this),
		CtoP.Height(1, this), CtoP.Height(1, this));
    l.setStretchy(-1,0);
    resize(CtoP.Width(55,this),CtoP.Height(25, this));
  }

public ResId getResId()
  {
    String idStr = mIdField.getText();

    // test to be sure that it only contains good chars
    String errMsg = ResConfigurator.checkId(idStr);
    if(errMsg != null)
      {
	MsgBox.show("Error", errMsg);
	return null;
      }

    // test to be sure that it doesn't already exist
    if(mResTab.get(idStr) != null)
      {
	MsgBox.show("Error", "ID string \"" + idStr + "\" is already in use");
	return null;
      }

    // it passed
    mResId.setId(idStr);
    if(!mIdConfigurator.write(mResId))
      return null;
    return mResId;
  }

public boolean action(Event e, Object arg) 
  {
    if(arg == "Add")
    {
      // let the gui parent know we're ready by just letting this one
      // "bubble up".  the parent is usually a RangeConfigurator object
      return false;		
    }

    if(arg == "Cancel")
      {
	dispose();
	return true;
      }

    return false;
  }
}
