/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   ResConfigurator.java
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

package nfr.rc;

import nfr.misc.*;

import java.awt.*;


public abstract class ResConfigurator extends Panel implements ResConsts
{
  ResTab mResTab = null;

public ResConfigurator(ResTab rt)
  {
    mResTab = rt;
    setFont(NFRInfo.font);
  }

public abstract boolean read(ResId rid);
public abstract boolean write(ResId rid);
public abstract void setEnabled(boolean b);
public void enable(boolean b) { setEnabled(b); }
public void enable() { enable(true); }
public void disable() { enable(false); }
public abstract void clear();

public static String checkId(String idStr)
  {
    String letters = new String("abcdefghijklmnopqrstuvwxyz");
    String LETTERS = new String("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
    String others = new String("1234567890_");

    if( (letters.indexOf(idStr.charAt(0)) == -1) &&
	(LETTERS.indexOf(idStr.charAt(0)) == -1) )
      {
	return new String("IDs must start with a letter");
      }
    for(int i = 1; i < idStr.length(); i++)
      {
	if( (letters.indexOf(idStr.charAt(i)) == -1) &&
	    (LETTERS.indexOf(idStr.charAt(i)) == -1) &&
	    (others.indexOf(idStr.charAt(i)) == -1) )
	  {
	    return new String(idStr + "is invalid, " +
			      "IDs may contain only letters, digits, and '_'");
	  }
      }
    return null; 
  }

public static String checkQuotes(String str)
  {
    boolean isEscaped = false;
    for(int i = 0; i < str.length(); i++)
      {
	switch(str.charAt(i))
	  {
	  case '"':
	    if(!isEscaped)
	      return new String(str + "is invalid, " + 
				"all \" characters must be represented as " +
				"\\\"");
	    break;

	  case '\\':
	    isEscaped = !isEscaped;
	    break;

	  default:
	    isEscaped = false;
	    break;
	  }
      }
    return null;
  }

}
