/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   ResId.java
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

package nfr.rc;

import java.net.*;
import java.io.*;
import java.awt.*;
import nfr.misc.*;


public class ResId extends ResCode
{
  ResTab mResTab = null;
  String mId = null;
  String mDispName = null;
  int mState = 0;

public ResId(ResId id, ResTab rt)
  {
    super(id.getResCode());
    mResTab = rt;
    setId(id.getId());
    setDispName(id.getDispName());
    setState(id.getState());
  }

public ResId(ResTab rt)
  {
    // System.out.println("in ResId()");
    mResTab = rt;
    mState = RESID_STATE_ADDED;
    // System.out.println("done with ResId()");
  }

public ResId(String Id,  ResTab rt)
  {
    mResTab = rt;
    setId(Id);
    mState = RESID_STATE_ADDED;
  }

public ResId(String Id, String DispName,  ResTab rt)
  {
    mResTab = rt;
    setId(Id);
    setDispName(DispName);
    mState = RESID_STATE_ADDED;
  }

public ResId(int rc, DataInputStream In, ResTab rt)
  {
    super(rc);
    try
      {
	int i = 0;
	int slen = In.readInt();
	char sbuf[] = new char[slen];
	while(i < slen)
	  sbuf[i++] = (char) In.readByte();
	mId = new String(sbuf);

	i = 0;
	slen = In.readInt();
	sbuf = new char[slen];
	while(i < slen)
	  sbuf[i++] = (char) In.readByte();
	mDispName = new String(sbuf);

	mState = In.readInt();
      }	
    catch(Exception e)
      {
	e.printStackTrace();
      }
    mResTab = rt;
    mState = RESID_STATE_STABLE;
  }

public static ResId create(int type, ResTab rt)
  {
    switch(type)
      {
      case ID_FAC:
	// System.out.println("gonna return a fac");
	return new FacId(rt);

      case RES_ALERT:
	// System.out.println("gonna return an alert");
	return new AlertRes(rt);
	
      case RES_ALERT_SRC:
	// System.out.println("gonna return an alert source");
	return new AlertSrcRes(rt);
	
      case RES_STRING:
	// System.out.println("gonna return a string");
	return new StringRes(rt);
	
      default:
	// System.out.println("**** got a bogus rescode");
	return null;
      }
  }

public String toString()
  {
    return new String("mId=" + mId + ", mDispName=" + mDispName + 
		      ", mState=" + String.valueOf(mState));
  }

public boolean dump(DataOutputStream Out)
  {
    try
      {
	Out.writeInt(mCode);

	Out.writeInt(mId.length());
	Out.writeBytes(mId);

	if(mDispName == null)
	  Out.writeInt(0);
	else
	  {
	    Out.writeInt(mDispName.length());
	    Out.writeBytes(mDispName);
	  }

	Out.writeInt(mState);
      }	
    catch(Exception e)
      {
	e.printStackTrace();
	return false;
      }
    return true;
  }

public ResConfigurator getConfigurator(ResTab rt)
  {
    return new ResIdConfigurator(this, rt);
  }

public ResConfigurator getConfigurator()
  {
    return new ResIdConfigurator(this, mResTab);
  }

public final int getResCode() { return mCode; }

public final void setResCode(int rc) { mCode = rc; }

public final int getState() { return mState; }

public final void setState(int s) { mState = s; }

public final String getDispName() { return mDispName; }

public void setDispName(String DispName) 
  { 
    // System.out.println("in ResId.DispName()");
    if ((mDispName != null) && (mDispName.equals(DispName)))
      return;

    mDispName = new String(DispName); 
    if(mState == RESID_STATE_STABLE)
      mState = RESID_STATE_ALTERED;
  }

public final String getId() { return mId; }

public void setId(String id) 
  { 
    if((mId != null) && (mId.equals(id)))
      return;

    mId = new String(id); 
    if(mState == RESID_STATE_STABLE)
      mState = RESID_STATE_ALTERED;
    // System.out.println("in setId() id now eq " + mId);
  }

//protected methods
protected void setResType(int n) 
  { 
    if(getType() == n)
      return;

    super.setType(n);
    if(mState == RESID_STATE_STABLE)
      mState = RESID_STATE_ALTERED;
  }

}
