/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   ResTab.java
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

package nfr.rc;

import nfr.misc.*;

import java.util.*;
import java.net.*;
import java.io.*;
import java.awt.*;

/*****************************************************************************
  ResTab is a class used to retrieve edit and save resources on an nfr server
*****************************************************************************/

public class ResTab extends Hashtable implements ResConsts
{
  Vector mRanges = null;
  ResTabCfgWindow mCfgWindow = null;
  Vector mFacVector = null;
  boolean mFacVectorDirty = false;
  boolean mUser = false;

  String mResFilename = null;
  String mURLEQuery = null;
  String mProto = null;
  String mHostName = null;
  int mPort = -1;


public ResTab()
  {
    mRanges = new Vector();
    mFacVector = new Vector();
  }         

public boolean isUser()
  { return mUser; }

public boolean dump()
  {
    return dump(true);
  }

public boolean dump(boolean commit)
  {
    // System.out.println("in ResTab.dump(), mResFilename=" + mResFilename);

    DataOutputStream Out = null;
    DataInputStream In = null;
    Enumeration allIds = null;
    URL u = null;
    URLConnection Conn = null;

    if(! isDirty())
      return true;

    for(int i = 0; i < mRanges.size(); i++)
      {
	RangeConfigurator rcfg = (RangeConfigurator)
	  ((RangeRes)mRanges.elementAt(i)).getConfigurator();
	if(rcfg != null)
	  {
	    if(!rcfg.write())
	      return false;
	  }
      }

    int Result = -1;

    CgiScript cgi = new CgiScript("/cgi-bin/resmerge");
    cgi.setPathInfo(mResFilename);
    Out = cgi.getOutputStream();
    if(Out == null)
      {
	MsgBox.show("Error", cgi.getRelevantMsg());
	return false;
      }
    else
      {
	allIds = elements();
	while(allIds.hasMoreElements())
	{
	  ResId id = (ResId) allIds.nextElement();
	  if(id.getType() == ID_FAC)
	    {
	      // send facs whether they're modified or not
	      id.dump(Out);
	    }
	}

	allIds = elements();
	while(allIds.hasMoreElements())
	{
	  ResId id = (ResId) allIds.nextElement();
	  if((id.getType() != ID_FAC) && (id.getState() != RESID_STATE_STABLE))
	    {
	      // send the non-fac things that are modified
	      id.dump(Out);
	    }
	}

	// System.out.println("done dumping restab, " + cgi.getBytesSent() + 
	//	   " bytes written");

	Result = cgi.getResponse();
	cgi.close();
	if(Result != 0)
	  {
	    MsgBox.show("Error", cgi.getRelevantMsg());
	    return false;
	  }
      }

    System.out.println("Result of dump: " + Result);
    
    if(Result != 0)
      return false;

    // the merge succeeded for the definition files, now go ahead and
    // and mark all the objects in the table as stable, and then run
    // rc to rebuild the database, if they want it.
    allIds = elements();
    while(allIds.hasMoreElements())
      {
	ResId id = (ResId) allIds.nextElement();
	id.setState(RESID_STATE_STABLE);
      }
    if(commit)
      return commit();

    return true;
  }

public boolean commit()
  {    
    // System.out.println("gonna run rc");

    DataOutputStream Out = null;
    DataInputStream In = null;
    URL u = null;
    URLConnection Conn = null;
    int Result = 0;

    CgiScript cgi = new CgiScript("/cgi-bin/call_rc");
    cgi.addDataNVPair("foo", "bar");
    Result = cgi.run();
    
    System.out.println("Result of cgi-bin/call_rc: " + Result);
    if(Result < 0)
      MsgBox.show("Error", cgi.getLocMsg());
    else if (Result > 0)
      MsgBox.show("Error", cgi.getRemMsg());
    //else
    //  MsgBox.show("Success", cgi.getRemMsg());

    if(Result != 0)
      return false;
    return true;
  }


public boolean load(URL BaseURL, String ResFilename)
  {
    return load(BaseURL, ResFilename, " ", null);
  }


public boolean load(URL BaseURL, String ResFilename, String Query)
  {
    return load(BaseURL, ResFilename, Query, null);
  }

public boolean load(URL BaseURL, String ResFilename, String Query,
		    ProgressThing progThing)
  {
    // System.out.println("in ResTab.load(BaseURL=" + BaseURL +
    //	       "ResFilename=" + ResFilename +
    //	       "Query=" + Query + ")");

    DataOutputStream Out = null;
    DataInputStream In = null;
    URL u = null;
    URLConnection Conn = null;

    mResFilename = ResFilename;
    mProto = BaseURL.getProtocol();
    mPort = BaseURL.getPort();
    mHostName = BaseURL.getHost();    

    if(progThing != null)
      progThing.set_max(-1);

    if(ResFilename.equals(NFRInfo.usr_resource_file_name))
      {
	//Query = "user=1&" + Query;
	mUser = true;
      }

    try
      {
	u = new URL(mProto, mHostName, mPort, "/cgi-bin/resdump");
	Conn = u.openConnection();
	Conn.setDoOutput(true);
	Conn.setDoInput(true);
	
	Out = new DataOutputStream(Conn.getOutputStream());
	Out.writeBytes("src_files=" + 
		       URLEncoder.encode(NFRInfo.fac_resource_file_name) 
		       + "," +
		       URLEncoder.encode(mResFilename) +
		       "&" + Query + "\r\n\r\n");
	In = new DataInputStream(Conn.getInputStream());
	int rc = In.readInt();
	while(rc > -1)
	  {
	    ResCode R = new ResCode(rc);
	    switch(R.getType())
	    {
	    case ID_FAC:
	      insert(new FacId(R.getCode(), In, this));
	      break;
	      
	    case ID_SEV:
	      insert(new ResId(R.getCode(), In, this));
	      break;
	      
	    case RES_RANGE:
	      insert(new RangeRes(R.getCode(), In, this));
	      break;
	      
	    case RES_ALERT:
	      insert(new AlertRes(R.getCode(), In, this));
	      break;
	      
	    case RES_ALERT_SRC:
	      insert(new AlertSrcRes(R.getCode(), In, this));
	      break;
	      
	    case RES_STRING:
	      insert(new StringRes(R.getCode(), In, this));
	      break;
	      
	    default:
	      System.out.println("**** got a bogus rescode");
	      return false;
	    }

	    rc = In.readInt();
	  }
      }

    catch(EOFException e)
      {
	//'sok, we were expecting this, now go ahead and fix up the table
      }

    catch(Exception e)
      {
	e.printStackTrace();
	return false;
      }


    try
      {
	Out.close();
	In.close();
      }
    catch(Exception e)
      {
	e.printStackTrace();
	return false;
      }

    // now stick all the resources in their respective ranges, we have
    // to do it here so we can be sure that all the ranges have been
    // defined.
    Enumeration ResIds = keys();
    while (ResIds.hasMoreElements())
      {
	String Id = (String) ResIds.nextElement();
	ResId CurResId = (ResId) get(Id);
	if(CurResId.getType() != RES_RANGE)
	  {
	    boolean foundRange = false;
	    for(int i = 0; i < mRanges.size(); i++)
	      {
		RangeRes range = (RangeRes) mRanges.elementAt(i);
		if ( (CurResId.getType() == range.getMemberType()) &&
		     (CurResId.getResNo() >= range.getMin()) &&
		     (CurResId.getResNo() <= range.getMax()) )
		  {
		    range.Members.addElement(CurResId);
		    foundRange = true;
		    break;
		  }
	      }
	    if(!foundRange)
	      System.out.println("couldn't find a range for " + 
				 CurResId.getId());
	  }
      }

    return true;
  }

public Frame getCfgWindow()
  {
    return mCfgWindow;
  }

public boolean insert(ResId r)
  {
    put(r.getId(), r);
    if(r.getType() == RES_RANGE)
      mRanges.addElement(r);
    else if(r.getType() == ID_FAC)
      {
	int idx = 0;

	if(r.getState() == RESID_STATE_ADDED)
	  {
	    r.setResNo(mFacVector.size());
	    mFacVector.addElement(r);
	  }
	else
	  {
	    // bug: clean up this block
	    idx = r.getResNo();
	    try { mFacVector.setElementAt(r, idx); }
	    catch(ArrayIndexOutOfBoundsException e)
	      {
		int cap = idx + 1;
		if (cap <= 255)
		  {
		    // System.out.println("growing mFacVector to " + cap);
		    
		    //mFacVector.ensureCapacity(cap);
		    if(idx >= mFacVector.size())
		      mFacVector.setSize(cap);

		    // System.out.println("mFacVector grown to " + 
		    //	       mFacVector.capacity());
		    
		  }
		// if idx was > 255 another exception gets thrown
		// here and we won't catch it 'cause that means
		// something is very wrong
		mFacVector.setElementAt(r, idx);
	      }
	  }
      }
    return true;
  }

public final ResId getResId(String Id) { return (ResId) get(Id); }

public ResId getFacId(int facCode) 
  { 
    ResId rid = null;
    try
      { rid = (ResId)mFacVector.elementAt(facCode); }
    catch(ArrayIndexOutOfBoundsException e)
      {
	// System.out.println("FacId does not exist: " + facCode);
	return (ResId)new FacId("FAC_NULL", "NULL Facility",  this);
      }
    return rid;
  }

public final Vector getRanges() { return mRanges; }

public Vector cloneFacVector() { return (Vector) mFacVector.clone(); }

public void setFacVector(Vector v)
  { 
    mFacVector = v; 
    mFacVectorDirty = true;
  }

public final boolean isDirty() 
  { 
    boolean dirty = false;
    Enumeration allIds = null;

    // first sync the table with the gui
    for(int i = 0; i < mRanges.size(); i++)
      {
	RangeConfigurator rcfg = (RangeConfigurator)
	  ((RangeRes)mRanges.elementAt(i)).getConfigurator();
	if(rcfg != null)
	  {
	    if(!rcfg.write())
	      return true;
	  }
      }

    //now see if anything has been changed
    allIds = elements();
    while(allIds.hasMoreElements())
      {
	ResId id = (ResId) allIds.nextElement();
	//if(id.getType() == ID_FAC)
	//System.out.println("testing fac: " + id);
	if(id.getState() != RESID_STATE_STABLE)
	  {
	    //System.out.println("dirty one is: " + id.getId());
	    return true;
	  }
      }
    return false;
  }

}

