/*****************************************************************************
  Copyright(C) 1996, 1997 Network Flight Recorder, Inc.
  All rights reserved.

  Use and distribution of this software and its source code
  are governed by the terms and conditions of the
  Network Flight Recorder Software License ("LICENSE.TXT")


  File:   ResTabCfgWindow.java
  Author: Andrew Lambeth <andrew@nfr.net>

 *****************************************************************************/

  // NOTE WELL: when dumping a restab pair for which the facilities may
  // have been modified, you must dump the user restab *before* the fixed
  // restab, since the copies of the facilities that get edited will
  // be in the user restab

package nfr.rc;

import nfr.misc.*;
import java.awt.*;
import java.util.*;


public class ResTabCfgWindow extends GoodFrame implements ResConsts
{
  ResTab mFixedResTab = null;
  ResTab mUsrResTab = null;
  Button mApplyBtn = null;

public ResTabCfgWindow(ResTab fixedRt, ResTab usrRt, Window w)
  {
    super((Component)w);
    mFixedResTab = fixedRt;
    mUsrResTab = usrRt;

    setFont(NFRInfo.font);

    repTile l = new repTile();
    setLayout(l);


    setFont(NFRInfo.font);

    tabthing tt = new tabthing(6);
    add(tt);
    l.setConstraints(0,0,0,0,"",tt);
    l.setStretchy(0,0);

    Vector rangeVector = mFixedResTab.getRanges();
    RangeCfgStack fixedSrcStack = new RangeCfgStack(rangeVector.size());
    RangeCfgStack fixedAlertStack = new RangeCfgStack(rangeVector.size());

    for(int i = 0; i < rangeVector.size(); i++)
      {
	RangeRes range = (RangeRes) rangeVector.elementAt(i);
	RangeConfigurator rcfgtr = 
	  (RangeConfigurator) range.getConfigurator(mFixedResTab);
	String dispName = range.getDispName();

	switch (range.getMemberType())
	  {
	  case RES_ALERT_SRC:
	    fixedSrcStack.addRangeCfg(rcfgtr, dispName);
	    break;
	  case RES_ALERT:
	    fixedAlertStack.addRangeCfg(rcfgtr, dispName);
	    break;
	  }
      }

    rangeVector = mUsrResTab.getRanges();
    RangeCfgStack usrSrcStack = new RangeCfgStack(rangeVector.size());
    RangeCfgStack usrAlertStack = new RangeCfgStack(rangeVector.size());
    RangeCfgStack destStack = new RangeCfgStack(rangeVector.size());

    for(int i = 0; i < rangeVector.size(); i++)
      {
	RangeRes range = (RangeRes) rangeVector.elementAt(i);
	RangeConfigurator rcfgtr = 
	  (RangeConfigurator) range.getConfigurator(mUsrResTab);
	String dispName = range.getDispName();

	switch (range.getMemberType())
	  {
	  case RES_ALERT_SRC:
	    usrSrcStack.addRangeCfg(rcfgtr, dispName);
	    break;
	  case RES_ALERT:
	    usrAlertStack.addRangeCfg(rcfgtr, dispName);
	    break;
	  case ID_FAC:
	    destStack.addRangeCfg(rcfgtr, dispName);
	    break;
	  }
      }

    // now build all the various card stack type things
    tt.addThing(fixedSrcStack, "Built-in Sources");
    tt.addThing(usrSrcStack, "User Sources");
    tt.addThing(fixedAlertStack, "Built-in Messages");
    tt.addThing(usrAlertStack, "User Messages");
    tt.addThing(destStack, "Destinations");
    tt.compute();
    fixedSrcStack.build();
    fixedAlertStack.build();
    usrSrcStack.build();
    usrAlertStack.build();
    destStack.build();

    //resize(CtoP.Width(115,this),CtoP.Height(30, this));
    resize(750, 520);
    setTitle("NFR Alert Configuration");

    Panel p = new Panel();
    mApplyBtn = new Button("Apply");
    p.add(mApplyBtn);
    p.add(new Button("Quit"));
    add(p);
    l.setConstraints(1,0,1,0,"ch",p);
  }

public int beforeDisposal()
  {
    if(mUsrResTab.isDirty() || mFixedResTab.isDirty())
      {
	MsgBox mb = new MsgBox("Warning","Unsaved changes exist!", false);
	mb.addButton("Save changes");
	mb.addButton("Lose changes");
	String ans = MsgBox.wait(mb);
	if(ans.equals("Save changes"))
	  {
	    mApplyBtn.disable();
	    onApply(false);
	    mApplyBtn.enable();
	    return AGAIN; // call me again later
	  }
      }
    return DISPOSABLE; // take me now
  }

public boolean action(Event e, Object arg) 
  {
    if(arg == "Apply")
      {
	mApplyBtn.disable();
	onApply(true);
	mApplyBtn.enable();
      }

    else if(arg == "Quit")
      {
	dispose();
      }

    else
      return false;

    return true;
  }

public void onApply(boolean reload)
  {
    boolean NeedReload = false;

    
    //first see if any facilities have been changed
    // we check the user restab since it contains the 
    // facility objects which actually get edited, then 
    // we need to sync the copies of the facilities that
    // exist in the fixed restab
    Enumeration allIds = mUsrResTab.elements();
    while(allIds.hasMoreElements())
      {
	ResId id = (ResId) allIds.nextElement();
	if((id.getType() == ID_FAC) &&
	   (id.getState() != RESID_STATE_STABLE))
	  {
	    //System.out.println("transferring dirty fac \"" +
	    //	       id.getId() +
	    //	       "\" to mFixedResTab");

	    // make sure the editor gets reloaded, unless we've been
	    // told not to, since that's just the easiest way to deal
	    // with all the things that get messed up when the user
	    // mucks with the facilities.
	    if(reload)
	      NeedReload = true;

	    // if this is a new facility, we'll need to create the 
	    // template for it on the server
	    if(id.getState() == RESID_STATE_ADDED)
	      {
		CgiScript mkfac = new CgiScript("/cgi-bin/mkfac");
		mkfac.addDataNVPair("name", id.getId());
		mkfac.addDataNVPair("type", ((FacId)id).getGuiType());
		int Result = mkfac.run();

		System.out.println("Result of cgi-bin/mkfac: " + Result);
		if(Result < 0)
		  {
		    MsgBox.show("Error", mkfac.getLocMsg());
		    return;
		  }
		else if (Result > 0)
		  {
		    MsgBox.show("Error", mkfac.getRemMsg());
		    return;
		  }
	      }

	    // if this is a facility that's being deleted, we'll need
	    // to removethe config files for it from the server
	    if(id.getState() == RESID_STATE_DELETED)
	      {
		CgiScript rmfac = new CgiScript("/cgi-bin/rmfac");
		rmfac.addDataNVPair("name", id.getId());
		int Result = rmfac.run();

		System.out.println("Result of cgi-bin/rmfac: " + Result);
		if(Result < 0)
		  {
		    MsgBox.show("Error", rmfac.getLocMsg());
		    return;
		  }
		else if (Result > 0)
		  {
		    MsgBox.show("Error", rmfac.getRemMsg());
		    return;
		  }
	      }

	    // make a copy of the changed fac id and put it in the 
	    // fixed restab
	    FacId newId = new FacId((FacId)id, mFixedResTab);
	    Object o = mFixedResTab.put(newId.getId(), newId);
	    // System.out.println("prev val of fac in mFixedResTab: " + o);
	    // System.out.println("current val of fac in mFixedResTab: " + 
	    //	       mFixedResTab.get(id.getId()) );
	  }
      }


    if(mUsrResTab.dump(false) && 
       mFixedResTab.dump(false) && 
       mFixedResTab.commit() )
      {
	// we only needed to run commit on one of the restabs, 
	// since the call_rc cgi rebuilds the entire database.
	// so we arbitrarily chose mFixedResTab
	if(NeedReload)
	  {
	    //MsgBox.wait("Result", "Changes saved");
	    ResTab nfrRt = new ResTab();
	    nfrRt.load(NFRInfo.document_base,
		       NFRInfo.nfr_resource_file_name);
	    
	    ResTab usrRt = new ResTab();
	    usrRt.load(NFRInfo.document_base,
		       NFRInfo.usr_resource_file_name);
	    
	    ResTabCfgWindow cfgWindow =
	      new ResTabCfgWindow(nfrRt, usrRt, NFRInfo.top);
	    NFRInfo.do_style(cfgWindow);
	    dispose();
	    cfgWindow.show();
	  }
	//else
	//MsgBox.show("Result", "Changes saved");
      }
    else
      MsgBox.show("Result", "Error, changes not saved");
    // bug: get error codes or msgs from the above cgi ops
  }

}

